package dev.pfaff.glass;

/**
 * Runtime configuration utilities.
 */
public final class Config {
	/**
	 * Parses the value of the environment variable {@code name} as a boolean where {@code false} is parsed from
	 * {@code 0} and {@code true} is parsed from {@code 1}.
	 *
	 * @param name         the name of the environment variable.
	 * @param defaultValue the value that should be returned if the environment variable is unset.
	 * @return {@code true} if the variable is set to {@code 1}, {@code false} if the variable is set to {@code 0}, or
	 * {@code defaultValue} otherwise.
	 */
	public static boolean getEnvFlag(String name, boolean defaultValue) {
		return switch (System.getenv(name)) {
			case "1" -> true;
			case "0" -> false;
			case null -> defaultValue;
			default -> {
				throw new IllegalArgumentException(STR."Environment variable \{name} must be '0', or '1'. If unset, it will default to '\{
						defaultValue
								? "1"
								: "0"}'.");
			}
		};
	}
}
