package dev.pfaff.glass.opengl;

import java.lang.foreign.*;
import java.lang.invoke.MethodHandle;
import java.util.Arrays;
import java.util.List;
import java.util.stream.IntStream;

import static dev.pfaff.glass.opengl.Util.*;
import static java.lang.invoke.MethodHandles.*;

public record UnlinkedGLFunction(String name, GLType retType, List<GLType> argTypes) {
	public UnlinkedGLFunction {
		argTypes = List.copyOf(argTypes);
	}

	public UnlinkedGLFunction(String name, Class<?> retType, MemoryLayout retLayout, List<Class<?>> argTypes, List<MemoryLayout> argLayouts) {
		this(name,
				GLTypeFromClassAndLayout(retType, retLayout),
				IntStream.range(0, argTypes.size()).mapToObj(i -> GLTypeFromClassAndLayout(argTypes.get(i),
						argLayouts.get(i))).toList());
	}

	public static MethodHandle prepare(String name,
											 MemoryLayout retLayout,
											 Class<?> retType,
											 List<MemoryLayout> argLayouts,
											 List<Class<?>> argTypes,
											 Linker.Option[] options) {
		return prepareTrusted(name, retLayout, retType, argLayouts, List.copyOf(argTypes), options);
	}

	static MethodHandle prepareTrusted(String name,
											 MemoryLayout retLayout,
											 Class<?> retType,
											 List<MemoryLayout> argLayouts,
											 List<Class<?>> argTypes,
											 Linker.Option[] options) {
		argLayouts = List.copyOf(argLayouts);
		FunctionDescriptor fd;
		try {
			fd = (FunctionDescriptor) MH_FunctionDescriptorImpl_of.invokeExact(retLayout, argLayouts);
		} catch (Throwable e) {
			throw throwUnchecked(e);
		}
		var MH = LINKER.downcallHandle(fd, options);
		var filters = new MethodHandle[argTypes.size()];
		for (int i = 0; i < filters.length; i++) {
			filters[i] = MH_toNType(argTypes.get(i), argLayouts.get(i));
		}
		MH = filterArguments(MH, 1, filters);
		var MH_fromNType = MH_fromNType(retType, retLayout);
		return MH_fromNType == null ? MH : filterReturnValue(MH, MH_fromNType);
	}

	public static MethodHandle link(SymbolLookup symbolLookup, String name, MethodHandle MH_dynamic) {
		var symbol = symbolLookup.find(name).orElse(null);
		if (symbol == null) {
			var msg = STR."Unsupported GL function '\{name}'";
			var MH = throwException(MH_dynamic.type().returnType(), UnsupportedOperationException.class);
			MH = filterReturnValue(MH_newUnsupportedOperationException.bindTo(msg), MH);
			return dropArguments(MH, 0, Arrays.copyOfRange(MH_dynamic.type().parameterArray(), 1, MH_dynamic.type().parameterCount()));
		}
		return MH_dynamic.bindTo(symbol);
	}

	public String name() {
		return name;
	}
}
