package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLPixelTransferParameter implements GLEnum {
	MAP_COLOR(0xd10),
	MAP_STENCIL(0xd11),
	INDEX_SHIFT(0xd12),
	INDEX_OFFSET(0xd13),
	RED_SCALE(0xd14),
	RED_BIAS(0xd15),
	GREEN_SCALE(0xd18),
	GREEN_BIAS(0xd19),
	BLUE_SCALE(0xd1a),
	BLUE_BIAS(0xd1b),
	ALPHA_SCALE(0xd1c),
	ALPHA_BIAS(0xd1d),
	DEPTH_SCALE(0xd1e),
	DEPTH_BIAS(0xd1f),
	POST_CONVOLUTION_RED_SCALE(0x801c),
	POST_CONVOLUTION_GREEN_SCALE(0x801d),
	POST_CONVOLUTION_BLUE_SCALE(0x801e),
	POST_CONVOLUTION_ALPHA_SCALE(0x801f),
	POST_CONVOLUTION_RED_BIAS(0x8020),
	POST_CONVOLUTION_GREEN_BIAS(0x8021),
	POST_CONVOLUTION_BLUE_BIAS(0x8022),
	POST_CONVOLUTION_ALPHA_BIAS(0x8023),
	POST_COLOR_MATRIX_RED_SCALE(0x80b4),
	POST_COLOR_MATRIX_GREEN_SCALE(0x80b5),
	POST_COLOR_MATRIX_BLUE_SCALE(0x80b6),
	POST_COLOR_MATRIX_ALPHA_SCALE(0x80b7),
	POST_COLOR_MATRIX_RED_BIAS(0x80b8),
	POST_COLOR_MATRIX_GREEN_BIAS(0x80b9),
	POST_COLOR_MATRIX_BLUE_BIAS(0x80ba),
	POST_COLOR_MATRIX_ALPHA_BIAS(0x80bb);
	
	private final int glEnum;

	GLPixelTransferParameter(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLPixelTransferParameter> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLPixelTransferParameter fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0xd10 -> MAP_COLOR;
			case 0xd11 -> MAP_STENCIL;
			case 0xd12 -> INDEX_SHIFT;
			case 0xd13 -> INDEX_OFFSET;
			case 0xd14 -> RED_SCALE;
			case 0xd15 -> RED_BIAS;
			case 0xd18 -> GREEN_SCALE;
			case 0xd19 -> GREEN_BIAS;
			case 0xd1a -> BLUE_SCALE;
			case 0xd1b -> BLUE_BIAS;
			case 0xd1c -> ALPHA_SCALE;
			case 0xd1d -> ALPHA_BIAS;
			case 0xd1e -> DEPTH_SCALE;
			case 0xd1f -> DEPTH_BIAS;
			case 0x801c -> POST_CONVOLUTION_RED_SCALE;
			case 0x801d -> POST_CONVOLUTION_GREEN_SCALE;
			case 0x801e -> POST_CONVOLUTION_BLUE_SCALE;
			case 0x801f -> POST_CONVOLUTION_ALPHA_SCALE;
			case 0x8020 -> POST_CONVOLUTION_RED_BIAS;
			case 0x8021 -> POST_CONVOLUTION_GREEN_BIAS;
			case 0x8022 -> POST_CONVOLUTION_BLUE_BIAS;
			case 0x8023 -> POST_CONVOLUTION_ALPHA_BIAS;
			case 0x80b4 -> POST_COLOR_MATRIX_RED_SCALE;
			case 0x80b5 -> POST_COLOR_MATRIX_GREEN_SCALE;
			case 0x80b6 -> POST_COLOR_MATRIX_BLUE_SCALE;
			case 0x80b7 -> POST_COLOR_MATRIX_ALPHA_SCALE;
			case 0x80b8 -> POST_COLOR_MATRIX_RED_BIAS;
			case 0x80b9 -> POST_COLOR_MATRIX_GREEN_BIAS;
			case 0x80ba -> POST_COLOR_MATRIX_BLUE_BIAS;
			case 0x80bb -> POST_COLOR_MATRIX_ALPHA_BIAS;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLPixelTransferParameter");
		};
	}
}