package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLProgramInterface implements GLEnum {
	TRANSFORM_FEEDBACK_BUFFER(0x8c8e),
	UNIFORM(0x92e1),
	UNIFORM_BLOCK(0x92e2),
	PROGRAM_INPUT(0x92e3),
	PROGRAM_OUTPUT(0x92e4),
	BUFFER_VARIABLE(0x92e5),
	SHADER_STORAGE_BLOCK(0x92e6),
	VERTEX_SUBROUTINE(0x92e8),
	TESS_CONTROL_SUBROUTINE(0x92e9),
	TESS_EVALUATION_SUBROUTINE(0x92ea),
	GEOMETRY_SUBROUTINE(0x92eb),
	FRAGMENT_SUBROUTINE(0x92ec),
	COMPUTE_SUBROUTINE(0x92ed),
	VERTEX_SUBROUTINE_UNIFORM(0x92ee),
	TESS_CONTROL_SUBROUTINE_UNIFORM(0x92ef),
	TESS_EVALUATION_SUBROUTINE_UNIFORM(0x92f0),
	GEOMETRY_SUBROUTINE_UNIFORM(0x92f1),
	FRAGMENT_SUBROUTINE_UNIFORM(0x92f2),
	COMPUTE_SUBROUTINE_UNIFORM(0x92f3),
	TRANSFORM_FEEDBACK_VARYING(0x92f4);
	
	private final int glEnum;

	GLProgramInterface(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLProgramInterface> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLProgramInterface fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x8c8e -> TRANSFORM_FEEDBACK_BUFFER;
			case 0x92e1 -> UNIFORM;
			case 0x92e2 -> UNIFORM_BLOCK;
			case 0x92e3 -> PROGRAM_INPUT;
			case 0x92e4 -> PROGRAM_OUTPUT;
			case 0x92e5 -> BUFFER_VARIABLE;
			case 0x92e6 -> SHADER_STORAGE_BLOCK;
			case 0x92e8 -> VERTEX_SUBROUTINE;
			case 0x92e9 -> TESS_CONTROL_SUBROUTINE;
			case 0x92ea -> TESS_EVALUATION_SUBROUTINE;
			case 0x92eb -> GEOMETRY_SUBROUTINE;
			case 0x92ec -> FRAGMENT_SUBROUTINE;
			case 0x92ed -> COMPUTE_SUBROUTINE;
			case 0x92ee -> VERTEX_SUBROUTINE_UNIFORM;
			case 0x92ef -> TESS_CONTROL_SUBROUTINE_UNIFORM;
			case 0x92f0 -> TESS_EVALUATION_SUBROUTINE_UNIFORM;
			case 0x92f1 -> GEOMETRY_SUBROUTINE_UNIFORM;
			case 0x92f2 -> FRAGMENT_SUBROUTINE_UNIFORM;
			case 0x92f3 -> COMPUTE_SUBROUTINE_UNIFORM;
			case 0x92f4 -> TRANSFORM_FEEDBACK_VARYING;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLProgramInterface");
		};
	}
}