package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLProgramResourceProperty implements GLEnum {
	NUM_COMPATIBLE_SUBROUTINES(0x8e4a),
	COMPATIBLE_SUBROUTINES(0x8e4b),
	UNIFORM(0x92e1),
	IS_PER_PATCH(0x92e7),
	NAME_LENGTH(0x92f9),
	TYPE(0x92fa),
	ARRAY_SIZE(0x92fb),
	OFFSET(0x92fc),
	BLOCK_INDEX(0x92fd),
	ARRAY_STRIDE(0x92fe),
	MATRIX_STRIDE(0x92ff),
	IS_ROW_MAJOR(0x9300),
	ATOMIC_COUNTER_BUFFER_INDEX(0x9301),
	BUFFER_BINDING(0x9302),
	BUFFER_DATA_SIZE(0x9303),
	NUM_ACTIVE_VARIABLES(0x9304),
	ACTIVE_VARIABLES(0x9305),
	REFERENCED_BY_VERTEX_SHADER(0x9306),
	REFERENCED_BY_TESS_CONTROL_SHADER(0x9307),
	REFERENCED_BY_TESS_EVALUATION_SHADER(0x9308),
	REFERENCED_BY_GEOMETRY_SHADER(0x9309),
	REFERENCED_BY_FRAGMENT_SHADER(0x930a),
	REFERENCED_BY_COMPUTE_SHADER(0x930b),
	TOP_LEVEL_ARRAY_SIZE(0x930c),
	TOP_LEVEL_ARRAY_STRIDE(0x930d),
	LOCATION(0x930e),
	LOCATION_INDEX(0x930f),
	LOCATION_COMPONENT(0x934a),
	TRANSFORM_FEEDBACK_BUFFER_INDEX(0x934b),
	TRANSFORM_FEEDBACK_BUFFER_STRIDE(0x934c);
	
	private final int glEnum;

	GLProgramResourceProperty(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLProgramResourceProperty> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLProgramResourceProperty fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x8e4a -> NUM_COMPATIBLE_SUBROUTINES;
			case 0x8e4b -> COMPATIBLE_SUBROUTINES;
			case 0x92e1 -> UNIFORM;
			case 0x92e7 -> IS_PER_PATCH;
			case 0x92f9 -> NAME_LENGTH;
			case 0x92fa -> TYPE;
			case 0x92fb -> ARRAY_SIZE;
			case 0x92fc -> OFFSET;
			case 0x92fd -> BLOCK_INDEX;
			case 0x92fe -> ARRAY_STRIDE;
			case 0x92ff -> MATRIX_STRIDE;
			case 0x9300 -> IS_ROW_MAJOR;
			case 0x9301 -> ATOMIC_COUNTER_BUFFER_INDEX;
			case 0x9302 -> BUFFER_BINDING;
			case 0x9303 -> BUFFER_DATA_SIZE;
			case 0x9304 -> NUM_ACTIVE_VARIABLES;
			case 0x9305 -> ACTIVE_VARIABLES;
			case 0x9306 -> REFERENCED_BY_VERTEX_SHADER;
			case 0x9307 -> REFERENCED_BY_TESS_CONTROL_SHADER;
			case 0x9308 -> REFERENCED_BY_TESS_EVALUATION_SHADER;
			case 0x9309 -> REFERENCED_BY_GEOMETRY_SHADER;
			case 0x930a -> REFERENCED_BY_FRAGMENT_SHADER;
			case 0x930b -> REFERENCED_BY_COMPUTE_SHADER;
			case 0x930c -> TOP_LEVEL_ARRAY_SIZE;
			case 0x930d -> TOP_LEVEL_ARRAY_STRIDE;
			case 0x930e -> LOCATION;
			case 0x930f -> LOCATION_INDEX;
			case 0x934a -> LOCATION_COMPONENT;
			case 0x934b -> TRANSFORM_FEEDBACK_BUFFER_INDEX;
			case 0x934c -> TRANSFORM_FEEDBACK_BUFFER_STRIDE;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLProgramResourceProperty");
		};
	}
}