package dev.pfaff.glass.opengl.enums;

import java.util.List;

import dev.pfaff.glass.opengl.GLEnum;

import static java.util.FormatProcessor.FMT;

public enum GLShaderType implements GLEnum {
	FRAGMENT_SHADER(0x8b30),
	VERTEX_SHADER(0x8b31),
	GEOMETRY_SHADER(0x8dd9),
	TESS_EVALUATION_SHADER(0x8e87),
	TESS_CONTROL_SHADER(0x8e88),
	COMPUTE_SHADER(0x91b9);
	
	private final int glEnum;

	GLShaderType(int glEnum) {
		this.glEnum = glEnum;
	}

	public static final List<GLShaderType> VALUES = List.of(values());

	@Override
	public int glEnum() {
		return glEnum;
	}

	/**
	 * Returns the enum constant corresponding to the provided raw value.
	 *
	 * @throws IllegalArgumentException if the provided raw value does not exist in this enum group.
	 */
	public static GLShaderType fromGLEnum(int glEnum) {
		return switch (glEnum) {
			case 0x8b30 -> FRAGMENT_SHADER;
			case 0x8b31 -> VERTEX_SHADER;
			case 0x8dd9 -> GEOMETRY_SHADER;
			case 0x8e87 -> TESS_EVALUATION_SHADER;
			case 0x8e88 -> TESS_CONTROL_SHADER;
			case 0x91b9 -> COMPUTE_SHADER;
			default -> throw RawGLEnum.newUnrecognizedValueError(glEnum, "GLShaderType");
		};
	}
}