package dev.pfaff.glass.opengl;

import java.lang.foreign.MemoryLayout;
import java.util.List;
import java.util.Map;

import dev.pfaff.glass.opengl.enums.*;

import static dev.pfaff.glass.opengl.GLParamTypeSets.*;

public final class Metadata {
	public static final class EnumTypes {
		public static final List<Class<? extends GLEnum>> ENUM_TYPES = List.of(GLQueryParameterName.class
		, GLConvolutionParameter.class
		, GLVertexAttribEnum.class
		, GLMaterialParameter.class
		, GLSubgroupSupportedFeatures.class
		, GLColorPointerType.class
		, GLFogCoordSrc.class
		, GLFenceConditionNV.class
		, GLProgramStringProperty.class
		, GLObjectTypeAPPLE.class
		, GLFoveationConfigBitQCOM.class
		, GLVertexArrayPName.class
		, GLGetMultisamplePNameNV.class
		, GLVertexAttribPointerPropertyARB.class
		, GLStencilFunction.class
		, GLTransformFeedbackPName.class
		, GLBufferPointerNameARB.class
		, GLPathColor.class
		, GLMapQuery.class
		, GLFragmentShaderDestMaskATI.class
		, GLContextFlagMask.class
		, GLGetHistogramParameterPNameEXT.class
		, GLFramebufferAttachmentParameterName.class
		, GLPathListMode.class
		, GLInvalidateFramebufferAttachment.class
		, GLTextureSwizzle.class
		, GLPixelType.class
		, GLTangentPointerTypeEXT.class
		, GLGetMapQuery.class
		, GLDepthStencilTextureMode.class
		, GLFragmentOp2ATI.class
		, GLTextureCompareMode.class
		, GLFeedbackType.class
		, GLMeshMode1.class
		, GLPathColorFormat.class
		, GLSizedInternalFormat.class
		, GLInstancedPathCoverMode.class
		, GLProgramTarget.class
		, GLClipControlDepth.class
		, GLTextureParameter.class
		, GLVertexStreamATI.class
		, GLFragmentOp1ATI.class
		, GLIndexFunctionEXT.class
		, GLFogMode.class
		, GLFogPointerTypeEXT.class
		, GLSyncObjectMask.class
		, GLFragmentLightModelParameterSGIX.class
		, GLAtomicCounterBufferPName.class
		, GLFenceParameterNameNV.class
		, GLLightParameter.class
		, GLMapBufferAccessMask.class
		, GLImageTransformPNameHP.class
		, GLMemoryObjectParameterName.class
		, GLCommandOpcodesNV.class
		, GLTexCoordPointerType.class
		, GLMinmaxTargetEXT.class
		, GLPixelTexGenParameterNameSGIS.class
		, GLSpecialNumbers.class
		, GLCombinerScaleNV.class
		, GLBufferStorageMask.class
		, GLAlphaFunction.class
		, GLFragmentShaderGenericSourceATI.class
		, GLBlendingFactor.class
		, GLSyncCondition.class
		, GLFragmentShaderValueRepATI.class
		, GLBufferPNameARB.class
		, GLTexBumpParameterATI.class
		, GLFogParameter.class
		, GLClipPlaneName.class
		, GLPathGenMode.class
		, GLTransformFeedbackBufferMode.class
		, GLPathElementType.class
		, GLCombinerBiasNV.class
		, GLPixelCopyType.class
		, GLBlitFramebufferFilter.class
		, GLSubroutineParameterName.class
		, GLGetPointervPName.class
		, GLGetMinmaxParameterPNameEXT.class
		, GLPatchParameterName.class
		, GLShadingRate.class
		, GLFeedBackToken.class
		, GLProgramFormat.class
		, GLMemoryBarrierMask.class
		, GLVertexAttribPointerType.class
		, GLBlendEquationModeEXT.class
		, GLEnableCap.class
		, GLMapTarget.class
		, GLClampColorTargetARB.class
		, GLLightEnvParameterSGIX.class
		, GLNormalPointerType.class
		, GLFogCoordinatePointerType.class
		, GLFogPointerTypeIBM.class
		, GLProgramResourceProperty.class
		, GLTraceMaskMESA.class
		, GLBufferAccess.class
		, GLBufferBitQCOM.class
		, GLConvolutionBorderModeEXT.class
		, GLFragmentShaderColorModMaskATI.class
		, GLDataTypeEXT.class
		, GLShadingRateQCOM.class
		, GLArrayObjectPNameATI.class
		, GLPixelTexGenModeSGIX.class
		, GLPixelTransformTargetEXT.class
		, GLScalarType.class
		, GLPixelMap.class
		, GLRenderingMode.class
		, GLBufferStorageTarget.class
		, GLFrontFaceDirection.class
		, GLPathFontStyle.class
		, GLTextureNormalModeEXT.class
		, GLFfdMaskSGIX.class
		, GLFogPName.class
		, GLPixelFormat.class
		, GLRenderbufferParameterName.class
		, GLPrimitiveType.class
		, GLDebugSeverity.class
		, GLLightTextureModeEXT.class
		, GLGetTextureParameter.class
		, GLExternalHandleType.class
		, GLSemaphoreParameterName.class
		, GLTriangleFace.class
		, GLTextureWrapMode.class
		, GLInterleavedArrayFormat.class
		, GLOcclusionQueryEventMaskAMD.class
		, GLOcclusionQueryParameterNameNV.class
		, GLTextureGenMode.class
		, GLLightTexturePNameEXT.class
		, GLPerfQueryDataFlags.class
		, GLClientAttribMask.class
		, GLVertexShaderWriteMaskEXT.class
		, GLVertexAttribIType.class
		, GLAccumOp.class
		, GLAttributeType.class
		, GLStringName.class
		, GLHintTarget.class
		, GLInternalFormatPName.class
		, GLPointParameterNameARB.class
		, GLCullParameterEXT.class
		, GLBufferUsage.class
		, GLFragmentShaderConATI.class
		, GLHistogramTargetEXT.class
		, GLArrayObjectUsageATI.class
		, GLSwizzleOpATI.class
		, GLPathCoverMode.class
		, GLTexStorageAttribs.class
		, GLColorTableParameterPName.class
		, GLCombinerComponentUsageNV.class
		, GLPrecisionType.class
		, GLPixelStoreParameter.class
		, GLShaderParameterName.class
		, GLPixelTransformPNameEXT.class
		, GLDebugType.class
		, GLFramebufferAttachment.class
		, GLProgramParameterPName.class
		, GLCombinerRegisterNV.class
		, GLGetFramebufferParameter.class
		, GLSpriteModeSGIX.class
		, GLPathHandleMissingGlyphs.class
		, GLVertexArrayPNameAPPLE.class
		, GLTextureEnvMode.class
		, GLFragmentOp3ATI.class
		, GLBinormalPointerTypeEXT.class
		, GLVertexProvokingMode.class
		, GLPixelStoreResampleMode.class
		, GLShadingRateCombinerOp.class
		, GLLightModelColorControl.class
		, GLVertexAttribType.class
		, GLShaderType.class
		, GLPixelStoreSubsampleRate.class
		, GLPathFillMode.class
		, GLPNTrianglesPNameATI.class
		, GLProgramStagePName.class
		, GLSpriteParameterNameSGIX.class
		, GLMapAttribParameterNV.class
		, GLVertexWeightPointerTypeEXT.class
		, GLGetPName.class
		, GLClampColorModeARB.class
		, GLLightModelParameter.class
		, GLListMode.class
		, GLFramebufferParameterName.class
		, GLColorTableTargetSGI.class
		, GLBoolean.class
		, GLVertexShaderTextureUnitParameter.class
		, GLSyncParameterName.class
		, GLMatrixIndexPointerTypeARB.class
		, GLConditionalRenderMode.class
		, GLVertexShaderOpEXT.class
		, GLClearBufferMask.class
		, GLColorTableTarget.class
		, GLTextureGenParameter.class
		, GLSamplePatternSGIS.class
		, GLMinmaxTarget.class
		, GLUniformBlockPName.class
		, GLVertexAttribPropertyARB.class
		, GLVertexShaderCoordOutEXT.class
		, GLTextureEnvParameter.class
		, GLCopyImageSubDataTarget.class
		, GLWeightPointerTypeARB.class
		, GLProgramPropertyARB.class
		, GLDrawElementsType.class
		, GLCombinerMappingNV.class
		, GLMapParameterNV.class
		, GLErrorCode.class
		, GLInternalFormat.class
		, GLBuffer.class
		, GLCopyBufferSubDataTarget.class
		, GLLightEnvModeSGIX.class
		, GLUngrouped.class
		, GLTextureTarget.class
		, GLMatrixMode.class
		, GLFramebufferTarget.class
		, GLVertexHintsMaskPGI.class
		, GLVertexAttribLType.class
		, GLUniformPName.class
		, GLSecondaryColorPointerTypeIBM.class
		, GLFragmentLightNameSGIX.class
		, GLFragmentShaderDestModMaskATI.class
		, GLShadingModel.class
		, GLSeparableTargetEXT.class
		, GLTextureEnvTarget.class
		, GLIndexPointerType.class
		, GLIndexMaterialParameterEXT.class
		, GLTextureMinFilter.class
		, GLColorMaterialParameter.class
		, GLPixelDataRangeTargetNV.class
		, GLPipelineParameterName.class
		, GLReplacementCodeTypeSUN.class
		, GLHintTargetPGI.class
		, GLMapTypeNV.class
		, GLHistogramTarget.class
		, GLGetVariantValueEXT.class
		, GLSeparableTarget.class
		, GLVariantCapEXT.class
		, GLGraphicsResetStatus.class
		, GLSamplePatternEXT.class
		, GLDrawBufferMode.class
		, GLPolygonMode.class
		, GLPreserveModeATI.class
		, GLPerformanceQueryCapsMaskINTEL.class
		, GLTextureMagFilter.class
		, GLRenderbufferTarget.class
		, GLPixelTransferParameter.class
		, GLProgramInterface.class
		, GLVertexShaderStorageTypeEXT.class
		, GLCombinerVariableNV.class
		, GLPathParameter.class
		, GLTextureFilterSGIS.class
		, GLReadBufferMode.class
		, GLPathFontTarget.class
		, GLEvalMapsModeNV.class
		, GLBindTransformFeedbackTarget.class
		, GLBufferTarget.class
		, GLConvolutionTargetEXT.class
		, GLTextureLayout.class
		, GLTextureParameterName.class
		, GLFfdTargetSGIX.class
		, GLHintMode.class
		, GLAttribMask.class
		, GLVertexPointerType.class
		, GLLightName.class
		, GLParameterRangeEXT.class
		, GLQueryTarget.class
		, GLQueryCounterTarget.class
		, GLPathTransformType.class
		, GLContainerType.class
		, GLProgramInterfacePName.class
		, GLPathCoordType.class
		, GLElementPointerTypeATI.class
		, GLSyncStatus.class
		, GLFogDistanceModeNV.class
		, GLImageTransformTargetHP.class
		, GLCombinerParameterNV.class
		, GLTextureUnit.class
		, GLCombinerStageNV.class
		, GLMapTextureFormatINTEL.class
		, GLUseProgramStageMask.class
		, GLColorBuffer.class
		, GLPathStringFormat.class
		, GLUniformType.class
		, GLFragmentShaderRegATI.class
		, GLListNameType.class
		, GLContextProfileMask.class
		, GLDebugSource.class
		, GLTransformFeedbackTokenNV.class
		, GLDepthFunction.class
		, GLTriangleListSUN.class
		, GLMeshMode2.class
		, GLTextureCoordName.class
		, GLFragmentLightParameterSGIX.class
		, GLCombinerPortionNV.class
		, GLLogicOp.class
		, GLListParameterName.class
		, GLVertexAttribEnumNV.class
		, GLGetTexBumpParameterATI.class
		, GLShaderBinaryFormat.class
		, GLQueryObjectParameterName.class
		, GLVertexBufferObjectUsage.class
		, GLClipControlOrigin.class
		, GLPathMetricMask.class
		, GLStencilOp.class
		, GLFragmentShaderTextureSourceATI.class
		, GLConvolutionTarget.class
		, GLSamplerParameterF.class
		, GLObjectIdentifier.class
		, GLSamplerParameterI.class
		, GLFramebufferStatus.class
		, GLTextureStorageMaskAMD.class
		, GLEvalTargetNV.class
		, GLSyncBehaviorFlags.class
		, GLVertexShaderParameterEXT.class
		);
	}

	public static final class Functions {
		public static final UnlinkedGLFunction glAccum = make_glAccum();
		public static final UnlinkedGLFunction glAccumxOES = make_glAccumxOES();
		public static final UnlinkedGLFunction glActiveProgramEXT = make_glActiveProgramEXT();
		public static final UnlinkedGLFunction glActiveShaderProgram = make_glActiveShaderProgram();
		public static final UnlinkedGLFunction glActiveShaderProgramEXT = make_glActiveShaderProgramEXT();
		public static final UnlinkedGLFunction glActiveStencilFaceEXT = make_glActiveStencilFaceEXT();
		public static final UnlinkedGLFunction glActiveTexture = make_glActiveTexture();
		public static final UnlinkedGLFunction glActiveTextureARB = make_glActiveTextureARB();
		public static final UnlinkedGLFunction glActiveVaryingNV = make_glActiveVaryingNV();
		public static final UnlinkedGLFunction glAlphaFragmentOp1ATI = make_glAlphaFragmentOp1ATI();
		public static final UnlinkedGLFunction glAlphaFragmentOp2ATI = make_glAlphaFragmentOp2ATI();
		public static final UnlinkedGLFunction glAlphaFragmentOp3ATI = make_glAlphaFragmentOp3ATI();
		public static final UnlinkedGLFunction glAlphaFunc = make_glAlphaFunc();
		public static final UnlinkedGLFunction glAlphaFuncQCOM = make_glAlphaFuncQCOM();
		public static final UnlinkedGLFunction glAlphaFuncx = make_glAlphaFuncx();
		public static final UnlinkedGLFunction glAlphaFuncxOES = make_glAlphaFuncxOES();
		public static final UnlinkedGLFunction glAlphaToCoverageDitherControlNV = make_glAlphaToCoverageDitherControlNV();
		public static final UnlinkedGLFunction glApplyFramebufferAttachmentCMAAINTEL = make_glApplyFramebufferAttachmentCMAAINTEL();
		public static final UnlinkedGLFunction glApplyTextureEXT = make_glApplyTextureEXT();
		public static final UnlinkedGLFunction glAcquireKeyedMutexWin32EXT = make_glAcquireKeyedMutexWin32EXT();
		public static final UnlinkedGLFunction glAreProgramsResidentNV = make_glAreProgramsResidentNV();
		public static final UnlinkedGLFunction glAreTexturesResident = make_glAreTexturesResident();
		public static final UnlinkedGLFunction glAreTexturesResidentEXT = make_glAreTexturesResidentEXT();
		public static final UnlinkedGLFunction glArrayElement = make_glArrayElement();
		public static final UnlinkedGLFunction glArrayElementEXT = make_glArrayElementEXT();
		public static final UnlinkedGLFunction glArrayObjectATI = make_glArrayObjectATI();
		public static final UnlinkedGLFunction glAsyncCopyBufferSubDataNVX = make_glAsyncCopyBufferSubDataNVX();
		public static final UnlinkedGLFunction glAsyncCopyImageSubDataNVX = make_glAsyncCopyImageSubDataNVX();
		public static final UnlinkedGLFunction glAsyncMarkerSGIX = make_glAsyncMarkerSGIX();
		public static final UnlinkedGLFunction glAttachObjectARB = make_glAttachObjectARB();
		public static final UnlinkedGLFunction glAttachShader = make_glAttachShader();
		public static final UnlinkedGLFunction glBegin = make_glBegin();
		public static final UnlinkedGLFunction glBeginConditionalRender = make_glBeginConditionalRender();
		public static final UnlinkedGLFunction glBeginConditionalRenderNV = make_glBeginConditionalRenderNV();
		public static final UnlinkedGLFunction glBeginConditionalRenderNVX = make_glBeginConditionalRenderNVX();
		public static final UnlinkedGLFunction glBeginFragmentShaderATI = make_glBeginFragmentShaderATI();
		public static final UnlinkedGLFunction glBeginOcclusionQueryNV = make_glBeginOcclusionQueryNV();
		public static final UnlinkedGLFunction glBeginPerfMonitorAMD = make_glBeginPerfMonitorAMD();
		public static final UnlinkedGLFunction glBeginPerfQueryINTEL = make_glBeginPerfQueryINTEL();
		public static final UnlinkedGLFunction glBeginQuery = make_glBeginQuery();
		public static final UnlinkedGLFunction glBeginQueryARB = make_glBeginQueryARB();
		public static final UnlinkedGLFunction glBeginQueryEXT = make_glBeginQueryEXT();
		public static final UnlinkedGLFunction glBeginQueryIndexed = make_glBeginQueryIndexed();
		public static final UnlinkedGLFunction glBeginTransformFeedback = make_glBeginTransformFeedback();
		public static final UnlinkedGLFunction glBeginTransformFeedbackEXT = make_glBeginTransformFeedbackEXT();
		public static final UnlinkedGLFunction glBeginTransformFeedbackNV = make_glBeginTransformFeedbackNV();
		public static final UnlinkedGLFunction glBeginVertexShaderEXT = make_glBeginVertexShaderEXT();
		public static final UnlinkedGLFunction glBeginVideoCaptureNV = make_glBeginVideoCaptureNV();
		public static final UnlinkedGLFunction glBindAttribLocation = make_glBindAttribLocation();
		public static final UnlinkedGLFunction glBindAttribLocationARB = make_glBindAttribLocationARB();
		public static final UnlinkedGLFunction glBindBuffer = make_glBindBuffer();
		public static final UnlinkedGLFunction glBindBufferARB = make_glBindBufferARB();
		public static final UnlinkedGLFunction glBindBufferBase = make_glBindBufferBase();
		public static final UnlinkedGLFunction glBindBufferBaseEXT = make_glBindBufferBaseEXT();
		public static final UnlinkedGLFunction glBindBufferBaseNV = make_glBindBufferBaseNV();
		public static final UnlinkedGLFunction glBindBufferOffsetEXT = make_glBindBufferOffsetEXT();
		public static final UnlinkedGLFunction glBindBufferOffsetNV = make_glBindBufferOffsetNV();
		public static final UnlinkedGLFunction glBindBufferRange = make_glBindBufferRange();
		public static final UnlinkedGLFunction glBindBufferRangeEXT = make_glBindBufferRangeEXT();
		public static final UnlinkedGLFunction glBindBufferRangeNV = make_glBindBufferRangeNV();
		public static final UnlinkedGLFunction glBindBuffersBase = make_glBindBuffersBase();
		public static final UnlinkedGLFunction glBindBuffersRange = make_glBindBuffersRange();
		public static final UnlinkedGLFunction glBindFragDataLocation = make_glBindFragDataLocation();
		public static final UnlinkedGLFunction glBindFragDataLocationEXT = make_glBindFragDataLocationEXT();
		public static final UnlinkedGLFunction glBindFragDataLocationIndexed = make_glBindFragDataLocationIndexed();
		public static final UnlinkedGLFunction glBindFragDataLocationIndexedEXT = make_glBindFragDataLocationIndexedEXT();
		public static final UnlinkedGLFunction glBindFragmentShaderATI = make_glBindFragmentShaderATI();
		public static final UnlinkedGLFunction glBindFramebuffer = make_glBindFramebuffer();
		public static final UnlinkedGLFunction glBindFramebufferEXT = make_glBindFramebufferEXT();
		public static final UnlinkedGLFunction glBindFramebufferOES = make_glBindFramebufferOES();
		public static final UnlinkedGLFunction glBindImageTexture = make_glBindImageTexture();
		public static final UnlinkedGLFunction glBindImageTextureEXT = make_glBindImageTextureEXT();
		public static final UnlinkedGLFunction glBindImageTextures = make_glBindImageTextures();
		public static final UnlinkedGLFunction glBindLightParameterEXT = make_glBindLightParameterEXT();
		public static final UnlinkedGLFunction glBindMaterialParameterEXT = make_glBindMaterialParameterEXT();
		public static final UnlinkedGLFunction glBindMultiTextureEXT = make_glBindMultiTextureEXT();
		public static final UnlinkedGLFunction glBindParameterEXT = make_glBindParameterEXT();
		public static final UnlinkedGLFunction glBindProgramARB = make_glBindProgramARB();
		public static final UnlinkedGLFunction glBindProgramNV = make_glBindProgramNV();
		public static final UnlinkedGLFunction glBindProgramPipeline = make_glBindProgramPipeline();
		public static final UnlinkedGLFunction glBindProgramPipelineEXT = make_glBindProgramPipelineEXT();
		public static final UnlinkedGLFunction glBindRenderbuffer = make_glBindRenderbuffer();
		public static final UnlinkedGLFunction glBindRenderbufferEXT = make_glBindRenderbufferEXT();
		public static final UnlinkedGLFunction glBindRenderbufferOES = make_glBindRenderbufferOES();
		public static final UnlinkedGLFunction glBindSampler = make_glBindSampler();
		public static final UnlinkedGLFunction glBindSamplers = make_glBindSamplers();
		public static final UnlinkedGLFunction glBindShadingRateImageNV = make_glBindShadingRateImageNV();
		public static final UnlinkedGLFunction glBindTexGenParameterEXT = make_glBindTexGenParameterEXT();
		public static final UnlinkedGLFunction glBindTexture = make_glBindTexture();
		public static final UnlinkedGLFunction glBindTextureEXT = make_glBindTextureEXT();
		public static final UnlinkedGLFunction glBindTextureUnit = make_glBindTextureUnit();
		public static final UnlinkedGLFunction glBindTextureUnitParameterEXT = make_glBindTextureUnitParameterEXT();
		public static final UnlinkedGLFunction glBindTextures = make_glBindTextures();
		public static final UnlinkedGLFunction glBindTransformFeedback = make_glBindTransformFeedback();
		public static final UnlinkedGLFunction glBindTransformFeedbackNV = make_glBindTransformFeedbackNV();
		public static final UnlinkedGLFunction glBindVertexArray = make_glBindVertexArray();
		public static final UnlinkedGLFunction glBindVertexArrayAPPLE = make_glBindVertexArrayAPPLE();
		public static final UnlinkedGLFunction glBindVertexArrayOES = make_glBindVertexArrayOES();
		public static final UnlinkedGLFunction glBindVertexBuffer = make_glBindVertexBuffer();
		public static final UnlinkedGLFunction glBindVertexBuffers = make_glBindVertexBuffers();
		public static final UnlinkedGLFunction glBindVertexShaderEXT = make_glBindVertexShaderEXT();
		public static final UnlinkedGLFunction glBindVideoCaptureStreamBufferNV = make_glBindVideoCaptureStreamBufferNV();
		public static final UnlinkedGLFunction glBindVideoCaptureStreamTextureNV = make_glBindVideoCaptureStreamTextureNV();
		public static final UnlinkedGLFunction glBinormal3bEXT = make_glBinormal3bEXT();
		public static final UnlinkedGLFunction glBinormal3bvEXT = make_glBinormal3bvEXT();
		public static final UnlinkedGLFunction glBinormal3dEXT = make_glBinormal3dEXT();
		public static final UnlinkedGLFunction glBinormal3dvEXT = make_glBinormal3dvEXT();
		public static final UnlinkedGLFunction glBinormal3fEXT = make_glBinormal3fEXT();
		public static final UnlinkedGLFunction glBinormal3fvEXT = make_glBinormal3fvEXT();
		public static final UnlinkedGLFunction glBinormal3iEXT = make_glBinormal3iEXT();
		public static final UnlinkedGLFunction glBinormal3ivEXT = make_glBinormal3ivEXT();
		public static final UnlinkedGLFunction glBinormal3sEXT = make_glBinormal3sEXT();
		public static final UnlinkedGLFunction glBinormal3svEXT = make_glBinormal3svEXT();
		public static final UnlinkedGLFunction glBinormalPointerEXT = make_glBinormalPointerEXT();
		public static final UnlinkedGLFunction glBitmap = make_glBitmap();
		public static final UnlinkedGLFunction glBitmapxOES = make_glBitmapxOES();
		public static final UnlinkedGLFunction glBlendBarrier = make_glBlendBarrier();
		public static final UnlinkedGLFunction glBlendBarrierKHR = make_glBlendBarrierKHR();
		public static final UnlinkedGLFunction glBlendBarrierNV = make_glBlendBarrierNV();
		public static final UnlinkedGLFunction glBlendColor = make_glBlendColor();
		public static final UnlinkedGLFunction glBlendColorEXT = make_glBlendColorEXT();
		public static final UnlinkedGLFunction glBlendColorxOES = make_glBlendColorxOES();
		public static final UnlinkedGLFunction glBlendEquation = make_glBlendEquation();
		public static final UnlinkedGLFunction glBlendEquationEXT = make_glBlendEquationEXT();
		public static final UnlinkedGLFunction glBlendEquationIndexedAMD = make_glBlendEquationIndexedAMD();
		public static final UnlinkedGLFunction glBlendEquationOES = make_glBlendEquationOES();
		public static final UnlinkedGLFunction glBlendEquationSeparate = make_glBlendEquationSeparate();
		public static final UnlinkedGLFunction glBlendEquationSeparateEXT = make_glBlendEquationSeparateEXT();
		public static final UnlinkedGLFunction glBlendEquationSeparateIndexedAMD = make_glBlendEquationSeparateIndexedAMD();
		public static final UnlinkedGLFunction glBlendEquationSeparateOES = make_glBlendEquationSeparateOES();
		public static final UnlinkedGLFunction glBlendEquationSeparatei = make_glBlendEquationSeparatei();
		public static final UnlinkedGLFunction glBlendEquationSeparateiARB = make_glBlendEquationSeparateiARB();
		public static final UnlinkedGLFunction glBlendEquationSeparateiEXT = make_glBlendEquationSeparateiEXT();
		public static final UnlinkedGLFunction glBlendEquationSeparateiOES = make_glBlendEquationSeparateiOES();
		public static final UnlinkedGLFunction glBlendEquationi = make_glBlendEquationi();
		public static final UnlinkedGLFunction glBlendEquationiARB = make_glBlendEquationiARB();
		public static final UnlinkedGLFunction glBlendEquationiEXT = make_glBlendEquationiEXT();
		public static final UnlinkedGLFunction glBlendEquationiOES = make_glBlendEquationiOES();
		public static final UnlinkedGLFunction glBlendFunc = make_glBlendFunc();
		public static final UnlinkedGLFunction glBlendFuncIndexedAMD = make_glBlendFuncIndexedAMD();
		public static final UnlinkedGLFunction glBlendFuncSeparate = make_glBlendFuncSeparate();
		public static final UnlinkedGLFunction glBlendFuncSeparateEXT = make_glBlendFuncSeparateEXT();
		public static final UnlinkedGLFunction glBlendFuncSeparateINGR = make_glBlendFuncSeparateINGR();
		public static final UnlinkedGLFunction glBlendFuncSeparateIndexedAMD = make_glBlendFuncSeparateIndexedAMD();
		public static final UnlinkedGLFunction glBlendFuncSeparateOES = make_glBlendFuncSeparateOES();
		public static final UnlinkedGLFunction glBlendFuncSeparatei = make_glBlendFuncSeparatei();
		public static final UnlinkedGLFunction glBlendFuncSeparateiARB = make_glBlendFuncSeparateiARB();
		public static final UnlinkedGLFunction glBlendFuncSeparateiEXT = make_glBlendFuncSeparateiEXT();
		public static final UnlinkedGLFunction glBlendFuncSeparateiOES = make_glBlendFuncSeparateiOES();
		public static final UnlinkedGLFunction glBlendFunci = make_glBlendFunci();
		public static final UnlinkedGLFunction glBlendFunciARB = make_glBlendFunciARB();
		public static final UnlinkedGLFunction glBlendFunciEXT = make_glBlendFunciEXT();
		public static final UnlinkedGLFunction glBlendFunciOES = make_glBlendFunciOES();
		public static final UnlinkedGLFunction glBlendParameteriNV = make_glBlendParameteriNV();
		public static final UnlinkedGLFunction glBlitFramebuffer = make_glBlitFramebuffer();
		public static final UnlinkedGLFunction glBlitFramebufferANGLE = make_glBlitFramebufferANGLE();
		public static final UnlinkedGLFunction glBlitFramebufferEXT = make_glBlitFramebufferEXT();
		public static final UnlinkedGLFunction glBlitFramebufferLayerEXT = make_glBlitFramebufferLayerEXT();
		public static final UnlinkedGLFunction glBlitFramebufferLayersEXT = make_glBlitFramebufferLayersEXT();
		public static final UnlinkedGLFunction glBlitFramebufferNV = make_glBlitFramebufferNV();
		public static final UnlinkedGLFunction glBlitNamedFramebuffer = make_glBlitNamedFramebuffer();
		public static final UnlinkedGLFunction glBufferAddressRangeNV = make_glBufferAddressRangeNV();
		public static final UnlinkedGLFunction glBufferAttachMemoryNV = make_glBufferAttachMemoryNV();
		public static final UnlinkedGLFunction glBufferData = make_glBufferData();
		public static final UnlinkedGLFunction glBufferDataARB = make_glBufferDataARB();
		public static final UnlinkedGLFunction glBufferPageCommitmentARB = make_glBufferPageCommitmentARB();
		public static final UnlinkedGLFunction glBufferPageCommitmentMemNV = make_glBufferPageCommitmentMemNV();
		public static final UnlinkedGLFunction glBufferParameteriAPPLE = make_glBufferParameteriAPPLE();
		public static final UnlinkedGLFunction glBufferStorage = make_glBufferStorage();
		public static final UnlinkedGLFunction glBufferStorageEXT = make_glBufferStorageEXT();
		public static final UnlinkedGLFunction glBufferStorageExternalEXT = make_glBufferStorageExternalEXT();
		public static final UnlinkedGLFunction glBufferStorageMemEXT = make_glBufferStorageMemEXT();
		public static final UnlinkedGLFunction glBufferSubData = make_glBufferSubData();
		public static final UnlinkedGLFunction glBufferSubDataARB = make_glBufferSubDataARB();
		public static final UnlinkedGLFunction glCallCommandListNV = make_glCallCommandListNV();
		public static final UnlinkedGLFunction glCallList = make_glCallList();
		public static final UnlinkedGLFunction glCallLists = make_glCallLists();
		public static final UnlinkedGLFunction glCheckFramebufferStatus = make_glCheckFramebufferStatus();
		public static final UnlinkedGLFunction glCheckFramebufferStatusEXT = make_glCheckFramebufferStatusEXT();
		public static final UnlinkedGLFunction glCheckFramebufferStatusOES = make_glCheckFramebufferStatusOES();
		public static final UnlinkedGLFunction glCheckNamedFramebufferStatus = make_glCheckNamedFramebufferStatus();
		public static final UnlinkedGLFunction glCheckNamedFramebufferStatusEXT = make_glCheckNamedFramebufferStatusEXT();
		public static final UnlinkedGLFunction glClampColor = make_glClampColor();
		public static final UnlinkedGLFunction glClampColorARB = make_glClampColorARB();
		public static final UnlinkedGLFunction glClear = make_glClear();
		public static final UnlinkedGLFunction glClearAccum = make_glClearAccum();
		public static final UnlinkedGLFunction glClearAccumxOES = make_glClearAccumxOES();
		public static final UnlinkedGLFunction glClearBufferData = make_glClearBufferData();
		public static final UnlinkedGLFunction glClearBufferSubData = make_glClearBufferSubData();
		public static final UnlinkedGLFunction glClearBufferfi = make_glClearBufferfi();
		public static final UnlinkedGLFunction glClearBufferfv = make_glClearBufferfv();
		public static final UnlinkedGLFunction glClearBufferiv = make_glClearBufferiv();
		public static final UnlinkedGLFunction glClearBufferuiv = make_glClearBufferuiv();
		public static final UnlinkedGLFunction glClearColor = make_glClearColor();
		public static final UnlinkedGLFunction glClearColorIiEXT = make_glClearColorIiEXT();
		public static final UnlinkedGLFunction glClearColorIuiEXT = make_glClearColorIuiEXT();
		public static final UnlinkedGLFunction glClearColorx = make_glClearColorx();
		public static final UnlinkedGLFunction glClearColorxOES = make_glClearColorxOES();
		public static final UnlinkedGLFunction glClearDepth = make_glClearDepth();
		public static final UnlinkedGLFunction glClearDepthdNV = make_glClearDepthdNV();
		public static final UnlinkedGLFunction glClearDepthf = make_glClearDepthf();
		public static final UnlinkedGLFunction glClearDepthfOES = make_glClearDepthfOES();
		public static final UnlinkedGLFunction glClearDepthx = make_glClearDepthx();
		public static final UnlinkedGLFunction glClearDepthxOES = make_glClearDepthxOES();
		public static final UnlinkedGLFunction glClearIndex = make_glClearIndex();
		public static final UnlinkedGLFunction glClearNamedBufferData = make_glClearNamedBufferData();
		public static final UnlinkedGLFunction glClearNamedBufferDataEXT = make_glClearNamedBufferDataEXT();
		public static final UnlinkedGLFunction glClearNamedBufferSubData = make_glClearNamedBufferSubData();
		public static final UnlinkedGLFunction glClearNamedBufferSubDataEXT = make_glClearNamedBufferSubDataEXT();
		public static final UnlinkedGLFunction glClearNamedFramebufferfi = make_glClearNamedFramebufferfi();
		public static final UnlinkedGLFunction glClearNamedFramebufferfv = make_glClearNamedFramebufferfv();
		public static final UnlinkedGLFunction glClearNamedFramebufferiv = make_glClearNamedFramebufferiv();
		public static final UnlinkedGLFunction glClearNamedFramebufferuiv = make_glClearNamedFramebufferuiv();
		public static final UnlinkedGLFunction glClearPixelLocalStorageuiEXT = make_glClearPixelLocalStorageuiEXT();
		public static final UnlinkedGLFunction glClearStencil = make_glClearStencil();
		public static final UnlinkedGLFunction glClearTexImage = make_glClearTexImage();
		public static final UnlinkedGLFunction glClearTexImageEXT = make_glClearTexImageEXT();
		public static final UnlinkedGLFunction glClearTexSubImage = make_glClearTexSubImage();
		public static final UnlinkedGLFunction glClearTexSubImageEXT = make_glClearTexSubImageEXT();
		public static final UnlinkedGLFunction glClientActiveTexture = make_glClientActiveTexture();
		public static final UnlinkedGLFunction glClientActiveTextureARB = make_glClientActiveTextureARB();
		public static final UnlinkedGLFunction glClientActiveVertexStreamATI = make_glClientActiveVertexStreamATI();
		public static final UnlinkedGLFunction glClientAttribDefaultEXT = make_glClientAttribDefaultEXT();
		public static final UnlinkedGLFunction glClientWaitSemaphoreui64NVX = make_glClientWaitSemaphoreui64NVX();
		public static final UnlinkedGLFunction glClientWaitSync = make_glClientWaitSync();
		public static final UnlinkedGLFunction glClientWaitSyncAPPLE = make_glClientWaitSyncAPPLE();
		public static final UnlinkedGLFunction glClipControl = make_glClipControl();
		public static final UnlinkedGLFunction glClipControlEXT = make_glClipControlEXT();
		public static final UnlinkedGLFunction glClipPlane = make_glClipPlane();
		public static final UnlinkedGLFunction glClipPlanef = make_glClipPlanef();
		public static final UnlinkedGLFunction glClipPlanefIMG = make_glClipPlanefIMG();
		public static final UnlinkedGLFunction glClipPlanefOES = make_glClipPlanefOES();
		public static final UnlinkedGLFunction glClipPlanex = make_glClipPlanex();
		public static final UnlinkedGLFunction glClipPlanexIMG = make_glClipPlanexIMG();
		public static final UnlinkedGLFunction glClipPlanexOES = make_glClipPlanexOES();
		public static final UnlinkedGLFunction glColor3b = make_glColor3b();
		public static final UnlinkedGLFunction glColor3bv = make_glColor3bv();
		public static final UnlinkedGLFunction glColor3d = make_glColor3d();
		public static final UnlinkedGLFunction glColor3dv = make_glColor3dv();
		public static final UnlinkedGLFunction glColor3f = make_glColor3f();
		public static final UnlinkedGLFunction glColor3fVertex3fSUN = make_glColor3fVertex3fSUN();
		public static final UnlinkedGLFunction glColor3fVertex3fvSUN = make_glColor3fVertex3fvSUN();
		public static final UnlinkedGLFunction glColor3fv = make_glColor3fv();
		public static final UnlinkedGLFunction glColor3hNV = make_glColor3hNV();
		public static final UnlinkedGLFunction glColor3hvNV = make_glColor3hvNV();
		public static final UnlinkedGLFunction glColor3i = make_glColor3i();
		public static final UnlinkedGLFunction glColor3iv = make_glColor3iv();
		public static final UnlinkedGLFunction glColor3s = make_glColor3s();
		public static final UnlinkedGLFunction glColor3sv = make_glColor3sv();
		public static final UnlinkedGLFunction glColor3ub = make_glColor3ub();
		public static final UnlinkedGLFunction glColor3ubv = make_glColor3ubv();
		public static final UnlinkedGLFunction glColor3ui = make_glColor3ui();
		public static final UnlinkedGLFunction glColor3uiv = make_glColor3uiv();
		public static final UnlinkedGLFunction glColor3us = make_glColor3us();
		public static final UnlinkedGLFunction glColor3usv = make_glColor3usv();
		public static final UnlinkedGLFunction glColor3xOES = make_glColor3xOES();
		public static final UnlinkedGLFunction glColor3xvOES = make_glColor3xvOES();
		public static final UnlinkedGLFunction glColor4b = make_glColor4b();
		public static final UnlinkedGLFunction glColor4bv = make_glColor4bv();
		public static final UnlinkedGLFunction glColor4d = make_glColor4d();
		public static final UnlinkedGLFunction glColor4dv = make_glColor4dv();
		public static final UnlinkedGLFunction glColor4f = make_glColor4f();
		public static final UnlinkedGLFunction glColor4fNormal3fVertex3fSUN = make_glColor4fNormal3fVertex3fSUN();
		public static final UnlinkedGLFunction glColor4fNormal3fVertex3fvSUN = make_glColor4fNormal3fVertex3fvSUN();
		public static final UnlinkedGLFunction glColor4fv = make_glColor4fv();
		public static final UnlinkedGLFunction glColor4hNV = make_glColor4hNV();
		public static final UnlinkedGLFunction glColor4hvNV = make_glColor4hvNV();
		public static final UnlinkedGLFunction glColor4i = make_glColor4i();
		public static final UnlinkedGLFunction glColor4iv = make_glColor4iv();
		public static final UnlinkedGLFunction glColor4s = make_glColor4s();
		public static final UnlinkedGLFunction glColor4sv = make_glColor4sv();
		public static final UnlinkedGLFunction glColor4ub = make_glColor4ub();
		public static final UnlinkedGLFunction glColor4ubVertex2fSUN = make_glColor4ubVertex2fSUN();
		public static final UnlinkedGLFunction glColor4ubVertex2fvSUN = make_glColor4ubVertex2fvSUN();
		public static final UnlinkedGLFunction glColor4ubVertex3fSUN = make_glColor4ubVertex3fSUN();
		public static final UnlinkedGLFunction glColor4ubVertex3fvSUN = make_glColor4ubVertex3fvSUN();
		public static final UnlinkedGLFunction glColor4ubv = make_glColor4ubv();
		public static final UnlinkedGLFunction glColor4ui = make_glColor4ui();
		public static final UnlinkedGLFunction glColor4uiv = make_glColor4uiv();
		public static final UnlinkedGLFunction glColor4us = make_glColor4us();
		public static final UnlinkedGLFunction glColor4usv = make_glColor4usv();
		public static final UnlinkedGLFunction glColor4x = make_glColor4x();
		public static final UnlinkedGLFunction glColor4xOES = make_glColor4xOES();
		public static final UnlinkedGLFunction glColor4xvOES = make_glColor4xvOES();
		public static final UnlinkedGLFunction glColorFormatNV = make_glColorFormatNV();
		public static final UnlinkedGLFunction glColorFragmentOp1ATI = make_glColorFragmentOp1ATI();
		public static final UnlinkedGLFunction glColorFragmentOp2ATI = make_glColorFragmentOp2ATI();
		public static final UnlinkedGLFunction glColorFragmentOp3ATI = make_glColorFragmentOp3ATI();
		public static final UnlinkedGLFunction glColorMask = make_glColorMask();
		public static final UnlinkedGLFunction glColorMaskIndexedEXT = make_glColorMaskIndexedEXT();
		public static final UnlinkedGLFunction glColorMaski = make_glColorMaski();
		public static final UnlinkedGLFunction glColorMaskiEXT = make_glColorMaskiEXT();
		public static final UnlinkedGLFunction glColorMaskiOES = make_glColorMaskiOES();
		public static final UnlinkedGLFunction glColorMaterial = make_glColorMaterial();
		public static final UnlinkedGLFunction glColorP3ui = make_glColorP3ui();
		public static final UnlinkedGLFunction glColorP3uiv = make_glColorP3uiv();
		public static final UnlinkedGLFunction glColorP4ui = make_glColorP4ui();
		public static final UnlinkedGLFunction glColorP4uiv = make_glColorP4uiv();
		public static final UnlinkedGLFunction glColorPointer = make_glColorPointer();
		public static final UnlinkedGLFunction glColorPointerEXT = make_glColorPointerEXT();
		public static final UnlinkedGLFunction glColorPointerListIBM = make_glColorPointerListIBM();
		public static final UnlinkedGLFunction glColorPointervINTEL = make_glColorPointervINTEL();
		public static final UnlinkedGLFunction glColorSubTable = make_glColorSubTable();
		public static final UnlinkedGLFunction glColorSubTableEXT = make_glColorSubTableEXT();
		public static final UnlinkedGLFunction glColorTable = make_glColorTable();
		public static final UnlinkedGLFunction glColorTableEXT = make_glColorTableEXT();
		public static final UnlinkedGLFunction glColorTableParameterfv = make_glColorTableParameterfv();
		public static final UnlinkedGLFunction glColorTableParameterfvSGI = make_glColorTableParameterfvSGI();
		public static final UnlinkedGLFunction glColorTableParameteriv = make_glColorTableParameteriv();
		public static final UnlinkedGLFunction glColorTableParameterivSGI = make_glColorTableParameterivSGI();
		public static final UnlinkedGLFunction glColorTableSGI = make_glColorTableSGI();
		public static final UnlinkedGLFunction glCombinerInputNV = make_glCombinerInputNV();
		public static final UnlinkedGLFunction glCombinerOutputNV = make_glCombinerOutputNV();
		public static final UnlinkedGLFunction glCombinerParameterfNV = make_glCombinerParameterfNV();
		public static final UnlinkedGLFunction glCombinerParameterfvNV = make_glCombinerParameterfvNV();
		public static final UnlinkedGLFunction glCombinerParameteriNV = make_glCombinerParameteriNV();
		public static final UnlinkedGLFunction glCombinerParameterivNV = make_glCombinerParameterivNV();
		public static final UnlinkedGLFunction glCombinerStageParameterfvNV = make_glCombinerStageParameterfvNV();
		public static final UnlinkedGLFunction glCommandListSegmentsNV = make_glCommandListSegmentsNV();
		public static final UnlinkedGLFunction glCompileCommandListNV = make_glCompileCommandListNV();
		public static final UnlinkedGLFunction glCompileShader = make_glCompileShader();
		public static final UnlinkedGLFunction glCompileShaderARB = make_glCompileShaderARB();
		public static final UnlinkedGLFunction glCompileShaderIncludeARB = make_glCompileShaderIncludeARB();
		public static final UnlinkedGLFunction glCompressedMultiTexImage1DEXT = make_glCompressedMultiTexImage1DEXT();
		public static final UnlinkedGLFunction glCompressedMultiTexImage2DEXT = make_glCompressedMultiTexImage2DEXT();
		public static final UnlinkedGLFunction glCompressedMultiTexImage3DEXT = make_glCompressedMultiTexImage3DEXT();
		public static final UnlinkedGLFunction glCompressedMultiTexSubImage1DEXT = make_glCompressedMultiTexSubImage1DEXT();
		public static final UnlinkedGLFunction glCompressedMultiTexSubImage2DEXT = make_glCompressedMultiTexSubImage2DEXT();
		public static final UnlinkedGLFunction glCompressedMultiTexSubImage3DEXT = make_glCompressedMultiTexSubImage3DEXT();
		public static final UnlinkedGLFunction glCompressedTexImage1D = make_glCompressedTexImage1D();
		public static final UnlinkedGLFunction glCompressedTexImage1DARB = make_glCompressedTexImage1DARB();
		public static final UnlinkedGLFunction glCompressedTexImage2D = make_glCompressedTexImage2D();
		public static final UnlinkedGLFunction glCompressedTexImage2DARB = make_glCompressedTexImage2DARB();
		public static final UnlinkedGLFunction glCompressedTexImage3D = make_glCompressedTexImage3D();
		public static final UnlinkedGLFunction glCompressedTexImage3DARB = make_glCompressedTexImage3DARB();
		public static final UnlinkedGLFunction glCompressedTexImage3DOES = make_glCompressedTexImage3DOES();
		public static final UnlinkedGLFunction glCompressedTexSubImage1D = make_glCompressedTexSubImage1D();
		public static final UnlinkedGLFunction glCompressedTexSubImage1DARB = make_glCompressedTexSubImage1DARB();
		public static final UnlinkedGLFunction glCompressedTexSubImage2D = make_glCompressedTexSubImage2D();
		public static final UnlinkedGLFunction glCompressedTexSubImage2DARB = make_glCompressedTexSubImage2DARB();
		public static final UnlinkedGLFunction glCompressedTexSubImage3D = make_glCompressedTexSubImage3D();
		public static final UnlinkedGLFunction glCompressedTexSubImage3DARB = make_glCompressedTexSubImage3DARB();
		public static final UnlinkedGLFunction glCompressedTexSubImage3DOES = make_glCompressedTexSubImage3DOES();
		public static final UnlinkedGLFunction glCompressedTextureImage1DEXT = make_glCompressedTextureImage1DEXT();
		public static final UnlinkedGLFunction glCompressedTextureImage2DEXT = make_glCompressedTextureImage2DEXT();
		public static final UnlinkedGLFunction glCompressedTextureImage3DEXT = make_glCompressedTextureImage3DEXT();
		public static final UnlinkedGLFunction glCompressedTextureSubImage1D = make_glCompressedTextureSubImage1D();
		public static final UnlinkedGLFunction glCompressedTextureSubImage1DEXT = make_glCompressedTextureSubImage1DEXT();
		public static final UnlinkedGLFunction glCompressedTextureSubImage2D = make_glCompressedTextureSubImage2D();
		public static final UnlinkedGLFunction glCompressedTextureSubImage2DEXT = make_glCompressedTextureSubImage2DEXT();
		public static final UnlinkedGLFunction glCompressedTextureSubImage3D = make_glCompressedTextureSubImage3D();
		public static final UnlinkedGLFunction glCompressedTextureSubImage3DEXT = make_glCompressedTextureSubImage3DEXT();
		public static final UnlinkedGLFunction glConservativeRasterParameterfNV = make_glConservativeRasterParameterfNV();
		public static final UnlinkedGLFunction glConservativeRasterParameteriNV = make_glConservativeRasterParameteriNV();
		public static final UnlinkedGLFunction glConvolutionFilter1D = make_glConvolutionFilter1D();
		public static final UnlinkedGLFunction glConvolutionFilter1DEXT = make_glConvolutionFilter1DEXT();
		public static final UnlinkedGLFunction glConvolutionFilter2D = make_glConvolutionFilter2D();
		public static final UnlinkedGLFunction glConvolutionFilter2DEXT = make_glConvolutionFilter2DEXT();
		public static final UnlinkedGLFunction glConvolutionParameterf = make_glConvolutionParameterf();
		public static final UnlinkedGLFunction glConvolutionParameterfEXT = make_glConvolutionParameterfEXT();
		public static final UnlinkedGLFunction glConvolutionParameterfv = make_glConvolutionParameterfv();
		public static final UnlinkedGLFunction glConvolutionParameterfvEXT = make_glConvolutionParameterfvEXT();
		public static final UnlinkedGLFunction glConvolutionParameteri = make_glConvolutionParameteri();
		public static final UnlinkedGLFunction glConvolutionParameteriEXT = make_glConvolutionParameteriEXT();
		public static final UnlinkedGLFunction glConvolutionParameteriv = make_glConvolutionParameteriv();
		public static final UnlinkedGLFunction glConvolutionParameterivEXT = make_glConvolutionParameterivEXT();
		public static final UnlinkedGLFunction glConvolutionParameterxOES = make_glConvolutionParameterxOES();
		public static final UnlinkedGLFunction glConvolutionParameterxvOES = make_glConvolutionParameterxvOES();
		public static final UnlinkedGLFunction glCopyBufferSubData = make_glCopyBufferSubData();
		public static final UnlinkedGLFunction glCopyBufferSubDataNV = make_glCopyBufferSubDataNV();
		public static final UnlinkedGLFunction glCopyColorSubTable = make_glCopyColorSubTable();
		public static final UnlinkedGLFunction glCopyColorSubTableEXT = make_glCopyColorSubTableEXT();
		public static final UnlinkedGLFunction glCopyColorTable = make_glCopyColorTable();
		public static final UnlinkedGLFunction glCopyColorTableSGI = make_glCopyColorTableSGI();
		public static final UnlinkedGLFunction glCopyConvolutionFilter1D = make_glCopyConvolutionFilter1D();
		public static final UnlinkedGLFunction glCopyConvolutionFilter1DEXT = make_glCopyConvolutionFilter1DEXT();
		public static final UnlinkedGLFunction glCopyConvolutionFilter2D = make_glCopyConvolutionFilter2D();
		public static final UnlinkedGLFunction glCopyConvolutionFilter2DEXT = make_glCopyConvolutionFilter2DEXT();
		public static final UnlinkedGLFunction glCopyImageSubData = make_glCopyImageSubData();
		public static final UnlinkedGLFunction glCopyImageSubDataEXT = make_glCopyImageSubDataEXT();
		public static final UnlinkedGLFunction glCopyImageSubDataNV = make_glCopyImageSubDataNV();
		public static final UnlinkedGLFunction glCopyImageSubDataOES = make_glCopyImageSubDataOES();
		public static final UnlinkedGLFunction glCopyMultiTexImage1DEXT = make_glCopyMultiTexImage1DEXT();
		public static final UnlinkedGLFunction glCopyMultiTexImage2DEXT = make_glCopyMultiTexImage2DEXT();
		public static final UnlinkedGLFunction glCopyMultiTexSubImage1DEXT = make_glCopyMultiTexSubImage1DEXT();
		public static final UnlinkedGLFunction glCopyMultiTexSubImage2DEXT = make_glCopyMultiTexSubImage2DEXT();
		public static final UnlinkedGLFunction glCopyMultiTexSubImage3DEXT = make_glCopyMultiTexSubImage3DEXT();
		public static final UnlinkedGLFunction glCopyNamedBufferSubData = make_glCopyNamedBufferSubData();
		public static final UnlinkedGLFunction glCopyPathNV = make_glCopyPathNV();
		public static final UnlinkedGLFunction glCopyPixels = make_glCopyPixels();
		public static final UnlinkedGLFunction glCopyTexImage1D = make_glCopyTexImage1D();
		public static final UnlinkedGLFunction glCopyTexImage1DEXT = make_glCopyTexImage1DEXT();
		public static final UnlinkedGLFunction glCopyTexImage2D = make_glCopyTexImage2D();
		public static final UnlinkedGLFunction glCopyTexImage2DEXT = make_glCopyTexImage2DEXT();
		public static final UnlinkedGLFunction glCopyTexSubImage1D = make_glCopyTexSubImage1D();
		public static final UnlinkedGLFunction glCopyTexSubImage1DEXT = make_glCopyTexSubImage1DEXT();
		public static final UnlinkedGLFunction glCopyTexSubImage2D = make_glCopyTexSubImage2D();
		public static final UnlinkedGLFunction glCopyTexSubImage2DEXT = make_glCopyTexSubImage2DEXT();
		public static final UnlinkedGLFunction glCopyTexSubImage3D = make_glCopyTexSubImage3D();
		public static final UnlinkedGLFunction glCopyTexSubImage3DEXT = make_glCopyTexSubImage3DEXT();
		public static final UnlinkedGLFunction glCopyTexSubImage3DOES = make_glCopyTexSubImage3DOES();
		public static final UnlinkedGLFunction glCopyTextureImage1DEXT = make_glCopyTextureImage1DEXT();
		public static final UnlinkedGLFunction glCopyTextureImage2DEXT = make_glCopyTextureImage2DEXT();
		public static final UnlinkedGLFunction glCopyTextureLevelsAPPLE = make_glCopyTextureLevelsAPPLE();
		public static final UnlinkedGLFunction glCopyTextureSubImage1D = make_glCopyTextureSubImage1D();
		public static final UnlinkedGLFunction glCopyTextureSubImage1DEXT = make_glCopyTextureSubImage1DEXT();
		public static final UnlinkedGLFunction glCopyTextureSubImage2D = make_glCopyTextureSubImage2D();
		public static final UnlinkedGLFunction glCopyTextureSubImage2DEXT = make_glCopyTextureSubImage2DEXT();
		public static final UnlinkedGLFunction glCopyTextureSubImage3D = make_glCopyTextureSubImage3D();
		public static final UnlinkedGLFunction glCopyTextureSubImage3DEXT = make_glCopyTextureSubImage3DEXT();
		public static final UnlinkedGLFunction glCoverFillPathInstancedNV = make_glCoverFillPathInstancedNV();
		public static final UnlinkedGLFunction glCoverFillPathNV = make_glCoverFillPathNV();
		public static final UnlinkedGLFunction glCoverStrokePathInstancedNV = make_glCoverStrokePathInstancedNV();
		public static final UnlinkedGLFunction glCoverStrokePathNV = make_glCoverStrokePathNV();
		public static final UnlinkedGLFunction glCoverageMaskNV = make_glCoverageMaskNV();
		public static final UnlinkedGLFunction glCoverageModulationNV = make_glCoverageModulationNV();
		public static final UnlinkedGLFunction glCoverageModulationTableNV = make_glCoverageModulationTableNV();
		public static final UnlinkedGLFunction glCoverageOperationNV = make_glCoverageOperationNV();
		public static final UnlinkedGLFunction glCreateBuffers = make_glCreateBuffers();
		public static final UnlinkedGLFunction glCreateCommandListsNV = make_glCreateCommandListsNV();
		public static final UnlinkedGLFunction glCreateFramebuffers = make_glCreateFramebuffers();
		public static final UnlinkedGLFunction glCreateMemoryObjectsEXT = make_glCreateMemoryObjectsEXT();
		public static final UnlinkedGLFunction glCreatePerfQueryINTEL = make_glCreatePerfQueryINTEL();
		public static final UnlinkedGLFunction glCreateProgram = make_glCreateProgram();
		public static final UnlinkedGLFunction glCreateProgramObjectARB = make_glCreateProgramObjectARB();
		public static final UnlinkedGLFunction glCreateProgramPipelines = make_glCreateProgramPipelines();
		public static final UnlinkedGLFunction glCreateProgressFenceNVX = make_glCreateProgressFenceNVX();
		public static final UnlinkedGLFunction glCreateQueries = make_glCreateQueries();
		public static final UnlinkedGLFunction glCreateRenderbuffers = make_glCreateRenderbuffers();
		public static final UnlinkedGLFunction glCreateSamplers = make_glCreateSamplers();
		public static final UnlinkedGLFunction glCreateSemaphoresNV = make_glCreateSemaphoresNV();
		public static final UnlinkedGLFunction glCreateShader = make_glCreateShader();
		public static final UnlinkedGLFunction glCreateShaderObjectARB = make_glCreateShaderObjectARB();
		public static final UnlinkedGLFunction glCreateShaderProgramEXT = make_glCreateShaderProgramEXT();
		public static final UnlinkedGLFunction glCreateShaderProgramv = make_glCreateShaderProgramv();
		public static final UnlinkedGLFunction glCreateShaderProgramvEXT = make_glCreateShaderProgramvEXT();
		public static final UnlinkedGLFunction glCreateStatesNV = make_glCreateStatesNV();
		public static final UnlinkedGLFunction glCreateSyncFromCLeventARB = make_glCreateSyncFromCLeventARB();
		public static final UnlinkedGLFunction glCreateTextures = make_glCreateTextures();
		public static final UnlinkedGLFunction glCreateTransformFeedbacks = make_glCreateTransformFeedbacks();
		public static final UnlinkedGLFunction glCreateVertexArrays = make_glCreateVertexArrays();
		public static final UnlinkedGLFunction glCullFace = make_glCullFace();
		public static final UnlinkedGLFunction glCullParameterdvEXT = make_glCullParameterdvEXT();
		public static final UnlinkedGLFunction glCullParameterfvEXT = make_glCullParameterfvEXT();
		public static final UnlinkedGLFunction glCurrentPaletteMatrixARB = make_glCurrentPaletteMatrixARB();
		public static final UnlinkedGLFunction glCurrentPaletteMatrixOES = make_glCurrentPaletteMatrixOES();
		public static final UnlinkedGLFunction glDebugMessageCallback = make_glDebugMessageCallback();
		public static final UnlinkedGLFunction glDebugMessageCallbackAMD = make_glDebugMessageCallbackAMD();
		public static final UnlinkedGLFunction glDebugMessageCallbackARB = make_glDebugMessageCallbackARB();
		public static final UnlinkedGLFunction glDebugMessageCallbackKHR = make_glDebugMessageCallbackKHR();
		public static final UnlinkedGLFunction glDebugMessageControl = make_glDebugMessageControl();
		public static final UnlinkedGLFunction glDebugMessageControlARB = make_glDebugMessageControlARB();
		public static final UnlinkedGLFunction glDebugMessageControlKHR = make_glDebugMessageControlKHR();
		public static final UnlinkedGLFunction glDebugMessageEnableAMD = make_glDebugMessageEnableAMD();
		public static final UnlinkedGLFunction glDebugMessageInsert = make_glDebugMessageInsert();
		public static final UnlinkedGLFunction glDebugMessageInsertAMD = make_glDebugMessageInsertAMD();
		public static final UnlinkedGLFunction glDebugMessageInsertARB = make_glDebugMessageInsertARB();
		public static final UnlinkedGLFunction glDebugMessageInsertKHR = make_glDebugMessageInsertKHR();
		public static final UnlinkedGLFunction glDeformSGIX = make_glDeformSGIX();
		public static final UnlinkedGLFunction glDeformationMap3dSGIX = make_glDeformationMap3dSGIX();
		public static final UnlinkedGLFunction glDeformationMap3fSGIX = make_glDeformationMap3fSGIX();
		public static final UnlinkedGLFunction glDeleteAsyncMarkersSGIX = make_glDeleteAsyncMarkersSGIX();
		public static final UnlinkedGLFunction glDeleteBuffers = make_glDeleteBuffers();
		public static final UnlinkedGLFunction glDeleteBuffersARB = make_glDeleteBuffersARB();
		public static final UnlinkedGLFunction glDeleteCommandListsNV = make_glDeleteCommandListsNV();
		public static final UnlinkedGLFunction glDeleteFencesAPPLE = make_glDeleteFencesAPPLE();
		public static final UnlinkedGLFunction glDeleteFencesNV = make_glDeleteFencesNV();
		public static final UnlinkedGLFunction glDeleteFragmentShaderATI = make_glDeleteFragmentShaderATI();
		public static final UnlinkedGLFunction glDeleteFramebuffers = make_glDeleteFramebuffers();
		public static final UnlinkedGLFunction glDeleteFramebuffersEXT = make_glDeleteFramebuffersEXT();
		public static final UnlinkedGLFunction glDeleteFramebuffersOES = make_glDeleteFramebuffersOES();
		public static final UnlinkedGLFunction glDeleteLists = make_glDeleteLists();
		public static final UnlinkedGLFunction glDeleteMemoryObjectsEXT = make_glDeleteMemoryObjectsEXT();
		public static final UnlinkedGLFunction glDeleteNamedStringARB = make_glDeleteNamedStringARB();
		public static final UnlinkedGLFunction glDeleteNamesAMD = make_glDeleteNamesAMD();
		public static final UnlinkedGLFunction glDeleteObjectARB = make_glDeleteObjectARB();
		public static final UnlinkedGLFunction glDeleteOcclusionQueriesNV = make_glDeleteOcclusionQueriesNV();
		public static final UnlinkedGLFunction glDeletePathsNV = make_glDeletePathsNV();
		public static final UnlinkedGLFunction glDeletePerfMonitorsAMD = make_glDeletePerfMonitorsAMD();
		public static final UnlinkedGLFunction glDeletePerfQueryINTEL = make_glDeletePerfQueryINTEL();
		public static final UnlinkedGLFunction glDeleteProgram = make_glDeleteProgram();
		public static final UnlinkedGLFunction glDeleteProgramPipelines = make_glDeleteProgramPipelines();
		public static final UnlinkedGLFunction glDeleteProgramPipelinesEXT = make_glDeleteProgramPipelinesEXT();
		public static final UnlinkedGLFunction glDeleteProgramsARB = make_glDeleteProgramsARB();
		public static final UnlinkedGLFunction glDeleteProgramsNV = make_glDeleteProgramsNV();
		public static final UnlinkedGLFunction glDeleteQueries = make_glDeleteQueries();
		public static final UnlinkedGLFunction glDeleteQueriesARB = make_glDeleteQueriesARB();
		public static final UnlinkedGLFunction glDeleteQueriesEXT = make_glDeleteQueriesEXT();
		public static final UnlinkedGLFunction glDeleteQueryResourceTagNV = make_glDeleteQueryResourceTagNV();
		public static final UnlinkedGLFunction glDeleteRenderbuffers = make_glDeleteRenderbuffers();
		public static final UnlinkedGLFunction glDeleteRenderbuffersEXT = make_glDeleteRenderbuffersEXT();
		public static final UnlinkedGLFunction glDeleteRenderbuffersOES = make_glDeleteRenderbuffersOES();
		public static final UnlinkedGLFunction glDeleteSamplers = make_glDeleteSamplers();
		public static final UnlinkedGLFunction glDeleteSemaphoresEXT = make_glDeleteSemaphoresEXT();
		public static final UnlinkedGLFunction glDeleteShader = make_glDeleteShader();
		public static final UnlinkedGLFunction glDeleteStatesNV = make_glDeleteStatesNV();
		public static final UnlinkedGLFunction glDeleteSync = make_glDeleteSync();
		public static final UnlinkedGLFunction glDeleteSyncAPPLE = make_glDeleteSyncAPPLE();
		public static final UnlinkedGLFunction glDeleteTextures = make_glDeleteTextures();
		public static final UnlinkedGLFunction glDeleteTexturesEXT = make_glDeleteTexturesEXT();
		public static final UnlinkedGLFunction glDeleteTransformFeedbacks = make_glDeleteTransformFeedbacks();
		public static final UnlinkedGLFunction glDeleteTransformFeedbacksNV = make_glDeleteTransformFeedbacksNV();
		public static final UnlinkedGLFunction glDeleteVertexArrays = make_glDeleteVertexArrays();
		public static final UnlinkedGLFunction glDeleteVertexArraysAPPLE = make_glDeleteVertexArraysAPPLE();
		public static final UnlinkedGLFunction glDeleteVertexArraysOES = make_glDeleteVertexArraysOES();
		public static final UnlinkedGLFunction glDeleteVertexShaderEXT = make_glDeleteVertexShaderEXT();
		public static final UnlinkedGLFunction glDepthBoundsEXT = make_glDepthBoundsEXT();
		public static final UnlinkedGLFunction glDepthBoundsdNV = make_glDepthBoundsdNV();
		public static final UnlinkedGLFunction glDepthFunc = make_glDepthFunc();
		public static final UnlinkedGLFunction glDepthMask = make_glDepthMask();
		public static final UnlinkedGLFunction glDepthRange = make_glDepthRange();
		public static final UnlinkedGLFunction glDepthRangeArraydvNV = make_glDepthRangeArraydvNV();
		public static final UnlinkedGLFunction glDepthRangeArrayfvNV = make_glDepthRangeArrayfvNV();
		public static final UnlinkedGLFunction glDepthRangeArrayfvOES = make_glDepthRangeArrayfvOES();
		public static final UnlinkedGLFunction glDepthRangeArrayv = make_glDepthRangeArrayv();
		public static final UnlinkedGLFunction glDepthRangeIndexed = make_glDepthRangeIndexed();
		public static final UnlinkedGLFunction glDepthRangeIndexeddNV = make_glDepthRangeIndexeddNV();
		public static final UnlinkedGLFunction glDepthRangeIndexedfNV = make_glDepthRangeIndexedfNV();
		public static final UnlinkedGLFunction glDepthRangeIndexedfOES = make_glDepthRangeIndexedfOES();
		public static final UnlinkedGLFunction glDepthRangedNV = make_glDepthRangedNV();
		public static final UnlinkedGLFunction glDepthRangef = make_glDepthRangef();
		public static final UnlinkedGLFunction glDepthRangefOES = make_glDepthRangefOES();
		public static final UnlinkedGLFunction glDepthRangex = make_glDepthRangex();
		public static final UnlinkedGLFunction glDepthRangexOES = make_glDepthRangexOES();
		public static final UnlinkedGLFunction glDetachObjectARB = make_glDetachObjectARB();
		public static final UnlinkedGLFunction glDetachShader = make_glDetachShader();
		public static final UnlinkedGLFunction glDetailTexFuncSGIS = make_glDetailTexFuncSGIS();
		public static final UnlinkedGLFunction glDisable = make_glDisable();
		public static final UnlinkedGLFunction glDisableClientState = make_glDisableClientState();
		public static final UnlinkedGLFunction glDisableClientStateIndexedEXT = make_glDisableClientStateIndexedEXT();
		public static final UnlinkedGLFunction glDisableClientStateiEXT = make_glDisableClientStateiEXT();
		public static final UnlinkedGLFunction glDisableDriverControlQCOM = make_glDisableDriverControlQCOM();
		public static final UnlinkedGLFunction glDisableIndexedEXT = make_glDisableIndexedEXT();
		public static final UnlinkedGLFunction glDisableVariantClientStateEXT = make_glDisableVariantClientStateEXT();
		public static final UnlinkedGLFunction glDisableVertexArrayAttrib = make_glDisableVertexArrayAttrib();
		public static final UnlinkedGLFunction glDisableVertexArrayAttribEXT = make_glDisableVertexArrayAttribEXT();
		public static final UnlinkedGLFunction glDisableVertexArrayEXT = make_glDisableVertexArrayEXT();
		public static final UnlinkedGLFunction glDisableVertexAttribAPPLE = make_glDisableVertexAttribAPPLE();
		public static final UnlinkedGLFunction glDisableVertexAttribArray = make_glDisableVertexAttribArray();
		public static final UnlinkedGLFunction glDisableVertexAttribArrayARB = make_glDisableVertexAttribArrayARB();
		public static final UnlinkedGLFunction glDisablei = make_glDisablei();
		public static final UnlinkedGLFunction glDisableiEXT = make_glDisableiEXT();
		public static final UnlinkedGLFunction glDisableiNV = make_glDisableiNV();
		public static final UnlinkedGLFunction glDisableiOES = make_glDisableiOES();
		public static final UnlinkedGLFunction glDiscardFramebufferEXT = make_glDiscardFramebufferEXT();
		public static final UnlinkedGLFunction glDispatchCompute = make_glDispatchCompute();
		public static final UnlinkedGLFunction glDispatchComputeGroupSizeARB = make_glDispatchComputeGroupSizeARB();
		public static final UnlinkedGLFunction glDispatchComputeIndirect = make_glDispatchComputeIndirect();
		public static final UnlinkedGLFunction glDrawArrays = make_glDrawArrays();
		public static final UnlinkedGLFunction glDrawArraysEXT = make_glDrawArraysEXT();
		public static final UnlinkedGLFunction glDrawArraysIndirect = make_glDrawArraysIndirect();
		public static final UnlinkedGLFunction glDrawArraysInstanced = make_glDrawArraysInstanced();
		public static final UnlinkedGLFunction glDrawArraysInstancedANGLE = make_glDrawArraysInstancedANGLE();
		public static final UnlinkedGLFunction glDrawArraysInstancedARB = make_glDrawArraysInstancedARB();
		public static final UnlinkedGLFunction glDrawArraysInstancedBaseInstance = make_glDrawArraysInstancedBaseInstance();
		public static final UnlinkedGLFunction glDrawArraysInstancedBaseInstanceEXT = make_glDrawArraysInstancedBaseInstanceEXT();
		public static final UnlinkedGLFunction glDrawArraysInstancedEXT = make_glDrawArraysInstancedEXT();
		public static final UnlinkedGLFunction glDrawArraysInstancedNV = make_glDrawArraysInstancedNV();
		public static final UnlinkedGLFunction glDrawBuffer = make_glDrawBuffer();
		public static final UnlinkedGLFunction glDrawBuffers = make_glDrawBuffers();
		public static final UnlinkedGLFunction glDrawBuffersARB = make_glDrawBuffersARB();
		public static final UnlinkedGLFunction glDrawBuffersATI = make_glDrawBuffersATI();
		public static final UnlinkedGLFunction glDrawBuffersEXT = make_glDrawBuffersEXT();
		public static final UnlinkedGLFunction glDrawBuffersIndexedEXT = make_glDrawBuffersIndexedEXT();
		public static final UnlinkedGLFunction glDrawBuffersNV = make_glDrawBuffersNV();
		public static final UnlinkedGLFunction glDrawCommandsAddressNV = make_glDrawCommandsAddressNV();
		public static final UnlinkedGLFunction glDrawCommandsNV = make_glDrawCommandsNV();
		public static final UnlinkedGLFunction glDrawCommandsStatesAddressNV = make_glDrawCommandsStatesAddressNV();
		public static final UnlinkedGLFunction glDrawCommandsStatesNV = make_glDrawCommandsStatesNV();
		public static final UnlinkedGLFunction glDrawElementArrayAPPLE = make_glDrawElementArrayAPPLE();
		public static final UnlinkedGLFunction glDrawElementArrayATI = make_glDrawElementArrayATI();
		public static final UnlinkedGLFunction glDrawElements = make_glDrawElements();
		public static final UnlinkedGLFunction glDrawElementsBaseVertex = make_glDrawElementsBaseVertex();
		public static final UnlinkedGLFunction glDrawElementsBaseVertexEXT = make_glDrawElementsBaseVertexEXT();
		public static final UnlinkedGLFunction glDrawElementsBaseVertexOES = make_glDrawElementsBaseVertexOES();
		public static final UnlinkedGLFunction glDrawElementsIndirect = make_glDrawElementsIndirect();
		public static final UnlinkedGLFunction glDrawElementsInstanced = make_glDrawElementsInstanced();
		public static final UnlinkedGLFunction glDrawElementsInstancedANGLE = make_glDrawElementsInstancedANGLE();
		public static final UnlinkedGLFunction glDrawElementsInstancedARB = make_glDrawElementsInstancedARB();
		public static final UnlinkedGLFunction glDrawElementsInstancedBaseInstance = make_glDrawElementsInstancedBaseInstance();
		public static final UnlinkedGLFunction glDrawElementsInstancedBaseInstanceEXT = make_glDrawElementsInstancedBaseInstanceEXT();
		public static final UnlinkedGLFunction glDrawElementsInstancedBaseVertex = make_glDrawElementsInstancedBaseVertex();
		public static final UnlinkedGLFunction glDrawElementsInstancedBaseVertexBaseInstance = make_glDrawElementsInstancedBaseVertexBaseInstance();
		public static final UnlinkedGLFunction glDrawElementsInstancedBaseVertexBaseInstanceEXT = make_glDrawElementsInstancedBaseVertexBaseInstanceEXT();
		public static final UnlinkedGLFunction glDrawElementsInstancedBaseVertexEXT = make_glDrawElementsInstancedBaseVertexEXT();
		public static final UnlinkedGLFunction glDrawElementsInstancedBaseVertexOES = make_glDrawElementsInstancedBaseVertexOES();
		public static final UnlinkedGLFunction glDrawElementsInstancedEXT = make_glDrawElementsInstancedEXT();
		public static final UnlinkedGLFunction glDrawElementsInstancedNV = make_glDrawElementsInstancedNV();
		public static final UnlinkedGLFunction glDrawMeshArraysSUN = make_glDrawMeshArraysSUN();
		public static final UnlinkedGLFunction glDrawMeshTasksNV = make_glDrawMeshTasksNV();
		public static final UnlinkedGLFunction glDrawMeshTasksIndirectNV = make_glDrawMeshTasksIndirectNV();
		public static final UnlinkedGLFunction glDrawPixels = make_glDrawPixels();
		public static final UnlinkedGLFunction glDrawRangeElementArrayAPPLE = make_glDrawRangeElementArrayAPPLE();
		public static final UnlinkedGLFunction glDrawRangeElementArrayATI = make_glDrawRangeElementArrayATI();
		public static final UnlinkedGLFunction glDrawRangeElements = make_glDrawRangeElements();
		public static final UnlinkedGLFunction glDrawRangeElementsBaseVertex = make_glDrawRangeElementsBaseVertex();
		public static final UnlinkedGLFunction glDrawRangeElementsBaseVertexEXT = make_glDrawRangeElementsBaseVertexEXT();
		public static final UnlinkedGLFunction glDrawRangeElementsBaseVertexOES = make_glDrawRangeElementsBaseVertexOES();
		public static final UnlinkedGLFunction glDrawRangeElementsEXT = make_glDrawRangeElementsEXT();
		public static final UnlinkedGLFunction glDrawTexfOES = make_glDrawTexfOES();
		public static final UnlinkedGLFunction glDrawTexfvOES = make_glDrawTexfvOES();
		public static final UnlinkedGLFunction glDrawTexiOES = make_glDrawTexiOES();
		public static final UnlinkedGLFunction glDrawTexivOES = make_glDrawTexivOES();
		public static final UnlinkedGLFunction glDrawTexsOES = make_glDrawTexsOES();
		public static final UnlinkedGLFunction glDrawTexsvOES = make_glDrawTexsvOES();
		public static final UnlinkedGLFunction glDrawTextureNV = make_glDrawTextureNV();
		public static final UnlinkedGLFunction glDrawTexxOES = make_glDrawTexxOES();
		public static final UnlinkedGLFunction glDrawTexxvOES = make_glDrawTexxvOES();
		public static final UnlinkedGLFunction glDrawTransformFeedback = make_glDrawTransformFeedback();
		public static final UnlinkedGLFunction glDrawTransformFeedbackEXT = make_glDrawTransformFeedbackEXT();
		public static final UnlinkedGLFunction glDrawTransformFeedbackInstanced = make_glDrawTransformFeedbackInstanced();
		public static final UnlinkedGLFunction glDrawTransformFeedbackInstancedEXT = make_glDrawTransformFeedbackInstancedEXT();
		public static final UnlinkedGLFunction glDrawTransformFeedbackNV = make_glDrawTransformFeedbackNV();
		public static final UnlinkedGLFunction glDrawTransformFeedbackStream = make_glDrawTransformFeedbackStream();
		public static final UnlinkedGLFunction glDrawTransformFeedbackStreamInstanced = make_glDrawTransformFeedbackStreamInstanced();
		public static final UnlinkedGLFunction glEGLImageTargetRenderbufferStorageOES = make_glEGLImageTargetRenderbufferStorageOES();
		public static final UnlinkedGLFunction glEGLImageTargetTexStorageEXT = make_glEGLImageTargetTexStorageEXT();
		public static final UnlinkedGLFunction glEGLImageTargetTexture2DOES = make_glEGLImageTargetTexture2DOES();
		public static final UnlinkedGLFunction glEGLImageTargetTextureStorageEXT = make_glEGLImageTargetTextureStorageEXT();
		public static final UnlinkedGLFunction glEdgeFlag = make_glEdgeFlag();
		public static final UnlinkedGLFunction glEdgeFlagFormatNV = make_glEdgeFlagFormatNV();
		public static final UnlinkedGLFunction glEdgeFlagPointer = make_glEdgeFlagPointer();
		public static final UnlinkedGLFunction glEdgeFlagPointerEXT = make_glEdgeFlagPointerEXT();
		public static final UnlinkedGLFunction glEdgeFlagPointerListIBM = make_glEdgeFlagPointerListIBM();
		public static final UnlinkedGLFunction glEdgeFlagv = make_glEdgeFlagv();
		public static final UnlinkedGLFunction glElementPointerAPPLE = make_glElementPointerAPPLE();
		public static final UnlinkedGLFunction glElementPointerATI = make_glElementPointerATI();
		public static final UnlinkedGLFunction glEnable = make_glEnable();
		public static final UnlinkedGLFunction glEnableClientState = make_glEnableClientState();
		public static final UnlinkedGLFunction glEnableClientStateIndexedEXT = make_glEnableClientStateIndexedEXT();
		public static final UnlinkedGLFunction glEnableClientStateiEXT = make_glEnableClientStateiEXT();
		public static final UnlinkedGLFunction glEnableDriverControlQCOM = make_glEnableDriverControlQCOM();
		public static final UnlinkedGLFunction glEnableIndexedEXT = make_glEnableIndexedEXT();
		public static final UnlinkedGLFunction glEnableVariantClientStateEXT = make_glEnableVariantClientStateEXT();
		public static final UnlinkedGLFunction glEnableVertexArrayAttrib = make_glEnableVertexArrayAttrib();
		public static final UnlinkedGLFunction glEnableVertexArrayAttribEXT = make_glEnableVertexArrayAttribEXT();
		public static final UnlinkedGLFunction glEnableVertexArrayEXT = make_glEnableVertexArrayEXT();
		public static final UnlinkedGLFunction glEnableVertexAttribAPPLE = make_glEnableVertexAttribAPPLE();
		public static final UnlinkedGLFunction glEnableVertexAttribArray = make_glEnableVertexAttribArray();
		public static final UnlinkedGLFunction glEnableVertexAttribArrayARB = make_glEnableVertexAttribArrayARB();
		public static final UnlinkedGLFunction glEnablei = make_glEnablei();
		public static final UnlinkedGLFunction glEnableiEXT = make_glEnableiEXT();
		public static final UnlinkedGLFunction glEnableiNV = make_glEnableiNV();
		public static final UnlinkedGLFunction glEnableiOES = make_glEnableiOES();
		public static final UnlinkedGLFunction glEnd = make_glEnd();
		public static final UnlinkedGLFunction glEndConditionalRender = make_glEndConditionalRender();
		public static final UnlinkedGLFunction glEndConditionalRenderNV = make_glEndConditionalRenderNV();
		public static final UnlinkedGLFunction glEndConditionalRenderNVX = make_glEndConditionalRenderNVX();
		public static final UnlinkedGLFunction glEndFragmentShaderATI = make_glEndFragmentShaderATI();
		public static final UnlinkedGLFunction glEndList = make_glEndList();
		public static final UnlinkedGLFunction glEndOcclusionQueryNV = make_glEndOcclusionQueryNV();
		public static final UnlinkedGLFunction glEndPerfMonitorAMD = make_glEndPerfMonitorAMD();
		public static final UnlinkedGLFunction glEndPerfQueryINTEL = make_glEndPerfQueryINTEL();
		public static final UnlinkedGLFunction glEndQuery = make_glEndQuery();
		public static final UnlinkedGLFunction glEndQueryARB = make_glEndQueryARB();
		public static final UnlinkedGLFunction glEndQueryEXT = make_glEndQueryEXT();
		public static final UnlinkedGLFunction glEndQueryIndexed = make_glEndQueryIndexed();
		public static final UnlinkedGLFunction glEndTilingQCOM = make_glEndTilingQCOM();
		public static final UnlinkedGLFunction glEndTransformFeedback = make_glEndTransformFeedback();
		public static final UnlinkedGLFunction glEndTransformFeedbackEXT = make_glEndTransformFeedbackEXT();
		public static final UnlinkedGLFunction glEndTransformFeedbackNV = make_glEndTransformFeedbackNV();
		public static final UnlinkedGLFunction glEndVertexShaderEXT = make_glEndVertexShaderEXT();
		public static final UnlinkedGLFunction glEndVideoCaptureNV = make_glEndVideoCaptureNV();
		public static final UnlinkedGLFunction glEvalCoord1d = make_glEvalCoord1d();
		public static final UnlinkedGLFunction glEvalCoord1dv = make_glEvalCoord1dv();
		public static final UnlinkedGLFunction glEvalCoord1f = make_glEvalCoord1f();
		public static final UnlinkedGLFunction glEvalCoord1fv = make_glEvalCoord1fv();
		public static final UnlinkedGLFunction glEvalCoord1xOES = make_glEvalCoord1xOES();
		public static final UnlinkedGLFunction glEvalCoord1xvOES = make_glEvalCoord1xvOES();
		public static final UnlinkedGLFunction glEvalCoord2d = make_glEvalCoord2d();
		public static final UnlinkedGLFunction glEvalCoord2dv = make_glEvalCoord2dv();
		public static final UnlinkedGLFunction glEvalCoord2f = make_glEvalCoord2f();
		public static final UnlinkedGLFunction glEvalCoord2fv = make_glEvalCoord2fv();
		public static final UnlinkedGLFunction glEvalCoord2xOES = make_glEvalCoord2xOES();
		public static final UnlinkedGLFunction glEvalCoord2xvOES = make_glEvalCoord2xvOES();
		public static final UnlinkedGLFunction glEvalMapsNV = make_glEvalMapsNV();
		public static final UnlinkedGLFunction glEvalMesh1 = make_glEvalMesh1();
		public static final UnlinkedGLFunction glEvalMesh2 = make_glEvalMesh2();
		public static final UnlinkedGLFunction glEvalPoint1 = make_glEvalPoint1();
		public static final UnlinkedGLFunction glEvalPoint2 = make_glEvalPoint2();
		public static final UnlinkedGLFunction glEvaluateDepthValuesARB = make_glEvaluateDepthValuesARB();
		public static final UnlinkedGLFunction glExecuteProgramNV = make_glExecuteProgramNV();
		public static final UnlinkedGLFunction glExtGetBufferPointervQCOM = make_glExtGetBufferPointervQCOM();
		public static final UnlinkedGLFunction glExtGetBuffersQCOM = make_glExtGetBuffersQCOM();
		public static final UnlinkedGLFunction glExtGetFramebuffersQCOM = make_glExtGetFramebuffersQCOM();
		public static final UnlinkedGLFunction glExtGetProgramBinarySourceQCOM = make_glExtGetProgramBinarySourceQCOM();
		public static final UnlinkedGLFunction glExtGetProgramsQCOM = make_glExtGetProgramsQCOM();
		public static final UnlinkedGLFunction glExtGetRenderbuffersQCOM = make_glExtGetRenderbuffersQCOM();
		public static final UnlinkedGLFunction glExtGetShadersQCOM = make_glExtGetShadersQCOM();
		public static final UnlinkedGLFunction glExtGetTexLevelParameterivQCOM = make_glExtGetTexLevelParameterivQCOM();
		public static final UnlinkedGLFunction glExtGetTexSubImageQCOM = make_glExtGetTexSubImageQCOM();
		public static final UnlinkedGLFunction glExtGetTexturesQCOM = make_glExtGetTexturesQCOM();
		public static final UnlinkedGLFunction glExtIsProgramBinaryQCOM = make_glExtIsProgramBinaryQCOM();
		public static final UnlinkedGLFunction glExtTexObjectStateOverrideiQCOM = make_glExtTexObjectStateOverrideiQCOM();
		public static final UnlinkedGLFunction glExtractComponentEXT = make_glExtractComponentEXT();
		public static final UnlinkedGLFunction glFeedbackBuffer = make_glFeedbackBuffer();
		public static final UnlinkedGLFunction glFeedbackBufferxOES = make_glFeedbackBufferxOES();
		public static final UnlinkedGLFunction glFenceSync = make_glFenceSync();
		public static final UnlinkedGLFunction glFenceSyncAPPLE = make_glFenceSyncAPPLE();
		public static final UnlinkedGLFunction glFinalCombinerInputNV = make_glFinalCombinerInputNV();
		public static final UnlinkedGLFunction glFinish = make_glFinish();
		public static final UnlinkedGLFunction glFinishAsyncSGIX = make_glFinishAsyncSGIX();
		public static final UnlinkedGLFunction glFinishFenceAPPLE = make_glFinishFenceAPPLE();
		public static final UnlinkedGLFunction glFinishFenceNV = make_glFinishFenceNV();
		public static final UnlinkedGLFunction glFinishObjectAPPLE = make_glFinishObjectAPPLE();
		public static final UnlinkedGLFunction glFinishTextureSUNX = make_glFinishTextureSUNX();
		public static final UnlinkedGLFunction glFlush = make_glFlush();
		public static final UnlinkedGLFunction glFlushMappedBufferRange = make_glFlushMappedBufferRange();
		public static final UnlinkedGLFunction glFlushMappedBufferRangeAPPLE = make_glFlushMappedBufferRangeAPPLE();
		public static final UnlinkedGLFunction glFlushMappedBufferRangeEXT = make_glFlushMappedBufferRangeEXT();
		public static final UnlinkedGLFunction glFlushMappedNamedBufferRange = make_glFlushMappedNamedBufferRange();
		public static final UnlinkedGLFunction glFlushMappedNamedBufferRangeEXT = make_glFlushMappedNamedBufferRangeEXT();
		public static final UnlinkedGLFunction glFlushPixelDataRangeNV = make_glFlushPixelDataRangeNV();
		public static final UnlinkedGLFunction glFlushRasterSGIX = make_glFlushRasterSGIX();
		public static final UnlinkedGLFunction glFlushStaticDataIBM = make_glFlushStaticDataIBM();
		public static final UnlinkedGLFunction glFlushVertexArrayRangeAPPLE = make_glFlushVertexArrayRangeAPPLE();
		public static final UnlinkedGLFunction glFlushVertexArrayRangeNV = make_glFlushVertexArrayRangeNV();
		public static final UnlinkedGLFunction glFogCoordFormatNV = make_glFogCoordFormatNV();
		public static final UnlinkedGLFunction glFogCoordPointer = make_glFogCoordPointer();
		public static final UnlinkedGLFunction glFogCoordPointerEXT = make_glFogCoordPointerEXT();
		public static final UnlinkedGLFunction glFogCoordPointerListIBM = make_glFogCoordPointerListIBM();
		public static final UnlinkedGLFunction glFogCoordd = make_glFogCoordd();
		public static final UnlinkedGLFunction glFogCoorddEXT = make_glFogCoorddEXT();
		public static final UnlinkedGLFunction glFogCoorddv = make_glFogCoorddv();
		public static final UnlinkedGLFunction glFogCoorddvEXT = make_glFogCoorddvEXT();
		public static final UnlinkedGLFunction glFogCoordf = make_glFogCoordf();
		public static final UnlinkedGLFunction glFogCoordfEXT = make_glFogCoordfEXT();
		public static final UnlinkedGLFunction glFogCoordfv = make_glFogCoordfv();
		public static final UnlinkedGLFunction glFogCoordfvEXT = make_glFogCoordfvEXT();
		public static final UnlinkedGLFunction glFogCoordhNV = make_glFogCoordhNV();
		public static final UnlinkedGLFunction glFogCoordhvNV = make_glFogCoordhvNV();
		public static final UnlinkedGLFunction glFogFuncSGIS = make_glFogFuncSGIS();
		public static final UnlinkedGLFunction glFogf = make_glFogf();
		public static final UnlinkedGLFunction glFogfv = make_glFogfv();
		public static final UnlinkedGLFunction glFogi = make_glFogi();
		public static final UnlinkedGLFunction glFogiv = make_glFogiv();
		public static final UnlinkedGLFunction glFogx = make_glFogx();
		public static final UnlinkedGLFunction glFogxOES = make_glFogxOES();
		public static final UnlinkedGLFunction glFogxv = make_glFogxv();
		public static final UnlinkedGLFunction glFogxvOES = make_glFogxvOES();
		public static final UnlinkedGLFunction glFragmentColorMaterialSGIX = make_glFragmentColorMaterialSGIX();
		public static final UnlinkedGLFunction glFragmentCoverageColorNV = make_glFragmentCoverageColorNV();
		public static final UnlinkedGLFunction glFragmentLightModelfSGIX = make_glFragmentLightModelfSGIX();
		public static final UnlinkedGLFunction glFragmentLightModelfvSGIX = make_glFragmentLightModelfvSGIX();
		public static final UnlinkedGLFunction glFragmentLightModeliSGIX = make_glFragmentLightModeliSGIX();
		public static final UnlinkedGLFunction glFragmentLightModelivSGIX = make_glFragmentLightModelivSGIX();
		public static final UnlinkedGLFunction glFragmentLightfSGIX = make_glFragmentLightfSGIX();
		public static final UnlinkedGLFunction glFragmentLightfvSGIX = make_glFragmentLightfvSGIX();
		public static final UnlinkedGLFunction glFragmentLightiSGIX = make_glFragmentLightiSGIX();
		public static final UnlinkedGLFunction glFragmentLightivSGIX = make_glFragmentLightivSGIX();
		public static final UnlinkedGLFunction glFragmentMaterialfSGIX = make_glFragmentMaterialfSGIX();
		public static final UnlinkedGLFunction glFragmentMaterialfvSGIX = make_glFragmentMaterialfvSGIX();
		public static final UnlinkedGLFunction glFragmentMaterialiSGIX = make_glFragmentMaterialiSGIX();
		public static final UnlinkedGLFunction glFragmentMaterialivSGIX = make_glFragmentMaterialivSGIX();
		public static final UnlinkedGLFunction glFrameTerminatorGREMEDY = make_glFrameTerminatorGREMEDY();
		public static final UnlinkedGLFunction glFrameZoomSGIX = make_glFrameZoomSGIX();
		public static final UnlinkedGLFunction glFramebufferDrawBufferEXT = make_glFramebufferDrawBufferEXT();
		public static final UnlinkedGLFunction glFramebufferDrawBuffersEXT = make_glFramebufferDrawBuffersEXT();
		public static final UnlinkedGLFunction glFramebufferFetchBarrierEXT = make_glFramebufferFetchBarrierEXT();
		public static final UnlinkedGLFunction glFramebufferFetchBarrierQCOM = make_glFramebufferFetchBarrierQCOM();
		public static final UnlinkedGLFunction glFramebufferFoveationConfigQCOM = make_glFramebufferFoveationConfigQCOM();
		public static final UnlinkedGLFunction glFramebufferFoveationParametersQCOM = make_glFramebufferFoveationParametersQCOM();
		public static final UnlinkedGLFunction glFramebufferParameteri = make_glFramebufferParameteri();
		public static final UnlinkedGLFunction glFramebufferPixelLocalStorageSizeEXT = make_glFramebufferPixelLocalStorageSizeEXT();
		public static final UnlinkedGLFunction glFramebufferReadBufferEXT = make_glFramebufferReadBufferEXT();
		public static final UnlinkedGLFunction glFramebufferRenderbuffer = make_glFramebufferRenderbuffer();
		public static final UnlinkedGLFunction glFramebufferRenderbufferEXT = make_glFramebufferRenderbufferEXT();
		public static final UnlinkedGLFunction glFramebufferRenderbufferOES = make_glFramebufferRenderbufferOES();
		public static final UnlinkedGLFunction glFramebufferSampleLocationsfvARB = make_glFramebufferSampleLocationsfvARB();
		public static final UnlinkedGLFunction glFramebufferSampleLocationsfvNV = make_glFramebufferSampleLocationsfvNV();
		public static final UnlinkedGLFunction glFramebufferSamplePositionsfvAMD = make_glFramebufferSamplePositionsfvAMD();
		public static final UnlinkedGLFunction glFramebufferShadingRateEXT = make_glFramebufferShadingRateEXT();
		public static final UnlinkedGLFunction glFramebufferTexture = make_glFramebufferTexture();
		public static final UnlinkedGLFunction glFramebufferTexture1D = make_glFramebufferTexture1D();
		public static final UnlinkedGLFunction glFramebufferTexture1DEXT = make_glFramebufferTexture1DEXT();
		public static final UnlinkedGLFunction glFramebufferTexture2D = make_glFramebufferTexture2D();
		public static final UnlinkedGLFunction glFramebufferTexture2DEXT = make_glFramebufferTexture2DEXT();
		public static final UnlinkedGLFunction glFramebufferTexture2DDownsampleIMG = make_glFramebufferTexture2DDownsampleIMG();
		public static final UnlinkedGLFunction glFramebufferTexture2DMultisampleEXT = make_glFramebufferTexture2DMultisampleEXT();
		public static final UnlinkedGLFunction glFramebufferTexture2DMultisampleIMG = make_glFramebufferTexture2DMultisampleIMG();
		public static final UnlinkedGLFunction glFramebufferTexture2DOES = make_glFramebufferTexture2DOES();
		public static final UnlinkedGLFunction glFramebufferTexture3D = make_glFramebufferTexture3D();
		public static final UnlinkedGLFunction glFramebufferTexture3DEXT = make_glFramebufferTexture3DEXT();
		public static final UnlinkedGLFunction glFramebufferTexture3DOES = make_glFramebufferTexture3DOES();
		public static final UnlinkedGLFunction glFramebufferTextureARB = make_glFramebufferTextureARB();
		public static final UnlinkedGLFunction glFramebufferTextureEXT = make_glFramebufferTextureEXT();
		public static final UnlinkedGLFunction glFramebufferTextureFaceARB = make_glFramebufferTextureFaceARB();
		public static final UnlinkedGLFunction glFramebufferTextureFaceEXT = make_glFramebufferTextureFaceEXT();
		public static final UnlinkedGLFunction glFramebufferTextureLayer = make_glFramebufferTextureLayer();
		public static final UnlinkedGLFunction glFramebufferTextureLayerARB = make_glFramebufferTextureLayerARB();
		public static final UnlinkedGLFunction glFramebufferTextureLayerEXT = make_glFramebufferTextureLayerEXT();
		public static final UnlinkedGLFunction glFramebufferTextureLayerDownsampleIMG = make_glFramebufferTextureLayerDownsampleIMG();
		public static final UnlinkedGLFunction glFramebufferTextureMultisampleMultiviewOVR = make_glFramebufferTextureMultisampleMultiviewOVR();
		public static final UnlinkedGLFunction glFramebufferTextureMultiviewOVR = make_glFramebufferTextureMultiviewOVR();
		public static final UnlinkedGLFunction glFramebufferTextureOES = make_glFramebufferTextureOES();
		public static final UnlinkedGLFunction glFreeObjectBufferATI = make_glFreeObjectBufferATI();
		public static final UnlinkedGLFunction glFrontFace = make_glFrontFace();
		public static final UnlinkedGLFunction glFrustum = make_glFrustum();
		public static final UnlinkedGLFunction glFrustumf = make_glFrustumf();
		public static final UnlinkedGLFunction glFrustumfOES = make_glFrustumfOES();
		public static final UnlinkedGLFunction glFrustumx = make_glFrustumx();
		public static final UnlinkedGLFunction glFrustumxOES = make_glFrustumxOES();
		public static final UnlinkedGLFunction glGenAsyncMarkersSGIX = make_glGenAsyncMarkersSGIX();
		public static final UnlinkedGLFunction glGenBuffers = make_glGenBuffers();
		public static final UnlinkedGLFunction glGenBuffersARB = make_glGenBuffersARB();
		public static final UnlinkedGLFunction glGenFencesAPPLE = make_glGenFencesAPPLE();
		public static final UnlinkedGLFunction glGenFencesNV = make_glGenFencesNV();
		public static final UnlinkedGLFunction glGenFragmentShadersATI = make_glGenFragmentShadersATI();
		public static final UnlinkedGLFunction glGenFramebuffers = make_glGenFramebuffers();
		public static final UnlinkedGLFunction glGenFramebuffersEXT = make_glGenFramebuffersEXT();
		public static final UnlinkedGLFunction glGenFramebuffersOES = make_glGenFramebuffersOES();
		public static final UnlinkedGLFunction glGenLists = make_glGenLists();
		public static final UnlinkedGLFunction glGenNamesAMD = make_glGenNamesAMD();
		public static final UnlinkedGLFunction glGenOcclusionQueriesNV = make_glGenOcclusionQueriesNV();
		public static final UnlinkedGLFunction glGenPathsNV = make_glGenPathsNV();
		public static final UnlinkedGLFunction glGenPerfMonitorsAMD = make_glGenPerfMonitorsAMD();
		public static final UnlinkedGLFunction glGenProgramPipelines = make_glGenProgramPipelines();
		public static final UnlinkedGLFunction glGenProgramPipelinesEXT = make_glGenProgramPipelinesEXT();
		public static final UnlinkedGLFunction glGenProgramsARB = make_glGenProgramsARB();
		public static final UnlinkedGLFunction glGenProgramsNV = make_glGenProgramsNV();
		public static final UnlinkedGLFunction glGenQueries = make_glGenQueries();
		public static final UnlinkedGLFunction glGenQueriesARB = make_glGenQueriesARB();
		public static final UnlinkedGLFunction glGenQueriesEXT = make_glGenQueriesEXT();
		public static final UnlinkedGLFunction glGenQueryResourceTagNV = make_glGenQueryResourceTagNV();
		public static final UnlinkedGLFunction glGenRenderbuffers = make_glGenRenderbuffers();
		public static final UnlinkedGLFunction glGenRenderbuffersEXT = make_glGenRenderbuffersEXT();
		public static final UnlinkedGLFunction glGenRenderbuffersOES = make_glGenRenderbuffersOES();
		public static final UnlinkedGLFunction glGenSamplers = make_glGenSamplers();
		public static final UnlinkedGLFunction glGenSemaphoresEXT = make_glGenSemaphoresEXT();
		public static final UnlinkedGLFunction glGenSymbolsEXT = make_glGenSymbolsEXT();
		public static final UnlinkedGLFunction glGenTextures = make_glGenTextures();
		public static final UnlinkedGLFunction glGenTexturesEXT = make_glGenTexturesEXT();
		public static final UnlinkedGLFunction glGenTransformFeedbacks = make_glGenTransformFeedbacks();
		public static final UnlinkedGLFunction glGenTransformFeedbacksNV = make_glGenTransformFeedbacksNV();
		public static final UnlinkedGLFunction glGenVertexArrays = make_glGenVertexArrays();
		public static final UnlinkedGLFunction glGenVertexArraysAPPLE = make_glGenVertexArraysAPPLE();
		public static final UnlinkedGLFunction glGenVertexArraysOES = make_glGenVertexArraysOES();
		public static final UnlinkedGLFunction glGenVertexShadersEXT = make_glGenVertexShadersEXT();
		public static final UnlinkedGLFunction glGenerateMipmap = make_glGenerateMipmap();
		public static final UnlinkedGLFunction glGenerateMipmapEXT = make_glGenerateMipmapEXT();
		public static final UnlinkedGLFunction glGenerateMipmapOES = make_glGenerateMipmapOES();
		public static final UnlinkedGLFunction glGenerateMultiTexMipmapEXT = make_glGenerateMultiTexMipmapEXT();
		public static final UnlinkedGLFunction glGenerateTextureMipmap = make_glGenerateTextureMipmap();
		public static final UnlinkedGLFunction glGenerateTextureMipmapEXT = make_glGenerateTextureMipmapEXT();
		public static final UnlinkedGLFunction glGetActiveAtomicCounterBufferiv = make_glGetActiveAtomicCounterBufferiv();
		public static final UnlinkedGLFunction glGetActiveAttrib = make_glGetActiveAttrib();
		public static final UnlinkedGLFunction glGetActiveAttribARB = make_glGetActiveAttribARB();
		public static final UnlinkedGLFunction glGetActiveSubroutineName = make_glGetActiveSubroutineName();
		public static final UnlinkedGLFunction glGetActiveSubroutineUniformName = make_glGetActiveSubroutineUniformName();
		public static final UnlinkedGLFunction glGetActiveSubroutineUniformiv = make_glGetActiveSubroutineUniformiv();
		public static final UnlinkedGLFunction glGetActiveUniform = make_glGetActiveUniform();
		public static final UnlinkedGLFunction glGetActiveUniformARB = make_glGetActiveUniformARB();
		public static final UnlinkedGLFunction glGetActiveUniformBlockName = make_glGetActiveUniformBlockName();
		public static final UnlinkedGLFunction glGetActiveUniformBlockiv = make_glGetActiveUniformBlockiv();
		public static final UnlinkedGLFunction glGetActiveUniformName = make_glGetActiveUniformName();
		public static final UnlinkedGLFunction glGetActiveUniformsiv = make_glGetActiveUniformsiv();
		public static final UnlinkedGLFunction glGetActiveVaryingNV = make_glGetActiveVaryingNV();
		public static final UnlinkedGLFunction glGetArrayObjectfvATI = make_glGetArrayObjectfvATI();
		public static final UnlinkedGLFunction glGetArrayObjectivATI = make_glGetArrayObjectivATI();
		public static final UnlinkedGLFunction glGetAttachedObjectsARB = make_glGetAttachedObjectsARB();
		public static final UnlinkedGLFunction glGetAttachedShaders = make_glGetAttachedShaders();
		public static final UnlinkedGLFunction glGetAttribLocation = make_glGetAttribLocation();
		public static final UnlinkedGLFunction glGetAttribLocationARB = make_glGetAttribLocationARB();
		public static final UnlinkedGLFunction glGetBooleanIndexedvEXT = make_glGetBooleanIndexedvEXT();
		public static final UnlinkedGLFunction glGetBooleani_v = make_glGetBooleani_v();
		public static final UnlinkedGLFunction glGetBooleanv = make_glGetBooleanv();
		public static final UnlinkedGLFunction glGetBufferParameteri64v = make_glGetBufferParameteri64v();
		public static final UnlinkedGLFunction glGetBufferParameteriv = make_glGetBufferParameteriv();
		public static final UnlinkedGLFunction glGetBufferParameterivARB = make_glGetBufferParameterivARB();
		public static final UnlinkedGLFunction glGetBufferParameterui64vNV = make_glGetBufferParameterui64vNV();
		public static final UnlinkedGLFunction glGetBufferPointerv = make_glGetBufferPointerv();
		public static final UnlinkedGLFunction glGetBufferPointervARB = make_glGetBufferPointervARB();
		public static final UnlinkedGLFunction glGetBufferPointervOES = make_glGetBufferPointervOES();
		public static final UnlinkedGLFunction glGetBufferSubData = make_glGetBufferSubData();
		public static final UnlinkedGLFunction glGetBufferSubDataARB = make_glGetBufferSubDataARB();
		public static final UnlinkedGLFunction glGetClipPlane = make_glGetClipPlane();
		public static final UnlinkedGLFunction glGetClipPlanef = make_glGetClipPlanef();
		public static final UnlinkedGLFunction glGetClipPlanefOES = make_glGetClipPlanefOES();
		public static final UnlinkedGLFunction glGetClipPlanex = make_glGetClipPlanex();
		public static final UnlinkedGLFunction glGetClipPlanexOES = make_glGetClipPlanexOES();
		public static final UnlinkedGLFunction glGetColorTable = make_glGetColorTable();
		public static final UnlinkedGLFunction glGetColorTableEXT = make_glGetColorTableEXT();
		public static final UnlinkedGLFunction glGetColorTableParameterfv = make_glGetColorTableParameterfv();
		public static final UnlinkedGLFunction glGetColorTableParameterfvEXT = make_glGetColorTableParameterfvEXT();
		public static final UnlinkedGLFunction glGetColorTableParameterfvSGI = make_glGetColorTableParameterfvSGI();
		public static final UnlinkedGLFunction glGetColorTableParameteriv = make_glGetColorTableParameteriv();
		public static final UnlinkedGLFunction glGetColorTableParameterivEXT = make_glGetColorTableParameterivEXT();
		public static final UnlinkedGLFunction glGetColorTableParameterivSGI = make_glGetColorTableParameterivSGI();
		public static final UnlinkedGLFunction glGetColorTableSGI = make_glGetColorTableSGI();
		public static final UnlinkedGLFunction glGetCombinerInputParameterfvNV = make_glGetCombinerInputParameterfvNV();
		public static final UnlinkedGLFunction glGetCombinerInputParameterivNV = make_glGetCombinerInputParameterivNV();
		public static final UnlinkedGLFunction glGetCombinerOutputParameterfvNV = make_glGetCombinerOutputParameterfvNV();
		public static final UnlinkedGLFunction glGetCombinerOutputParameterivNV = make_glGetCombinerOutputParameterivNV();
		public static final UnlinkedGLFunction glGetCombinerStageParameterfvNV = make_glGetCombinerStageParameterfvNV();
		public static final UnlinkedGLFunction glGetCommandHeaderNV = make_glGetCommandHeaderNV();
		public static final UnlinkedGLFunction glGetCompressedMultiTexImageEXT = make_glGetCompressedMultiTexImageEXT();
		public static final UnlinkedGLFunction glGetCompressedTexImage = make_glGetCompressedTexImage();
		public static final UnlinkedGLFunction glGetCompressedTexImageARB = make_glGetCompressedTexImageARB();
		public static final UnlinkedGLFunction glGetCompressedTextureImage = make_glGetCompressedTextureImage();
		public static final UnlinkedGLFunction glGetCompressedTextureImageEXT = make_glGetCompressedTextureImageEXT();
		public static final UnlinkedGLFunction glGetCompressedTextureSubImage = make_glGetCompressedTextureSubImage();
		public static final UnlinkedGLFunction glGetConvolutionFilter = make_glGetConvolutionFilter();
		public static final UnlinkedGLFunction glGetConvolutionFilterEXT = make_glGetConvolutionFilterEXT();
		public static final UnlinkedGLFunction glGetConvolutionParameterfv = make_glGetConvolutionParameterfv();
		public static final UnlinkedGLFunction glGetConvolutionParameterfvEXT = make_glGetConvolutionParameterfvEXT();
		public static final UnlinkedGLFunction glGetConvolutionParameteriv = make_glGetConvolutionParameteriv();
		public static final UnlinkedGLFunction glGetConvolutionParameterivEXT = make_glGetConvolutionParameterivEXT();
		public static final UnlinkedGLFunction glGetConvolutionParameterxvOES = make_glGetConvolutionParameterxvOES();
		public static final UnlinkedGLFunction glGetCoverageModulationTableNV = make_glGetCoverageModulationTableNV();
		public static final UnlinkedGLFunction glGetDebugMessageLog = make_glGetDebugMessageLog();
		public static final UnlinkedGLFunction glGetDebugMessageLogAMD = make_glGetDebugMessageLogAMD();
		public static final UnlinkedGLFunction glGetDebugMessageLogARB = make_glGetDebugMessageLogARB();
		public static final UnlinkedGLFunction glGetDebugMessageLogKHR = make_glGetDebugMessageLogKHR();
		public static final UnlinkedGLFunction glGetDetailTexFuncSGIS = make_glGetDetailTexFuncSGIS();
		public static final UnlinkedGLFunction glGetDoubleIndexedvEXT = make_glGetDoubleIndexedvEXT();
		public static final UnlinkedGLFunction glGetDoublei_v = make_glGetDoublei_v();
		public static final UnlinkedGLFunction glGetDoublei_vEXT = make_glGetDoublei_vEXT();
		public static final UnlinkedGLFunction glGetDoublev = make_glGetDoublev();
		public static final UnlinkedGLFunction glGetDriverControlStringQCOM = make_glGetDriverControlStringQCOM();
		public static final UnlinkedGLFunction glGetDriverControlsQCOM = make_glGetDriverControlsQCOM();
		public static final UnlinkedGLFunction glGetError = make_glGetError();
		public static final UnlinkedGLFunction glGetFenceivNV = make_glGetFenceivNV();
		public static final UnlinkedGLFunction glGetFinalCombinerInputParameterfvNV = make_glGetFinalCombinerInputParameterfvNV();
		public static final UnlinkedGLFunction glGetFinalCombinerInputParameterivNV = make_glGetFinalCombinerInputParameterivNV();
		public static final UnlinkedGLFunction glGetFirstPerfQueryIdINTEL = make_glGetFirstPerfQueryIdINTEL();
		public static final UnlinkedGLFunction glGetFixedv = make_glGetFixedv();
		public static final UnlinkedGLFunction glGetFixedvOES = make_glGetFixedvOES();
		public static final UnlinkedGLFunction glGetFloatIndexedvEXT = make_glGetFloatIndexedvEXT();
		public static final UnlinkedGLFunction glGetFloati_v = make_glGetFloati_v();
		public static final UnlinkedGLFunction glGetFloati_vEXT = make_glGetFloati_vEXT();
		public static final UnlinkedGLFunction glGetFloati_vNV = make_glGetFloati_vNV();
		public static final UnlinkedGLFunction glGetFloati_vOES = make_glGetFloati_vOES();
		public static final UnlinkedGLFunction glGetFloatv = make_glGetFloatv();
		public static final UnlinkedGLFunction glGetFogFuncSGIS = make_glGetFogFuncSGIS();
		public static final UnlinkedGLFunction glGetFragDataIndex = make_glGetFragDataIndex();
		public static final UnlinkedGLFunction glGetFragDataIndexEXT = make_glGetFragDataIndexEXT();
		public static final UnlinkedGLFunction glGetFragDataLocation = make_glGetFragDataLocation();
		public static final UnlinkedGLFunction glGetFragDataLocationEXT = make_glGetFragDataLocationEXT();
		public static final UnlinkedGLFunction glGetFragmentLightfvSGIX = make_glGetFragmentLightfvSGIX();
		public static final UnlinkedGLFunction glGetFragmentLightivSGIX = make_glGetFragmentLightivSGIX();
		public static final UnlinkedGLFunction glGetFragmentMaterialfvSGIX = make_glGetFragmentMaterialfvSGIX();
		public static final UnlinkedGLFunction glGetFragmentMaterialivSGIX = make_glGetFragmentMaterialivSGIX();
		public static final UnlinkedGLFunction glGetFragmentShadingRatesEXT = make_glGetFragmentShadingRatesEXT();
		public static final UnlinkedGLFunction glGetFramebufferAttachmentParameteriv = make_glGetFramebufferAttachmentParameteriv();
		public static final UnlinkedGLFunction glGetFramebufferAttachmentParameterivEXT = make_glGetFramebufferAttachmentParameterivEXT();
		public static final UnlinkedGLFunction glGetFramebufferAttachmentParameterivOES = make_glGetFramebufferAttachmentParameterivOES();
		public static final UnlinkedGLFunction glGetFramebufferParameterfvAMD = make_glGetFramebufferParameterfvAMD();
		public static final UnlinkedGLFunction glGetFramebufferParameteriv = make_glGetFramebufferParameteriv();
		public static final UnlinkedGLFunction glGetFramebufferParameterivEXT = make_glGetFramebufferParameterivEXT();
		public static final UnlinkedGLFunction glGetFramebufferPixelLocalStorageSizeEXT = make_glGetFramebufferPixelLocalStorageSizeEXT();
		public static final UnlinkedGLFunction glGetGraphicsResetStatus = make_glGetGraphicsResetStatus();
		public static final UnlinkedGLFunction glGetGraphicsResetStatusARB = make_glGetGraphicsResetStatusARB();
		public static final UnlinkedGLFunction glGetGraphicsResetStatusEXT = make_glGetGraphicsResetStatusEXT();
		public static final UnlinkedGLFunction glGetGraphicsResetStatusKHR = make_glGetGraphicsResetStatusKHR();
		public static final UnlinkedGLFunction glGetHandleARB = make_glGetHandleARB();
		public static final UnlinkedGLFunction glGetHistogram = make_glGetHistogram();
		public static final UnlinkedGLFunction glGetHistogramEXT = make_glGetHistogramEXT();
		public static final UnlinkedGLFunction glGetHistogramParameterfv = make_glGetHistogramParameterfv();
		public static final UnlinkedGLFunction glGetHistogramParameterfvEXT = make_glGetHistogramParameterfvEXT();
		public static final UnlinkedGLFunction glGetHistogramParameteriv = make_glGetHistogramParameteriv();
		public static final UnlinkedGLFunction glGetHistogramParameterivEXT = make_glGetHistogramParameterivEXT();
		public static final UnlinkedGLFunction glGetHistogramParameterxvOES = make_glGetHistogramParameterxvOES();
		public static final UnlinkedGLFunction glGetImageHandleARB = make_glGetImageHandleARB();
		public static final UnlinkedGLFunction glGetImageHandleNV = make_glGetImageHandleNV();
		public static final UnlinkedGLFunction glGetImageTransformParameterfvHP = make_glGetImageTransformParameterfvHP();
		public static final UnlinkedGLFunction glGetImageTransformParameterivHP = make_glGetImageTransformParameterivHP();
		public static final UnlinkedGLFunction glGetInfoLogARB = make_glGetInfoLogARB();
		public static final UnlinkedGLFunction glGetInstrumentsSGIX = make_glGetInstrumentsSGIX();
		public static final UnlinkedGLFunction glGetInteger64i_v = make_glGetInteger64i_v();
		public static final UnlinkedGLFunction glGetInteger64v = make_glGetInteger64v();
		public static final UnlinkedGLFunction glGetInteger64vAPPLE = make_glGetInteger64vAPPLE();
		public static final UnlinkedGLFunction glGetInteger64vEXT = make_glGetInteger64vEXT();
		public static final UnlinkedGLFunction glGetIntegerIndexedvEXT = make_glGetIntegerIndexedvEXT();
		public static final UnlinkedGLFunction glGetIntegeri_v = make_glGetIntegeri_v();
		public static final UnlinkedGLFunction glGetIntegeri_vEXT = make_glGetIntegeri_vEXT();
		public static final UnlinkedGLFunction glGetIntegerui64i_vNV = make_glGetIntegerui64i_vNV();
		public static final UnlinkedGLFunction glGetIntegerui64vNV = make_glGetIntegerui64vNV();
		public static final UnlinkedGLFunction glGetIntegerv = make_glGetIntegerv();
		public static final UnlinkedGLFunction glGetInternalformatSampleivNV = make_glGetInternalformatSampleivNV();
		public static final UnlinkedGLFunction glGetInternalformati64v = make_glGetInternalformati64v();
		public static final UnlinkedGLFunction glGetInternalformativ = make_glGetInternalformativ();
		public static final UnlinkedGLFunction glGetInvariantBooleanvEXT = make_glGetInvariantBooleanvEXT();
		public static final UnlinkedGLFunction glGetInvariantFloatvEXT = make_glGetInvariantFloatvEXT();
		public static final UnlinkedGLFunction glGetInvariantIntegervEXT = make_glGetInvariantIntegervEXT();
		public static final UnlinkedGLFunction glGetLightfv = make_glGetLightfv();
		public static final UnlinkedGLFunction glGetLightiv = make_glGetLightiv();
		public static final UnlinkedGLFunction glGetLightxOES = make_glGetLightxOES();
		public static final UnlinkedGLFunction glGetLightxv = make_glGetLightxv();
		public static final UnlinkedGLFunction glGetLightxvOES = make_glGetLightxvOES();
		public static final UnlinkedGLFunction glGetListParameterfvSGIX = make_glGetListParameterfvSGIX();
		public static final UnlinkedGLFunction glGetListParameterivSGIX = make_glGetListParameterivSGIX();
		public static final UnlinkedGLFunction glGetLocalConstantBooleanvEXT = make_glGetLocalConstantBooleanvEXT();
		public static final UnlinkedGLFunction glGetLocalConstantFloatvEXT = make_glGetLocalConstantFloatvEXT();
		public static final UnlinkedGLFunction glGetLocalConstantIntegervEXT = make_glGetLocalConstantIntegervEXT();
		public static final UnlinkedGLFunction glGetMapAttribParameterfvNV = make_glGetMapAttribParameterfvNV();
		public static final UnlinkedGLFunction glGetMapAttribParameterivNV = make_glGetMapAttribParameterivNV();
		public static final UnlinkedGLFunction glGetMapControlPointsNV = make_glGetMapControlPointsNV();
		public static final UnlinkedGLFunction glGetMapParameterfvNV = make_glGetMapParameterfvNV();
		public static final UnlinkedGLFunction glGetMapParameterivNV = make_glGetMapParameterivNV();
		public static final UnlinkedGLFunction glGetMapdv = make_glGetMapdv();
		public static final UnlinkedGLFunction glGetMapfv = make_glGetMapfv();
		public static final UnlinkedGLFunction glGetMapiv = make_glGetMapiv();
		public static final UnlinkedGLFunction glGetMapxvOES = make_glGetMapxvOES();
		public static final UnlinkedGLFunction glGetMaterialfv = make_glGetMaterialfv();
		public static final UnlinkedGLFunction glGetMaterialiv = make_glGetMaterialiv();
		public static final UnlinkedGLFunction glGetMaterialxOES = make_glGetMaterialxOES();
		public static final UnlinkedGLFunction glGetMaterialxv = make_glGetMaterialxv();
		public static final UnlinkedGLFunction glGetMaterialxvOES = make_glGetMaterialxvOES();
		public static final UnlinkedGLFunction glGetMemoryObjectDetachedResourcesuivNV = make_glGetMemoryObjectDetachedResourcesuivNV();
		public static final UnlinkedGLFunction glGetMemoryObjectParameterivEXT = make_glGetMemoryObjectParameterivEXT();
		public static final UnlinkedGLFunction glGetMinmax = make_glGetMinmax();
		public static final UnlinkedGLFunction glGetMinmaxEXT = make_glGetMinmaxEXT();
		public static final UnlinkedGLFunction glGetMinmaxParameterfv = make_glGetMinmaxParameterfv();
		public static final UnlinkedGLFunction glGetMinmaxParameterfvEXT = make_glGetMinmaxParameterfvEXT();
		public static final UnlinkedGLFunction glGetMinmaxParameteriv = make_glGetMinmaxParameteriv();
		public static final UnlinkedGLFunction glGetMinmaxParameterivEXT = make_glGetMinmaxParameterivEXT();
		public static final UnlinkedGLFunction glGetMultiTexEnvfvEXT = make_glGetMultiTexEnvfvEXT();
		public static final UnlinkedGLFunction glGetMultiTexEnvivEXT = make_glGetMultiTexEnvivEXT();
		public static final UnlinkedGLFunction glGetMultiTexGendvEXT = make_glGetMultiTexGendvEXT();
		public static final UnlinkedGLFunction glGetMultiTexGenfvEXT = make_glGetMultiTexGenfvEXT();
		public static final UnlinkedGLFunction glGetMultiTexGenivEXT = make_glGetMultiTexGenivEXT();
		public static final UnlinkedGLFunction glGetMultiTexImageEXT = make_glGetMultiTexImageEXT();
		public static final UnlinkedGLFunction glGetMultiTexLevelParameterfvEXT = make_glGetMultiTexLevelParameterfvEXT();
		public static final UnlinkedGLFunction glGetMultiTexLevelParameterivEXT = make_glGetMultiTexLevelParameterivEXT();
		public static final UnlinkedGLFunction glGetMultiTexParameterIivEXT = make_glGetMultiTexParameterIivEXT();
		public static final UnlinkedGLFunction glGetMultiTexParameterIuivEXT = make_glGetMultiTexParameterIuivEXT();
		public static final UnlinkedGLFunction glGetMultiTexParameterfvEXT = make_glGetMultiTexParameterfvEXT();
		public static final UnlinkedGLFunction glGetMultiTexParameterivEXT = make_glGetMultiTexParameterivEXT();
		public static final UnlinkedGLFunction glGetMultisamplefv = make_glGetMultisamplefv();
		public static final UnlinkedGLFunction glGetMultisamplefvNV = make_glGetMultisamplefvNV();
		public static final UnlinkedGLFunction glGetNamedBufferParameteri64v = make_glGetNamedBufferParameteri64v();
		public static final UnlinkedGLFunction glGetNamedBufferParameteriv = make_glGetNamedBufferParameteriv();
		public static final UnlinkedGLFunction glGetNamedBufferParameterivEXT = make_glGetNamedBufferParameterivEXT();
		public static final UnlinkedGLFunction glGetNamedBufferParameterui64vNV = make_glGetNamedBufferParameterui64vNV();
		public static final UnlinkedGLFunction glGetNamedBufferPointerv = make_glGetNamedBufferPointerv();
		public static final UnlinkedGLFunction glGetNamedBufferPointervEXT = make_glGetNamedBufferPointervEXT();
		public static final UnlinkedGLFunction glGetNamedBufferSubData = make_glGetNamedBufferSubData();
		public static final UnlinkedGLFunction glGetNamedBufferSubDataEXT = make_glGetNamedBufferSubDataEXT();
		public static final UnlinkedGLFunction glGetNamedFramebufferParameterfvAMD = make_glGetNamedFramebufferParameterfvAMD();
		public static final UnlinkedGLFunction glGetNamedFramebufferAttachmentParameteriv = make_glGetNamedFramebufferAttachmentParameteriv();
		public static final UnlinkedGLFunction glGetNamedFramebufferAttachmentParameterivEXT = make_glGetNamedFramebufferAttachmentParameterivEXT();
		public static final UnlinkedGLFunction glGetNamedFramebufferParameteriv = make_glGetNamedFramebufferParameteriv();
		public static final UnlinkedGLFunction glGetNamedFramebufferParameterivEXT = make_glGetNamedFramebufferParameterivEXT();
		public static final UnlinkedGLFunction glGetNamedProgramLocalParameterIivEXT = make_glGetNamedProgramLocalParameterIivEXT();
		public static final UnlinkedGLFunction glGetNamedProgramLocalParameterIuivEXT = make_glGetNamedProgramLocalParameterIuivEXT();
		public static final UnlinkedGLFunction glGetNamedProgramLocalParameterdvEXT = make_glGetNamedProgramLocalParameterdvEXT();
		public static final UnlinkedGLFunction glGetNamedProgramLocalParameterfvEXT = make_glGetNamedProgramLocalParameterfvEXT();
		public static final UnlinkedGLFunction glGetNamedProgramStringEXT = make_glGetNamedProgramStringEXT();
		public static final UnlinkedGLFunction glGetNamedProgramivEXT = make_glGetNamedProgramivEXT();
		public static final UnlinkedGLFunction glGetNamedRenderbufferParameteriv = make_glGetNamedRenderbufferParameteriv();
		public static final UnlinkedGLFunction glGetNamedRenderbufferParameterivEXT = make_glGetNamedRenderbufferParameterivEXT();
		public static final UnlinkedGLFunction glGetNamedStringARB = make_glGetNamedStringARB();
		public static final UnlinkedGLFunction glGetNamedStringivARB = make_glGetNamedStringivARB();
		public static final UnlinkedGLFunction glGetNextPerfQueryIdINTEL = make_glGetNextPerfQueryIdINTEL();
		public static final UnlinkedGLFunction glGetObjectBufferfvATI = make_glGetObjectBufferfvATI();
		public static final UnlinkedGLFunction glGetObjectBufferivATI = make_glGetObjectBufferivATI();
		public static final UnlinkedGLFunction glGetObjectLabel = make_glGetObjectLabel();
		public static final UnlinkedGLFunction glGetObjectLabelEXT = make_glGetObjectLabelEXT();
		public static final UnlinkedGLFunction glGetObjectLabelKHR = make_glGetObjectLabelKHR();
		public static final UnlinkedGLFunction glGetObjectParameterfvARB = make_glGetObjectParameterfvARB();
		public static final UnlinkedGLFunction glGetObjectParameterivAPPLE = make_glGetObjectParameterivAPPLE();
		public static final UnlinkedGLFunction glGetObjectParameterivARB = make_glGetObjectParameterivARB();
		public static final UnlinkedGLFunction glGetObjectPtrLabel = make_glGetObjectPtrLabel();
		public static final UnlinkedGLFunction glGetObjectPtrLabelKHR = make_glGetObjectPtrLabelKHR();
		public static final UnlinkedGLFunction glGetOcclusionQueryivNV = make_glGetOcclusionQueryivNV();
		public static final UnlinkedGLFunction glGetOcclusionQueryuivNV = make_glGetOcclusionQueryuivNV();
		public static final UnlinkedGLFunction glGetPathColorGenfvNV = make_glGetPathColorGenfvNV();
		public static final UnlinkedGLFunction glGetPathColorGenivNV = make_glGetPathColorGenivNV();
		public static final UnlinkedGLFunction glGetPathCommandsNV = make_glGetPathCommandsNV();
		public static final UnlinkedGLFunction glGetPathCoordsNV = make_glGetPathCoordsNV();
		public static final UnlinkedGLFunction glGetPathDashArrayNV = make_glGetPathDashArrayNV();
		public static final UnlinkedGLFunction glGetPathLengthNV = make_glGetPathLengthNV();
		public static final UnlinkedGLFunction glGetPathMetricRangeNV = make_glGetPathMetricRangeNV();
		public static final UnlinkedGLFunction glGetPathMetricsNV = make_glGetPathMetricsNV();
		public static final UnlinkedGLFunction glGetPathParameterfvNV = make_glGetPathParameterfvNV();
		public static final UnlinkedGLFunction glGetPathParameterivNV = make_glGetPathParameterivNV();
		public static final UnlinkedGLFunction glGetPathSpacingNV = make_glGetPathSpacingNV();
		public static final UnlinkedGLFunction glGetPathTexGenfvNV = make_glGetPathTexGenfvNV();
		public static final UnlinkedGLFunction glGetPathTexGenivNV = make_glGetPathTexGenivNV();
		public static final UnlinkedGLFunction glGetPerfCounterInfoINTEL = make_glGetPerfCounterInfoINTEL();
		public static final UnlinkedGLFunction glGetPerfMonitorCounterDataAMD = make_glGetPerfMonitorCounterDataAMD();
		public static final UnlinkedGLFunction glGetPerfMonitorCounterInfoAMD = make_glGetPerfMonitorCounterInfoAMD();
		public static final UnlinkedGLFunction glGetPerfMonitorCounterStringAMD = make_glGetPerfMonitorCounterStringAMD();
		public static final UnlinkedGLFunction glGetPerfMonitorCountersAMD = make_glGetPerfMonitorCountersAMD();
		public static final UnlinkedGLFunction glGetPerfMonitorGroupStringAMD = make_glGetPerfMonitorGroupStringAMD();
		public static final UnlinkedGLFunction glGetPerfMonitorGroupsAMD = make_glGetPerfMonitorGroupsAMD();
		public static final UnlinkedGLFunction glGetPerfQueryDataINTEL = make_glGetPerfQueryDataINTEL();
		public static final UnlinkedGLFunction glGetPerfQueryIdByNameINTEL = make_glGetPerfQueryIdByNameINTEL();
		public static final UnlinkedGLFunction glGetPerfQueryInfoINTEL = make_glGetPerfQueryInfoINTEL();
		public static final UnlinkedGLFunction glGetPixelMapfv = make_glGetPixelMapfv();
		public static final UnlinkedGLFunction glGetPixelMapuiv = make_glGetPixelMapuiv();
		public static final UnlinkedGLFunction glGetPixelMapusv = make_glGetPixelMapusv();
		public static final UnlinkedGLFunction glGetPixelMapxv = make_glGetPixelMapxv();
		public static final UnlinkedGLFunction glGetPixelTexGenParameterfvSGIS = make_glGetPixelTexGenParameterfvSGIS();
		public static final UnlinkedGLFunction glGetPixelTexGenParameterivSGIS = make_glGetPixelTexGenParameterivSGIS();
		public static final UnlinkedGLFunction glGetPixelTransformParameterfvEXT = make_glGetPixelTransformParameterfvEXT();
		public static final UnlinkedGLFunction glGetPixelTransformParameterivEXT = make_glGetPixelTransformParameterivEXT();
		public static final UnlinkedGLFunction glGetPointerIndexedvEXT = make_glGetPointerIndexedvEXT();
		public static final UnlinkedGLFunction glGetPointeri_vEXT = make_glGetPointeri_vEXT();
		public static final UnlinkedGLFunction glGetPointerv = make_glGetPointerv();
		public static final UnlinkedGLFunction glGetPointervEXT = make_glGetPointervEXT();
		public static final UnlinkedGLFunction glGetPointervKHR = make_glGetPointervKHR();
		public static final UnlinkedGLFunction glGetPolygonStipple = make_glGetPolygonStipple();
		public static final UnlinkedGLFunction glGetProgramBinary = make_glGetProgramBinary();
		public static final UnlinkedGLFunction glGetProgramBinaryOES = make_glGetProgramBinaryOES();
		public static final UnlinkedGLFunction glGetProgramEnvParameterIivNV = make_glGetProgramEnvParameterIivNV();
		public static final UnlinkedGLFunction glGetProgramEnvParameterIuivNV = make_glGetProgramEnvParameterIuivNV();
		public static final UnlinkedGLFunction glGetProgramEnvParameterdvARB = make_glGetProgramEnvParameterdvARB();
		public static final UnlinkedGLFunction glGetProgramEnvParameterfvARB = make_glGetProgramEnvParameterfvARB();
		public static final UnlinkedGLFunction glGetProgramInfoLog = make_glGetProgramInfoLog();
		public static final UnlinkedGLFunction glGetProgramInterfaceiv = make_glGetProgramInterfaceiv();
		public static final UnlinkedGLFunction glGetProgramLocalParameterIivNV = make_glGetProgramLocalParameterIivNV();
		public static final UnlinkedGLFunction glGetProgramLocalParameterIuivNV = make_glGetProgramLocalParameterIuivNV();
		public static final UnlinkedGLFunction glGetProgramLocalParameterdvARB = make_glGetProgramLocalParameterdvARB();
		public static final UnlinkedGLFunction glGetProgramLocalParameterfvARB = make_glGetProgramLocalParameterfvARB();
		public static final UnlinkedGLFunction glGetProgramNamedParameterdvNV = make_glGetProgramNamedParameterdvNV();
		public static final UnlinkedGLFunction glGetProgramNamedParameterfvNV = make_glGetProgramNamedParameterfvNV();
		public static final UnlinkedGLFunction glGetProgramParameterdvNV = make_glGetProgramParameterdvNV();
		public static final UnlinkedGLFunction glGetProgramParameterfvNV = make_glGetProgramParameterfvNV();
		public static final UnlinkedGLFunction glGetProgramPipelineInfoLog = make_glGetProgramPipelineInfoLog();
		public static final UnlinkedGLFunction glGetProgramPipelineInfoLogEXT = make_glGetProgramPipelineInfoLogEXT();
		public static final UnlinkedGLFunction glGetProgramPipelineiv = make_glGetProgramPipelineiv();
		public static final UnlinkedGLFunction glGetProgramPipelineivEXT = make_glGetProgramPipelineivEXT();
		public static final UnlinkedGLFunction glGetProgramResourceIndex = make_glGetProgramResourceIndex();
		public static final UnlinkedGLFunction glGetProgramResourceLocation = make_glGetProgramResourceLocation();
		public static final UnlinkedGLFunction glGetProgramResourceLocationIndex = make_glGetProgramResourceLocationIndex();
		public static final UnlinkedGLFunction glGetProgramResourceLocationIndexEXT = make_glGetProgramResourceLocationIndexEXT();
		public static final UnlinkedGLFunction glGetProgramResourceName = make_glGetProgramResourceName();
		public static final UnlinkedGLFunction glGetProgramResourcefvNV = make_glGetProgramResourcefvNV();
		public static final UnlinkedGLFunction glGetProgramResourceiv = make_glGetProgramResourceiv();
		public static final UnlinkedGLFunction glGetProgramStageiv = make_glGetProgramStageiv();
		public static final UnlinkedGLFunction glGetProgramStringARB = make_glGetProgramStringARB();
		public static final UnlinkedGLFunction glGetProgramStringNV = make_glGetProgramStringNV();
		public static final UnlinkedGLFunction glGetProgramSubroutineParameteruivNV = make_glGetProgramSubroutineParameteruivNV();
		public static final UnlinkedGLFunction glGetProgramiv = make_glGetProgramiv();
		public static final UnlinkedGLFunction glGetProgramivARB = make_glGetProgramivARB();
		public static final UnlinkedGLFunction glGetProgramivNV = make_glGetProgramivNV();
		public static final UnlinkedGLFunction glGetQueryBufferObjecti64v = make_glGetQueryBufferObjecti64v();
		public static final UnlinkedGLFunction glGetQueryBufferObjectiv = make_glGetQueryBufferObjectiv();
		public static final UnlinkedGLFunction glGetQueryBufferObjectui64v = make_glGetQueryBufferObjectui64v();
		public static final UnlinkedGLFunction glGetQueryBufferObjectuiv = make_glGetQueryBufferObjectuiv();
		public static final UnlinkedGLFunction glGetQueryIndexediv = make_glGetQueryIndexediv();
		public static final UnlinkedGLFunction glGetQueryObjecti64v = make_glGetQueryObjecti64v();
		public static final UnlinkedGLFunction glGetQueryObjecti64vEXT = make_glGetQueryObjecti64vEXT();
		public static final UnlinkedGLFunction glGetQueryObjectiv = make_glGetQueryObjectiv();
		public static final UnlinkedGLFunction glGetQueryObjectivARB = make_glGetQueryObjectivARB();
		public static final UnlinkedGLFunction glGetQueryObjectivEXT = make_glGetQueryObjectivEXT();
		public static final UnlinkedGLFunction glGetQueryObjectui64v = make_glGetQueryObjectui64v();
		public static final UnlinkedGLFunction glGetQueryObjectui64vEXT = make_glGetQueryObjectui64vEXT();
		public static final UnlinkedGLFunction glGetQueryObjectuiv = make_glGetQueryObjectuiv();
		public static final UnlinkedGLFunction glGetQueryObjectuivARB = make_glGetQueryObjectuivARB();
		public static final UnlinkedGLFunction glGetQueryObjectuivEXT = make_glGetQueryObjectuivEXT();
		public static final UnlinkedGLFunction glGetQueryiv = make_glGetQueryiv();
		public static final UnlinkedGLFunction glGetQueryivARB = make_glGetQueryivARB();
		public static final UnlinkedGLFunction glGetQueryivEXT = make_glGetQueryivEXT();
		public static final UnlinkedGLFunction glGetRenderbufferParameteriv = make_glGetRenderbufferParameteriv();
		public static final UnlinkedGLFunction glGetRenderbufferParameterivEXT = make_glGetRenderbufferParameterivEXT();
		public static final UnlinkedGLFunction glGetRenderbufferParameterivOES = make_glGetRenderbufferParameterivOES();
		public static final UnlinkedGLFunction glGetSamplerParameterIiv = make_glGetSamplerParameterIiv();
		public static final UnlinkedGLFunction glGetSamplerParameterIivEXT = make_glGetSamplerParameterIivEXT();
		public static final UnlinkedGLFunction glGetSamplerParameterIivOES = make_glGetSamplerParameterIivOES();
		public static final UnlinkedGLFunction glGetSamplerParameterIuiv = make_glGetSamplerParameterIuiv();
		public static final UnlinkedGLFunction glGetSamplerParameterIuivEXT = make_glGetSamplerParameterIuivEXT();
		public static final UnlinkedGLFunction glGetSamplerParameterIuivOES = make_glGetSamplerParameterIuivOES();
		public static final UnlinkedGLFunction glGetSamplerParameterfv = make_glGetSamplerParameterfv();
		public static final UnlinkedGLFunction glGetSamplerParameteriv = make_glGetSamplerParameteriv();
		public static final UnlinkedGLFunction glGetSemaphoreParameterivNV = make_glGetSemaphoreParameterivNV();
		public static final UnlinkedGLFunction glGetSemaphoreParameterui64vEXT = make_glGetSemaphoreParameterui64vEXT();
		public static final UnlinkedGLFunction glGetSeparableFilter = make_glGetSeparableFilter();
		public static final UnlinkedGLFunction glGetSeparableFilterEXT = make_glGetSeparableFilterEXT();
		public static final UnlinkedGLFunction glGetShaderInfoLog = make_glGetShaderInfoLog();
		public static final UnlinkedGLFunction glGetShaderPrecisionFormat = make_glGetShaderPrecisionFormat();
		public static final UnlinkedGLFunction glGetShaderSource = make_glGetShaderSource();
		public static final UnlinkedGLFunction glGetShaderSourceARB = make_glGetShaderSourceARB();
		public static final UnlinkedGLFunction glGetShaderiv = make_glGetShaderiv();
		public static final UnlinkedGLFunction glGetShadingRateImagePaletteNV = make_glGetShadingRateImagePaletteNV();
		public static final UnlinkedGLFunction glGetShadingRateSampleLocationivNV = make_glGetShadingRateSampleLocationivNV();
		public static final UnlinkedGLFunction glGetSharpenTexFuncSGIS = make_glGetSharpenTexFuncSGIS();
		public static final UnlinkedGLFunction glGetStageIndexNV = make_glGetStageIndexNV();
		public static final UnlinkedGLFunction glGetString = make_glGetString();
		public static final UnlinkedGLFunction glGetStringi = make_glGetStringi();
		public static final UnlinkedGLFunction glGetSubroutineIndex = make_glGetSubroutineIndex();
		public static final UnlinkedGLFunction glGetSubroutineUniformLocation = make_glGetSubroutineUniformLocation();
		public static final UnlinkedGLFunction glGetSynciv = make_glGetSynciv();
		public static final UnlinkedGLFunction glGetSyncivAPPLE = make_glGetSyncivAPPLE();
		public static final UnlinkedGLFunction glGetTexBumpParameterfvATI = make_glGetTexBumpParameterfvATI();
		public static final UnlinkedGLFunction glGetTexBumpParameterivATI = make_glGetTexBumpParameterivATI();
		public static final UnlinkedGLFunction glGetTexEnvfv = make_glGetTexEnvfv();
		public static final UnlinkedGLFunction glGetTexEnviv = make_glGetTexEnviv();
		public static final UnlinkedGLFunction glGetTexEnvxv = make_glGetTexEnvxv();
		public static final UnlinkedGLFunction glGetTexEnvxvOES = make_glGetTexEnvxvOES();
		public static final UnlinkedGLFunction glGetTexFilterFuncSGIS = make_glGetTexFilterFuncSGIS();
		public static final UnlinkedGLFunction glGetTexGendv = make_glGetTexGendv();
		public static final UnlinkedGLFunction glGetTexGenfv = make_glGetTexGenfv();
		public static final UnlinkedGLFunction glGetTexGenfvOES = make_glGetTexGenfvOES();
		public static final UnlinkedGLFunction glGetTexGeniv = make_glGetTexGeniv();
		public static final UnlinkedGLFunction glGetTexGenivOES = make_glGetTexGenivOES();
		public static final UnlinkedGLFunction glGetTexGenxvOES = make_glGetTexGenxvOES();
		public static final UnlinkedGLFunction glGetTexImage = make_glGetTexImage();
		public static final UnlinkedGLFunction glGetTexLevelParameterfv = make_glGetTexLevelParameterfv();
		public static final UnlinkedGLFunction glGetTexLevelParameteriv = make_glGetTexLevelParameteriv();
		public static final UnlinkedGLFunction glGetTexLevelParameterxvOES = make_glGetTexLevelParameterxvOES();
		public static final UnlinkedGLFunction glGetTexParameterIiv = make_glGetTexParameterIiv();
		public static final UnlinkedGLFunction glGetTexParameterIivEXT = make_glGetTexParameterIivEXT();
		public static final UnlinkedGLFunction glGetTexParameterIivOES = make_glGetTexParameterIivOES();
		public static final UnlinkedGLFunction glGetTexParameterIuiv = make_glGetTexParameterIuiv();
		public static final UnlinkedGLFunction glGetTexParameterIuivEXT = make_glGetTexParameterIuivEXT();
		public static final UnlinkedGLFunction glGetTexParameterIuivOES = make_glGetTexParameterIuivOES();
		public static final UnlinkedGLFunction glGetTexParameterPointervAPPLE = make_glGetTexParameterPointervAPPLE();
		public static final UnlinkedGLFunction glGetTexParameterfv = make_glGetTexParameterfv();
		public static final UnlinkedGLFunction glGetTexParameteriv = make_glGetTexParameteriv();
		public static final UnlinkedGLFunction glGetTexParameterxv = make_glGetTexParameterxv();
		public static final UnlinkedGLFunction glGetTexParameterxvOES = make_glGetTexParameterxvOES();
		public static final UnlinkedGLFunction glGetTextureHandleARB = make_glGetTextureHandleARB();
		public static final UnlinkedGLFunction glGetTextureHandleIMG = make_glGetTextureHandleIMG();
		public static final UnlinkedGLFunction glGetTextureHandleNV = make_glGetTextureHandleNV();
		public static final UnlinkedGLFunction glGetTextureImage = make_glGetTextureImage();
		public static final UnlinkedGLFunction glGetTextureImageEXT = make_glGetTextureImageEXT();
		public static final UnlinkedGLFunction glGetTextureLevelParameterfv = make_glGetTextureLevelParameterfv();
		public static final UnlinkedGLFunction glGetTextureLevelParameterfvEXT = make_glGetTextureLevelParameterfvEXT();
		public static final UnlinkedGLFunction glGetTextureLevelParameteriv = make_glGetTextureLevelParameteriv();
		public static final UnlinkedGLFunction glGetTextureLevelParameterivEXT = make_glGetTextureLevelParameterivEXT();
		public static final UnlinkedGLFunction glGetTextureParameterIiv = make_glGetTextureParameterIiv();
		public static final UnlinkedGLFunction glGetTextureParameterIivEXT = make_glGetTextureParameterIivEXT();
		public static final UnlinkedGLFunction glGetTextureParameterIuiv = make_glGetTextureParameterIuiv();
		public static final UnlinkedGLFunction glGetTextureParameterIuivEXT = make_glGetTextureParameterIuivEXT();
		public static final UnlinkedGLFunction glGetTextureParameterfv = make_glGetTextureParameterfv();
		public static final UnlinkedGLFunction glGetTextureParameterfvEXT = make_glGetTextureParameterfvEXT();
		public static final UnlinkedGLFunction glGetTextureParameteriv = make_glGetTextureParameteriv();
		public static final UnlinkedGLFunction glGetTextureParameterivEXT = make_glGetTextureParameterivEXT();
		public static final UnlinkedGLFunction glGetTextureSamplerHandleARB = make_glGetTextureSamplerHandleARB();
		public static final UnlinkedGLFunction glGetTextureSamplerHandleIMG = make_glGetTextureSamplerHandleIMG();
		public static final UnlinkedGLFunction glGetTextureSamplerHandleNV = make_glGetTextureSamplerHandleNV();
		public static final UnlinkedGLFunction glGetTextureSubImage = make_glGetTextureSubImage();
		public static final UnlinkedGLFunction glGetTrackMatrixivNV = make_glGetTrackMatrixivNV();
		public static final UnlinkedGLFunction glGetTransformFeedbackVarying = make_glGetTransformFeedbackVarying();
		public static final UnlinkedGLFunction glGetTransformFeedbackVaryingEXT = make_glGetTransformFeedbackVaryingEXT();
		public static final UnlinkedGLFunction glGetTransformFeedbackVaryingNV = make_glGetTransformFeedbackVaryingNV();
		public static final UnlinkedGLFunction glGetTransformFeedbacki64_v = make_glGetTransformFeedbacki64_v();
		public static final UnlinkedGLFunction glGetTransformFeedbacki_v = make_glGetTransformFeedbacki_v();
		public static final UnlinkedGLFunction glGetTransformFeedbackiv = make_glGetTransformFeedbackiv();
		public static final UnlinkedGLFunction glGetTranslatedShaderSourceANGLE = make_glGetTranslatedShaderSourceANGLE();
		public static final UnlinkedGLFunction glGetUniformBlockIndex = make_glGetUniformBlockIndex();
		public static final UnlinkedGLFunction glGetUniformBufferSizeEXT = make_glGetUniformBufferSizeEXT();
		public static final UnlinkedGLFunction glGetUniformIndices = make_glGetUniformIndices();
		public static final UnlinkedGLFunction glGetUniformLocation = make_glGetUniformLocation();
		public static final UnlinkedGLFunction glGetUniformLocationARB = make_glGetUniformLocationARB();
		public static final UnlinkedGLFunction glGetUniformOffsetEXT = make_glGetUniformOffsetEXT();
		public static final UnlinkedGLFunction glGetUniformSubroutineuiv = make_glGetUniformSubroutineuiv();
		public static final UnlinkedGLFunction glGetUniformdv = make_glGetUniformdv();
		public static final UnlinkedGLFunction glGetUniformfv = make_glGetUniformfv();
		public static final UnlinkedGLFunction glGetUniformfvARB = make_glGetUniformfvARB();
		public static final UnlinkedGLFunction glGetUniformi64vARB = make_glGetUniformi64vARB();
		public static final UnlinkedGLFunction glGetUniformi64vNV = make_glGetUniformi64vNV();
		public static final UnlinkedGLFunction glGetUniformiv = make_glGetUniformiv();
		public static final UnlinkedGLFunction glGetUniformivARB = make_glGetUniformivARB();
		public static final UnlinkedGLFunction glGetUniformui64vARB = make_glGetUniformui64vARB();
		public static final UnlinkedGLFunction glGetUniformui64vNV = make_glGetUniformui64vNV();
		public static final UnlinkedGLFunction glGetUniformuiv = make_glGetUniformuiv();
		public static final UnlinkedGLFunction glGetUniformuivEXT = make_glGetUniformuivEXT();
		public static final UnlinkedGLFunction glGetUnsignedBytevEXT = make_glGetUnsignedBytevEXT();
		public static final UnlinkedGLFunction glGetUnsignedBytei_vEXT = make_glGetUnsignedBytei_vEXT();
		public static final UnlinkedGLFunction glGetVariantArrayObjectfvATI = make_glGetVariantArrayObjectfvATI();
		public static final UnlinkedGLFunction glGetVariantArrayObjectivATI = make_glGetVariantArrayObjectivATI();
		public static final UnlinkedGLFunction glGetVariantBooleanvEXT = make_glGetVariantBooleanvEXT();
		public static final UnlinkedGLFunction glGetVariantFloatvEXT = make_glGetVariantFloatvEXT();
		public static final UnlinkedGLFunction glGetVariantIntegervEXT = make_glGetVariantIntegervEXT();
		public static final UnlinkedGLFunction glGetVariantPointervEXT = make_glGetVariantPointervEXT();
		public static final UnlinkedGLFunction glGetVaryingLocationNV = make_glGetVaryingLocationNV();
		public static final UnlinkedGLFunction glGetVertexArrayIndexed64iv = make_glGetVertexArrayIndexed64iv();
		public static final UnlinkedGLFunction glGetVertexArrayIndexediv = make_glGetVertexArrayIndexediv();
		public static final UnlinkedGLFunction glGetVertexArrayIntegeri_vEXT = make_glGetVertexArrayIntegeri_vEXT();
		public static final UnlinkedGLFunction glGetVertexArrayIntegervEXT = make_glGetVertexArrayIntegervEXT();
		public static final UnlinkedGLFunction glGetVertexArrayPointeri_vEXT = make_glGetVertexArrayPointeri_vEXT();
		public static final UnlinkedGLFunction glGetVertexArrayPointervEXT = make_glGetVertexArrayPointervEXT();
		public static final UnlinkedGLFunction glGetVertexArrayiv = make_glGetVertexArrayiv();
		public static final UnlinkedGLFunction glGetVertexAttribArrayObjectfvATI = make_glGetVertexAttribArrayObjectfvATI();
		public static final UnlinkedGLFunction glGetVertexAttribArrayObjectivATI = make_glGetVertexAttribArrayObjectivATI();
		public static final UnlinkedGLFunction glGetVertexAttribIiv = make_glGetVertexAttribIiv();
		public static final UnlinkedGLFunction glGetVertexAttribIivEXT = make_glGetVertexAttribIivEXT();
		public static final UnlinkedGLFunction glGetVertexAttribIuiv = make_glGetVertexAttribIuiv();
		public static final UnlinkedGLFunction glGetVertexAttribIuivEXT = make_glGetVertexAttribIuivEXT();
		public static final UnlinkedGLFunction glGetVertexAttribLdv = make_glGetVertexAttribLdv();
		public static final UnlinkedGLFunction glGetVertexAttribLdvEXT = make_glGetVertexAttribLdvEXT();
		public static final UnlinkedGLFunction glGetVertexAttribLi64vNV = make_glGetVertexAttribLi64vNV();
		public static final UnlinkedGLFunction glGetVertexAttribLui64vARB = make_glGetVertexAttribLui64vARB();
		public static final UnlinkedGLFunction glGetVertexAttribLui64vNV = make_glGetVertexAttribLui64vNV();
		public static final UnlinkedGLFunction glGetVertexAttribPointerv = make_glGetVertexAttribPointerv();
		public static final UnlinkedGLFunction glGetVertexAttribPointervARB = make_glGetVertexAttribPointervARB();
		public static final UnlinkedGLFunction glGetVertexAttribPointervNV = make_glGetVertexAttribPointervNV();
		public static final UnlinkedGLFunction glGetVertexAttribdv = make_glGetVertexAttribdv();
		public static final UnlinkedGLFunction glGetVertexAttribdvARB = make_glGetVertexAttribdvARB();
		public static final UnlinkedGLFunction glGetVertexAttribdvNV = make_glGetVertexAttribdvNV();
		public static final UnlinkedGLFunction glGetVertexAttribfv = make_glGetVertexAttribfv();
		public static final UnlinkedGLFunction glGetVertexAttribfvARB = make_glGetVertexAttribfvARB();
		public static final UnlinkedGLFunction glGetVertexAttribfvNV = make_glGetVertexAttribfvNV();
		public static final UnlinkedGLFunction glGetVertexAttribiv = make_glGetVertexAttribiv();
		public static final UnlinkedGLFunction glGetVertexAttribivARB = make_glGetVertexAttribivARB();
		public static final UnlinkedGLFunction glGetVertexAttribivNV = make_glGetVertexAttribivNV();
		public static final UnlinkedGLFunction glGetVideoCaptureStreamdvNV = make_glGetVideoCaptureStreamdvNV();
		public static final UnlinkedGLFunction glGetVideoCaptureStreamfvNV = make_glGetVideoCaptureStreamfvNV();
		public static final UnlinkedGLFunction glGetVideoCaptureStreamivNV = make_glGetVideoCaptureStreamivNV();
		public static final UnlinkedGLFunction glGetVideoCaptureivNV = make_glGetVideoCaptureivNV();
		public static final UnlinkedGLFunction glGetVideoi64vNV = make_glGetVideoi64vNV();
		public static final UnlinkedGLFunction glGetVideoivNV = make_glGetVideoivNV();
		public static final UnlinkedGLFunction glGetVideoui64vNV = make_glGetVideoui64vNV();
		public static final UnlinkedGLFunction glGetVideouivNV = make_glGetVideouivNV();
		public static final UnlinkedGLFunction glGetnColorTable = make_glGetnColorTable();
		public static final UnlinkedGLFunction glGetnColorTableARB = make_glGetnColorTableARB();
		public static final UnlinkedGLFunction glGetnCompressedTexImage = make_glGetnCompressedTexImage();
		public static final UnlinkedGLFunction glGetnCompressedTexImageARB = make_glGetnCompressedTexImageARB();
		public static final UnlinkedGLFunction glGetnConvolutionFilter = make_glGetnConvolutionFilter();
		public static final UnlinkedGLFunction glGetnConvolutionFilterARB = make_glGetnConvolutionFilterARB();
		public static final UnlinkedGLFunction glGetnHistogram = make_glGetnHistogram();
		public static final UnlinkedGLFunction glGetnHistogramARB = make_glGetnHistogramARB();
		public static final UnlinkedGLFunction glGetnMapdv = make_glGetnMapdv();
		public static final UnlinkedGLFunction glGetnMapdvARB = make_glGetnMapdvARB();
		public static final UnlinkedGLFunction glGetnMapfv = make_glGetnMapfv();
		public static final UnlinkedGLFunction glGetnMapfvARB = make_glGetnMapfvARB();
		public static final UnlinkedGLFunction glGetnMapiv = make_glGetnMapiv();
		public static final UnlinkedGLFunction glGetnMapivARB = make_glGetnMapivARB();
		public static final UnlinkedGLFunction glGetnMinmax = make_glGetnMinmax();
		public static final UnlinkedGLFunction glGetnMinmaxARB = make_glGetnMinmaxARB();
		public static final UnlinkedGLFunction glGetnPixelMapfv = make_glGetnPixelMapfv();
		public static final UnlinkedGLFunction glGetnPixelMapfvARB = make_glGetnPixelMapfvARB();
		public static final UnlinkedGLFunction glGetnPixelMapuiv = make_glGetnPixelMapuiv();
		public static final UnlinkedGLFunction glGetnPixelMapuivARB = make_glGetnPixelMapuivARB();
		public static final UnlinkedGLFunction glGetnPixelMapusv = make_glGetnPixelMapusv();
		public static final UnlinkedGLFunction glGetnPixelMapusvARB = make_glGetnPixelMapusvARB();
		public static final UnlinkedGLFunction glGetnPolygonStipple = make_glGetnPolygonStipple();
		public static final UnlinkedGLFunction glGetnPolygonStippleARB = make_glGetnPolygonStippleARB();
		public static final UnlinkedGLFunction glGetnSeparableFilter = make_glGetnSeparableFilter();
		public static final UnlinkedGLFunction glGetnSeparableFilterARB = make_glGetnSeparableFilterARB();
		public static final UnlinkedGLFunction glGetnTexImage = make_glGetnTexImage();
		public static final UnlinkedGLFunction glGetnTexImageARB = make_glGetnTexImageARB();
		public static final UnlinkedGLFunction glGetnUniformdv = make_glGetnUniformdv();
		public static final UnlinkedGLFunction glGetnUniformdvARB = make_glGetnUniformdvARB();
		public static final UnlinkedGLFunction glGetnUniformfv = make_glGetnUniformfv();
		public static final UnlinkedGLFunction glGetnUniformfvARB = make_glGetnUniformfvARB();
		public static final UnlinkedGLFunction glGetnUniformfvEXT = make_glGetnUniformfvEXT();
		public static final UnlinkedGLFunction glGetnUniformfvKHR = make_glGetnUniformfvKHR();
		public static final UnlinkedGLFunction glGetnUniformi64vARB = make_glGetnUniformi64vARB();
		public static final UnlinkedGLFunction glGetnUniformiv = make_glGetnUniformiv();
		public static final UnlinkedGLFunction glGetnUniformivARB = make_glGetnUniformivARB();
		public static final UnlinkedGLFunction glGetnUniformivEXT = make_glGetnUniformivEXT();
		public static final UnlinkedGLFunction glGetnUniformivKHR = make_glGetnUniformivKHR();
		public static final UnlinkedGLFunction glGetnUniformui64vARB = make_glGetnUniformui64vARB();
		public static final UnlinkedGLFunction glGetnUniformuiv = make_glGetnUniformuiv();
		public static final UnlinkedGLFunction glGetnUniformuivARB = make_glGetnUniformuivARB();
		public static final UnlinkedGLFunction glGetnUniformuivKHR = make_glGetnUniformuivKHR();
		public static final UnlinkedGLFunction glGlobalAlphaFactorbSUN = make_glGlobalAlphaFactorbSUN();
		public static final UnlinkedGLFunction glGlobalAlphaFactordSUN = make_glGlobalAlphaFactordSUN();
		public static final UnlinkedGLFunction glGlobalAlphaFactorfSUN = make_glGlobalAlphaFactorfSUN();
		public static final UnlinkedGLFunction glGlobalAlphaFactoriSUN = make_glGlobalAlphaFactoriSUN();
		public static final UnlinkedGLFunction glGlobalAlphaFactorsSUN = make_glGlobalAlphaFactorsSUN();
		public static final UnlinkedGLFunction glGlobalAlphaFactorubSUN = make_glGlobalAlphaFactorubSUN();
		public static final UnlinkedGLFunction glGlobalAlphaFactoruiSUN = make_glGlobalAlphaFactoruiSUN();
		public static final UnlinkedGLFunction glGlobalAlphaFactorusSUN = make_glGlobalAlphaFactorusSUN();
		public static final UnlinkedGLFunction glHint = make_glHint();
		public static final UnlinkedGLFunction glHintPGI = make_glHintPGI();
		public static final UnlinkedGLFunction glHistogram = make_glHistogram();
		public static final UnlinkedGLFunction glHistogramEXT = make_glHistogramEXT();
		public static final UnlinkedGLFunction glIglooInterfaceSGIX = make_glIglooInterfaceSGIX();
		public static final UnlinkedGLFunction glImageTransformParameterfHP = make_glImageTransformParameterfHP();
		public static final UnlinkedGLFunction glImageTransformParameterfvHP = make_glImageTransformParameterfvHP();
		public static final UnlinkedGLFunction glImageTransformParameteriHP = make_glImageTransformParameteriHP();
		public static final UnlinkedGLFunction glImageTransformParameterivHP = make_glImageTransformParameterivHP();
		public static final UnlinkedGLFunction glImportMemoryFdEXT = make_glImportMemoryFdEXT();
		public static final UnlinkedGLFunction glImportMemoryWin32HandleEXT = make_glImportMemoryWin32HandleEXT();
		public static final UnlinkedGLFunction glImportMemoryWin32NameEXT = make_glImportMemoryWin32NameEXT();
		public static final UnlinkedGLFunction glImportSemaphoreFdEXT = make_glImportSemaphoreFdEXT();
		public static final UnlinkedGLFunction glImportSemaphoreWin32HandleEXT = make_glImportSemaphoreWin32HandleEXT();
		public static final UnlinkedGLFunction glImportSemaphoreWin32NameEXT = make_glImportSemaphoreWin32NameEXT();
		public static final UnlinkedGLFunction glImportSyncEXT = make_glImportSyncEXT();
		public static final UnlinkedGLFunction glIndexFormatNV = make_glIndexFormatNV();
		public static final UnlinkedGLFunction glIndexFuncEXT = make_glIndexFuncEXT();
		public static final UnlinkedGLFunction glIndexMask = make_glIndexMask();
		public static final UnlinkedGLFunction glIndexMaterialEXT = make_glIndexMaterialEXT();
		public static final UnlinkedGLFunction glIndexPointer = make_glIndexPointer();
		public static final UnlinkedGLFunction glIndexPointerEXT = make_glIndexPointerEXT();
		public static final UnlinkedGLFunction glIndexPointerListIBM = make_glIndexPointerListIBM();
		public static final UnlinkedGLFunction glIndexd = make_glIndexd();
		public static final UnlinkedGLFunction glIndexdv = make_glIndexdv();
		public static final UnlinkedGLFunction glIndexf = make_glIndexf();
		public static final UnlinkedGLFunction glIndexfv = make_glIndexfv();
		public static final UnlinkedGLFunction glIndexi = make_glIndexi();
		public static final UnlinkedGLFunction glIndexiv = make_glIndexiv();
		public static final UnlinkedGLFunction glIndexs = make_glIndexs();
		public static final UnlinkedGLFunction glIndexsv = make_glIndexsv();
		public static final UnlinkedGLFunction glIndexub = make_glIndexub();
		public static final UnlinkedGLFunction glIndexubv = make_glIndexubv();
		public static final UnlinkedGLFunction glIndexxOES = make_glIndexxOES();
		public static final UnlinkedGLFunction glIndexxvOES = make_glIndexxvOES();
		public static final UnlinkedGLFunction glInitNames = make_glInitNames();
		public static final UnlinkedGLFunction glInsertComponentEXT = make_glInsertComponentEXT();
		public static final UnlinkedGLFunction glInsertEventMarkerEXT = make_glInsertEventMarkerEXT();
		public static final UnlinkedGLFunction glInstrumentsBufferSGIX = make_glInstrumentsBufferSGIX();
		public static final UnlinkedGLFunction glInterleavedArrays = make_glInterleavedArrays();
		public static final UnlinkedGLFunction glInterpolatePathsNV = make_glInterpolatePathsNV();
		public static final UnlinkedGLFunction glInvalidateBufferData = make_glInvalidateBufferData();
		public static final UnlinkedGLFunction glInvalidateBufferSubData = make_glInvalidateBufferSubData();
		public static final UnlinkedGLFunction glInvalidateFramebuffer = make_glInvalidateFramebuffer();
		public static final UnlinkedGLFunction glInvalidateNamedFramebufferData = make_glInvalidateNamedFramebufferData();
		public static final UnlinkedGLFunction glInvalidateNamedFramebufferSubData = make_glInvalidateNamedFramebufferSubData();
		public static final UnlinkedGLFunction glInvalidateSubFramebuffer = make_glInvalidateSubFramebuffer();
		public static final UnlinkedGLFunction glInvalidateTexImage = make_glInvalidateTexImage();
		public static final UnlinkedGLFunction glInvalidateTexSubImage = make_glInvalidateTexSubImage();
		public static final UnlinkedGLFunction glIsAsyncMarkerSGIX = make_glIsAsyncMarkerSGIX();
		public static final UnlinkedGLFunction glIsBuffer = make_glIsBuffer();
		public static final UnlinkedGLFunction glIsBufferARB = make_glIsBufferARB();
		public static final UnlinkedGLFunction glIsBufferResidentNV = make_glIsBufferResidentNV();
		public static final UnlinkedGLFunction glIsCommandListNV = make_glIsCommandListNV();
		public static final UnlinkedGLFunction glIsEnabled = make_glIsEnabled();
		public static final UnlinkedGLFunction glIsEnabledIndexedEXT = make_glIsEnabledIndexedEXT();
		public static final UnlinkedGLFunction glIsEnabledi = make_glIsEnabledi();
		public static final UnlinkedGLFunction glIsEnablediEXT = make_glIsEnablediEXT();
		public static final UnlinkedGLFunction glIsEnablediNV = make_glIsEnablediNV();
		public static final UnlinkedGLFunction glIsEnablediOES = make_glIsEnablediOES();
		public static final UnlinkedGLFunction glIsFenceAPPLE = make_glIsFenceAPPLE();
		public static final UnlinkedGLFunction glIsFenceNV = make_glIsFenceNV();
		public static final UnlinkedGLFunction glIsFramebuffer = make_glIsFramebuffer();
		public static final UnlinkedGLFunction glIsFramebufferEXT = make_glIsFramebufferEXT();
		public static final UnlinkedGLFunction glIsFramebufferOES = make_glIsFramebufferOES();
		public static final UnlinkedGLFunction glIsImageHandleResidentARB = make_glIsImageHandleResidentARB();
		public static final UnlinkedGLFunction glIsImageHandleResidentNV = make_glIsImageHandleResidentNV();
		public static final UnlinkedGLFunction glIsList = make_glIsList();
		public static final UnlinkedGLFunction glIsMemoryObjectEXT = make_glIsMemoryObjectEXT();
		public static final UnlinkedGLFunction glIsNameAMD = make_glIsNameAMD();
		public static final UnlinkedGLFunction glIsNamedBufferResidentNV = make_glIsNamedBufferResidentNV();
		public static final UnlinkedGLFunction glIsNamedStringARB = make_glIsNamedStringARB();
		public static final UnlinkedGLFunction glIsObjectBufferATI = make_glIsObjectBufferATI();
		public static final UnlinkedGLFunction glIsOcclusionQueryNV = make_glIsOcclusionQueryNV();
		public static final UnlinkedGLFunction glIsPathNV = make_glIsPathNV();
		public static final UnlinkedGLFunction glIsPointInFillPathNV = make_glIsPointInFillPathNV();
		public static final UnlinkedGLFunction glIsPointInStrokePathNV = make_glIsPointInStrokePathNV();
		public static final UnlinkedGLFunction glIsProgram = make_glIsProgram();
		public static final UnlinkedGLFunction glIsProgramARB = make_glIsProgramARB();
		public static final UnlinkedGLFunction glIsProgramNV = make_glIsProgramNV();
		public static final UnlinkedGLFunction glIsProgramPipeline = make_glIsProgramPipeline();
		public static final UnlinkedGLFunction glIsProgramPipelineEXT = make_glIsProgramPipelineEXT();
		public static final UnlinkedGLFunction glIsQuery = make_glIsQuery();
		public static final UnlinkedGLFunction glIsQueryARB = make_glIsQueryARB();
		public static final UnlinkedGLFunction glIsQueryEXT = make_glIsQueryEXT();
		public static final UnlinkedGLFunction glIsRenderbuffer = make_glIsRenderbuffer();
		public static final UnlinkedGLFunction glIsRenderbufferEXT = make_glIsRenderbufferEXT();
		public static final UnlinkedGLFunction glIsRenderbufferOES = make_glIsRenderbufferOES();
		public static final UnlinkedGLFunction glIsSemaphoreEXT = make_glIsSemaphoreEXT();
		public static final UnlinkedGLFunction glIsSampler = make_glIsSampler();
		public static final UnlinkedGLFunction glIsShader = make_glIsShader();
		public static final UnlinkedGLFunction glIsStateNV = make_glIsStateNV();
		public static final UnlinkedGLFunction glIsSync = make_glIsSync();
		public static final UnlinkedGLFunction glIsSyncAPPLE = make_glIsSyncAPPLE();
		public static final UnlinkedGLFunction glIsTexture = make_glIsTexture();
		public static final UnlinkedGLFunction glIsTextureEXT = make_glIsTextureEXT();
		public static final UnlinkedGLFunction glIsTextureHandleResidentARB = make_glIsTextureHandleResidentARB();
		public static final UnlinkedGLFunction glIsTextureHandleResidentNV = make_glIsTextureHandleResidentNV();
		public static final UnlinkedGLFunction glIsTransformFeedback = make_glIsTransformFeedback();
		public static final UnlinkedGLFunction glIsTransformFeedbackNV = make_glIsTransformFeedbackNV();
		public static final UnlinkedGLFunction glIsVariantEnabledEXT = make_glIsVariantEnabledEXT();
		public static final UnlinkedGLFunction glIsVertexArray = make_glIsVertexArray();
		public static final UnlinkedGLFunction glIsVertexArrayAPPLE = make_glIsVertexArrayAPPLE();
		public static final UnlinkedGLFunction glIsVertexArrayOES = make_glIsVertexArrayOES();
		public static final UnlinkedGLFunction glIsVertexAttribEnabledAPPLE = make_glIsVertexAttribEnabledAPPLE();
		public static final UnlinkedGLFunction glLGPUCopyImageSubDataNVX = make_glLGPUCopyImageSubDataNVX();
		public static final UnlinkedGLFunction glLGPUInterlockNVX = make_glLGPUInterlockNVX();
		public static final UnlinkedGLFunction glLGPUNamedBufferSubDataNVX = make_glLGPUNamedBufferSubDataNVX();
		public static final UnlinkedGLFunction glLabelObjectEXT = make_glLabelObjectEXT();
		public static final UnlinkedGLFunction glLightEnviSGIX = make_glLightEnviSGIX();
		public static final UnlinkedGLFunction glLightModelf = make_glLightModelf();
		public static final UnlinkedGLFunction glLightModelfv = make_glLightModelfv();
		public static final UnlinkedGLFunction glLightModeli = make_glLightModeli();
		public static final UnlinkedGLFunction glLightModeliv = make_glLightModeliv();
		public static final UnlinkedGLFunction glLightModelx = make_glLightModelx();
		public static final UnlinkedGLFunction glLightModelxOES = make_glLightModelxOES();
		public static final UnlinkedGLFunction glLightModelxv = make_glLightModelxv();
		public static final UnlinkedGLFunction glLightModelxvOES = make_glLightModelxvOES();
		public static final UnlinkedGLFunction glLightf = make_glLightf();
		public static final UnlinkedGLFunction glLightfv = make_glLightfv();
		public static final UnlinkedGLFunction glLighti = make_glLighti();
		public static final UnlinkedGLFunction glLightiv = make_glLightiv();
		public static final UnlinkedGLFunction glLightx = make_glLightx();
		public static final UnlinkedGLFunction glLightxOES = make_glLightxOES();
		public static final UnlinkedGLFunction glLightxv = make_glLightxv();
		public static final UnlinkedGLFunction glLightxvOES = make_glLightxvOES();
		public static final UnlinkedGLFunction glLineStipple = make_glLineStipple();
		public static final UnlinkedGLFunction glLineWidth = make_glLineWidth();
		public static final UnlinkedGLFunction glLineWidthx = make_glLineWidthx();
		public static final UnlinkedGLFunction glLineWidthxOES = make_glLineWidthxOES();
		public static final UnlinkedGLFunction glLinkProgram = make_glLinkProgram();
		public static final UnlinkedGLFunction glLinkProgramARB = make_glLinkProgramARB();
		public static final UnlinkedGLFunction glListBase = make_glListBase();
		public static final UnlinkedGLFunction glListDrawCommandsStatesClientNV = make_glListDrawCommandsStatesClientNV();
		public static final UnlinkedGLFunction glListParameterfSGIX = make_glListParameterfSGIX();
		public static final UnlinkedGLFunction glListParameterfvSGIX = make_glListParameterfvSGIX();
		public static final UnlinkedGLFunction glListParameteriSGIX = make_glListParameteriSGIX();
		public static final UnlinkedGLFunction glListParameterivSGIX = make_glListParameterivSGIX();
		public static final UnlinkedGLFunction glLoadIdentity = make_glLoadIdentity();
		public static final UnlinkedGLFunction glLoadIdentityDeformationMapSGIX = make_glLoadIdentityDeformationMapSGIX();
		public static final UnlinkedGLFunction glLoadMatrixd = make_glLoadMatrixd();
		public static final UnlinkedGLFunction glLoadMatrixf = make_glLoadMatrixf();
		public static final UnlinkedGLFunction glLoadMatrixx = make_glLoadMatrixx();
		public static final UnlinkedGLFunction glLoadMatrixxOES = make_glLoadMatrixxOES();
		public static final UnlinkedGLFunction glLoadName = make_glLoadName();
		public static final UnlinkedGLFunction glLoadPaletteFromModelViewMatrixOES = make_glLoadPaletteFromModelViewMatrixOES();
		public static final UnlinkedGLFunction glLoadProgramNV = make_glLoadProgramNV();
		public static final UnlinkedGLFunction glLoadTransposeMatrixd = make_glLoadTransposeMatrixd();
		public static final UnlinkedGLFunction glLoadTransposeMatrixdARB = make_glLoadTransposeMatrixdARB();
		public static final UnlinkedGLFunction glLoadTransposeMatrixf = make_glLoadTransposeMatrixf();
		public static final UnlinkedGLFunction glLoadTransposeMatrixfARB = make_glLoadTransposeMatrixfARB();
		public static final UnlinkedGLFunction glLoadTransposeMatrixxOES = make_glLoadTransposeMatrixxOES();
		public static final UnlinkedGLFunction glLockArraysEXT = make_glLockArraysEXT();
		public static final UnlinkedGLFunction glLogicOp = make_glLogicOp();
		public static final UnlinkedGLFunction glMakeBufferNonResidentNV = make_glMakeBufferNonResidentNV();
		public static final UnlinkedGLFunction glMakeBufferResidentNV = make_glMakeBufferResidentNV();
		public static final UnlinkedGLFunction glMakeImageHandleNonResidentARB = make_glMakeImageHandleNonResidentARB();
		public static final UnlinkedGLFunction glMakeImageHandleNonResidentNV = make_glMakeImageHandleNonResidentNV();
		public static final UnlinkedGLFunction glMakeImageHandleResidentARB = make_glMakeImageHandleResidentARB();
		public static final UnlinkedGLFunction glMakeImageHandleResidentNV = make_glMakeImageHandleResidentNV();
		public static final UnlinkedGLFunction glMakeNamedBufferNonResidentNV = make_glMakeNamedBufferNonResidentNV();
		public static final UnlinkedGLFunction glMakeNamedBufferResidentNV = make_glMakeNamedBufferResidentNV();
		public static final UnlinkedGLFunction glMakeTextureHandleNonResidentARB = make_glMakeTextureHandleNonResidentARB();
		public static final UnlinkedGLFunction glMakeTextureHandleNonResidentNV = make_glMakeTextureHandleNonResidentNV();
		public static final UnlinkedGLFunction glMakeTextureHandleResidentARB = make_glMakeTextureHandleResidentARB();
		public static final UnlinkedGLFunction glMakeTextureHandleResidentNV = make_glMakeTextureHandleResidentNV();
		public static final UnlinkedGLFunction glMap1d = make_glMap1d();
		public static final UnlinkedGLFunction glMap1f = make_glMap1f();
		public static final UnlinkedGLFunction glMap1xOES = make_glMap1xOES();
		public static final UnlinkedGLFunction glMap2d = make_glMap2d();
		public static final UnlinkedGLFunction glMap2f = make_glMap2f();
		public static final UnlinkedGLFunction glMap2xOES = make_glMap2xOES();
		public static final UnlinkedGLFunction glMapBuffer = make_glMapBuffer();
		public static final UnlinkedGLFunction glMapBufferARB = make_glMapBufferARB();
		public static final UnlinkedGLFunction glMapBufferOES = make_glMapBufferOES();
		public static final UnlinkedGLFunction glMapBufferRange = make_glMapBufferRange();
		public static final UnlinkedGLFunction glMapBufferRangeEXT = make_glMapBufferRangeEXT();
		public static final UnlinkedGLFunction glMapControlPointsNV = make_glMapControlPointsNV();
		public static final UnlinkedGLFunction glMapGrid1d = make_glMapGrid1d();
		public static final UnlinkedGLFunction glMapGrid1f = make_glMapGrid1f();
		public static final UnlinkedGLFunction glMapGrid1xOES = make_glMapGrid1xOES();
		public static final UnlinkedGLFunction glMapGrid2d = make_glMapGrid2d();
		public static final UnlinkedGLFunction glMapGrid2f = make_glMapGrid2f();
		public static final UnlinkedGLFunction glMapGrid2xOES = make_glMapGrid2xOES();
		public static final UnlinkedGLFunction glMapNamedBuffer = make_glMapNamedBuffer();
		public static final UnlinkedGLFunction glMapNamedBufferEXT = make_glMapNamedBufferEXT();
		public static final UnlinkedGLFunction glMapNamedBufferRange = make_glMapNamedBufferRange();
		public static final UnlinkedGLFunction glMapNamedBufferRangeEXT = make_glMapNamedBufferRangeEXT();
		public static final UnlinkedGLFunction glMapObjectBufferATI = make_glMapObjectBufferATI();
		public static final UnlinkedGLFunction glMapParameterfvNV = make_glMapParameterfvNV();
		public static final UnlinkedGLFunction glMapParameterivNV = make_glMapParameterivNV();
		public static final UnlinkedGLFunction glMapTexture2DINTEL = make_glMapTexture2DINTEL();
		public static final UnlinkedGLFunction glMapVertexAttrib1dAPPLE = make_glMapVertexAttrib1dAPPLE();
		public static final UnlinkedGLFunction glMapVertexAttrib1fAPPLE = make_glMapVertexAttrib1fAPPLE();
		public static final UnlinkedGLFunction glMapVertexAttrib2dAPPLE = make_glMapVertexAttrib2dAPPLE();
		public static final UnlinkedGLFunction glMapVertexAttrib2fAPPLE = make_glMapVertexAttrib2fAPPLE();
		public static final UnlinkedGLFunction glMaterialf = make_glMaterialf();
		public static final UnlinkedGLFunction glMaterialfv = make_glMaterialfv();
		public static final UnlinkedGLFunction glMateriali = make_glMateriali();
		public static final UnlinkedGLFunction glMaterialiv = make_glMaterialiv();
		public static final UnlinkedGLFunction glMaterialx = make_glMaterialx();
		public static final UnlinkedGLFunction glMaterialxOES = make_glMaterialxOES();
		public static final UnlinkedGLFunction glMaterialxv = make_glMaterialxv();
		public static final UnlinkedGLFunction glMaterialxvOES = make_glMaterialxvOES();
		public static final UnlinkedGLFunction glMatrixFrustumEXT = make_glMatrixFrustumEXT();
		public static final UnlinkedGLFunction glMatrixIndexPointerARB = make_glMatrixIndexPointerARB();
		public static final UnlinkedGLFunction glMatrixIndexPointerOES = make_glMatrixIndexPointerOES();
		public static final UnlinkedGLFunction glMatrixIndexubvARB = make_glMatrixIndexubvARB();
		public static final UnlinkedGLFunction glMatrixIndexuivARB = make_glMatrixIndexuivARB();
		public static final UnlinkedGLFunction glMatrixIndexusvARB = make_glMatrixIndexusvARB();
		public static final UnlinkedGLFunction glMatrixLoad3x2fNV = make_glMatrixLoad3x2fNV();
		public static final UnlinkedGLFunction glMatrixLoad3x3fNV = make_glMatrixLoad3x3fNV();
		public static final UnlinkedGLFunction glMatrixLoadIdentityEXT = make_glMatrixLoadIdentityEXT();
		public static final UnlinkedGLFunction glMatrixLoadTranspose3x3fNV = make_glMatrixLoadTranspose3x3fNV();
		public static final UnlinkedGLFunction glMatrixLoadTransposedEXT = make_glMatrixLoadTransposedEXT();
		public static final UnlinkedGLFunction glMatrixLoadTransposefEXT = make_glMatrixLoadTransposefEXT();
		public static final UnlinkedGLFunction glMatrixLoaddEXT = make_glMatrixLoaddEXT();
		public static final UnlinkedGLFunction glMatrixLoadfEXT = make_glMatrixLoadfEXT();
		public static final UnlinkedGLFunction glMatrixMode = make_glMatrixMode();
		public static final UnlinkedGLFunction glMatrixMult3x2fNV = make_glMatrixMult3x2fNV();
		public static final UnlinkedGLFunction glMatrixMult3x3fNV = make_glMatrixMult3x3fNV();
		public static final UnlinkedGLFunction glMatrixMultTranspose3x3fNV = make_glMatrixMultTranspose3x3fNV();
		public static final UnlinkedGLFunction glMatrixMultTransposedEXT = make_glMatrixMultTransposedEXT();
		public static final UnlinkedGLFunction glMatrixMultTransposefEXT = make_glMatrixMultTransposefEXT();
		public static final UnlinkedGLFunction glMatrixMultdEXT = make_glMatrixMultdEXT();
		public static final UnlinkedGLFunction glMatrixMultfEXT = make_glMatrixMultfEXT();
		public static final UnlinkedGLFunction glMatrixOrthoEXT = make_glMatrixOrthoEXT();
		public static final UnlinkedGLFunction glMatrixPopEXT = make_glMatrixPopEXT();
		public static final UnlinkedGLFunction glMatrixPushEXT = make_glMatrixPushEXT();
		public static final UnlinkedGLFunction glMatrixRotatedEXT = make_glMatrixRotatedEXT();
		public static final UnlinkedGLFunction glMatrixRotatefEXT = make_glMatrixRotatefEXT();
		public static final UnlinkedGLFunction glMatrixScaledEXT = make_glMatrixScaledEXT();
		public static final UnlinkedGLFunction glMatrixScalefEXT = make_glMatrixScalefEXT();
		public static final UnlinkedGLFunction glMatrixTranslatedEXT = make_glMatrixTranslatedEXT();
		public static final UnlinkedGLFunction glMatrixTranslatefEXT = make_glMatrixTranslatefEXT();
		public static final UnlinkedGLFunction glMaxActiveShaderCoresARM = make_glMaxActiveShaderCoresARM();
		public static final UnlinkedGLFunction glMaxShaderCompilerThreadsKHR = make_glMaxShaderCompilerThreadsKHR();
		public static final UnlinkedGLFunction glMaxShaderCompilerThreadsARB = make_glMaxShaderCompilerThreadsARB();
		public static final UnlinkedGLFunction glMemoryBarrier = make_glMemoryBarrier();
		public static final UnlinkedGLFunction glMemoryBarrierByRegion = make_glMemoryBarrierByRegion();
		public static final UnlinkedGLFunction glMemoryBarrierEXT = make_glMemoryBarrierEXT();
		public static final UnlinkedGLFunction glMemoryObjectParameterivEXT = make_glMemoryObjectParameterivEXT();
		public static final UnlinkedGLFunction glMinSampleShading = make_glMinSampleShading();
		public static final UnlinkedGLFunction glMinSampleShadingARB = make_glMinSampleShadingARB();
		public static final UnlinkedGLFunction glMinSampleShadingOES = make_glMinSampleShadingOES();
		public static final UnlinkedGLFunction glMinmax = make_glMinmax();
		public static final UnlinkedGLFunction glMinmaxEXT = make_glMinmaxEXT();
		public static final UnlinkedGLFunction glMultMatrixd = make_glMultMatrixd();
		public static final UnlinkedGLFunction glMultMatrixf = make_glMultMatrixf();
		public static final UnlinkedGLFunction glMultMatrixx = make_glMultMatrixx();
		public static final UnlinkedGLFunction glMultMatrixxOES = make_glMultMatrixxOES();
		public static final UnlinkedGLFunction glMultTransposeMatrixd = make_glMultTransposeMatrixd();
		public static final UnlinkedGLFunction glMultTransposeMatrixdARB = make_glMultTransposeMatrixdARB();
		public static final UnlinkedGLFunction glMultTransposeMatrixf = make_glMultTransposeMatrixf();
		public static final UnlinkedGLFunction glMultTransposeMatrixfARB = make_glMultTransposeMatrixfARB();
		public static final UnlinkedGLFunction glMultTransposeMatrixxOES = make_glMultTransposeMatrixxOES();
		public static final UnlinkedGLFunction glMultiDrawArrays = make_glMultiDrawArrays();
		public static final UnlinkedGLFunction glMultiDrawArraysEXT = make_glMultiDrawArraysEXT();
		public static final UnlinkedGLFunction glMultiDrawArraysIndirect = make_glMultiDrawArraysIndirect();
		public static final UnlinkedGLFunction glMultiDrawArraysIndirectAMD = make_glMultiDrawArraysIndirectAMD();
		public static final UnlinkedGLFunction glMultiDrawArraysIndirectBindlessCountNV = make_glMultiDrawArraysIndirectBindlessCountNV();
		public static final UnlinkedGLFunction glMultiDrawArraysIndirectBindlessNV = make_glMultiDrawArraysIndirectBindlessNV();
		public static final UnlinkedGLFunction glMultiDrawArraysIndirectCount = make_glMultiDrawArraysIndirectCount();
		public static final UnlinkedGLFunction glMultiDrawArraysIndirectCountARB = make_glMultiDrawArraysIndirectCountARB();
		public static final UnlinkedGLFunction glMultiDrawArraysIndirectEXT = make_glMultiDrawArraysIndirectEXT();
		public static final UnlinkedGLFunction glMultiDrawElementArrayAPPLE = make_glMultiDrawElementArrayAPPLE();
		public static final UnlinkedGLFunction glMultiDrawElements = make_glMultiDrawElements();
		public static final UnlinkedGLFunction glMultiDrawElementsBaseVertex = make_glMultiDrawElementsBaseVertex();
		public static final UnlinkedGLFunction glMultiDrawElementsBaseVertexEXT = make_glMultiDrawElementsBaseVertexEXT();
		public static final UnlinkedGLFunction glMultiDrawElementsEXT = make_glMultiDrawElementsEXT();
		public static final UnlinkedGLFunction glMultiDrawElementsIndirect = make_glMultiDrawElementsIndirect();
		public static final UnlinkedGLFunction glMultiDrawElementsIndirectAMD = make_glMultiDrawElementsIndirectAMD();
		public static final UnlinkedGLFunction glMultiDrawElementsIndirectBindlessCountNV = make_glMultiDrawElementsIndirectBindlessCountNV();
		public static final UnlinkedGLFunction glMultiDrawElementsIndirectBindlessNV = make_glMultiDrawElementsIndirectBindlessNV();
		public static final UnlinkedGLFunction glMultiDrawElementsIndirectCount = make_glMultiDrawElementsIndirectCount();
		public static final UnlinkedGLFunction glMultiDrawElementsIndirectCountARB = make_glMultiDrawElementsIndirectCountARB();
		public static final UnlinkedGLFunction glMultiDrawElementsIndirectEXT = make_glMultiDrawElementsIndirectEXT();
		public static final UnlinkedGLFunction glMultiDrawMeshTasksIndirectNV = make_glMultiDrawMeshTasksIndirectNV();
		public static final UnlinkedGLFunction glMultiDrawMeshTasksIndirectCountNV = make_glMultiDrawMeshTasksIndirectCountNV();
		public static final UnlinkedGLFunction glMultiDrawRangeElementArrayAPPLE = make_glMultiDrawRangeElementArrayAPPLE();
		public static final UnlinkedGLFunction glMultiModeDrawArraysIBM = make_glMultiModeDrawArraysIBM();
		public static final UnlinkedGLFunction glMultiModeDrawElementsIBM = make_glMultiModeDrawElementsIBM();
		public static final UnlinkedGLFunction glMultiTexBufferEXT = make_glMultiTexBufferEXT();
		public static final UnlinkedGLFunction glMultiTexCoord1bOES = make_glMultiTexCoord1bOES();
		public static final UnlinkedGLFunction glMultiTexCoord1bvOES = make_glMultiTexCoord1bvOES();
		public static final UnlinkedGLFunction glMultiTexCoord1d = make_glMultiTexCoord1d();
		public static final UnlinkedGLFunction glMultiTexCoord1dARB = make_glMultiTexCoord1dARB();
		public static final UnlinkedGLFunction glMultiTexCoord1dv = make_glMultiTexCoord1dv();
		public static final UnlinkedGLFunction glMultiTexCoord1dvARB = make_glMultiTexCoord1dvARB();
		public static final UnlinkedGLFunction glMultiTexCoord1f = make_glMultiTexCoord1f();
		public static final UnlinkedGLFunction glMultiTexCoord1fARB = make_glMultiTexCoord1fARB();
		public static final UnlinkedGLFunction glMultiTexCoord1fv = make_glMultiTexCoord1fv();
		public static final UnlinkedGLFunction glMultiTexCoord1fvARB = make_glMultiTexCoord1fvARB();
		public static final UnlinkedGLFunction glMultiTexCoord1hNV = make_glMultiTexCoord1hNV();
		public static final UnlinkedGLFunction glMultiTexCoord1hvNV = make_glMultiTexCoord1hvNV();
		public static final UnlinkedGLFunction glMultiTexCoord1i = make_glMultiTexCoord1i();
		public static final UnlinkedGLFunction glMultiTexCoord1iARB = make_glMultiTexCoord1iARB();
		public static final UnlinkedGLFunction glMultiTexCoord1iv = make_glMultiTexCoord1iv();
		public static final UnlinkedGLFunction glMultiTexCoord1ivARB = make_glMultiTexCoord1ivARB();
		public static final UnlinkedGLFunction glMultiTexCoord1s = make_glMultiTexCoord1s();
		public static final UnlinkedGLFunction glMultiTexCoord1sARB = make_glMultiTexCoord1sARB();
		public static final UnlinkedGLFunction glMultiTexCoord1sv = make_glMultiTexCoord1sv();
		public static final UnlinkedGLFunction glMultiTexCoord1svARB = make_glMultiTexCoord1svARB();
		public static final UnlinkedGLFunction glMultiTexCoord1xOES = make_glMultiTexCoord1xOES();
		public static final UnlinkedGLFunction glMultiTexCoord1xvOES = make_glMultiTexCoord1xvOES();
		public static final UnlinkedGLFunction glMultiTexCoord2bOES = make_glMultiTexCoord2bOES();
		public static final UnlinkedGLFunction glMultiTexCoord2bvOES = make_glMultiTexCoord2bvOES();
		public static final UnlinkedGLFunction glMultiTexCoord2d = make_glMultiTexCoord2d();
		public static final UnlinkedGLFunction glMultiTexCoord2dARB = make_glMultiTexCoord2dARB();
		public static final UnlinkedGLFunction glMultiTexCoord2dv = make_glMultiTexCoord2dv();
		public static final UnlinkedGLFunction glMultiTexCoord2dvARB = make_glMultiTexCoord2dvARB();
		public static final UnlinkedGLFunction glMultiTexCoord2f = make_glMultiTexCoord2f();
		public static final UnlinkedGLFunction glMultiTexCoord2fARB = make_glMultiTexCoord2fARB();
		public static final UnlinkedGLFunction glMultiTexCoord2fv = make_glMultiTexCoord2fv();
		public static final UnlinkedGLFunction glMultiTexCoord2fvARB = make_glMultiTexCoord2fvARB();
		public static final UnlinkedGLFunction glMultiTexCoord2hNV = make_glMultiTexCoord2hNV();
		public static final UnlinkedGLFunction glMultiTexCoord2hvNV = make_glMultiTexCoord2hvNV();
		public static final UnlinkedGLFunction glMultiTexCoord2i = make_glMultiTexCoord2i();
		public static final UnlinkedGLFunction glMultiTexCoord2iARB = make_glMultiTexCoord2iARB();
		public static final UnlinkedGLFunction glMultiTexCoord2iv = make_glMultiTexCoord2iv();
		public static final UnlinkedGLFunction glMultiTexCoord2ivARB = make_glMultiTexCoord2ivARB();
		public static final UnlinkedGLFunction glMultiTexCoord2s = make_glMultiTexCoord2s();
		public static final UnlinkedGLFunction glMultiTexCoord2sARB = make_glMultiTexCoord2sARB();
		public static final UnlinkedGLFunction glMultiTexCoord2sv = make_glMultiTexCoord2sv();
		public static final UnlinkedGLFunction glMultiTexCoord2svARB = make_glMultiTexCoord2svARB();
		public static final UnlinkedGLFunction glMultiTexCoord2xOES = make_glMultiTexCoord2xOES();
		public static final UnlinkedGLFunction glMultiTexCoord2xvOES = make_glMultiTexCoord2xvOES();
		public static final UnlinkedGLFunction glMultiTexCoord3bOES = make_glMultiTexCoord3bOES();
		public static final UnlinkedGLFunction glMultiTexCoord3bvOES = make_glMultiTexCoord3bvOES();
		public static final UnlinkedGLFunction glMultiTexCoord3d = make_glMultiTexCoord3d();
		public static final UnlinkedGLFunction glMultiTexCoord3dARB = make_glMultiTexCoord3dARB();
		public static final UnlinkedGLFunction glMultiTexCoord3dv = make_glMultiTexCoord3dv();
		public static final UnlinkedGLFunction glMultiTexCoord3dvARB = make_glMultiTexCoord3dvARB();
		public static final UnlinkedGLFunction glMultiTexCoord3f = make_glMultiTexCoord3f();
		public static final UnlinkedGLFunction glMultiTexCoord3fARB = make_glMultiTexCoord3fARB();
		public static final UnlinkedGLFunction glMultiTexCoord3fv = make_glMultiTexCoord3fv();
		public static final UnlinkedGLFunction glMultiTexCoord3fvARB = make_glMultiTexCoord3fvARB();
		public static final UnlinkedGLFunction glMultiTexCoord3hNV = make_glMultiTexCoord3hNV();
		public static final UnlinkedGLFunction glMultiTexCoord3hvNV = make_glMultiTexCoord3hvNV();
		public static final UnlinkedGLFunction glMultiTexCoord3i = make_glMultiTexCoord3i();
		public static final UnlinkedGLFunction glMultiTexCoord3iARB = make_glMultiTexCoord3iARB();
		public static final UnlinkedGLFunction glMultiTexCoord3iv = make_glMultiTexCoord3iv();
		public static final UnlinkedGLFunction glMultiTexCoord3ivARB = make_glMultiTexCoord3ivARB();
		public static final UnlinkedGLFunction glMultiTexCoord3s = make_glMultiTexCoord3s();
		public static final UnlinkedGLFunction glMultiTexCoord3sARB = make_glMultiTexCoord3sARB();
		public static final UnlinkedGLFunction glMultiTexCoord3sv = make_glMultiTexCoord3sv();
		public static final UnlinkedGLFunction glMultiTexCoord3svARB = make_glMultiTexCoord3svARB();
		public static final UnlinkedGLFunction glMultiTexCoord3xOES = make_glMultiTexCoord3xOES();
		public static final UnlinkedGLFunction glMultiTexCoord3xvOES = make_glMultiTexCoord3xvOES();
		public static final UnlinkedGLFunction glMultiTexCoord4bOES = make_glMultiTexCoord4bOES();
		public static final UnlinkedGLFunction glMultiTexCoord4bvOES = make_glMultiTexCoord4bvOES();
		public static final UnlinkedGLFunction glMultiTexCoord4d = make_glMultiTexCoord4d();
		public static final UnlinkedGLFunction glMultiTexCoord4dARB = make_glMultiTexCoord4dARB();
		public static final UnlinkedGLFunction glMultiTexCoord4dv = make_glMultiTexCoord4dv();
		public static final UnlinkedGLFunction glMultiTexCoord4dvARB = make_glMultiTexCoord4dvARB();
		public static final UnlinkedGLFunction glMultiTexCoord4f = make_glMultiTexCoord4f();
		public static final UnlinkedGLFunction glMultiTexCoord4fARB = make_glMultiTexCoord4fARB();
		public static final UnlinkedGLFunction glMultiTexCoord4fv = make_glMultiTexCoord4fv();
		public static final UnlinkedGLFunction glMultiTexCoord4fvARB = make_glMultiTexCoord4fvARB();
		public static final UnlinkedGLFunction glMultiTexCoord4hNV = make_glMultiTexCoord4hNV();
		public static final UnlinkedGLFunction glMultiTexCoord4hvNV = make_glMultiTexCoord4hvNV();
		public static final UnlinkedGLFunction glMultiTexCoord4i = make_glMultiTexCoord4i();
		public static final UnlinkedGLFunction glMultiTexCoord4iARB = make_glMultiTexCoord4iARB();
		public static final UnlinkedGLFunction glMultiTexCoord4iv = make_glMultiTexCoord4iv();
		public static final UnlinkedGLFunction glMultiTexCoord4ivARB = make_glMultiTexCoord4ivARB();
		public static final UnlinkedGLFunction glMultiTexCoord4s = make_glMultiTexCoord4s();
		public static final UnlinkedGLFunction glMultiTexCoord4sARB = make_glMultiTexCoord4sARB();
		public static final UnlinkedGLFunction glMultiTexCoord4sv = make_glMultiTexCoord4sv();
		public static final UnlinkedGLFunction glMultiTexCoord4svARB = make_glMultiTexCoord4svARB();
		public static final UnlinkedGLFunction glMultiTexCoord4x = make_glMultiTexCoord4x();
		public static final UnlinkedGLFunction glMultiTexCoord4xOES = make_glMultiTexCoord4xOES();
		public static final UnlinkedGLFunction glMultiTexCoord4xvOES = make_glMultiTexCoord4xvOES();
		public static final UnlinkedGLFunction glMultiTexCoordP1ui = make_glMultiTexCoordP1ui();
		public static final UnlinkedGLFunction glMultiTexCoordP1uiv = make_glMultiTexCoordP1uiv();
		public static final UnlinkedGLFunction glMultiTexCoordP2ui = make_glMultiTexCoordP2ui();
		public static final UnlinkedGLFunction glMultiTexCoordP2uiv = make_glMultiTexCoordP2uiv();
		public static final UnlinkedGLFunction glMultiTexCoordP3ui = make_glMultiTexCoordP3ui();
		public static final UnlinkedGLFunction glMultiTexCoordP3uiv = make_glMultiTexCoordP3uiv();
		public static final UnlinkedGLFunction glMultiTexCoordP4ui = make_glMultiTexCoordP4ui();
		public static final UnlinkedGLFunction glMultiTexCoordP4uiv = make_glMultiTexCoordP4uiv();
		public static final UnlinkedGLFunction glMultiTexCoordPointerEXT = make_glMultiTexCoordPointerEXT();
		public static final UnlinkedGLFunction glMultiTexEnvfEXT = make_glMultiTexEnvfEXT();
		public static final UnlinkedGLFunction glMultiTexEnvfvEXT = make_glMultiTexEnvfvEXT();
		public static final UnlinkedGLFunction glMultiTexEnviEXT = make_glMultiTexEnviEXT();
		public static final UnlinkedGLFunction glMultiTexEnvivEXT = make_glMultiTexEnvivEXT();
		public static final UnlinkedGLFunction glMultiTexGendEXT = make_glMultiTexGendEXT();
		public static final UnlinkedGLFunction glMultiTexGendvEXT = make_glMultiTexGendvEXT();
		public static final UnlinkedGLFunction glMultiTexGenfEXT = make_glMultiTexGenfEXT();
		public static final UnlinkedGLFunction glMultiTexGenfvEXT = make_glMultiTexGenfvEXT();
		public static final UnlinkedGLFunction glMultiTexGeniEXT = make_glMultiTexGeniEXT();
		public static final UnlinkedGLFunction glMultiTexGenivEXT = make_glMultiTexGenivEXT();
		public static final UnlinkedGLFunction glMultiTexImage1DEXT = make_glMultiTexImage1DEXT();
		public static final UnlinkedGLFunction glMultiTexImage2DEXT = make_glMultiTexImage2DEXT();
		public static final UnlinkedGLFunction glMultiTexImage3DEXT = make_glMultiTexImage3DEXT();
		public static final UnlinkedGLFunction glMultiTexParameterIivEXT = make_glMultiTexParameterIivEXT();
		public static final UnlinkedGLFunction glMultiTexParameterIuivEXT = make_glMultiTexParameterIuivEXT();
		public static final UnlinkedGLFunction glMultiTexParameterfEXT = make_glMultiTexParameterfEXT();
		public static final UnlinkedGLFunction glMultiTexParameterfvEXT = make_glMultiTexParameterfvEXT();
		public static final UnlinkedGLFunction glMultiTexParameteriEXT = make_glMultiTexParameteriEXT();
		public static final UnlinkedGLFunction glMultiTexParameterivEXT = make_glMultiTexParameterivEXT();
		public static final UnlinkedGLFunction glMultiTexRenderbufferEXT = make_glMultiTexRenderbufferEXT();
		public static final UnlinkedGLFunction glMultiTexSubImage1DEXT = make_glMultiTexSubImage1DEXT();
		public static final UnlinkedGLFunction glMultiTexSubImage2DEXT = make_glMultiTexSubImage2DEXT();
		public static final UnlinkedGLFunction glMultiTexSubImage3DEXT = make_glMultiTexSubImage3DEXT();
		public static final UnlinkedGLFunction glMulticastBarrierNV = make_glMulticastBarrierNV();
		public static final UnlinkedGLFunction glMulticastBlitFramebufferNV = make_glMulticastBlitFramebufferNV();
		public static final UnlinkedGLFunction glMulticastBufferSubDataNV = make_glMulticastBufferSubDataNV();
		public static final UnlinkedGLFunction glMulticastCopyBufferSubDataNV = make_glMulticastCopyBufferSubDataNV();
		public static final UnlinkedGLFunction glMulticastCopyImageSubDataNV = make_glMulticastCopyImageSubDataNV();
		public static final UnlinkedGLFunction glMulticastFramebufferSampleLocationsfvNV = make_glMulticastFramebufferSampleLocationsfvNV();
		public static final UnlinkedGLFunction glMulticastGetQueryObjecti64vNV = make_glMulticastGetQueryObjecti64vNV();
		public static final UnlinkedGLFunction glMulticastGetQueryObjectivNV = make_glMulticastGetQueryObjectivNV();
		public static final UnlinkedGLFunction glMulticastGetQueryObjectui64vNV = make_glMulticastGetQueryObjectui64vNV();
		public static final UnlinkedGLFunction glMulticastGetQueryObjectuivNV = make_glMulticastGetQueryObjectuivNV();
		public static final UnlinkedGLFunction glMulticastScissorArrayvNVX = make_glMulticastScissorArrayvNVX();
		public static final UnlinkedGLFunction glMulticastViewportArrayvNVX = make_glMulticastViewportArrayvNVX();
		public static final UnlinkedGLFunction glMulticastViewportPositionWScaleNVX = make_glMulticastViewportPositionWScaleNVX();
		public static final UnlinkedGLFunction glMulticastWaitSyncNV = make_glMulticastWaitSyncNV();
		public static final UnlinkedGLFunction glNamedBufferAttachMemoryNV = make_glNamedBufferAttachMemoryNV();
		public static final UnlinkedGLFunction glNamedBufferData = make_glNamedBufferData();
		public static final UnlinkedGLFunction glNamedBufferDataEXT = make_glNamedBufferDataEXT();
		public static final UnlinkedGLFunction glNamedBufferPageCommitmentARB = make_glNamedBufferPageCommitmentARB();
		public static final UnlinkedGLFunction glNamedBufferPageCommitmentEXT = make_glNamedBufferPageCommitmentEXT();
		public static final UnlinkedGLFunction glNamedBufferPageCommitmentMemNV = make_glNamedBufferPageCommitmentMemNV();
		public static final UnlinkedGLFunction glNamedBufferStorage = make_glNamedBufferStorage();
		public static final UnlinkedGLFunction glNamedBufferStorageExternalEXT = make_glNamedBufferStorageExternalEXT();
		public static final UnlinkedGLFunction glNamedBufferStorageEXT = make_glNamedBufferStorageEXT();
		public static final UnlinkedGLFunction glNamedBufferStorageMemEXT = make_glNamedBufferStorageMemEXT();
		public static final UnlinkedGLFunction glNamedBufferSubData = make_glNamedBufferSubData();
		public static final UnlinkedGLFunction glNamedBufferSubDataEXT = make_glNamedBufferSubDataEXT();
		public static final UnlinkedGLFunction glNamedCopyBufferSubDataEXT = make_glNamedCopyBufferSubDataEXT();
		public static final UnlinkedGLFunction glNamedFramebufferDrawBuffer = make_glNamedFramebufferDrawBuffer();
		public static final UnlinkedGLFunction glNamedFramebufferDrawBuffers = make_glNamedFramebufferDrawBuffers();
		public static final UnlinkedGLFunction glNamedFramebufferParameteri = make_glNamedFramebufferParameteri();
		public static final UnlinkedGLFunction glNamedFramebufferParameteriEXT = make_glNamedFramebufferParameteriEXT();
		public static final UnlinkedGLFunction glNamedFramebufferReadBuffer = make_glNamedFramebufferReadBuffer();
		public static final UnlinkedGLFunction glNamedFramebufferRenderbuffer = make_glNamedFramebufferRenderbuffer();
		public static final UnlinkedGLFunction glNamedFramebufferRenderbufferEXT = make_glNamedFramebufferRenderbufferEXT();
		public static final UnlinkedGLFunction glNamedFramebufferSampleLocationsfvARB = make_glNamedFramebufferSampleLocationsfvARB();
		public static final UnlinkedGLFunction glNamedFramebufferSampleLocationsfvNV = make_glNamedFramebufferSampleLocationsfvNV();
		public static final UnlinkedGLFunction glNamedFramebufferTexture = make_glNamedFramebufferTexture();
		public static final UnlinkedGLFunction glNamedFramebufferSamplePositionsfvAMD = make_glNamedFramebufferSamplePositionsfvAMD();
		public static final UnlinkedGLFunction glNamedFramebufferTexture1DEXT = make_glNamedFramebufferTexture1DEXT();
		public static final UnlinkedGLFunction glNamedFramebufferTexture2DEXT = make_glNamedFramebufferTexture2DEXT();
		public static final UnlinkedGLFunction glNamedFramebufferTexture3DEXT = make_glNamedFramebufferTexture3DEXT();
		public static final UnlinkedGLFunction glNamedFramebufferTextureEXT = make_glNamedFramebufferTextureEXT();
		public static final UnlinkedGLFunction glNamedFramebufferTextureFaceEXT = make_glNamedFramebufferTextureFaceEXT();
		public static final UnlinkedGLFunction glNamedFramebufferTextureLayer = make_glNamedFramebufferTextureLayer();
		public static final UnlinkedGLFunction glNamedFramebufferTextureLayerEXT = make_glNamedFramebufferTextureLayerEXT();
		public static final UnlinkedGLFunction glNamedFramebufferTextureMultiviewOVR = make_glNamedFramebufferTextureMultiviewOVR();
		public static final UnlinkedGLFunction glNamedProgramLocalParameter4dEXT = make_glNamedProgramLocalParameter4dEXT();
		public static final UnlinkedGLFunction glNamedProgramLocalParameter4dvEXT = make_glNamedProgramLocalParameter4dvEXT();
		public static final UnlinkedGLFunction glNamedProgramLocalParameter4fEXT = make_glNamedProgramLocalParameter4fEXT();
		public static final UnlinkedGLFunction glNamedProgramLocalParameter4fvEXT = make_glNamedProgramLocalParameter4fvEXT();
		public static final UnlinkedGLFunction glNamedProgramLocalParameterI4iEXT = make_glNamedProgramLocalParameterI4iEXT();
		public static final UnlinkedGLFunction glNamedProgramLocalParameterI4ivEXT = make_glNamedProgramLocalParameterI4ivEXT();
		public static final UnlinkedGLFunction glNamedProgramLocalParameterI4uiEXT = make_glNamedProgramLocalParameterI4uiEXT();
		public static final UnlinkedGLFunction glNamedProgramLocalParameterI4uivEXT = make_glNamedProgramLocalParameterI4uivEXT();
		public static final UnlinkedGLFunction glNamedProgramLocalParameters4fvEXT = make_glNamedProgramLocalParameters4fvEXT();
		public static final UnlinkedGLFunction glNamedProgramLocalParametersI4ivEXT = make_glNamedProgramLocalParametersI4ivEXT();
		public static final UnlinkedGLFunction glNamedProgramLocalParametersI4uivEXT = make_glNamedProgramLocalParametersI4uivEXT();
		public static final UnlinkedGLFunction glNamedProgramStringEXT = make_glNamedProgramStringEXT();
		public static final UnlinkedGLFunction glNamedRenderbufferStorage = make_glNamedRenderbufferStorage();
		public static final UnlinkedGLFunction glNamedRenderbufferStorageEXT = make_glNamedRenderbufferStorageEXT();
		public static final UnlinkedGLFunction glNamedRenderbufferStorageMultisample = make_glNamedRenderbufferStorageMultisample();
		public static final UnlinkedGLFunction glNamedRenderbufferStorageMultisampleAdvancedAMD = make_glNamedRenderbufferStorageMultisampleAdvancedAMD();
		public static final UnlinkedGLFunction glNamedRenderbufferStorageMultisampleCoverageEXT = make_glNamedRenderbufferStorageMultisampleCoverageEXT();
		public static final UnlinkedGLFunction glNamedRenderbufferStorageMultisampleEXT = make_glNamedRenderbufferStorageMultisampleEXT();
		public static final UnlinkedGLFunction glNamedStringARB = make_glNamedStringARB();
		public static final UnlinkedGLFunction glNewList = make_glNewList();
		public static final UnlinkedGLFunction glNewObjectBufferATI = make_glNewObjectBufferATI();
		public static final UnlinkedGLFunction glNormal3b = make_glNormal3b();
		public static final UnlinkedGLFunction glNormal3bv = make_glNormal3bv();
		public static final UnlinkedGLFunction glNormal3d = make_glNormal3d();
		public static final UnlinkedGLFunction glNormal3dv = make_glNormal3dv();
		public static final UnlinkedGLFunction glNormal3f = make_glNormal3f();
		public static final UnlinkedGLFunction glNormal3fVertex3fSUN = make_glNormal3fVertex3fSUN();
		public static final UnlinkedGLFunction glNormal3fVertex3fvSUN = make_glNormal3fVertex3fvSUN();
		public static final UnlinkedGLFunction glNormal3fv = make_glNormal3fv();
		public static final UnlinkedGLFunction glNormal3hNV = make_glNormal3hNV();
		public static final UnlinkedGLFunction glNormal3hvNV = make_glNormal3hvNV();
		public static final UnlinkedGLFunction glNormal3i = make_glNormal3i();
		public static final UnlinkedGLFunction glNormal3iv = make_glNormal3iv();
		public static final UnlinkedGLFunction glNormal3s = make_glNormal3s();
		public static final UnlinkedGLFunction glNormal3sv = make_glNormal3sv();
		public static final UnlinkedGLFunction glNormal3x = make_glNormal3x();
		public static final UnlinkedGLFunction glNormal3xOES = make_glNormal3xOES();
		public static final UnlinkedGLFunction glNormal3xvOES = make_glNormal3xvOES();
		public static final UnlinkedGLFunction glNormalFormatNV = make_glNormalFormatNV();
		public static final UnlinkedGLFunction glNormalP3ui = make_glNormalP3ui();
		public static final UnlinkedGLFunction glNormalP3uiv = make_glNormalP3uiv();
		public static final UnlinkedGLFunction glNormalPointer = make_glNormalPointer();
		public static final UnlinkedGLFunction glNormalPointerEXT = make_glNormalPointerEXT();
		public static final UnlinkedGLFunction glNormalPointerListIBM = make_glNormalPointerListIBM();
		public static final UnlinkedGLFunction glNormalPointervINTEL = make_glNormalPointervINTEL();
		public static final UnlinkedGLFunction glNormalStream3bATI = make_glNormalStream3bATI();
		public static final UnlinkedGLFunction glNormalStream3bvATI = make_glNormalStream3bvATI();
		public static final UnlinkedGLFunction glNormalStream3dATI = make_glNormalStream3dATI();
		public static final UnlinkedGLFunction glNormalStream3dvATI = make_glNormalStream3dvATI();
		public static final UnlinkedGLFunction glNormalStream3fATI = make_glNormalStream3fATI();
		public static final UnlinkedGLFunction glNormalStream3fvATI = make_glNormalStream3fvATI();
		public static final UnlinkedGLFunction glNormalStream3iATI = make_glNormalStream3iATI();
		public static final UnlinkedGLFunction glNormalStream3ivATI = make_glNormalStream3ivATI();
		public static final UnlinkedGLFunction glNormalStream3sATI = make_glNormalStream3sATI();
		public static final UnlinkedGLFunction glNormalStream3svATI = make_glNormalStream3svATI();
		public static final UnlinkedGLFunction glObjectLabel = make_glObjectLabel();
		public static final UnlinkedGLFunction glObjectLabelKHR = make_glObjectLabelKHR();
		public static final UnlinkedGLFunction glObjectPtrLabel = make_glObjectPtrLabel();
		public static final UnlinkedGLFunction glObjectPtrLabelKHR = make_glObjectPtrLabelKHR();
		public static final UnlinkedGLFunction glObjectPurgeableAPPLE = make_glObjectPurgeableAPPLE();
		public static final UnlinkedGLFunction glObjectUnpurgeableAPPLE = make_glObjectUnpurgeableAPPLE();
		public static final UnlinkedGLFunction glOrtho = make_glOrtho();
		public static final UnlinkedGLFunction glOrthof = make_glOrthof();
		public static final UnlinkedGLFunction glOrthofOES = make_glOrthofOES();
		public static final UnlinkedGLFunction glOrthox = make_glOrthox();
		public static final UnlinkedGLFunction glOrthoxOES = make_glOrthoxOES();
		public static final UnlinkedGLFunction glPNTrianglesfATI = make_glPNTrianglesfATI();
		public static final UnlinkedGLFunction glPNTrianglesiATI = make_glPNTrianglesiATI();
		public static final UnlinkedGLFunction glPassTexCoordATI = make_glPassTexCoordATI();
		public static final UnlinkedGLFunction glPassThrough = make_glPassThrough();
		public static final UnlinkedGLFunction glPassThroughxOES = make_glPassThroughxOES();
		public static final UnlinkedGLFunction glPatchParameterfv = make_glPatchParameterfv();
		public static final UnlinkedGLFunction glPatchParameteri = make_glPatchParameteri();
		public static final UnlinkedGLFunction glPatchParameteriEXT = make_glPatchParameteriEXT();
		public static final UnlinkedGLFunction glPatchParameteriOES = make_glPatchParameteriOES();
		public static final UnlinkedGLFunction glPathColorGenNV = make_glPathColorGenNV();
		public static final UnlinkedGLFunction glPathCommandsNV = make_glPathCommandsNV();
		public static final UnlinkedGLFunction glPathCoordsNV = make_glPathCoordsNV();
		public static final UnlinkedGLFunction glPathCoverDepthFuncNV = make_glPathCoverDepthFuncNV();
		public static final UnlinkedGLFunction glPathDashArrayNV = make_glPathDashArrayNV();
		public static final UnlinkedGLFunction glPathFogGenNV = make_glPathFogGenNV();
		public static final UnlinkedGLFunction glPathGlyphIndexArrayNV = make_glPathGlyphIndexArrayNV();
		public static final UnlinkedGLFunction glPathGlyphIndexRangeNV = make_glPathGlyphIndexRangeNV();
		public static final UnlinkedGLFunction glPathGlyphRangeNV = make_glPathGlyphRangeNV();
		public static final UnlinkedGLFunction glPathGlyphsNV = make_glPathGlyphsNV();
		public static final UnlinkedGLFunction glPathMemoryGlyphIndexArrayNV = make_glPathMemoryGlyphIndexArrayNV();
		public static final UnlinkedGLFunction glPathParameterfNV = make_glPathParameterfNV();
		public static final UnlinkedGLFunction glPathParameterfvNV = make_glPathParameterfvNV();
		public static final UnlinkedGLFunction glPathParameteriNV = make_glPathParameteriNV();
		public static final UnlinkedGLFunction glPathParameterivNV = make_glPathParameterivNV();
		public static final UnlinkedGLFunction glPathStencilDepthOffsetNV = make_glPathStencilDepthOffsetNV();
		public static final UnlinkedGLFunction glPathStencilFuncNV = make_glPathStencilFuncNV();
		public static final UnlinkedGLFunction glPathStringNV = make_glPathStringNV();
		public static final UnlinkedGLFunction glPathSubCommandsNV = make_glPathSubCommandsNV();
		public static final UnlinkedGLFunction glPathSubCoordsNV = make_glPathSubCoordsNV();
		public static final UnlinkedGLFunction glPathTexGenNV = make_glPathTexGenNV();
		public static final UnlinkedGLFunction glPauseTransformFeedback = make_glPauseTransformFeedback();
		public static final UnlinkedGLFunction glPauseTransformFeedbackNV = make_glPauseTransformFeedbackNV();
		public static final UnlinkedGLFunction glPixelDataRangeNV = make_glPixelDataRangeNV();
		public static final UnlinkedGLFunction glPixelMapfv = make_glPixelMapfv();
		public static final UnlinkedGLFunction glPixelMapuiv = make_glPixelMapuiv();
		public static final UnlinkedGLFunction glPixelMapusv = make_glPixelMapusv();
		public static final UnlinkedGLFunction glPixelMapx = make_glPixelMapx();
		public static final UnlinkedGLFunction glPixelStoref = make_glPixelStoref();
		public static final UnlinkedGLFunction glPixelStorei = make_glPixelStorei();
		public static final UnlinkedGLFunction glPixelStorex = make_glPixelStorex();
		public static final UnlinkedGLFunction glPixelTexGenParameterfSGIS = make_glPixelTexGenParameterfSGIS();
		public static final UnlinkedGLFunction glPixelTexGenParameterfvSGIS = make_glPixelTexGenParameterfvSGIS();
		public static final UnlinkedGLFunction glPixelTexGenParameteriSGIS = make_glPixelTexGenParameteriSGIS();
		public static final UnlinkedGLFunction glPixelTexGenParameterivSGIS = make_glPixelTexGenParameterivSGIS();
		public static final UnlinkedGLFunction glPixelTexGenSGIX = make_glPixelTexGenSGIX();
		public static final UnlinkedGLFunction glPixelTransferf = make_glPixelTransferf();
		public static final UnlinkedGLFunction glPixelTransferi = make_glPixelTransferi();
		public static final UnlinkedGLFunction glPixelTransferxOES = make_glPixelTransferxOES();
		public static final UnlinkedGLFunction glPixelTransformParameterfEXT = make_glPixelTransformParameterfEXT();
		public static final UnlinkedGLFunction glPixelTransformParameterfvEXT = make_glPixelTransformParameterfvEXT();
		public static final UnlinkedGLFunction glPixelTransformParameteriEXT = make_glPixelTransformParameteriEXT();
		public static final UnlinkedGLFunction glPixelTransformParameterivEXT = make_glPixelTransformParameterivEXT();
		public static final UnlinkedGLFunction glPixelZoom = make_glPixelZoom();
		public static final UnlinkedGLFunction glPixelZoomxOES = make_glPixelZoomxOES();
		public static final UnlinkedGLFunction glPointAlongPathNV = make_glPointAlongPathNV();
		public static final UnlinkedGLFunction glPointParameterf = make_glPointParameterf();
		public static final UnlinkedGLFunction glPointParameterfARB = make_glPointParameterfARB();
		public static final UnlinkedGLFunction glPointParameterfEXT = make_glPointParameterfEXT();
		public static final UnlinkedGLFunction glPointParameterfSGIS = make_glPointParameterfSGIS();
		public static final UnlinkedGLFunction glPointParameterfv = make_glPointParameterfv();
		public static final UnlinkedGLFunction glPointParameterfvARB = make_glPointParameterfvARB();
		public static final UnlinkedGLFunction glPointParameterfvEXT = make_glPointParameterfvEXT();
		public static final UnlinkedGLFunction glPointParameterfvSGIS = make_glPointParameterfvSGIS();
		public static final UnlinkedGLFunction glPointParameteri = make_glPointParameteri();
		public static final UnlinkedGLFunction glPointParameteriNV = make_glPointParameteriNV();
		public static final UnlinkedGLFunction glPointParameteriv = make_glPointParameteriv();
		public static final UnlinkedGLFunction glPointParameterivNV = make_glPointParameterivNV();
		public static final UnlinkedGLFunction glPointParameterx = make_glPointParameterx();
		public static final UnlinkedGLFunction glPointParameterxOES = make_glPointParameterxOES();
		public static final UnlinkedGLFunction glPointParameterxv = make_glPointParameterxv();
		public static final UnlinkedGLFunction glPointParameterxvOES = make_glPointParameterxvOES();
		public static final UnlinkedGLFunction glPointSize = make_glPointSize();
		public static final UnlinkedGLFunction glPointSizePointerOES = make_glPointSizePointerOES();
		public static final UnlinkedGLFunction glPointSizex = make_glPointSizex();
		public static final UnlinkedGLFunction glPointSizexOES = make_glPointSizexOES();
		public static final UnlinkedGLFunction glPollAsyncSGIX = make_glPollAsyncSGIX();
		public static final UnlinkedGLFunction glPollInstrumentsSGIX = make_glPollInstrumentsSGIX();
		public static final UnlinkedGLFunction glPolygonMode = make_glPolygonMode();
		public static final UnlinkedGLFunction glPolygonModeNV = make_glPolygonModeNV();
		public static final UnlinkedGLFunction glPolygonOffset = make_glPolygonOffset();
		public static final UnlinkedGLFunction glPolygonOffsetClamp = make_glPolygonOffsetClamp();
		public static final UnlinkedGLFunction glPolygonOffsetClampEXT = make_glPolygonOffsetClampEXT();
		public static final UnlinkedGLFunction glPolygonOffsetEXT = make_glPolygonOffsetEXT();
		public static final UnlinkedGLFunction glPolygonOffsetx = make_glPolygonOffsetx();
		public static final UnlinkedGLFunction glPolygonOffsetxOES = make_glPolygonOffsetxOES();
		public static final UnlinkedGLFunction glPolygonStipple = make_glPolygonStipple();
		public static final UnlinkedGLFunction glPopAttrib = make_glPopAttrib();
		public static final UnlinkedGLFunction glPopClientAttrib = make_glPopClientAttrib();
		public static final UnlinkedGLFunction glPopDebugGroup = make_glPopDebugGroup();
		public static final UnlinkedGLFunction glPopDebugGroupKHR = make_glPopDebugGroupKHR();
		public static final UnlinkedGLFunction glPopGroupMarkerEXT = make_glPopGroupMarkerEXT();
		public static final UnlinkedGLFunction glPopMatrix = make_glPopMatrix();
		public static final UnlinkedGLFunction glPopName = make_glPopName();
		public static final UnlinkedGLFunction glPresentFrameDualFillNV = make_glPresentFrameDualFillNV();
		public static final UnlinkedGLFunction glPresentFrameKeyedNV = make_glPresentFrameKeyedNV();
		public static final UnlinkedGLFunction glPrimitiveBoundingBox = make_glPrimitiveBoundingBox();
		public static final UnlinkedGLFunction glPrimitiveBoundingBoxARB = make_glPrimitiveBoundingBoxARB();
		public static final UnlinkedGLFunction glPrimitiveBoundingBoxEXT = make_glPrimitiveBoundingBoxEXT();
		public static final UnlinkedGLFunction glPrimitiveBoundingBoxOES = make_glPrimitiveBoundingBoxOES();
		public static final UnlinkedGLFunction glPrimitiveRestartIndex = make_glPrimitiveRestartIndex();
		public static final UnlinkedGLFunction glPrimitiveRestartIndexNV = make_glPrimitiveRestartIndexNV();
		public static final UnlinkedGLFunction glPrimitiveRestartNV = make_glPrimitiveRestartNV();
		public static final UnlinkedGLFunction glPrioritizeTextures = make_glPrioritizeTextures();
		public static final UnlinkedGLFunction glPrioritizeTexturesEXT = make_glPrioritizeTexturesEXT();
		public static final UnlinkedGLFunction glPrioritizeTexturesxOES = make_glPrioritizeTexturesxOES();
		public static final UnlinkedGLFunction glProgramBinary = make_glProgramBinary();
		public static final UnlinkedGLFunction glProgramBinaryOES = make_glProgramBinaryOES();
		public static final UnlinkedGLFunction glProgramBufferParametersIivNV = make_glProgramBufferParametersIivNV();
		public static final UnlinkedGLFunction glProgramBufferParametersIuivNV = make_glProgramBufferParametersIuivNV();
		public static final UnlinkedGLFunction glProgramBufferParametersfvNV = make_glProgramBufferParametersfvNV();
		public static final UnlinkedGLFunction glProgramEnvParameter4dARB = make_glProgramEnvParameter4dARB();
		public static final UnlinkedGLFunction glProgramEnvParameter4dvARB = make_glProgramEnvParameter4dvARB();
		public static final UnlinkedGLFunction glProgramEnvParameter4fARB = make_glProgramEnvParameter4fARB();
		public static final UnlinkedGLFunction glProgramEnvParameter4fvARB = make_glProgramEnvParameter4fvARB();
		public static final UnlinkedGLFunction glProgramEnvParameterI4iNV = make_glProgramEnvParameterI4iNV();
		public static final UnlinkedGLFunction glProgramEnvParameterI4ivNV = make_glProgramEnvParameterI4ivNV();
		public static final UnlinkedGLFunction glProgramEnvParameterI4uiNV = make_glProgramEnvParameterI4uiNV();
		public static final UnlinkedGLFunction glProgramEnvParameterI4uivNV = make_glProgramEnvParameterI4uivNV();
		public static final UnlinkedGLFunction glProgramEnvParameters4fvEXT = make_glProgramEnvParameters4fvEXT();
		public static final UnlinkedGLFunction glProgramEnvParametersI4ivNV = make_glProgramEnvParametersI4ivNV();
		public static final UnlinkedGLFunction glProgramEnvParametersI4uivNV = make_glProgramEnvParametersI4uivNV();
		public static final UnlinkedGLFunction glProgramLocalParameter4dARB = make_glProgramLocalParameter4dARB();
		public static final UnlinkedGLFunction glProgramLocalParameter4dvARB = make_glProgramLocalParameter4dvARB();
		public static final UnlinkedGLFunction glProgramLocalParameter4fARB = make_glProgramLocalParameter4fARB();
		public static final UnlinkedGLFunction glProgramLocalParameter4fvARB = make_glProgramLocalParameter4fvARB();
		public static final UnlinkedGLFunction glProgramLocalParameterI4iNV = make_glProgramLocalParameterI4iNV();
		public static final UnlinkedGLFunction glProgramLocalParameterI4ivNV = make_glProgramLocalParameterI4ivNV();
		public static final UnlinkedGLFunction glProgramLocalParameterI4uiNV = make_glProgramLocalParameterI4uiNV();
		public static final UnlinkedGLFunction glProgramLocalParameterI4uivNV = make_glProgramLocalParameterI4uivNV();
		public static final UnlinkedGLFunction glProgramLocalParameters4fvEXT = make_glProgramLocalParameters4fvEXT();
		public static final UnlinkedGLFunction glProgramLocalParametersI4ivNV = make_glProgramLocalParametersI4ivNV();
		public static final UnlinkedGLFunction glProgramLocalParametersI4uivNV = make_glProgramLocalParametersI4uivNV();
		public static final UnlinkedGLFunction glProgramNamedParameter4dNV = make_glProgramNamedParameter4dNV();
		public static final UnlinkedGLFunction glProgramNamedParameter4dvNV = make_glProgramNamedParameter4dvNV();
		public static final UnlinkedGLFunction glProgramNamedParameter4fNV = make_glProgramNamedParameter4fNV();
		public static final UnlinkedGLFunction glProgramNamedParameter4fvNV = make_glProgramNamedParameter4fvNV();
		public static final UnlinkedGLFunction glProgramParameter4dNV = make_glProgramParameter4dNV();
		public static final UnlinkedGLFunction glProgramParameter4dvNV = make_glProgramParameter4dvNV();
		public static final UnlinkedGLFunction glProgramParameter4fNV = make_glProgramParameter4fNV();
		public static final UnlinkedGLFunction glProgramParameter4fvNV = make_glProgramParameter4fvNV();
		public static final UnlinkedGLFunction glProgramParameteri = make_glProgramParameteri();
		public static final UnlinkedGLFunction glProgramParameteriARB = make_glProgramParameteriARB();
		public static final UnlinkedGLFunction glProgramParameteriEXT = make_glProgramParameteriEXT();
		public static final UnlinkedGLFunction glProgramParameters4dvNV = make_glProgramParameters4dvNV();
		public static final UnlinkedGLFunction glProgramParameters4fvNV = make_glProgramParameters4fvNV();
		public static final UnlinkedGLFunction glProgramPathFragmentInputGenNV = make_glProgramPathFragmentInputGenNV();
		public static final UnlinkedGLFunction glProgramStringARB = make_glProgramStringARB();
		public static final UnlinkedGLFunction glProgramSubroutineParametersuivNV = make_glProgramSubroutineParametersuivNV();
		public static final UnlinkedGLFunction glProgramUniform1d = make_glProgramUniform1d();
		public static final UnlinkedGLFunction glProgramUniform1dEXT = make_glProgramUniform1dEXT();
		public static final UnlinkedGLFunction glProgramUniform1dv = make_glProgramUniform1dv();
		public static final UnlinkedGLFunction glProgramUniform1dvEXT = make_glProgramUniform1dvEXT();
		public static final UnlinkedGLFunction glProgramUniform1f = make_glProgramUniform1f();
		public static final UnlinkedGLFunction glProgramUniform1fEXT = make_glProgramUniform1fEXT();
		public static final UnlinkedGLFunction glProgramUniform1fv = make_glProgramUniform1fv();
		public static final UnlinkedGLFunction glProgramUniform1fvEXT = make_glProgramUniform1fvEXT();
		public static final UnlinkedGLFunction glProgramUniform1i = make_glProgramUniform1i();
		public static final UnlinkedGLFunction glProgramUniform1i64ARB = make_glProgramUniform1i64ARB();
		public static final UnlinkedGLFunction glProgramUniform1i64NV = make_glProgramUniform1i64NV();
		public static final UnlinkedGLFunction glProgramUniform1i64vARB = make_glProgramUniform1i64vARB();
		public static final UnlinkedGLFunction glProgramUniform1i64vNV = make_glProgramUniform1i64vNV();
		public static final UnlinkedGLFunction glProgramUniform1iEXT = make_glProgramUniform1iEXT();
		public static final UnlinkedGLFunction glProgramUniform1iv = make_glProgramUniform1iv();
		public static final UnlinkedGLFunction glProgramUniform1ivEXT = make_glProgramUniform1ivEXT();
		public static final UnlinkedGLFunction glProgramUniform1ui = make_glProgramUniform1ui();
		public static final UnlinkedGLFunction glProgramUniform1ui64ARB = make_glProgramUniform1ui64ARB();
		public static final UnlinkedGLFunction glProgramUniform1ui64NV = make_glProgramUniform1ui64NV();
		public static final UnlinkedGLFunction glProgramUniform1ui64vARB = make_glProgramUniform1ui64vARB();
		public static final UnlinkedGLFunction glProgramUniform1ui64vNV = make_glProgramUniform1ui64vNV();
		public static final UnlinkedGLFunction glProgramUniform1uiEXT = make_glProgramUniform1uiEXT();
		public static final UnlinkedGLFunction glProgramUniform1uiv = make_glProgramUniform1uiv();
		public static final UnlinkedGLFunction glProgramUniform1uivEXT = make_glProgramUniform1uivEXT();
		public static final UnlinkedGLFunction glProgramUniform2d = make_glProgramUniform2d();
		public static final UnlinkedGLFunction glProgramUniform2dEXT = make_glProgramUniform2dEXT();
		public static final UnlinkedGLFunction glProgramUniform2dv = make_glProgramUniform2dv();
		public static final UnlinkedGLFunction glProgramUniform2dvEXT = make_glProgramUniform2dvEXT();
		public static final UnlinkedGLFunction glProgramUniform2f = make_glProgramUniform2f();
		public static final UnlinkedGLFunction glProgramUniform2fEXT = make_glProgramUniform2fEXT();
		public static final UnlinkedGLFunction glProgramUniform2fv = make_glProgramUniform2fv();
		public static final UnlinkedGLFunction glProgramUniform2fvEXT = make_glProgramUniform2fvEXT();
		public static final UnlinkedGLFunction glProgramUniform2i = make_glProgramUniform2i();
		public static final UnlinkedGLFunction glProgramUniform2i64ARB = make_glProgramUniform2i64ARB();
		public static final UnlinkedGLFunction glProgramUniform2i64NV = make_glProgramUniform2i64NV();
		public static final UnlinkedGLFunction glProgramUniform2i64vARB = make_glProgramUniform2i64vARB();
		public static final UnlinkedGLFunction glProgramUniform2i64vNV = make_glProgramUniform2i64vNV();
		public static final UnlinkedGLFunction glProgramUniform2iEXT = make_glProgramUniform2iEXT();
		public static final UnlinkedGLFunction glProgramUniform2iv = make_glProgramUniform2iv();
		public static final UnlinkedGLFunction glProgramUniform2ivEXT = make_glProgramUniform2ivEXT();
		public static final UnlinkedGLFunction glProgramUniform2ui = make_glProgramUniform2ui();
		public static final UnlinkedGLFunction glProgramUniform2ui64ARB = make_glProgramUniform2ui64ARB();
		public static final UnlinkedGLFunction glProgramUniform2ui64NV = make_glProgramUniform2ui64NV();
		public static final UnlinkedGLFunction glProgramUniform2ui64vARB = make_glProgramUniform2ui64vARB();
		public static final UnlinkedGLFunction glProgramUniform2ui64vNV = make_glProgramUniform2ui64vNV();
		public static final UnlinkedGLFunction glProgramUniform2uiEXT = make_glProgramUniform2uiEXT();
		public static final UnlinkedGLFunction glProgramUniform2uiv = make_glProgramUniform2uiv();
		public static final UnlinkedGLFunction glProgramUniform2uivEXT = make_glProgramUniform2uivEXT();
		public static final UnlinkedGLFunction glProgramUniform3d = make_glProgramUniform3d();
		public static final UnlinkedGLFunction glProgramUniform3dEXT = make_glProgramUniform3dEXT();
		public static final UnlinkedGLFunction glProgramUniform3dv = make_glProgramUniform3dv();
		public static final UnlinkedGLFunction glProgramUniform3dvEXT = make_glProgramUniform3dvEXT();
		public static final UnlinkedGLFunction glProgramUniform3f = make_glProgramUniform3f();
		public static final UnlinkedGLFunction glProgramUniform3fEXT = make_glProgramUniform3fEXT();
		public static final UnlinkedGLFunction glProgramUniform3fv = make_glProgramUniform3fv();
		public static final UnlinkedGLFunction glProgramUniform3fvEXT = make_glProgramUniform3fvEXT();
		public static final UnlinkedGLFunction glProgramUniform3i = make_glProgramUniform3i();
		public static final UnlinkedGLFunction glProgramUniform3i64ARB = make_glProgramUniform3i64ARB();
		public static final UnlinkedGLFunction glProgramUniform3i64NV = make_glProgramUniform3i64NV();
		public static final UnlinkedGLFunction glProgramUniform3i64vARB = make_glProgramUniform3i64vARB();
		public static final UnlinkedGLFunction glProgramUniform3i64vNV = make_glProgramUniform3i64vNV();
		public static final UnlinkedGLFunction glProgramUniform3iEXT = make_glProgramUniform3iEXT();
		public static final UnlinkedGLFunction glProgramUniform3iv = make_glProgramUniform3iv();
		public static final UnlinkedGLFunction glProgramUniform3ivEXT = make_glProgramUniform3ivEXT();
		public static final UnlinkedGLFunction glProgramUniform3ui = make_glProgramUniform3ui();
		public static final UnlinkedGLFunction glProgramUniform3ui64ARB = make_glProgramUniform3ui64ARB();
		public static final UnlinkedGLFunction glProgramUniform3ui64NV = make_glProgramUniform3ui64NV();
		public static final UnlinkedGLFunction glProgramUniform3ui64vARB = make_glProgramUniform3ui64vARB();
		public static final UnlinkedGLFunction glProgramUniform3ui64vNV = make_glProgramUniform3ui64vNV();
		public static final UnlinkedGLFunction glProgramUniform3uiEXT = make_glProgramUniform3uiEXT();
		public static final UnlinkedGLFunction glProgramUniform3uiv = make_glProgramUniform3uiv();
		public static final UnlinkedGLFunction glProgramUniform3uivEXT = make_glProgramUniform3uivEXT();
		public static final UnlinkedGLFunction glProgramUniform4d = make_glProgramUniform4d();
		public static final UnlinkedGLFunction glProgramUniform4dEXT = make_glProgramUniform4dEXT();
		public static final UnlinkedGLFunction glProgramUniform4dv = make_glProgramUniform4dv();
		public static final UnlinkedGLFunction glProgramUniform4dvEXT = make_glProgramUniform4dvEXT();
		public static final UnlinkedGLFunction glProgramUniform4f = make_glProgramUniform4f();
		public static final UnlinkedGLFunction glProgramUniform4fEXT = make_glProgramUniform4fEXT();
		public static final UnlinkedGLFunction glProgramUniform4fv = make_glProgramUniform4fv();
		public static final UnlinkedGLFunction glProgramUniform4fvEXT = make_glProgramUniform4fvEXT();
		public static final UnlinkedGLFunction glProgramUniform4i = make_glProgramUniform4i();
		public static final UnlinkedGLFunction glProgramUniform4i64ARB = make_glProgramUniform4i64ARB();
		public static final UnlinkedGLFunction glProgramUniform4i64NV = make_glProgramUniform4i64NV();
		public static final UnlinkedGLFunction glProgramUniform4i64vARB = make_glProgramUniform4i64vARB();
		public static final UnlinkedGLFunction glProgramUniform4i64vNV = make_glProgramUniform4i64vNV();
		public static final UnlinkedGLFunction glProgramUniform4iEXT = make_glProgramUniform4iEXT();
		public static final UnlinkedGLFunction glProgramUniform4iv = make_glProgramUniform4iv();
		public static final UnlinkedGLFunction glProgramUniform4ivEXT = make_glProgramUniform4ivEXT();
		public static final UnlinkedGLFunction glProgramUniform4ui = make_glProgramUniform4ui();
		public static final UnlinkedGLFunction glProgramUniform4ui64ARB = make_glProgramUniform4ui64ARB();
		public static final UnlinkedGLFunction glProgramUniform4ui64NV = make_glProgramUniform4ui64NV();
		public static final UnlinkedGLFunction glProgramUniform4ui64vARB = make_glProgramUniform4ui64vARB();
		public static final UnlinkedGLFunction glProgramUniform4ui64vNV = make_glProgramUniform4ui64vNV();
		public static final UnlinkedGLFunction glProgramUniform4uiEXT = make_glProgramUniform4uiEXT();
		public static final UnlinkedGLFunction glProgramUniform4uiv = make_glProgramUniform4uiv();
		public static final UnlinkedGLFunction glProgramUniform4uivEXT = make_glProgramUniform4uivEXT();
		public static final UnlinkedGLFunction glProgramUniformHandleui64ARB = make_glProgramUniformHandleui64ARB();
		public static final UnlinkedGLFunction glProgramUniformHandleui64IMG = make_glProgramUniformHandleui64IMG();
		public static final UnlinkedGLFunction glProgramUniformHandleui64NV = make_glProgramUniformHandleui64NV();
		public static final UnlinkedGLFunction glProgramUniformHandleui64vARB = make_glProgramUniformHandleui64vARB();
		public static final UnlinkedGLFunction glProgramUniformHandleui64vIMG = make_glProgramUniformHandleui64vIMG();
		public static final UnlinkedGLFunction glProgramUniformHandleui64vNV = make_glProgramUniformHandleui64vNV();
		public static final UnlinkedGLFunction glProgramUniformMatrix2dv = make_glProgramUniformMatrix2dv();
		public static final UnlinkedGLFunction glProgramUniformMatrix2dvEXT = make_glProgramUniformMatrix2dvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix2fv = make_glProgramUniformMatrix2fv();
		public static final UnlinkedGLFunction glProgramUniformMatrix2fvEXT = make_glProgramUniformMatrix2fvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix2x3dv = make_glProgramUniformMatrix2x3dv();
		public static final UnlinkedGLFunction glProgramUniformMatrix2x3dvEXT = make_glProgramUniformMatrix2x3dvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix2x3fv = make_glProgramUniformMatrix2x3fv();
		public static final UnlinkedGLFunction glProgramUniformMatrix2x3fvEXT = make_glProgramUniformMatrix2x3fvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix2x4dv = make_glProgramUniformMatrix2x4dv();
		public static final UnlinkedGLFunction glProgramUniformMatrix2x4dvEXT = make_glProgramUniformMatrix2x4dvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix2x4fv = make_glProgramUniformMatrix2x4fv();
		public static final UnlinkedGLFunction glProgramUniformMatrix2x4fvEXT = make_glProgramUniformMatrix2x4fvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix3dv = make_glProgramUniformMatrix3dv();
		public static final UnlinkedGLFunction glProgramUniformMatrix3dvEXT = make_glProgramUniformMatrix3dvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix3fv = make_glProgramUniformMatrix3fv();
		public static final UnlinkedGLFunction glProgramUniformMatrix3fvEXT = make_glProgramUniformMatrix3fvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix3x2dv = make_glProgramUniformMatrix3x2dv();
		public static final UnlinkedGLFunction glProgramUniformMatrix3x2dvEXT = make_glProgramUniformMatrix3x2dvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix3x2fv = make_glProgramUniformMatrix3x2fv();
		public static final UnlinkedGLFunction glProgramUniformMatrix3x2fvEXT = make_glProgramUniformMatrix3x2fvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix3x4dv = make_glProgramUniformMatrix3x4dv();
		public static final UnlinkedGLFunction glProgramUniformMatrix3x4dvEXT = make_glProgramUniformMatrix3x4dvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix3x4fv = make_glProgramUniformMatrix3x4fv();
		public static final UnlinkedGLFunction glProgramUniformMatrix3x4fvEXT = make_glProgramUniformMatrix3x4fvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix4dv = make_glProgramUniformMatrix4dv();
		public static final UnlinkedGLFunction glProgramUniformMatrix4dvEXT = make_glProgramUniformMatrix4dvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix4fv = make_glProgramUniformMatrix4fv();
		public static final UnlinkedGLFunction glProgramUniformMatrix4fvEXT = make_glProgramUniformMatrix4fvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix4x2dv = make_glProgramUniformMatrix4x2dv();
		public static final UnlinkedGLFunction glProgramUniformMatrix4x2dvEXT = make_glProgramUniformMatrix4x2dvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix4x2fv = make_glProgramUniformMatrix4x2fv();
		public static final UnlinkedGLFunction glProgramUniformMatrix4x2fvEXT = make_glProgramUniformMatrix4x2fvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix4x3dv = make_glProgramUniformMatrix4x3dv();
		public static final UnlinkedGLFunction glProgramUniformMatrix4x3dvEXT = make_glProgramUniformMatrix4x3dvEXT();
		public static final UnlinkedGLFunction glProgramUniformMatrix4x3fv = make_glProgramUniformMatrix4x3fv();
		public static final UnlinkedGLFunction glProgramUniformMatrix4x3fvEXT = make_glProgramUniformMatrix4x3fvEXT();
		public static final UnlinkedGLFunction glProgramUniformui64NV = make_glProgramUniformui64NV();
		public static final UnlinkedGLFunction glProgramUniformui64vNV = make_glProgramUniformui64vNV();
		public static final UnlinkedGLFunction glProgramVertexLimitNV = make_glProgramVertexLimitNV();
		public static final UnlinkedGLFunction glProvokingVertex = make_glProvokingVertex();
		public static final UnlinkedGLFunction glProvokingVertexEXT = make_glProvokingVertexEXT();
		public static final UnlinkedGLFunction glPushAttrib = make_glPushAttrib();
		public static final UnlinkedGLFunction glPushClientAttrib = make_glPushClientAttrib();
		public static final UnlinkedGLFunction glPushClientAttribDefaultEXT = make_glPushClientAttribDefaultEXT();
		public static final UnlinkedGLFunction glPushDebugGroup = make_glPushDebugGroup();
		public static final UnlinkedGLFunction glPushDebugGroupKHR = make_glPushDebugGroupKHR();
		public static final UnlinkedGLFunction glPushGroupMarkerEXT = make_glPushGroupMarkerEXT();
		public static final UnlinkedGLFunction glPushMatrix = make_glPushMatrix();
		public static final UnlinkedGLFunction glPushName = make_glPushName();
		public static final UnlinkedGLFunction glQueryCounter = make_glQueryCounter();
		public static final UnlinkedGLFunction glQueryCounterEXT = make_glQueryCounterEXT();
		public static final UnlinkedGLFunction glQueryMatrixxOES = make_glQueryMatrixxOES();
		public static final UnlinkedGLFunction glQueryObjectParameteruiAMD = make_glQueryObjectParameteruiAMD();
		public static final UnlinkedGLFunction glQueryResourceNV = make_glQueryResourceNV();
		public static final UnlinkedGLFunction glQueryResourceTagNV = make_glQueryResourceTagNV();
		public static final UnlinkedGLFunction glRasterPos2d = make_glRasterPos2d();
		public static final UnlinkedGLFunction glRasterPos2dv = make_glRasterPos2dv();
		public static final UnlinkedGLFunction glRasterPos2f = make_glRasterPos2f();
		public static final UnlinkedGLFunction glRasterPos2fv = make_glRasterPos2fv();
		public static final UnlinkedGLFunction glRasterPos2i = make_glRasterPos2i();
		public static final UnlinkedGLFunction glRasterPos2iv = make_glRasterPos2iv();
		public static final UnlinkedGLFunction glRasterPos2s = make_glRasterPos2s();
		public static final UnlinkedGLFunction glRasterPos2sv = make_glRasterPos2sv();
		public static final UnlinkedGLFunction glRasterPos2xOES = make_glRasterPos2xOES();
		public static final UnlinkedGLFunction glRasterPos2xvOES = make_glRasterPos2xvOES();
		public static final UnlinkedGLFunction glRasterPos3d = make_glRasterPos3d();
		public static final UnlinkedGLFunction glRasterPos3dv = make_glRasterPos3dv();
		public static final UnlinkedGLFunction glRasterPos3f = make_glRasterPos3f();
		public static final UnlinkedGLFunction glRasterPos3fv = make_glRasterPos3fv();
		public static final UnlinkedGLFunction glRasterPos3i = make_glRasterPos3i();
		public static final UnlinkedGLFunction glRasterPos3iv = make_glRasterPos3iv();
		public static final UnlinkedGLFunction glRasterPos3s = make_glRasterPos3s();
		public static final UnlinkedGLFunction glRasterPos3sv = make_glRasterPos3sv();
		public static final UnlinkedGLFunction glRasterPos3xOES = make_glRasterPos3xOES();
		public static final UnlinkedGLFunction glRasterPos3xvOES = make_glRasterPos3xvOES();
		public static final UnlinkedGLFunction glRasterPos4d = make_glRasterPos4d();
		public static final UnlinkedGLFunction glRasterPos4dv = make_glRasterPos4dv();
		public static final UnlinkedGLFunction glRasterPos4f = make_glRasterPos4f();
		public static final UnlinkedGLFunction glRasterPos4fv = make_glRasterPos4fv();
		public static final UnlinkedGLFunction glRasterPos4i = make_glRasterPos4i();
		public static final UnlinkedGLFunction glRasterPos4iv = make_glRasterPos4iv();
		public static final UnlinkedGLFunction glRasterPos4s = make_glRasterPos4s();
		public static final UnlinkedGLFunction glRasterPos4sv = make_glRasterPos4sv();
		public static final UnlinkedGLFunction glRasterPos4xOES = make_glRasterPos4xOES();
		public static final UnlinkedGLFunction glRasterPos4xvOES = make_glRasterPos4xvOES();
		public static final UnlinkedGLFunction glRasterSamplesEXT = make_glRasterSamplesEXT();
		public static final UnlinkedGLFunction glReadBuffer = make_glReadBuffer();
		public static final UnlinkedGLFunction glReadBufferIndexedEXT = make_glReadBufferIndexedEXT();
		public static final UnlinkedGLFunction glReadBufferNV = make_glReadBufferNV();
		public static final UnlinkedGLFunction glReadInstrumentsSGIX = make_glReadInstrumentsSGIX();
		public static final UnlinkedGLFunction glReadPixels = make_glReadPixels();
		public static final UnlinkedGLFunction glReadnPixels = make_glReadnPixels();
		public static final UnlinkedGLFunction glReadnPixelsARB = make_glReadnPixelsARB();
		public static final UnlinkedGLFunction glReadnPixelsEXT = make_glReadnPixelsEXT();
		public static final UnlinkedGLFunction glReadnPixelsKHR = make_glReadnPixelsKHR();
		public static final UnlinkedGLFunction glReleaseKeyedMutexWin32EXT = make_glReleaseKeyedMutexWin32EXT();
		public static final UnlinkedGLFunction glRectd = make_glRectd();
		public static final UnlinkedGLFunction glRectdv = make_glRectdv();
		public static final UnlinkedGLFunction glRectf = make_glRectf();
		public static final UnlinkedGLFunction glRectfv = make_glRectfv();
		public static final UnlinkedGLFunction glRecti = make_glRecti();
		public static final UnlinkedGLFunction glRectiv = make_glRectiv();
		public static final UnlinkedGLFunction glRects = make_glRects();
		public static final UnlinkedGLFunction glRectsv = make_glRectsv();
		public static final UnlinkedGLFunction glRectxOES = make_glRectxOES();
		public static final UnlinkedGLFunction glRectxvOES = make_glRectxvOES();
		public static final UnlinkedGLFunction glReferencePlaneSGIX = make_glReferencePlaneSGIX();
		public static final UnlinkedGLFunction glReleaseShaderCompiler = make_glReleaseShaderCompiler();
		public static final UnlinkedGLFunction glRenderGpuMaskNV = make_glRenderGpuMaskNV();
		public static final UnlinkedGLFunction glRenderMode = make_glRenderMode();
		public static final UnlinkedGLFunction glRenderbufferStorage = make_glRenderbufferStorage();
		public static final UnlinkedGLFunction glRenderbufferStorageEXT = make_glRenderbufferStorageEXT();
		public static final UnlinkedGLFunction glRenderbufferStorageMultisample = make_glRenderbufferStorageMultisample();
		public static final UnlinkedGLFunction glRenderbufferStorageMultisampleANGLE = make_glRenderbufferStorageMultisampleANGLE();
		public static final UnlinkedGLFunction glRenderbufferStorageMultisampleAPPLE = make_glRenderbufferStorageMultisampleAPPLE();
		public static final UnlinkedGLFunction glRenderbufferStorageMultisampleAdvancedAMD = make_glRenderbufferStorageMultisampleAdvancedAMD();
		public static final UnlinkedGLFunction glRenderbufferStorageMultisampleCoverageNV = make_glRenderbufferStorageMultisampleCoverageNV();
		public static final UnlinkedGLFunction glRenderbufferStorageMultisampleEXT = make_glRenderbufferStorageMultisampleEXT();
		public static final UnlinkedGLFunction glRenderbufferStorageMultisampleIMG = make_glRenderbufferStorageMultisampleIMG();
		public static final UnlinkedGLFunction glRenderbufferStorageMultisampleNV = make_glRenderbufferStorageMultisampleNV();
		public static final UnlinkedGLFunction glRenderbufferStorageOES = make_glRenderbufferStorageOES();
		public static final UnlinkedGLFunction glReplacementCodePointerSUN = make_glReplacementCodePointerSUN();
		public static final UnlinkedGLFunction glReplacementCodeubSUN = make_glReplacementCodeubSUN();
		public static final UnlinkedGLFunction glReplacementCodeubvSUN = make_glReplacementCodeubvSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiColor3fVertex3fSUN = make_glReplacementCodeuiColor3fVertex3fSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiColor3fVertex3fvSUN = make_glReplacementCodeuiColor3fVertex3fvSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiColor4fNormal3fVertex3fSUN = make_glReplacementCodeuiColor4fNormal3fVertex3fSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiColor4fNormal3fVertex3fvSUN = make_glReplacementCodeuiColor4fNormal3fVertex3fvSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiColor4ubVertex3fSUN = make_glReplacementCodeuiColor4ubVertex3fSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiColor4ubVertex3fvSUN = make_glReplacementCodeuiColor4ubVertex3fvSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiNormal3fVertex3fSUN = make_glReplacementCodeuiNormal3fVertex3fSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiNormal3fVertex3fvSUN = make_glReplacementCodeuiNormal3fVertex3fvSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiSUN = make_glReplacementCodeuiSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN = make_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN = make_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN = make_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN = make_glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiTexCoord2fVertex3fSUN = make_glReplacementCodeuiTexCoord2fVertex3fSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiTexCoord2fVertex3fvSUN = make_glReplacementCodeuiTexCoord2fVertex3fvSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiVertex3fSUN = make_glReplacementCodeuiVertex3fSUN();
		public static final UnlinkedGLFunction glReplacementCodeuiVertex3fvSUN = make_glReplacementCodeuiVertex3fvSUN();
		public static final UnlinkedGLFunction glReplacementCodeuivSUN = make_glReplacementCodeuivSUN();
		public static final UnlinkedGLFunction glReplacementCodeusSUN = make_glReplacementCodeusSUN();
		public static final UnlinkedGLFunction glReplacementCodeusvSUN = make_glReplacementCodeusvSUN();
		public static final UnlinkedGLFunction glRequestResidentProgramsNV = make_glRequestResidentProgramsNV();
		public static final UnlinkedGLFunction glResetHistogram = make_glResetHistogram();
		public static final UnlinkedGLFunction glResetHistogramEXT = make_glResetHistogramEXT();
		public static final UnlinkedGLFunction glResetMemoryObjectParameterNV = make_glResetMemoryObjectParameterNV();
		public static final UnlinkedGLFunction glResetMinmax = make_glResetMinmax();
		public static final UnlinkedGLFunction glResetMinmaxEXT = make_glResetMinmaxEXT();
		public static final UnlinkedGLFunction glResizeBuffersMESA = make_glResizeBuffersMESA();
		public static final UnlinkedGLFunction glResolveDepthValuesNV = make_glResolveDepthValuesNV();
		public static final UnlinkedGLFunction glResolveMultisampleFramebufferAPPLE = make_glResolveMultisampleFramebufferAPPLE();
		public static final UnlinkedGLFunction glResumeTransformFeedback = make_glResumeTransformFeedback();
		public static final UnlinkedGLFunction glResumeTransformFeedbackNV = make_glResumeTransformFeedbackNV();
		public static final UnlinkedGLFunction glRotated = make_glRotated();
		public static final UnlinkedGLFunction glRotatef = make_glRotatef();
		public static final UnlinkedGLFunction glRotatex = make_glRotatex();
		public static final UnlinkedGLFunction glRotatexOES = make_glRotatexOES();
		public static final UnlinkedGLFunction glSampleCoverage = make_glSampleCoverage();
		public static final UnlinkedGLFunction glSampleCoverageARB = make_glSampleCoverageARB();
		public static final UnlinkedGLFunction glSampleCoveragex = make_glSampleCoveragex();
		public static final UnlinkedGLFunction glSampleCoveragexOES = make_glSampleCoveragexOES();
		public static final UnlinkedGLFunction glSampleMapATI = make_glSampleMapATI();
		public static final UnlinkedGLFunction glSampleMaskEXT = make_glSampleMaskEXT();
		public static final UnlinkedGLFunction glSampleMaskIndexedNV = make_glSampleMaskIndexedNV();
		public static final UnlinkedGLFunction glSampleMaskSGIS = make_glSampleMaskSGIS();
		public static final UnlinkedGLFunction glSampleMaski = make_glSampleMaski();
		public static final UnlinkedGLFunction glSamplePatternEXT = make_glSamplePatternEXT();
		public static final UnlinkedGLFunction glSamplePatternSGIS = make_glSamplePatternSGIS();
		public static final UnlinkedGLFunction glSamplerParameterIiv = make_glSamplerParameterIiv();
		public static final UnlinkedGLFunction glSamplerParameterIivEXT = make_glSamplerParameterIivEXT();
		public static final UnlinkedGLFunction glSamplerParameterIivOES = make_glSamplerParameterIivOES();
		public static final UnlinkedGLFunction glSamplerParameterIuiv = make_glSamplerParameterIuiv();
		public static final UnlinkedGLFunction glSamplerParameterIuivEXT = make_glSamplerParameterIuivEXT();
		public static final UnlinkedGLFunction glSamplerParameterIuivOES = make_glSamplerParameterIuivOES();
		public static final UnlinkedGLFunction glSamplerParameterf = make_glSamplerParameterf();
		public static final UnlinkedGLFunction glSamplerParameterfv = make_glSamplerParameterfv();
		public static final UnlinkedGLFunction glSamplerParameteri = make_glSamplerParameteri();
		public static final UnlinkedGLFunction glSamplerParameteriv = make_glSamplerParameteriv();
		public static final UnlinkedGLFunction glScaled = make_glScaled();
		public static final UnlinkedGLFunction glScalef = make_glScalef();
		public static final UnlinkedGLFunction glScalex = make_glScalex();
		public static final UnlinkedGLFunction glScalexOES = make_glScalexOES();
		public static final UnlinkedGLFunction glScissor = make_glScissor();
		public static final UnlinkedGLFunction glScissorArrayv = make_glScissorArrayv();
		public static final UnlinkedGLFunction glScissorArrayvNV = make_glScissorArrayvNV();
		public static final UnlinkedGLFunction glScissorArrayvOES = make_glScissorArrayvOES();
		public static final UnlinkedGLFunction glScissorExclusiveArrayvNV = make_glScissorExclusiveArrayvNV();
		public static final UnlinkedGLFunction glScissorExclusiveNV = make_glScissorExclusiveNV();
		public static final UnlinkedGLFunction glScissorIndexed = make_glScissorIndexed();
		public static final UnlinkedGLFunction glScissorIndexedNV = make_glScissorIndexedNV();
		public static final UnlinkedGLFunction glScissorIndexedOES = make_glScissorIndexedOES();
		public static final UnlinkedGLFunction glScissorIndexedv = make_glScissorIndexedv();
		public static final UnlinkedGLFunction glScissorIndexedvNV = make_glScissorIndexedvNV();
		public static final UnlinkedGLFunction glScissorIndexedvOES = make_glScissorIndexedvOES();
		public static final UnlinkedGLFunction glSecondaryColor3b = make_glSecondaryColor3b();
		public static final UnlinkedGLFunction glSecondaryColor3bEXT = make_glSecondaryColor3bEXT();
		public static final UnlinkedGLFunction glSecondaryColor3bv = make_glSecondaryColor3bv();
		public static final UnlinkedGLFunction glSecondaryColor3bvEXT = make_glSecondaryColor3bvEXT();
		public static final UnlinkedGLFunction glSecondaryColor3d = make_glSecondaryColor3d();
		public static final UnlinkedGLFunction glSecondaryColor3dEXT = make_glSecondaryColor3dEXT();
		public static final UnlinkedGLFunction glSecondaryColor3dv = make_glSecondaryColor3dv();
		public static final UnlinkedGLFunction glSecondaryColor3dvEXT = make_glSecondaryColor3dvEXT();
		public static final UnlinkedGLFunction glSecondaryColor3f = make_glSecondaryColor3f();
		public static final UnlinkedGLFunction glSecondaryColor3fEXT = make_glSecondaryColor3fEXT();
		public static final UnlinkedGLFunction glSecondaryColor3fv = make_glSecondaryColor3fv();
		public static final UnlinkedGLFunction glSecondaryColor3fvEXT = make_glSecondaryColor3fvEXT();
		public static final UnlinkedGLFunction glSecondaryColor3hNV = make_glSecondaryColor3hNV();
		public static final UnlinkedGLFunction glSecondaryColor3hvNV = make_glSecondaryColor3hvNV();
		public static final UnlinkedGLFunction glSecondaryColor3i = make_glSecondaryColor3i();
		public static final UnlinkedGLFunction glSecondaryColor3iEXT = make_glSecondaryColor3iEXT();
		public static final UnlinkedGLFunction glSecondaryColor3iv = make_glSecondaryColor3iv();
		public static final UnlinkedGLFunction glSecondaryColor3ivEXT = make_glSecondaryColor3ivEXT();
		public static final UnlinkedGLFunction glSecondaryColor3s = make_glSecondaryColor3s();
		public static final UnlinkedGLFunction glSecondaryColor3sEXT = make_glSecondaryColor3sEXT();
		public static final UnlinkedGLFunction glSecondaryColor3sv = make_glSecondaryColor3sv();
		public static final UnlinkedGLFunction glSecondaryColor3svEXT = make_glSecondaryColor3svEXT();
		public static final UnlinkedGLFunction glSecondaryColor3ub = make_glSecondaryColor3ub();
		public static final UnlinkedGLFunction glSecondaryColor3ubEXT = make_glSecondaryColor3ubEXT();
		public static final UnlinkedGLFunction glSecondaryColor3ubv = make_glSecondaryColor3ubv();
		public static final UnlinkedGLFunction glSecondaryColor3ubvEXT = make_glSecondaryColor3ubvEXT();
		public static final UnlinkedGLFunction glSecondaryColor3ui = make_glSecondaryColor3ui();
		public static final UnlinkedGLFunction glSecondaryColor3uiEXT = make_glSecondaryColor3uiEXT();
		public static final UnlinkedGLFunction glSecondaryColor3uiv = make_glSecondaryColor3uiv();
		public static final UnlinkedGLFunction glSecondaryColor3uivEXT = make_glSecondaryColor3uivEXT();
		public static final UnlinkedGLFunction glSecondaryColor3us = make_glSecondaryColor3us();
		public static final UnlinkedGLFunction glSecondaryColor3usEXT = make_glSecondaryColor3usEXT();
		public static final UnlinkedGLFunction glSecondaryColor3usv = make_glSecondaryColor3usv();
		public static final UnlinkedGLFunction glSecondaryColor3usvEXT = make_glSecondaryColor3usvEXT();
		public static final UnlinkedGLFunction glSecondaryColorFormatNV = make_glSecondaryColorFormatNV();
		public static final UnlinkedGLFunction glSecondaryColorP3ui = make_glSecondaryColorP3ui();
		public static final UnlinkedGLFunction glSecondaryColorP3uiv = make_glSecondaryColorP3uiv();
		public static final UnlinkedGLFunction glSecondaryColorPointer = make_glSecondaryColorPointer();
		public static final UnlinkedGLFunction glSecondaryColorPointerEXT = make_glSecondaryColorPointerEXT();
		public static final UnlinkedGLFunction glSecondaryColorPointerListIBM = make_glSecondaryColorPointerListIBM();
		public static final UnlinkedGLFunction glSelectBuffer = make_glSelectBuffer();
		public static final UnlinkedGLFunction glSelectPerfMonitorCountersAMD = make_glSelectPerfMonitorCountersAMD();
		public static final UnlinkedGLFunction glSemaphoreParameterivNV = make_glSemaphoreParameterivNV();
		public static final UnlinkedGLFunction glSemaphoreParameterui64vEXT = make_glSemaphoreParameterui64vEXT();
		public static final UnlinkedGLFunction glSeparableFilter2D = make_glSeparableFilter2D();
		public static final UnlinkedGLFunction glSeparableFilter2DEXT = make_glSeparableFilter2DEXT();
		public static final UnlinkedGLFunction glSetFenceAPPLE = make_glSetFenceAPPLE();
		public static final UnlinkedGLFunction glSetFenceNV = make_glSetFenceNV();
		public static final UnlinkedGLFunction glSetFragmentShaderConstantATI = make_glSetFragmentShaderConstantATI();
		public static final UnlinkedGLFunction glSetInvariantEXT = make_glSetInvariantEXT();
		public static final UnlinkedGLFunction glSetLocalConstantEXT = make_glSetLocalConstantEXT();
		public static final UnlinkedGLFunction glSetMultisamplefvAMD = make_glSetMultisamplefvAMD();
		public static final UnlinkedGLFunction glShadeModel = make_glShadeModel();
		public static final UnlinkedGLFunction glShaderBinary = make_glShaderBinary();
		public static final UnlinkedGLFunction glShaderOp1EXT = make_glShaderOp1EXT();
		public static final UnlinkedGLFunction glShaderOp2EXT = make_glShaderOp2EXT();
		public static final UnlinkedGLFunction glShaderOp3EXT = make_glShaderOp3EXT();
		public static final UnlinkedGLFunction glShaderSource = make_glShaderSource();
		public static final UnlinkedGLFunction glShaderSourceARB = make_glShaderSourceARB();
		public static final UnlinkedGLFunction glShaderStorageBlockBinding = make_glShaderStorageBlockBinding();
		public static final UnlinkedGLFunction glShadingRateEXT = make_glShadingRateEXT();
		public static final UnlinkedGLFunction glShadingRateCombinerOpsEXT = make_glShadingRateCombinerOpsEXT();
		public static final UnlinkedGLFunction glShadingRateImageBarrierNV = make_glShadingRateImageBarrierNV();
		public static final UnlinkedGLFunction glShadingRateQCOM = make_glShadingRateQCOM();
		public static final UnlinkedGLFunction glShadingRateImagePaletteNV = make_glShadingRateImagePaletteNV();
		public static final UnlinkedGLFunction glShadingRateSampleOrderNV = make_glShadingRateSampleOrderNV();
		public static final UnlinkedGLFunction glShadingRateSampleOrderCustomNV = make_glShadingRateSampleOrderCustomNV();
		public static final UnlinkedGLFunction glSharpenTexFuncSGIS = make_glSharpenTexFuncSGIS();
		public static final UnlinkedGLFunction glSignalSemaphoreEXT = make_glSignalSemaphoreEXT();
		public static final UnlinkedGLFunction glSignalSemaphoreui64NVX = make_glSignalSemaphoreui64NVX();
		public static final UnlinkedGLFunction glSpecializeShader = make_glSpecializeShader();
		public static final UnlinkedGLFunction glSpecializeShaderARB = make_glSpecializeShaderARB();
		public static final UnlinkedGLFunction glSpriteParameterfSGIX = make_glSpriteParameterfSGIX();
		public static final UnlinkedGLFunction glSpriteParameterfvSGIX = make_glSpriteParameterfvSGIX();
		public static final UnlinkedGLFunction glSpriteParameteriSGIX = make_glSpriteParameteriSGIX();
		public static final UnlinkedGLFunction glSpriteParameterivSGIX = make_glSpriteParameterivSGIX();
		public static final UnlinkedGLFunction glStartInstrumentsSGIX = make_glStartInstrumentsSGIX();
		public static final UnlinkedGLFunction glStartTilingQCOM = make_glStartTilingQCOM();
		public static final UnlinkedGLFunction glStateCaptureNV = make_glStateCaptureNV();
		public static final UnlinkedGLFunction glStencilClearTagEXT = make_glStencilClearTagEXT();
		public static final UnlinkedGLFunction glStencilFillPathInstancedNV = make_glStencilFillPathInstancedNV();
		public static final UnlinkedGLFunction glStencilFillPathNV = make_glStencilFillPathNV();
		public static final UnlinkedGLFunction glStencilFunc = make_glStencilFunc();
		public static final UnlinkedGLFunction glStencilFuncSeparate = make_glStencilFuncSeparate();
		public static final UnlinkedGLFunction glStencilFuncSeparateATI = make_glStencilFuncSeparateATI();
		public static final UnlinkedGLFunction glStencilMask = make_glStencilMask();
		public static final UnlinkedGLFunction glStencilMaskSeparate = make_glStencilMaskSeparate();
		public static final UnlinkedGLFunction glStencilOp = make_glStencilOp();
		public static final UnlinkedGLFunction glStencilOpSeparate = make_glStencilOpSeparate();
		public static final UnlinkedGLFunction glStencilOpSeparateATI = make_glStencilOpSeparateATI();
		public static final UnlinkedGLFunction glStencilOpValueAMD = make_glStencilOpValueAMD();
		public static final UnlinkedGLFunction glStencilStrokePathInstancedNV = make_glStencilStrokePathInstancedNV();
		public static final UnlinkedGLFunction glStencilStrokePathNV = make_glStencilStrokePathNV();
		public static final UnlinkedGLFunction glStencilThenCoverFillPathInstancedNV = make_glStencilThenCoverFillPathInstancedNV();
		public static final UnlinkedGLFunction glStencilThenCoverFillPathNV = make_glStencilThenCoverFillPathNV();
		public static final UnlinkedGLFunction glStencilThenCoverStrokePathInstancedNV = make_glStencilThenCoverStrokePathInstancedNV();
		public static final UnlinkedGLFunction glStencilThenCoverStrokePathNV = make_glStencilThenCoverStrokePathNV();
		public static final UnlinkedGLFunction glStopInstrumentsSGIX = make_glStopInstrumentsSGIX();
		public static final UnlinkedGLFunction glStringMarkerGREMEDY = make_glStringMarkerGREMEDY();
		public static final UnlinkedGLFunction glSubpixelPrecisionBiasNV = make_glSubpixelPrecisionBiasNV();
		public static final UnlinkedGLFunction glSwizzleEXT = make_glSwizzleEXT();
		public static final UnlinkedGLFunction glSyncTextureINTEL = make_glSyncTextureINTEL();
		public static final UnlinkedGLFunction glTagSampleBufferSGIX = make_glTagSampleBufferSGIX();
		public static final UnlinkedGLFunction glTangent3bEXT = make_glTangent3bEXT();
		public static final UnlinkedGLFunction glTangent3bvEXT = make_glTangent3bvEXT();
		public static final UnlinkedGLFunction glTangent3dEXT = make_glTangent3dEXT();
		public static final UnlinkedGLFunction glTangent3dvEXT = make_glTangent3dvEXT();
		public static final UnlinkedGLFunction glTangent3fEXT = make_glTangent3fEXT();
		public static final UnlinkedGLFunction glTangent3fvEXT = make_glTangent3fvEXT();
		public static final UnlinkedGLFunction glTangent3iEXT = make_glTangent3iEXT();
		public static final UnlinkedGLFunction glTangent3ivEXT = make_glTangent3ivEXT();
		public static final UnlinkedGLFunction glTangent3sEXT = make_glTangent3sEXT();
		public static final UnlinkedGLFunction glTangent3svEXT = make_glTangent3svEXT();
		public static final UnlinkedGLFunction glTangentPointerEXT = make_glTangentPointerEXT();
		public static final UnlinkedGLFunction glTbufferMask3DFX = make_glTbufferMask3DFX();
		public static final UnlinkedGLFunction glTessellationFactorAMD = make_glTessellationFactorAMD();
		public static final UnlinkedGLFunction glTessellationModeAMD = make_glTessellationModeAMD();
		public static final UnlinkedGLFunction glTestFenceAPPLE = make_glTestFenceAPPLE();
		public static final UnlinkedGLFunction glTestFenceNV = make_glTestFenceNV();
		public static final UnlinkedGLFunction glTestObjectAPPLE = make_glTestObjectAPPLE();
		public static final UnlinkedGLFunction glTexAttachMemoryNV = make_glTexAttachMemoryNV();
		public static final UnlinkedGLFunction glTexBuffer = make_glTexBuffer();
		public static final UnlinkedGLFunction glTexBufferARB = make_glTexBufferARB();
		public static final UnlinkedGLFunction glTexBufferEXT = make_glTexBufferEXT();
		public static final UnlinkedGLFunction glTexBufferOES = make_glTexBufferOES();
		public static final UnlinkedGLFunction glTexBufferRange = make_glTexBufferRange();
		public static final UnlinkedGLFunction glTexBufferRangeEXT = make_glTexBufferRangeEXT();
		public static final UnlinkedGLFunction glTexBufferRangeOES = make_glTexBufferRangeOES();
		public static final UnlinkedGLFunction glTexBumpParameterfvATI = make_glTexBumpParameterfvATI();
		public static final UnlinkedGLFunction glTexBumpParameterivATI = make_glTexBumpParameterivATI();
		public static final UnlinkedGLFunction glTexCoord1bOES = make_glTexCoord1bOES();
		public static final UnlinkedGLFunction glTexCoord1bvOES = make_glTexCoord1bvOES();
		public static final UnlinkedGLFunction glTexCoord1d = make_glTexCoord1d();
		public static final UnlinkedGLFunction glTexCoord1dv = make_glTexCoord1dv();
		public static final UnlinkedGLFunction glTexCoord1f = make_glTexCoord1f();
		public static final UnlinkedGLFunction glTexCoord1fv = make_glTexCoord1fv();
		public static final UnlinkedGLFunction glTexCoord1hNV = make_glTexCoord1hNV();
		public static final UnlinkedGLFunction glTexCoord1hvNV = make_glTexCoord1hvNV();
		public static final UnlinkedGLFunction glTexCoord1i = make_glTexCoord1i();
		public static final UnlinkedGLFunction glTexCoord1iv = make_glTexCoord1iv();
		public static final UnlinkedGLFunction glTexCoord1s = make_glTexCoord1s();
		public static final UnlinkedGLFunction glTexCoord1sv = make_glTexCoord1sv();
		public static final UnlinkedGLFunction glTexCoord1xOES = make_glTexCoord1xOES();
		public static final UnlinkedGLFunction glTexCoord1xvOES = make_glTexCoord1xvOES();
		public static final UnlinkedGLFunction glTexCoord2bOES = make_glTexCoord2bOES();
		public static final UnlinkedGLFunction glTexCoord2bvOES = make_glTexCoord2bvOES();
		public static final UnlinkedGLFunction glTexCoord2d = make_glTexCoord2d();
		public static final UnlinkedGLFunction glTexCoord2dv = make_glTexCoord2dv();
		public static final UnlinkedGLFunction glTexCoord2f = make_glTexCoord2f();
		public static final UnlinkedGLFunction glTexCoord2fColor3fVertex3fSUN = make_glTexCoord2fColor3fVertex3fSUN();
		public static final UnlinkedGLFunction glTexCoord2fColor3fVertex3fvSUN = make_glTexCoord2fColor3fVertex3fvSUN();
		public static final UnlinkedGLFunction glTexCoord2fColor4fNormal3fVertex3fSUN = make_glTexCoord2fColor4fNormal3fVertex3fSUN();
		public static final UnlinkedGLFunction glTexCoord2fColor4fNormal3fVertex3fvSUN = make_glTexCoord2fColor4fNormal3fVertex3fvSUN();
		public static final UnlinkedGLFunction glTexCoord2fColor4ubVertex3fSUN = make_glTexCoord2fColor4ubVertex3fSUN();
		public static final UnlinkedGLFunction glTexCoord2fColor4ubVertex3fvSUN = make_glTexCoord2fColor4ubVertex3fvSUN();
		public static final UnlinkedGLFunction glTexCoord2fNormal3fVertex3fSUN = make_glTexCoord2fNormal3fVertex3fSUN();
		public static final UnlinkedGLFunction glTexCoord2fNormal3fVertex3fvSUN = make_glTexCoord2fNormal3fVertex3fvSUN();
		public static final UnlinkedGLFunction glTexCoord2fVertex3fSUN = make_glTexCoord2fVertex3fSUN();
		public static final UnlinkedGLFunction glTexCoord2fVertex3fvSUN = make_glTexCoord2fVertex3fvSUN();
		public static final UnlinkedGLFunction glTexCoord2fv = make_glTexCoord2fv();
		public static final UnlinkedGLFunction glTexCoord2hNV = make_glTexCoord2hNV();
		public static final UnlinkedGLFunction glTexCoord2hvNV = make_glTexCoord2hvNV();
		public static final UnlinkedGLFunction glTexCoord2i = make_glTexCoord2i();
		public static final UnlinkedGLFunction glTexCoord2iv = make_glTexCoord2iv();
		public static final UnlinkedGLFunction glTexCoord2s = make_glTexCoord2s();
		public static final UnlinkedGLFunction glTexCoord2sv = make_glTexCoord2sv();
		public static final UnlinkedGLFunction glTexCoord2xOES = make_glTexCoord2xOES();
		public static final UnlinkedGLFunction glTexCoord2xvOES = make_glTexCoord2xvOES();
		public static final UnlinkedGLFunction glTexCoord3bOES = make_glTexCoord3bOES();
		public static final UnlinkedGLFunction glTexCoord3bvOES = make_glTexCoord3bvOES();
		public static final UnlinkedGLFunction glTexCoord3d = make_glTexCoord3d();
		public static final UnlinkedGLFunction glTexCoord3dv = make_glTexCoord3dv();
		public static final UnlinkedGLFunction glTexCoord3f = make_glTexCoord3f();
		public static final UnlinkedGLFunction glTexCoord3fv = make_glTexCoord3fv();
		public static final UnlinkedGLFunction glTexCoord3hNV = make_glTexCoord3hNV();
		public static final UnlinkedGLFunction glTexCoord3hvNV = make_glTexCoord3hvNV();
		public static final UnlinkedGLFunction glTexCoord3i = make_glTexCoord3i();
		public static final UnlinkedGLFunction glTexCoord3iv = make_glTexCoord3iv();
		public static final UnlinkedGLFunction glTexCoord3s = make_glTexCoord3s();
		public static final UnlinkedGLFunction glTexCoord3sv = make_glTexCoord3sv();
		public static final UnlinkedGLFunction glTexCoord3xOES = make_glTexCoord3xOES();
		public static final UnlinkedGLFunction glTexCoord3xvOES = make_glTexCoord3xvOES();
		public static final UnlinkedGLFunction glTexCoord4bOES = make_glTexCoord4bOES();
		public static final UnlinkedGLFunction glTexCoord4bvOES = make_glTexCoord4bvOES();
		public static final UnlinkedGLFunction glTexCoord4d = make_glTexCoord4d();
		public static final UnlinkedGLFunction glTexCoord4dv = make_glTexCoord4dv();
		public static final UnlinkedGLFunction glTexCoord4f = make_glTexCoord4f();
		public static final UnlinkedGLFunction glTexCoord4fColor4fNormal3fVertex4fSUN = make_glTexCoord4fColor4fNormal3fVertex4fSUN();
		public static final UnlinkedGLFunction glTexCoord4fColor4fNormal3fVertex4fvSUN = make_glTexCoord4fColor4fNormal3fVertex4fvSUN();
		public static final UnlinkedGLFunction glTexCoord4fVertex4fSUN = make_glTexCoord4fVertex4fSUN();
		public static final UnlinkedGLFunction glTexCoord4fVertex4fvSUN = make_glTexCoord4fVertex4fvSUN();
		public static final UnlinkedGLFunction glTexCoord4fv = make_glTexCoord4fv();
		public static final UnlinkedGLFunction glTexCoord4hNV = make_glTexCoord4hNV();
		public static final UnlinkedGLFunction glTexCoord4hvNV = make_glTexCoord4hvNV();
		public static final UnlinkedGLFunction glTexCoord4i = make_glTexCoord4i();
		public static final UnlinkedGLFunction glTexCoord4iv = make_glTexCoord4iv();
		public static final UnlinkedGLFunction glTexCoord4s = make_glTexCoord4s();
		public static final UnlinkedGLFunction glTexCoord4sv = make_glTexCoord4sv();
		public static final UnlinkedGLFunction glTexCoord4xOES = make_glTexCoord4xOES();
		public static final UnlinkedGLFunction glTexCoord4xvOES = make_glTexCoord4xvOES();
		public static final UnlinkedGLFunction glTexCoordFormatNV = make_glTexCoordFormatNV();
		public static final UnlinkedGLFunction glTexCoordP1ui = make_glTexCoordP1ui();
		public static final UnlinkedGLFunction glTexCoordP1uiv = make_glTexCoordP1uiv();
		public static final UnlinkedGLFunction glTexCoordP2ui = make_glTexCoordP2ui();
		public static final UnlinkedGLFunction glTexCoordP2uiv = make_glTexCoordP2uiv();
		public static final UnlinkedGLFunction glTexCoordP3ui = make_glTexCoordP3ui();
		public static final UnlinkedGLFunction glTexCoordP3uiv = make_glTexCoordP3uiv();
		public static final UnlinkedGLFunction glTexCoordP4ui = make_glTexCoordP4ui();
		public static final UnlinkedGLFunction glTexCoordP4uiv = make_glTexCoordP4uiv();
		public static final UnlinkedGLFunction glTexCoordPointer = make_glTexCoordPointer();
		public static final UnlinkedGLFunction glTexCoordPointerEXT = make_glTexCoordPointerEXT();
		public static final UnlinkedGLFunction glTexCoordPointerListIBM = make_glTexCoordPointerListIBM();
		public static final UnlinkedGLFunction glTexCoordPointervINTEL = make_glTexCoordPointervINTEL();
		public static final UnlinkedGLFunction glTexEnvf = make_glTexEnvf();
		public static final UnlinkedGLFunction glTexEnvfv = make_glTexEnvfv();
		public static final UnlinkedGLFunction glTexEnvi = make_glTexEnvi();
		public static final UnlinkedGLFunction glTexEnviv = make_glTexEnviv();
		public static final UnlinkedGLFunction glTexEnvx = make_glTexEnvx();
		public static final UnlinkedGLFunction glTexEnvxOES = make_glTexEnvxOES();
		public static final UnlinkedGLFunction glTexEnvxv = make_glTexEnvxv();
		public static final UnlinkedGLFunction glTexEnvxvOES = make_glTexEnvxvOES();
		public static final UnlinkedGLFunction glTexEstimateMotionQCOM = make_glTexEstimateMotionQCOM();
		public static final UnlinkedGLFunction glTexEstimateMotionRegionsQCOM = make_glTexEstimateMotionRegionsQCOM();
		public static final UnlinkedGLFunction glExtrapolateTex2DQCOM = make_glExtrapolateTex2DQCOM();
		public static final UnlinkedGLFunction glTexFilterFuncSGIS = make_glTexFilterFuncSGIS();
		public static final UnlinkedGLFunction glTexGend = make_glTexGend();
		public static final UnlinkedGLFunction glTexGendv = make_glTexGendv();
		public static final UnlinkedGLFunction glTexGenf = make_glTexGenf();
		public static final UnlinkedGLFunction glTexGenfOES = make_glTexGenfOES();
		public static final UnlinkedGLFunction glTexGenfv = make_glTexGenfv();
		public static final UnlinkedGLFunction glTexGenfvOES = make_glTexGenfvOES();
		public static final UnlinkedGLFunction glTexGeni = make_glTexGeni();
		public static final UnlinkedGLFunction glTexGeniOES = make_glTexGeniOES();
		public static final UnlinkedGLFunction glTexGeniv = make_glTexGeniv();
		public static final UnlinkedGLFunction glTexGenivOES = make_glTexGenivOES();
		public static final UnlinkedGLFunction glTexGenxOES = make_glTexGenxOES();
		public static final UnlinkedGLFunction glTexGenxvOES = make_glTexGenxvOES();
		public static final UnlinkedGLFunction glTexImage1D = make_glTexImage1D();
		public static final UnlinkedGLFunction glTexImage2D = make_glTexImage2D();
		public static final UnlinkedGLFunction glTexImage2DMultisample = make_glTexImage2DMultisample();
		public static final UnlinkedGLFunction glTexImage2DMultisampleCoverageNV = make_glTexImage2DMultisampleCoverageNV();
		public static final UnlinkedGLFunction glTexImage3D = make_glTexImage3D();
		public static final UnlinkedGLFunction glTexImage3DEXT = make_glTexImage3DEXT();
		public static final UnlinkedGLFunction glTexImage3DMultisample = make_glTexImage3DMultisample();
		public static final UnlinkedGLFunction glTexImage3DMultisampleCoverageNV = make_glTexImage3DMultisampleCoverageNV();
		public static final UnlinkedGLFunction glTexImage3DOES = make_glTexImage3DOES();
		public static final UnlinkedGLFunction glTexImage4DSGIS = make_glTexImage4DSGIS();
		public static final UnlinkedGLFunction glTexPageCommitmentARB = make_glTexPageCommitmentARB();
		public static final UnlinkedGLFunction glTexPageCommitmentEXT = make_glTexPageCommitmentEXT();
		public static final UnlinkedGLFunction glTexPageCommitmentMemNV = make_glTexPageCommitmentMemNV();
		public static final UnlinkedGLFunction glTexParameterIiv = make_glTexParameterIiv();
		public static final UnlinkedGLFunction glTexParameterIivEXT = make_glTexParameterIivEXT();
		public static final UnlinkedGLFunction glTexParameterIivOES = make_glTexParameterIivOES();
		public static final UnlinkedGLFunction glTexParameterIuiv = make_glTexParameterIuiv();
		public static final UnlinkedGLFunction glTexParameterIuivEXT = make_glTexParameterIuivEXT();
		public static final UnlinkedGLFunction glTexParameterIuivOES = make_glTexParameterIuivOES();
		public static final UnlinkedGLFunction glTexParameterf = make_glTexParameterf();
		public static final UnlinkedGLFunction glTexParameterfv = make_glTexParameterfv();
		public static final UnlinkedGLFunction glTexParameteri = make_glTexParameteri();
		public static final UnlinkedGLFunction glTexParameteriv = make_glTexParameteriv();
		public static final UnlinkedGLFunction glTexParameterx = make_glTexParameterx();
		public static final UnlinkedGLFunction glTexParameterxOES = make_glTexParameterxOES();
		public static final UnlinkedGLFunction glTexParameterxv = make_glTexParameterxv();
		public static final UnlinkedGLFunction glTexParameterxvOES = make_glTexParameterxvOES();
		public static final UnlinkedGLFunction glTexRenderbufferNV = make_glTexRenderbufferNV();
		public static final UnlinkedGLFunction glTexStorage1D = make_glTexStorage1D();
		public static final UnlinkedGLFunction glTexStorage1DEXT = make_glTexStorage1DEXT();
		public static final UnlinkedGLFunction glTexStorage2D = make_glTexStorage2D();
		public static final UnlinkedGLFunction glTexStorage2DEXT = make_glTexStorage2DEXT();
		public static final UnlinkedGLFunction glTexStorage2DMultisample = make_glTexStorage2DMultisample();
		public static final UnlinkedGLFunction glTexStorage3D = make_glTexStorage3D();
		public static final UnlinkedGLFunction glTexStorage3DEXT = make_glTexStorage3DEXT();
		public static final UnlinkedGLFunction glTexStorage3DMultisample = make_glTexStorage3DMultisample();
		public static final UnlinkedGLFunction glTexStorage3DMultisampleOES = make_glTexStorage3DMultisampleOES();
		public static final UnlinkedGLFunction glTexStorageAttribs2DEXT = make_glTexStorageAttribs2DEXT();
		public static final UnlinkedGLFunction glTexStorageAttribs3DEXT = make_glTexStorageAttribs3DEXT();
		public static final UnlinkedGLFunction glTexStorageMem1DEXT = make_glTexStorageMem1DEXT();
		public static final UnlinkedGLFunction glTexStorageMem2DEXT = make_glTexStorageMem2DEXT();
		public static final UnlinkedGLFunction glTexStorageMem2DMultisampleEXT = make_glTexStorageMem2DMultisampleEXT();
		public static final UnlinkedGLFunction glTexStorageMem3DEXT = make_glTexStorageMem3DEXT();
		public static final UnlinkedGLFunction glTexStorageMem3DMultisampleEXT = make_glTexStorageMem3DMultisampleEXT();
		public static final UnlinkedGLFunction glTexStorageSparseAMD = make_glTexStorageSparseAMD();
		public static final UnlinkedGLFunction glTexSubImage1D = make_glTexSubImage1D();
		public static final UnlinkedGLFunction glTexSubImage1DEXT = make_glTexSubImage1DEXT();
		public static final UnlinkedGLFunction glTexSubImage2D = make_glTexSubImage2D();
		public static final UnlinkedGLFunction glTexSubImage2DEXT = make_glTexSubImage2DEXT();
		public static final UnlinkedGLFunction glTexSubImage3D = make_glTexSubImage3D();
		public static final UnlinkedGLFunction glTexSubImage3DEXT = make_glTexSubImage3DEXT();
		public static final UnlinkedGLFunction glTexSubImage3DOES = make_glTexSubImage3DOES();
		public static final UnlinkedGLFunction glTexSubImage4DSGIS = make_glTexSubImage4DSGIS();
		public static final UnlinkedGLFunction glTextureAttachMemoryNV = make_glTextureAttachMemoryNV();
		public static final UnlinkedGLFunction glTextureBarrier = make_glTextureBarrier();
		public static final UnlinkedGLFunction glTextureBarrierNV = make_glTextureBarrierNV();
		public static final UnlinkedGLFunction glTextureBuffer = make_glTextureBuffer();
		public static final UnlinkedGLFunction glTextureBufferEXT = make_glTextureBufferEXT();
		public static final UnlinkedGLFunction glTextureBufferRange = make_glTextureBufferRange();
		public static final UnlinkedGLFunction glTextureBufferRangeEXT = make_glTextureBufferRangeEXT();
		public static final UnlinkedGLFunction glTextureColorMaskSGIS = make_glTextureColorMaskSGIS();
		public static final UnlinkedGLFunction glTextureFoveationParametersQCOM = make_glTextureFoveationParametersQCOM();
		public static final UnlinkedGLFunction glTextureImage1DEXT = make_glTextureImage1DEXT();
		public static final UnlinkedGLFunction glTextureImage2DEXT = make_glTextureImage2DEXT();
		public static final UnlinkedGLFunction glTextureImage2DMultisampleCoverageNV = make_glTextureImage2DMultisampleCoverageNV();
		public static final UnlinkedGLFunction glTextureImage2DMultisampleNV = make_glTextureImage2DMultisampleNV();
		public static final UnlinkedGLFunction glTextureImage3DEXT = make_glTextureImage3DEXT();
		public static final UnlinkedGLFunction glTextureImage3DMultisampleCoverageNV = make_glTextureImage3DMultisampleCoverageNV();
		public static final UnlinkedGLFunction glTextureImage3DMultisampleNV = make_glTextureImage3DMultisampleNV();
		public static final UnlinkedGLFunction glTextureLightEXT = make_glTextureLightEXT();
		public static final UnlinkedGLFunction glTextureMaterialEXT = make_glTextureMaterialEXT();
		public static final UnlinkedGLFunction glTextureNormalEXT = make_glTextureNormalEXT();
		public static final UnlinkedGLFunction glTexturePageCommitmentEXT = make_glTexturePageCommitmentEXT();
		public static final UnlinkedGLFunction glTexturePageCommitmentMemNV = make_glTexturePageCommitmentMemNV();
		public static final UnlinkedGLFunction glTextureParameterIiv = make_glTextureParameterIiv();
		public static final UnlinkedGLFunction glTextureParameterIivEXT = make_glTextureParameterIivEXT();
		public static final UnlinkedGLFunction glTextureParameterIuiv = make_glTextureParameterIuiv();
		public static final UnlinkedGLFunction glTextureParameterIuivEXT = make_glTextureParameterIuivEXT();
		public static final UnlinkedGLFunction glTextureParameterf = make_glTextureParameterf();
		public static final UnlinkedGLFunction glTextureParameterfEXT = make_glTextureParameterfEXT();
		public static final UnlinkedGLFunction glTextureParameterfv = make_glTextureParameterfv();
		public static final UnlinkedGLFunction glTextureParameterfvEXT = make_glTextureParameterfvEXT();
		public static final UnlinkedGLFunction glTextureParameteri = make_glTextureParameteri();
		public static final UnlinkedGLFunction glTextureParameteriEXT = make_glTextureParameteriEXT();
		public static final UnlinkedGLFunction glTextureParameteriv = make_glTextureParameteriv();
		public static final UnlinkedGLFunction glTextureParameterivEXT = make_glTextureParameterivEXT();
		public static final UnlinkedGLFunction glTextureRangeAPPLE = make_glTextureRangeAPPLE();
		public static final UnlinkedGLFunction glTextureRenderbufferEXT = make_glTextureRenderbufferEXT();
		public static final UnlinkedGLFunction glTextureStorage1D = make_glTextureStorage1D();
		public static final UnlinkedGLFunction glTextureStorage1DEXT = make_glTextureStorage1DEXT();
		public static final UnlinkedGLFunction glTextureStorage2D = make_glTextureStorage2D();
		public static final UnlinkedGLFunction glTextureStorage2DEXT = make_glTextureStorage2DEXT();
		public static final UnlinkedGLFunction glTextureStorage2DMultisample = make_glTextureStorage2DMultisample();
		public static final UnlinkedGLFunction glTextureStorage2DMultisampleEXT = make_glTextureStorage2DMultisampleEXT();
		public static final UnlinkedGLFunction glTextureStorage3D = make_glTextureStorage3D();
		public static final UnlinkedGLFunction glTextureStorage3DEXT = make_glTextureStorage3DEXT();
		public static final UnlinkedGLFunction glTextureStorage3DMultisample = make_glTextureStorage3DMultisample();
		public static final UnlinkedGLFunction glTextureStorage3DMultisampleEXT = make_glTextureStorage3DMultisampleEXT();
		public static final UnlinkedGLFunction glTextureStorageMem1DEXT = make_glTextureStorageMem1DEXT();
		public static final UnlinkedGLFunction glTextureStorageMem2DEXT = make_glTextureStorageMem2DEXT();
		public static final UnlinkedGLFunction glTextureStorageMem2DMultisampleEXT = make_glTextureStorageMem2DMultisampleEXT();
		public static final UnlinkedGLFunction glTextureStorageMem3DEXT = make_glTextureStorageMem3DEXT();
		public static final UnlinkedGLFunction glTextureStorageMem3DMultisampleEXT = make_glTextureStorageMem3DMultisampleEXT();
		public static final UnlinkedGLFunction glTextureStorageSparseAMD = make_glTextureStorageSparseAMD();
		public static final UnlinkedGLFunction glTextureSubImage1D = make_glTextureSubImage1D();
		public static final UnlinkedGLFunction glTextureSubImage1DEXT = make_glTextureSubImage1DEXT();
		public static final UnlinkedGLFunction glTextureSubImage2D = make_glTextureSubImage2D();
		public static final UnlinkedGLFunction glTextureSubImage2DEXT = make_glTextureSubImage2DEXT();
		public static final UnlinkedGLFunction glTextureSubImage3D = make_glTextureSubImage3D();
		public static final UnlinkedGLFunction glTextureSubImage3DEXT = make_glTextureSubImage3DEXT();
		public static final UnlinkedGLFunction glTextureView = make_glTextureView();
		public static final UnlinkedGLFunction glTextureViewEXT = make_glTextureViewEXT();
		public static final UnlinkedGLFunction glTextureViewOES = make_glTextureViewOES();
		public static final UnlinkedGLFunction glTrackMatrixNV = make_glTrackMatrixNV();
		public static final UnlinkedGLFunction glTransformFeedbackAttribsNV = make_glTransformFeedbackAttribsNV();
		public static final UnlinkedGLFunction glTransformFeedbackBufferBase = make_glTransformFeedbackBufferBase();
		public static final UnlinkedGLFunction glTransformFeedbackBufferRange = make_glTransformFeedbackBufferRange();
		public static final UnlinkedGLFunction glTransformFeedbackStreamAttribsNV = make_glTransformFeedbackStreamAttribsNV();
		public static final UnlinkedGLFunction glTransformFeedbackVaryings = make_glTransformFeedbackVaryings();
		public static final UnlinkedGLFunction glTransformFeedbackVaryingsEXT = make_glTransformFeedbackVaryingsEXT();
		public static final UnlinkedGLFunction glTransformFeedbackVaryingsNV = make_glTransformFeedbackVaryingsNV();
		public static final UnlinkedGLFunction glTransformPathNV = make_glTransformPathNV();
		public static final UnlinkedGLFunction glTranslated = make_glTranslated();
		public static final UnlinkedGLFunction glTranslatef = make_glTranslatef();
		public static final UnlinkedGLFunction glTranslatex = make_glTranslatex();
		public static final UnlinkedGLFunction glTranslatexOES = make_glTranslatexOES();
		public static final UnlinkedGLFunction glUniform1d = make_glUniform1d();
		public static final UnlinkedGLFunction glUniform1dv = make_glUniform1dv();
		public static final UnlinkedGLFunction glUniform1f = make_glUniform1f();
		public static final UnlinkedGLFunction glUniform1fARB = make_glUniform1fARB();
		public static final UnlinkedGLFunction glUniform1fv = make_glUniform1fv();
		public static final UnlinkedGLFunction glUniform1fvARB = make_glUniform1fvARB();
		public static final UnlinkedGLFunction glUniform1i = make_glUniform1i();
		public static final UnlinkedGLFunction glUniform1i64ARB = make_glUniform1i64ARB();
		public static final UnlinkedGLFunction glUniform1i64NV = make_glUniform1i64NV();
		public static final UnlinkedGLFunction glUniform1i64vARB = make_glUniform1i64vARB();
		public static final UnlinkedGLFunction glUniform1i64vNV = make_glUniform1i64vNV();
		public static final UnlinkedGLFunction glUniform1iARB = make_glUniform1iARB();
		public static final UnlinkedGLFunction glUniform1iv = make_glUniform1iv();
		public static final UnlinkedGLFunction glUniform1ivARB = make_glUniform1ivARB();
		public static final UnlinkedGLFunction glUniform1ui = make_glUniform1ui();
		public static final UnlinkedGLFunction glUniform1ui64ARB = make_glUniform1ui64ARB();
		public static final UnlinkedGLFunction glUniform1ui64NV = make_glUniform1ui64NV();
		public static final UnlinkedGLFunction glUniform1ui64vARB = make_glUniform1ui64vARB();
		public static final UnlinkedGLFunction glUniform1ui64vNV = make_glUniform1ui64vNV();
		public static final UnlinkedGLFunction glUniform1uiEXT = make_glUniform1uiEXT();
		public static final UnlinkedGLFunction glUniform1uiv = make_glUniform1uiv();
		public static final UnlinkedGLFunction glUniform1uivEXT = make_glUniform1uivEXT();
		public static final UnlinkedGLFunction glUniform2d = make_glUniform2d();
		public static final UnlinkedGLFunction glUniform2dv = make_glUniform2dv();
		public static final UnlinkedGLFunction glUniform2f = make_glUniform2f();
		public static final UnlinkedGLFunction glUniform2fARB = make_glUniform2fARB();
		public static final UnlinkedGLFunction glUniform2fv = make_glUniform2fv();
		public static final UnlinkedGLFunction glUniform2fvARB = make_glUniform2fvARB();
		public static final UnlinkedGLFunction glUniform2i = make_glUniform2i();
		public static final UnlinkedGLFunction glUniform2i64ARB = make_glUniform2i64ARB();
		public static final UnlinkedGLFunction glUniform2i64NV = make_glUniform2i64NV();
		public static final UnlinkedGLFunction glUniform2i64vARB = make_glUniform2i64vARB();
		public static final UnlinkedGLFunction glUniform2i64vNV = make_glUniform2i64vNV();
		public static final UnlinkedGLFunction glUniform2iARB = make_glUniform2iARB();
		public static final UnlinkedGLFunction glUniform2iv = make_glUniform2iv();
		public static final UnlinkedGLFunction glUniform2ivARB = make_glUniform2ivARB();
		public static final UnlinkedGLFunction glUniform2ui = make_glUniform2ui();
		public static final UnlinkedGLFunction glUniform2ui64ARB = make_glUniform2ui64ARB();
		public static final UnlinkedGLFunction glUniform2ui64NV = make_glUniform2ui64NV();
		public static final UnlinkedGLFunction glUniform2ui64vARB = make_glUniform2ui64vARB();
		public static final UnlinkedGLFunction glUniform2ui64vNV = make_glUniform2ui64vNV();
		public static final UnlinkedGLFunction glUniform2uiEXT = make_glUniform2uiEXT();
		public static final UnlinkedGLFunction glUniform2uiv = make_glUniform2uiv();
		public static final UnlinkedGLFunction glUniform2uivEXT = make_glUniform2uivEXT();
		public static final UnlinkedGLFunction glUniform3d = make_glUniform3d();
		public static final UnlinkedGLFunction glUniform3dv = make_glUniform3dv();
		public static final UnlinkedGLFunction glUniform3f = make_glUniform3f();
		public static final UnlinkedGLFunction glUniform3fARB = make_glUniform3fARB();
		public static final UnlinkedGLFunction glUniform3fv = make_glUniform3fv();
		public static final UnlinkedGLFunction glUniform3fvARB = make_glUniform3fvARB();
		public static final UnlinkedGLFunction glUniform3i = make_glUniform3i();
		public static final UnlinkedGLFunction glUniform3i64ARB = make_glUniform3i64ARB();
		public static final UnlinkedGLFunction glUniform3i64NV = make_glUniform3i64NV();
		public static final UnlinkedGLFunction glUniform3i64vARB = make_glUniform3i64vARB();
		public static final UnlinkedGLFunction glUniform3i64vNV = make_glUniform3i64vNV();
		public static final UnlinkedGLFunction glUniform3iARB = make_glUniform3iARB();
		public static final UnlinkedGLFunction glUniform3iv = make_glUniform3iv();
		public static final UnlinkedGLFunction glUniform3ivARB = make_glUniform3ivARB();
		public static final UnlinkedGLFunction glUniform3ui = make_glUniform3ui();
		public static final UnlinkedGLFunction glUniform3ui64ARB = make_glUniform3ui64ARB();
		public static final UnlinkedGLFunction glUniform3ui64NV = make_glUniform3ui64NV();
		public static final UnlinkedGLFunction glUniform3ui64vARB = make_glUniform3ui64vARB();
		public static final UnlinkedGLFunction glUniform3ui64vNV = make_glUniform3ui64vNV();
		public static final UnlinkedGLFunction glUniform3uiEXT = make_glUniform3uiEXT();
		public static final UnlinkedGLFunction glUniform3uiv = make_glUniform3uiv();
		public static final UnlinkedGLFunction glUniform3uivEXT = make_glUniform3uivEXT();
		public static final UnlinkedGLFunction glUniform4d = make_glUniform4d();
		public static final UnlinkedGLFunction glUniform4dv = make_glUniform4dv();
		public static final UnlinkedGLFunction glUniform4f = make_glUniform4f();
		public static final UnlinkedGLFunction glUniform4fARB = make_glUniform4fARB();
		public static final UnlinkedGLFunction glUniform4fv = make_glUniform4fv();
		public static final UnlinkedGLFunction glUniform4fvARB = make_glUniform4fvARB();
		public static final UnlinkedGLFunction glUniform4i = make_glUniform4i();
		public static final UnlinkedGLFunction glUniform4i64ARB = make_glUniform4i64ARB();
		public static final UnlinkedGLFunction glUniform4i64NV = make_glUniform4i64NV();
		public static final UnlinkedGLFunction glUniform4i64vARB = make_glUniform4i64vARB();
		public static final UnlinkedGLFunction glUniform4i64vNV = make_glUniform4i64vNV();
		public static final UnlinkedGLFunction glUniform4iARB = make_glUniform4iARB();
		public static final UnlinkedGLFunction glUniform4iv = make_glUniform4iv();
		public static final UnlinkedGLFunction glUniform4ivARB = make_glUniform4ivARB();
		public static final UnlinkedGLFunction glUniform4ui = make_glUniform4ui();
		public static final UnlinkedGLFunction glUniform4ui64ARB = make_glUniform4ui64ARB();
		public static final UnlinkedGLFunction glUniform4ui64NV = make_glUniform4ui64NV();
		public static final UnlinkedGLFunction glUniform4ui64vARB = make_glUniform4ui64vARB();
		public static final UnlinkedGLFunction glUniform4ui64vNV = make_glUniform4ui64vNV();
		public static final UnlinkedGLFunction glUniform4uiEXT = make_glUniform4uiEXT();
		public static final UnlinkedGLFunction glUniform4uiv = make_glUniform4uiv();
		public static final UnlinkedGLFunction glUniform4uivEXT = make_glUniform4uivEXT();
		public static final UnlinkedGLFunction glUniformBlockBinding = make_glUniformBlockBinding();
		public static final UnlinkedGLFunction glUniformBufferEXT = make_glUniformBufferEXT();
		public static final UnlinkedGLFunction glUniformHandleui64ARB = make_glUniformHandleui64ARB();
		public static final UnlinkedGLFunction glUniformHandleui64IMG = make_glUniformHandleui64IMG();
		public static final UnlinkedGLFunction glUniformHandleui64NV = make_glUniformHandleui64NV();
		public static final UnlinkedGLFunction glUniformHandleui64vARB = make_glUniformHandleui64vARB();
		public static final UnlinkedGLFunction glUniformHandleui64vIMG = make_glUniformHandleui64vIMG();
		public static final UnlinkedGLFunction glUniformHandleui64vNV = make_glUniformHandleui64vNV();
		public static final UnlinkedGLFunction glUniformMatrix2dv = make_glUniformMatrix2dv();
		public static final UnlinkedGLFunction glUniformMatrix2fv = make_glUniformMatrix2fv();
		public static final UnlinkedGLFunction glUniformMatrix2fvARB = make_glUniformMatrix2fvARB();
		public static final UnlinkedGLFunction glUniformMatrix2x3dv = make_glUniformMatrix2x3dv();
		public static final UnlinkedGLFunction glUniformMatrix2x3fv = make_glUniformMatrix2x3fv();
		public static final UnlinkedGLFunction glUniformMatrix2x3fvNV = make_glUniformMatrix2x3fvNV();
		public static final UnlinkedGLFunction glUniformMatrix2x4dv = make_glUniformMatrix2x4dv();
		public static final UnlinkedGLFunction glUniformMatrix2x4fv = make_glUniformMatrix2x4fv();
		public static final UnlinkedGLFunction glUniformMatrix2x4fvNV = make_glUniformMatrix2x4fvNV();
		public static final UnlinkedGLFunction glUniformMatrix3dv = make_glUniformMatrix3dv();
		public static final UnlinkedGLFunction glUniformMatrix3fv = make_glUniformMatrix3fv();
		public static final UnlinkedGLFunction glUniformMatrix3fvARB = make_glUniformMatrix3fvARB();
		public static final UnlinkedGLFunction glUniformMatrix3x2dv = make_glUniformMatrix3x2dv();
		public static final UnlinkedGLFunction glUniformMatrix3x2fv = make_glUniformMatrix3x2fv();
		public static final UnlinkedGLFunction glUniformMatrix3x2fvNV = make_glUniformMatrix3x2fvNV();
		public static final UnlinkedGLFunction glUniformMatrix3x4dv = make_glUniformMatrix3x4dv();
		public static final UnlinkedGLFunction glUniformMatrix3x4fv = make_glUniformMatrix3x4fv();
		public static final UnlinkedGLFunction glUniformMatrix3x4fvNV = make_glUniformMatrix3x4fvNV();
		public static final UnlinkedGLFunction glUniformMatrix4dv = make_glUniformMatrix4dv();
		public static final UnlinkedGLFunction glUniformMatrix4fv = make_glUniformMatrix4fv();
		public static final UnlinkedGLFunction glUniformMatrix4fvARB = make_glUniformMatrix4fvARB();
		public static final UnlinkedGLFunction glUniformMatrix4x2dv = make_glUniformMatrix4x2dv();
		public static final UnlinkedGLFunction glUniformMatrix4x2fv = make_glUniformMatrix4x2fv();
		public static final UnlinkedGLFunction glUniformMatrix4x2fvNV = make_glUniformMatrix4x2fvNV();
		public static final UnlinkedGLFunction glUniformMatrix4x3dv = make_glUniformMatrix4x3dv();
		public static final UnlinkedGLFunction glUniformMatrix4x3fv = make_glUniformMatrix4x3fv();
		public static final UnlinkedGLFunction glUniformMatrix4x3fvNV = make_glUniformMatrix4x3fvNV();
		public static final UnlinkedGLFunction glUniformSubroutinesuiv = make_glUniformSubroutinesuiv();
		public static final UnlinkedGLFunction glUniformui64NV = make_glUniformui64NV();
		public static final UnlinkedGLFunction glUniformui64vNV = make_glUniformui64vNV();
		public static final UnlinkedGLFunction glUnlockArraysEXT = make_glUnlockArraysEXT();
		public static final UnlinkedGLFunction glUnmapBuffer = make_glUnmapBuffer();
		public static final UnlinkedGLFunction glUnmapBufferARB = make_glUnmapBufferARB();
		public static final UnlinkedGLFunction glUnmapBufferOES = make_glUnmapBufferOES();
		public static final UnlinkedGLFunction glUnmapNamedBuffer = make_glUnmapNamedBuffer();
		public static final UnlinkedGLFunction glUnmapNamedBufferEXT = make_glUnmapNamedBufferEXT();
		public static final UnlinkedGLFunction glUnmapObjectBufferATI = make_glUnmapObjectBufferATI();
		public static final UnlinkedGLFunction glUnmapTexture2DINTEL = make_glUnmapTexture2DINTEL();
		public static final UnlinkedGLFunction glUpdateObjectBufferATI = make_glUpdateObjectBufferATI();
		public static final UnlinkedGLFunction glUploadGpuMaskNVX = make_glUploadGpuMaskNVX();
		public static final UnlinkedGLFunction glUseProgram = make_glUseProgram();
		public static final UnlinkedGLFunction glUseProgramObjectARB = make_glUseProgramObjectARB();
		public static final UnlinkedGLFunction glUseProgramStages = make_glUseProgramStages();
		public static final UnlinkedGLFunction glUseProgramStagesEXT = make_glUseProgramStagesEXT();
		public static final UnlinkedGLFunction glUseShaderProgramEXT = make_glUseShaderProgramEXT();
		public static final UnlinkedGLFunction glVDPAUFiniNV = make_glVDPAUFiniNV();
		public static final UnlinkedGLFunction glVDPAUGetSurfaceivNV = make_glVDPAUGetSurfaceivNV();
		public static final UnlinkedGLFunction glVDPAUInitNV = make_glVDPAUInitNV();
		public static final UnlinkedGLFunction glVDPAUIsSurfaceNV = make_glVDPAUIsSurfaceNV();
		public static final UnlinkedGLFunction glVDPAUMapSurfacesNV = make_glVDPAUMapSurfacesNV();
		public static final UnlinkedGLFunction glVDPAURegisterOutputSurfaceNV = make_glVDPAURegisterOutputSurfaceNV();
		public static final UnlinkedGLFunction glVDPAURegisterVideoSurfaceNV = make_glVDPAURegisterVideoSurfaceNV();
		public static final UnlinkedGLFunction glVDPAURegisterVideoSurfaceWithPictureStructureNV = make_glVDPAURegisterVideoSurfaceWithPictureStructureNV();
		public static final UnlinkedGLFunction glVDPAUSurfaceAccessNV = make_glVDPAUSurfaceAccessNV();
		public static final UnlinkedGLFunction glVDPAUUnmapSurfacesNV = make_glVDPAUUnmapSurfacesNV();
		public static final UnlinkedGLFunction glVDPAUUnregisterSurfaceNV = make_glVDPAUUnregisterSurfaceNV();
		public static final UnlinkedGLFunction glValidateProgram = make_glValidateProgram();
		public static final UnlinkedGLFunction glValidateProgramARB = make_glValidateProgramARB();
		public static final UnlinkedGLFunction glValidateProgramPipeline = make_glValidateProgramPipeline();
		public static final UnlinkedGLFunction glValidateProgramPipelineEXT = make_glValidateProgramPipelineEXT();
		public static final UnlinkedGLFunction glVariantArrayObjectATI = make_glVariantArrayObjectATI();
		public static final UnlinkedGLFunction glVariantPointerEXT = make_glVariantPointerEXT();
		public static final UnlinkedGLFunction glVariantbvEXT = make_glVariantbvEXT();
		public static final UnlinkedGLFunction glVariantdvEXT = make_glVariantdvEXT();
		public static final UnlinkedGLFunction glVariantfvEXT = make_glVariantfvEXT();
		public static final UnlinkedGLFunction glVariantivEXT = make_glVariantivEXT();
		public static final UnlinkedGLFunction glVariantsvEXT = make_glVariantsvEXT();
		public static final UnlinkedGLFunction glVariantubvEXT = make_glVariantubvEXT();
		public static final UnlinkedGLFunction glVariantuivEXT = make_glVariantuivEXT();
		public static final UnlinkedGLFunction glVariantusvEXT = make_glVariantusvEXT();
		public static final UnlinkedGLFunction glVertex2bOES = make_glVertex2bOES();
		public static final UnlinkedGLFunction glVertex2bvOES = make_glVertex2bvOES();
		public static final UnlinkedGLFunction glVertex2d = make_glVertex2d();
		public static final UnlinkedGLFunction glVertex2dv = make_glVertex2dv();
		public static final UnlinkedGLFunction glVertex2f = make_glVertex2f();
		public static final UnlinkedGLFunction glVertex2fv = make_glVertex2fv();
		public static final UnlinkedGLFunction glVertex2hNV = make_glVertex2hNV();
		public static final UnlinkedGLFunction glVertex2hvNV = make_glVertex2hvNV();
		public static final UnlinkedGLFunction glVertex2i = make_glVertex2i();
		public static final UnlinkedGLFunction glVertex2iv = make_glVertex2iv();
		public static final UnlinkedGLFunction glVertex2s = make_glVertex2s();
		public static final UnlinkedGLFunction glVertex2sv = make_glVertex2sv();
		public static final UnlinkedGLFunction glVertex2xOES = make_glVertex2xOES();
		public static final UnlinkedGLFunction glVertex2xvOES = make_glVertex2xvOES();
		public static final UnlinkedGLFunction glVertex3bOES = make_glVertex3bOES();
		public static final UnlinkedGLFunction glVertex3bvOES = make_glVertex3bvOES();
		public static final UnlinkedGLFunction glVertex3d = make_glVertex3d();
		public static final UnlinkedGLFunction glVertex3dv = make_glVertex3dv();
		public static final UnlinkedGLFunction glVertex3f = make_glVertex3f();
		public static final UnlinkedGLFunction glVertex3fv = make_glVertex3fv();
		public static final UnlinkedGLFunction glVertex3hNV = make_glVertex3hNV();
		public static final UnlinkedGLFunction glVertex3hvNV = make_glVertex3hvNV();
		public static final UnlinkedGLFunction glVertex3i = make_glVertex3i();
		public static final UnlinkedGLFunction glVertex3iv = make_glVertex3iv();
		public static final UnlinkedGLFunction glVertex3s = make_glVertex3s();
		public static final UnlinkedGLFunction glVertex3sv = make_glVertex3sv();
		public static final UnlinkedGLFunction glVertex3xOES = make_glVertex3xOES();
		public static final UnlinkedGLFunction glVertex3xvOES = make_glVertex3xvOES();
		public static final UnlinkedGLFunction glVertex4bOES = make_glVertex4bOES();
		public static final UnlinkedGLFunction glVertex4bvOES = make_glVertex4bvOES();
		public static final UnlinkedGLFunction glVertex4d = make_glVertex4d();
		public static final UnlinkedGLFunction glVertex4dv = make_glVertex4dv();
		public static final UnlinkedGLFunction glVertex4f = make_glVertex4f();
		public static final UnlinkedGLFunction glVertex4fv = make_glVertex4fv();
		public static final UnlinkedGLFunction glVertex4hNV = make_glVertex4hNV();
		public static final UnlinkedGLFunction glVertex4hvNV = make_glVertex4hvNV();
		public static final UnlinkedGLFunction glVertex4i = make_glVertex4i();
		public static final UnlinkedGLFunction glVertex4iv = make_glVertex4iv();
		public static final UnlinkedGLFunction glVertex4s = make_glVertex4s();
		public static final UnlinkedGLFunction glVertex4sv = make_glVertex4sv();
		public static final UnlinkedGLFunction glVertex4xOES = make_glVertex4xOES();
		public static final UnlinkedGLFunction glVertex4xvOES = make_glVertex4xvOES();
		public static final UnlinkedGLFunction glVertexArrayAttribBinding = make_glVertexArrayAttribBinding();
		public static final UnlinkedGLFunction glVertexArrayAttribFormat = make_glVertexArrayAttribFormat();
		public static final UnlinkedGLFunction glVertexArrayAttribIFormat = make_glVertexArrayAttribIFormat();
		public static final UnlinkedGLFunction glVertexArrayAttribLFormat = make_glVertexArrayAttribLFormat();
		public static final UnlinkedGLFunction glVertexArrayBindVertexBufferEXT = make_glVertexArrayBindVertexBufferEXT();
		public static final UnlinkedGLFunction glVertexArrayBindingDivisor = make_glVertexArrayBindingDivisor();
		public static final UnlinkedGLFunction glVertexArrayColorOffsetEXT = make_glVertexArrayColorOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayEdgeFlagOffsetEXT = make_glVertexArrayEdgeFlagOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayElementBuffer = make_glVertexArrayElementBuffer();
		public static final UnlinkedGLFunction glVertexArrayFogCoordOffsetEXT = make_glVertexArrayFogCoordOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayIndexOffsetEXT = make_glVertexArrayIndexOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayMultiTexCoordOffsetEXT = make_glVertexArrayMultiTexCoordOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayNormalOffsetEXT = make_glVertexArrayNormalOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayParameteriAPPLE = make_glVertexArrayParameteriAPPLE();
		public static final UnlinkedGLFunction glVertexArrayRangeAPPLE = make_glVertexArrayRangeAPPLE();
		public static final UnlinkedGLFunction glVertexArrayRangeNV = make_glVertexArrayRangeNV();
		public static final UnlinkedGLFunction glVertexArraySecondaryColorOffsetEXT = make_glVertexArraySecondaryColorOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayTexCoordOffsetEXT = make_glVertexArrayTexCoordOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayVertexAttribBindingEXT = make_glVertexArrayVertexAttribBindingEXT();
		public static final UnlinkedGLFunction glVertexArrayVertexAttribDivisorEXT = make_glVertexArrayVertexAttribDivisorEXT();
		public static final UnlinkedGLFunction glVertexArrayVertexAttribFormatEXT = make_glVertexArrayVertexAttribFormatEXT();
		public static final UnlinkedGLFunction glVertexArrayVertexAttribIFormatEXT = make_glVertexArrayVertexAttribIFormatEXT();
		public static final UnlinkedGLFunction glVertexArrayVertexAttribIOffsetEXT = make_glVertexArrayVertexAttribIOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayVertexAttribLFormatEXT = make_glVertexArrayVertexAttribLFormatEXT();
		public static final UnlinkedGLFunction glVertexArrayVertexAttribLOffsetEXT = make_glVertexArrayVertexAttribLOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayVertexAttribOffsetEXT = make_glVertexArrayVertexAttribOffsetEXT();
		public static final UnlinkedGLFunction glVertexArrayVertexBindingDivisorEXT = make_glVertexArrayVertexBindingDivisorEXT();
		public static final UnlinkedGLFunction glVertexArrayVertexBuffer = make_glVertexArrayVertexBuffer();
		public static final UnlinkedGLFunction glVertexArrayVertexBuffers = make_glVertexArrayVertexBuffers();
		public static final UnlinkedGLFunction glVertexArrayVertexOffsetEXT = make_glVertexArrayVertexOffsetEXT();
		public static final UnlinkedGLFunction glVertexAttrib1d = make_glVertexAttrib1d();
		public static final UnlinkedGLFunction glVertexAttrib1dARB = make_glVertexAttrib1dARB();
		public static final UnlinkedGLFunction glVertexAttrib1dNV = make_glVertexAttrib1dNV();
		public static final UnlinkedGLFunction glVertexAttrib1dv = make_glVertexAttrib1dv();
		public static final UnlinkedGLFunction glVertexAttrib1dvARB = make_glVertexAttrib1dvARB();
		public static final UnlinkedGLFunction glVertexAttrib1dvNV = make_glVertexAttrib1dvNV();
		public static final UnlinkedGLFunction glVertexAttrib1f = make_glVertexAttrib1f();
		public static final UnlinkedGLFunction glVertexAttrib1fARB = make_glVertexAttrib1fARB();
		public static final UnlinkedGLFunction glVertexAttrib1fNV = make_glVertexAttrib1fNV();
		public static final UnlinkedGLFunction glVertexAttrib1fv = make_glVertexAttrib1fv();
		public static final UnlinkedGLFunction glVertexAttrib1fvARB = make_glVertexAttrib1fvARB();
		public static final UnlinkedGLFunction glVertexAttrib1fvNV = make_glVertexAttrib1fvNV();
		public static final UnlinkedGLFunction glVertexAttrib1hNV = make_glVertexAttrib1hNV();
		public static final UnlinkedGLFunction glVertexAttrib1hvNV = make_glVertexAttrib1hvNV();
		public static final UnlinkedGLFunction glVertexAttrib1s = make_glVertexAttrib1s();
		public static final UnlinkedGLFunction glVertexAttrib1sARB = make_glVertexAttrib1sARB();
		public static final UnlinkedGLFunction glVertexAttrib1sNV = make_glVertexAttrib1sNV();
		public static final UnlinkedGLFunction glVertexAttrib1sv = make_glVertexAttrib1sv();
		public static final UnlinkedGLFunction glVertexAttrib1svARB = make_glVertexAttrib1svARB();
		public static final UnlinkedGLFunction glVertexAttrib1svNV = make_glVertexAttrib1svNV();
		public static final UnlinkedGLFunction glVertexAttrib2d = make_glVertexAttrib2d();
		public static final UnlinkedGLFunction glVertexAttrib2dARB = make_glVertexAttrib2dARB();
		public static final UnlinkedGLFunction glVertexAttrib2dNV = make_glVertexAttrib2dNV();
		public static final UnlinkedGLFunction glVertexAttrib2dv = make_glVertexAttrib2dv();
		public static final UnlinkedGLFunction glVertexAttrib2dvARB = make_glVertexAttrib2dvARB();
		public static final UnlinkedGLFunction glVertexAttrib2dvNV = make_glVertexAttrib2dvNV();
		public static final UnlinkedGLFunction glVertexAttrib2f = make_glVertexAttrib2f();
		public static final UnlinkedGLFunction glVertexAttrib2fARB = make_glVertexAttrib2fARB();
		public static final UnlinkedGLFunction glVertexAttrib2fNV = make_glVertexAttrib2fNV();
		public static final UnlinkedGLFunction glVertexAttrib2fv = make_glVertexAttrib2fv();
		public static final UnlinkedGLFunction glVertexAttrib2fvARB = make_glVertexAttrib2fvARB();
		public static final UnlinkedGLFunction glVertexAttrib2fvNV = make_glVertexAttrib2fvNV();
		public static final UnlinkedGLFunction glVertexAttrib2hNV = make_glVertexAttrib2hNV();
		public static final UnlinkedGLFunction glVertexAttrib2hvNV = make_glVertexAttrib2hvNV();
		public static final UnlinkedGLFunction glVertexAttrib2s = make_glVertexAttrib2s();
		public static final UnlinkedGLFunction glVertexAttrib2sARB = make_glVertexAttrib2sARB();
		public static final UnlinkedGLFunction glVertexAttrib2sNV = make_glVertexAttrib2sNV();
		public static final UnlinkedGLFunction glVertexAttrib2sv = make_glVertexAttrib2sv();
		public static final UnlinkedGLFunction glVertexAttrib2svARB = make_glVertexAttrib2svARB();
		public static final UnlinkedGLFunction glVertexAttrib2svNV = make_glVertexAttrib2svNV();
		public static final UnlinkedGLFunction glVertexAttrib3d = make_glVertexAttrib3d();
		public static final UnlinkedGLFunction glVertexAttrib3dARB = make_glVertexAttrib3dARB();
		public static final UnlinkedGLFunction glVertexAttrib3dNV = make_glVertexAttrib3dNV();
		public static final UnlinkedGLFunction glVertexAttrib3dv = make_glVertexAttrib3dv();
		public static final UnlinkedGLFunction glVertexAttrib3dvARB = make_glVertexAttrib3dvARB();
		public static final UnlinkedGLFunction glVertexAttrib3dvNV = make_glVertexAttrib3dvNV();
		public static final UnlinkedGLFunction glVertexAttrib3f = make_glVertexAttrib3f();
		public static final UnlinkedGLFunction glVertexAttrib3fARB = make_glVertexAttrib3fARB();
		public static final UnlinkedGLFunction glVertexAttrib3fNV = make_glVertexAttrib3fNV();
		public static final UnlinkedGLFunction glVertexAttrib3fv = make_glVertexAttrib3fv();
		public static final UnlinkedGLFunction glVertexAttrib3fvARB = make_glVertexAttrib3fvARB();
		public static final UnlinkedGLFunction glVertexAttrib3fvNV = make_glVertexAttrib3fvNV();
		public static final UnlinkedGLFunction glVertexAttrib3hNV = make_glVertexAttrib3hNV();
		public static final UnlinkedGLFunction glVertexAttrib3hvNV = make_glVertexAttrib3hvNV();
		public static final UnlinkedGLFunction glVertexAttrib3s = make_glVertexAttrib3s();
		public static final UnlinkedGLFunction glVertexAttrib3sARB = make_glVertexAttrib3sARB();
		public static final UnlinkedGLFunction glVertexAttrib3sNV = make_glVertexAttrib3sNV();
		public static final UnlinkedGLFunction glVertexAttrib3sv = make_glVertexAttrib3sv();
		public static final UnlinkedGLFunction glVertexAttrib3svARB = make_glVertexAttrib3svARB();
		public static final UnlinkedGLFunction glVertexAttrib3svNV = make_glVertexAttrib3svNV();
		public static final UnlinkedGLFunction glVertexAttrib4Nbv = make_glVertexAttrib4Nbv();
		public static final UnlinkedGLFunction glVertexAttrib4NbvARB = make_glVertexAttrib4NbvARB();
		public static final UnlinkedGLFunction glVertexAttrib4Niv = make_glVertexAttrib4Niv();
		public static final UnlinkedGLFunction glVertexAttrib4NivARB = make_glVertexAttrib4NivARB();
		public static final UnlinkedGLFunction glVertexAttrib4Nsv = make_glVertexAttrib4Nsv();
		public static final UnlinkedGLFunction glVertexAttrib4NsvARB = make_glVertexAttrib4NsvARB();
		public static final UnlinkedGLFunction glVertexAttrib4Nub = make_glVertexAttrib4Nub();
		public static final UnlinkedGLFunction glVertexAttrib4NubARB = make_glVertexAttrib4NubARB();
		public static final UnlinkedGLFunction glVertexAttrib4Nubv = make_glVertexAttrib4Nubv();
		public static final UnlinkedGLFunction glVertexAttrib4NubvARB = make_glVertexAttrib4NubvARB();
		public static final UnlinkedGLFunction glVertexAttrib4Nuiv = make_glVertexAttrib4Nuiv();
		public static final UnlinkedGLFunction glVertexAttrib4NuivARB = make_glVertexAttrib4NuivARB();
		public static final UnlinkedGLFunction glVertexAttrib4Nusv = make_glVertexAttrib4Nusv();
		public static final UnlinkedGLFunction glVertexAttrib4NusvARB = make_glVertexAttrib4NusvARB();
		public static final UnlinkedGLFunction glVertexAttrib4bv = make_glVertexAttrib4bv();
		public static final UnlinkedGLFunction glVertexAttrib4bvARB = make_glVertexAttrib4bvARB();
		public static final UnlinkedGLFunction glVertexAttrib4d = make_glVertexAttrib4d();
		public static final UnlinkedGLFunction glVertexAttrib4dARB = make_glVertexAttrib4dARB();
		public static final UnlinkedGLFunction glVertexAttrib4dNV = make_glVertexAttrib4dNV();
		public static final UnlinkedGLFunction glVertexAttrib4dv = make_glVertexAttrib4dv();
		public static final UnlinkedGLFunction glVertexAttrib4dvARB = make_glVertexAttrib4dvARB();
		public static final UnlinkedGLFunction glVertexAttrib4dvNV = make_glVertexAttrib4dvNV();
		public static final UnlinkedGLFunction glVertexAttrib4f = make_glVertexAttrib4f();
		public static final UnlinkedGLFunction glVertexAttrib4fARB = make_glVertexAttrib4fARB();
		public static final UnlinkedGLFunction glVertexAttrib4fNV = make_glVertexAttrib4fNV();
		public static final UnlinkedGLFunction glVertexAttrib4fv = make_glVertexAttrib4fv();
		public static final UnlinkedGLFunction glVertexAttrib4fvARB = make_glVertexAttrib4fvARB();
		public static final UnlinkedGLFunction glVertexAttrib4fvNV = make_glVertexAttrib4fvNV();
		public static final UnlinkedGLFunction glVertexAttrib4hNV = make_glVertexAttrib4hNV();
		public static final UnlinkedGLFunction glVertexAttrib4hvNV = make_glVertexAttrib4hvNV();
		public static final UnlinkedGLFunction glVertexAttrib4iv = make_glVertexAttrib4iv();
		public static final UnlinkedGLFunction glVertexAttrib4ivARB = make_glVertexAttrib4ivARB();
		public static final UnlinkedGLFunction glVertexAttrib4s = make_glVertexAttrib4s();
		public static final UnlinkedGLFunction glVertexAttrib4sARB = make_glVertexAttrib4sARB();
		public static final UnlinkedGLFunction glVertexAttrib4sNV = make_glVertexAttrib4sNV();
		public static final UnlinkedGLFunction glVertexAttrib4sv = make_glVertexAttrib4sv();
		public static final UnlinkedGLFunction glVertexAttrib4svARB = make_glVertexAttrib4svARB();
		public static final UnlinkedGLFunction glVertexAttrib4svNV = make_glVertexAttrib4svNV();
		public static final UnlinkedGLFunction glVertexAttrib4ubNV = make_glVertexAttrib4ubNV();
		public static final UnlinkedGLFunction glVertexAttrib4ubv = make_glVertexAttrib4ubv();
		public static final UnlinkedGLFunction glVertexAttrib4ubvARB = make_glVertexAttrib4ubvARB();
		public static final UnlinkedGLFunction glVertexAttrib4ubvNV = make_glVertexAttrib4ubvNV();
		public static final UnlinkedGLFunction glVertexAttrib4uiv = make_glVertexAttrib4uiv();
		public static final UnlinkedGLFunction glVertexAttrib4uivARB = make_glVertexAttrib4uivARB();
		public static final UnlinkedGLFunction glVertexAttrib4usv = make_glVertexAttrib4usv();
		public static final UnlinkedGLFunction glVertexAttrib4usvARB = make_glVertexAttrib4usvARB();
		public static final UnlinkedGLFunction glVertexAttribArrayObjectATI = make_glVertexAttribArrayObjectATI();
		public static final UnlinkedGLFunction glVertexAttribBinding = make_glVertexAttribBinding();
		public static final UnlinkedGLFunction glVertexAttribDivisor = make_glVertexAttribDivisor();
		public static final UnlinkedGLFunction glVertexAttribDivisorANGLE = make_glVertexAttribDivisorANGLE();
		public static final UnlinkedGLFunction glVertexAttribDivisorARB = make_glVertexAttribDivisorARB();
		public static final UnlinkedGLFunction glVertexAttribDivisorEXT = make_glVertexAttribDivisorEXT();
		public static final UnlinkedGLFunction glVertexAttribDivisorNV = make_glVertexAttribDivisorNV();
		public static final UnlinkedGLFunction glVertexAttribFormat = make_glVertexAttribFormat();
		public static final UnlinkedGLFunction glVertexAttribFormatNV = make_glVertexAttribFormatNV();
		public static final UnlinkedGLFunction glVertexAttribI1i = make_glVertexAttribI1i();
		public static final UnlinkedGLFunction glVertexAttribI1iEXT = make_glVertexAttribI1iEXT();
		public static final UnlinkedGLFunction glVertexAttribI1iv = make_glVertexAttribI1iv();
		public static final UnlinkedGLFunction glVertexAttribI1ivEXT = make_glVertexAttribI1ivEXT();
		public static final UnlinkedGLFunction glVertexAttribI1ui = make_glVertexAttribI1ui();
		public static final UnlinkedGLFunction glVertexAttribI1uiEXT = make_glVertexAttribI1uiEXT();
		public static final UnlinkedGLFunction glVertexAttribI1uiv = make_glVertexAttribI1uiv();
		public static final UnlinkedGLFunction glVertexAttribI1uivEXT = make_glVertexAttribI1uivEXT();
		public static final UnlinkedGLFunction glVertexAttribI2i = make_glVertexAttribI2i();
		public static final UnlinkedGLFunction glVertexAttribI2iEXT = make_glVertexAttribI2iEXT();
		public static final UnlinkedGLFunction glVertexAttribI2iv = make_glVertexAttribI2iv();
		public static final UnlinkedGLFunction glVertexAttribI2ivEXT = make_glVertexAttribI2ivEXT();
		public static final UnlinkedGLFunction glVertexAttribI2ui = make_glVertexAttribI2ui();
		public static final UnlinkedGLFunction glVertexAttribI2uiEXT = make_glVertexAttribI2uiEXT();
		public static final UnlinkedGLFunction glVertexAttribI2uiv = make_glVertexAttribI2uiv();
		public static final UnlinkedGLFunction glVertexAttribI2uivEXT = make_glVertexAttribI2uivEXT();
		public static final UnlinkedGLFunction glVertexAttribI3i = make_glVertexAttribI3i();
		public static final UnlinkedGLFunction glVertexAttribI3iEXT = make_glVertexAttribI3iEXT();
		public static final UnlinkedGLFunction glVertexAttribI3iv = make_glVertexAttribI3iv();
		public static final UnlinkedGLFunction glVertexAttribI3ivEXT = make_glVertexAttribI3ivEXT();
		public static final UnlinkedGLFunction glVertexAttribI3ui = make_glVertexAttribI3ui();
		public static final UnlinkedGLFunction glVertexAttribI3uiEXT = make_glVertexAttribI3uiEXT();
		public static final UnlinkedGLFunction glVertexAttribI3uiv = make_glVertexAttribI3uiv();
		public static final UnlinkedGLFunction glVertexAttribI3uivEXT = make_glVertexAttribI3uivEXT();
		public static final UnlinkedGLFunction glVertexAttribI4bv = make_glVertexAttribI4bv();
		public static final UnlinkedGLFunction glVertexAttribI4bvEXT = make_glVertexAttribI4bvEXT();
		public static final UnlinkedGLFunction glVertexAttribI4i = make_glVertexAttribI4i();
		public static final UnlinkedGLFunction glVertexAttribI4iEXT = make_glVertexAttribI4iEXT();
		public static final UnlinkedGLFunction glVertexAttribI4iv = make_glVertexAttribI4iv();
		public static final UnlinkedGLFunction glVertexAttribI4ivEXT = make_glVertexAttribI4ivEXT();
		public static final UnlinkedGLFunction glVertexAttribI4sv = make_glVertexAttribI4sv();
		public static final UnlinkedGLFunction glVertexAttribI4svEXT = make_glVertexAttribI4svEXT();
		public static final UnlinkedGLFunction glVertexAttribI4ubv = make_glVertexAttribI4ubv();
		public static final UnlinkedGLFunction glVertexAttribI4ubvEXT = make_glVertexAttribI4ubvEXT();
		public static final UnlinkedGLFunction glVertexAttribI4ui = make_glVertexAttribI4ui();
		public static final UnlinkedGLFunction glVertexAttribI4uiEXT = make_glVertexAttribI4uiEXT();
		public static final UnlinkedGLFunction glVertexAttribI4uiv = make_glVertexAttribI4uiv();
		public static final UnlinkedGLFunction glVertexAttribI4uivEXT = make_glVertexAttribI4uivEXT();
		public static final UnlinkedGLFunction glVertexAttribI4usv = make_glVertexAttribI4usv();
		public static final UnlinkedGLFunction glVertexAttribI4usvEXT = make_glVertexAttribI4usvEXT();
		public static final UnlinkedGLFunction glVertexAttribIFormat = make_glVertexAttribIFormat();
		public static final UnlinkedGLFunction glVertexAttribIFormatNV = make_glVertexAttribIFormatNV();
		public static final UnlinkedGLFunction glVertexAttribIPointer = make_glVertexAttribIPointer();
		public static final UnlinkedGLFunction glVertexAttribIPointerEXT = make_glVertexAttribIPointerEXT();
		public static final UnlinkedGLFunction glVertexAttribL1d = make_glVertexAttribL1d();
		public static final UnlinkedGLFunction glVertexAttribL1dEXT = make_glVertexAttribL1dEXT();
		public static final UnlinkedGLFunction glVertexAttribL1dv = make_glVertexAttribL1dv();
		public static final UnlinkedGLFunction glVertexAttribL1dvEXT = make_glVertexAttribL1dvEXT();
		public static final UnlinkedGLFunction glVertexAttribL1i64NV = make_glVertexAttribL1i64NV();
		public static final UnlinkedGLFunction glVertexAttribL1i64vNV = make_glVertexAttribL1i64vNV();
		public static final UnlinkedGLFunction glVertexAttribL1ui64ARB = make_glVertexAttribL1ui64ARB();
		public static final UnlinkedGLFunction glVertexAttribL1ui64NV = make_glVertexAttribL1ui64NV();
		public static final UnlinkedGLFunction glVertexAttribL1ui64vARB = make_glVertexAttribL1ui64vARB();
		public static final UnlinkedGLFunction glVertexAttribL1ui64vNV = make_glVertexAttribL1ui64vNV();
		public static final UnlinkedGLFunction glVertexAttribL2d = make_glVertexAttribL2d();
		public static final UnlinkedGLFunction glVertexAttribL2dEXT = make_glVertexAttribL2dEXT();
		public static final UnlinkedGLFunction glVertexAttribL2dv = make_glVertexAttribL2dv();
		public static final UnlinkedGLFunction glVertexAttribL2dvEXT = make_glVertexAttribL2dvEXT();
		public static final UnlinkedGLFunction glVertexAttribL2i64NV = make_glVertexAttribL2i64NV();
		public static final UnlinkedGLFunction glVertexAttribL2i64vNV = make_glVertexAttribL2i64vNV();
		public static final UnlinkedGLFunction glVertexAttribL2ui64NV = make_glVertexAttribL2ui64NV();
		public static final UnlinkedGLFunction glVertexAttribL2ui64vNV = make_glVertexAttribL2ui64vNV();
		public static final UnlinkedGLFunction glVertexAttribL3d = make_glVertexAttribL3d();
		public static final UnlinkedGLFunction glVertexAttribL3dEXT = make_glVertexAttribL3dEXT();
		public static final UnlinkedGLFunction glVertexAttribL3dv = make_glVertexAttribL3dv();
		public static final UnlinkedGLFunction glVertexAttribL3dvEXT = make_glVertexAttribL3dvEXT();
		public static final UnlinkedGLFunction glVertexAttribL3i64NV = make_glVertexAttribL3i64NV();
		public static final UnlinkedGLFunction glVertexAttribL3i64vNV = make_glVertexAttribL3i64vNV();
		public static final UnlinkedGLFunction glVertexAttribL3ui64NV = make_glVertexAttribL3ui64NV();
		public static final UnlinkedGLFunction glVertexAttribL3ui64vNV = make_glVertexAttribL3ui64vNV();
		public static final UnlinkedGLFunction glVertexAttribL4d = make_glVertexAttribL4d();
		public static final UnlinkedGLFunction glVertexAttribL4dEXT = make_glVertexAttribL4dEXT();
		public static final UnlinkedGLFunction glVertexAttribL4dv = make_glVertexAttribL4dv();
		public static final UnlinkedGLFunction glVertexAttribL4dvEXT = make_glVertexAttribL4dvEXT();
		public static final UnlinkedGLFunction glVertexAttribL4i64NV = make_glVertexAttribL4i64NV();
		public static final UnlinkedGLFunction glVertexAttribL4i64vNV = make_glVertexAttribL4i64vNV();
		public static final UnlinkedGLFunction glVertexAttribL4ui64NV = make_glVertexAttribL4ui64NV();
		public static final UnlinkedGLFunction glVertexAttribL4ui64vNV = make_glVertexAttribL4ui64vNV();
		public static final UnlinkedGLFunction glVertexAttribLFormat = make_glVertexAttribLFormat();
		public static final UnlinkedGLFunction glVertexAttribLFormatNV = make_glVertexAttribLFormatNV();
		public static final UnlinkedGLFunction glVertexAttribLPointer = make_glVertexAttribLPointer();
		public static final UnlinkedGLFunction glVertexAttribLPointerEXT = make_glVertexAttribLPointerEXT();
		public static final UnlinkedGLFunction glVertexAttribP1ui = make_glVertexAttribP1ui();
		public static final UnlinkedGLFunction glVertexAttribP1uiv = make_glVertexAttribP1uiv();
		public static final UnlinkedGLFunction glVertexAttribP2ui = make_glVertexAttribP2ui();
		public static final UnlinkedGLFunction glVertexAttribP2uiv = make_glVertexAttribP2uiv();
		public static final UnlinkedGLFunction glVertexAttribP3ui = make_glVertexAttribP3ui();
		public static final UnlinkedGLFunction glVertexAttribP3uiv = make_glVertexAttribP3uiv();
		public static final UnlinkedGLFunction glVertexAttribP4ui = make_glVertexAttribP4ui();
		public static final UnlinkedGLFunction glVertexAttribP4uiv = make_glVertexAttribP4uiv();
		public static final UnlinkedGLFunction glVertexAttribParameteriAMD = make_glVertexAttribParameteriAMD();
		public static final UnlinkedGLFunction glVertexAttribPointer = make_glVertexAttribPointer();
		public static final UnlinkedGLFunction glVertexAttribPointerARB = make_glVertexAttribPointerARB();
		public static final UnlinkedGLFunction glVertexAttribPointerNV = make_glVertexAttribPointerNV();
		public static final UnlinkedGLFunction glVertexAttribs1dvNV = make_glVertexAttribs1dvNV();
		public static final UnlinkedGLFunction glVertexAttribs1fvNV = make_glVertexAttribs1fvNV();
		public static final UnlinkedGLFunction glVertexAttribs1hvNV = make_glVertexAttribs1hvNV();
		public static final UnlinkedGLFunction glVertexAttribs1svNV = make_glVertexAttribs1svNV();
		public static final UnlinkedGLFunction glVertexAttribs2dvNV = make_glVertexAttribs2dvNV();
		public static final UnlinkedGLFunction glVertexAttribs2fvNV = make_glVertexAttribs2fvNV();
		public static final UnlinkedGLFunction glVertexAttribs2hvNV = make_glVertexAttribs2hvNV();
		public static final UnlinkedGLFunction glVertexAttribs2svNV = make_glVertexAttribs2svNV();
		public static final UnlinkedGLFunction glVertexAttribs3dvNV = make_glVertexAttribs3dvNV();
		public static final UnlinkedGLFunction glVertexAttribs3fvNV = make_glVertexAttribs3fvNV();
		public static final UnlinkedGLFunction glVertexAttribs3hvNV = make_glVertexAttribs3hvNV();
		public static final UnlinkedGLFunction glVertexAttribs3svNV = make_glVertexAttribs3svNV();
		public static final UnlinkedGLFunction glVertexAttribs4dvNV = make_glVertexAttribs4dvNV();
		public static final UnlinkedGLFunction glVertexAttribs4fvNV = make_glVertexAttribs4fvNV();
		public static final UnlinkedGLFunction glVertexAttribs4hvNV = make_glVertexAttribs4hvNV();
		public static final UnlinkedGLFunction glVertexAttribs4svNV = make_glVertexAttribs4svNV();
		public static final UnlinkedGLFunction glVertexAttribs4ubvNV = make_glVertexAttribs4ubvNV();
		public static final UnlinkedGLFunction glVertexBindingDivisor = make_glVertexBindingDivisor();
		public static final UnlinkedGLFunction glVertexBlendARB = make_glVertexBlendARB();
		public static final UnlinkedGLFunction glVertexBlendEnvfATI = make_glVertexBlendEnvfATI();
		public static final UnlinkedGLFunction glVertexBlendEnviATI = make_glVertexBlendEnviATI();
		public static final UnlinkedGLFunction glVertexFormatNV = make_glVertexFormatNV();
		public static final UnlinkedGLFunction glVertexP2ui = make_glVertexP2ui();
		public static final UnlinkedGLFunction glVertexP2uiv = make_glVertexP2uiv();
		public static final UnlinkedGLFunction glVertexP3ui = make_glVertexP3ui();
		public static final UnlinkedGLFunction glVertexP3uiv = make_glVertexP3uiv();
		public static final UnlinkedGLFunction glVertexP4ui = make_glVertexP4ui();
		public static final UnlinkedGLFunction glVertexP4uiv = make_glVertexP4uiv();
		public static final UnlinkedGLFunction glVertexPointer = make_glVertexPointer();
		public static final UnlinkedGLFunction glVertexPointerEXT = make_glVertexPointerEXT();
		public static final UnlinkedGLFunction glVertexPointerListIBM = make_glVertexPointerListIBM();
		public static final UnlinkedGLFunction glVertexPointervINTEL = make_glVertexPointervINTEL();
		public static final UnlinkedGLFunction glVertexStream1dATI = make_glVertexStream1dATI();
		public static final UnlinkedGLFunction glVertexStream1dvATI = make_glVertexStream1dvATI();
		public static final UnlinkedGLFunction glVertexStream1fATI = make_glVertexStream1fATI();
		public static final UnlinkedGLFunction glVertexStream1fvATI = make_glVertexStream1fvATI();
		public static final UnlinkedGLFunction glVertexStream1iATI = make_glVertexStream1iATI();
		public static final UnlinkedGLFunction glVertexStream1ivATI = make_glVertexStream1ivATI();
		public static final UnlinkedGLFunction glVertexStream1sATI = make_glVertexStream1sATI();
		public static final UnlinkedGLFunction glVertexStream1svATI = make_glVertexStream1svATI();
		public static final UnlinkedGLFunction glVertexStream2dATI = make_glVertexStream2dATI();
		public static final UnlinkedGLFunction glVertexStream2dvATI = make_glVertexStream2dvATI();
		public static final UnlinkedGLFunction glVertexStream2fATI = make_glVertexStream2fATI();
		public static final UnlinkedGLFunction glVertexStream2fvATI = make_glVertexStream2fvATI();
		public static final UnlinkedGLFunction glVertexStream2iATI = make_glVertexStream2iATI();
		public static final UnlinkedGLFunction glVertexStream2ivATI = make_glVertexStream2ivATI();
		public static final UnlinkedGLFunction glVertexStream2sATI = make_glVertexStream2sATI();
		public static final UnlinkedGLFunction glVertexStream2svATI = make_glVertexStream2svATI();
		public static final UnlinkedGLFunction glVertexStream3dATI = make_glVertexStream3dATI();
		public static final UnlinkedGLFunction glVertexStream3dvATI = make_glVertexStream3dvATI();
		public static final UnlinkedGLFunction glVertexStream3fATI = make_glVertexStream3fATI();
		public static final UnlinkedGLFunction glVertexStream3fvATI = make_glVertexStream3fvATI();
		public static final UnlinkedGLFunction glVertexStream3iATI = make_glVertexStream3iATI();
		public static final UnlinkedGLFunction glVertexStream3ivATI = make_glVertexStream3ivATI();
		public static final UnlinkedGLFunction glVertexStream3sATI = make_glVertexStream3sATI();
		public static final UnlinkedGLFunction glVertexStream3svATI = make_glVertexStream3svATI();
		public static final UnlinkedGLFunction glVertexStream4dATI = make_glVertexStream4dATI();
		public static final UnlinkedGLFunction glVertexStream4dvATI = make_glVertexStream4dvATI();
		public static final UnlinkedGLFunction glVertexStream4fATI = make_glVertexStream4fATI();
		public static final UnlinkedGLFunction glVertexStream4fvATI = make_glVertexStream4fvATI();
		public static final UnlinkedGLFunction glVertexStream4iATI = make_glVertexStream4iATI();
		public static final UnlinkedGLFunction glVertexStream4ivATI = make_glVertexStream4ivATI();
		public static final UnlinkedGLFunction glVertexStream4sATI = make_glVertexStream4sATI();
		public static final UnlinkedGLFunction glVertexStream4svATI = make_glVertexStream4svATI();
		public static final UnlinkedGLFunction glVertexWeightPointerEXT = make_glVertexWeightPointerEXT();
		public static final UnlinkedGLFunction glVertexWeightfEXT = make_glVertexWeightfEXT();
		public static final UnlinkedGLFunction glVertexWeightfvEXT = make_glVertexWeightfvEXT();
		public static final UnlinkedGLFunction glVertexWeighthNV = make_glVertexWeighthNV();
		public static final UnlinkedGLFunction glVertexWeighthvNV = make_glVertexWeighthvNV();
		public static final UnlinkedGLFunction glVideoCaptureNV = make_glVideoCaptureNV();
		public static final UnlinkedGLFunction glVideoCaptureStreamParameterdvNV = make_glVideoCaptureStreamParameterdvNV();
		public static final UnlinkedGLFunction glVideoCaptureStreamParameterfvNV = make_glVideoCaptureStreamParameterfvNV();
		public static final UnlinkedGLFunction glVideoCaptureStreamParameterivNV = make_glVideoCaptureStreamParameterivNV();
		public static final UnlinkedGLFunction glViewport = make_glViewport();
		public static final UnlinkedGLFunction glViewportArrayv = make_glViewportArrayv();
		public static final UnlinkedGLFunction glViewportArrayvNV = make_glViewportArrayvNV();
		public static final UnlinkedGLFunction glViewportArrayvOES = make_glViewportArrayvOES();
		public static final UnlinkedGLFunction glViewportIndexedf = make_glViewportIndexedf();
		public static final UnlinkedGLFunction glViewportIndexedfOES = make_glViewportIndexedfOES();
		public static final UnlinkedGLFunction glViewportIndexedfNV = make_glViewportIndexedfNV();
		public static final UnlinkedGLFunction glViewportIndexedfv = make_glViewportIndexedfv();
		public static final UnlinkedGLFunction glViewportIndexedfvOES = make_glViewportIndexedfvOES();
		public static final UnlinkedGLFunction glViewportIndexedfvNV = make_glViewportIndexedfvNV();
		public static final UnlinkedGLFunction glViewportPositionWScaleNV = make_glViewportPositionWScaleNV();
		public static final UnlinkedGLFunction glViewportSwizzleNV = make_glViewportSwizzleNV();
		public static final UnlinkedGLFunction glWaitSemaphoreEXT = make_glWaitSemaphoreEXT();
		public static final UnlinkedGLFunction glWaitSemaphoreui64NVX = make_glWaitSemaphoreui64NVX();
		public static final UnlinkedGLFunction glWaitSync = make_glWaitSync();
		public static final UnlinkedGLFunction glWaitSyncAPPLE = make_glWaitSyncAPPLE();
		public static final UnlinkedGLFunction glWeightPathsNV = make_glWeightPathsNV();
		public static final UnlinkedGLFunction glWeightPointerARB = make_glWeightPointerARB();
		public static final UnlinkedGLFunction glWeightPointerOES = make_glWeightPointerOES();
		public static final UnlinkedGLFunction glWeightbvARB = make_glWeightbvARB();
		public static final UnlinkedGLFunction glWeightdvARB = make_glWeightdvARB();
		public static final UnlinkedGLFunction glWeightfvARB = make_glWeightfvARB();
		public static final UnlinkedGLFunction glWeightivARB = make_glWeightivARB();
		public static final UnlinkedGLFunction glWeightsvARB = make_glWeightsvARB();
		public static final UnlinkedGLFunction glWeightubvARB = make_glWeightubvARB();
		public static final UnlinkedGLFunction glWeightuivARB = make_glWeightuivARB();
		public static final UnlinkedGLFunction glWeightusvARB = make_glWeightusvARB();
		public static final UnlinkedGLFunction glWindowPos2d = make_glWindowPos2d();
		public static final UnlinkedGLFunction glWindowPos2dARB = make_glWindowPos2dARB();
		public static final UnlinkedGLFunction glWindowPos2dMESA = make_glWindowPos2dMESA();
		public static final UnlinkedGLFunction glWindowPos2dv = make_glWindowPos2dv();
		public static final UnlinkedGLFunction glWindowPos2dvARB = make_glWindowPos2dvARB();
		public static final UnlinkedGLFunction glWindowPos2dvMESA = make_glWindowPos2dvMESA();
		public static final UnlinkedGLFunction glWindowPos2f = make_glWindowPos2f();
		public static final UnlinkedGLFunction glWindowPos2fARB = make_glWindowPos2fARB();
		public static final UnlinkedGLFunction glWindowPos2fMESA = make_glWindowPos2fMESA();
		public static final UnlinkedGLFunction glWindowPos2fv = make_glWindowPos2fv();
		public static final UnlinkedGLFunction glWindowPos2fvARB = make_glWindowPos2fvARB();
		public static final UnlinkedGLFunction glWindowPos2fvMESA = make_glWindowPos2fvMESA();
		public static final UnlinkedGLFunction glWindowPos2i = make_glWindowPos2i();
		public static final UnlinkedGLFunction glWindowPos2iARB = make_glWindowPos2iARB();
		public static final UnlinkedGLFunction glWindowPos2iMESA = make_glWindowPos2iMESA();
		public static final UnlinkedGLFunction glWindowPos2iv = make_glWindowPos2iv();
		public static final UnlinkedGLFunction glWindowPos2ivARB = make_glWindowPos2ivARB();
		public static final UnlinkedGLFunction glWindowPos2ivMESA = make_glWindowPos2ivMESA();
		public static final UnlinkedGLFunction glWindowPos2s = make_glWindowPos2s();
		public static final UnlinkedGLFunction glWindowPos2sARB = make_glWindowPos2sARB();
		public static final UnlinkedGLFunction glWindowPos2sMESA = make_glWindowPos2sMESA();
		public static final UnlinkedGLFunction glWindowPos2sv = make_glWindowPos2sv();
		public static final UnlinkedGLFunction glWindowPos2svARB = make_glWindowPos2svARB();
		public static final UnlinkedGLFunction glWindowPos2svMESA = make_glWindowPos2svMESA();
		public static final UnlinkedGLFunction glWindowPos3d = make_glWindowPos3d();
		public static final UnlinkedGLFunction glWindowPos3dARB = make_glWindowPos3dARB();
		public static final UnlinkedGLFunction glWindowPos3dMESA = make_glWindowPos3dMESA();
		public static final UnlinkedGLFunction glWindowPos3dv = make_glWindowPos3dv();
		public static final UnlinkedGLFunction glWindowPos3dvARB = make_glWindowPos3dvARB();
		public static final UnlinkedGLFunction glWindowPos3dvMESA = make_glWindowPos3dvMESA();
		public static final UnlinkedGLFunction glWindowPos3f = make_glWindowPos3f();
		public static final UnlinkedGLFunction glWindowPos3fARB = make_glWindowPos3fARB();
		public static final UnlinkedGLFunction glWindowPos3fMESA = make_glWindowPos3fMESA();
		public static final UnlinkedGLFunction glWindowPos3fv = make_glWindowPos3fv();
		public static final UnlinkedGLFunction glWindowPos3fvARB = make_glWindowPos3fvARB();
		public static final UnlinkedGLFunction glWindowPos3fvMESA = make_glWindowPos3fvMESA();
		public static final UnlinkedGLFunction glWindowPos3i = make_glWindowPos3i();
		public static final UnlinkedGLFunction glWindowPos3iARB = make_glWindowPos3iARB();
		public static final UnlinkedGLFunction glWindowPos3iMESA = make_glWindowPos3iMESA();
		public static final UnlinkedGLFunction glWindowPos3iv = make_glWindowPos3iv();
		public static final UnlinkedGLFunction glWindowPos3ivARB = make_glWindowPos3ivARB();
		public static final UnlinkedGLFunction glWindowPos3ivMESA = make_glWindowPos3ivMESA();
		public static final UnlinkedGLFunction glWindowPos3s = make_glWindowPos3s();
		public static final UnlinkedGLFunction glWindowPos3sARB = make_glWindowPos3sARB();
		public static final UnlinkedGLFunction glWindowPos3sMESA = make_glWindowPos3sMESA();
		public static final UnlinkedGLFunction glWindowPos3sv = make_glWindowPos3sv();
		public static final UnlinkedGLFunction glWindowPos3svARB = make_glWindowPos3svARB();
		public static final UnlinkedGLFunction glWindowPos3svMESA = make_glWindowPos3svMESA();
		public static final UnlinkedGLFunction glWindowPos4dMESA = make_glWindowPos4dMESA();
		public static final UnlinkedGLFunction glWindowPos4dvMESA = make_glWindowPos4dvMESA();
		public static final UnlinkedGLFunction glWindowPos4fMESA = make_glWindowPos4fMESA();
		public static final UnlinkedGLFunction glWindowPos4fvMESA = make_glWindowPos4fvMESA();
		public static final UnlinkedGLFunction glWindowPos4iMESA = make_glWindowPos4iMESA();
		public static final UnlinkedGLFunction glWindowPos4ivMESA = make_glWindowPos4ivMESA();
		public static final UnlinkedGLFunction glWindowPos4sMESA = make_glWindowPos4sMESA();
		public static final UnlinkedGLFunction glWindowPos4svMESA = make_glWindowPos4svMESA();
		public static final UnlinkedGLFunction glWindowRectanglesEXT = make_glWindowRectanglesEXT();
		public static final UnlinkedGLFunction glWriteMaskEXT = make_glWriteMaskEXT();
		public static final UnlinkedGLFunction glDrawVkImageNV = make_glDrawVkImageNV();
		public static final UnlinkedGLFunction glGetVkProcAddrNV = make_glGetVkProcAddrNV();
		public static final UnlinkedGLFunction glWaitVkSemaphoreNV = make_glWaitVkSemaphoreNV();
		public static final UnlinkedGLFunction glSignalVkSemaphoreNV = make_glSignalVkSemaphoreNV();
		public static final UnlinkedGLFunction glSignalVkFenceNV = make_glSignalVkFenceNV();
		public static final UnlinkedGLFunction glFramebufferParameteriMESA = make_glFramebufferParameteriMESA();
		public static final UnlinkedGLFunction glGetFramebufferParameterivMESA = make_glGetFramebufferParameterivMESA();

		public static final List<UnlinkedGLFunction> FUNCTIONS = makeList();

		// INVOKESTATIC is smaller than NEW + INVOKESPECIAL
		private static UnlinkedGLFunction newUnlinkedGLFunction(String name, Class<?> retType, MemoryLayout retLayout, List<Class<?>> argTypes, List<MemoryLayout> argLayouts) {
			return new UnlinkedGLFunction(name, retType, retLayout, argTypes, argLayouts);
		}

		private static UnlinkedGLFunction make_glAccum() {
			return newUnlinkedGLFunction("glAccum", void.class, GLParamTypeSets.ML_V(), PS_LAccumOp_F(), MLS_LAccumOp_F());
		}
		private static UnlinkedGLFunction make_glAccumxOES() {
			return newUnlinkedGLFunction("glAccumxOES", void.class, GLParamTypeSets.ML_V(), PS_EI(), MLS_EI());
		}
		private static UnlinkedGLFunction make_glActiveProgramEXT() {
			return newUnlinkedGLFunction("glActiveProgramEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glActiveShaderProgram() {
			return newUnlinkedGLFunction("glActiveShaderProgram", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glActiveShaderProgramEXT() {
			return newUnlinkedGLFunction("glActiveShaderProgramEXT", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glActiveStencilFaceEXT() {
			return newUnlinkedGLFunction("glActiveStencilFaceEXT", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_(), MLS_LTriangleFace_());
		}
		private static UnlinkedGLFunction make_glActiveTexture() {
			return newUnlinkedGLFunction("glActiveTexture", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_(), MLS_LTextureUnit_());
		}
		private static UnlinkedGLFunction make_glActiveTextureARB() {
			return newUnlinkedGLFunction("glActiveTextureARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_(), MLS_LTextureUnit_());
		}
		private static UnlinkedGLFunction make_glActiveVaryingNV() {
			return newUnlinkedGLFunction("glActiveVaryingNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glAlphaFragmentOp1ATI() {
			return newUnlinkedGLFunction("glAlphaFragmentOp1ATI", void.class, GLParamTypeSets.ML_V(), PS_LFragmentOp1ATI_IIIII(), MLS_LFragmentOp1ATI_IIIII());
		}
		private static UnlinkedGLFunction make_glAlphaFragmentOp2ATI() {
			return newUnlinkedGLFunction("glAlphaFragmentOp2ATI", void.class, GLParamTypeSets.ML_V(), PS_LFragmentOp2ATI_IIIIIIII(), MLS_LFragmentOp2ATI_IIIIIIII());
		}
		private static UnlinkedGLFunction make_glAlphaFragmentOp3ATI() {
			return newUnlinkedGLFunction("glAlphaFragmentOp3ATI", void.class, GLParamTypeSets.ML_V(), PS_LFragmentOp3ATI_IIIIIIIIIII(), MLS_LFragmentOp3ATI_IIIIIIIIIII());
		}
		private static UnlinkedGLFunction make_glAlphaFunc() {
			return newUnlinkedGLFunction("glAlphaFunc", void.class, GLParamTypeSets.ML_V(), PS_LAlphaFunction_F(), MLS_LAlphaFunction_F());
		}
		private static UnlinkedGLFunction make_glAlphaFuncQCOM() {
			return newUnlinkedGLFunction("glAlphaFuncQCOM", void.class, GLParamTypeSets.ML_V(), PS_EF(), MLS_EF());
		}
		private static UnlinkedGLFunction make_glAlphaFuncx() {
			return newUnlinkedGLFunction("glAlphaFuncx", void.class, GLParamTypeSets.ML_V(), PS_LAlphaFunction_I(), MLS_LAlphaFunction_I());
		}
		private static UnlinkedGLFunction make_glAlphaFuncxOES() {
			return newUnlinkedGLFunction("glAlphaFuncxOES", void.class, GLParamTypeSets.ML_V(), PS_LAlphaFunction_I(), MLS_LAlphaFunction_I());
		}
		private static UnlinkedGLFunction make_glAlphaToCoverageDitherControlNV() {
			return newUnlinkedGLFunction("glAlphaToCoverageDitherControlNV", void.class, GLParamTypeSets.ML_V(), PS_E(), MLS_E());
		}
		private static UnlinkedGLFunction make_glApplyFramebufferAttachmentCMAAINTEL() {
			return newUnlinkedGLFunction("glApplyFramebufferAttachmentCMAAINTEL", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glApplyTextureEXT() {
			return newUnlinkedGLFunction("glApplyTextureEXT", void.class, GLParamTypeSets.ML_V(), PS_LLightTextureModeEXT_(), MLS_LLightTextureModeEXT_());
		}
		private static UnlinkedGLFunction make_glAcquireKeyedMutexWin32EXT() {
			return newUnlinkedGLFunction("glAcquireKeyedMutexWin32EXT", boolean.class, GLParamTypeSets.ML_Z(), PS_IJI(), MLS_IJI());
		}
		private static UnlinkedGLFunction make_glAreProgramsResidentNV() {
			return newUnlinkedGLFunction("glAreProgramsResidentNV", boolean.class, GLParamTypeSets.ML_Z(), PS_IpP(), MLS_IpP());
		}
		private static UnlinkedGLFunction make_glAreTexturesResident() {
			return newUnlinkedGLFunction("glAreTexturesResident", boolean.class, GLParamTypeSets.ML_Z(), PS_IpP(), MLS_IpP());
		}
		private static UnlinkedGLFunction make_glAreTexturesResidentEXT() {
			return newUnlinkedGLFunction("glAreTexturesResidentEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_IpP(), MLS_IpP());
		}
		private static UnlinkedGLFunction make_glArrayElement() {
			return newUnlinkedGLFunction("glArrayElement", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glArrayElementEXT() {
			return newUnlinkedGLFunction("glArrayElementEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glArrayObjectATI() {
			return newUnlinkedGLFunction("glArrayObjectATI", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_ILScalarType_III(), MLS_LEnableCap_ILScalarType_III());
		}
		private static UnlinkedGLFunction make_glAsyncCopyBufferSubDataNVX() {
			return newUnlinkedGLFunction("glAsyncCopyBufferSubDataNVX", int.class, GLParamTypeSets.ML_I(), PS_IppIIIIJJJIpp(), MLS_IppIIIIJJJIpp());
		}
		private static UnlinkedGLFunction make_glAsyncCopyImageSubDataNVX() {
			return newUnlinkedGLFunction("glAsyncCopyImageSubDataNVX", int.class, GLParamTypeSets.ML_I(), PS_IppIIIEIIIIIEIIIIIIIIpp(), MLS_IppIIIEIIIIIEIIIIIIIIpp());
		}
		private static UnlinkedGLFunction make_glAsyncMarkerSGIX() {
			return newUnlinkedGLFunction("glAsyncMarkerSGIX", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glAttachObjectARB() {
			return newUnlinkedGLFunction("glAttachObjectARB", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glAttachShader() {
			return newUnlinkedGLFunction("glAttachShader", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glBegin() {
			return newUnlinkedGLFunction("glBegin", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_(), MLS_LPrimitiveType_());
		}
		private static UnlinkedGLFunction make_glBeginConditionalRender() {
			return newUnlinkedGLFunction("glBeginConditionalRender", void.class, GLParamTypeSets.ML_V(), PS_ILConditionalRenderMode_(), MLS_ILConditionalRenderMode_());
		}
		private static UnlinkedGLFunction make_glBeginConditionalRenderNV() {
			return newUnlinkedGLFunction("glBeginConditionalRenderNV", void.class, GLParamTypeSets.ML_V(), PS_ILConditionalRenderMode_(), MLS_ILConditionalRenderMode_());
		}
		private static UnlinkedGLFunction make_glBeginConditionalRenderNVX() {
			return newUnlinkedGLFunction("glBeginConditionalRenderNVX", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBeginFragmentShaderATI() {
			return newUnlinkedGLFunction("glBeginFragmentShaderATI", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glBeginOcclusionQueryNV() {
			return newUnlinkedGLFunction("glBeginOcclusionQueryNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBeginPerfMonitorAMD() {
			return newUnlinkedGLFunction("glBeginPerfMonitorAMD", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBeginPerfQueryINTEL() {
			return newUnlinkedGLFunction("glBeginPerfQueryINTEL", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBeginQuery() {
			return newUnlinkedGLFunction("glBeginQuery", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_I(), MLS_LQueryTarget_I());
		}
		private static UnlinkedGLFunction make_glBeginQueryARB() {
			return newUnlinkedGLFunction("glBeginQueryARB", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_I(), MLS_LQueryTarget_I());
		}
		private static UnlinkedGLFunction make_glBeginQueryEXT() {
			return newUnlinkedGLFunction("glBeginQueryEXT", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_I(), MLS_LQueryTarget_I());
		}
		private static UnlinkedGLFunction make_glBeginQueryIndexed() {
			return newUnlinkedGLFunction("glBeginQueryIndexed", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_II(), MLS_LQueryTarget_II());
		}
		private static UnlinkedGLFunction make_glBeginTransformFeedback() {
			return newUnlinkedGLFunction("glBeginTransformFeedback", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_(), MLS_LPrimitiveType_());
		}
		private static UnlinkedGLFunction make_glBeginTransformFeedbackEXT() {
			return newUnlinkedGLFunction("glBeginTransformFeedbackEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_(), MLS_LPrimitiveType_());
		}
		private static UnlinkedGLFunction make_glBeginTransformFeedbackNV() {
			return newUnlinkedGLFunction("glBeginTransformFeedbackNV", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_(), MLS_LPrimitiveType_());
		}
		private static UnlinkedGLFunction make_glBeginVertexShaderEXT() {
			return newUnlinkedGLFunction("glBeginVertexShaderEXT", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glBeginVideoCaptureNV() {
			return newUnlinkedGLFunction("glBeginVideoCaptureNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBindAttribLocation() {
			return newUnlinkedGLFunction("glBindAttribLocation", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glBindAttribLocationARB() {
			return newUnlinkedGLFunction("glBindAttribLocationARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glBindBuffer() {
			return newUnlinkedGLFunction("glBindBuffer", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_I(), MLS_LBufferTarget_I());
		}
		private static UnlinkedGLFunction make_glBindBufferARB() {
			return newUnlinkedGLFunction("glBindBufferARB", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_I(), MLS_LBufferTarget_I());
		}
		private static UnlinkedGLFunction make_glBindBufferBase() {
			return newUnlinkedGLFunction("glBindBufferBase", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_II(), MLS_LBufferTarget_II());
		}
		private static UnlinkedGLFunction make_glBindBufferBaseEXT() {
			return newUnlinkedGLFunction("glBindBufferBaseEXT", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_II(), MLS_LBufferTarget_II());
		}
		private static UnlinkedGLFunction make_glBindBufferBaseNV() {
			return newUnlinkedGLFunction("glBindBufferBaseNV", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_II(), MLS_LBufferTarget_II());
		}
		private static UnlinkedGLFunction make_glBindBufferOffsetEXT() {
			return newUnlinkedGLFunction("glBindBufferOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_IIJ(), MLS_LBufferTarget_IIJ());
		}
		private static UnlinkedGLFunction make_glBindBufferOffsetNV() {
			return newUnlinkedGLFunction("glBindBufferOffsetNV", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_IIJ(), MLS_LBufferTarget_IIJ());
		}
		private static UnlinkedGLFunction make_glBindBufferRange() {
			return newUnlinkedGLFunction("glBindBufferRange", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_IIJJ(), MLS_LBufferTarget_IIJJ());
		}
		private static UnlinkedGLFunction make_glBindBufferRangeEXT() {
			return newUnlinkedGLFunction("glBindBufferRangeEXT", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_IIJJ(), MLS_LBufferTarget_IIJJ());
		}
		private static UnlinkedGLFunction make_glBindBufferRangeNV() {
			return newUnlinkedGLFunction("glBindBufferRangeNV", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_IIJJ(), MLS_LBufferTarget_IIJJ());
		}
		private static UnlinkedGLFunction make_glBindBuffersBase() {
			return newUnlinkedGLFunction("glBindBuffersBase", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_IIp(), MLS_LBufferTarget_IIp());
		}
		private static UnlinkedGLFunction make_glBindBuffersRange() {
			return newUnlinkedGLFunction("glBindBuffersRange", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_IIppp(), MLS_LBufferTarget_IIppp());
		}
		private static UnlinkedGLFunction make_glBindFragDataLocation() {
			return newUnlinkedGLFunction("glBindFragDataLocation", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glBindFragDataLocationEXT() {
			return newUnlinkedGLFunction("glBindFragDataLocationEXT", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glBindFragDataLocationIndexed() {
			return newUnlinkedGLFunction("glBindFragDataLocationIndexed", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glBindFragDataLocationIndexedEXT() {
			return newUnlinkedGLFunction("glBindFragDataLocationIndexedEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glBindFragmentShaderATI() {
			return newUnlinkedGLFunction("glBindFragmentShaderATI", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBindFramebuffer() {
			return newUnlinkedGLFunction("glBindFramebuffer", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_I(), MLS_LFramebufferTarget_I());
		}
		private static UnlinkedGLFunction make_glBindFramebufferEXT() {
			return newUnlinkedGLFunction("glBindFramebufferEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_I(), MLS_LFramebufferTarget_I());
		}
		private static UnlinkedGLFunction make_glBindFramebufferOES() {
			return newUnlinkedGLFunction("glBindFramebufferOES", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_I(), MLS_LFramebufferTarget_I());
		}
		private static UnlinkedGLFunction make_glBindImageTexture() {
			return newUnlinkedGLFunction("glBindImageTexture", void.class, GLParamTypeSets.ML_V(), PS_IIIZILBufferAccess_LInternalFormat_(), MLS_IIIZILBufferAccess_LInternalFormat_());
		}
		private static UnlinkedGLFunction make_glBindImageTextureEXT() {
			return newUnlinkedGLFunction("glBindImageTextureEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZILBufferAccess_I(), MLS_IIIZILBufferAccess_I());
		}
		private static UnlinkedGLFunction make_glBindImageTextures() {
			return newUnlinkedGLFunction("glBindImageTextures", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glBindLightParameterEXT() {
			return newUnlinkedGLFunction("glBindLightParameterEXT", int.class, GLParamTypeSets.ML_I(), PS_LLightName_LLightParameter_(), MLS_LLightName_LLightParameter_());
		}
		private static UnlinkedGLFunction make_glBindMaterialParameterEXT() {
			return newUnlinkedGLFunction("glBindMaterialParameterEXT", int.class, GLParamTypeSets.ML_I(), PS_LTriangleFace_LMaterialParameter_(), MLS_LTriangleFace_LMaterialParameter_());
		}
		private static UnlinkedGLFunction make_glBindMultiTextureEXT() {
			return newUnlinkedGLFunction("glBindMultiTextureEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_I(), MLS_LTextureUnit_LTextureTarget_I());
		}
		private static UnlinkedGLFunction make_glBindParameterEXT() {
			return newUnlinkedGLFunction("glBindParameterEXT", int.class, GLParamTypeSets.ML_I(), PS_LVertexShaderParameterEXT_(), MLS_LVertexShaderParameterEXT_());
		}
		private static UnlinkedGLFunction make_glBindProgramARB() {
			return newUnlinkedGLFunction("glBindProgramARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_I(), MLS_LProgramTarget_I());
		}
		private static UnlinkedGLFunction make_glBindProgramNV() {
			return newUnlinkedGLFunction("glBindProgramNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_I(), MLS_LVertexAttribEnumNV_I());
		}
		private static UnlinkedGLFunction make_glBindProgramPipeline() {
			return newUnlinkedGLFunction("glBindProgramPipeline", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBindProgramPipelineEXT() {
			return newUnlinkedGLFunction("glBindProgramPipelineEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBindRenderbuffer() {
			return newUnlinkedGLFunction("glBindRenderbuffer", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_I(), MLS_LRenderbufferTarget_I());
		}
		private static UnlinkedGLFunction make_glBindRenderbufferEXT() {
			return newUnlinkedGLFunction("glBindRenderbufferEXT", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_I(), MLS_LRenderbufferTarget_I());
		}
		private static UnlinkedGLFunction make_glBindRenderbufferOES() {
			return newUnlinkedGLFunction("glBindRenderbufferOES", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_I(), MLS_LRenderbufferTarget_I());
		}
		private static UnlinkedGLFunction make_glBindSampler() {
			return newUnlinkedGLFunction("glBindSampler", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glBindSamplers() {
			return newUnlinkedGLFunction("glBindSamplers", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glBindShadingRateImageNV() {
			return newUnlinkedGLFunction("glBindShadingRateImageNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBindTexGenParameterEXT() {
			return newUnlinkedGLFunction("glBindTexGenParameterEXT", int.class, GLParamTypeSets.ML_I(), PS_LTextureUnit_LTextureCoordName_LTextureGenParameter_(), MLS_LTextureUnit_LTextureCoordName_LTextureGenParameter_());
		}
		private static UnlinkedGLFunction make_glBindTexture() {
			return newUnlinkedGLFunction("glBindTexture", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_I(), MLS_LTextureTarget_I());
		}
		private static UnlinkedGLFunction make_glBindTextureEXT() {
			return newUnlinkedGLFunction("glBindTextureEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_I(), MLS_LTextureTarget_I());
		}
		private static UnlinkedGLFunction make_glBindTextureUnit() {
			return newUnlinkedGLFunction("glBindTextureUnit", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glBindTextureUnitParameterEXT() {
			return newUnlinkedGLFunction("glBindTextureUnitParameterEXT", int.class, GLParamTypeSets.ML_I(), PS_LTextureUnit_LVertexShaderTextureUnitParameter_(), MLS_LTextureUnit_LVertexShaderTextureUnitParameter_());
		}
		private static UnlinkedGLFunction make_glBindTextures() {
			return newUnlinkedGLFunction("glBindTextures", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glBindTransformFeedback() {
			return newUnlinkedGLFunction("glBindTransformFeedback", void.class, GLParamTypeSets.ML_V(), PS_LBindTransformFeedbackTarget_I(), MLS_LBindTransformFeedbackTarget_I());
		}
		private static UnlinkedGLFunction make_glBindTransformFeedbackNV() {
			return newUnlinkedGLFunction("glBindTransformFeedbackNV", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_I(), MLS_LBufferTarget_I());
		}
		private static UnlinkedGLFunction make_glBindVertexArray() {
			return newUnlinkedGLFunction("glBindVertexArray", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBindVertexArrayAPPLE() {
			return newUnlinkedGLFunction("glBindVertexArrayAPPLE", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBindVertexArrayOES() {
			return newUnlinkedGLFunction("glBindVertexArrayOES", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBindVertexBuffer() {
			return newUnlinkedGLFunction("glBindVertexBuffer", void.class, GLParamTypeSets.ML_V(), PS_IIJI(), MLS_IIJI());
		}
		private static UnlinkedGLFunction make_glBindVertexBuffers() {
			return newUnlinkedGLFunction("glBindVertexBuffers", void.class, GLParamTypeSets.ML_V(), PS_IIppp(), MLS_IIppp());
		}
		private static UnlinkedGLFunction make_glBindVertexShaderEXT() {
			return newUnlinkedGLFunction("glBindVertexShaderEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glBindVideoCaptureStreamBufferNV() {
			return newUnlinkedGLFunction("glBindVideoCaptureStreamBufferNV", void.class, GLParamTypeSets.ML_V(), PS_IIEJ(), MLS_IIEJ());
		}
		private static UnlinkedGLFunction make_glBindVideoCaptureStreamTextureNV() {
			return newUnlinkedGLFunction("glBindVideoCaptureStreamTextureNV", void.class, GLParamTypeSets.ML_V(), PS_IIEEI(), MLS_IIEEI());
		}
		private static UnlinkedGLFunction make_glBinormal3bEXT() {
			return newUnlinkedGLFunction("glBinormal3bEXT", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glBinormal3bvEXT() {
			return newUnlinkedGLFunction("glBinormal3bvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glBinormal3dEXT() {
			return newUnlinkedGLFunction("glBinormal3dEXT", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glBinormal3dvEXT() {
			return newUnlinkedGLFunction("glBinormal3dvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glBinormal3fEXT() {
			return newUnlinkedGLFunction("glBinormal3fEXT", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glBinormal3fvEXT() {
			return newUnlinkedGLFunction("glBinormal3fvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glBinormal3iEXT() {
			return newUnlinkedGLFunction("glBinormal3iEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glBinormal3ivEXT() {
			return newUnlinkedGLFunction("glBinormal3ivEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glBinormal3sEXT() {
			return newUnlinkedGLFunction("glBinormal3sEXT", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glBinormal3svEXT() {
			return newUnlinkedGLFunction("glBinormal3svEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glBinormalPointerEXT() {
			return newUnlinkedGLFunction("glBinormalPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_LBinormalPointerTypeEXT_Ip(), MLS_LBinormalPointerTypeEXT_Ip());
		}
		private static UnlinkedGLFunction make_glBitmap() {
			return newUnlinkedGLFunction("glBitmap", void.class, GLParamTypeSets.ML_V(), PS_IIFFFFp(), MLS_IIFFFFp());
		}
		private static UnlinkedGLFunction make_glBitmapxOES() {
			return newUnlinkedGLFunction("glBitmapxOES", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIp(), MLS_IIIIIIp());
		}
		private static UnlinkedGLFunction make_glBlendBarrier() {
			return newUnlinkedGLFunction("glBlendBarrier", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glBlendBarrierKHR() {
			return newUnlinkedGLFunction("glBlendBarrierKHR", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glBlendBarrierNV() {
			return newUnlinkedGLFunction("glBlendBarrierNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glBlendColor() {
			return newUnlinkedGLFunction("glBlendColor", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glBlendColorEXT() {
			return newUnlinkedGLFunction("glBlendColorEXT", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glBlendColorxOES() {
			return newUnlinkedGLFunction("glBlendColorxOES", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glBlendEquation() {
			return newUnlinkedGLFunction("glBlendEquation", void.class, GLParamTypeSets.ML_V(), PS_LBlendEquationModeEXT_(), MLS_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationEXT() {
			return newUnlinkedGLFunction("glBlendEquationEXT", void.class, GLParamTypeSets.ML_V(), PS_LBlendEquationModeEXT_(), MLS_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationIndexedAMD() {
			return newUnlinkedGLFunction("glBlendEquationIndexedAMD", void.class, GLParamTypeSets.ML_V(), PS_ILBlendEquationModeEXT_(), MLS_ILBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationOES() {
			return newUnlinkedGLFunction("glBlendEquationOES", void.class, GLParamTypeSets.ML_V(), PS_LBlendEquationModeEXT_(), MLS_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationSeparate() {
			return newUnlinkedGLFunction("glBlendEquationSeparate", void.class, GLParamTypeSets.ML_V(), PS_LBlendEquationModeEXT_LBlendEquationModeEXT_(), MLS_LBlendEquationModeEXT_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationSeparateEXT() {
			return newUnlinkedGLFunction("glBlendEquationSeparateEXT", void.class, GLParamTypeSets.ML_V(), PS_LBlendEquationModeEXT_LBlendEquationModeEXT_(), MLS_LBlendEquationModeEXT_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationSeparateIndexedAMD() {
			return newUnlinkedGLFunction("glBlendEquationSeparateIndexedAMD", void.class, GLParamTypeSets.ML_V(), PS_ILBlendEquationModeEXT_LBlendEquationModeEXT_(), MLS_ILBlendEquationModeEXT_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationSeparateOES() {
			return newUnlinkedGLFunction("glBlendEquationSeparateOES", void.class, GLParamTypeSets.ML_V(), PS_LBlendEquationModeEXT_LBlendEquationModeEXT_(), MLS_LBlendEquationModeEXT_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationSeparatei() {
			return newUnlinkedGLFunction("glBlendEquationSeparatei", void.class, GLParamTypeSets.ML_V(), PS_ILBlendEquationModeEXT_LBlendEquationModeEXT_(), MLS_ILBlendEquationModeEXT_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationSeparateiARB() {
			return newUnlinkedGLFunction("glBlendEquationSeparateiARB", void.class, GLParamTypeSets.ML_V(), PS_ILBlendEquationModeEXT_LBlendEquationModeEXT_(), MLS_ILBlendEquationModeEXT_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationSeparateiEXT() {
			return newUnlinkedGLFunction("glBlendEquationSeparateiEXT", void.class, GLParamTypeSets.ML_V(), PS_ILBlendEquationModeEXT_LBlendEquationModeEXT_(), MLS_ILBlendEquationModeEXT_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationSeparateiOES() {
			return newUnlinkedGLFunction("glBlendEquationSeparateiOES", void.class, GLParamTypeSets.ML_V(), PS_ILBlendEquationModeEXT_LBlendEquationModeEXT_(), MLS_ILBlendEquationModeEXT_LBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationi() {
			return newUnlinkedGLFunction("glBlendEquationi", void.class, GLParamTypeSets.ML_V(), PS_ILBlendEquationModeEXT_(), MLS_ILBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationiARB() {
			return newUnlinkedGLFunction("glBlendEquationiARB", void.class, GLParamTypeSets.ML_V(), PS_ILBlendEquationModeEXT_(), MLS_ILBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationiEXT() {
			return newUnlinkedGLFunction("glBlendEquationiEXT", void.class, GLParamTypeSets.ML_V(), PS_ILBlendEquationModeEXT_(), MLS_ILBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendEquationiOES() {
			return newUnlinkedGLFunction("glBlendEquationiOES", void.class, GLParamTypeSets.ML_V(), PS_ILBlendEquationModeEXT_(), MLS_ILBlendEquationModeEXT_());
		}
		private static UnlinkedGLFunction make_glBlendFunc() {
			return newUnlinkedGLFunction("glBlendFunc", void.class, GLParamTypeSets.ML_V(), PS_LBlendingFactor_LBlendingFactor_(), MLS_LBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFuncIndexedAMD() {
			return newUnlinkedGLFunction("glBlendFuncIndexedAMD", void.class, GLParamTypeSets.ML_V(), PS_IEE(), MLS_IEE());
		}
		private static UnlinkedGLFunction make_glBlendFuncSeparate() {
			return newUnlinkedGLFunction("glBlendFuncSeparate", void.class, GLParamTypeSets.ML_V(), PS_LBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_(), MLS_LBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFuncSeparateEXT() {
			return newUnlinkedGLFunction("glBlendFuncSeparateEXT", void.class, GLParamTypeSets.ML_V(), PS_LBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_(), MLS_LBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFuncSeparateINGR() {
			return newUnlinkedGLFunction("glBlendFuncSeparateINGR", void.class, GLParamTypeSets.ML_V(), PS_LBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_(), MLS_LBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFuncSeparateIndexedAMD() {
			return newUnlinkedGLFunction("glBlendFuncSeparateIndexedAMD", void.class, GLParamTypeSets.ML_V(), PS_ILBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_(), MLS_ILBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFuncSeparateOES() {
			return newUnlinkedGLFunction("glBlendFuncSeparateOES", void.class, GLParamTypeSets.ML_V(), PS_LBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_(), MLS_LBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFuncSeparatei() {
			return newUnlinkedGLFunction("glBlendFuncSeparatei", void.class, GLParamTypeSets.ML_V(), PS_ILBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_(), MLS_ILBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFuncSeparateiARB() {
			return newUnlinkedGLFunction("glBlendFuncSeparateiARB", void.class, GLParamTypeSets.ML_V(), PS_ILBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_(), MLS_ILBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFuncSeparateiEXT() {
			return newUnlinkedGLFunction("glBlendFuncSeparateiEXT", void.class, GLParamTypeSets.ML_V(), PS_ILBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_(), MLS_ILBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFuncSeparateiOES() {
			return newUnlinkedGLFunction("glBlendFuncSeparateiOES", void.class, GLParamTypeSets.ML_V(), PS_ILBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_(), MLS_ILBlendingFactor_LBlendingFactor_LBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFunci() {
			return newUnlinkedGLFunction("glBlendFunci", void.class, GLParamTypeSets.ML_V(), PS_ILBlendingFactor_LBlendingFactor_(), MLS_ILBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFunciARB() {
			return newUnlinkedGLFunction("glBlendFunciARB", void.class, GLParamTypeSets.ML_V(), PS_ILBlendingFactor_LBlendingFactor_(), MLS_ILBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFunciEXT() {
			return newUnlinkedGLFunction("glBlendFunciEXT", void.class, GLParamTypeSets.ML_V(), PS_ILBlendingFactor_LBlendingFactor_(), MLS_ILBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendFunciOES() {
			return newUnlinkedGLFunction("glBlendFunciOES", void.class, GLParamTypeSets.ML_V(), PS_ILBlendingFactor_LBlendingFactor_(), MLS_ILBlendingFactor_LBlendingFactor_());
		}
		private static UnlinkedGLFunction make_glBlendParameteriNV() {
			return newUnlinkedGLFunction("glBlendParameteriNV", void.class, GLParamTypeSets.ML_V(), PS_EI(), MLS_EI());
		}
		private static UnlinkedGLFunction make_glBlitFramebuffer() {
			return newUnlinkedGLFunction("glBlitFramebuffer", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIILBlitFramebufferFilter_(), MLS_IIIIIIIIILBlitFramebufferFilter_());
		}
		private static UnlinkedGLFunction make_glBlitFramebufferANGLE() {
			return newUnlinkedGLFunction("glBlitFramebufferANGLE", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIILBlitFramebufferFilter_(), MLS_IIIIIIIIILBlitFramebufferFilter_());
		}
		private static UnlinkedGLFunction make_glBlitFramebufferEXT() {
			return newUnlinkedGLFunction("glBlitFramebufferEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIILBlitFramebufferFilter_(), MLS_IIIIIIIIILBlitFramebufferFilter_());
		}
		private static UnlinkedGLFunction make_glBlitFramebufferLayerEXT() {
			return newUnlinkedGLFunction("glBlitFramebufferLayerEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIIIILBlitFramebufferFilter_(), MLS_IIIIIIIIIIILBlitFramebufferFilter_());
		}
		private static UnlinkedGLFunction make_glBlitFramebufferLayersEXT() {
			return newUnlinkedGLFunction("glBlitFramebufferLayersEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIILBlitFramebufferFilter_(), MLS_IIIIIIIIILBlitFramebufferFilter_());
		}
		private static UnlinkedGLFunction make_glBlitFramebufferNV() {
			return newUnlinkedGLFunction("glBlitFramebufferNV", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIILBlitFramebufferFilter_(), MLS_IIIIIIIIILBlitFramebufferFilter_());
		}
		private static UnlinkedGLFunction make_glBlitNamedFramebuffer() {
			return newUnlinkedGLFunction("glBlitNamedFramebuffer", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIIIILBlitFramebufferFilter_(), MLS_IIIIIIIIIIILBlitFramebufferFilter_());
		}
		private static UnlinkedGLFunction make_glBufferAddressRangeNV() {
			return newUnlinkedGLFunction("glBufferAddressRangeNV", void.class, GLParamTypeSets.ML_V(), PS_EIJJ(), MLS_EIJJ());
		}
		private static UnlinkedGLFunction make_glBufferAttachMemoryNV() {
			return newUnlinkedGLFunction("glBufferAttachMemoryNV", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_IJ(), MLS_LBufferTarget_IJ());
		}
		private static UnlinkedGLFunction make_glBufferData() {
			return newUnlinkedGLFunction("glBufferData", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_JpLBufferUsage_(), MLS_LBufferTarget_JpLBufferUsage_());
		}
		private static UnlinkedGLFunction make_glBufferDataARB() {
			return newUnlinkedGLFunction("glBufferDataARB", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_JpLBufferUsage_(), MLS_LBufferTarget_JpLBufferUsage_());
		}
		private static UnlinkedGLFunction make_glBufferPageCommitmentARB() {
			return newUnlinkedGLFunction("glBufferPageCommitmentARB", void.class, GLParamTypeSets.ML_V(), PS_EJJZ(), MLS_EJJZ());
		}
		private static UnlinkedGLFunction make_glBufferPageCommitmentMemNV() {
			return newUnlinkedGLFunction("glBufferPageCommitmentMemNV", void.class, GLParamTypeSets.ML_V(), PS_LBufferStorageTarget_JJIJZ(), MLS_LBufferStorageTarget_JJIJZ());
		}
		private static UnlinkedGLFunction make_glBufferParameteriAPPLE() {
			return newUnlinkedGLFunction("glBufferParameteriAPPLE", void.class, GLParamTypeSets.ML_V(), PS_EEI(), MLS_EEI());
		}
		private static UnlinkedGLFunction make_glBufferStorage() {
			return newUnlinkedGLFunction("glBufferStorage", void.class, GLParamTypeSets.ML_V(), PS_LBufferStorageTarget_JpI(), MLS_LBufferStorageTarget_JpI());
		}
		private static UnlinkedGLFunction make_glBufferStorageEXT() {
			return newUnlinkedGLFunction("glBufferStorageEXT", void.class, GLParamTypeSets.ML_V(), PS_LBufferStorageTarget_JpI(), MLS_LBufferStorageTarget_JpI());
		}
		private static UnlinkedGLFunction make_glBufferStorageExternalEXT() {
			return newUnlinkedGLFunction("glBufferStorageExternalEXT", void.class, GLParamTypeSets.ML_V(), PS_EJJII(), MLS_EJJII());
		}
		private static UnlinkedGLFunction make_glBufferStorageMemEXT() {
			return newUnlinkedGLFunction("glBufferStorageMemEXT", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_JIJ(), MLS_LBufferTarget_JIJ());
		}
		private static UnlinkedGLFunction make_glBufferSubData() {
			return newUnlinkedGLFunction("glBufferSubData", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_JJp(), MLS_LBufferTarget_JJp());
		}
		private static UnlinkedGLFunction make_glBufferSubDataARB() {
			return newUnlinkedGLFunction("glBufferSubDataARB", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_JJp(), MLS_LBufferTarget_JJp());
		}
		private static UnlinkedGLFunction make_glCallCommandListNV() {
			return newUnlinkedGLFunction("glCallCommandListNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glCallList() {
			return newUnlinkedGLFunction("glCallList", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glCallLists() {
			return newUnlinkedGLFunction("glCallLists", void.class, GLParamTypeSets.ML_V(), PS_ILListNameType_p(), MLS_ILListNameType_p());
		}
		private static UnlinkedGLFunction make_glCheckFramebufferStatus() {
			return newUnlinkedGLFunction("glCheckFramebufferStatus", GLFramebufferStatus.class, GLParamTypeSets.ML_LFramebufferStatus_(), PS_LFramebufferTarget_(), MLS_LFramebufferTarget_());
		}
		private static UnlinkedGLFunction make_glCheckFramebufferStatusEXT() {
			return newUnlinkedGLFunction("glCheckFramebufferStatusEXT", GLFramebufferStatus.class, GLParamTypeSets.ML_LFramebufferStatus_(), PS_LFramebufferTarget_(), MLS_LFramebufferTarget_());
		}
		private static UnlinkedGLFunction make_glCheckFramebufferStatusOES() {
			return newUnlinkedGLFunction("glCheckFramebufferStatusOES", GLFramebufferStatus.class, GLParamTypeSets.ML_LFramebufferStatus_(), PS_LFramebufferTarget_(), MLS_LFramebufferTarget_());
		}
		private static UnlinkedGLFunction make_glCheckNamedFramebufferStatus() {
			return newUnlinkedGLFunction("glCheckNamedFramebufferStatus", GLFramebufferStatus.class, GLParamTypeSets.ML_LFramebufferStatus_(), PS_ILFramebufferTarget_(), MLS_ILFramebufferTarget_());
		}
		private static UnlinkedGLFunction make_glCheckNamedFramebufferStatusEXT() {
			return newUnlinkedGLFunction("glCheckNamedFramebufferStatusEXT", GLFramebufferStatus.class, GLParamTypeSets.ML_LFramebufferStatus_(), PS_ILFramebufferTarget_(), MLS_ILFramebufferTarget_());
		}
		private static UnlinkedGLFunction make_glClampColor() {
			return newUnlinkedGLFunction("glClampColor", void.class, GLParamTypeSets.ML_V(), PS_LClampColorTargetARB_LClampColorModeARB_(), MLS_LClampColorTargetARB_LClampColorModeARB_());
		}
		private static UnlinkedGLFunction make_glClampColorARB() {
			return newUnlinkedGLFunction("glClampColorARB", void.class, GLParamTypeSets.ML_V(), PS_LClampColorTargetARB_LClampColorModeARB_(), MLS_LClampColorTargetARB_LClampColorModeARB_());
		}
		private static UnlinkedGLFunction make_glClear() {
			return newUnlinkedGLFunction("glClear", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glClearAccum() {
			return newUnlinkedGLFunction("glClearAccum", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glClearAccumxOES() {
			return newUnlinkedGLFunction("glClearAccumxOES", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glClearBufferData() {
			return newUnlinkedGLFunction("glClearBufferData", void.class, GLParamTypeSets.ML_V(), PS_LBufferStorageTarget_LSizedInternalFormat_LPixelFormat_LPixelType_p(), MLS_LBufferStorageTarget_LSizedInternalFormat_LPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glClearBufferSubData() {
			return newUnlinkedGLFunction("glClearBufferSubData", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_LSizedInternalFormat_JJLPixelFormat_LPixelType_p(), MLS_LBufferTarget_LSizedInternalFormat_JJLPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glClearBufferfi() {
			return newUnlinkedGLFunction("glClearBufferfi", void.class, GLParamTypeSets.ML_V(), PS_LBuffer_IFI(), MLS_LBuffer_IFI());
		}
		private static UnlinkedGLFunction make_glClearBufferfv() {
			return newUnlinkedGLFunction("glClearBufferfv", void.class, GLParamTypeSets.ML_V(), PS_LBuffer_Ip(), MLS_LBuffer_Ip());
		}
		private static UnlinkedGLFunction make_glClearBufferiv() {
			return newUnlinkedGLFunction("glClearBufferiv", void.class, GLParamTypeSets.ML_V(), PS_LBuffer_Ip(), MLS_LBuffer_Ip());
		}
		private static UnlinkedGLFunction make_glClearBufferuiv() {
			return newUnlinkedGLFunction("glClearBufferuiv", void.class, GLParamTypeSets.ML_V(), PS_LBuffer_Ip(), MLS_LBuffer_Ip());
		}
		private static UnlinkedGLFunction make_glClearColor() {
			return newUnlinkedGLFunction("glClearColor", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glClearColorIiEXT() {
			return newUnlinkedGLFunction("glClearColorIiEXT", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glClearColorIuiEXT() {
			return newUnlinkedGLFunction("glClearColorIuiEXT", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glClearColorx() {
			return newUnlinkedGLFunction("glClearColorx", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glClearColorxOES() {
			return newUnlinkedGLFunction("glClearColorxOES", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glClearDepth() {
			return newUnlinkedGLFunction("glClearDepth", void.class, GLParamTypeSets.ML_V(), PS_D(), MLS_D());
		}
		private static UnlinkedGLFunction make_glClearDepthdNV() {
			return newUnlinkedGLFunction("glClearDepthdNV", void.class, GLParamTypeSets.ML_V(), PS_D(), MLS_D());
		}
		private static UnlinkedGLFunction make_glClearDepthf() {
			return newUnlinkedGLFunction("glClearDepthf", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glClearDepthfOES() {
			return newUnlinkedGLFunction("glClearDepthfOES", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glClearDepthx() {
			return newUnlinkedGLFunction("glClearDepthx", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glClearDepthxOES() {
			return newUnlinkedGLFunction("glClearDepthxOES", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glClearIndex() {
			return newUnlinkedGLFunction("glClearIndex", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glClearNamedBufferData() {
			return newUnlinkedGLFunction("glClearNamedBufferData", void.class, GLParamTypeSets.ML_V(), PS_ILSizedInternalFormat_LPixelFormat_LPixelType_p(), MLS_ILSizedInternalFormat_LPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glClearNamedBufferDataEXT() {
			return newUnlinkedGLFunction("glClearNamedBufferDataEXT", void.class, GLParamTypeSets.ML_V(), PS_ILSizedInternalFormat_LPixelFormat_LPixelType_p(), MLS_ILSizedInternalFormat_LPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glClearNamedBufferSubData() {
			return newUnlinkedGLFunction("glClearNamedBufferSubData", void.class, GLParamTypeSets.ML_V(), PS_ILSizedInternalFormat_JJLPixelFormat_LPixelType_p(), MLS_ILSizedInternalFormat_JJLPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glClearNamedBufferSubDataEXT() {
			return newUnlinkedGLFunction("glClearNamedBufferSubDataEXT", void.class, GLParamTypeSets.ML_V(), PS_ILSizedInternalFormat_JJLPixelFormat_LPixelType_p(), MLS_ILSizedInternalFormat_JJLPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glClearNamedFramebufferfi() {
			return newUnlinkedGLFunction("glClearNamedFramebufferfi", void.class, GLParamTypeSets.ML_V(), PS_ILBuffer_IFI(), MLS_ILBuffer_IFI());
		}
		private static UnlinkedGLFunction make_glClearNamedFramebufferfv() {
			return newUnlinkedGLFunction("glClearNamedFramebufferfv", void.class, GLParamTypeSets.ML_V(), PS_ILBuffer_Ip(), MLS_ILBuffer_Ip());
		}
		private static UnlinkedGLFunction make_glClearNamedFramebufferiv() {
			return newUnlinkedGLFunction("glClearNamedFramebufferiv", void.class, GLParamTypeSets.ML_V(), PS_ILBuffer_Ip(), MLS_ILBuffer_Ip());
		}
		private static UnlinkedGLFunction make_glClearNamedFramebufferuiv() {
			return newUnlinkedGLFunction("glClearNamedFramebufferuiv", void.class, GLParamTypeSets.ML_V(), PS_ILBuffer_Ip(), MLS_ILBuffer_Ip());
		}
		private static UnlinkedGLFunction make_glClearPixelLocalStorageuiEXT() {
			return newUnlinkedGLFunction("glClearPixelLocalStorageuiEXT", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glClearStencil() {
			return newUnlinkedGLFunction("glClearStencil", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glClearTexImage() {
			return newUnlinkedGLFunction("glClearTexImage", void.class, GLParamTypeSets.ML_V(), PS_IILPixelFormat_LPixelType_p(), MLS_IILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glClearTexImageEXT() {
			return newUnlinkedGLFunction("glClearTexImageEXT", void.class, GLParamTypeSets.ML_V(), PS_IILPixelFormat_LPixelType_p(), MLS_IILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glClearTexSubImage() {
			return newUnlinkedGLFunction("glClearTexSubImage", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIILPixelFormat_LPixelType_p(), MLS_IIIIIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glClearTexSubImageEXT() {
			return newUnlinkedGLFunction("glClearTexSubImageEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIILPixelFormat_LPixelType_p(), MLS_IIIIIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glClientActiveTexture() {
			return newUnlinkedGLFunction("glClientActiveTexture", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_(), MLS_LTextureUnit_());
		}
		private static UnlinkedGLFunction make_glClientActiveTextureARB() {
			return newUnlinkedGLFunction("glClientActiveTextureARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_(), MLS_LTextureUnit_());
		}
		private static UnlinkedGLFunction make_glClientActiveVertexStreamATI() {
			return newUnlinkedGLFunction("glClientActiveVertexStreamATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_(), MLS_LVertexStreamATI_());
		}
		private static UnlinkedGLFunction make_glClientAttribDefaultEXT() {
			return newUnlinkedGLFunction("glClientAttribDefaultEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glClientWaitSemaphoreui64NVX() {
			return newUnlinkedGLFunction("glClientWaitSemaphoreui64NVX", void.class, GLParamTypeSets.ML_V(), PS_Ipp(), MLS_Ipp());
		}
		private static UnlinkedGLFunction make_glClientWaitSync() {
			return newUnlinkedGLFunction("glClientWaitSync", GLSyncStatus.class, GLParamTypeSets.ML_LSyncStatus_(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glClientWaitSyncAPPLE() {
			return newUnlinkedGLFunction("glClientWaitSyncAPPLE", GLSyncStatus.class, GLParamTypeSets.ML_LSyncStatus_(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glClipControl() {
			return newUnlinkedGLFunction("glClipControl", void.class, GLParamTypeSets.ML_V(), PS_LClipControlOrigin_LClipControlDepth_(), MLS_LClipControlOrigin_LClipControlDepth_());
		}
		private static UnlinkedGLFunction make_glClipControlEXT() {
			return newUnlinkedGLFunction("glClipControlEXT", void.class, GLParamTypeSets.ML_V(), PS_EE(), MLS_EE());
		}
		private static UnlinkedGLFunction make_glClipPlane() {
			return newUnlinkedGLFunction("glClipPlane", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_p(), MLS_LClipPlaneName_p());
		}
		private static UnlinkedGLFunction make_glClipPlanef() {
			return newUnlinkedGLFunction("glClipPlanef", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_p(), MLS_LClipPlaneName_p());
		}
		private static UnlinkedGLFunction make_glClipPlanefIMG() {
			return newUnlinkedGLFunction("glClipPlanefIMG", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_p(), MLS_LClipPlaneName_p());
		}
		private static UnlinkedGLFunction make_glClipPlanefOES() {
			return newUnlinkedGLFunction("glClipPlanefOES", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_p(), MLS_LClipPlaneName_p());
		}
		private static UnlinkedGLFunction make_glClipPlanex() {
			return newUnlinkedGLFunction("glClipPlanex", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_p(), MLS_LClipPlaneName_p());
		}
		private static UnlinkedGLFunction make_glClipPlanexIMG() {
			return newUnlinkedGLFunction("glClipPlanexIMG", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_p(), MLS_LClipPlaneName_p());
		}
		private static UnlinkedGLFunction make_glClipPlanexOES() {
			return newUnlinkedGLFunction("glClipPlanexOES", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_p(), MLS_LClipPlaneName_p());
		}
		private static UnlinkedGLFunction make_glColor3b() {
			return newUnlinkedGLFunction("glColor3b", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glColor3bv() {
			return newUnlinkedGLFunction("glColor3bv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor3d() {
			return newUnlinkedGLFunction("glColor3d", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glColor3dv() {
			return newUnlinkedGLFunction("glColor3dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor3f() {
			return newUnlinkedGLFunction("glColor3f", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glColor3fVertex3fSUN() {
			return newUnlinkedGLFunction("glColor3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_FFFFFF(), MLS_FFFFFF());
		}
		private static UnlinkedGLFunction make_glColor3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glColor3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glColor3fv() {
			return newUnlinkedGLFunction("glColor3fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor3hNV() {
			return newUnlinkedGLFunction("glColor3hNV", void.class, GLParamTypeSets.ML_V(), PS_CCC(), MLS_CCC());
		}
		private static UnlinkedGLFunction make_glColor3hvNV() {
			return newUnlinkedGLFunction("glColor3hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor3i() {
			return newUnlinkedGLFunction("glColor3i", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glColor3iv() {
			return newUnlinkedGLFunction("glColor3iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor3s() {
			return newUnlinkedGLFunction("glColor3s", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glColor3sv() {
			return newUnlinkedGLFunction("glColor3sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor3ub() {
			return newUnlinkedGLFunction("glColor3ub", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glColor3ubv() {
			return newUnlinkedGLFunction("glColor3ubv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor3ui() {
			return newUnlinkedGLFunction("glColor3ui", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glColor3uiv() {
			return newUnlinkedGLFunction("glColor3uiv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor3us() {
			return newUnlinkedGLFunction("glColor3us", void.class, GLParamTypeSets.ML_V(), PS_CCC(), MLS_CCC());
		}
		private static UnlinkedGLFunction make_glColor3usv() {
			return newUnlinkedGLFunction("glColor3usv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor3xOES() {
			return newUnlinkedGLFunction("glColor3xOES", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glColor3xvOES() {
			return newUnlinkedGLFunction("glColor3xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor4b() {
			return newUnlinkedGLFunction("glColor4b", void.class, GLParamTypeSets.ML_V(), PS_BBBB(), MLS_BBBB());
		}
		private static UnlinkedGLFunction make_glColor4bv() {
			return newUnlinkedGLFunction("glColor4bv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor4d() {
			return newUnlinkedGLFunction("glColor4d", void.class, GLParamTypeSets.ML_V(), PS_DDDD(), MLS_DDDD());
		}
		private static UnlinkedGLFunction make_glColor4dv() {
			return newUnlinkedGLFunction("glColor4dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor4f() {
			return newUnlinkedGLFunction("glColor4f", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glColor4fNormal3fVertex3fSUN() {
			return newUnlinkedGLFunction("glColor4fNormal3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_FFFFFFFFFF(), MLS_FFFFFFFFFF());
		}
		private static UnlinkedGLFunction make_glColor4fNormal3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glColor4fNormal3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_ppp(), MLS_ppp());
		}
		private static UnlinkedGLFunction make_glColor4fv() {
			return newUnlinkedGLFunction("glColor4fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor4hNV() {
			return newUnlinkedGLFunction("glColor4hNV", void.class, GLParamTypeSets.ML_V(), PS_CCCC(), MLS_CCCC());
		}
		private static UnlinkedGLFunction make_glColor4hvNV() {
			return newUnlinkedGLFunction("glColor4hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor4i() {
			return newUnlinkedGLFunction("glColor4i", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glColor4iv() {
			return newUnlinkedGLFunction("glColor4iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor4s() {
			return newUnlinkedGLFunction("glColor4s", void.class, GLParamTypeSets.ML_V(), PS_SSSS(), MLS_SSSS());
		}
		private static UnlinkedGLFunction make_glColor4sv() {
			return newUnlinkedGLFunction("glColor4sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor4ub() {
			return newUnlinkedGLFunction("glColor4ub", void.class, GLParamTypeSets.ML_V(), PS_BBBB(), MLS_BBBB());
		}
		private static UnlinkedGLFunction make_glColor4ubVertex2fSUN() {
			return newUnlinkedGLFunction("glColor4ubVertex2fSUN", void.class, GLParamTypeSets.ML_V(), PS_BBBBFF(), MLS_BBBBFF());
		}
		private static UnlinkedGLFunction make_glColor4ubVertex2fvSUN() {
			return newUnlinkedGLFunction("glColor4ubVertex2fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glColor4ubVertex3fSUN() {
			return newUnlinkedGLFunction("glColor4ubVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_BBBBFFF(), MLS_BBBBFFF());
		}
		private static UnlinkedGLFunction make_glColor4ubVertex3fvSUN() {
			return newUnlinkedGLFunction("glColor4ubVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glColor4ubv() {
			return newUnlinkedGLFunction("glColor4ubv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor4ui() {
			return newUnlinkedGLFunction("glColor4ui", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glColor4uiv() {
			return newUnlinkedGLFunction("glColor4uiv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor4us() {
			return newUnlinkedGLFunction("glColor4us", void.class, GLParamTypeSets.ML_V(), PS_CCCC(), MLS_CCCC());
		}
		private static UnlinkedGLFunction make_glColor4usv() {
			return newUnlinkedGLFunction("glColor4usv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColor4x() {
			return newUnlinkedGLFunction("glColor4x", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glColor4xOES() {
			return newUnlinkedGLFunction("glColor4xOES", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glColor4xvOES() {
			return newUnlinkedGLFunction("glColor4xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glColorFormatNV() {
			return newUnlinkedGLFunction("glColorFormatNV", void.class, GLParamTypeSets.ML_V(), PS_ILColorPointerType_I(), MLS_ILColorPointerType_I());
		}
		private static UnlinkedGLFunction make_glColorFragmentOp1ATI() {
			return newUnlinkedGLFunction("glColorFragmentOp1ATI", void.class, GLParamTypeSets.ML_V(), PS_LFragmentOp1ATI_IIIIII(), MLS_LFragmentOp1ATI_IIIIII());
		}
		private static UnlinkedGLFunction make_glColorFragmentOp2ATI() {
			return newUnlinkedGLFunction("glColorFragmentOp2ATI", void.class, GLParamTypeSets.ML_V(), PS_LFragmentOp2ATI_IIIIIIIII(), MLS_LFragmentOp2ATI_IIIIIIIII());
		}
		private static UnlinkedGLFunction make_glColorFragmentOp3ATI() {
			return newUnlinkedGLFunction("glColorFragmentOp3ATI", void.class, GLParamTypeSets.ML_V(), PS_LFragmentOp3ATI_IIIIIIIIIIII(), MLS_LFragmentOp3ATI_IIIIIIIIIIII());
		}
		private static UnlinkedGLFunction make_glColorMask() {
			return newUnlinkedGLFunction("glColorMask", void.class, GLParamTypeSets.ML_V(), PS_ZZZZ(), MLS_ZZZZ());
		}
		private static UnlinkedGLFunction make_glColorMaskIndexedEXT() {
			return newUnlinkedGLFunction("glColorMaskIndexedEXT", void.class, GLParamTypeSets.ML_V(), PS_IZZZZ(), MLS_IZZZZ());
		}
		private static UnlinkedGLFunction make_glColorMaski() {
			return newUnlinkedGLFunction("glColorMaski", void.class, GLParamTypeSets.ML_V(), PS_IZZZZ(), MLS_IZZZZ());
		}
		private static UnlinkedGLFunction make_glColorMaskiEXT() {
			return newUnlinkedGLFunction("glColorMaskiEXT", void.class, GLParamTypeSets.ML_V(), PS_IZZZZ(), MLS_IZZZZ());
		}
		private static UnlinkedGLFunction make_glColorMaskiOES() {
			return newUnlinkedGLFunction("glColorMaskiOES", void.class, GLParamTypeSets.ML_V(), PS_IZZZZ(), MLS_IZZZZ());
		}
		private static UnlinkedGLFunction make_glColorMaterial() {
			return newUnlinkedGLFunction("glColorMaterial", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LColorMaterialParameter_(), MLS_LTriangleFace_LColorMaterialParameter_());
		}
		private static UnlinkedGLFunction make_glColorP3ui() {
			return newUnlinkedGLFunction("glColorP3ui", void.class, GLParamTypeSets.ML_V(), PS_LColorPointerType_I(), MLS_LColorPointerType_I());
		}
		private static UnlinkedGLFunction make_glColorP3uiv() {
			return newUnlinkedGLFunction("glColorP3uiv", void.class, GLParamTypeSets.ML_V(), PS_LColorPointerType_p(), MLS_LColorPointerType_p());
		}
		private static UnlinkedGLFunction make_glColorP4ui() {
			return newUnlinkedGLFunction("glColorP4ui", void.class, GLParamTypeSets.ML_V(), PS_LColorPointerType_I(), MLS_LColorPointerType_I());
		}
		private static UnlinkedGLFunction make_glColorP4uiv() {
			return newUnlinkedGLFunction("glColorP4uiv", void.class, GLParamTypeSets.ML_V(), PS_LColorPointerType_p(), MLS_LColorPointerType_p());
		}
		private static UnlinkedGLFunction make_glColorPointer() {
			return newUnlinkedGLFunction("glColorPointer", void.class, GLParamTypeSets.ML_V(), PS_ILColorPointerType_Ip(), MLS_ILColorPointerType_Ip());
		}
		private static UnlinkedGLFunction make_glColorPointerEXT() {
			return newUnlinkedGLFunction("glColorPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_ILColorPointerType_IIp(), MLS_ILColorPointerType_IIp());
		}
		private static UnlinkedGLFunction make_glColorPointerListIBM() {
			return newUnlinkedGLFunction("glColorPointerListIBM", void.class, GLParamTypeSets.ML_V(), PS_ILColorPointerType_IPI(), MLS_ILColorPointerType_IPI());
		}
		private static UnlinkedGLFunction make_glColorPointervINTEL() {
			return newUnlinkedGLFunction("glColorPointervINTEL", void.class, GLParamTypeSets.ML_V(), PS_ILVertexPointerType_P(), MLS_ILVertexPointerType_P());
		}
		private static UnlinkedGLFunction make_glColorSubTable() {
			return newUnlinkedGLFunction("glColorSubTable", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_IILPixelFormat_LPixelType_p(), MLS_LColorTableTarget_IILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glColorSubTableEXT() {
			return newUnlinkedGLFunction("glColorSubTableEXT", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_IILPixelFormat_LPixelType_p(), MLS_LColorTableTarget_IILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glColorTable() {
			return newUnlinkedGLFunction("glColorTable", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LInternalFormat_ILPixelFormat_LPixelType_p(), MLS_LColorTableTarget_LInternalFormat_ILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glColorTableEXT() {
			return newUnlinkedGLFunction("glColorTableEXT", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LInternalFormat_ILPixelFormat_LPixelType_p(), MLS_LColorTableTarget_LInternalFormat_ILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glColorTableParameterfv() {
			return newUnlinkedGLFunction("glColorTableParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LColorTableParameterPName_p(), MLS_LColorTableTarget_LColorTableParameterPName_p());
		}
		private static UnlinkedGLFunction make_glColorTableParameterfvSGI() {
			return newUnlinkedGLFunction("glColorTableParameterfvSGI", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTargetSGI_LColorTableParameterPName_p(), MLS_LColorTableTargetSGI_LColorTableParameterPName_p());
		}
		private static UnlinkedGLFunction make_glColorTableParameteriv() {
			return newUnlinkedGLFunction("glColorTableParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LColorTableParameterPName_p(), MLS_LColorTableTarget_LColorTableParameterPName_p());
		}
		private static UnlinkedGLFunction make_glColorTableParameterivSGI() {
			return newUnlinkedGLFunction("glColorTableParameterivSGI", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTargetSGI_LColorTableParameterPName_p(), MLS_LColorTableTargetSGI_LColorTableParameterPName_p());
		}
		private static UnlinkedGLFunction make_glColorTableSGI() {
			return newUnlinkedGLFunction("glColorTableSGI", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTargetSGI_LInternalFormat_ILPixelFormat_LPixelType_p(), MLS_LColorTableTargetSGI_LInternalFormat_ILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glCombinerInputNV() {
			return newUnlinkedGLFunction("glCombinerInputNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerStageNV_LCombinerPortionNV_LCombinerVariableNV_LCombinerRegisterNV_LCombinerMappingNV_LCombinerComponentUsageNV_(), MLS_LCombinerStageNV_LCombinerPortionNV_LCombinerVariableNV_LCombinerRegisterNV_LCombinerMappingNV_LCombinerComponentUsageNV_());
		}
		private static UnlinkedGLFunction make_glCombinerOutputNV() {
			return newUnlinkedGLFunction("glCombinerOutputNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerStageNV_LCombinerPortionNV_LCombinerRegisterNV_LCombinerRegisterNV_LCombinerRegisterNV_LCombinerScaleNV_LCombinerBiasNV_ZZZ(), MLS_LCombinerStageNV_LCombinerPortionNV_LCombinerRegisterNV_LCombinerRegisterNV_LCombinerRegisterNV_LCombinerScaleNV_LCombinerBiasNV_ZZZ());
		}
		private static UnlinkedGLFunction make_glCombinerParameterfNV() {
			return newUnlinkedGLFunction("glCombinerParameterfNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerParameterNV_F(), MLS_LCombinerParameterNV_F());
		}
		private static UnlinkedGLFunction make_glCombinerParameterfvNV() {
			return newUnlinkedGLFunction("glCombinerParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerParameterNV_p(), MLS_LCombinerParameterNV_p());
		}
		private static UnlinkedGLFunction make_glCombinerParameteriNV() {
			return newUnlinkedGLFunction("glCombinerParameteriNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerParameterNV_I(), MLS_LCombinerParameterNV_I());
		}
		private static UnlinkedGLFunction make_glCombinerParameterivNV() {
			return newUnlinkedGLFunction("glCombinerParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerParameterNV_p(), MLS_LCombinerParameterNV_p());
		}
		private static UnlinkedGLFunction make_glCombinerStageParameterfvNV() {
			return newUnlinkedGLFunction("glCombinerStageParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerStageNV_LCombinerParameterNV_p(), MLS_LCombinerStageNV_LCombinerParameterNV_p());
		}
		private static UnlinkedGLFunction make_glCommandListSegmentsNV() {
			return newUnlinkedGLFunction("glCommandListSegmentsNV", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glCompileCommandListNV() {
			return newUnlinkedGLFunction("glCompileCommandListNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glCompileShader() {
			return newUnlinkedGLFunction("glCompileShader", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glCompileShaderARB() {
			return newUnlinkedGLFunction("glCompileShaderARB", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glCompileShaderIncludeARB() {
			return newUnlinkedGLFunction("glCompileShaderIncludeARB", void.class, GLParamTypeSets.ML_V(), PS_IIpp(), MLS_IIpp());
		}
		private static UnlinkedGLFunction make_glCompressedMultiTexImage1DEXT() {
			return newUnlinkedGLFunction("glCompressedMultiTexImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILInternalFormat_IIIp(), MLS_LTextureUnit_LTextureTarget_ILInternalFormat_IIIp());
		}
		private static UnlinkedGLFunction make_glCompressedMultiTexImage2DEXT() {
			return newUnlinkedGLFunction("glCompressedMultiTexImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILInternalFormat_IIIIp(), MLS_LTextureUnit_LTextureTarget_ILInternalFormat_IIIIp());
		}
		private static UnlinkedGLFunction make_glCompressedMultiTexImage3DEXT() {
			return newUnlinkedGLFunction("glCompressedMultiTexImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILInternalFormat_IIIIIp(), MLS_LTextureUnit_LTextureTarget_ILInternalFormat_IIIIIp());
		}
		private static UnlinkedGLFunction make_glCompressedMultiTexSubImage1DEXT() {
			return newUnlinkedGLFunction("glCompressedMultiTexSubImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_IIILInternalFormat_Ip(), MLS_LTextureUnit_LTextureTarget_IIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedMultiTexSubImage2DEXT() {
			return newUnlinkedGLFunction("glCompressedMultiTexSubImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_IIIIILInternalFormat_Ip(), MLS_LTextureUnit_LTextureTarget_IIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedMultiTexSubImage3DEXT() {
			return newUnlinkedGLFunction("glCompressedMultiTexSubImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_IIIIIIILInternalFormat_Ip(), MLS_LTextureUnit_LTextureTarget_IIIIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTexImage1D() {
			return newUnlinkedGLFunction("glCompressedTexImage1D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIp(), MLS_LTextureTarget_ILInternalFormat_IIIp());
		}
		private static UnlinkedGLFunction make_glCompressedTexImage1DARB() {
			return newUnlinkedGLFunction("glCompressedTexImage1DARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIp(), MLS_LTextureTarget_ILInternalFormat_IIIp());
		}
		private static UnlinkedGLFunction make_glCompressedTexImage2D() {
			return newUnlinkedGLFunction("glCompressedTexImage2D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIIp(), MLS_LTextureTarget_ILInternalFormat_IIIIp());
		}
		private static UnlinkedGLFunction make_glCompressedTexImage2DARB() {
			return newUnlinkedGLFunction("glCompressedTexImage2DARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIIp(), MLS_LTextureTarget_ILInternalFormat_IIIIp());
		}
		private static UnlinkedGLFunction make_glCompressedTexImage3D() {
			return newUnlinkedGLFunction("glCompressedTexImage3D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIIIp(), MLS_LTextureTarget_ILInternalFormat_IIIIIp());
		}
		private static UnlinkedGLFunction make_glCompressedTexImage3DARB() {
			return newUnlinkedGLFunction("glCompressedTexImage3DARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIIIp(), MLS_LTextureTarget_ILInternalFormat_IIIIIp());
		}
		private static UnlinkedGLFunction make_glCompressedTexImage3DOES() {
			return newUnlinkedGLFunction("glCompressedTexImage3DOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIIIp(), MLS_LTextureTarget_ILInternalFormat_IIIIIp());
		}
		private static UnlinkedGLFunction make_glCompressedTexSubImage1D() {
			return newUnlinkedGLFunction("glCompressedTexSubImage1D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIILInternalFormat_Ip(), MLS_LTextureTarget_IIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTexSubImage1DARB() {
			return newUnlinkedGLFunction("glCompressedTexSubImage1DARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIILInternalFormat_Ip(), MLS_LTextureTarget_IIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTexSubImage2D() {
			return newUnlinkedGLFunction("glCompressedTexSubImage2D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIILInternalFormat_Ip(), MLS_LTextureTarget_IIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTexSubImage2DARB() {
			return newUnlinkedGLFunction("glCompressedTexSubImage2DARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIILInternalFormat_Ip(), MLS_LTextureTarget_IIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTexSubImage3D() {
			return newUnlinkedGLFunction("glCompressedTexSubImage3D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIIILInternalFormat_Ip(), MLS_LTextureTarget_IIIIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTexSubImage3DARB() {
			return newUnlinkedGLFunction("glCompressedTexSubImage3DARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIIILInternalFormat_Ip(), MLS_LTextureTarget_IIIIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTexSubImage3DOES() {
			return newUnlinkedGLFunction("glCompressedTexSubImage3DOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIIILInternalFormat_Ip(), MLS_LTextureTarget_IIIIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTextureImage1DEXT() {
			return newUnlinkedGLFunction("glCompressedTextureImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILInternalFormat_IIIp(), MLS_ILTextureTarget_ILInternalFormat_IIIp());
		}
		private static UnlinkedGLFunction make_glCompressedTextureImage2DEXT() {
			return newUnlinkedGLFunction("glCompressedTextureImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILInternalFormat_IIIIp(), MLS_ILTextureTarget_ILInternalFormat_IIIIp());
		}
		private static UnlinkedGLFunction make_glCompressedTextureImage3DEXT() {
			return newUnlinkedGLFunction("glCompressedTextureImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILInternalFormat_IIIIIp(), MLS_ILTextureTarget_ILInternalFormat_IIIIIp());
		}
		private static UnlinkedGLFunction make_glCompressedTextureSubImage1D() {
			return newUnlinkedGLFunction("glCompressedTextureSubImage1D", void.class, GLParamTypeSets.ML_V(), PS_IIIILInternalFormat_Ip(), MLS_IIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTextureSubImage1DEXT() {
			return newUnlinkedGLFunction("glCompressedTextureSubImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IIILInternalFormat_Ip(), MLS_ILTextureTarget_IIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTextureSubImage2D() {
			return newUnlinkedGLFunction("glCompressedTextureSubImage2D", void.class, GLParamTypeSets.ML_V(), PS_IIIIIILInternalFormat_Ip(), MLS_IIIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTextureSubImage2DEXT() {
			return newUnlinkedGLFunction("glCompressedTextureSubImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IIIIILInternalFormat_Ip(), MLS_ILTextureTarget_IIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTextureSubImage3D() {
			return newUnlinkedGLFunction("glCompressedTextureSubImage3D", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIILInternalFormat_Ip(), MLS_IIIIIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glCompressedTextureSubImage3DEXT() {
			return newUnlinkedGLFunction("glCompressedTextureSubImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IIIIIIILInternalFormat_Ip(), MLS_ILTextureTarget_IIIIIIILInternalFormat_Ip());
		}
		private static UnlinkedGLFunction make_glConservativeRasterParameterfNV() {
			return newUnlinkedGLFunction("glConservativeRasterParameterfNV", void.class, GLParamTypeSets.ML_V(), PS_EF(), MLS_EF());
		}
		private static UnlinkedGLFunction make_glConservativeRasterParameteriNV() {
			return newUnlinkedGLFunction("glConservativeRasterParameteriNV", void.class, GLParamTypeSets.ML_V(), PS_EI(), MLS_EI());
		}
		private static UnlinkedGLFunction make_glConvolutionFilter1D() {
			return newUnlinkedGLFunction("glConvolutionFilter1D", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LInternalFormat_ILPixelFormat_LPixelType_p(), MLS_LConvolutionTarget_LInternalFormat_ILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glConvolutionFilter1DEXT() {
			return newUnlinkedGLFunction("glConvolutionFilter1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LInternalFormat_ILPixelFormat_LPixelType_p(), MLS_LConvolutionTargetEXT_LInternalFormat_ILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glConvolutionFilter2D() {
			return newUnlinkedGLFunction("glConvolutionFilter2D", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LInternalFormat_IILPixelFormat_LPixelType_p(), MLS_LConvolutionTarget_LInternalFormat_IILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glConvolutionFilter2DEXT() {
			return newUnlinkedGLFunction("glConvolutionFilter2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LInternalFormat_IILPixelFormat_LPixelType_p(), MLS_LConvolutionTargetEXT_LInternalFormat_IILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glConvolutionParameterf() {
			return newUnlinkedGLFunction("glConvolutionParameterf", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LConvolutionParameter_F(), MLS_LConvolutionTarget_LConvolutionParameter_F());
		}
		private static UnlinkedGLFunction make_glConvolutionParameterfEXT() {
			return newUnlinkedGLFunction("glConvolutionParameterfEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LConvolutionParameter_F(), MLS_LConvolutionTargetEXT_LConvolutionParameter_F());
		}
		private static UnlinkedGLFunction make_glConvolutionParameterfv() {
			return newUnlinkedGLFunction("glConvolutionParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LConvolutionParameter_p(), MLS_LConvolutionTarget_LConvolutionParameter_p());
		}
		private static UnlinkedGLFunction make_glConvolutionParameterfvEXT() {
			return newUnlinkedGLFunction("glConvolutionParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LConvolutionParameter_p(), MLS_LConvolutionTargetEXT_LConvolutionParameter_p());
		}
		private static UnlinkedGLFunction make_glConvolutionParameteri() {
			return newUnlinkedGLFunction("glConvolutionParameteri", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LConvolutionParameter_I(), MLS_LConvolutionTarget_LConvolutionParameter_I());
		}
		private static UnlinkedGLFunction make_glConvolutionParameteriEXT() {
			return newUnlinkedGLFunction("glConvolutionParameteriEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LConvolutionParameter_I(), MLS_LConvolutionTargetEXT_LConvolutionParameter_I());
		}
		private static UnlinkedGLFunction make_glConvolutionParameteriv() {
			return newUnlinkedGLFunction("glConvolutionParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LConvolutionParameter_p(), MLS_LConvolutionTarget_LConvolutionParameter_p());
		}
		private static UnlinkedGLFunction make_glConvolutionParameterivEXT() {
			return newUnlinkedGLFunction("glConvolutionParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LConvolutionParameter_p(), MLS_LConvolutionTargetEXT_LConvolutionParameter_p());
		}
		private static UnlinkedGLFunction make_glConvolutionParameterxOES() {
			return newUnlinkedGLFunction("glConvolutionParameterxOES", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LConvolutionParameter_I(), MLS_LConvolutionTargetEXT_LConvolutionParameter_I());
		}
		private static UnlinkedGLFunction make_glConvolutionParameterxvOES() {
			return newUnlinkedGLFunction("glConvolutionParameterxvOES", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LConvolutionParameter_p(), MLS_LConvolutionTargetEXT_LConvolutionParameter_p());
		}
		private static UnlinkedGLFunction make_glCopyBufferSubData() {
			return newUnlinkedGLFunction("glCopyBufferSubData", void.class, GLParamTypeSets.ML_V(), PS_LCopyBufferSubDataTarget_LCopyBufferSubDataTarget_JJJ(), MLS_LCopyBufferSubDataTarget_LCopyBufferSubDataTarget_JJJ());
		}
		private static UnlinkedGLFunction make_glCopyBufferSubDataNV() {
			return newUnlinkedGLFunction("glCopyBufferSubDataNV", void.class, GLParamTypeSets.ML_V(), PS_LCopyBufferSubDataTarget_LCopyBufferSubDataTarget_JJJ(), MLS_LCopyBufferSubDataTarget_LCopyBufferSubDataTarget_JJJ());
		}
		private static UnlinkedGLFunction make_glCopyColorSubTable() {
			return newUnlinkedGLFunction("glCopyColorSubTable", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_IIII(), MLS_LColorTableTarget_IIII());
		}
		private static UnlinkedGLFunction make_glCopyColorSubTableEXT() {
			return newUnlinkedGLFunction("glCopyColorSubTableEXT", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_IIII(), MLS_LColorTableTarget_IIII());
		}
		private static UnlinkedGLFunction make_glCopyColorTable() {
			return newUnlinkedGLFunction("glCopyColorTable", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LInternalFormat_III(), MLS_LColorTableTarget_LInternalFormat_III());
		}
		private static UnlinkedGLFunction make_glCopyColorTableSGI() {
			return newUnlinkedGLFunction("glCopyColorTableSGI", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTargetSGI_LInternalFormat_III(), MLS_LColorTableTargetSGI_LInternalFormat_III());
		}
		private static UnlinkedGLFunction make_glCopyConvolutionFilter1D() {
			return newUnlinkedGLFunction("glCopyConvolutionFilter1D", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LInternalFormat_III(), MLS_LConvolutionTarget_LInternalFormat_III());
		}
		private static UnlinkedGLFunction make_glCopyConvolutionFilter1DEXT() {
			return newUnlinkedGLFunction("glCopyConvolutionFilter1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LInternalFormat_III(), MLS_LConvolutionTargetEXT_LInternalFormat_III());
		}
		private static UnlinkedGLFunction make_glCopyConvolutionFilter2D() {
			return newUnlinkedGLFunction("glCopyConvolutionFilter2D", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LInternalFormat_IIII(), MLS_LConvolutionTarget_LInternalFormat_IIII());
		}
		private static UnlinkedGLFunction make_glCopyConvolutionFilter2DEXT() {
			return newUnlinkedGLFunction("glCopyConvolutionFilter2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LInternalFormat_IIII(), MLS_LConvolutionTargetEXT_LInternalFormat_IIII());
		}
		private static UnlinkedGLFunction make_glCopyImageSubData() {
			return newUnlinkedGLFunction("glCopyImageSubData", void.class, GLParamTypeSets.ML_V(), PS_ILCopyImageSubDataTarget_IIIIILCopyImageSubDataTarget_IIIIIII(), MLS_ILCopyImageSubDataTarget_IIIIILCopyImageSubDataTarget_IIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyImageSubDataEXT() {
			return newUnlinkedGLFunction("glCopyImageSubDataEXT", void.class, GLParamTypeSets.ML_V(), PS_ILCopyBufferSubDataTarget_IIIIILCopyBufferSubDataTarget_IIIIIII(), MLS_ILCopyBufferSubDataTarget_IIIIILCopyBufferSubDataTarget_IIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyImageSubDataNV() {
			return newUnlinkedGLFunction("glCopyImageSubDataNV", void.class, GLParamTypeSets.ML_V(), PS_ILCopyBufferSubDataTarget_IIIIILCopyBufferSubDataTarget_IIIIIII(), MLS_ILCopyBufferSubDataTarget_IIIIILCopyBufferSubDataTarget_IIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyImageSubDataOES() {
			return newUnlinkedGLFunction("glCopyImageSubDataOES", void.class, GLParamTypeSets.ML_V(), PS_ILCopyBufferSubDataTarget_IIIIILCopyBufferSubDataTarget_IIIIIII(), MLS_ILCopyBufferSubDataTarget_IIIIILCopyBufferSubDataTarget_IIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyMultiTexImage1DEXT() {
			return newUnlinkedGLFunction("glCopyMultiTexImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILInternalFormat_IIII(), MLS_LTextureUnit_LTextureTarget_ILInternalFormat_IIII());
		}
		private static UnlinkedGLFunction make_glCopyMultiTexImage2DEXT() {
			return newUnlinkedGLFunction("glCopyMultiTexImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILInternalFormat_IIIII(), MLS_LTextureUnit_LTextureTarget_ILInternalFormat_IIIII());
		}
		private static UnlinkedGLFunction make_glCopyMultiTexSubImage1DEXT() {
			return newUnlinkedGLFunction("glCopyMultiTexSubImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_IIIII(), MLS_LTextureUnit_LTextureTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glCopyMultiTexSubImage2DEXT() {
			return newUnlinkedGLFunction("glCopyMultiTexSubImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_IIIIIII(), MLS_LTextureUnit_LTextureTarget_IIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyMultiTexSubImage3DEXT() {
			return newUnlinkedGLFunction("glCopyMultiTexSubImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_IIIIIIII(), MLS_LTextureUnit_LTextureTarget_IIIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyNamedBufferSubData() {
			return newUnlinkedGLFunction("glCopyNamedBufferSubData", void.class, GLParamTypeSets.ML_V(), PS_IIJJJ(), MLS_IIJJJ());
		}
		private static UnlinkedGLFunction make_glCopyPathNV() {
			return newUnlinkedGLFunction("glCopyPathNV", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glCopyPixels() {
			return newUnlinkedGLFunction("glCopyPixels", void.class, GLParamTypeSets.ML_V(), PS_IIIILPixelCopyType_(), MLS_IIIILPixelCopyType_());
		}
		private static UnlinkedGLFunction make_glCopyTexImage1D() {
			return newUnlinkedGLFunction("glCopyTexImage1D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIII(), MLS_LTextureTarget_ILInternalFormat_IIII());
		}
		private static UnlinkedGLFunction make_glCopyTexImage1DEXT() {
			return newUnlinkedGLFunction("glCopyTexImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIII(), MLS_LTextureTarget_ILInternalFormat_IIII());
		}
		private static UnlinkedGLFunction make_glCopyTexImage2D() {
			return newUnlinkedGLFunction("glCopyTexImage2D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIII(), MLS_LTextureTarget_ILInternalFormat_IIIII());
		}
		private static UnlinkedGLFunction make_glCopyTexImage2DEXT() {
			return newUnlinkedGLFunction("glCopyTexImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIII(), MLS_LTextureTarget_ILInternalFormat_IIIII());
		}
		private static UnlinkedGLFunction make_glCopyTexSubImage1D() {
			return newUnlinkedGLFunction("glCopyTexSubImage1D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIII(), MLS_LTextureTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glCopyTexSubImage1DEXT() {
			return newUnlinkedGLFunction("glCopyTexSubImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIII(), MLS_LTextureTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glCopyTexSubImage2D() {
			return newUnlinkedGLFunction("glCopyTexSubImage2D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIII(), MLS_LTextureTarget_IIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyTexSubImage2DEXT() {
			return newUnlinkedGLFunction("glCopyTexSubImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIII(), MLS_LTextureTarget_IIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyTexSubImage3D() {
			return newUnlinkedGLFunction("glCopyTexSubImage3D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIIII(), MLS_LTextureTarget_IIIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyTexSubImage3DEXT() {
			return newUnlinkedGLFunction("glCopyTexSubImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIIII(), MLS_LTextureTarget_IIIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyTexSubImage3DOES() {
			return newUnlinkedGLFunction("glCopyTexSubImage3DOES", void.class, GLParamTypeSets.ML_V(), PS_EIIIIIIII(), MLS_EIIIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyTextureImage1DEXT() {
			return newUnlinkedGLFunction("glCopyTextureImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILInternalFormat_IIII(), MLS_ILTextureTarget_ILInternalFormat_IIII());
		}
		private static UnlinkedGLFunction make_glCopyTextureImage2DEXT() {
			return newUnlinkedGLFunction("glCopyTextureImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILInternalFormat_IIIII(), MLS_ILTextureTarget_ILInternalFormat_IIIII());
		}
		private static UnlinkedGLFunction make_glCopyTextureLevelsAPPLE() {
			return newUnlinkedGLFunction("glCopyTextureLevelsAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glCopyTextureSubImage1D() {
			return newUnlinkedGLFunction("glCopyTextureSubImage1D", void.class, GLParamTypeSets.ML_V(), PS_IIIIII(), MLS_IIIIII());
		}
		private static UnlinkedGLFunction make_glCopyTextureSubImage1DEXT() {
			return newUnlinkedGLFunction("glCopyTextureSubImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IIIII(), MLS_ILTextureTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glCopyTextureSubImage2D() {
			return newUnlinkedGLFunction("glCopyTextureSubImage2D", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIII(), MLS_IIIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyTextureSubImage2DEXT() {
			return newUnlinkedGLFunction("glCopyTextureSubImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IIIIIII(), MLS_ILTextureTarget_IIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyTextureSubImage3D() {
			return newUnlinkedGLFunction("glCopyTextureSubImage3D", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIII(), MLS_IIIIIIIII());
		}
		private static UnlinkedGLFunction make_glCopyTextureSubImage3DEXT() {
			return newUnlinkedGLFunction("glCopyTextureSubImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IIIIIIII(), MLS_ILTextureTarget_IIIIIIII());
		}
		private static UnlinkedGLFunction make_glCoverFillPathInstancedNV() {
			return newUnlinkedGLFunction("glCoverFillPathInstancedNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathElementType_pILInstancedPathCoverMode_LPathTransformType_p(), MLS_ILPathElementType_pILInstancedPathCoverMode_LPathTransformType_p());
		}
		private static UnlinkedGLFunction make_glCoverFillPathNV() {
			return newUnlinkedGLFunction("glCoverFillPathNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathCoverMode_(), MLS_ILPathCoverMode_());
		}
		private static UnlinkedGLFunction make_glCoverStrokePathInstancedNV() {
			return newUnlinkedGLFunction("glCoverStrokePathInstancedNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathElementType_pILInstancedPathCoverMode_LPathTransformType_p(), MLS_ILPathElementType_pILInstancedPathCoverMode_LPathTransformType_p());
		}
		private static UnlinkedGLFunction make_glCoverStrokePathNV() {
			return newUnlinkedGLFunction("glCoverStrokePathNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathCoverMode_(), MLS_ILPathCoverMode_());
		}
		private static UnlinkedGLFunction make_glCoverageMaskNV() {
			return newUnlinkedGLFunction("glCoverageMaskNV", void.class, GLParamTypeSets.ML_V(), PS_Z(), MLS_Z());
		}
		private static UnlinkedGLFunction make_glCoverageModulationNV() {
			return newUnlinkedGLFunction("glCoverageModulationNV", void.class, GLParamTypeSets.ML_V(), PS_E(), MLS_E());
		}
		private static UnlinkedGLFunction make_glCoverageModulationTableNV() {
			return newUnlinkedGLFunction("glCoverageModulationTableNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glCoverageOperationNV() {
			return newUnlinkedGLFunction("glCoverageOperationNV", void.class, GLParamTypeSets.ML_V(), PS_E(), MLS_E());
		}
		private static UnlinkedGLFunction make_glCreateBuffers() {
			return newUnlinkedGLFunction("glCreateBuffers", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreateCommandListsNV() {
			return newUnlinkedGLFunction("glCreateCommandListsNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreateFramebuffers() {
			return newUnlinkedGLFunction("glCreateFramebuffers", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreateMemoryObjectsEXT() {
			return newUnlinkedGLFunction("glCreateMemoryObjectsEXT", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreatePerfQueryINTEL() {
			return newUnlinkedGLFunction("glCreatePerfQueryINTEL", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreateProgram() {
			return newUnlinkedGLFunction("glCreateProgram", int.class, GLParamTypeSets.ML_I(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glCreateProgramObjectARB() {
			return newUnlinkedGLFunction("glCreateProgramObjectARB", int.class, GLParamTypeSets.ML_I(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glCreateProgramPipelines() {
			return newUnlinkedGLFunction("glCreateProgramPipelines", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreateProgressFenceNVX() {
			return newUnlinkedGLFunction("glCreateProgressFenceNVX", int.class, GLParamTypeSets.ML_I(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glCreateQueries() {
			return newUnlinkedGLFunction("glCreateQueries", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_IP(), MLS_LQueryTarget_IP());
		}
		private static UnlinkedGLFunction make_glCreateRenderbuffers() {
			return newUnlinkedGLFunction("glCreateRenderbuffers", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreateSamplers() {
			return newUnlinkedGLFunction("glCreateSamplers", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreateSemaphoresNV() {
			return newUnlinkedGLFunction("glCreateSemaphoresNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreateShader() {
			return newUnlinkedGLFunction("glCreateShader", int.class, GLParamTypeSets.ML_I(), PS_LShaderType_(), MLS_LShaderType_());
		}
		private static UnlinkedGLFunction make_glCreateShaderObjectARB() {
			return newUnlinkedGLFunction("glCreateShaderObjectARB", int.class, GLParamTypeSets.ML_I(), PS_LShaderType_(), MLS_LShaderType_());
		}
		private static UnlinkedGLFunction make_glCreateShaderProgramEXT() {
			return newUnlinkedGLFunction("glCreateShaderProgramEXT", int.class, GLParamTypeSets.ML_I(), PS_LShaderType_p(), MLS_LShaderType_p());
		}
		private static UnlinkedGLFunction make_glCreateShaderProgramv() {
			return newUnlinkedGLFunction("glCreateShaderProgramv", int.class, GLParamTypeSets.ML_I(), PS_LShaderType_Ip(), MLS_LShaderType_Ip());
		}
		private static UnlinkedGLFunction make_glCreateShaderProgramvEXT() {
			return newUnlinkedGLFunction("glCreateShaderProgramvEXT", int.class, GLParamTypeSets.ML_I(), PS_LShaderType_Ip(), MLS_LShaderType_Ip());
		}
		private static UnlinkedGLFunction make_glCreateStatesNV() {
			return newUnlinkedGLFunction("glCreateStatesNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreateSyncFromCLeventARB() {
			return newUnlinkedGLFunction("glCreateSyncFromCLeventARB", int.class, GLParamTypeSets.ML_I(), PS_PPI(), MLS_PPI());
		}
		private static UnlinkedGLFunction make_glCreateTextures() {
			return newUnlinkedGLFunction("glCreateTextures", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IP(), MLS_LTextureTarget_IP());
		}
		private static UnlinkedGLFunction make_glCreateTransformFeedbacks() {
			return newUnlinkedGLFunction("glCreateTransformFeedbacks", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCreateVertexArrays() {
			return newUnlinkedGLFunction("glCreateVertexArrays", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glCullFace() {
			return newUnlinkedGLFunction("glCullFace", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_(), MLS_LTriangleFace_());
		}
		private static UnlinkedGLFunction make_glCullParameterdvEXT() {
			return newUnlinkedGLFunction("glCullParameterdvEXT", void.class, GLParamTypeSets.ML_V(), PS_LCullParameterEXT_P(), MLS_LCullParameterEXT_P());
		}
		private static UnlinkedGLFunction make_glCullParameterfvEXT() {
			return newUnlinkedGLFunction("glCullParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LCullParameterEXT_P(), MLS_LCullParameterEXT_P());
		}
		private static UnlinkedGLFunction make_glCurrentPaletteMatrixARB() {
			return newUnlinkedGLFunction("glCurrentPaletteMatrixARB", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glCurrentPaletteMatrixOES() {
			return newUnlinkedGLFunction("glCurrentPaletteMatrixOES", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDebugMessageCallback() {
			return newUnlinkedGLFunction("glDebugMessageCallback", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDebugMessageCallbackAMD() {
			return newUnlinkedGLFunction("glDebugMessageCallbackAMD", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glDebugMessageCallbackARB() {
			return newUnlinkedGLFunction("glDebugMessageCallbackARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDebugMessageCallbackKHR() {
			return newUnlinkedGLFunction("glDebugMessageCallbackKHR", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDebugMessageControl() {
			return newUnlinkedGLFunction("glDebugMessageControl", void.class, GLParamTypeSets.ML_V(), PS_LDebugSource_LDebugType_LDebugSeverity_IpZ(), MLS_LDebugSource_LDebugType_LDebugSeverity_IpZ());
		}
		private static UnlinkedGLFunction make_glDebugMessageControlARB() {
			return newUnlinkedGLFunction("glDebugMessageControlARB", void.class, GLParamTypeSets.ML_V(), PS_LDebugSource_LDebugType_LDebugSeverity_IpZ(), MLS_LDebugSource_LDebugType_LDebugSeverity_IpZ());
		}
		private static UnlinkedGLFunction make_glDebugMessageControlKHR() {
			return newUnlinkedGLFunction("glDebugMessageControlKHR", void.class, GLParamTypeSets.ML_V(), PS_LDebugSource_LDebugType_LDebugSeverity_IpZ(), MLS_LDebugSource_LDebugType_LDebugSeverity_IpZ());
		}
		private static UnlinkedGLFunction make_glDebugMessageEnableAMD() {
			return newUnlinkedGLFunction("glDebugMessageEnableAMD", void.class, GLParamTypeSets.ML_V(), PS_ELDebugSeverity_IpZ(), MLS_ELDebugSeverity_IpZ());
		}
		private static UnlinkedGLFunction make_glDebugMessageInsert() {
			return newUnlinkedGLFunction("glDebugMessageInsert", void.class, GLParamTypeSets.ML_V(), PS_LDebugSource_LDebugType_ILDebugSeverity_Ip(), MLS_LDebugSource_LDebugType_ILDebugSeverity_Ip());
		}
		private static UnlinkedGLFunction make_glDebugMessageInsertAMD() {
			return newUnlinkedGLFunction("glDebugMessageInsertAMD", void.class, GLParamTypeSets.ML_V(), PS_ELDebugSeverity_IIp(), MLS_ELDebugSeverity_IIp());
		}
		private static UnlinkedGLFunction make_glDebugMessageInsertARB() {
			return newUnlinkedGLFunction("glDebugMessageInsertARB", void.class, GLParamTypeSets.ML_V(), PS_LDebugSource_LDebugType_ILDebugSeverity_Ip(), MLS_LDebugSource_LDebugType_ILDebugSeverity_Ip());
		}
		private static UnlinkedGLFunction make_glDebugMessageInsertKHR() {
			return newUnlinkedGLFunction("glDebugMessageInsertKHR", void.class, GLParamTypeSets.ML_V(), PS_LDebugSource_LDebugType_ILDebugSeverity_Ip(), MLS_LDebugSource_LDebugType_ILDebugSeverity_Ip());
		}
		private static UnlinkedGLFunction make_glDeformSGIX() {
			return newUnlinkedGLFunction("glDeformSGIX", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDeformationMap3dSGIX() {
			return newUnlinkedGLFunction("glDeformationMap3dSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFfdTargetSGIX_DDIIDDIIDDIIp(), MLS_LFfdTargetSGIX_DDIIDDIIDDIIp());
		}
		private static UnlinkedGLFunction make_glDeformationMap3fSGIX() {
			return newUnlinkedGLFunction("glDeformationMap3fSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFfdTargetSGIX_FFIIFFIIFFIIp(), MLS_LFfdTargetSGIX_FFIIFFIIFFIIp());
		}
		private static UnlinkedGLFunction make_glDeleteAsyncMarkersSGIX() {
			return newUnlinkedGLFunction("glDeleteAsyncMarkersSGIX", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glDeleteBuffers() {
			return newUnlinkedGLFunction("glDeleteBuffers", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteBuffersARB() {
			return newUnlinkedGLFunction("glDeleteBuffersARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteCommandListsNV() {
			return newUnlinkedGLFunction("glDeleteCommandListsNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteFencesAPPLE() {
			return newUnlinkedGLFunction("glDeleteFencesAPPLE", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteFencesNV() {
			return newUnlinkedGLFunction("glDeleteFencesNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteFragmentShaderATI() {
			return newUnlinkedGLFunction("glDeleteFragmentShaderATI", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDeleteFramebuffers() {
			return newUnlinkedGLFunction("glDeleteFramebuffers", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteFramebuffersEXT() {
			return newUnlinkedGLFunction("glDeleteFramebuffersEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteFramebuffersOES() {
			return newUnlinkedGLFunction("glDeleteFramebuffersOES", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteLists() {
			return newUnlinkedGLFunction("glDeleteLists", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glDeleteMemoryObjectsEXT() {
			return newUnlinkedGLFunction("glDeleteMemoryObjectsEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteNamedStringARB() {
			return newUnlinkedGLFunction("glDeleteNamedStringARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteNamesAMD() {
			return newUnlinkedGLFunction("glDeleteNamesAMD", void.class, GLParamTypeSets.ML_V(), PS_EIp(), MLS_EIp());
		}
		private static UnlinkedGLFunction make_glDeleteObjectARB() {
			return newUnlinkedGLFunction("glDeleteObjectARB", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDeleteOcclusionQueriesNV() {
			return newUnlinkedGLFunction("glDeleteOcclusionQueriesNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeletePathsNV() {
			return newUnlinkedGLFunction("glDeletePathsNV", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glDeletePerfMonitorsAMD() {
			return newUnlinkedGLFunction("glDeletePerfMonitorsAMD", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glDeletePerfQueryINTEL() {
			return newUnlinkedGLFunction("glDeletePerfQueryINTEL", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDeleteProgram() {
			return newUnlinkedGLFunction("glDeleteProgram", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDeleteProgramPipelines() {
			return newUnlinkedGLFunction("glDeleteProgramPipelines", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteProgramPipelinesEXT() {
			return newUnlinkedGLFunction("glDeleteProgramPipelinesEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteProgramsARB() {
			return newUnlinkedGLFunction("glDeleteProgramsARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteProgramsNV() {
			return newUnlinkedGLFunction("glDeleteProgramsNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteQueries() {
			return newUnlinkedGLFunction("glDeleteQueries", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteQueriesARB() {
			return newUnlinkedGLFunction("glDeleteQueriesARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteQueriesEXT() {
			return newUnlinkedGLFunction("glDeleteQueriesEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteQueryResourceTagNV() {
			return newUnlinkedGLFunction("glDeleteQueryResourceTagNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteRenderbuffers() {
			return newUnlinkedGLFunction("glDeleteRenderbuffers", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteRenderbuffersEXT() {
			return newUnlinkedGLFunction("glDeleteRenderbuffersEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteRenderbuffersOES() {
			return newUnlinkedGLFunction("glDeleteRenderbuffersOES", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteSamplers() {
			return newUnlinkedGLFunction("glDeleteSamplers", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteSemaphoresEXT() {
			return newUnlinkedGLFunction("glDeleteSemaphoresEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteShader() {
			return newUnlinkedGLFunction("glDeleteShader", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDeleteStatesNV() {
			return newUnlinkedGLFunction("glDeleteStatesNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteSync() {
			return newUnlinkedGLFunction("glDeleteSync", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDeleteSyncAPPLE() {
			return newUnlinkedGLFunction("glDeleteSyncAPPLE", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDeleteTextures() {
			return newUnlinkedGLFunction("glDeleteTextures", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteTexturesEXT() {
			return newUnlinkedGLFunction("glDeleteTexturesEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteTransformFeedbacks() {
			return newUnlinkedGLFunction("glDeleteTransformFeedbacks", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteTransformFeedbacksNV() {
			return newUnlinkedGLFunction("glDeleteTransformFeedbacksNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteVertexArrays() {
			return newUnlinkedGLFunction("glDeleteVertexArrays", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteVertexArraysAPPLE() {
			return newUnlinkedGLFunction("glDeleteVertexArraysAPPLE", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteVertexArraysOES() {
			return newUnlinkedGLFunction("glDeleteVertexArraysOES", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDeleteVertexShaderEXT() {
			return newUnlinkedGLFunction("glDeleteVertexShaderEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDepthBoundsEXT() {
			return newUnlinkedGLFunction("glDepthBoundsEXT", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glDepthBoundsdNV() {
			return newUnlinkedGLFunction("glDepthBoundsdNV", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glDepthFunc() {
			return newUnlinkedGLFunction("glDepthFunc", void.class, GLParamTypeSets.ML_V(), PS_LDepthFunction_(), MLS_LDepthFunction_());
		}
		private static UnlinkedGLFunction make_glDepthMask() {
			return newUnlinkedGLFunction("glDepthMask", void.class, GLParamTypeSets.ML_V(), PS_Z(), MLS_Z());
		}
		private static UnlinkedGLFunction make_glDepthRange() {
			return newUnlinkedGLFunction("glDepthRange", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glDepthRangeArraydvNV() {
			return newUnlinkedGLFunction("glDepthRangeArraydvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glDepthRangeArrayfvNV() {
			return newUnlinkedGLFunction("glDepthRangeArrayfvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glDepthRangeArrayfvOES() {
			return newUnlinkedGLFunction("glDepthRangeArrayfvOES", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glDepthRangeArrayv() {
			return newUnlinkedGLFunction("glDepthRangeArrayv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glDepthRangeIndexed() {
			return newUnlinkedGLFunction("glDepthRangeIndexed", void.class, GLParamTypeSets.ML_V(), PS_IDD(), MLS_IDD());
		}
		private static UnlinkedGLFunction make_glDepthRangeIndexeddNV() {
			return newUnlinkedGLFunction("glDepthRangeIndexeddNV", void.class, GLParamTypeSets.ML_V(), PS_IDD(), MLS_IDD());
		}
		private static UnlinkedGLFunction make_glDepthRangeIndexedfNV() {
			return newUnlinkedGLFunction("glDepthRangeIndexedfNV", void.class, GLParamTypeSets.ML_V(), PS_IFF(), MLS_IFF());
		}
		private static UnlinkedGLFunction make_glDepthRangeIndexedfOES() {
			return newUnlinkedGLFunction("glDepthRangeIndexedfOES", void.class, GLParamTypeSets.ML_V(), PS_IFF(), MLS_IFF());
		}
		private static UnlinkedGLFunction make_glDepthRangedNV() {
			return newUnlinkedGLFunction("glDepthRangedNV", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glDepthRangef() {
			return newUnlinkedGLFunction("glDepthRangef", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glDepthRangefOES() {
			return newUnlinkedGLFunction("glDepthRangefOES", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glDepthRangex() {
			return newUnlinkedGLFunction("glDepthRangex", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glDepthRangexOES() {
			return newUnlinkedGLFunction("glDepthRangexOES", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glDetachObjectARB() {
			return newUnlinkedGLFunction("glDetachObjectARB", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glDetachShader() {
			return newUnlinkedGLFunction("glDetachShader", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glDetailTexFuncSGIS() {
			return newUnlinkedGLFunction("glDetailTexFuncSGIS", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_Ip(), MLS_LTextureTarget_Ip());
		}
		private static UnlinkedGLFunction make_glDisable() {
			return newUnlinkedGLFunction("glDisable", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_(), MLS_LEnableCap_());
		}
		private static UnlinkedGLFunction make_glDisableClientState() {
			return newUnlinkedGLFunction("glDisableClientState", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_(), MLS_LEnableCap_());
		}
		private static UnlinkedGLFunction make_glDisableClientStateIndexedEXT() {
			return newUnlinkedGLFunction("glDisableClientStateIndexedEXT", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glDisableClientStateiEXT() {
			return newUnlinkedGLFunction("glDisableClientStateiEXT", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glDisableDriverControlQCOM() {
			return newUnlinkedGLFunction("glDisableDriverControlQCOM", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDisableIndexedEXT() {
			return newUnlinkedGLFunction("glDisableIndexedEXT", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glDisableVariantClientStateEXT() {
			return newUnlinkedGLFunction("glDisableVariantClientStateEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDisableVertexArrayAttrib() {
			return newUnlinkedGLFunction("glDisableVertexArrayAttrib", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glDisableVertexArrayAttribEXT() {
			return newUnlinkedGLFunction("glDisableVertexArrayAttribEXT", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glDisableVertexArrayEXT() {
			return newUnlinkedGLFunction("glDisableVertexArrayEXT", void.class, GLParamTypeSets.ML_V(), PS_ILEnableCap_(), MLS_ILEnableCap_());
		}
		private static UnlinkedGLFunction make_glDisableVertexAttribAPPLE() {
			return newUnlinkedGLFunction("glDisableVertexAttribAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IE(), MLS_IE());
		}
		private static UnlinkedGLFunction make_glDisableVertexAttribArray() {
			return newUnlinkedGLFunction("glDisableVertexAttribArray", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDisableVertexAttribArrayARB() {
			return newUnlinkedGLFunction("glDisableVertexAttribArrayARB", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glDisablei() {
			return newUnlinkedGLFunction("glDisablei", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glDisableiEXT() {
			return newUnlinkedGLFunction("glDisableiEXT", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glDisableiNV() {
			return newUnlinkedGLFunction("glDisableiNV", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glDisableiOES() {
			return newUnlinkedGLFunction("glDisableiOES", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glDiscardFramebufferEXT() {
			return newUnlinkedGLFunction("glDiscardFramebufferEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_Ip(), MLS_LFramebufferTarget_Ip());
		}
		private static UnlinkedGLFunction make_glDispatchCompute() {
			return newUnlinkedGLFunction("glDispatchCompute", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glDispatchComputeGroupSizeARB() {
			return newUnlinkedGLFunction("glDispatchComputeGroupSizeARB", void.class, GLParamTypeSets.ML_V(), PS_IIIIII(), MLS_IIIIII());
		}
		private static UnlinkedGLFunction make_glDispatchComputeIndirect() {
			return newUnlinkedGLFunction("glDispatchComputeIndirect", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glDrawArrays() {
			return newUnlinkedGLFunction("glDrawArrays", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_II(), MLS_LPrimitiveType_II());
		}
		private static UnlinkedGLFunction make_glDrawArraysEXT() {
			return newUnlinkedGLFunction("glDrawArraysEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_II(), MLS_LPrimitiveType_II());
		}
		private static UnlinkedGLFunction make_glDrawArraysIndirect() {
			return newUnlinkedGLFunction("glDrawArraysIndirect", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_p(), MLS_LPrimitiveType_p());
		}
		private static UnlinkedGLFunction make_glDrawArraysInstanced() {
			return newUnlinkedGLFunction("glDrawArraysInstanced", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_III(), MLS_LPrimitiveType_III());
		}
		private static UnlinkedGLFunction make_glDrawArraysInstancedANGLE() {
			return newUnlinkedGLFunction("glDrawArraysInstancedANGLE", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_III(), MLS_LPrimitiveType_III());
		}
		private static UnlinkedGLFunction make_glDrawArraysInstancedARB() {
			return newUnlinkedGLFunction("glDrawArraysInstancedARB", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_III(), MLS_LPrimitiveType_III());
		}
		private static UnlinkedGLFunction make_glDrawArraysInstancedBaseInstance() {
			return newUnlinkedGLFunction("glDrawArraysInstancedBaseInstance", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_IIII(), MLS_LPrimitiveType_IIII());
		}
		private static UnlinkedGLFunction make_glDrawArraysInstancedBaseInstanceEXT() {
			return newUnlinkedGLFunction("glDrawArraysInstancedBaseInstanceEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_IIII(), MLS_LPrimitiveType_IIII());
		}
		private static UnlinkedGLFunction make_glDrawArraysInstancedEXT() {
			return newUnlinkedGLFunction("glDrawArraysInstancedEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_III(), MLS_LPrimitiveType_III());
		}
		private static UnlinkedGLFunction make_glDrawArraysInstancedNV() {
			return newUnlinkedGLFunction("glDrawArraysInstancedNV", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_III(), MLS_LPrimitiveType_III());
		}
		private static UnlinkedGLFunction make_glDrawBuffer() {
			return newUnlinkedGLFunction("glDrawBuffer", void.class, GLParamTypeSets.ML_V(), PS_LDrawBufferMode_(), MLS_LDrawBufferMode_());
		}
		private static UnlinkedGLFunction make_glDrawBuffers() {
			return newUnlinkedGLFunction("glDrawBuffers", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDrawBuffersARB() {
			return newUnlinkedGLFunction("glDrawBuffersARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDrawBuffersATI() {
			return newUnlinkedGLFunction("glDrawBuffersATI", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDrawBuffersEXT() {
			return newUnlinkedGLFunction("glDrawBuffersEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDrawBuffersIndexedEXT() {
			return newUnlinkedGLFunction("glDrawBuffersIndexedEXT", void.class, GLParamTypeSets.ML_V(), PS_Ipp(), MLS_Ipp());
		}
		private static UnlinkedGLFunction make_glDrawBuffersNV() {
			return newUnlinkedGLFunction("glDrawBuffersNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glDrawCommandsAddressNV() {
			return newUnlinkedGLFunction("glDrawCommandsAddressNV", void.class, GLParamTypeSets.ML_V(), PS_EppI(), MLS_EppI());
		}
		private static UnlinkedGLFunction make_glDrawCommandsNV() {
			return newUnlinkedGLFunction("glDrawCommandsNV", void.class, GLParamTypeSets.ML_V(), PS_EIppI(), MLS_EIppI());
		}
		private static UnlinkedGLFunction make_glDrawCommandsStatesAddressNV() {
			return newUnlinkedGLFunction("glDrawCommandsStatesAddressNV", void.class, GLParamTypeSets.ML_V(), PS_ppppI(), MLS_ppppI());
		}
		private static UnlinkedGLFunction make_glDrawCommandsStatesNV() {
			return newUnlinkedGLFunction("glDrawCommandsStatesNV", void.class, GLParamTypeSets.ML_V(), PS_IppppI(), MLS_IppppI());
		}
		private static UnlinkedGLFunction make_glDrawElementArrayAPPLE() {
			return newUnlinkedGLFunction("glDrawElementArrayAPPLE", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_II(), MLS_LPrimitiveType_II());
		}
		private static UnlinkedGLFunction make_glDrawElementArrayATI() {
			return newUnlinkedGLFunction("glDrawElementArrayATI", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_I(), MLS_LPrimitiveType_I());
		}
		private static UnlinkedGLFunction make_glDrawElements() {
			return newUnlinkedGLFunction("glDrawElements", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_p(), MLS_LPrimitiveType_ILDrawElementsType_p());
		}
		private static UnlinkedGLFunction make_glDrawElementsBaseVertex() {
			return newUnlinkedGLFunction("glDrawElementsBaseVertex", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pI(), MLS_LPrimitiveType_ILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawElementsBaseVertexEXT() {
			return newUnlinkedGLFunction("glDrawElementsBaseVertexEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pI(), MLS_LPrimitiveType_ILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawElementsBaseVertexOES() {
			return newUnlinkedGLFunction("glDrawElementsBaseVertexOES", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pI(), MLS_LPrimitiveType_ILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawElementsIndirect() {
			return newUnlinkedGLFunction("glDrawElementsIndirect", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_LDrawElementsType_p(), MLS_LPrimitiveType_LDrawElementsType_p());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstanced() {
			return newUnlinkedGLFunction("glDrawElementsInstanced", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pI(), MLS_LPrimitiveType_ILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedANGLE() {
			return newUnlinkedGLFunction("glDrawElementsInstancedANGLE", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pI(), MLS_LPrimitiveType_ILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedARB() {
			return newUnlinkedGLFunction("glDrawElementsInstancedARB", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pI(), MLS_LPrimitiveType_ILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedBaseInstance() {
			return newUnlinkedGLFunction("glDrawElementsInstancedBaseInstance", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pII(), MLS_LPrimitiveType_ILDrawElementsType_pII());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedBaseInstanceEXT() {
			return newUnlinkedGLFunction("glDrawElementsInstancedBaseInstanceEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pII(), MLS_LPrimitiveType_ILDrawElementsType_pII());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedBaseVertex() {
			return newUnlinkedGLFunction("glDrawElementsInstancedBaseVertex", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pII(), MLS_LPrimitiveType_ILDrawElementsType_pII());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedBaseVertexBaseInstance() {
			return newUnlinkedGLFunction("glDrawElementsInstancedBaseVertexBaseInstance", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pIII(), MLS_LPrimitiveType_ILDrawElementsType_pIII());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedBaseVertexBaseInstanceEXT() {
			return newUnlinkedGLFunction("glDrawElementsInstancedBaseVertexBaseInstanceEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pIII(), MLS_LPrimitiveType_ILDrawElementsType_pIII());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedBaseVertexEXT() {
			return newUnlinkedGLFunction("glDrawElementsInstancedBaseVertexEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pII(), MLS_LPrimitiveType_ILDrawElementsType_pII());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedBaseVertexOES() {
			return newUnlinkedGLFunction("glDrawElementsInstancedBaseVertexOES", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pII(), MLS_LPrimitiveType_ILDrawElementsType_pII());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedEXT() {
			return newUnlinkedGLFunction("glDrawElementsInstancedEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pI(), MLS_LPrimitiveType_ILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawElementsInstancedNV() {
			return newUnlinkedGLFunction("glDrawElementsInstancedNV", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ILDrawElementsType_pI(), MLS_LPrimitiveType_ILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawMeshArraysSUN() {
			return newUnlinkedGLFunction("glDrawMeshArraysSUN", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_III(), MLS_LPrimitiveType_III());
		}
		private static UnlinkedGLFunction make_glDrawMeshTasksNV() {
			return newUnlinkedGLFunction("glDrawMeshTasksNV", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glDrawMeshTasksIndirectNV() {
			return newUnlinkedGLFunction("glDrawMeshTasksIndirectNV", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glDrawPixels() {
			return newUnlinkedGLFunction("glDrawPixels", void.class, GLParamTypeSets.ML_V(), PS_IILPixelFormat_LPixelType_p(), MLS_IILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glDrawRangeElementArrayAPPLE() {
			return newUnlinkedGLFunction("glDrawRangeElementArrayAPPLE", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_IIII(), MLS_LPrimitiveType_IIII());
		}
		private static UnlinkedGLFunction make_glDrawRangeElementArrayATI() {
			return newUnlinkedGLFunction("glDrawRangeElementArrayATI", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_III(), MLS_LPrimitiveType_III());
		}
		private static UnlinkedGLFunction make_glDrawRangeElements() {
			return newUnlinkedGLFunction("glDrawRangeElements", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_IIILDrawElementsType_p(), MLS_LPrimitiveType_IIILDrawElementsType_p());
		}
		private static UnlinkedGLFunction make_glDrawRangeElementsBaseVertex() {
			return newUnlinkedGLFunction("glDrawRangeElementsBaseVertex", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_IIILDrawElementsType_pI(), MLS_LPrimitiveType_IIILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawRangeElementsBaseVertexEXT() {
			return newUnlinkedGLFunction("glDrawRangeElementsBaseVertexEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_IIILDrawElementsType_pI(), MLS_LPrimitiveType_IIILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawRangeElementsBaseVertexOES() {
			return newUnlinkedGLFunction("glDrawRangeElementsBaseVertexOES", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_IIILDrawElementsType_pI(), MLS_LPrimitiveType_IIILDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glDrawRangeElementsEXT() {
			return newUnlinkedGLFunction("glDrawRangeElementsEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_IIILDrawElementsType_p(), MLS_LPrimitiveType_IIILDrawElementsType_p());
		}
		private static UnlinkedGLFunction make_glDrawTexfOES() {
			return newUnlinkedGLFunction("glDrawTexfOES", void.class, GLParamTypeSets.ML_V(), PS_FFFFF(), MLS_FFFFF());
		}
		private static UnlinkedGLFunction make_glDrawTexfvOES() {
			return newUnlinkedGLFunction("glDrawTexfvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glDrawTexiOES() {
			return newUnlinkedGLFunction("glDrawTexiOES", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glDrawTexivOES() {
			return newUnlinkedGLFunction("glDrawTexivOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glDrawTexsOES() {
			return newUnlinkedGLFunction("glDrawTexsOES", void.class, GLParamTypeSets.ML_V(), PS_SSSSS(), MLS_SSSSS());
		}
		private static UnlinkedGLFunction make_glDrawTexsvOES() {
			return newUnlinkedGLFunction("glDrawTexsvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glDrawTextureNV() {
			return newUnlinkedGLFunction("glDrawTextureNV", void.class, GLParamTypeSets.ML_V(), PS_IIFFFFFFFFF(), MLS_IIFFFFFFFFF());
		}
		private static UnlinkedGLFunction make_glDrawTexxOES() {
			return newUnlinkedGLFunction("glDrawTexxOES", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glDrawTexxvOES() {
			return newUnlinkedGLFunction("glDrawTexxvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glDrawTransformFeedback() {
			return newUnlinkedGLFunction("glDrawTransformFeedback", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_I(), MLS_LPrimitiveType_I());
		}
		private static UnlinkedGLFunction make_glDrawTransformFeedbackEXT() {
			return newUnlinkedGLFunction("glDrawTransformFeedbackEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_I(), MLS_LPrimitiveType_I());
		}
		private static UnlinkedGLFunction make_glDrawTransformFeedbackInstanced() {
			return newUnlinkedGLFunction("glDrawTransformFeedbackInstanced", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_II(), MLS_LPrimitiveType_II());
		}
		private static UnlinkedGLFunction make_glDrawTransformFeedbackInstancedEXT() {
			return newUnlinkedGLFunction("glDrawTransformFeedbackInstancedEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_II(), MLS_LPrimitiveType_II());
		}
		private static UnlinkedGLFunction make_glDrawTransformFeedbackNV() {
			return newUnlinkedGLFunction("glDrawTransformFeedbackNV", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_I(), MLS_LPrimitiveType_I());
		}
		private static UnlinkedGLFunction make_glDrawTransformFeedbackStream() {
			return newUnlinkedGLFunction("glDrawTransformFeedbackStream", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_II(), MLS_LPrimitiveType_II());
		}
		private static UnlinkedGLFunction make_glDrawTransformFeedbackStreamInstanced() {
			return newUnlinkedGLFunction("glDrawTransformFeedbackStreamInstanced", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_III(), MLS_LPrimitiveType_III());
		}
		private static UnlinkedGLFunction make_glEGLImageTargetRenderbufferStorageOES() {
			return newUnlinkedGLFunction("glEGLImageTargetRenderbufferStorageOES", void.class, GLParamTypeSets.ML_V(), PS_EI(), MLS_EI());
		}
		private static UnlinkedGLFunction make_glEGLImageTargetTexStorageEXT() {
			return newUnlinkedGLFunction("glEGLImageTargetTexStorageEXT", void.class, GLParamTypeSets.ML_V(), PS_EIp(), MLS_EIp());
		}
		private static UnlinkedGLFunction make_glEGLImageTargetTexture2DOES() {
			return newUnlinkedGLFunction("glEGLImageTargetTexture2DOES", void.class, GLParamTypeSets.ML_V(), PS_EI(), MLS_EI());
		}
		private static UnlinkedGLFunction make_glEGLImageTargetTextureStorageEXT() {
			return newUnlinkedGLFunction("glEGLImageTargetTextureStorageEXT", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glEdgeFlag() {
			return newUnlinkedGLFunction("glEdgeFlag", void.class, GLParamTypeSets.ML_V(), PS_Z(), MLS_Z());
		}
		private static UnlinkedGLFunction make_glEdgeFlagFormatNV() {
			return newUnlinkedGLFunction("glEdgeFlagFormatNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEdgeFlagPointer() {
			return newUnlinkedGLFunction("glEdgeFlagPointer", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glEdgeFlagPointerEXT() {
			return newUnlinkedGLFunction("glEdgeFlagPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glEdgeFlagPointerListIBM() {
			return newUnlinkedGLFunction("glEdgeFlagPointerListIBM", void.class, GLParamTypeSets.ML_V(), PS_IPI(), MLS_IPI());
		}
		private static UnlinkedGLFunction make_glEdgeFlagv() {
			return newUnlinkedGLFunction("glEdgeFlagv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glElementPointerAPPLE() {
			return newUnlinkedGLFunction("glElementPointerAPPLE", void.class, GLParamTypeSets.ML_V(), PS_LElementPointerTypeATI_p(), MLS_LElementPointerTypeATI_p());
		}
		private static UnlinkedGLFunction make_glElementPointerATI() {
			return newUnlinkedGLFunction("glElementPointerATI", void.class, GLParamTypeSets.ML_V(), PS_LElementPointerTypeATI_p(), MLS_LElementPointerTypeATI_p());
		}
		private static UnlinkedGLFunction make_glEnable() {
			return newUnlinkedGLFunction("glEnable", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_(), MLS_LEnableCap_());
		}
		private static UnlinkedGLFunction make_glEnableClientState() {
			return newUnlinkedGLFunction("glEnableClientState", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_(), MLS_LEnableCap_());
		}
		private static UnlinkedGLFunction make_glEnableClientStateIndexedEXT() {
			return newUnlinkedGLFunction("glEnableClientStateIndexedEXT", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glEnableClientStateiEXT() {
			return newUnlinkedGLFunction("glEnableClientStateiEXT", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glEnableDriverControlQCOM() {
			return newUnlinkedGLFunction("glEnableDriverControlQCOM", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEnableIndexedEXT() {
			return newUnlinkedGLFunction("glEnableIndexedEXT", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glEnableVariantClientStateEXT() {
			return newUnlinkedGLFunction("glEnableVariantClientStateEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEnableVertexArrayAttrib() {
			return newUnlinkedGLFunction("glEnableVertexArrayAttrib", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glEnableVertexArrayAttribEXT() {
			return newUnlinkedGLFunction("glEnableVertexArrayAttribEXT", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glEnableVertexArrayEXT() {
			return newUnlinkedGLFunction("glEnableVertexArrayEXT", void.class, GLParamTypeSets.ML_V(), PS_ILEnableCap_(), MLS_ILEnableCap_());
		}
		private static UnlinkedGLFunction make_glEnableVertexAttribAPPLE() {
			return newUnlinkedGLFunction("glEnableVertexAttribAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IE(), MLS_IE());
		}
		private static UnlinkedGLFunction make_glEnableVertexAttribArray() {
			return newUnlinkedGLFunction("glEnableVertexAttribArray", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEnableVertexAttribArrayARB() {
			return newUnlinkedGLFunction("glEnableVertexAttribArrayARB", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEnablei() {
			return newUnlinkedGLFunction("glEnablei", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glEnableiEXT() {
			return newUnlinkedGLFunction("glEnableiEXT", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glEnableiNV() {
			return newUnlinkedGLFunction("glEnableiNV", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glEnableiOES() {
			return newUnlinkedGLFunction("glEnableiOES", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glEnd() {
			return newUnlinkedGLFunction("glEnd", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndConditionalRender() {
			return newUnlinkedGLFunction("glEndConditionalRender", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndConditionalRenderNV() {
			return newUnlinkedGLFunction("glEndConditionalRenderNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndConditionalRenderNVX() {
			return newUnlinkedGLFunction("glEndConditionalRenderNVX", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndFragmentShaderATI() {
			return newUnlinkedGLFunction("glEndFragmentShaderATI", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndList() {
			return newUnlinkedGLFunction("glEndList", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndOcclusionQueryNV() {
			return newUnlinkedGLFunction("glEndOcclusionQueryNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndPerfMonitorAMD() {
			return newUnlinkedGLFunction("glEndPerfMonitorAMD", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEndPerfQueryINTEL() {
			return newUnlinkedGLFunction("glEndPerfQueryINTEL", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEndQuery() {
			return newUnlinkedGLFunction("glEndQuery", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_(), MLS_LQueryTarget_());
		}
		private static UnlinkedGLFunction make_glEndQueryARB() {
			return newUnlinkedGLFunction("glEndQueryARB", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_(), MLS_LQueryTarget_());
		}
		private static UnlinkedGLFunction make_glEndQueryEXT() {
			return newUnlinkedGLFunction("glEndQueryEXT", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_(), MLS_LQueryTarget_());
		}
		private static UnlinkedGLFunction make_glEndQueryIndexed() {
			return newUnlinkedGLFunction("glEndQueryIndexed", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_I(), MLS_LQueryTarget_I());
		}
		private static UnlinkedGLFunction make_glEndTilingQCOM() {
			return newUnlinkedGLFunction("glEndTilingQCOM", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEndTransformFeedback() {
			return newUnlinkedGLFunction("glEndTransformFeedback", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndTransformFeedbackEXT() {
			return newUnlinkedGLFunction("glEndTransformFeedbackEXT", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndTransformFeedbackNV() {
			return newUnlinkedGLFunction("glEndTransformFeedbackNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndVertexShaderEXT() {
			return newUnlinkedGLFunction("glEndVertexShaderEXT", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glEndVideoCaptureNV() {
			return newUnlinkedGLFunction("glEndVideoCaptureNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEvalCoord1d() {
			return newUnlinkedGLFunction("glEvalCoord1d", void.class, GLParamTypeSets.ML_V(), PS_D(), MLS_D());
		}
		private static UnlinkedGLFunction make_glEvalCoord1dv() {
			return newUnlinkedGLFunction("glEvalCoord1dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glEvalCoord1f() {
			return newUnlinkedGLFunction("glEvalCoord1f", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glEvalCoord1fv() {
			return newUnlinkedGLFunction("glEvalCoord1fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glEvalCoord1xOES() {
			return newUnlinkedGLFunction("glEvalCoord1xOES", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEvalCoord1xvOES() {
			return newUnlinkedGLFunction("glEvalCoord1xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glEvalCoord2d() {
			return newUnlinkedGLFunction("glEvalCoord2d", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glEvalCoord2dv() {
			return newUnlinkedGLFunction("glEvalCoord2dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glEvalCoord2f() {
			return newUnlinkedGLFunction("glEvalCoord2f", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glEvalCoord2fv() {
			return newUnlinkedGLFunction("glEvalCoord2fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glEvalCoord2xOES() {
			return newUnlinkedGLFunction("glEvalCoord2xOES", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glEvalCoord2xvOES() {
			return newUnlinkedGLFunction("glEvalCoord2xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glEvalMapsNV() {
			return newUnlinkedGLFunction("glEvalMapsNV", void.class, GLParamTypeSets.ML_V(), PS_LEvalTargetNV_LEvalMapsModeNV_(), MLS_LEvalTargetNV_LEvalMapsModeNV_());
		}
		private static UnlinkedGLFunction make_glEvalMesh1() {
			return newUnlinkedGLFunction("glEvalMesh1", void.class, GLParamTypeSets.ML_V(), PS_LMeshMode1_II(), MLS_LMeshMode1_II());
		}
		private static UnlinkedGLFunction make_glEvalMesh2() {
			return newUnlinkedGLFunction("glEvalMesh2", void.class, GLParamTypeSets.ML_V(), PS_LMeshMode2_IIII(), MLS_LMeshMode2_IIII());
		}
		private static UnlinkedGLFunction make_glEvalPoint1() {
			return newUnlinkedGLFunction("glEvalPoint1", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glEvalPoint2() {
			return newUnlinkedGLFunction("glEvalPoint2", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glEvaluateDepthValuesARB() {
			return newUnlinkedGLFunction("glEvaluateDepthValuesARB", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glExecuteProgramNV() {
			return newUnlinkedGLFunction("glExecuteProgramNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_Ip(), MLS_LVertexAttribEnumNV_Ip());
		}
		private static UnlinkedGLFunction make_glExtGetBufferPointervQCOM() {
			return newUnlinkedGLFunction("glExtGetBufferPointervQCOM", void.class, GLParamTypeSets.ML_V(), PS_EP(), MLS_EP());
		}
		private static UnlinkedGLFunction make_glExtGetBuffersQCOM() {
			return newUnlinkedGLFunction("glExtGetBuffersQCOM", void.class, GLParamTypeSets.ML_V(), PS_PIP(), MLS_PIP());
		}
		private static UnlinkedGLFunction make_glExtGetFramebuffersQCOM() {
			return newUnlinkedGLFunction("glExtGetFramebuffersQCOM", void.class, GLParamTypeSets.ML_V(), PS_PIP(), MLS_PIP());
		}
		private static UnlinkedGLFunction make_glExtGetProgramBinarySourceQCOM() {
			return newUnlinkedGLFunction("glExtGetProgramBinarySourceQCOM", void.class, GLParamTypeSets.ML_V(), PS_ILShaderType_PP(), MLS_ILShaderType_PP());
		}
		private static UnlinkedGLFunction make_glExtGetProgramsQCOM() {
			return newUnlinkedGLFunction("glExtGetProgramsQCOM", void.class, GLParamTypeSets.ML_V(), PS_PIP(), MLS_PIP());
		}
		private static UnlinkedGLFunction make_glExtGetRenderbuffersQCOM() {
			return newUnlinkedGLFunction("glExtGetRenderbuffersQCOM", void.class, GLParamTypeSets.ML_V(), PS_PIP(), MLS_PIP());
		}
		private static UnlinkedGLFunction make_glExtGetShadersQCOM() {
			return newUnlinkedGLFunction("glExtGetShadersQCOM", void.class, GLParamTypeSets.ML_V(), PS_PIP(), MLS_PIP());
		}
		private static UnlinkedGLFunction make_glExtGetTexLevelParameterivQCOM() {
			return newUnlinkedGLFunction("glExtGetTexLevelParameterivQCOM", void.class, GLParamTypeSets.ML_V(), PS_IEIEP(), MLS_IEIEP());
		}
		private static UnlinkedGLFunction make_glExtGetTexSubImageQCOM() {
			return newUnlinkedGLFunction("glExtGetTexSubImageQCOM", void.class, GLParamTypeSets.ML_V(), PS_EIIIIIIILPixelFormat_LPixelType_P(), MLS_EIIIIIIILPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glExtGetTexturesQCOM() {
			return newUnlinkedGLFunction("glExtGetTexturesQCOM", void.class, GLParamTypeSets.ML_V(), PS_PIP(), MLS_PIP());
		}
		private static UnlinkedGLFunction make_glExtIsProgramBinaryQCOM() {
			return newUnlinkedGLFunction("glExtIsProgramBinaryQCOM", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glExtTexObjectStateOverrideiQCOM() {
			return newUnlinkedGLFunction("glExtTexObjectStateOverrideiQCOM", void.class, GLParamTypeSets.ML_V(), PS_EEI(), MLS_EEI());
		}
		private static UnlinkedGLFunction make_glExtractComponentEXT() {
			return newUnlinkedGLFunction("glExtractComponentEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glFeedbackBuffer() {
			return newUnlinkedGLFunction("glFeedbackBuffer", void.class, GLParamTypeSets.ML_V(), PS_ILFeedbackType_P(), MLS_ILFeedbackType_P());
		}
		private static UnlinkedGLFunction make_glFeedbackBufferxOES() {
			return newUnlinkedGLFunction("glFeedbackBufferxOES", void.class, GLParamTypeSets.ML_V(), PS_IEp(), MLS_IEp());
		}
		private static UnlinkedGLFunction make_glFenceSync() {
			return newUnlinkedGLFunction("glFenceSync", int.class, GLParamTypeSets.ML_I(), PS_LSyncCondition_I(), MLS_LSyncCondition_I());
		}
		private static UnlinkedGLFunction make_glFenceSyncAPPLE() {
			return newUnlinkedGLFunction("glFenceSyncAPPLE", int.class, GLParamTypeSets.ML_I(), PS_LSyncCondition_I(), MLS_LSyncCondition_I());
		}
		private static UnlinkedGLFunction make_glFinalCombinerInputNV() {
			return newUnlinkedGLFunction("glFinalCombinerInputNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerVariableNV_LCombinerRegisterNV_LCombinerMappingNV_LCombinerComponentUsageNV_(), MLS_LCombinerVariableNV_LCombinerRegisterNV_LCombinerMappingNV_LCombinerComponentUsageNV_());
		}
		private static UnlinkedGLFunction make_glFinish() {
			return newUnlinkedGLFunction("glFinish", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glFinishAsyncSGIX() {
			return newUnlinkedGLFunction("glFinishAsyncSGIX", int.class, GLParamTypeSets.ML_I(), PS_P(), MLS_P());
		}
		private static UnlinkedGLFunction make_glFinishFenceAPPLE() {
			return newUnlinkedGLFunction("glFinishFenceAPPLE", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glFinishFenceNV() {
			return newUnlinkedGLFunction("glFinishFenceNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glFinishObjectAPPLE() {
			return newUnlinkedGLFunction("glFinishObjectAPPLE", void.class, GLParamTypeSets.ML_V(), PS_LObjectTypeAPPLE_I(), MLS_LObjectTypeAPPLE_I());
		}
		private static UnlinkedGLFunction make_glFinishTextureSUNX() {
			return newUnlinkedGLFunction("glFinishTextureSUNX", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glFlush() {
			return newUnlinkedGLFunction("glFlush", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glFlushMappedBufferRange() {
			return newUnlinkedGLFunction("glFlushMappedBufferRange", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_JJ(), MLS_LBufferTarget_JJ());
		}
		private static UnlinkedGLFunction make_glFlushMappedBufferRangeAPPLE() {
			return newUnlinkedGLFunction("glFlushMappedBufferRangeAPPLE", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_JJ(), MLS_LBufferTarget_JJ());
		}
		private static UnlinkedGLFunction make_glFlushMappedBufferRangeEXT() {
			return newUnlinkedGLFunction("glFlushMappedBufferRangeEXT", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_JJ(), MLS_LBufferTarget_JJ());
		}
		private static UnlinkedGLFunction make_glFlushMappedNamedBufferRange() {
			return newUnlinkedGLFunction("glFlushMappedNamedBufferRange", void.class, GLParamTypeSets.ML_V(), PS_IJJ(), MLS_IJJ());
		}
		private static UnlinkedGLFunction make_glFlushMappedNamedBufferRangeEXT() {
			return newUnlinkedGLFunction("glFlushMappedNamedBufferRangeEXT", void.class, GLParamTypeSets.ML_V(), PS_IJJ(), MLS_IJJ());
		}
		private static UnlinkedGLFunction make_glFlushPixelDataRangeNV() {
			return newUnlinkedGLFunction("glFlushPixelDataRangeNV", void.class, GLParamTypeSets.ML_V(), PS_LPixelDataRangeTargetNV_(), MLS_LPixelDataRangeTargetNV_());
		}
		private static UnlinkedGLFunction make_glFlushRasterSGIX() {
			return newUnlinkedGLFunction("glFlushRasterSGIX", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glFlushStaticDataIBM() {
			return newUnlinkedGLFunction("glFlushStaticDataIBM", void.class, GLParamTypeSets.ML_V(), PS_E(), MLS_E());
		}
		private static UnlinkedGLFunction make_glFlushVertexArrayRangeAPPLE() {
			return newUnlinkedGLFunction("glFlushVertexArrayRangeAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glFlushVertexArrayRangeNV() {
			return newUnlinkedGLFunction("glFlushVertexArrayRangeNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glFogCoordFormatNV() {
			return newUnlinkedGLFunction("glFogCoordFormatNV", void.class, GLParamTypeSets.ML_V(), PS_EI(), MLS_EI());
		}
		private static UnlinkedGLFunction make_glFogCoordPointer() {
			return newUnlinkedGLFunction("glFogCoordPointer", void.class, GLParamTypeSets.ML_V(), PS_LFogPointerTypeEXT_Ip(), MLS_LFogPointerTypeEXT_Ip());
		}
		private static UnlinkedGLFunction make_glFogCoordPointerEXT() {
			return newUnlinkedGLFunction("glFogCoordPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_LFogPointerTypeEXT_Ip(), MLS_LFogPointerTypeEXT_Ip());
		}
		private static UnlinkedGLFunction make_glFogCoordPointerListIBM() {
			return newUnlinkedGLFunction("glFogCoordPointerListIBM", void.class, GLParamTypeSets.ML_V(), PS_LFogPointerTypeIBM_IPI(), MLS_LFogPointerTypeIBM_IPI());
		}
		private static UnlinkedGLFunction make_glFogCoordd() {
			return newUnlinkedGLFunction("glFogCoordd", void.class, GLParamTypeSets.ML_V(), PS_D(), MLS_D());
		}
		private static UnlinkedGLFunction make_glFogCoorddEXT() {
			return newUnlinkedGLFunction("glFogCoorddEXT", void.class, GLParamTypeSets.ML_V(), PS_D(), MLS_D());
		}
		private static UnlinkedGLFunction make_glFogCoorddv() {
			return newUnlinkedGLFunction("glFogCoorddv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glFogCoorddvEXT() {
			return newUnlinkedGLFunction("glFogCoorddvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glFogCoordf() {
			return newUnlinkedGLFunction("glFogCoordf", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glFogCoordfEXT() {
			return newUnlinkedGLFunction("glFogCoordfEXT", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glFogCoordfv() {
			return newUnlinkedGLFunction("glFogCoordfv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glFogCoordfvEXT() {
			return newUnlinkedGLFunction("glFogCoordfvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glFogCoordhNV() {
			return newUnlinkedGLFunction("glFogCoordhNV", void.class, GLParamTypeSets.ML_V(), PS_C(), MLS_C());
		}
		private static UnlinkedGLFunction make_glFogCoordhvNV() {
			return newUnlinkedGLFunction("glFogCoordhvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glFogFuncSGIS() {
			return newUnlinkedGLFunction("glFogFuncSGIS", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glFogf() {
			return newUnlinkedGLFunction("glFogf", void.class, GLParamTypeSets.ML_V(), PS_LFogParameter_F(), MLS_LFogParameter_F());
		}
		private static UnlinkedGLFunction make_glFogfv() {
			return newUnlinkedGLFunction("glFogfv", void.class, GLParamTypeSets.ML_V(), PS_LFogParameter_p(), MLS_LFogParameter_p());
		}
		private static UnlinkedGLFunction make_glFogi() {
			return newUnlinkedGLFunction("glFogi", void.class, GLParamTypeSets.ML_V(), PS_LFogParameter_I(), MLS_LFogParameter_I());
		}
		private static UnlinkedGLFunction make_glFogiv() {
			return newUnlinkedGLFunction("glFogiv", void.class, GLParamTypeSets.ML_V(), PS_LFogParameter_p(), MLS_LFogParameter_p());
		}
		private static UnlinkedGLFunction make_glFogx() {
			return newUnlinkedGLFunction("glFogx", void.class, GLParamTypeSets.ML_V(), PS_LFogPName_I(), MLS_LFogPName_I());
		}
		private static UnlinkedGLFunction make_glFogxOES() {
			return newUnlinkedGLFunction("glFogxOES", void.class, GLParamTypeSets.ML_V(), PS_LFogPName_I(), MLS_LFogPName_I());
		}
		private static UnlinkedGLFunction make_glFogxv() {
			return newUnlinkedGLFunction("glFogxv", void.class, GLParamTypeSets.ML_V(), PS_LFogPName_p(), MLS_LFogPName_p());
		}
		private static UnlinkedGLFunction make_glFogxvOES() {
			return newUnlinkedGLFunction("glFogxvOES", void.class, GLParamTypeSets.ML_V(), PS_LFogPName_p(), MLS_LFogPName_p());
		}
		private static UnlinkedGLFunction make_glFragmentColorMaterialSGIX() {
			return newUnlinkedGLFunction("glFragmentColorMaterialSGIX", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_(), MLS_LTriangleFace_LMaterialParameter_());
		}
		private static UnlinkedGLFunction make_glFragmentCoverageColorNV() {
			return newUnlinkedGLFunction("glFragmentCoverageColorNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glFragmentLightModelfSGIX() {
			return newUnlinkedGLFunction("glFragmentLightModelfSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFragmentLightModelParameterSGIX_F(), MLS_LFragmentLightModelParameterSGIX_F());
		}
		private static UnlinkedGLFunction make_glFragmentLightModelfvSGIX() {
			return newUnlinkedGLFunction("glFragmentLightModelfvSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFragmentLightModelParameterSGIX_p(), MLS_LFragmentLightModelParameterSGIX_p());
		}
		private static UnlinkedGLFunction make_glFragmentLightModeliSGIX() {
			return newUnlinkedGLFunction("glFragmentLightModeliSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFragmentLightModelParameterSGIX_I(), MLS_LFragmentLightModelParameterSGIX_I());
		}
		private static UnlinkedGLFunction make_glFragmentLightModelivSGIX() {
			return newUnlinkedGLFunction("glFragmentLightModelivSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFragmentLightModelParameterSGIX_p(), MLS_LFragmentLightModelParameterSGIX_p());
		}
		private static UnlinkedGLFunction make_glFragmentLightfSGIX() {
			return newUnlinkedGLFunction("glFragmentLightfSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_F(), MLS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_F());
		}
		private static UnlinkedGLFunction make_glFragmentLightfvSGIX() {
			return newUnlinkedGLFunction("glFragmentLightfvSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_p(), MLS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_p());
		}
		private static UnlinkedGLFunction make_glFragmentLightiSGIX() {
			return newUnlinkedGLFunction("glFragmentLightiSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_I(), MLS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_I());
		}
		private static UnlinkedGLFunction make_glFragmentLightivSGIX() {
			return newUnlinkedGLFunction("glFragmentLightivSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_p(), MLS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_p());
		}
		private static UnlinkedGLFunction make_glFragmentMaterialfSGIX() {
			return newUnlinkedGLFunction("glFragmentMaterialfSGIX", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_F(), MLS_LTriangleFace_LMaterialParameter_F());
		}
		private static UnlinkedGLFunction make_glFragmentMaterialfvSGIX() {
			return newUnlinkedGLFunction("glFragmentMaterialfvSGIX", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_p(), MLS_LTriangleFace_LMaterialParameter_p());
		}
		private static UnlinkedGLFunction make_glFragmentMaterialiSGIX() {
			return newUnlinkedGLFunction("glFragmentMaterialiSGIX", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_I(), MLS_LTriangleFace_LMaterialParameter_I());
		}
		private static UnlinkedGLFunction make_glFragmentMaterialivSGIX() {
			return newUnlinkedGLFunction("glFragmentMaterialivSGIX", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_p(), MLS_LTriangleFace_LMaterialParameter_p());
		}
		private static UnlinkedGLFunction make_glFrameTerminatorGREMEDY() {
			return newUnlinkedGLFunction("glFrameTerminatorGREMEDY", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glFrameZoomSGIX() {
			return newUnlinkedGLFunction("glFrameZoomSGIX", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glFramebufferDrawBufferEXT() {
			return newUnlinkedGLFunction("glFramebufferDrawBufferEXT", void.class, GLParamTypeSets.ML_V(), PS_ILDrawBufferMode_(), MLS_ILDrawBufferMode_());
		}
		private static UnlinkedGLFunction make_glFramebufferDrawBuffersEXT() {
			return newUnlinkedGLFunction("glFramebufferDrawBuffersEXT", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glFramebufferFetchBarrierEXT() {
			return newUnlinkedGLFunction("glFramebufferFetchBarrierEXT", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glFramebufferFetchBarrierQCOM() {
			return newUnlinkedGLFunction("glFramebufferFetchBarrierQCOM", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glFramebufferFoveationConfigQCOM() {
			return newUnlinkedGLFunction("glFramebufferFoveationConfigQCOM", void.class, GLParamTypeSets.ML_V(), PS_IIIIP(), MLS_IIIIP());
		}
		private static UnlinkedGLFunction make_glFramebufferFoveationParametersQCOM() {
			return newUnlinkedGLFunction("glFramebufferFoveationParametersQCOM", void.class, GLParamTypeSets.ML_V(), PS_IIIFFFFF(), MLS_IIIFFFFF());
		}
		private static UnlinkedGLFunction make_glFramebufferParameteri() {
			return newUnlinkedGLFunction("glFramebufferParameteri", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferParameterName_I(), MLS_LFramebufferTarget_LFramebufferParameterName_I());
		}
		private static UnlinkedGLFunction make_glFramebufferPixelLocalStorageSizeEXT() {
			return newUnlinkedGLFunction("glFramebufferPixelLocalStorageSizeEXT", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glFramebufferReadBufferEXT() {
			return newUnlinkedGLFunction("glFramebufferReadBufferEXT", void.class, GLParamTypeSets.ML_V(), PS_ILReadBufferMode_(), MLS_ILReadBufferMode_());
		}
		private static UnlinkedGLFunction make_glFramebufferRenderbuffer() {
			return newUnlinkedGLFunction("glFramebufferRenderbuffer", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LRenderbufferTarget_I(), MLS_LFramebufferTarget_LFramebufferAttachment_LRenderbufferTarget_I());
		}
		private static UnlinkedGLFunction make_glFramebufferRenderbufferEXT() {
			return newUnlinkedGLFunction("glFramebufferRenderbufferEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LRenderbufferTarget_I(), MLS_LFramebufferTarget_LFramebufferAttachment_LRenderbufferTarget_I());
		}
		private static UnlinkedGLFunction make_glFramebufferRenderbufferOES() {
			return newUnlinkedGLFunction("glFramebufferRenderbufferOES", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LRenderbufferTarget_I(), MLS_LFramebufferTarget_LFramebufferAttachment_LRenderbufferTarget_I());
		}
		private static UnlinkedGLFunction make_glFramebufferSampleLocationsfvARB() {
			return newUnlinkedGLFunction("glFramebufferSampleLocationsfvARB", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_IIp(), MLS_LFramebufferTarget_IIp());
		}
		private static UnlinkedGLFunction make_glFramebufferSampleLocationsfvNV() {
			return newUnlinkedGLFunction("glFramebufferSampleLocationsfvNV", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_IIp(), MLS_LFramebufferTarget_IIp());
		}
		private static UnlinkedGLFunction make_glFramebufferSamplePositionsfvAMD() {
			return newUnlinkedGLFunction("glFramebufferSamplePositionsfvAMD", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_IIp(), MLS_LFramebufferTarget_IIp());
		}
		private static UnlinkedGLFunction make_glFramebufferShadingRateEXT() {
			return newUnlinkedGLFunction("glFramebufferShadingRateEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_IIIII(), MLS_LFramebufferTarget_LFramebufferAttachment_IIIII());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture() {
			return newUnlinkedGLFunction("glFramebufferTexture", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_II(), MLS_LFramebufferTarget_LFramebufferAttachment_II());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture1D() {
			return newUnlinkedGLFunction("glFramebufferTexture1D", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_II(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_II());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture1DEXT() {
			return newUnlinkedGLFunction("glFramebufferTexture1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_II(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_II());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture2D() {
			return newUnlinkedGLFunction("glFramebufferTexture2D", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_II(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_II());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture2DEXT() {
			return newUnlinkedGLFunction("glFramebufferTexture2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_II(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_II());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture2DDownsampleIMG() {
			return newUnlinkedGLFunction("glFramebufferTexture2DDownsampleIMG", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_IIII(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_IIII());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture2DMultisampleEXT() {
			return newUnlinkedGLFunction("glFramebufferTexture2DMultisampleEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_III(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_III());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture2DMultisampleIMG() {
			return newUnlinkedGLFunction("glFramebufferTexture2DMultisampleIMG", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_III(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_III());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture2DOES() {
			return newUnlinkedGLFunction("glFramebufferTexture2DOES", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_II(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_II());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture3D() {
			return newUnlinkedGLFunction("glFramebufferTexture3D", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_III(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_III());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture3DEXT() {
			return newUnlinkedGLFunction("glFramebufferTexture3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_III(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_III());
		}
		private static UnlinkedGLFunction make_glFramebufferTexture3DOES() {
			return newUnlinkedGLFunction("glFramebufferTexture3DOES", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_III(), MLS_LFramebufferTarget_LFramebufferAttachment_LTextureTarget_III());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureARB() {
			return newUnlinkedGLFunction("glFramebufferTextureARB", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_II(), MLS_LFramebufferTarget_LFramebufferAttachment_II());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureEXT() {
			return newUnlinkedGLFunction("glFramebufferTextureEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_II(), MLS_LFramebufferTarget_LFramebufferAttachment_II());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureFaceARB() {
			return newUnlinkedGLFunction("glFramebufferTextureFaceARB", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_IILTextureTarget_(), MLS_LFramebufferTarget_LFramebufferAttachment_IILTextureTarget_());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureFaceEXT() {
			return newUnlinkedGLFunction("glFramebufferTextureFaceEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_IILTextureTarget_(), MLS_LFramebufferTarget_LFramebufferAttachment_IILTextureTarget_());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureLayer() {
			return newUnlinkedGLFunction("glFramebufferTextureLayer", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_III(), MLS_LFramebufferTarget_LFramebufferAttachment_III());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureLayerARB() {
			return newUnlinkedGLFunction("glFramebufferTextureLayerARB", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_III(), MLS_LFramebufferTarget_LFramebufferAttachment_III());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureLayerEXT() {
			return newUnlinkedGLFunction("glFramebufferTextureLayerEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_III(), MLS_LFramebufferTarget_LFramebufferAttachment_III());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureLayerDownsampleIMG() {
			return newUnlinkedGLFunction("glFramebufferTextureLayerDownsampleIMG", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_IIIII(), MLS_LFramebufferTarget_LFramebufferAttachment_IIIII());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureMultisampleMultiviewOVR() {
			return newUnlinkedGLFunction("glFramebufferTextureMultisampleMultiviewOVR", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_IIIII(), MLS_LFramebufferTarget_LFramebufferAttachment_IIIII());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureMultiviewOVR() {
			return newUnlinkedGLFunction("glFramebufferTextureMultiviewOVR", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_IIII(), MLS_LFramebufferTarget_LFramebufferAttachment_IIII());
		}
		private static UnlinkedGLFunction make_glFramebufferTextureOES() {
			return newUnlinkedGLFunction("glFramebufferTextureOES", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_II(), MLS_LFramebufferTarget_LFramebufferAttachment_II());
		}
		private static UnlinkedGLFunction make_glFreeObjectBufferATI() {
			return newUnlinkedGLFunction("glFreeObjectBufferATI", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glFrontFace() {
			return newUnlinkedGLFunction("glFrontFace", void.class, GLParamTypeSets.ML_V(), PS_LFrontFaceDirection_(), MLS_LFrontFaceDirection_());
		}
		private static UnlinkedGLFunction make_glFrustum() {
			return newUnlinkedGLFunction("glFrustum", void.class, GLParamTypeSets.ML_V(), PS_DDDDDD(), MLS_DDDDDD());
		}
		private static UnlinkedGLFunction make_glFrustumf() {
			return newUnlinkedGLFunction("glFrustumf", void.class, GLParamTypeSets.ML_V(), PS_FFFFFF(), MLS_FFFFFF());
		}
		private static UnlinkedGLFunction make_glFrustumfOES() {
			return newUnlinkedGLFunction("glFrustumfOES", void.class, GLParamTypeSets.ML_V(), PS_FFFFFF(), MLS_FFFFFF());
		}
		private static UnlinkedGLFunction make_glFrustumx() {
			return newUnlinkedGLFunction("glFrustumx", void.class, GLParamTypeSets.ML_V(), PS_IIIIII(), MLS_IIIIII());
		}
		private static UnlinkedGLFunction make_glFrustumxOES() {
			return newUnlinkedGLFunction("glFrustumxOES", void.class, GLParamTypeSets.ML_V(), PS_IIIIII(), MLS_IIIIII());
		}
		private static UnlinkedGLFunction make_glGenAsyncMarkersSGIX() {
			return newUnlinkedGLFunction("glGenAsyncMarkersSGIX", int.class, GLParamTypeSets.ML_I(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGenBuffers() {
			return newUnlinkedGLFunction("glGenBuffers", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenBuffersARB() {
			return newUnlinkedGLFunction("glGenBuffersARB", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenFencesAPPLE() {
			return newUnlinkedGLFunction("glGenFencesAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenFencesNV() {
			return newUnlinkedGLFunction("glGenFencesNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenFragmentShadersATI() {
			return newUnlinkedGLFunction("glGenFragmentShadersATI", int.class, GLParamTypeSets.ML_I(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGenFramebuffers() {
			return newUnlinkedGLFunction("glGenFramebuffers", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenFramebuffersEXT() {
			return newUnlinkedGLFunction("glGenFramebuffersEXT", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenFramebuffersOES() {
			return newUnlinkedGLFunction("glGenFramebuffersOES", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenLists() {
			return newUnlinkedGLFunction("glGenLists", int.class, GLParamTypeSets.ML_I(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGenNamesAMD() {
			return newUnlinkedGLFunction("glGenNamesAMD", void.class, GLParamTypeSets.ML_V(), PS_EIP(), MLS_EIP());
		}
		private static UnlinkedGLFunction make_glGenOcclusionQueriesNV() {
			return newUnlinkedGLFunction("glGenOcclusionQueriesNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenPathsNV() {
			return newUnlinkedGLFunction("glGenPathsNV", int.class, GLParamTypeSets.ML_I(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGenPerfMonitorsAMD() {
			return newUnlinkedGLFunction("glGenPerfMonitorsAMD", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenProgramPipelines() {
			return newUnlinkedGLFunction("glGenProgramPipelines", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenProgramPipelinesEXT() {
			return newUnlinkedGLFunction("glGenProgramPipelinesEXT", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenProgramsARB() {
			return newUnlinkedGLFunction("glGenProgramsARB", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenProgramsNV() {
			return newUnlinkedGLFunction("glGenProgramsNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenQueries() {
			return newUnlinkedGLFunction("glGenQueries", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenQueriesARB() {
			return newUnlinkedGLFunction("glGenQueriesARB", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenQueriesEXT() {
			return newUnlinkedGLFunction("glGenQueriesEXT", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenQueryResourceTagNV() {
			return newUnlinkedGLFunction("glGenQueryResourceTagNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenRenderbuffers() {
			return newUnlinkedGLFunction("glGenRenderbuffers", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenRenderbuffersEXT() {
			return newUnlinkedGLFunction("glGenRenderbuffersEXT", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenRenderbuffersOES() {
			return newUnlinkedGLFunction("glGenRenderbuffersOES", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenSamplers() {
			return newUnlinkedGLFunction("glGenSamplers", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenSemaphoresEXT() {
			return newUnlinkedGLFunction("glGenSemaphoresEXT", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenSymbolsEXT() {
			return newUnlinkedGLFunction("glGenSymbolsEXT", int.class, GLParamTypeSets.ML_I(), PS_LDataTypeEXT_LVertexShaderStorageTypeEXT_LParameterRangeEXT_I(), MLS_LDataTypeEXT_LVertexShaderStorageTypeEXT_LParameterRangeEXT_I());
		}
		private static UnlinkedGLFunction make_glGenTextures() {
			return newUnlinkedGLFunction("glGenTextures", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenTexturesEXT() {
			return newUnlinkedGLFunction("glGenTexturesEXT", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenTransformFeedbacks() {
			return newUnlinkedGLFunction("glGenTransformFeedbacks", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenTransformFeedbacksNV() {
			return newUnlinkedGLFunction("glGenTransformFeedbacksNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenVertexArrays() {
			return newUnlinkedGLFunction("glGenVertexArrays", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenVertexArraysAPPLE() {
			return newUnlinkedGLFunction("glGenVertexArraysAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenVertexArraysOES() {
			return newUnlinkedGLFunction("glGenVertexArraysOES", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGenVertexShadersEXT() {
			return newUnlinkedGLFunction("glGenVertexShadersEXT", int.class, GLParamTypeSets.ML_I(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGenerateMipmap() {
			return newUnlinkedGLFunction("glGenerateMipmap", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_(), MLS_LTextureTarget_());
		}
		private static UnlinkedGLFunction make_glGenerateMipmapEXT() {
			return newUnlinkedGLFunction("glGenerateMipmapEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_(), MLS_LTextureTarget_());
		}
		private static UnlinkedGLFunction make_glGenerateMipmapOES() {
			return newUnlinkedGLFunction("glGenerateMipmapOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_(), MLS_LTextureTarget_());
		}
		private static UnlinkedGLFunction make_glGenerateMultiTexMipmapEXT() {
			return newUnlinkedGLFunction("glGenerateMultiTexMipmapEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_(), MLS_LTextureUnit_LTextureTarget_());
		}
		private static UnlinkedGLFunction make_glGenerateTextureMipmap() {
			return newUnlinkedGLFunction("glGenerateTextureMipmap", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGenerateTextureMipmapEXT() {
			return newUnlinkedGLFunction("glGenerateTextureMipmapEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_(), MLS_ILTextureTarget_());
		}
		private static UnlinkedGLFunction make_glGetActiveAtomicCounterBufferiv() {
			return newUnlinkedGLFunction("glGetActiveAtomicCounterBufferiv", void.class, GLParamTypeSets.ML_V(), PS_IILAtomicCounterBufferPName_P(), MLS_IILAtomicCounterBufferPName_P());
		}
		private static UnlinkedGLFunction make_glGetActiveAttrib() {
			return newUnlinkedGLFunction("glGetActiveAttrib", void.class, GLParamTypeSets.ML_V(), PS_IIIPPPP(), MLS_IIIPPPP());
		}
		private static UnlinkedGLFunction make_glGetActiveAttribARB() {
			return newUnlinkedGLFunction("glGetActiveAttribARB", void.class, GLParamTypeSets.ML_V(), PS_IIIPPPP(), MLS_IIIPPPP());
		}
		private static UnlinkedGLFunction make_glGetActiveSubroutineName() {
			return newUnlinkedGLFunction("glGetActiveSubroutineName", void.class, GLParamTypeSets.ML_V(), PS_ILShaderType_IIPP(), MLS_ILShaderType_IIPP());
		}
		private static UnlinkedGLFunction make_glGetActiveSubroutineUniformName() {
			return newUnlinkedGLFunction("glGetActiveSubroutineUniformName", void.class, GLParamTypeSets.ML_V(), PS_ILShaderType_IIPP(), MLS_ILShaderType_IIPP());
		}
		private static UnlinkedGLFunction make_glGetActiveSubroutineUniformiv() {
			return newUnlinkedGLFunction("glGetActiveSubroutineUniformiv", void.class, GLParamTypeSets.ML_V(), PS_ILShaderType_ILSubroutineParameterName_P(), MLS_ILShaderType_ILSubroutineParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetActiveUniform() {
			return newUnlinkedGLFunction("glGetActiveUniform", void.class, GLParamTypeSets.ML_V(), PS_IIIPPPP(), MLS_IIIPPPP());
		}
		private static UnlinkedGLFunction make_glGetActiveUniformARB() {
			return newUnlinkedGLFunction("glGetActiveUniformARB", void.class, GLParamTypeSets.ML_V(), PS_IIIPPPP(), MLS_IIIPPPP());
		}
		private static UnlinkedGLFunction make_glGetActiveUniformBlockName() {
			return newUnlinkedGLFunction("glGetActiveUniformBlockName", void.class, GLParamTypeSets.ML_V(), PS_IIIPP(), MLS_IIIPP());
		}
		private static UnlinkedGLFunction make_glGetActiveUniformBlockiv() {
			return newUnlinkedGLFunction("glGetActiveUniformBlockiv", void.class, GLParamTypeSets.ML_V(), PS_IILUniformBlockPName_P(), MLS_IILUniformBlockPName_P());
		}
		private static UnlinkedGLFunction make_glGetActiveUniformName() {
			return newUnlinkedGLFunction("glGetActiveUniformName", void.class, GLParamTypeSets.ML_V(), PS_IIIPP(), MLS_IIIPP());
		}
		private static UnlinkedGLFunction make_glGetActiveUniformsiv() {
			return newUnlinkedGLFunction("glGetActiveUniformsiv", void.class, GLParamTypeSets.ML_V(), PS_IIpLUniformPName_P(), MLS_IIpLUniformPName_P());
		}
		private static UnlinkedGLFunction make_glGetActiveVaryingNV() {
			return newUnlinkedGLFunction("glGetActiveVaryingNV", void.class, GLParamTypeSets.ML_V(), PS_IIIPPPP(), MLS_IIIPPPP());
		}
		private static UnlinkedGLFunction make_glGetArrayObjectfvATI() {
			return newUnlinkedGLFunction("glGetArrayObjectfvATI", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_LArrayObjectPNameATI_P(), MLS_LEnableCap_LArrayObjectPNameATI_P());
		}
		private static UnlinkedGLFunction make_glGetArrayObjectivATI() {
			return newUnlinkedGLFunction("glGetArrayObjectivATI", void.class, GLParamTypeSets.ML_V(), PS_LEnableCap_LArrayObjectPNameATI_P(), MLS_LEnableCap_LArrayObjectPNameATI_P());
		}
		private static UnlinkedGLFunction make_glGetAttachedObjectsARB() {
			return newUnlinkedGLFunction("glGetAttachedObjectsARB", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetAttachedShaders() {
			return newUnlinkedGLFunction("glGetAttachedShaders", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetAttribLocation() {
			return newUnlinkedGLFunction("glGetAttribLocation", int.class, GLParamTypeSets.ML_I(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glGetAttribLocationARB() {
			return newUnlinkedGLFunction("glGetAttribLocationARB", int.class, GLParamTypeSets.ML_I(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glGetBooleanIndexedvEXT() {
			return newUnlinkedGLFunction("glGetBooleanIndexedvEXT", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_IP(), MLS_LBufferTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetBooleani_v() {
			return newUnlinkedGLFunction("glGetBooleani_v", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_IP(), MLS_LBufferTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetBooleanv() {
			return newUnlinkedGLFunction("glGetBooleanv", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_P(), MLS_LGetPName_P());
		}
		private static UnlinkedGLFunction make_glGetBufferParameteri64v() {
			return newUnlinkedGLFunction("glGetBufferParameteri64v", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_LBufferPNameARB_P(), MLS_LBufferTarget_LBufferPNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetBufferParameteriv() {
			return newUnlinkedGLFunction("glGetBufferParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_LBufferPNameARB_P(), MLS_LBufferTarget_LBufferPNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetBufferParameterivARB() {
			return newUnlinkedGLFunction("glGetBufferParameterivARB", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_LBufferPNameARB_P(), MLS_LBufferTarget_LBufferPNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetBufferParameterui64vNV() {
			return newUnlinkedGLFunction("glGetBufferParameterui64vNV", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_EP(), MLS_LBufferTarget_EP());
		}
		private static UnlinkedGLFunction make_glGetBufferPointerv() {
			return newUnlinkedGLFunction("glGetBufferPointerv", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_LBufferPointerNameARB_P(), MLS_LBufferTarget_LBufferPointerNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetBufferPointervARB() {
			return newUnlinkedGLFunction("glGetBufferPointervARB", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_LBufferPointerNameARB_P(), MLS_LBufferTarget_LBufferPointerNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetBufferPointervOES() {
			return newUnlinkedGLFunction("glGetBufferPointervOES", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_LBufferPointerNameARB_P(), MLS_LBufferTarget_LBufferPointerNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetBufferSubData() {
			return newUnlinkedGLFunction("glGetBufferSubData", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_JJP(), MLS_LBufferTarget_JJP());
		}
		private static UnlinkedGLFunction make_glGetBufferSubDataARB() {
			return newUnlinkedGLFunction("glGetBufferSubDataARB", void.class, GLParamTypeSets.ML_V(), PS_LBufferTarget_JJP(), MLS_LBufferTarget_JJP());
		}
		private static UnlinkedGLFunction make_glGetClipPlane() {
			return newUnlinkedGLFunction("glGetClipPlane", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_P(), MLS_LClipPlaneName_P());
		}
		private static UnlinkedGLFunction make_glGetClipPlanef() {
			return newUnlinkedGLFunction("glGetClipPlanef", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_P(), MLS_LClipPlaneName_P());
		}
		private static UnlinkedGLFunction make_glGetClipPlanefOES() {
			return newUnlinkedGLFunction("glGetClipPlanefOES", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_P(), MLS_LClipPlaneName_P());
		}
		private static UnlinkedGLFunction make_glGetClipPlanex() {
			return newUnlinkedGLFunction("glGetClipPlanex", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_P(), MLS_LClipPlaneName_P());
		}
		private static UnlinkedGLFunction make_glGetClipPlanexOES() {
			return newUnlinkedGLFunction("glGetClipPlanexOES", void.class, GLParamTypeSets.ML_V(), PS_LClipPlaneName_P(), MLS_LClipPlaneName_P());
		}
		private static UnlinkedGLFunction make_glGetColorTable() {
			return newUnlinkedGLFunction("glGetColorTable", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LPixelFormat_LPixelType_P(), MLS_LColorTableTarget_LPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetColorTableEXT() {
			return newUnlinkedGLFunction("glGetColorTableEXT", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LPixelFormat_LPixelType_P(), MLS_LColorTableTarget_LPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetColorTableParameterfv() {
			return newUnlinkedGLFunction("glGetColorTableParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LColorTableParameterPName_P(), MLS_LColorTableTarget_LColorTableParameterPName_P());
		}
		private static UnlinkedGLFunction make_glGetColorTableParameterfvEXT() {
			return newUnlinkedGLFunction("glGetColorTableParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LColorTableParameterPName_P(), MLS_LColorTableTarget_LColorTableParameterPName_P());
		}
		private static UnlinkedGLFunction make_glGetColorTableParameterfvSGI() {
			return newUnlinkedGLFunction("glGetColorTableParameterfvSGI", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTargetSGI_LColorTableParameterPName_P(), MLS_LColorTableTargetSGI_LColorTableParameterPName_P());
		}
		private static UnlinkedGLFunction make_glGetColorTableParameteriv() {
			return newUnlinkedGLFunction("glGetColorTableParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LColorTableParameterPName_P(), MLS_LColorTableTarget_LColorTableParameterPName_P());
		}
		private static UnlinkedGLFunction make_glGetColorTableParameterivEXT() {
			return newUnlinkedGLFunction("glGetColorTableParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LColorTableParameterPName_P(), MLS_LColorTableTarget_LColorTableParameterPName_P());
		}
		private static UnlinkedGLFunction make_glGetColorTableParameterivSGI() {
			return newUnlinkedGLFunction("glGetColorTableParameterivSGI", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTargetSGI_LColorTableParameterPName_P(), MLS_LColorTableTargetSGI_LColorTableParameterPName_P());
		}
		private static UnlinkedGLFunction make_glGetColorTableSGI() {
			return newUnlinkedGLFunction("glGetColorTableSGI", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTargetSGI_LPixelFormat_LPixelType_P(), MLS_LColorTableTargetSGI_LPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetCombinerInputParameterfvNV() {
			return newUnlinkedGLFunction("glGetCombinerInputParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerStageNV_LCombinerPortionNV_LCombinerVariableNV_LCombinerParameterNV_P(), MLS_LCombinerStageNV_LCombinerPortionNV_LCombinerVariableNV_LCombinerParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetCombinerInputParameterivNV() {
			return newUnlinkedGLFunction("glGetCombinerInputParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerStageNV_LCombinerPortionNV_LCombinerVariableNV_LCombinerParameterNV_P(), MLS_LCombinerStageNV_LCombinerPortionNV_LCombinerVariableNV_LCombinerParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetCombinerOutputParameterfvNV() {
			return newUnlinkedGLFunction("glGetCombinerOutputParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerStageNV_LCombinerPortionNV_LCombinerParameterNV_P(), MLS_LCombinerStageNV_LCombinerPortionNV_LCombinerParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetCombinerOutputParameterivNV() {
			return newUnlinkedGLFunction("glGetCombinerOutputParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerStageNV_LCombinerPortionNV_LCombinerParameterNV_P(), MLS_LCombinerStageNV_LCombinerPortionNV_LCombinerParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetCombinerStageParameterfvNV() {
			return newUnlinkedGLFunction("glGetCombinerStageParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerStageNV_LCombinerParameterNV_P(), MLS_LCombinerStageNV_LCombinerParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetCommandHeaderNV() {
			return newUnlinkedGLFunction("glGetCommandHeaderNV", int.class, GLParamTypeSets.ML_I(), PS_LCommandOpcodesNV_I(), MLS_LCommandOpcodesNV_I());
		}
		private static UnlinkedGLFunction make_glGetCompressedMultiTexImageEXT() {
			return newUnlinkedGLFunction("glGetCompressedMultiTexImageEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_IP(), MLS_LTextureUnit_LTextureTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetCompressedTexImage() {
			return newUnlinkedGLFunction("glGetCompressedTexImage", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IP(), MLS_LTextureTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetCompressedTexImageARB() {
			return newUnlinkedGLFunction("glGetCompressedTexImageARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IP(), MLS_LTextureTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetCompressedTextureImage() {
			return newUnlinkedGLFunction("glGetCompressedTextureImage", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetCompressedTextureImageEXT() {
			return newUnlinkedGLFunction("glGetCompressedTextureImageEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IP(), MLS_ILTextureTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetCompressedTextureSubImage() {
			return newUnlinkedGLFunction("glGetCompressedTextureSubImage", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIIP(), MLS_IIIIIIIIIP());
		}
		private static UnlinkedGLFunction make_glGetConvolutionFilter() {
			return newUnlinkedGLFunction("glGetConvolutionFilter", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LPixelFormat_LPixelType_P(), MLS_LConvolutionTarget_LPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetConvolutionFilterEXT() {
			return newUnlinkedGLFunction("glGetConvolutionFilterEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LPixelFormat_LPixelType_P(), MLS_LConvolutionTargetEXT_LPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetConvolutionParameterfv() {
			return newUnlinkedGLFunction("glGetConvolutionParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LConvolutionParameter_P(), MLS_LConvolutionTarget_LConvolutionParameter_P());
		}
		private static UnlinkedGLFunction make_glGetConvolutionParameterfvEXT() {
			return newUnlinkedGLFunction("glGetConvolutionParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LConvolutionParameter_P(), MLS_LConvolutionTargetEXT_LConvolutionParameter_P());
		}
		private static UnlinkedGLFunction make_glGetConvolutionParameteriv() {
			return newUnlinkedGLFunction("glGetConvolutionParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LConvolutionParameter_P(), MLS_LConvolutionTarget_LConvolutionParameter_P());
		}
		private static UnlinkedGLFunction make_glGetConvolutionParameterivEXT() {
			return newUnlinkedGLFunction("glGetConvolutionParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTargetEXT_LConvolutionParameter_P(), MLS_LConvolutionTargetEXT_LConvolutionParameter_P());
		}
		private static UnlinkedGLFunction make_glGetConvolutionParameterxvOES() {
			return newUnlinkedGLFunction("glGetConvolutionParameterxvOES", void.class, GLParamTypeSets.ML_V(), PS_EEP(), MLS_EEP());
		}
		private static UnlinkedGLFunction make_glGetCoverageModulationTableNV() {
			return newUnlinkedGLFunction("glGetCoverageModulationTableNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGetDebugMessageLog() {
			return newUnlinkedGLFunction("glGetDebugMessageLog", int.class, GLParamTypeSets.ML_I(), PS_IIPPPPPP(), MLS_IIPPPPPP());
		}
		private static UnlinkedGLFunction make_glGetDebugMessageLogAMD() {
			return newUnlinkedGLFunction("glGetDebugMessageLogAMD", int.class, GLParamTypeSets.ML_I(), PS_IIPPPPP(), MLS_IIPPPPP());
		}
		private static UnlinkedGLFunction make_glGetDebugMessageLogARB() {
			return newUnlinkedGLFunction("glGetDebugMessageLogARB", int.class, GLParamTypeSets.ML_I(), PS_IIPPPPPP(), MLS_IIPPPPPP());
		}
		private static UnlinkedGLFunction make_glGetDebugMessageLogKHR() {
			return newUnlinkedGLFunction("glGetDebugMessageLogKHR", int.class, GLParamTypeSets.ML_I(), PS_IIPPPPPP(), MLS_IIPPPPPP());
		}
		private static UnlinkedGLFunction make_glGetDetailTexFuncSGIS() {
			return newUnlinkedGLFunction("glGetDetailTexFuncSGIS", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_P(), MLS_LTextureTarget_P());
		}
		private static UnlinkedGLFunction make_glGetDoubleIndexedvEXT() {
			return newUnlinkedGLFunction("glGetDoubleIndexedvEXT", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetDoublei_v() {
			return newUnlinkedGLFunction("glGetDoublei_v", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetDoublei_vEXT() {
			return newUnlinkedGLFunction("glGetDoublei_vEXT", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetDoublev() {
			return newUnlinkedGLFunction("glGetDoublev", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_P(), MLS_LGetPName_P());
		}
		private static UnlinkedGLFunction make_glGetDriverControlStringQCOM() {
			return newUnlinkedGLFunction("glGetDriverControlStringQCOM", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetDriverControlsQCOM() {
			return newUnlinkedGLFunction("glGetDriverControlsQCOM", void.class, GLParamTypeSets.ML_V(), PS_PIP(), MLS_PIP());
		}
		private static UnlinkedGLFunction make_glGetError() {
			return newUnlinkedGLFunction("glGetError", GLErrorCode.class, GLParamTypeSets.ML_LErrorCode_(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glGetFenceivNV() {
			return newUnlinkedGLFunction("glGetFenceivNV", void.class, GLParamTypeSets.ML_V(), PS_ILFenceParameterNameNV_P(), MLS_ILFenceParameterNameNV_P());
		}
		private static UnlinkedGLFunction make_glGetFinalCombinerInputParameterfvNV() {
			return newUnlinkedGLFunction("glGetFinalCombinerInputParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerVariableNV_LCombinerParameterNV_P(), MLS_LCombinerVariableNV_LCombinerParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetFinalCombinerInputParameterivNV() {
			return newUnlinkedGLFunction("glGetFinalCombinerInputParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_LCombinerVariableNV_LCombinerParameterNV_P(), MLS_LCombinerVariableNV_LCombinerParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetFirstPerfQueryIdINTEL() {
			return newUnlinkedGLFunction("glGetFirstPerfQueryIdINTEL", void.class, GLParamTypeSets.ML_V(), PS_P(), MLS_P());
		}
		private static UnlinkedGLFunction make_glGetFixedv() {
			return newUnlinkedGLFunction("glGetFixedv", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_P(), MLS_LGetPName_P());
		}
		private static UnlinkedGLFunction make_glGetFixedvOES() {
			return newUnlinkedGLFunction("glGetFixedvOES", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_P(), MLS_LGetPName_P());
		}
		private static UnlinkedGLFunction make_glGetFloatIndexedvEXT() {
			return newUnlinkedGLFunction("glGetFloatIndexedvEXT", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetFloati_v() {
			return newUnlinkedGLFunction("glGetFloati_v", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetFloati_vEXT() {
			return newUnlinkedGLFunction("glGetFloati_vEXT", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetFloati_vNV() {
			return newUnlinkedGLFunction("glGetFloati_vNV", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetFloati_vOES() {
			return newUnlinkedGLFunction("glGetFloati_vOES", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetFloatv() {
			return newUnlinkedGLFunction("glGetFloatv", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_P(), MLS_LGetPName_P());
		}
		private static UnlinkedGLFunction make_glGetFogFuncSGIS() {
			return newUnlinkedGLFunction("glGetFogFuncSGIS", void.class, GLParamTypeSets.ML_V(), PS_P(), MLS_P());
		}
		private static UnlinkedGLFunction make_glGetFragDataIndex() {
			return newUnlinkedGLFunction("glGetFragDataIndex", int.class, GLParamTypeSets.ML_I(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glGetFragDataIndexEXT() {
			return newUnlinkedGLFunction("glGetFragDataIndexEXT", int.class, GLParamTypeSets.ML_I(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glGetFragDataLocation() {
			return newUnlinkedGLFunction("glGetFragDataLocation", int.class, GLParamTypeSets.ML_I(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glGetFragDataLocationEXT() {
			return newUnlinkedGLFunction("glGetFragDataLocationEXT", int.class, GLParamTypeSets.ML_I(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glGetFragmentLightfvSGIX() {
			return newUnlinkedGLFunction("glGetFragmentLightfvSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_P(), MLS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_P());
		}
		private static UnlinkedGLFunction make_glGetFragmentLightivSGIX() {
			return newUnlinkedGLFunction("glGetFragmentLightivSGIX", void.class, GLParamTypeSets.ML_V(), PS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_P(), MLS_LFragmentLightNameSGIX_LFragmentLightParameterSGIX_P());
		}
		private static UnlinkedGLFunction make_glGetFragmentMaterialfvSGIX() {
			return newUnlinkedGLFunction("glGetFragmentMaterialfvSGIX", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_P(), MLS_LTriangleFace_LMaterialParameter_P());
		}
		private static UnlinkedGLFunction make_glGetFragmentMaterialivSGIX() {
			return newUnlinkedGLFunction("glGetFragmentMaterialivSGIX", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_P(), MLS_LTriangleFace_LMaterialParameter_P());
		}
		private static UnlinkedGLFunction make_glGetFragmentShadingRatesEXT() {
			return newUnlinkedGLFunction("glGetFragmentShadingRatesEXT", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetFramebufferAttachmentParameteriv() {
			return newUnlinkedGLFunction("glGetFramebufferAttachmentParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LFramebufferAttachmentParameterName_P(), MLS_LFramebufferTarget_LFramebufferAttachment_LFramebufferAttachmentParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetFramebufferAttachmentParameterivEXT() {
			return newUnlinkedGLFunction("glGetFramebufferAttachmentParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LFramebufferAttachmentParameterName_P(), MLS_LFramebufferTarget_LFramebufferAttachment_LFramebufferAttachmentParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetFramebufferAttachmentParameterivOES() {
			return newUnlinkedGLFunction("glGetFramebufferAttachmentParameterivOES", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachment_LFramebufferAttachmentParameterName_P(), MLS_LFramebufferTarget_LFramebufferAttachment_LFramebufferAttachmentParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetFramebufferParameterfvAMD() {
			return newUnlinkedGLFunction("glGetFramebufferParameterfvAMD", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachmentParameterName_IIIP(), MLS_LFramebufferTarget_LFramebufferAttachmentParameterName_IIIP());
		}
		private static UnlinkedGLFunction make_glGetFramebufferParameteriv() {
			return newUnlinkedGLFunction("glGetFramebufferParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachmentParameterName_P(), MLS_LFramebufferTarget_LFramebufferAttachmentParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetFramebufferParameterivEXT() {
			return newUnlinkedGLFunction("glGetFramebufferParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetFramebufferParameter_P(), MLS_ILGetFramebufferParameter_P());
		}
		private static UnlinkedGLFunction make_glGetFramebufferPixelLocalStorageSizeEXT() {
			return newUnlinkedGLFunction("glGetFramebufferPixelLocalStorageSizeEXT", int.class, GLParamTypeSets.ML_I(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGetGraphicsResetStatus() {
			return newUnlinkedGLFunction("glGetGraphicsResetStatus", GLGraphicsResetStatus.class, GLParamTypeSets.ML_LGraphicsResetStatus_(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glGetGraphicsResetStatusARB() {
			return newUnlinkedGLFunction("glGetGraphicsResetStatusARB", GLGraphicsResetStatus.class, GLParamTypeSets.ML_LGraphicsResetStatus_(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glGetGraphicsResetStatusEXT() {
			return newUnlinkedGLFunction("glGetGraphicsResetStatusEXT", GLGraphicsResetStatus.class, GLParamTypeSets.ML_LGraphicsResetStatus_(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glGetGraphicsResetStatusKHR() {
			return newUnlinkedGLFunction("glGetGraphicsResetStatusKHR", GLGraphicsResetStatus.class, GLParamTypeSets.ML_LGraphicsResetStatus_(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glGetHandleARB() {
			return newUnlinkedGLFunction("glGetHandleARB", int.class, GLParamTypeSets.ML_I(), PS_LContainerType_(), MLS_LContainerType_());
		}
		private static UnlinkedGLFunction make_glGetHistogram() {
			return newUnlinkedGLFunction("glGetHistogram", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_ZLPixelFormat_LPixelType_P(), MLS_LHistogramTargetEXT_ZLPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetHistogramEXT() {
			return newUnlinkedGLFunction("glGetHistogramEXT", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_ZLPixelFormat_LPixelType_P(), MLS_LHistogramTargetEXT_ZLPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetHistogramParameterfv() {
			return newUnlinkedGLFunction("glGetHistogramParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_LGetHistogramParameterPNameEXT_P(), MLS_LHistogramTargetEXT_LGetHistogramParameterPNameEXT_P());
		}
		private static UnlinkedGLFunction make_glGetHistogramParameterfvEXT() {
			return newUnlinkedGLFunction("glGetHistogramParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_LGetHistogramParameterPNameEXT_P(), MLS_LHistogramTargetEXT_LGetHistogramParameterPNameEXT_P());
		}
		private static UnlinkedGLFunction make_glGetHistogramParameteriv() {
			return newUnlinkedGLFunction("glGetHistogramParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_LGetHistogramParameterPNameEXT_P(), MLS_LHistogramTargetEXT_LGetHistogramParameterPNameEXT_P());
		}
		private static UnlinkedGLFunction make_glGetHistogramParameterivEXT() {
			return newUnlinkedGLFunction("glGetHistogramParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_LGetHistogramParameterPNameEXT_P(), MLS_LHistogramTargetEXT_LGetHistogramParameterPNameEXT_P());
		}
		private static UnlinkedGLFunction make_glGetHistogramParameterxvOES() {
			return newUnlinkedGLFunction("glGetHistogramParameterxvOES", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_LGetHistogramParameterPNameEXT_P(), MLS_LHistogramTargetEXT_LGetHistogramParameterPNameEXT_P());
		}
		private static UnlinkedGLFunction make_glGetImageHandleARB() {
			return newUnlinkedGLFunction("glGetImageHandleARB", long.class, GLParamTypeSets.ML_J(), PS_IIZILPixelFormat_(), MLS_IIZILPixelFormat_());
		}
		private static UnlinkedGLFunction make_glGetImageHandleNV() {
			return newUnlinkedGLFunction("glGetImageHandleNV", long.class, GLParamTypeSets.ML_J(), PS_IIZILPixelFormat_(), MLS_IIZILPixelFormat_());
		}
		private static UnlinkedGLFunction make_glGetImageTransformParameterfvHP() {
			return newUnlinkedGLFunction("glGetImageTransformParameterfvHP", void.class, GLParamTypeSets.ML_V(), PS_LImageTransformTargetHP_LImageTransformPNameHP_P(), MLS_LImageTransformTargetHP_LImageTransformPNameHP_P());
		}
		private static UnlinkedGLFunction make_glGetImageTransformParameterivHP() {
			return newUnlinkedGLFunction("glGetImageTransformParameterivHP", void.class, GLParamTypeSets.ML_V(), PS_LImageTransformTargetHP_LImageTransformPNameHP_P(), MLS_LImageTransformTargetHP_LImageTransformPNameHP_P());
		}
		private static UnlinkedGLFunction make_glGetInfoLogARB() {
			return newUnlinkedGLFunction("glGetInfoLogARB", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetInstrumentsSGIX() {
			return newUnlinkedGLFunction("glGetInstrumentsSGIX", int.class, GLParamTypeSets.ML_I(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glGetInteger64i_v() {
			return newUnlinkedGLFunction("glGetInteger64i_v", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetInteger64v() {
			return newUnlinkedGLFunction("glGetInteger64v", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_P(), MLS_LGetPName_P());
		}
		private static UnlinkedGLFunction make_glGetInteger64vAPPLE() {
			return newUnlinkedGLFunction("glGetInteger64vAPPLE", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_P(), MLS_LGetPName_P());
		}
		private static UnlinkedGLFunction make_glGetInteger64vEXT() {
			return newUnlinkedGLFunction("glGetInteger64vEXT", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_P(), MLS_LGetPName_P());
		}
		private static UnlinkedGLFunction make_glGetIntegerIndexedvEXT() {
			return newUnlinkedGLFunction("glGetIntegerIndexedvEXT", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetIntegeri_v() {
			return newUnlinkedGLFunction("glGetIntegeri_v", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetIntegeri_vEXT() {
			return newUnlinkedGLFunction("glGetIntegeri_vEXT", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_IP(), MLS_LGetPName_IP());
		}
		private static UnlinkedGLFunction make_glGetIntegerui64i_vNV() {
			return newUnlinkedGLFunction("glGetIntegerui64i_vNV", void.class, GLParamTypeSets.ML_V(), PS_EIP(), MLS_EIP());
		}
		private static UnlinkedGLFunction make_glGetIntegerui64vNV() {
			return newUnlinkedGLFunction("glGetIntegerui64vNV", void.class, GLParamTypeSets.ML_V(), PS_EP(), MLS_EP());
		}
		private static UnlinkedGLFunction make_glGetIntegerv() {
			return newUnlinkedGLFunction("glGetIntegerv", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_P(), MLS_LGetPName_P());
		}
		private static UnlinkedGLFunction make_glGetInternalformatSampleivNV() {
			return newUnlinkedGLFunction("glGetInternalformatSampleivNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LInternalFormat_ILInternalFormatPName_IP(), MLS_LTextureTarget_LInternalFormat_ILInternalFormatPName_IP());
		}
		private static UnlinkedGLFunction make_glGetInternalformati64v() {
			return newUnlinkedGLFunction("glGetInternalformati64v", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LInternalFormat_LInternalFormatPName_IP(), MLS_LTextureTarget_LInternalFormat_LInternalFormatPName_IP());
		}
		private static UnlinkedGLFunction make_glGetInternalformativ() {
			return newUnlinkedGLFunction("glGetInternalformativ", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LInternalFormat_LInternalFormatPName_IP(), MLS_LTextureTarget_LInternalFormat_LInternalFormatPName_IP());
		}
		private static UnlinkedGLFunction make_glGetInvariantBooleanvEXT() {
			return newUnlinkedGLFunction("glGetInvariantBooleanvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetVariantValueEXT_P(), MLS_ILGetVariantValueEXT_P());
		}
		private static UnlinkedGLFunction make_glGetInvariantFloatvEXT() {
			return newUnlinkedGLFunction("glGetInvariantFloatvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetVariantValueEXT_P(), MLS_ILGetVariantValueEXT_P());
		}
		private static UnlinkedGLFunction make_glGetInvariantIntegervEXT() {
			return newUnlinkedGLFunction("glGetInvariantIntegervEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetVariantValueEXT_P(), MLS_ILGetVariantValueEXT_P());
		}
		private static UnlinkedGLFunction make_glGetLightfv() {
			return newUnlinkedGLFunction("glGetLightfv", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_P(), MLS_LLightName_LLightParameter_P());
		}
		private static UnlinkedGLFunction make_glGetLightiv() {
			return newUnlinkedGLFunction("glGetLightiv", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_P(), MLS_LLightName_LLightParameter_P());
		}
		private static UnlinkedGLFunction make_glGetLightxOES() {
			return newUnlinkedGLFunction("glGetLightxOES", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_P(), MLS_LLightName_LLightParameter_P());
		}
		private static UnlinkedGLFunction make_glGetLightxv() {
			return newUnlinkedGLFunction("glGetLightxv", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_P(), MLS_LLightName_LLightParameter_P());
		}
		private static UnlinkedGLFunction make_glGetLightxvOES() {
			return newUnlinkedGLFunction("glGetLightxvOES", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_P(), MLS_LLightName_LLightParameter_P());
		}
		private static UnlinkedGLFunction make_glGetListParameterfvSGIX() {
			return newUnlinkedGLFunction("glGetListParameterfvSGIX", void.class, GLParamTypeSets.ML_V(), PS_ILListParameterName_P(), MLS_ILListParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetListParameterivSGIX() {
			return newUnlinkedGLFunction("glGetListParameterivSGIX", void.class, GLParamTypeSets.ML_V(), PS_ILListParameterName_P(), MLS_ILListParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetLocalConstantBooleanvEXT() {
			return newUnlinkedGLFunction("glGetLocalConstantBooleanvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetVariantValueEXT_P(), MLS_ILGetVariantValueEXT_P());
		}
		private static UnlinkedGLFunction make_glGetLocalConstantFloatvEXT() {
			return newUnlinkedGLFunction("glGetLocalConstantFloatvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetVariantValueEXT_P(), MLS_ILGetVariantValueEXT_P());
		}
		private static UnlinkedGLFunction make_glGetLocalConstantIntegervEXT() {
			return newUnlinkedGLFunction("glGetLocalConstantIntegervEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetVariantValueEXT_P(), MLS_ILGetVariantValueEXT_P());
		}
		private static UnlinkedGLFunction make_glGetMapAttribParameterfvNV() {
			return newUnlinkedGLFunction("glGetMapAttribParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_LEvalTargetNV_ILMapAttribParameterNV_P(), MLS_LEvalTargetNV_ILMapAttribParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetMapAttribParameterivNV() {
			return newUnlinkedGLFunction("glGetMapAttribParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_LEvalTargetNV_ILMapAttribParameterNV_P(), MLS_LEvalTargetNV_ILMapAttribParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetMapControlPointsNV() {
			return newUnlinkedGLFunction("glGetMapControlPointsNV", void.class, GLParamTypeSets.ML_V(), PS_LEvalTargetNV_ILMapTypeNV_IIZP(), MLS_LEvalTargetNV_ILMapTypeNV_IIZP());
		}
		private static UnlinkedGLFunction make_glGetMapParameterfvNV() {
			return newUnlinkedGLFunction("glGetMapParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_LEvalTargetNV_LMapParameterNV_P(), MLS_LEvalTargetNV_LMapParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetMapParameterivNV() {
			return newUnlinkedGLFunction("glGetMapParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_LEvalTargetNV_LMapParameterNV_P(), MLS_LEvalTargetNV_LMapParameterNV_P());
		}
		private static UnlinkedGLFunction make_glGetMapdv() {
			return newUnlinkedGLFunction("glGetMapdv", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_LGetMapQuery_P(), MLS_LMapTarget_LGetMapQuery_P());
		}
		private static UnlinkedGLFunction make_glGetMapfv() {
			return newUnlinkedGLFunction("glGetMapfv", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_LGetMapQuery_P(), MLS_LMapTarget_LGetMapQuery_P());
		}
		private static UnlinkedGLFunction make_glGetMapiv() {
			return newUnlinkedGLFunction("glGetMapiv", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_LGetMapQuery_P(), MLS_LMapTarget_LGetMapQuery_P());
		}
		private static UnlinkedGLFunction make_glGetMapxvOES() {
			return newUnlinkedGLFunction("glGetMapxvOES", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_LGetMapQuery_P(), MLS_LMapTarget_LGetMapQuery_P());
		}
		private static UnlinkedGLFunction make_glGetMaterialfv() {
			return newUnlinkedGLFunction("glGetMaterialfv", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_P(), MLS_LTriangleFace_LMaterialParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMaterialiv() {
			return newUnlinkedGLFunction("glGetMaterialiv", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_P(), MLS_LTriangleFace_LMaterialParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMaterialxOES() {
			return newUnlinkedGLFunction("glGetMaterialxOES", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_I(), MLS_LTriangleFace_LMaterialParameter_I());
		}
		private static UnlinkedGLFunction make_glGetMaterialxv() {
			return newUnlinkedGLFunction("glGetMaterialxv", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_P(), MLS_LTriangleFace_LMaterialParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMaterialxvOES() {
			return newUnlinkedGLFunction("glGetMaterialxvOES", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_P(), MLS_LTriangleFace_LMaterialParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMemoryObjectDetachedResourcesuivNV() {
			return newUnlinkedGLFunction("glGetMemoryObjectDetachedResourcesuivNV", void.class, GLParamTypeSets.ML_V(), PS_IEIIP(), MLS_IEIIP());
		}
		private static UnlinkedGLFunction make_glGetMemoryObjectParameterivEXT() {
			return newUnlinkedGLFunction("glGetMemoryObjectParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILMemoryObjectParameterName_P(), MLS_ILMemoryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetMinmax() {
			return newUnlinkedGLFunction("glGetMinmax", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_ZLPixelFormat_LPixelType_P(), MLS_LMinmaxTargetEXT_ZLPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetMinmaxEXT() {
			return newUnlinkedGLFunction("glGetMinmaxEXT", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_ZLPixelFormat_LPixelType_P(), MLS_LMinmaxTargetEXT_ZLPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetMinmaxParameterfv() {
			return newUnlinkedGLFunction("glGetMinmaxParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_LGetMinmaxParameterPNameEXT_P(), MLS_LMinmaxTargetEXT_LGetMinmaxParameterPNameEXT_P());
		}
		private static UnlinkedGLFunction make_glGetMinmaxParameterfvEXT() {
			return newUnlinkedGLFunction("glGetMinmaxParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_LGetMinmaxParameterPNameEXT_P(), MLS_LMinmaxTargetEXT_LGetMinmaxParameterPNameEXT_P());
		}
		private static UnlinkedGLFunction make_glGetMinmaxParameteriv() {
			return newUnlinkedGLFunction("glGetMinmaxParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_LGetMinmaxParameterPNameEXT_P(), MLS_LMinmaxTargetEXT_LGetMinmaxParameterPNameEXT_P());
		}
		private static UnlinkedGLFunction make_glGetMinmaxParameterivEXT() {
			return newUnlinkedGLFunction("glGetMinmaxParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_LGetMinmaxParameterPNameEXT_P(), MLS_LMinmaxTargetEXT_LGetMinmaxParameterPNameEXT_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexEnvfvEXT() {
			return newUnlinkedGLFunction("glGetMultiTexEnvfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_P(), MLS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexEnvivEXT() {
			return newUnlinkedGLFunction("glGetMultiTexEnvivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_P(), MLS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexGendvEXT() {
			return newUnlinkedGLFunction("glGetMultiTexGendvEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureCoordName_LTextureGenParameter_P(), MLS_LTextureUnit_LTextureCoordName_LTextureGenParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexGenfvEXT() {
			return newUnlinkedGLFunction("glGetMultiTexGenfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureCoordName_LTextureGenParameter_P(), MLS_LTextureUnit_LTextureCoordName_LTextureGenParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexGenivEXT() {
			return newUnlinkedGLFunction("glGetMultiTexGenivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureCoordName_LTextureGenParameter_P(), MLS_LTextureUnit_LTextureCoordName_LTextureGenParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexImageEXT() {
			return newUnlinkedGLFunction("glGetMultiTexImageEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILPixelFormat_LPixelType_P(), MLS_LTextureUnit_LTextureTarget_ILPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexLevelParameterfvEXT() {
			return newUnlinkedGLFunction("glGetMultiTexLevelParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILGetTextureParameter_P(), MLS_LTextureUnit_LTextureTarget_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexLevelParameterivEXT() {
			return newUnlinkedGLFunction("glGetMultiTexLevelParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILGetTextureParameter_P(), MLS_LTextureUnit_LTextureTarget_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexParameterIivEXT() {
			return newUnlinkedGLFunction("glGetMultiTexParameterIivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureUnit_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexParameterIuivEXT() {
			return newUnlinkedGLFunction("glGetMultiTexParameterIuivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureUnit_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexParameterfvEXT() {
			return newUnlinkedGLFunction("glGetMultiTexParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureUnit_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultiTexParameterivEXT() {
			return newUnlinkedGLFunction("glGetMultiTexParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureUnit_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetMultisamplefv() {
			return newUnlinkedGLFunction("glGetMultisamplefv", void.class, GLParamTypeSets.ML_V(), PS_LGetMultisamplePNameNV_IP(), MLS_LGetMultisamplePNameNV_IP());
		}
		private static UnlinkedGLFunction make_glGetMultisamplefvNV() {
			return newUnlinkedGLFunction("glGetMultisamplefvNV", void.class, GLParamTypeSets.ML_V(), PS_LGetMultisamplePNameNV_IP(), MLS_LGetMultisamplePNameNV_IP());
		}
		private static UnlinkedGLFunction make_glGetNamedBufferParameteri64v() {
			return newUnlinkedGLFunction("glGetNamedBufferParameteri64v", void.class, GLParamTypeSets.ML_V(), PS_ILBufferPNameARB_P(), MLS_ILBufferPNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetNamedBufferParameteriv() {
			return newUnlinkedGLFunction("glGetNamedBufferParameteriv", void.class, GLParamTypeSets.ML_V(), PS_ILBufferPNameARB_P(), MLS_ILBufferPNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetNamedBufferParameterivEXT() {
			return newUnlinkedGLFunction("glGetNamedBufferParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILBufferPNameARB_P(), MLS_ILBufferPNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetNamedBufferParameterui64vNV() {
			return newUnlinkedGLFunction("glGetNamedBufferParameterui64vNV", void.class, GLParamTypeSets.ML_V(), PS_ILBufferPNameARB_P(), MLS_ILBufferPNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetNamedBufferPointerv() {
			return newUnlinkedGLFunction("glGetNamedBufferPointerv", void.class, GLParamTypeSets.ML_V(), PS_ILBufferPointerNameARB_P(), MLS_ILBufferPointerNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetNamedBufferPointervEXT() {
			return newUnlinkedGLFunction("glGetNamedBufferPointervEXT", void.class, GLParamTypeSets.ML_V(), PS_ILBufferPointerNameARB_P(), MLS_ILBufferPointerNameARB_P());
		}
		private static UnlinkedGLFunction make_glGetNamedBufferSubData() {
			return newUnlinkedGLFunction("glGetNamedBufferSubData", void.class, GLParamTypeSets.ML_V(), PS_IJJP(), MLS_IJJP());
		}
		private static UnlinkedGLFunction make_glGetNamedBufferSubDataEXT() {
			return newUnlinkedGLFunction("glGetNamedBufferSubDataEXT", void.class, GLParamTypeSets.ML_V(), PS_IJJP(), MLS_IJJP());
		}
		private static UnlinkedGLFunction make_glGetNamedFramebufferParameterfvAMD() {
			return newUnlinkedGLFunction("glGetNamedFramebufferParameterfvAMD", void.class, GLParamTypeSets.ML_V(), PS_IEIIIP(), MLS_IEIIIP());
		}
		private static UnlinkedGLFunction make_glGetNamedFramebufferAttachmentParameteriv() {
			return newUnlinkedGLFunction("glGetNamedFramebufferAttachmentParameteriv", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_LFramebufferAttachmentParameterName_P(), MLS_ILFramebufferAttachment_LFramebufferAttachmentParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetNamedFramebufferAttachmentParameterivEXT() {
			return newUnlinkedGLFunction("glGetNamedFramebufferAttachmentParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_LFramebufferAttachmentParameterName_P(), MLS_ILFramebufferAttachment_LFramebufferAttachmentParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetNamedFramebufferParameteriv() {
			return newUnlinkedGLFunction("glGetNamedFramebufferParameteriv", void.class, GLParamTypeSets.ML_V(), PS_ILGetFramebufferParameter_P(), MLS_ILGetFramebufferParameter_P());
		}
		private static UnlinkedGLFunction make_glGetNamedFramebufferParameterivEXT() {
			return newUnlinkedGLFunction("glGetNamedFramebufferParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetFramebufferParameter_P(), MLS_ILGetFramebufferParameter_P());
		}
		private static UnlinkedGLFunction make_glGetNamedProgramLocalParameterIivEXT() {
			return newUnlinkedGLFunction("glGetNamedProgramLocalParameterIivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IP(), MLS_ILProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetNamedProgramLocalParameterIuivEXT() {
			return newUnlinkedGLFunction("glGetNamedProgramLocalParameterIuivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IP(), MLS_ILProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetNamedProgramLocalParameterdvEXT() {
			return newUnlinkedGLFunction("glGetNamedProgramLocalParameterdvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IP(), MLS_ILProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetNamedProgramLocalParameterfvEXT() {
			return newUnlinkedGLFunction("glGetNamedProgramLocalParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IP(), MLS_ILProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetNamedProgramStringEXT() {
			return newUnlinkedGLFunction("glGetNamedProgramStringEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_LProgramStringProperty_P(), MLS_ILProgramTarget_LProgramStringProperty_P());
		}
		private static UnlinkedGLFunction make_glGetNamedProgramivEXT() {
			return newUnlinkedGLFunction("glGetNamedProgramivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_LProgramPropertyARB_P(), MLS_ILProgramTarget_LProgramPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetNamedRenderbufferParameteriv() {
			return newUnlinkedGLFunction("glGetNamedRenderbufferParameteriv", void.class, GLParamTypeSets.ML_V(), PS_ILRenderbufferParameterName_P(), MLS_ILRenderbufferParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetNamedRenderbufferParameterivEXT() {
			return newUnlinkedGLFunction("glGetNamedRenderbufferParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILRenderbufferParameterName_P(), MLS_ILRenderbufferParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetNamedStringARB() {
			return newUnlinkedGLFunction("glGetNamedStringARB", void.class, GLParamTypeSets.ML_V(), PS_IpIPP(), MLS_IpIPP());
		}
		private static UnlinkedGLFunction make_glGetNamedStringivARB() {
			return newUnlinkedGLFunction("glGetNamedStringivARB", void.class, GLParamTypeSets.ML_V(), PS_IpEP(), MLS_IpEP());
		}
		private static UnlinkedGLFunction make_glGetNextPerfQueryIdINTEL() {
			return newUnlinkedGLFunction("glGetNextPerfQueryIdINTEL", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGetObjectBufferfvATI() {
			return newUnlinkedGLFunction("glGetObjectBufferfvATI", void.class, GLParamTypeSets.ML_V(), PS_ILArrayObjectPNameATI_P(), MLS_ILArrayObjectPNameATI_P());
		}
		private static UnlinkedGLFunction make_glGetObjectBufferivATI() {
			return newUnlinkedGLFunction("glGetObjectBufferivATI", void.class, GLParamTypeSets.ML_V(), PS_ILArrayObjectPNameATI_P(), MLS_ILArrayObjectPNameATI_P());
		}
		private static UnlinkedGLFunction make_glGetObjectLabel() {
			return newUnlinkedGLFunction("glGetObjectLabel", void.class, GLParamTypeSets.ML_V(), PS_LObjectIdentifier_IIPP(), MLS_LObjectIdentifier_IIPP());
		}
		private static UnlinkedGLFunction make_glGetObjectLabelEXT() {
			return newUnlinkedGLFunction("glGetObjectLabelEXT", void.class, GLParamTypeSets.ML_V(), PS_EIIPP(), MLS_EIIPP());
		}
		private static UnlinkedGLFunction make_glGetObjectLabelKHR() {
			return newUnlinkedGLFunction("glGetObjectLabelKHR", void.class, GLParamTypeSets.ML_V(), PS_EIIPP(), MLS_EIIPP());
		}
		private static UnlinkedGLFunction make_glGetObjectParameterfvARB() {
			return newUnlinkedGLFunction("glGetObjectParameterfvARB", void.class, GLParamTypeSets.ML_V(), PS_IEP(), MLS_IEP());
		}
		private static UnlinkedGLFunction make_glGetObjectParameterivAPPLE() {
			return newUnlinkedGLFunction("glGetObjectParameterivAPPLE", void.class, GLParamTypeSets.ML_V(), PS_EIEP(), MLS_EIEP());
		}
		private static UnlinkedGLFunction make_glGetObjectParameterivARB() {
			return newUnlinkedGLFunction("glGetObjectParameterivARB", void.class, GLParamTypeSets.ML_V(), PS_IEP(), MLS_IEP());
		}
		private static UnlinkedGLFunction make_glGetObjectPtrLabel() {
			return newUnlinkedGLFunction("glGetObjectPtrLabel", void.class, GLParamTypeSets.ML_V(), PS_pIPP(), MLS_pIPP());
		}
		private static UnlinkedGLFunction make_glGetObjectPtrLabelKHR() {
			return newUnlinkedGLFunction("glGetObjectPtrLabelKHR", void.class, GLParamTypeSets.ML_V(), PS_pIPP(), MLS_pIPP());
		}
		private static UnlinkedGLFunction make_glGetOcclusionQueryivNV() {
			return newUnlinkedGLFunction("glGetOcclusionQueryivNV", void.class, GLParamTypeSets.ML_V(), PS_ILOcclusionQueryParameterNameNV_P(), MLS_ILOcclusionQueryParameterNameNV_P());
		}
		private static UnlinkedGLFunction make_glGetOcclusionQueryuivNV() {
			return newUnlinkedGLFunction("glGetOcclusionQueryuivNV", void.class, GLParamTypeSets.ML_V(), PS_ILOcclusionQueryParameterNameNV_P(), MLS_ILOcclusionQueryParameterNameNV_P());
		}
		private static UnlinkedGLFunction make_glGetPathColorGenfvNV() {
			return newUnlinkedGLFunction("glGetPathColorGenfvNV", void.class, GLParamTypeSets.ML_V(), PS_LPathColor_LPathGenMode_P(), MLS_LPathColor_LPathGenMode_P());
		}
		private static UnlinkedGLFunction make_glGetPathColorGenivNV() {
			return newUnlinkedGLFunction("glGetPathColorGenivNV", void.class, GLParamTypeSets.ML_V(), PS_LPathColor_LPathGenMode_P(), MLS_LPathColor_LPathGenMode_P());
		}
		private static UnlinkedGLFunction make_glGetPathCommandsNV() {
			return newUnlinkedGLFunction("glGetPathCommandsNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGetPathCoordsNV() {
			return newUnlinkedGLFunction("glGetPathCoordsNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGetPathDashArrayNV() {
			return newUnlinkedGLFunction("glGetPathDashArrayNV", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGetPathLengthNV() {
			return newUnlinkedGLFunction("glGetPathLengthNV", float.class, GLParamTypeSets.ML_F(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glGetPathMetricRangeNV() {
			return newUnlinkedGLFunction("glGetPathMetricRangeNV", void.class, GLParamTypeSets.ML_V(), PS_IIIIP(), MLS_IIIIP());
		}
		private static UnlinkedGLFunction make_glGetPathMetricsNV() {
			return newUnlinkedGLFunction("glGetPathMetricsNV", void.class, GLParamTypeSets.ML_V(), PS_IILPathElementType_pIIP(), MLS_IILPathElementType_pIIP());
		}
		private static UnlinkedGLFunction make_glGetPathParameterfvNV() {
			return newUnlinkedGLFunction("glGetPathParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathParameter_P(), MLS_ILPathParameter_P());
		}
		private static UnlinkedGLFunction make_glGetPathParameterivNV() {
			return newUnlinkedGLFunction("glGetPathParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathParameter_P(), MLS_ILPathParameter_P());
		}
		private static UnlinkedGLFunction make_glGetPathSpacingNV() {
			return newUnlinkedGLFunction("glGetPathSpacingNV", void.class, GLParamTypeSets.ML_V(), PS_LPathListMode_ILPathElementType_pIFFLPathTransformType_P(), MLS_LPathListMode_ILPathElementType_pIFFLPathTransformType_P());
		}
		private static UnlinkedGLFunction make_glGetPathTexGenfvNV() {
			return newUnlinkedGLFunction("glGetPathTexGenfvNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LPathGenMode_P(), MLS_LTextureUnit_LPathGenMode_P());
		}
		private static UnlinkedGLFunction make_glGetPathTexGenivNV() {
			return newUnlinkedGLFunction("glGetPathTexGenivNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LPathGenMode_P(), MLS_LTextureUnit_LPathGenMode_P());
		}
		private static UnlinkedGLFunction make_glGetPerfCounterInfoINTEL() {
			return newUnlinkedGLFunction("glGetPerfCounterInfoINTEL", void.class, GLParamTypeSets.ML_V(), PS_IIIPIPPPPPP(), MLS_IIIPIPPPPPP());
		}
		private static UnlinkedGLFunction make_glGetPerfMonitorCounterDataAMD() {
			return newUnlinkedGLFunction("glGetPerfMonitorCounterDataAMD", void.class, GLParamTypeSets.ML_V(), PS_IEIPP(), MLS_IEIPP());
		}
		private static UnlinkedGLFunction make_glGetPerfMonitorCounterInfoAMD() {
			return newUnlinkedGLFunction("glGetPerfMonitorCounterInfoAMD", void.class, GLParamTypeSets.ML_V(), PS_IIEP(), MLS_IIEP());
		}
		private static UnlinkedGLFunction make_glGetPerfMonitorCounterStringAMD() {
			return newUnlinkedGLFunction("glGetPerfMonitorCounterStringAMD", void.class, GLParamTypeSets.ML_V(), PS_IIIPP(), MLS_IIIPP());
		}
		private static UnlinkedGLFunction make_glGetPerfMonitorCountersAMD() {
			return newUnlinkedGLFunction("glGetPerfMonitorCountersAMD", void.class, GLParamTypeSets.ML_V(), PS_IPPIP(), MLS_IPPIP());
		}
		private static UnlinkedGLFunction make_glGetPerfMonitorGroupStringAMD() {
			return newUnlinkedGLFunction("glGetPerfMonitorGroupStringAMD", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetPerfMonitorGroupsAMD() {
			return newUnlinkedGLFunction("glGetPerfMonitorGroupsAMD", void.class, GLParamTypeSets.ML_V(), PS_PIP(), MLS_PIP());
		}
		private static UnlinkedGLFunction make_glGetPerfQueryDataINTEL() {
			return newUnlinkedGLFunction("glGetPerfQueryDataINTEL", void.class, GLParamTypeSets.ML_V(), PS_IIIPP(), MLS_IIIPP());
		}
		private static UnlinkedGLFunction make_glGetPerfQueryIdByNameINTEL() {
			return newUnlinkedGLFunction("glGetPerfQueryIdByNameINTEL", void.class, GLParamTypeSets.ML_V(), PS_PP(), MLS_PP());
		}
		private static UnlinkedGLFunction make_glGetPerfQueryInfoINTEL() {
			return newUnlinkedGLFunction("glGetPerfQueryInfoINTEL", void.class, GLParamTypeSets.ML_V(), PS_IIPPPPP(), MLS_IIPPPPP());
		}
		private static UnlinkedGLFunction make_glGetPixelMapfv() {
			return newUnlinkedGLFunction("glGetPixelMapfv", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_P(), MLS_LPixelMap_P());
		}
		private static UnlinkedGLFunction make_glGetPixelMapuiv() {
			return newUnlinkedGLFunction("glGetPixelMapuiv", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_P(), MLS_LPixelMap_P());
		}
		private static UnlinkedGLFunction make_glGetPixelMapusv() {
			return newUnlinkedGLFunction("glGetPixelMapusv", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_P(), MLS_LPixelMap_P());
		}
		private static UnlinkedGLFunction make_glGetPixelMapxv() {
			return newUnlinkedGLFunction("glGetPixelMapxv", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_IP(), MLS_LPixelMap_IP());
		}
		private static UnlinkedGLFunction make_glGetPixelTexGenParameterfvSGIS() {
			return newUnlinkedGLFunction("glGetPixelTexGenParameterfvSGIS", void.class, GLParamTypeSets.ML_V(), PS_LPixelTexGenParameterNameSGIS_P(), MLS_LPixelTexGenParameterNameSGIS_P());
		}
		private static UnlinkedGLFunction make_glGetPixelTexGenParameterivSGIS() {
			return newUnlinkedGLFunction("glGetPixelTexGenParameterivSGIS", void.class, GLParamTypeSets.ML_V(), PS_LPixelTexGenParameterNameSGIS_P(), MLS_LPixelTexGenParameterNameSGIS_P());
		}
		private static UnlinkedGLFunction make_glGetPixelTransformParameterfvEXT() {
			return newUnlinkedGLFunction("glGetPixelTransformParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_EEP(), MLS_EEP());
		}
		private static UnlinkedGLFunction make_glGetPixelTransformParameterivEXT() {
			return newUnlinkedGLFunction("glGetPixelTransformParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_EEP(), MLS_EEP());
		}
		private static UnlinkedGLFunction make_glGetPointerIndexedvEXT() {
			return newUnlinkedGLFunction("glGetPointerIndexedvEXT", void.class, GLParamTypeSets.ML_V(), PS_EIP(), MLS_EIP());
		}
		private static UnlinkedGLFunction make_glGetPointeri_vEXT() {
			return newUnlinkedGLFunction("glGetPointeri_vEXT", void.class, GLParamTypeSets.ML_V(), PS_EIP(), MLS_EIP());
		}
		private static UnlinkedGLFunction make_glGetPointerv() {
			return newUnlinkedGLFunction("glGetPointerv", void.class, GLParamTypeSets.ML_V(), PS_LGetPointervPName_P(), MLS_LGetPointervPName_P());
		}
		private static UnlinkedGLFunction make_glGetPointervEXT() {
			return newUnlinkedGLFunction("glGetPointervEXT", void.class, GLParamTypeSets.ML_V(), PS_LGetPointervPName_P(), MLS_LGetPointervPName_P());
		}
		private static UnlinkedGLFunction make_glGetPointervKHR() {
			return newUnlinkedGLFunction("glGetPointervKHR", void.class, GLParamTypeSets.ML_V(), PS_EP(), MLS_EP());
		}
		private static UnlinkedGLFunction make_glGetPolygonStipple() {
			return newUnlinkedGLFunction("glGetPolygonStipple", void.class, GLParamTypeSets.ML_V(), PS_P(), MLS_P());
		}
		private static UnlinkedGLFunction make_glGetProgramBinary() {
			return newUnlinkedGLFunction("glGetProgramBinary", void.class, GLParamTypeSets.ML_V(), PS_IIPPP(), MLS_IIPPP());
		}
		private static UnlinkedGLFunction make_glGetProgramBinaryOES() {
			return newUnlinkedGLFunction("glGetProgramBinaryOES", void.class, GLParamTypeSets.ML_V(), PS_IIPPP(), MLS_IIPPP());
		}
		private static UnlinkedGLFunction make_glGetProgramEnvParameterIivNV() {
			return newUnlinkedGLFunction("glGetProgramEnvParameterIivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IP(), MLS_LProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetProgramEnvParameterIuivNV() {
			return newUnlinkedGLFunction("glGetProgramEnvParameterIuivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IP(), MLS_LProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetProgramEnvParameterdvARB() {
			return newUnlinkedGLFunction("glGetProgramEnvParameterdvARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IP(), MLS_LProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetProgramEnvParameterfvARB() {
			return newUnlinkedGLFunction("glGetProgramEnvParameterfvARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IP(), MLS_LProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetProgramInfoLog() {
			return newUnlinkedGLFunction("glGetProgramInfoLog", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetProgramInterfaceiv() {
			return newUnlinkedGLFunction("glGetProgramInterfaceiv", void.class, GLParamTypeSets.ML_V(), PS_ILProgramInterface_LProgramInterfacePName_P(), MLS_ILProgramInterface_LProgramInterfacePName_P());
		}
		private static UnlinkedGLFunction make_glGetProgramLocalParameterIivNV() {
			return newUnlinkedGLFunction("glGetProgramLocalParameterIivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IP(), MLS_LProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetProgramLocalParameterIuivNV() {
			return newUnlinkedGLFunction("glGetProgramLocalParameterIuivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IP(), MLS_LProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetProgramLocalParameterdvARB() {
			return newUnlinkedGLFunction("glGetProgramLocalParameterdvARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IP(), MLS_LProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetProgramLocalParameterfvARB() {
			return newUnlinkedGLFunction("glGetProgramLocalParameterfvARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IP(), MLS_LProgramTarget_IP());
		}
		private static UnlinkedGLFunction make_glGetProgramNamedParameterdvNV() {
			return newUnlinkedGLFunction("glGetProgramNamedParameterdvNV", void.class, GLParamTypeSets.ML_V(), PS_IIpP(), MLS_IIpP());
		}
		private static UnlinkedGLFunction make_glGetProgramNamedParameterfvNV() {
			return newUnlinkedGLFunction("glGetProgramNamedParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_IIpP(), MLS_IIpP());
		}
		private static UnlinkedGLFunction make_glGetProgramParameterdvNV() {
			return newUnlinkedGLFunction("glGetProgramParameterdvNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_ILVertexAttribEnumNV_P(), MLS_LVertexAttribEnumNV_ILVertexAttribEnumNV_P());
		}
		private static UnlinkedGLFunction make_glGetProgramParameterfvNV() {
			return newUnlinkedGLFunction("glGetProgramParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_ILVertexAttribEnumNV_P(), MLS_LVertexAttribEnumNV_ILVertexAttribEnumNV_P());
		}
		private static UnlinkedGLFunction make_glGetProgramPipelineInfoLog() {
			return newUnlinkedGLFunction("glGetProgramPipelineInfoLog", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetProgramPipelineInfoLogEXT() {
			return newUnlinkedGLFunction("glGetProgramPipelineInfoLogEXT", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetProgramPipelineiv() {
			return newUnlinkedGLFunction("glGetProgramPipelineiv", void.class, GLParamTypeSets.ML_V(), PS_ILPipelineParameterName_P(), MLS_ILPipelineParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetProgramPipelineivEXT() {
			return newUnlinkedGLFunction("glGetProgramPipelineivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILPipelineParameterName_P(), MLS_ILPipelineParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetProgramResourceIndex() {
			return newUnlinkedGLFunction("glGetProgramResourceIndex", int.class, GLParamTypeSets.ML_I(), PS_ILProgramInterface_p(), MLS_ILProgramInterface_p());
		}
		private static UnlinkedGLFunction make_glGetProgramResourceLocation() {
			return newUnlinkedGLFunction("glGetProgramResourceLocation", int.class, GLParamTypeSets.ML_I(), PS_ILProgramInterface_p(), MLS_ILProgramInterface_p());
		}
		private static UnlinkedGLFunction make_glGetProgramResourceLocationIndex() {
			return newUnlinkedGLFunction("glGetProgramResourceLocationIndex", int.class, GLParamTypeSets.ML_I(), PS_ILProgramInterface_p(), MLS_ILProgramInterface_p());
		}
		private static UnlinkedGLFunction make_glGetProgramResourceLocationIndexEXT() {
			return newUnlinkedGLFunction("glGetProgramResourceLocationIndexEXT", int.class, GLParamTypeSets.ML_I(), PS_ILProgramInterface_p(), MLS_ILProgramInterface_p());
		}
		private static UnlinkedGLFunction make_glGetProgramResourceName() {
			return newUnlinkedGLFunction("glGetProgramResourceName", void.class, GLParamTypeSets.ML_V(), PS_ILProgramInterface_IIPP(), MLS_ILProgramInterface_IIPP());
		}
		private static UnlinkedGLFunction make_glGetProgramResourcefvNV() {
			return newUnlinkedGLFunction("glGetProgramResourcefvNV", void.class, GLParamTypeSets.ML_V(), PS_ILProgramInterface_IIpIPP(), MLS_ILProgramInterface_IIpIPP());
		}
		private static UnlinkedGLFunction make_glGetProgramResourceiv() {
			return newUnlinkedGLFunction("glGetProgramResourceiv", void.class, GLParamTypeSets.ML_V(), PS_ILProgramInterface_IIpIPP(), MLS_ILProgramInterface_IIpIPP());
		}
		private static UnlinkedGLFunction make_glGetProgramStageiv() {
			return newUnlinkedGLFunction("glGetProgramStageiv", void.class, GLParamTypeSets.ML_V(), PS_ILShaderType_LProgramStagePName_P(), MLS_ILShaderType_LProgramStagePName_P());
		}
		private static UnlinkedGLFunction make_glGetProgramStringARB() {
			return newUnlinkedGLFunction("glGetProgramStringARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_LProgramStringProperty_P(), MLS_LProgramTarget_LProgramStringProperty_P());
		}
		private static UnlinkedGLFunction make_glGetProgramStringNV() {
			return newUnlinkedGLFunction("glGetProgramStringNV", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnumNV_P(), MLS_ILVertexAttribEnumNV_P());
		}
		private static UnlinkedGLFunction make_glGetProgramSubroutineParameteruivNV() {
			return newUnlinkedGLFunction("glGetProgramSubroutineParameteruivNV", void.class, GLParamTypeSets.ML_V(), PS_EIP(), MLS_EIP());
		}
		private static UnlinkedGLFunction make_glGetProgramiv() {
			return newUnlinkedGLFunction("glGetProgramiv", void.class, GLParamTypeSets.ML_V(), PS_ILProgramPropertyARB_P(), MLS_ILProgramPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetProgramivARB() {
			return newUnlinkedGLFunction("glGetProgramivARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_LProgramPropertyARB_P(), MLS_LProgramTarget_LProgramPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetProgramivNV() {
			return newUnlinkedGLFunction("glGetProgramivNV", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnumNV_P(), MLS_ILVertexAttribEnumNV_P());
		}
		private static UnlinkedGLFunction make_glGetQueryBufferObjecti64v() {
			return newUnlinkedGLFunction("glGetQueryBufferObjecti64v", void.class, GLParamTypeSets.ML_V(), PS_IILQueryObjectParameterName_J(), MLS_IILQueryObjectParameterName_J());
		}
		private static UnlinkedGLFunction make_glGetQueryBufferObjectiv() {
			return newUnlinkedGLFunction("glGetQueryBufferObjectiv", void.class, GLParamTypeSets.ML_V(), PS_IILQueryObjectParameterName_J(), MLS_IILQueryObjectParameterName_J());
		}
		private static UnlinkedGLFunction make_glGetQueryBufferObjectui64v() {
			return newUnlinkedGLFunction("glGetQueryBufferObjectui64v", void.class, GLParamTypeSets.ML_V(), PS_IILQueryObjectParameterName_J(), MLS_IILQueryObjectParameterName_J());
		}
		private static UnlinkedGLFunction make_glGetQueryBufferObjectuiv() {
			return newUnlinkedGLFunction("glGetQueryBufferObjectuiv", void.class, GLParamTypeSets.ML_V(), PS_IILQueryObjectParameterName_J(), MLS_IILQueryObjectParameterName_J());
		}
		private static UnlinkedGLFunction make_glGetQueryIndexediv() {
			return newUnlinkedGLFunction("glGetQueryIndexediv", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_ILQueryParameterName_P(), MLS_LQueryTarget_ILQueryParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryObjecti64v() {
			return newUnlinkedGLFunction("glGetQueryObjecti64v", void.class, GLParamTypeSets.ML_V(), PS_ILQueryObjectParameterName_P(), MLS_ILQueryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryObjecti64vEXT() {
			return newUnlinkedGLFunction("glGetQueryObjecti64vEXT", void.class, GLParamTypeSets.ML_V(), PS_ILQueryObjectParameterName_P(), MLS_ILQueryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryObjectiv() {
			return newUnlinkedGLFunction("glGetQueryObjectiv", void.class, GLParamTypeSets.ML_V(), PS_ILQueryObjectParameterName_P(), MLS_ILQueryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryObjectivARB() {
			return newUnlinkedGLFunction("glGetQueryObjectivARB", void.class, GLParamTypeSets.ML_V(), PS_ILQueryObjectParameterName_P(), MLS_ILQueryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryObjectivEXT() {
			return newUnlinkedGLFunction("glGetQueryObjectivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILQueryObjectParameterName_P(), MLS_ILQueryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryObjectui64v() {
			return newUnlinkedGLFunction("glGetQueryObjectui64v", void.class, GLParamTypeSets.ML_V(), PS_ILQueryObjectParameterName_P(), MLS_ILQueryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryObjectui64vEXT() {
			return newUnlinkedGLFunction("glGetQueryObjectui64vEXT", void.class, GLParamTypeSets.ML_V(), PS_ILQueryObjectParameterName_P(), MLS_ILQueryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryObjectuiv() {
			return newUnlinkedGLFunction("glGetQueryObjectuiv", void.class, GLParamTypeSets.ML_V(), PS_ILQueryObjectParameterName_P(), MLS_ILQueryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryObjectuivARB() {
			return newUnlinkedGLFunction("glGetQueryObjectuivARB", void.class, GLParamTypeSets.ML_V(), PS_ILQueryObjectParameterName_P(), MLS_ILQueryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryObjectuivEXT() {
			return newUnlinkedGLFunction("glGetQueryObjectuivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILQueryObjectParameterName_P(), MLS_ILQueryObjectParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryiv() {
			return newUnlinkedGLFunction("glGetQueryiv", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_LQueryParameterName_P(), MLS_LQueryTarget_LQueryParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryivARB() {
			return newUnlinkedGLFunction("glGetQueryivARB", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_LQueryParameterName_P(), MLS_LQueryTarget_LQueryParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetQueryivEXT() {
			return newUnlinkedGLFunction("glGetQueryivEXT", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_LQueryParameterName_P(), MLS_LQueryTarget_LQueryParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetRenderbufferParameteriv() {
			return newUnlinkedGLFunction("glGetRenderbufferParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_LRenderbufferParameterName_P(), MLS_LRenderbufferTarget_LRenderbufferParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetRenderbufferParameterivEXT() {
			return newUnlinkedGLFunction("glGetRenderbufferParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_LRenderbufferParameterName_P(), MLS_LRenderbufferTarget_LRenderbufferParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetRenderbufferParameterivOES() {
			return newUnlinkedGLFunction("glGetRenderbufferParameterivOES", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_LRenderbufferParameterName_P(), MLS_LRenderbufferTarget_LRenderbufferParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetSamplerParameterIiv() {
			return newUnlinkedGLFunction("glGetSamplerParameterIiv", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_P(), MLS_ILSamplerParameterI_P());
		}
		private static UnlinkedGLFunction make_glGetSamplerParameterIivEXT() {
			return newUnlinkedGLFunction("glGetSamplerParameterIivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_P(), MLS_ILSamplerParameterI_P());
		}
		private static UnlinkedGLFunction make_glGetSamplerParameterIivOES() {
			return newUnlinkedGLFunction("glGetSamplerParameterIivOES", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_P(), MLS_ILSamplerParameterI_P());
		}
		private static UnlinkedGLFunction make_glGetSamplerParameterIuiv() {
			return newUnlinkedGLFunction("glGetSamplerParameterIuiv", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_P(), MLS_ILSamplerParameterI_P());
		}
		private static UnlinkedGLFunction make_glGetSamplerParameterIuivEXT() {
			return newUnlinkedGLFunction("glGetSamplerParameterIuivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_P(), MLS_ILSamplerParameterI_P());
		}
		private static UnlinkedGLFunction make_glGetSamplerParameterIuivOES() {
			return newUnlinkedGLFunction("glGetSamplerParameterIuivOES", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_P(), MLS_ILSamplerParameterI_P());
		}
		private static UnlinkedGLFunction make_glGetSamplerParameterfv() {
			return newUnlinkedGLFunction("glGetSamplerParameterfv", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterF_P(), MLS_ILSamplerParameterF_P());
		}
		private static UnlinkedGLFunction make_glGetSamplerParameteriv() {
			return newUnlinkedGLFunction("glGetSamplerParameteriv", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_P(), MLS_ILSamplerParameterI_P());
		}
		private static UnlinkedGLFunction make_glGetSemaphoreParameterivNV() {
			return newUnlinkedGLFunction("glGetSemaphoreParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_ILSemaphoreParameterName_P(), MLS_ILSemaphoreParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetSemaphoreParameterui64vEXT() {
			return newUnlinkedGLFunction("glGetSemaphoreParameterui64vEXT", void.class, GLParamTypeSets.ML_V(), PS_ILSemaphoreParameterName_P(), MLS_ILSemaphoreParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetSeparableFilter() {
			return newUnlinkedGLFunction("glGetSeparableFilter", void.class, GLParamTypeSets.ML_V(), PS_LSeparableTargetEXT_LPixelFormat_LPixelType_PPP(), MLS_LSeparableTargetEXT_LPixelFormat_LPixelType_PPP());
		}
		private static UnlinkedGLFunction make_glGetSeparableFilterEXT() {
			return newUnlinkedGLFunction("glGetSeparableFilterEXT", void.class, GLParamTypeSets.ML_V(), PS_LSeparableTargetEXT_LPixelFormat_LPixelType_PPP(), MLS_LSeparableTargetEXT_LPixelFormat_LPixelType_PPP());
		}
		private static UnlinkedGLFunction make_glGetShaderInfoLog() {
			return newUnlinkedGLFunction("glGetShaderInfoLog", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetShaderPrecisionFormat() {
			return newUnlinkedGLFunction("glGetShaderPrecisionFormat", void.class, GLParamTypeSets.ML_V(), PS_LShaderType_LPrecisionType_PP(), MLS_LShaderType_LPrecisionType_PP());
		}
		private static UnlinkedGLFunction make_glGetShaderSource() {
			return newUnlinkedGLFunction("glGetShaderSource", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetShaderSourceARB() {
			return newUnlinkedGLFunction("glGetShaderSourceARB", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetShaderiv() {
			return newUnlinkedGLFunction("glGetShaderiv", void.class, GLParamTypeSets.ML_V(), PS_ILShaderParameterName_P(), MLS_ILShaderParameterName_P());
		}
		private static UnlinkedGLFunction make_glGetShadingRateImagePaletteNV() {
			return newUnlinkedGLFunction("glGetShadingRateImagePaletteNV", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetShadingRateSampleLocationivNV() {
			return newUnlinkedGLFunction("glGetShadingRateSampleLocationivNV", void.class, GLParamTypeSets.ML_V(), PS_EIIP(), MLS_EIIP());
		}
		private static UnlinkedGLFunction make_glGetSharpenTexFuncSGIS() {
			return newUnlinkedGLFunction("glGetSharpenTexFuncSGIS", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_P(), MLS_LTextureTarget_P());
		}
		private static UnlinkedGLFunction make_glGetStageIndexNV() {
			return newUnlinkedGLFunction("glGetStageIndexNV", char.class, GLParamTypeSets.ML_C(), PS_LShaderType_(), MLS_LShaderType_());
		}
		private static UnlinkedGLFunction make_glGetString() {
			return newUnlinkedGLFunction("glGetString", ConstPointer.class, GLParamTypeSets.ML_p(), PS_LStringName_(), MLS_LStringName_());
		}
		private static UnlinkedGLFunction make_glGetStringi() {
			return newUnlinkedGLFunction("glGetStringi", ConstPointer.class, GLParamTypeSets.ML_p(), PS_LStringName_I(), MLS_LStringName_I());
		}
		private static UnlinkedGLFunction make_glGetSubroutineIndex() {
			return newUnlinkedGLFunction("glGetSubroutineIndex", int.class, GLParamTypeSets.ML_I(), PS_ILShaderType_p(), MLS_ILShaderType_p());
		}
		private static UnlinkedGLFunction make_glGetSubroutineUniformLocation() {
			return newUnlinkedGLFunction("glGetSubroutineUniformLocation", int.class, GLParamTypeSets.ML_I(), PS_ILShaderType_p(), MLS_ILShaderType_p());
		}
		private static UnlinkedGLFunction make_glGetSynciv() {
			return newUnlinkedGLFunction("glGetSynciv", void.class, GLParamTypeSets.ML_V(), PS_ILSyncParameterName_IPP(), MLS_ILSyncParameterName_IPP());
		}
		private static UnlinkedGLFunction make_glGetSyncivAPPLE() {
			return newUnlinkedGLFunction("glGetSyncivAPPLE", void.class, GLParamTypeSets.ML_V(), PS_ILSyncParameterName_IPP(), MLS_ILSyncParameterName_IPP());
		}
		private static UnlinkedGLFunction make_glGetTexBumpParameterfvATI() {
			return newUnlinkedGLFunction("glGetTexBumpParameterfvATI", void.class, GLParamTypeSets.ML_V(), PS_LGetTexBumpParameterATI_P(), MLS_LGetTexBumpParameterATI_P());
		}
		private static UnlinkedGLFunction make_glGetTexBumpParameterivATI() {
			return newUnlinkedGLFunction("glGetTexBumpParameterivATI", void.class, GLParamTypeSets.ML_V(), PS_LGetTexBumpParameterATI_P(), MLS_LGetTexBumpParameterATI_P());
		}
		private static UnlinkedGLFunction make_glGetTexEnvfv() {
			return newUnlinkedGLFunction("glGetTexEnvfv", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_P(), MLS_LTextureEnvTarget_LTextureEnvParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexEnviv() {
			return newUnlinkedGLFunction("glGetTexEnviv", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_P(), MLS_LTextureEnvTarget_LTextureEnvParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexEnvxv() {
			return newUnlinkedGLFunction("glGetTexEnvxv", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_P(), MLS_LTextureEnvTarget_LTextureEnvParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexEnvxvOES() {
			return newUnlinkedGLFunction("glGetTexEnvxvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_P(), MLS_LTextureEnvTarget_LTextureEnvParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexFilterFuncSGIS() {
			return newUnlinkedGLFunction("glGetTexFilterFuncSGIS", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureFilterSGIS_P(), MLS_LTextureTarget_LTextureFilterSGIS_P());
		}
		private static UnlinkedGLFunction make_glGetTexGendv() {
			return newUnlinkedGLFunction("glGetTexGendv", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_P(), MLS_LTextureCoordName_LTextureGenParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexGenfv() {
			return newUnlinkedGLFunction("glGetTexGenfv", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_P(), MLS_LTextureCoordName_LTextureGenParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexGenfvOES() {
			return newUnlinkedGLFunction("glGetTexGenfvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_P(), MLS_LTextureCoordName_LTextureGenParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexGeniv() {
			return newUnlinkedGLFunction("glGetTexGeniv", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_P(), MLS_LTextureCoordName_LTextureGenParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexGenivOES() {
			return newUnlinkedGLFunction("glGetTexGenivOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_P(), MLS_LTextureCoordName_LTextureGenParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexGenxvOES() {
			return newUnlinkedGLFunction("glGetTexGenxvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_P(), MLS_LTextureCoordName_LTextureGenParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexImage() {
			return newUnlinkedGLFunction("glGetTexImage", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILPixelFormat_LPixelType_P(), MLS_LTextureTarget_ILPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetTexLevelParameterfv() {
			return newUnlinkedGLFunction("glGetTexLevelParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILGetTextureParameter_P(), MLS_LTextureTarget_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexLevelParameteriv() {
			return newUnlinkedGLFunction("glGetTexLevelParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILGetTextureParameter_P(), MLS_LTextureTarget_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexLevelParameterxvOES() {
			return newUnlinkedGLFunction("glGetTexLevelParameterxvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILGetTextureParameter_P(), MLS_LTextureTarget_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexParameterIiv() {
			return newUnlinkedGLFunction("glGetTexParameterIiv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexParameterIivEXT() {
			return newUnlinkedGLFunction("glGetTexParameterIivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexParameterIivOES() {
			return newUnlinkedGLFunction("glGetTexParameterIivOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexParameterIuiv() {
			return newUnlinkedGLFunction("glGetTexParameterIuiv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexParameterIuivEXT() {
			return newUnlinkedGLFunction("glGetTexParameterIuivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexParameterIuivOES() {
			return newUnlinkedGLFunction("glGetTexParameterIuivOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexParameterPointervAPPLE() {
			return newUnlinkedGLFunction("glGetTexParameterPointervAPPLE", void.class, GLParamTypeSets.ML_V(), PS_EEP(), MLS_EEP());
		}
		private static UnlinkedGLFunction make_glGetTexParameterfv() {
			return newUnlinkedGLFunction("glGetTexParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexParameteriv() {
			return newUnlinkedGLFunction("glGetTexParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexParameterxv() {
			return newUnlinkedGLFunction("glGetTexParameterxv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTexParameterxvOES() {
			return newUnlinkedGLFunction("glGetTexParameterxvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_P(), MLS_LTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureHandleARB() {
			return newUnlinkedGLFunction("glGetTextureHandleARB", long.class, GLParamTypeSets.ML_J(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGetTextureHandleIMG() {
			return newUnlinkedGLFunction("glGetTextureHandleIMG", long.class, GLParamTypeSets.ML_J(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGetTextureHandleNV() {
			return newUnlinkedGLFunction("glGetTextureHandleNV", long.class, GLParamTypeSets.ML_J(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGetTextureImage() {
			return newUnlinkedGLFunction("glGetTextureImage", void.class, GLParamTypeSets.ML_V(), PS_IILPixelFormat_LPixelType_IP(), MLS_IILPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetTextureImageEXT() {
			return newUnlinkedGLFunction("glGetTextureImageEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILPixelFormat_LPixelType_P(), MLS_ILTextureTarget_ILPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glGetTextureLevelParameterfv() {
			return newUnlinkedGLFunction("glGetTextureLevelParameterfv", void.class, GLParamTypeSets.ML_V(), PS_IILGetTextureParameter_P(), MLS_IILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureLevelParameterfvEXT() {
			return newUnlinkedGLFunction("glGetTextureLevelParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILGetTextureParameter_P(), MLS_ILTextureTarget_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureLevelParameteriv() {
			return newUnlinkedGLFunction("glGetTextureLevelParameteriv", void.class, GLParamTypeSets.ML_V(), PS_IILGetTextureParameter_P(), MLS_IILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureLevelParameterivEXT() {
			return newUnlinkedGLFunction("glGetTextureLevelParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILGetTextureParameter_P(), MLS_ILTextureTarget_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureParameterIiv() {
			return newUnlinkedGLFunction("glGetTextureParameterIiv", void.class, GLParamTypeSets.ML_V(), PS_ILGetTextureParameter_P(), MLS_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureParameterIivEXT() {
			return newUnlinkedGLFunction("glGetTextureParameterIivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LGetTextureParameter_P(), MLS_ILTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureParameterIuiv() {
			return newUnlinkedGLFunction("glGetTextureParameterIuiv", void.class, GLParamTypeSets.ML_V(), PS_ILGetTextureParameter_P(), MLS_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureParameterIuivEXT() {
			return newUnlinkedGLFunction("glGetTextureParameterIuivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LGetTextureParameter_P(), MLS_ILTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureParameterfv() {
			return newUnlinkedGLFunction("glGetTextureParameterfv", void.class, GLParamTypeSets.ML_V(), PS_ILGetTextureParameter_P(), MLS_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureParameterfvEXT() {
			return newUnlinkedGLFunction("glGetTextureParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LGetTextureParameter_P(), MLS_ILTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureParameteriv() {
			return newUnlinkedGLFunction("glGetTextureParameteriv", void.class, GLParamTypeSets.ML_V(), PS_ILGetTextureParameter_P(), MLS_ILGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureParameterivEXT() {
			return newUnlinkedGLFunction("glGetTextureParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LGetTextureParameter_P(), MLS_ILTextureTarget_LGetTextureParameter_P());
		}
		private static UnlinkedGLFunction make_glGetTextureSamplerHandleARB() {
			return newUnlinkedGLFunction("glGetTextureSamplerHandleARB", long.class, GLParamTypeSets.ML_J(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glGetTextureSamplerHandleIMG() {
			return newUnlinkedGLFunction("glGetTextureSamplerHandleIMG", long.class, GLParamTypeSets.ML_J(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glGetTextureSamplerHandleNV() {
			return newUnlinkedGLFunction("glGetTextureSamplerHandleNV", long.class, GLParamTypeSets.ML_J(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glGetTextureSubImage() {
			return newUnlinkedGLFunction("glGetTextureSubImage", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIILPixelFormat_LPixelType_IP(), MLS_IIIIIIIILPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetTrackMatrixivNV() {
			return newUnlinkedGLFunction("glGetTrackMatrixivNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_ILVertexAttribEnumNV_P(), MLS_LVertexAttribEnumNV_ILVertexAttribEnumNV_P());
		}
		private static UnlinkedGLFunction make_glGetTransformFeedbackVarying() {
			return newUnlinkedGLFunction("glGetTransformFeedbackVarying", void.class, GLParamTypeSets.ML_V(), PS_IIIPPPP(), MLS_IIIPPPP());
		}
		private static UnlinkedGLFunction make_glGetTransformFeedbackVaryingEXT() {
			return newUnlinkedGLFunction("glGetTransformFeedbackVaryingEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIPPPP(), MLS_IIIPPPP());
		}
		private static UnlinkedGLFunction make_glGetTransformFeedbackVaryingNV() {
			return newUnlinkedGLFunction("glGetTransformFeedbackVaryingNV", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetTransformFeedbacki64_v() {
			return newUnlinkedGLFunction("glGetTransformFeedbacki64_v", void.class, GLParamTypeSets.ML_V(), PS_ILTransformFeedbackPName_IP(), MLS_ILTransformFeedbackPName_IP());
		}
		private static UnlinkedGLFunction make_glGetTransformFeedbacki_v() {
			return newUnlinkedGLFunction("glGetTransformFeedbacki_v", void.class, GLParamTypeSets.ML_V(), PS_ILTransformFeedbackPName_IP(), MLS_ILTransformFeedbackPName_IP());
		}
		private static UnlinkedGLFunction make_glGetTransformFeedbackiv() {
			return newUnlinkedGLFunction("glGetTransformFeedbackiv", void.class, GLParamTypeSets.ML_V(), PS_ILTransformFeedbackPName_P(), MLS_ILTransformFeedbackPName_P());
		}
		private static UnlinkedGLFunction make_glGetTranslatedShaderSourceANGLE() {
			return newUnlinkedGLFunction("glGetTranslatedShaderSourceANGLE", void.class, GLParamTypeSets.ML_V(), PS_IIPP(), MLS_IIPP());
		}
		private static UnlinkedGLFunction make_glGetUniformBlockIndex() {
			return newUnlinkedGLFunction("glGetUniformBlockIndex", int.class, GLParamTypeSets.ML_I(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glGetUniformBufferSizeEXT() {
			return newUnlinkedGLFunction("glGetUniformBufferSizeEXT", int.class, GLParamTypeSets.ML_I(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glGetUniformIndices() {
			return newUnlinkedGLFunction("glGetUniformIndices", void.class, GLParamTypeSets.ML_V(), PS_IIpP(), MLS_IIpP());
		}
		private static UnlinkedGLFunction make_glGetUniformLocation() {
			return newUnlinkedGLFunction("glGetUniformLocation", int.class, GLParamTypeSets.ML_I(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glGetUniformLocationARB() {
			return newUnlinkedGLFunction("glGetUniformLocationARB", int.class, GLParamTypeSets.ML_I(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glGetUniformOffsetEXT() {
			return newUnlinkedGLFunction("glGetUniformOffsetEXT", long.class, GLParamTypeSets.ML_J(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glGetUniformSubroutineuiv() {
			return newUnlinkedGLFunction("glGetUniformSubroutineuiv", void.class, GLParamTypeSets.ML_V(), PS_LShaderType_IP(), MLS_LShaderType_IP());
		}
		private static UnlinkedGLFunction make_glGetUniformdv() {
			return newUnlinkedGLFunction("glGetUniformdv", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUniformfv() {
			return newUnlinkedGLFunction("glGetUniformfv", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUniformfvARB() {
			return newUnlinkedGLFunction("glGetUniformfvARB", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUniformi64vARB() {
			return newUnlinkedGLFunction("glGetUniformi64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUniformi64vNV() {
			return newUnlinkedGLFunction("glGetUniformi64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUniformiv() {
			return newUnlinkedGLFunction("glGetUniformiv", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUniformivARB() {
			return newUnlinkedGLFunction("glGetUniformivARB", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUniformui64vARB() {
			return newUnlinkedGLFunction("glGetUniformui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUniformui64vNV() {
			return newUnlinkedGLFunction("glGetUniformui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUniformuiv() {
			return newUnlinkedGLFunction("glGetUniformuiv", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUniformuivEXT() {
			return newUnlinkedGLFunction("glGetUniformuivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIP(), MLS_IIP());
		}
		private static UnlinkedGLFunction make_glGetUnsignedBytevEXT() {
			return newUnlinkedGLFunction("glGetUnsignedBytevEXT", void.class, GLParamTypeSets.ML_V(), PS_LGetPName_P(), MLS_LGetPName_P());
		}
		private static UnlinkedGLFunction make_glGetUnsignedBytei_vEXT() {
			return newUnlinkedGLFunction("glGetUnsignedBytei_vEXT", void.class, GLParamTypeSets.ML_V(), PS_EIP(), MLS_EIP());
		}
		private static UnlinkedGLFunction make_glGetVariantArrayObjectfvATI() {
			return newUnlinkedGLFunction("glGetVariantArrayObjectfvATI", void.class, GLParamTypeSets.ML_V(), PS_ILArrayObjectPNameATI_P(), MLS_ILArrayObjectPNameATI_P());
		}
		private static UnlinkedGLFunction make_glGetVariantArrayObjectivATI() {
			return newUnlinkedGLFunction("glGetVariantArrayObjectivATI", void.class, GLParamTypeSets.ML_V(), PS_ILArrayObjectPNameATI_P(), MLS_ILArrayObjectPNameATI_P());
		}
		private static UnlinkedGLFunction make_glGetVariantBooleanvEXT() {
			return newUnlinkedGLFunction("glGetVariantBooleanvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetVariantValueEXT_P(), MLS_ILGetVariantValueEXT_P());
		}
		private static UnlinkedGLFunction make_glGetVariantFloatvEXT() {
			return newUnlinkedGLFunction("glGetVariantFloatvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetVariantValueEXT_P(), MLS_ILGetVariantValueEXT_P());
		}
		private static UnlinkedGLFunction make_glGetVariantIntegervEXT() {
			return newUnlinkedGLFunction("glGetVariantIntegervEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetVariantValueEXT_P(), MLS_ILGetVariantValueEXT_P());
		}
		private static UnlinkedGLFunction make_glGetVariantPointervEXT() {
			return newUnlinkedGLFunction("glGetVariantPointervEXT", void.class, GLParamTypeSets.ML_V(), PS_ILGetVariantValueEXT_P(), MLS_ILGetVariantValueEXT_P());
		}
		private static UnlinkedGLFunction make_glGetVaryingLocationNV() {
			return newUnlinkedGLFunction("glGetVaryingLocationNV", int.class, GLParamTypeSets.ML_I(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glGetVertexArrayIndexed64iv() {
			return newUnlinkedGLFunction("glGetVertexArrayIndexed64iv", void.class, GLParamTypeSets.ML_V(), PS_IILVertexArrayPName_P(), MLS_IILVertexArrayPName_P());
		}
		private static UnlinkedGLFunction make_glGetVertexArrayIndexediv() {
			return newUnlinkedGLFunction("glGetVertexArrayIndexediv", void.class, GLParamTypeSets.ML_V(), PS_IILVertexArrayPName_P(), MLS_IILVertexArrayPName_P());
		}
		private static UnlinkedGLFunction make_glGetVertexArrayIntegeri_vEXT() {
			return newUnlinkedGLFunction("glGetVertexArrayIntegeri_vEXT", void.class, GLParamTypeSets.ML_V(), PS_IILVertexArrayPName_P(), MLS_IILVertexArrayPName_P());
		}
		private static UnlinkedGLFunction make_glGetVertexArrayIntegervEXT() {
			return newUnlinkedGLFunction("glGetVertexArrayIntegervEXT", void.class, GLParamTypeSets.ML_V(), PS_ILVertexArrayPName_P(), MLS_ILVertexArrayPName_P());
		}
		private static UnlinkedGLFunction make_glGetVertexArrayPointeri_vEXT() {
			return newUnlinkedGLFunction("glGetVertexArrayPointeri_vEXT", void.class, GLParamTypeSets.ML_V(), PS_IILVertexArrayPName_P(), MLS_IILVertexArrayPName_P());
		}
		private static UnlinkedGLFunction make_glGetVertexArrayPointervEXT() {
			return newUnlinkedGLFunction("glGetVertexArrayPointervEXT", void.class, GLParamTypeSets.ML_V(), PS_ILVertexArrayPName_P(), MLS_ILVertexArrayPName_P());
		}
		private static UnlinkedGLFunction make_glGetVertexArrayiv() {
			return newUnlinkedGLFunction("glGetVertexArrayiv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexArrayPName_P(), MLS_ILVertexArrayPName_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribArrayObjectfvATI() {
			return newUnlinkedGLFunction("glGetVertexAttribArrayObjectfvATI", void.class, GLParamTypeSets.ML_V(), PS_ILArrayObjectPNameATI_P(), MLS_ILArrayObjectPNameATI_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribArrayObjectivATI() {
			return newUnlinkedGLFunction("glGetVertexAttribArrayObjectivATI", void.class, GLParamTypeSets.ML_V(), PS_ILArrayObjectPNameATI_P(), MLS_ILArrayObjectPNameATI_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribIiv() {
			return newUnlinkedGLFunction("glGetVertexAttribIiv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnum_P(), MLS_ILVertexAttribEnum_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribIivEXT() {
			return newUnlinkedGLFunction("glGetVertexAttribIivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnum_P(), MLS_ILVertexAttribEnum_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribIuiv() {
			return newUnlinkedGLFunction("glGetVertexAttribIuiv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnum_P(), MLS_ILVertexAttribEnum_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribIuivEXT() {
			return newUnlinkedGLFunction("glGetVertexAttribIuivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnum_P(), MLS_ILVertexAttribEnum_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribLdv() {
			return newUnlinkedGLFunction("glGetVertexAttribLdv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnum_P(), MLS_ILVertexAttribEnum_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribLdvEXT() {
			return newUnlinkedGLFunction("glGetVertexAttribLdvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnum_P(), MLS_ILVertexAttribEnum_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribLi64vNV() {
			return newUnlinkedGLFunction("glGetVertexAttribLi64vNV", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnum_P(), MLS_ILVertexAttribEnum_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribLui64vARB() {
			return newUnlinkedGLFunction("glGetVertexAttribLui64vARB", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnum_P(), MLS_ILVertexAttribEnum_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribLui64vNV() {
			return newUnlinkedGLFunction("glGetVertexAttribLui64vNV", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnum_P(), MLS_ILVertexAttribEnum_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribPointerv() {
			return newUnlinkedGLFunction("glGetVertexAttribPointerv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPointerPropertyARB_P(), MLS_ILVertexAttribPointerPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribPointervARB() {
			return newUnlinkedGLFunction("glGetVertexAttribPointervARB", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPointerPropertyARB_P(), MLS_ILVertexAttribPointerPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribPointervNV() {
			return newUnlinkedGLFunction("glGetVertexAttribPointervNV", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnumNV_P(), MLS_ILVertexAttribEnumNV_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribdv() {
			return newUnlinkedGLFunction("glGetVertexAttribdv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPropertyARB_P(), MLS_ILVertexAttribPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribdvARB() {
			return newUnlinkedGLFunction("glGetVertexAttribdvARB", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPropertyARB_P(), MLS_ILVertexAttribPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribdvNV() {
			return newUnlinkedGLFunction("glGetVertexAttribdvNV", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnumNV_P(), MLS_ILVertexAttribEnumNV_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribfv() {
			return newUnlinkedGLFunction("glGetVertexAttribfv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPropertyARB_P(), MLS_ILVertexAttribPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribfvARB() {
			return newUnlinkedGLFunction("glGetVertexAttribfvARB", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPropertyARB_P(), MLS_ILVertexAttribPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribfvNV() {
			return newUnlinkedGLFunction("glGetVertexAttribfvNV", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnumNV_P(), MLS_ILVertexAttribEnumNV_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribiv() {
			return newUnlinkedGLFunction("glGetVertexAttribiv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPropertyARB_P(), MLS_ILVertexAttribPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribivARB() {
			return newUnlinkedGLFunction("glGetVertexAttribivARB", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPropertyARB_P(), MLS_ILVertexAttribPropertyARB_P());
		}
		private static UnlinkedGLFunction make_glGetVertexAttribivNV() {
			return newUnlinkedGLFunction("glGetVertexAttribivNV", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribEnumNV_P(), MLS_ILVertexAttribEnumNV_P());
		}
		private static UnlinkedGLFunction make_glGetVideoCaptureStreamdvNV() {
			return newUnlinkedGLFunction("glGetVideoCaptureStreamdvNV", void.class, GLParamTypeSets.ML_V(), PS_IIEP(), MLS_IIEP());
		}
		private static UnlinkedGLFunction make_glGetVideoCaptureStreamfvNV() {
			return newUnlinkedGLFunction("glGetVideoCaptureStreamfvNV", void.class, GLParamTypeSets.ML_V(), PS_IIEP(), MLS_IIEP());
		}
		private static UnlinkedGLFunction make_glGetVideoCaptureStreamivNV() {
			return newUnlinkedGLFunction("glGetVideoCaptureStreamivNV", void.class, GLParamTypeSets.ML_V(), PS_IIEP(), MLS_IIEP());
		}
		private static UnlinkedGLFunction make_glGetVideoCaptureivNV() {
			return newUnlinkedGLFunction("glGetVideoCaptureivNV", void.class, GLParamTypeSets.ML_V(), PS_IEP(), MLS_IEP());
		}
		private static UnlinkedGLFunction make_glGetVideoi64vNV() {
			return newUnlinkedGLFunction("glGetVideoi64vNV", void.class, GLParamTypeSets.ML_V(), PS_IEP(), MLS_IEP());
		}
		private static UnlinkedGLFunction make_glGetVideoivNV() {
			return newUnlinkedGLFunction("glGetVideoivNV", void.class, GLParamTypeSets.ML_V(), PS_IEP(), MLS_IEP());
		}
		private static UnlinkedGLFunction make_glGetVideoui64vNV() {
			return newUnlinkedGLFunction("glGetVideoui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IEP(), MLS_IEP());
		}
		private static UnlinkedGLFunction make_glGetVideouivNV() {
			return newUnlinkedGLFunction("glGetVideouivNV", void.class, GLParamTypeSets.ML_V(), PS_IEP(), MLS_IEP());
		}
		private static UnlinkedGLFunction make_glGetnColorTable() {
			return newUnlinkedGLFunction("glGetnColorTable", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LPixelFormat_LPixelType_IP(), MLS_LColorTableTarget_LPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetnColorTableARB() {
			return newUnlinkedGLFunction("glGetnColorTableARB", void.class, GLParamTypeSets.ML_V(), PS_LColorTableTarget_LPixelFormat_LPixelType_IP(), MLS_LColorTableTarget_LPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetnCompressedTexImage() {
			return newUnlinkedGLFunction("glGetnCompressedTexImage", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIP(), MLS_LTextureTarget_IIP());
		}
		private static UnlinkedGLFunction make_glGetnCompressedTexImageARB() {
			return newUnlinkedGLFunction("glGetnCompressedTexImageARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIP(), MLS_LTextureTarget_IIP());
		}
		private static UnlinkedGLFunction make_glGetnConvolutionFilter() {
			return newUnlinkedGLFunction("glGetnConvolutionFilter", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LPixelFormat_LPixelType_IP(), MLS_LConvolutionTarget_LPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetnConvolutionFilterARB() {
			return newUnlinkedGLFunction("glGetnConvolutionFilterARB", void.class, GLParamTypeSets.ML_V(), PS_LConvolutionTarget_LPixelFormat_LPixelType_IP(), MLS_LConvolutionTarget_LPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetnHistogram() {
			return newUnlinkedGLFunction("glGetnHistogram", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTarget_ZLPixelFormat_LPixelType_IP(), MLS_LHistogramTarget_ZLPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetnHistogramARB() {
			return newUnlinkedGLFunction("glGetnHistogramARB", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_ZLPixelFormat_LPixelType_IP(), MLS_LHistogramTargetEXT_ZLPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetnMapdv() {
			return newUnlinkedGLFunction("glGetnMapdv", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_LMapQuery_IP(), MLS_LMapTarget_LMapQuery_IP());
		}
		private static UnlinkedGLFunction make_glGetnMapdvARB() {
			return newUnlinkedGLFunction("glGetnMapdvARB", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_LMapQuery_IP(), MLS_LMapTarget_LMapQuery_IP());
		}
		private static UnlinkedGLFunction make_glGetnMapfv() {
			return newUnlinkedGLFunction("glGetnMapfv", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_LMapQuery_IP(), MLS_LMapTarget_LMapQuery_IP());
		}
		private static UnlinkedGLFunction make_glGetnMapfvARB() {
			return newUnlinkedGLFunction("glGetnMapfvARB", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_LMapQuery_IP(), MLS_LMapTarget_LMapQuery_IP());
		}
		private static UnlinkedGLFunction make_glGetnMapiv() {
			return newUnlinkedGLFunction("glGetnMapiv", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_LMapQuery_IP(), MLS_LMapTarget_LMapQuery_IP());
		}
		private static UnlinkedGLFunction make_glGetnMapivARB() {
			return newUnlinkedGLFunction("glGetnMapivARB", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_LMapQuery_IP(), MLS_LMapTarget_LMapQuery_IP());
		}
		private static UnlinkedGLFunction make_glGetnMinmax() {
			return newUnlinkedGLFunction("glGetnMinmax", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTarget_ZLPixelFormat_LPixelType_IP(), MLS_LMinmaxTarget_ZLPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetnMinmaxARB() {
			return newUnlinkedGLFunction("glGetnMinmaxARB", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_ZLPixelFormat_LPixelType_IP(), MLS_LMinmaxTargetEXT_ZLPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetnPixelMapfv() {
			return newUnlinkedGLFunction("glGetnPixelMapfv", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_IP(), MLS_LPixelMap_IP());
		}
		private static UnlinkedGLFunction make_glGetnPixelMapfvARB() {
			return newUnlinkedGLFunction("glGetnPixelMapfvARB", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_IP(), MLS_LPixelMap_IP());
		}
		private static UnlinkedGLFunction make_glGetnPixelMapuiv() {
			return newUnlinkedGLFunction("glGetnPixelMapuiv", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_IP(), MLS_LPixelMap_IP());
		}
		private static UnlinkedGLFunction make_glGetnPixelMapuivARB() {
			return newUnlinkedGLFunction("glGetnPixelMapuivARB", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_IP(), MLS_LPixelMap_IP());
		}
		private static UnlinkedGLFunction make_glGetnPixelMapusv() {
			return newUnlinkedGLFunction("glGetnPixelMapusv", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_IP(), MLS_LPixelMap_IP());
		}
		private static UnlinkedGLFunction make_glGetnPixelMapusvARB() {
			return newUnlinkedGLFunction("glGetnPixelMapusvARB", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_IP(), MLS_LPixelMap_IP());
		}
		private static UnlinkedGLFunction make_glGetnPolygonStipple() {
			return newUnlinkedGLFunction("glGetnPolygonStipple", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGetnPolygonStippleARB() {
			return newUnlinkedGLFunction("glGetnPolygonStippleARB", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glGetnSeparableFilter() {
			return newUnlinkedGLFunction("glGetnSeparableFilter", void.class, GLParamTypeSets.ML_V(), PS_LSeparableTarget_LPixelFormat_LPixelType_IPIPP(), MLS_LSeparableTarget_LPixelFormat_LPixelType_IPIPP());
		}
		private static UnlinkedGLFunction make_glGetnSeparableFilterARB() {
			return newUnlinkedGLFunction("glGetnSeparableFilterARB", void.class, GLParamTypeSets.ML_V(), PS_LSeparableTargetEXT_LPixelFormat_LPixelType_IPIPP(), MLS_LSeparableTargetEXT_LPixelFormat_LPixelType_IPIPP());
		}
		private static UnlinkedGLFunction make_glGetnTexImage() {
			return newUnlinkedGLFunction("glGetnTexImage", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILPixelFormat_LPixelType_IP(), MLS_LTextureTarget_ILPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetnTexImageARB() {
			return newUnlinkedGLFunction("glGetnTexImageARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILPixelFormat_LPixelType_IP(), MLS_LTextureTarget_ILPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glGetnUniformdv() {
			return newUnlinkedGLFunction("glGetnUniformdv", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformdvARB() {
			return newUnlinkedGLFunction("glGetnUniformdvARB", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformfv() {
			return newUnlinkedGLFunction("glGetnUniformfv", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformfvARB() {
			return newUnlinkedGLFunction("glGetnUniformfvARB", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformfvEXT() {
			return newUnlinkedGLFunction("glGetnUniformfvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformfvKHR() {
			return newUnlinkedGLFunction("glGetnUniformfvKHR", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformi64vARB() {
			return newUnlinkedGLFunction("glGetnUniformi64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformiv() {
			return newUnlinkedGLFunction("glGetnUniformiv", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformivARB() {
			return newUnlinkedGLFunction("glGetnUniformivARB", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformivEXT() {
			return newUnlinkedGLFunction("glGetnUniformivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformivKHR() {
			return newUnlinkedGLFunction("glGetnUniformivKHR", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformui64vARB() {
			return newUnlinkedGLFunction("glGetnUniformui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformuiv() {
			return newUnlinkedGLFunction("glGetnUniformuiv", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformuivARB() {
			return newUnlinkedGLFunction("glGetnUniformuivARB", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGetnUniformuivKHR() {
			return newUnlinkedGLFunction("glGetnUniformuivKHR", void.class, GLParamTypeSets.ML_V(), PS_IIIP(), MLS_IIIP());
		}
		private static UnlinkedGLFunction make_glGlobalAlphaFactorbSUN() {
			return newUnlinkedGLFunction("glGlobalAlphaFactorbSUN", void.class, GLParamTypeSets.ML_V(), PS_B(), MLS_B());
		}
		private static UnlinkedGLFunction make_glGlobalAlphaFactordSUN() {
			return newUnlinkedGLFunction("glGlobalAlphaFactordSUN", void.class, GLParamTypeSets.ML_V(), PS_D(), MLS_D());
		}
		private static UnlinkedGLFunction make_glGlobalAlphaFactorfSUN() {
			return newUnlinkedGLFunction("glGlobalAlphaFactorfSUN", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glGlobalAlphaFactoriSUN() {
			return newUnlinkedGLFunction("glGlobalAlphaFactoriSUN", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGlobalAlphaFactorsSUN() {
			return newUnlinkedGLFunction("glGlobalAlphaFactorsSUN", void.class, GLParamTypeSets.ML_V(), PS_S(), MLS_S());
		}
		private static UnlinkedGLFunction make_glGlobalAlphaFactorubSUN() {
			return newUnlinkedGLFunction("glGlobalAlphaFactorubSUN", void.class, GLParamTypeSets.ML_V(), PS_B(), MLS_B());
		}
		private static UnlinkedGLFunction make_glGlobalAlphaFactoruiSUN() {
			return newUnlinkedGLFunction("glGlobalAlphaFactoruiSUN", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glGlobalAlphaFactorusSUN() {
			return newUnlinkedGLFunction("glGlobalAlphaFactorusSUN", void.class, GLParamTypeSets.ML_V(), PS_C(), MLS_C());
		}
		private static UnlinkedGLFunction make_glHint() {
			return newUnlinkedGLFunction("glHint", void.class, GLParamTypeSets.ML_V(), PS_LHintTarget_LHintMode_(), MLS_LHintTarget_LHintMode_());
		}
		private static UnlinkedGLFunction make_glHintPGI() {
			return newUnlinkedGLFunction("glHintPGI", void.class, GLParamTypeSets.ML_V(), PS_LHintTargetPGI_LVertexHintsMaskPGI_(), MLS_LHintTargetPGI_LVertexHintsMaskPGI_());
		}
		private static UnlinkedGLFunction make_glHistogram() {
			return newUnlinkedGLFunction("glHistogram", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_ILInternalFormat_Z(), MLS_LHistogramTargetEXT_ILInternalFormat_Z());
		}
		private static UnlinkedGLFunction make_glHistogramEXT() {
			return newUnlinkedGLFunction("glHistogramEXT", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_ILInternalFormat_Z(), MLS_LHistogramTargetEXT_ILInternalFormat_Z());
		}
		private static UnlinkedGLFunction make_glIglooInterfaceSGIX() {
			return newUnlinkedGLFunction("glIglooInterfaceSGIX", void.class, GLParamTypeSets.ML_V(), PS_Ep(), MLS_Ep());
		}
		private static UnlinkedGLFunction make_glImageTransformParameterfHP() {
			return newUnlinkedGLFunction("glImageTransformParameterfHP", void.class, GLParamTypeSets.ML_V(), PS_LImageTransformTargetHP_LImageTransformPNameHP_F(), MLS_LImageTransformTargetHP_LImageTransformPNameHP_F());
		}
		private static UnlinkedGLFunction make_glImageTransformParameterfvHP() {
			return newUnlinkedGLFunction("glImageTransformParameterfvHP", void.class, GLParamTypeSets.ML_V(), PS_LImageTransformTargetHP_LImageTransformPNameHP_p(), MLS_LImageTransformTargetHP_LImageTransformPNameHP_p());
		}
		private static UnlinkedGLFunction make_glImageTransformParameteriHP() {
			return newUnlinkedGLFunction("glImageTransformParameteriHP", void.class, GLParamTypeSets.ML_V(), PS_LImageTransformTargetHP_LImageTransformPNameHP_I(), MLS_LImageTransformTargetHP_LImageTransformPNameHP_I());
		}
		private static UnlinkedGLFunction make_glImageTransformParameterivHP() {
			return newUnlinkedGLFunction("glImageTransformParameterivHP", void.class, GLParamTypeSets.ML_V(), PS_LImageTransformTargetHP_LImageTransformPNameHP_p(), MLS_LImageTransformTargetHP_LImageTransformPNameHP_p());
		}
		private static UnlinkedGLFunction make_glImportMemoryFdEXT() {
			return newUnlinkedGLFunction("glImportMemoryFdEXT", void.class, GLParamTypeSets.ML_V(), PS_IJLExternalHandleType_I(), MLS_IJLExternalHandleType_I());
		}
		private static UnlinkedGLFunction make_glImportMemoryWin32HandleEXT() {
			return newUnlinkedGLFunction("glImportMemoryWin32HandleEXT", void.class, GLParamTypeSets.ML_V(), PS_IJLExternalHandleType_P(), MLS_IJLExternalHandleType_P());
		}
		private static UnlinkedGLFunction make_glImportMemoryWin32NameEXT() {
			return newUnlinkedGLFunction("glImportMemoryWin32NameEXT", void.class, GLParamTypeSets.ML_V(), PS_IJLExternalHandleType_p(), MLS_IJLExternalHandleType_p());
		}
		private static UnlinkedGLFunction make_glImportSemaphoreFdEXT() {
			return newUnlinkedGLFunction("glImportSemaphoreFdEXT", void.class, GLParamTypeSets.ML_V(), PS_ILExternalHandleType_I(), MLS_ILExternalHandleType_I());
		}
		private static UnlinkedGLFunction make_glImportSemaphoreWin32HandleEXT() {
			return newUnlinkedGLFunction("glImportSemaphoreWin32HandleEXT", void.class, GLParamTypeSets.ML_V(), PS_ILExternalHandleType_P(), MLS_ILExternalHandleType_P());
		}
		private static UnlinkedGLFunction make_glImportSemaphoreWin32NameEXT() {
			return newUnlinkedGLFunction("glImportSemaphoreWin32NameEXT", void.class, GLParamTypeSets.ML_V(), PS_ILExternalHandleType_p(), MLS_ILExternalHandleType_p());
		}
		private static UnlinkedGLFunction make_glImportSyncEXT() {
			return newUnlinkedGLFunction("glImportSyncEXT", int.class, GLParamTypeSets.ML_I(), PS_EJI(), MLS_EJI());
		}
		private static UnlinkedGLFunction make_glIndexFormatNV() {
			return newUnlinkedGLFunction("glIndexFormatNV", void.class, GLParamTypeSets.ML_V(), PS_EI(), MLS_EI());
		}
		private static UnlinkedGLFunction make_glIndexFuncEXT() {
			return newUnlinkedGLFunction("glIndexFuncEXT", void.class, GLParamTypeSets.ML_V(), PS_LIndexFunctionEXT_F(), MLS_LIndexFunctionEXT_F());
		}
		private static UnlinkedGLFunction make_glIndexMask() {
			return newUnlinkedGLFunction("glIndexMask", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIndexMaterialEXT() {
			return newUnlinkedGLFunction("glIndexMaterialEXT", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LIndexMaterialParameterEXT_(), MLS_LTriangleFace_LIndexMaterialParameterEXT_());
		}
		private static UnlinkedGLFunction make_glIndexPointer() {
			return newUnlinkedGLFunction("glIndexPointer", void.class, GLParamTypeSets.ML_V(), PS_LIndexPointerType_Ip(), MLS_LIndexPointerType_Ip());
		}
		private static UnlinkedGLFunction make_glIndexPointerEXT() {
			return newUnlinkedGLFunction("glIndexPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_LIndexPointerType_IIp(), MLS_LIndexPointerType_IIp());
		}
		private static UnlinkedGLFunction make_glIndexPointerListIBM() {
			return newUnlinkedGLFunction("glIndexPointerListIBM", void.class, GLParamTypeSets.ML_V(), PS_LIndexPointerType_IPI(), MLS_LIndexPointerType_IPI());
		}
		private static UnlinkedGLFunction make_glIndexd() {
			return newUnlinkedGLFunction("glIndexd", void.class, GLParamTypeSets.ML_V(), PS_D(), MLS_D());
		}
		private static UnlinkedGLFunction make_glIndexdv() {
			return newUnlinkedGLFunction("glIndexdv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glIndexf() {
			return newUnlinkedGLFunction("glIndexf", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glIndexfv() {
			return newUnlinkedGLFunction("glIndexfv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glIndexi() {
			return newUnlinkedGLFunction("glIndexi", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIndexiv() {
			return newUnlinkedGLFunction("glIndexiv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glIndexs() {
			return newUnlinkedGLFunction("glIndexs", void.class, GLParamTypeSets.ML_V(), PS_S(), MLS_S());
		}
		private static UnlinkedGLFunction make_glIndexsv() {
			return newUnlinkedGLFunction("glIndexsv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glIndexub() {
			return newUnlinkedGLFunction("glIndexub", void.class, GLParamTypeSets.ML_V(), PS_B(), MLS_B());
		}
		private static UnlinkedGLFunction make_glIndexubv() {
			return newUnlinkedGLFunction("glIndexubv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glIndexxOES() {
			return newUnlinkedGLFunction("glIndexxOES", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIndexxvOES() {
			return newUnlinkedGLFunction("glIndexxvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glInitNames() {
			return newUnlinkedGLFunction("glInitNames", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glInsertComponentEXT() {
			return newUnlinkedGLFunction("glInsertComponentEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glInsertEventMarkerEXT() {
			return newUnlinkedGLFunction("glInsertEventMarkerEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glInstrumentsBufferSGIX() {
			return newUnlinkedGLFunction("glInstrumentsBufferSGIX", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glInterleavedArrays() {
			return newUnlinkedGLFunction("glInterleavedArrays", void.class, GLParamTypeSets.ML_V(), PS_LInterleavedArrayFormat_Ip(), MLS_LInterleavedArrayFormat_Ip());
		}
		private static UnlinkedGLFunction make_glInterpolatePathsNV() {
			return newUnlinkedGLFunction("glInterpolatePathsNV", void.class, GLParamTypeSets.ML_V(), PS_IIIF(), MLS_IIIF());
		}
		private static UnlinkedGLFunction make_glInvalidateBufferData() {
			return newUnlinkedGLFunction("glInvalidateBufferData", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glInvalidateBufferSubData() {
			return newUnlinkedGLFunction("glInvalidateBufferSubData", void.class, GLParamTypeSets.ML_V(), PS_IJJ(), MLS_IJJ());
		}
		private static UnlinkedGLFunction make_glInvalidateFramebuffer() {
			return newUnlinkedGLFunction("glInvalidateFramebuffer", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_Ip(), MLS_LFramebufferTarget_Ip());
		}
		private static UnlinkedGLFunction make_glInvalidateNamedFramebufferData() {
			return newUnlinkedGLFunction("glInvalidateNamedFramebufferData", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glInvalidateNamedFramebufferSubData() {
			return newUnlinkedGLFunction("glInvalidateNamedFramebufferSubData", void.class, GLParamTypeSets.ML_V(), PS_IIpIIII(), MLS_IIpIIII());
		}
		private static UnlinkedGLFunction make_glInvalidateSubFramebuffer() {
			return newUnlinkedGLFunction("glInvalidateSubFramebuffer", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_IpIIII(), MLS_LFramebufferTarget_IpIIII());
		}
		private static UnlinkedGLFunction make_glInvalidateTexImage() {
			return newUnlinkedGLFunction("glInvalidateTexImage", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glInvalidateTexSubImage() {
			return newUnlinkedGLFunction("glInvalidateTexSubImage", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIII(), MLS_IIIIIIII());
		}
		private static UnlinkedGLFunction make_glIsAsyncMarkerSGIX() {
			return newUnlinkedGLFunction("glIsAsyncMarkerSGIX", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsBuffer() {
			return newUnlinkedGLFunction("glIsBuffer", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsBufferARB() {
			return newUnlinkedGLFunction("glIsBufferARB", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsBufferResidentNV() {
			return newUnlinkedGLFunction("glIsBufferResidentNV", boolean.class, GLParamTypeSets.ML_Z(), PS_E(), MLS_E());
		}
		private static UnlinkedGLFunction make_glIsCommandListNV() {
			return newUnlinkedGLFunction("glIsCommandListNV", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsEnabled() {
			return newUnlinkedGLFunction("glIsEnabled", boolean.class, GLParamTypeSets.ML_Z(), PS_LEnableCap_(), MLS_LEnableCap_());
		}
		private static UnlinkedGLFunction make_glIsEnabledIndexedEXT() {
			return newUnlinkedGLFunction("glIsEnabledIndexedEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glIsEnabledi() {
			return newUnlinkedGLFunction("glIsEnabledi", boolean.class, GLParamTypeSets.ML_Z(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glIsEnablediEXT() {
			return newUnlinkedGLFunction("glIsEnablediEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glIsEnablediNV() {
			return newUnlinkedGLFunction("glIsEnablediNV", boolean.class, GLParamTypeSets.ML_Z(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glIsEnablediOES() {
			return newUnlinkedGLFunction("glIsEnablediOES", boolean.class, GLParamTypeSets.ML_Z(), PS_LEnableCap_I(), MLS_LEnableCap_I());
		}
		private static UnlinkedGLFunction make_glIsFenceAPPLE() {
			return newUnlinkedGLFunction("glIsFenceAPPLE", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsFenceNV() {
			return newUnlinkedGLFunction("glIsFenceNV", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsFramebuffer() {
			return newUnlinkedGLFunction("glIsFramebuffer", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsFramebufferEXT() {
			return newUnlinkedGLFunction("glIsFramebufferEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsFramebufferOES() {
			return newUnlinkedGLFunction("glIsFramebufferOES", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsImageHandleResidentARB() {
			return newUnlinkedGLFunction("glIsImageHandleResidentARB", boolean.class, GLParamTypeSets.ML_Z(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glIsImageHandleResidentNV() {
			return newUnlinkedGLFunction("glIsImageHandleResidentNV", boolean.class, GLParamTypeSets.ML_Z(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glIsList() {
			return newUnlinkedGLFunction("glIsList", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsMemoryObjectEXT() {
			return newUnlinkedGLFunction("glIsMemoryObjectEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsNameAMD() {
			return newUnlinkedGLFunction("glIsNameAMD", boolean.class, GLParamTypeSets.ML_Z(), PS_EI(), MLS_EI());
		}
		private static UnlinkedGLFunction make_glIsNamedBufferResidentNV() {
			return newUnlinkedGLFunction("glIsNamedBufferResidentNV", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsNamedStringARB() {
			return newUnlinkedGLFunction("glIsNamedStringARB", boolean.class, GLParamTypeSets.ML_Z(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glIsObjectBufferATI() {
			return newUnlinkedGLFunction("glIsObjectBufferATI", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsOcclusionQueryNV() {
			return newUnlinkedGLFunction("glIsOcclusionQueryNV", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsPathNV() {
			return newUnlinkedGLFunction("glIsPathNV", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsPointInFillPathNV() {
			return newUnlinkedGLFunction("glIsPointInFillPathNV", boolean.class, GLParamTypeSets.ML_Z(), PS_IIFF(), MLS_IIFF());
		}
		private static UnlinkedGLFunction make_glIsPointInStrokePathNV() {
			return newUnlinkedGLFunction("glIsPointInStrokePathNV", boolean.class, GLParamTypeSets.ML_Z(), PS_IFF(), MLS_IFF());
		}
		private static UnlinkedGLFunction make_glIsProgram() {
			return newUnlinkedGLFunction("glIsProgram", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsProgramARB() {
			return newUnlinkedGLFunction("glIsProgramARB", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsProgramNV() {
			return newUnlinkedGLFunction("glIsProgramNV", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsProgramPipeline() {
			return newUnlinkedGLFunction("glIsProgramPipeline", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsProgramPipelineEXT() {
			return newUnlinkedGLFunction("glIsProgramPipelineEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsQuery() {
			return newUnlinkedGLFunction("glIsQuery", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsQueryARB() {
			return newUnlinkedGLFunction("glIsQueryARB", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsQueryEXT() {
			return newUnlinkedGLFunction("glIsQueryEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsRenderbuffer() {
			return newUnlinkedGLFunction("glIsRenderbuffer", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsRenderbufferEXT() {
			return newUnlinkedGLFunction("glIsRenderbufferEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsRenderbufferOES() {
			return newUnlinkedGLFunction("glIsRenderbufferOES", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsSemaphoreEXT() {
			return newUnlinkedGLFunction("glIsSemaphoreEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsSampler() {
			return newUnlinkedGLFunction("glIsSampler", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsShader() {
			return newUnlinkedGLFunction("glIsShader", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsStateNV() {
			return newUnlinkedGLFunction("glIsStateNV", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsSync() {
			return newUnlinkedGLFunction("glIsSync", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsSyncAPPLE() {
			return newUnlinkedGLFunction("glIsSyncAPPLE", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsTexture() {
			return newUnlinkedGLFunction("glIsTexture", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsTextureEXT() {
			return newUnlinkedGLFunction("glIsTextureEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsTextureHandleResidentARB() {
			return newUnlinkedGLFunction("glIsTextureHandleResidentARB", boolean.class, GLParamTypeSets.ML_Z(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glIsTextureHandleResidentNV() {
			return newUnlinkedGLFunction("glIsTextureHandleResidentNV", boolean.class, GLParamTypeSets.ML_Z(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glIsTransformFeedback() {
			return newUnlinkedGLFunction("glIsTransformFeedback", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsTransformFeedbackNV() {
			return newUnlinkedGLFunction("glIsTransformFeedbackNV", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsVariantEnabledEXT() {
			return newUnlinkedGLFunction("glIsVariantEnabledEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_ILVariantCapEXT_(), MLS_ILVariantCapEXT_());
		}
		private static UnlinkedGLFunction make_glIsVertexArray() {
			return newUnlinkedGLFunction("glIsVertexArray", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsVertexArrayAPPLE() {
			return newUnlinkedGLFunction("glIsVertexArrayAPPLE", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsVertexArrayOES() {
			return newUnlinkedGLFunction("glIsVertexArrayOES", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glIsVertexAttribEnabledAPPLE() {
			return newUnlinkedGLFunction("glIsVertexAttribEnabledAPPLE", boolean.class, GLParamTypeSets.ML_Z(), PS_IE(), MLS_IE());
		}
		private static UnlinkedGLFunction make_glLGPUCopyImageSubDataNVX() {
			return newUnlinkedGLFunction("glLGPUCopyImageSubDataNVX", void.class, GLParamTypeSets.ML_V(), PS_IIIEIIIIIEIIIIIII(), MLS_IIIEIIIIIEIIIIIII());
		}
		private static UnlinkedGLFunction make_glLGPUInterlockNVX() {
			return newUnlinkedGLFunction("glLGPUInterlockNVX", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glLGPUNamedBufferSubDataNVX() {
			return newUnlinkedGLFunction("glLGPUNamedBufferSubDataNVX", void.class, GLParamTypeSets.ML_V(), PS_IIJJp(), MLS_IIJJp());
		}
		private static UnlinkedGLFunction make_glLabelObjectEXT() {
			return newUnlinkedGLFunction("glLabelObjectEXT", void.class, GLParamTypeSets.ML_V(), PS_EIIp(), MLS_EIIp());
		}
		private static UnlinkedGLFunction make_glLightEnviSGIX() {
			return newUnlinkedGLFunction("glLightEnviSGIX", void.class, GLParamTypeSets.ML_V(), PS_LLightEnvParameterSGIX_LLightEnvModeSGIX_(), MLS_LLightEnvParameterSGIX_LLightEnvModeSGIX_());
		}
		private static UnlinkedGLFunction make_glLightModelf() {
			return newUnlinkedGLFunction("glLightModelf", void.class, GLParamTypeSets.ML_V(), PS_LLightModelParameter_F(), MLS_LLightModelParameter_F());
		}
		private static UnlinkedGLFunction make_glLightModelfv() {
			return newUnlinkedGLFunction("glLightModelfv", void.class, GLParamTypeSets.ML_V(), PS_LLightModelParameter_p(), MLS_LLightModelParameter_p());
		}
		private static UnlinkedGLFunction make_glLightModeli() {
			return newUnlinkedGLFunction("glLightModeli", void.class, GLParamTypeSets.ML_V(), PS_LLightModelParameter_I(), MLS_LLightModelParameter_I());
		}
		private static UnlinkedGLFunction make_glLightModeliv() {
			return newUnlinkedGLFunction("glLightModeliv", void.class, GLParamTypeSets.ML_V(), PS_LLightModelParameter_p(), MLS_LLightModelParameter_p());
		}
		private static UnlinkedGLFunction make_glLightModelx() {
			return newUnlinkedGLFunction("glLightModelx", void.class, GLParamTypeSets.ML_V(), PS_LLightModelParameter_I(), MLS_LLightModelParameter_I());
		}
		private static UnlinkedGLFunction make_glLightModelxOES() {
			return newUnlinkedGLFunction("glLightModelxOES", void.class, GLParamTypeSets.ML_V(), PS_LLightModelParameter_I(), MLS_LLightModelParameter_I());
		}
		private static UnlinkedGLFunction make_glLightModelxv() {
			return newUnlinkedGLFunction("glLightModelxv", void.class, GLParamTypeSets.ML_V(), PS_LLightModelParameter_p(), MLS_LLightModelParameter_p());
		}
		private static UnlinkedGLFunction make_glLightModelxvOES() {
			return newUnlinkedGLFunction("glLightModelxvOES", void.class, GLParamTypeSets.ML_V(), PS_LLightModelParameter_p(), MLS_LLightModelParameter_p());
		}
		private static UnlinkedGLFunction make_glLightf() {
			return newUnlinkedGLFunction("glLightf", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_F(), MLS_LLightName_LLightParameter_F());
		}
		private static UnlinkedGLFunction make_glLightfv() {
			return newUnlinkedGLFunction("glLightfv", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_p(), MLS_LLightName_LLightParameter_p());
		}
		private static UnlinkedGLFunction make_glLighti() {
			return newUnlinkedGLFunction("glLighti", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_I(), MLS_LLightName_LLightParameter_I());
		}
		private static UnlinkedGLFunction make_glLightiv() {
			return newUnlinkedGLFunction("glLightiv", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_p(), MLS_LLightName_LLightParameter_p());
		}
		private static UnlinkedGLFunction make_glLightx() {
			return newUnlinkedGLFunction("glLightx", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_I(), MLS_LLightName_LLightParameter_I());
		}
		private static UnlinkedGLFunction make_glLightxOES() {
			return newUnlinkedGLFunction("glLightxOES", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_I(), MLS_LLightName_LLightParameter_I());
		}
		private static UnlinkedGLFunction make_glLightxv() {
			return newUnlinkedGLFunction("glLightxv", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_p(), MLS_LLightName_LLightParameter_p());
		}
		private static UnlinkedGLFunction make_glLightxvOES() {
			return newUnlinkedGLFunction("glLightxvOES", void.class, GLParamTypeSets.ML_V(), PS_LLightName_LLightParameter_p(), MLS_LLightName_LLightParameter_p());
		}
		private static UnlinkedGLFunction make_glLineStipple() {
			return newUnlinkedGLFunction("glLineStipple", void.class, GLParamTypeSets.ML_V(), PS_IC(), MLS_IC());
		}
		private static UnlinkedGLFunction make_glLineWidth() {
			return newUnlinkedGLFunction("glLineWidth", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glLineWidthx() {
			return newUnlinkedGLFunction("glLineWidthx", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glLineWidthxOES() {
			return newUnlinkedGLFunction("glLineWidthxOES", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glLinkProgram() {
			return newUnlinkedGLFunction("glLinkProgram", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glLinkProgramARB() {
			return newUnlinkedGLFunction("glLinkProgramARB", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glListBase() {
			return newUnlinkedGLFunction("glListBase", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glListDrawCommandsStatesClientNV() {
			return newUnlinkedGLFunction("glListDrawCommandsStatesClientNV", void.class, GLParamTypeSets.ML_V(), PS_IIPpppI(), MLS_IIPpppI());
		}
		private static UnlinkedGLFunction make_glListParameterfSGIX() {
			return newUnlinkedGLFunction("glListParameterfSGIX", void.class, GLParamTypeSets.ML_V(), PS_ILListParameterName_F(), MLS_ILListParameterName_F());
		}
		private static UnlinkedGLFunction make_glListParameterfvSGIX() {
			return newUnlinkedGLFunction("glListParameterfvSGIX", void.class, GLParamTypeSets.ML_V(), PS_ILListParameterName_p(), MLS_ILListParameterName_p());
		}
		private static UnlinkedGLFunction make_glListParameteriSGIX() {
			return newUnlinkedGLFunction("glListParameteriSGIX", void.class, GLParamTypeSets.ML_V(), PS_ILListParameterName_I(), MLS_ILListParameterName_I());
		}
		private static UnlinkedGLFunction make_glListParameterivSGIX() {
			return newUnlinkedGLFunction("glListParameterivSGIX", void.class, GLParamTypeSets.ML_V(), PS_ILListParameterName_p(), MLS_ILListParameterName_p());
		}
		private static UnlinkedGLFunction make_glLoadIdentity() {
			return newUnlinkedGLFunction("glLoadIdentity", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glLoadIdentityDeformationMapSGIX() {
			return newUnlinkedGLFunction("glLoadIdentityDeformationMapSGIX", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glLoadMatrixd() {
			return newUnlinkedGLFunction("glLoadMatrixd", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glLoadMatrixf() {
			return newUnlinkedGLFunction("glLoadMatrixf", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glLoadMatrixx() {
			return newUnlinkedGLFunction("glLoadMatrixx", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glLoadMatrixxOES() {
			return newUnlinkedGLFunction("glLoadMatrixxOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glLoadName() {
			return newUnlinkedGLFunction("glLoadName", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glLoadPaletteFromModelViewMatrixOES() {
			return newUnlinkedGLFunction("glLoadPaletteFromModelViewMatrixOES", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glLoadProgramNV() {
			return newUnlinkedGLFunction("glLoadProgramNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_IIp(), MLS_LVertexAttribEnumNV_IIp());
		}
		private static UnlinkedGLFunction make_glLoadTransposeMatrixd() {
			return newUnlinkedGLFunction("glLoadTransposeMatrixd", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glLoadTransposeMatrixdARB() {
			return newUnlinkedGLFunction("glLoadTransposeMatrixdARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glLoadTransposeMatrixf() {
			return newUnlinkedGLFunction("glLoadTransposeMatrixf", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glLoadTransposeMatrixfARB() {
			return newUnlinkedGLFunction("glLoadTransposeMatrixfARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glLoadTransposeMatrixxOES() {
			return newUnlinkedGLFunction("glLoadTransposeMatrixxOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glLockArraysEXT() {
			return newUnlinkedGLFunction("glLockArraysEXT", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glLogicOp() {
			return newUnlinkedGLFunction("glLogicOp", void.class, GLParamTypeSets.ML_V(), PS_LLogicOp_(), MLS_LLogicOp_());
		}
		private static UnlinkedGLFunction make_glMakeBufferNonResidentNV() {
			return newUnlinkedGLFunction("glMakeBufferNonResidentNV", void.class, GLParamTypeSets.ML_V(), PS_E(), MLS_E());
		}
		private static UnlinkedGLFunction make_glMakeBufferResidentNV() {
			return newUnlinkedGLFunction("glMakeBufferResidentNV", void.class, GLParamTypeSets.ML_V(), PS_EE(), MLS_EE());
		}
		private static UnlinkedGLFunction make_glMakeImageHandleNonResidentARB() {
			return newUnlinkedGLFunction("glMakeImageHandleNonResidentARB", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glMakeImageHandleNonResidentNV() {
			return newUnlinkedGLFunction("glMakeImageHandleNonResidentNV", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glMakeImageHandleResidentARB() {
			return newUnlinkedGLFunction("glMakeImageHandleResidentARB", void.class, GLParamTypeSets.ML_V(), PS_JE(), MLS_JE());
		}
		private static UnlinkedGLFunction make_glMakeImageHandleResidentNV() {
			return newUnlinkedGLFunction("glMakeImageHandleResidentNV", void.class, GLParamTypeSets.ML_V(), PS_JE(), MLS_JE());
		}
		private static UnlinkedGLFunction make_glMakeNamedBufferNonResidentNV() {
			return newUnlinkedGLFunction("glMakeNamedBufferNonResidentNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glMakeNamedBufferResidentNV() {
			return newUnlinkedGLFunction("glMakeNamedBufferResidentNV", void.class, GLParamTypeSets.ML_V(), PS_IE(), MLS_IE());
		}
		private static UnlinkedGLFunction make_glMakeTextureHandleNonResidentARB() {
			return newUnlinkedGLFunction("glMakeTextureHandleNonResidentARB", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glMakeTextureHandleNonResidentNV() {
			return newUnlinkedGLFunction("glMakeTextureHandleNonResidentNV", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glMakeTextureHandleResidentARB() {
			return newUnlinkedGLFunction("glMakeTextureHandleResidentARB", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glMakeTextureHandleResidentNV() {
			return newUnlinkedGLFunction("glMakeTextureHandleResidentNV", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glMap1d() {
			return newUnlinkedGLFunction("glMap1d", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_DDIIp(), MLS_LMapTarget_DDIIp());
		}
		private static UnlinkedGLFunction make_glMap1f() {
			return newUnlinkedGLFunction("glMap1f", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_FFIIp(), MLS_LMapTarget_FFIIp());
		}
		private static UnlinkedGLFunction make_glMap1xOES() {
			return newUnlinkedGLFunction("glMap1xOES", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_IIIII(), MLS_LMapTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glMap2d() {
			return newUnlinkedGLFunction("glMap2d", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_DDIIDDIIp(), MLS_LMapTarget_DDIIDDIIp());
		}
		private static UnlinkedGLFunction make_glMap2f() {
			return newUnlinkedGLFunction("glMap2f", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_FFIIFFIIp(), MLS_LMapTarget_FFIIFFIIp());
		}
		private static UnlinkedGLFunction make_glMap2xOES() {
			return newUnlinkedGLFunction("glMap2xOES", void.class, GLParamTypeSets.ML_V(), PS_LMapTarget_IIIIIIIII(), MLS_LMapTarget_IIIIIIIII());
		}
		private static UnlinkedGLFunction make_glMapBuffer() {
			return newUnlinkedGLFunction("glMapBuffer", MutPointer.class, GLParamTypeSets.ML_P(), PS_LBufferTarget_LBufferAccess_(), MLS_LBufferTarget_LBufferAccess_());
		}
		private static UnlinkedGLFunction make_glMapBufferARB() {
			return newUnlinkedGLFunction("glMapBufferARB", MutPointer.class, GLParamTypeSets.ML_P(), PS_LBufferTarget_LBufferAccess_(), MLS_LBufferTarget_LBufferAccess_());
		}
		private static UnlinkedGLFunction make_glMapBufferOES() {
			return newUnlinkedGLFunction("glMapBufferOES", MutPointer.class, GLParamTypeSets.ML_P(), PS_LBufferTarget_LBufferAccess_(), MLS_LBufferTarget_LBufferAccess_());
		}
		private static UnlinkedGLFunction make_glMapBufferRange() {
			return newUnlinkedGLFunction("glMapBufferRange", MutPointer.class, GLParamTypeSets.ML_P(), PS_LBufferTarget_JJI(), MLS_LBufferTarget_JJI());
		}
		private static UnlinkedGLFunction make_glMapBufferRangeEXT() {
			return newUnlinkedGLFunction("glMapBufferRangeEXT", MutPointer.class, GLParamTypeSets.ML_P(), PS_LBufferTarget_JJI(), MLS_LBufferTarget_JJI());
		}
		private static UnlinkedGLFunction make_glMapControlPointsNV() {
			return newUnlinkedGLFunction("glMapControlPointsNV", void.class, GLParamTypeSets.ML_V(), PS_LEvalTargetNV_ILMapTypeNV_IIIIZp(), MLS_LEvalTargetNV_ILMapTypeNV_IIIIZp());
		}
		private static UnlinkedGLFunction make_glMapGrid1d() {
			return newUnlinkedGLFunction("glMapGrid1d", void.class, GLParamTypeSets.ML_V(), PS_IDD(), MLS_IDD());
		}
		private static UnlinkedGLFunction make_glMapGrid1f() {
			return newUnlinkedGLFunction("glMapGrid1f", void.class, GLParamTypeSets.ML_V(), PS_IFF(), MLS_IFF());
		}
		private static UnlinkedGLFunction make_glMapGrid1xOES() {
			return newUnlinkedGLFunction("glMapGrid1xOES", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glMapGrid2d() {
			return newUnlinkedGLFunction("glMapGrid2d", void.class, GLParamTypeSets.ML_V(), PS_IDDIDD(), MLS_IDDIDD());
		}
		private static UnlinkedGLFunction make_glMapGrid2f() {
			return newUnlinkedGLFunction("glMapGrid2f", void.class, GLParamTypeSets.ML_V(), PS_IFFIFF(), MLS_IFFIFF());
		}
		private static UnlinkedGLFunction make_glMapGrid2xOES() {
			return newUnlinkedGLFunction("glMapGrid2xOES", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glMapNamedBuffer() {
			return newUnlinkedGLFunction("glMapNamedBuffer", MutPointer.class, GLParamTypeSets.ML_P(), PS_ILBufferAccess_(), MLS_ILBufferAccess_());
		}
		private static UnlinkedGLFunction make_glMapNamedBufferEXT() {
			return newUnlinkedGLFunction("glMapNamedBufferEXT", MutPointer.class, GLParamTypeSets.ML_P(), PS_ILBufferAccess_(), MLS_ILBufferAccess_());
		}
		private static UnlinkedGLFunction make_glMapNamedBufferRange() {
			return newUnlinkedGLFunction("glMapNamedBufferRange", MutPointer.class, GLParamTypeSets.ML_P(), PS_IJJI(), MLS_IJJI());
		}
		private static UnlinkedGLFunction make_glMapNamedBufferRangeEXT() {
			return newUnlinkedGLFunction("glMapNamedBufferRangeEXT", MutPointer.class, GLParamTypeSets.ML_P(), PS_IJJI(), MLS_IJJI());
		}
		private static UnlinkedGLFunction make_glMapObjectBufferATI() {
			return newUnlinkedGLFunction("glMapObjectBufferATI", MutPointer.class, GLParamTypeSets.ML_P(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glMapParameterfvNV() {
			return newUnlinkedGLFunction("glMapParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_LEvalTargetNV_LMapParameterNV_p(), MLS_LEvalTargetNV_LMapParameterNV_p());
		}
		private static UnlinkedGLFunction make_glMapParameterivNV() {
			return newUnlinkedGLFunction("glMapParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_LEvalTargetNV_LMapParameterNV_p(), MLS_LEvalTargetNV_LMapParameterNV_p());
		}
		private static UnlinkedGLFunction make_glMapTexture2DINTEL() {
			return newUnlinkedGLFunction("glMapTexture2DINTEL", MutPointer.class, GLParamTypeSets.ML_P(), PS_IIIPP(), MLS_IIIPP());
		}
		private static UnlinkedGLFunction make_glMapVertexAttrib1dAPPLE() {
			return newUnlinkedGLFunction("glMapVertexAttrib1dAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IIDDIIp(), MLS_IIDDIIp());
		}
		private static UnlinkedGLFunction make_glMapVertexAttrib1fAPPLE() {
			return newUnlinkedGLFunction("glMapVertexAttrib1fAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IIFFIIp(), MLS_IIFFIIp());
		}
		private static UnlinkedGLFunction make_glMapVertexAttrib2dAPPLE() {
			return newUnlinkedGLFunction("glMapVertexAttrib2dAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IIDDIIDDIIp(), MLS_IIDDIIDDIIp());
		}
		private static UnlinkedGLFunction make_glMapVertexAttrib2fAPPLE() {
			return newUnlinkedGLFunction("glMapVertexAttrib2fAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IIFFIIFFIIp(), MLS_IIFFIIFFIIp());
		}
		private static UnlinkedGLFunction make_glMaterialf() {
			return newUnlinkedGLFunction("glMaterialf", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_F(), MLS_LTriangleFace_LMaterialParameter_F());
		}
		private static UnlinkedGLFunction make_glMaterialfv() {
			return newUnlinkedGLFunction("glMaterialfv", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_p(), MLS_LTriangleFace_LMaterialParameter_p());
		}
		private static UnlinkedGLFunction make_glMateriali() {
			return newUnlinkedGLFunction("glMateriali", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_I(), MLS_LTriangleFace_LMaterialParameter_I());
		}
		private static UnlinkedGLFunction make_glMaterialiv() {
			return newUnlinkedGLFunction("glMaterialiv", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_p(), MLS_LTriangleFace_LMaterialParameter_p());
		}
		private static UnlinkedGLFunction make_glMaterialx() {
			return newUnlinkedGLFunction("glMaterialx", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_I(), MLS_LTriangleFace_LMaterialParameter_I());
		}
		private static UnlinkedGLFunction make_glMaterialxOES() {
			return newUnlinkedGLFunction("glMaterialxOES", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_I(), MLS_LTriangleFace_LMaterialParameter_I());
		}
		private static UnlinkedGLFunction make_glMaterialxv() {
			return newUnlinkedGLFunction("glMaterialxv", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_p(), MLS_LTriangleFace_LMaterialParameter_p());
		}
		private static UnlinkedGLFunction make_glMaterialxvOES() {
			return newUnlinkedGLFunction("glMaterialxvOES", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_p(), MLS_LTriangleFace_LMaterialParameter_p());
		}
		private static UnlinkedGLFunction make_glMatrixFrustumEXT() {
			return newUnlinkedGLFunction("glMatrixFrustumEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_DDDDDD(), MLS_LMatrixMode_DDDDDD());
		}
		private static UnlinkedGLFunction make_glMatrixIndexPointerARB() {
			return newUnlinkedGLFunction("glMatrixIndexPointerARB", void.class, GLParamTypeSets.ML_V(), PS_ILMatrixIndexPointerTypeARB_Ip(), MLS_ILMatrixIndexPointerTypeARB_Ip());
		}
		private static UnlinkedGLFunction make_glMatrixIndexPointerOES() {
			return newUnlinkedGLFunction("glMatrixIndexPointerOES", void.class, GLParamTypeSets.ML_V(), PS_ILMatrixIndexPointerTypeARB_Ip(), MLS_ILMatrixIndexPointerTypeARB_Ip());
		}
		private static UnlinkedGLFunction make_glMatrixIndexubvARB() {
			return newUnlinkedGLFunction("glMatrixIndexubvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glMatrixIndexuivARB() {
			return newUnlinkedGLFunction("glMatrixIndexuivARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glMatrixIndexusvARB() {
			return newUnlinkedGLFunction("glMatrixIndexusvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glMatrixLoad3x2fNV() {
			return newUnlinkedGLFunction("glMatrixLoad3x2fNV", void.class, GLParamTypeSets.ML_V(), PS_Ep(), MLS_Ep());
		}
		private static UnlinkedGLFunction make_glMatrixLoad3x3fNV() {
			return newUnlinkedGLFunction("glMatrixLoad3x3fNV", void.class, GLParamTypeSets.ML_V(), PS_Ep(), MLS_Ep());
		}
		private static UnlinkedGLFunction make_glMatrixLoadIdentityEXT() {
			return newUnlinkedGLFunction("glMatrixLoadIdentityEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_(), MLS_LMatrixMode_());
		}
		private static UnlinkedGLFunction make_glMatrixLoadTranspose3x3fNV() {
			return newUnlinkedGLFunction("glMatrixLoadTranspose3x3fNV", void.class, GLParamTypeSets.ML_V(), PS_Ep(), MLS_Ep());
		}
		private static UnlinkedGLFunction make_glMatrixLoadTransposedEXT() {
			return newUnlinkedGLFunction("glMatrixLoadTransposedEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_p(), MLS_LMatrixMode_p());
		}
		private static UnlinkedGLFunction make_glMatrixLoadTransposefEXT() {
			return newUnlinkedGLFunction("glMatrixLoadTransposefEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_p(), MLS_LMatrixMode_p());
		}
		private static UnlinkedGLFunction make_glMatrixLoaddEXT() {
			return newUnlinkedGLFunction("glMatrixLoaddEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_p(), MLS_LMatrixMode_p());
		}
		private static UnlinkedGLFunction make_glMatrixLoadfEXT() {
			return newUnlinkedGLFunction("glMatrixLoadfEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_p(), MLS_LMatrixMode_p());
		}
		private static UnlinkedGLFunction make_glMatrixMode() {
			return newUnlinkedGLFunction("glMatrixMode", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_(), MLS_LMatrixMode_());
		}
		private static UnlinkedGLFunction make_glMatrixMult3x2fNV() {
			return newUnlinkedGLFunction("glMatrixMult3x2fNV", void.class, GLParamTypeSets.ML_V(), PS_Ep(), MLS_Ep());
		}
		private static UnlinkedGLFunction make_glMatrixMult3x3fNV() {
			return newUnlinkedGLFunction("glMatrixMult3x3fNV", void.class, GLParamTypeSets.ML_V(), PS_Ep(), MLS_Ep());
		}
		private static UnlinkedGLFunction make_glMatrixMultTranspose3x3fNV() {
			return newUnlinkedGLFunction("glMatrixMultTranspose3x3fNV", void.class, GLParamTypeSets.ML_V(), PS_Ep(), MLS_Ep());
		}
		private static UnlinkedGLFunction make_glMatrixMultTransposedEXT() {
			return newUnlinkedGLFunction("glMatrixMultTransposedEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_p(), MLS_LMatrixMode_p());
		}
		private static UnlinkedGLFunction make_glMatrixMultTransposefEXT() {
			return newUnlinkedGLFunction("glMatrixMultTransposefEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_p(), MLS_LMatrixMode_p());
		}
		private static UnlinkedGLFunction make_glMatrixMultdEXT() {
			return newUnlinkedGLFunction("glMatrixMultdEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_p(), MLS_LMatrixMode_p());
		}
		private static UnlinkedGLFunction make_glMatrixMultfEXT() {
			return newUnlinkedGLFunction("glMatrixMultfEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_p(), MLS_LMatrixMode_p());
		}
		private static UnlinkedGLFunction make_glMatrixOrthoEXT() {
			return newUnlinkedGLFunction("glMatrixOrthoEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_DDDDDD(), MLS_LMatrixMode_DDDDDD());
		}
		private static UnlinkedGLFunction make_glMatrixPopEXT() {
			return newUnlinkedGLFunction("glMatrixPopEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_(), MLS_LMatrixMode_());
		}
		private static UnlinkedGLFunction make_glMatrixPushEXT() {
			return newUnlinkedGLFunction("glMatrixPushEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_(), MLS_LMatrixMode_());
		}
		private static UnlinkedGLFunction make_glMatrixRotatedEXT() {
			return newUnlinkedGLFunction("glMatrixRotatedEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_DDDD(), MLS_LMatrixMode_DDDD());
		}
		private static UnlinkedGLFunction make_glMatrixRotatefEXT() {
			return newUnlinkedGLFunction("glMatrixRotatefEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_FFFF(), MLS_LMatrixMode_FFFF());
		}
		private static UnlinkedGLFunction make_glMatrixScaledEXT() {
			return newUnlinkedGLFunction("glMatrixScaledEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_DDD(), MLS_LMatrixMode_DDD());
		}
		private static UnlinkedGLFunction make_glMatrixScalefEXT() {
			return newUnlinkedGLFunction("glMatrixScalefEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_FFF(), MLS_LMatrixMode_FFF());
		}
		private static UnlinkedGLFunction make_glMatrixTranslatedEXT() {
			return newUnlinkedGLFunction("glMatrixTranslatedEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_DDD(), MLS_LMatrixMode_DDD());
		}
		private static UnlinkedGLFunction make_glMatrixTranslatefEXT() {
			return newUnlinkedGLFunction("glMatrixTranslatefEXT", void.class, GLParamTypeSets.ML_V(), PS_LMatrixMode_FFF(), MLS_LMatrixMode_FFF());
		}
		private static UnlinkedGLFunction make_glMaxActiveShaderCoresARM() {
			return newUnlinkedGLFunction("glMaxActiveShaderCoresARM", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glMaxShaderCompilerThreadsKHR() {
			return newUnlinkedGLFunction("glMaxShaderCompilerThreadsKHR", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glMaxShaderCompilerThreadsARB() {
			return newUnlinkedGLFunction("glMaxShaderCompilerThreadsARB", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glMemoryBarrier() {
			return newUnlinkedGLFunction("glMemoryBarrier", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glMemoryBarrierByRegion() {
			return newUnlinkedGLFunction("glMemoryBarrierByRegion", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glMemoryBarrierEXT() {
			return newUnlinkedGLFunction("glMemoryBarrierEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glMemoryObjectParameterivEXT() {
			return newUnlinkedGLFunction("glMemoryObjectParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILMemoryObjectParameterName_p(), MLS_ILMemoryObjectParameterName_p());
		}
		private static UnlinkedGLFunction make_glMinSampleShading() {
			return newUnlinkedGLFunction("glMinSampleShading", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glMinSampleShadingARB() {
			return newUnlinkedGLFunction("glMinSampleShadingARB", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glMinSampleShadingOES() {
			return newUnlinkedGLFunction("glMinSampleShadingOES", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glMinmax() {
			return newUnlinkedGLFunction("glMinmax", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_LInternalFormat_Z(), MLS_LMinmaxTargetEXT_LInternalFormat_Z());
		}
		private static UnlinkedGLFunction make_glMinmaxEXT() {
			return newUnlinkedGLFunction("glMinmaxEXT", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_LInternalFormat_Z(), MLS_LMinmaxTargetEXT_LInternalFormat_Z());
		}
		private static UnlinkedGLFunction make_glMultMatrixd() {
			return newUnlinkedGLFunction("glMultMatrixd", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glMultMatrixf() {
			return newUnlinkedGLFunction("glMultMatrixf", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glMultMatrixx() {
			return newUnlinkedGLFunction("glMultMatrixx", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glMultMatrixxOES() {
			return newUnlinkedGLFunction("glMultMatrixxOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glMultTransposeMatrixd() {
			return newUnlinkedGLFunction("glMultTransposeMatrixd", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glMultTransposeMatrixdARB() {
			return newUnlinkedGLFunction("glMultTransposeMatrixdARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glMultTransposeMatrixf() {
			return newUnlinkedGLFunction("glMultTransposeMatrixf", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glMultTransposeMatrixfARB() {
			return newUnlinkedGLFunction("glMultTransposeMatrixfARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glMultTransposeMatrixxOES() {
			return newUnlinkedGLFunction("glMultTransposeMatrixxOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glMultiDrawArrays() {
			return newUnlinkedGLFunction("glMultiDrawArrays", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ppI(), MLS_LPrimitiveType_ppI());
		}
		private static UnlinkedGLFunction make_glMultiDrawArraysEXT() {
			return newUnlinkedGLFunction("glMultiDrawArraysEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ppI(), MLS_LPrimitiveType_ppI());
		}
		private static UnlinkedGLFunction make_glMultiDrawArraysIndirect() {
			return newUnlinkedGLFunction("glMultiDrawArraysIndirect", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pII(), MLS_LPrimitiveType_pII());
		}
		private static UnlinkedGLFunction make_glMultiDrawArraysIndirectAMD() {
			return newUnlinkedGLFunction("glMultiDrawArraysIndirectAMD", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pII(), MLS_LPrimitiveType_pII());
		}
		private static UnlinkedGLFunction make_glMultiDrawArraysIndirectBindlessCountNV() {
			return newUnlinkedGLFunction("glMultiDrawArraysIndirectBindlessCountNV", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pIIII(), MLS_LPrimitiveType_pIIII());
		}
		private static UnlinkedGLFunction make_glMultiDrawArraysIndirectBindlessNV() {
			return newUnlinkedGLFunction("glMultiDrawArraysIndirectBindlessNV", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pIII(), MLS_LPrimitiveType_pIII());
		}
		private static UnlinkedGLFunction make_glMultiDrawArraysIndirectCount() {
			return newUnlinkedGLFunction("glMultiDrawArraysIndirectCount", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pJII(), MLS_LPrimitiveType_pJII());
		}
		private static UnlinkedGLFunction make_glMultiDrawArraysIndirectCountARB() {
			return newUnlinkedGLFunction("glMultiDrawArraysIndirectCountARB", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pJII(), MLS_LPrimitiveType_pJII());
		}
		private static UnlinkedGLFunction make_glMultiDrawArraysIndirectEXT() {
			return newUnlinkedGLFunction("glMultiDrawArraysIndirectEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pII(), MLS_LPrimitiveType_pII());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementArrayAPPLE() {
			return newUnlinkedGLFunction("glMultiDrawElementArrayAPPLE", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_ppI(), MLS_LPrimitiveType_ppI());
		}
		private static UnlinkedGLFunction make_glMultiDrawElements() {
			return newUnlinkedGLFunction("glMultiDrawElements", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pLDrawElementsType_pI(), MLS_LPrimitiveType_pLDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementsBaseVertex() {
			return newUnlinkedGLFunction("glMultiDrawElementsBaseVertex", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pLDrawElementsType_pIp(), MLS_LPrimitiveType_pLDrawElementsType_pIp());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementsBaseVertexEXT() {
			return newUnlinkedGLFunction("glMultiDrawElementsBaseVertexEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pLDrawElementsType_pIp(), MLS_LPrimitiveType_pLDrawElementsType_pIp());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementsEXT() {
			return newUnlinkedGLFunction("glMultiDrawElementsEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_pLDrawElementsType_pI(), MLS_LPrimitiveType_pLDrawElementsType_pI());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementsIndirect() {
			return newUnlinkedGLFunction("glMultiDrawElementsIndirect", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_LDrawElementsType_pII(), MLS_LPrimitiveType_LDrawElementsType_pII());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementsIndirectAMD() {
			return newUnlinkedGLFunction("glMultiDrawElementsIndirectAMD", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_LDrawElementsType_pII(), MLS_LPrimitiveType_LDrawElementsType_pII());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementsIndirectBindlessCountNV() {
			return newUnlinkedGLFunction("glMultiDrawElementsIndirectBindlessCountNV", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_LDrawElementsType_pIIII(), MLS_LPrimitiveType_LDrawElementsType_pIIII());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementsIndirectBindlessNV() {
			return newUnlinkedGLFunction("glMultiDrawElementsIndirectBindlessNV", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_LDrawElementsType_pIII(), MLS_LPrimitiveType_LDrawElementsType_pIII());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementsIndirectCount() {
			return newUnlinkedGLFunction("glMultiDrawElementsIndirectCount", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_LDrawElementsType_pJII(), MLS_LPrimitiveType_LDrawElementsType_pJII());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementsIndirectCountARB() {
			return newUnlinkedGLFunction("glMultiDrawElementsIndirectCountARB", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_LDrawElementsType_pJII(), MLS_LPrimitiveType_LDrawElementsType_pJII());
		}
		private static UnlinkedGLFunction make_glMultiDrawElementsIndirectEXT() {
			return newUnlinkedGLFunction("glMultiDrawElementsIndirectEXT", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_LDrawElementsType_pII(), MLS_LPrimitiveType_LDrawElementsType_pII());
		}
		private static UnlinkedGLFunction make_glMultiDrawMeshTasksIndirectNV() {
			return newUnlinkedGLFunction("glMultiDrawMeshTasksIndirectNV", void.class, GLParamTypeSets.ML_V(), PS_JII(), MLS_JII());
		}
		private static UnlinkedGLFunction make_glMultiDrawMeshTasksIndirectCountNV() {
			return newUnlinkedGLFunction("glMultiDrawMeshTasksIndirectCountNV", void.class, GLParamTypeSets.ML_V(), PS_JJII(), MLS_JJII());
		}
		private static UnlinkedGLFunction make_glMultiDrawRangeElementArrayAPPLE() {
			return newUnlinkedGLFunction("glMultiDrawRangeElementArrayAPPLE", void.class, GLParamTypeSets.ML_V(), PS_LPrimitiveType_IIppI(), MLS_LPrimitiveType_IIppI());
		}
		private static UnlinkedGLFunction make_glMultiModeDrawArraysIBM() {
			return newUnlinkedGLFunction("glMultiModeDrawArraysIBM", void.class, GLParamTypeSets.ML_V(), PS_pppII(), MLS_pppII());
		}
		private static UnlinkedGLFunction make_glMultiModeDrawElementsIBM() {
			return newUnlinkedGLFunction("glMultiModeDrawElementsIBM", void.class, GLParamTypeSets.ML_V(), PS_ppLDrawElementsType_pII(), MLS_ppLDrawElementsType_pII());
		}
		private static UnlinkedGLFunction make_glMultiTexBufferEXT() {
			return newUnlinkedGLFunction("glMultiTexBufferEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LInternalFormat_I(), MLS_LTextureUnit_LTextureTarget_LInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1bOES() {
			return newUnlinkedGLFunction("glMultiTexCoord1bOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_B(), MLS_LTextureUnit_B());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1bvOES() {
			return newUnlinkedGLFunction("glMultiTexCoord1bvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1d() {
			return newUnlinkedGLFunction("glMultiTexCoord1d", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_D(), MLS_LTextureUnit_D());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1dARB() {
			return newUnlinkedGLFunction("glMultiTexCoord1dARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_D(), MLS_LTextureUnit_D());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1dv() {
			return newUnlinkedGLFunction("glMultiTexCoord1dv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1dvARB() {
			return newUnlinkedGLFunction("glMultiTexCoord1dvARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1f() {
			return newUnlinkedGLFunction("glMultiTexCoord1f", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_F(), MLS_LTextureUnit_F());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1fARB() {
			return newUnlinkedGLFunction("glMultiTexCoord1fARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_F(), MLS_LTextureUnit_F());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1fv() {
			return newUnlinkedGLFunction("glMultiTexCoord1fv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1fvARB() {
			return newUnlinkedGLFunction("glMultiTexCoord1fvARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1hNV() {
			return newUnlinkedGLFunction("glMultiTexCoord1hNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_C(), MLS_LTextureUnit_C());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1hvNV() {
			return newUnlinkedGLFunction("glMultiTexCoord1hvNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1i() {
			return newUnlinkedGLFunction("glMultiTexCoord1i", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_I(), MLS_LTextureUnit_I());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1iARB() {
			return newUnlinkedGLFunction("glMultiTexCoord1iARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_I(), MLS_LTextureUnit_I());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1iv() {
			return newUnlinkedGLFunction("glMultiTexCoord1iv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1ivARB() {
			return newUnlinkedGLFunction("glMultiTexCoord1ivARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1s() {
			return newUnlinkedGLFunction("glMultiTexCoord1s", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_S(), MLS_LTextureUnit_S());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1sARB() {
			return newUnlinkedGLFunction("glMultiTexCoord1sARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_S(), MLS_LTextureUnit_S());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1sv() {
			return newUnlinkedGLFunction("glMultiTexCoord1sv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1svARB() {
			return newUnlinkedGLFunction("glMultiTexCoord1svARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1xOES() {
			return newUnlinkedGLFunction("glMultiTexCoord1xOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_I(), MLS_LTextureUnit_I());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord1xvOES() {
			return newUnlinkedGLFunction("glMultiTexCoord1xvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2bOES() {
			return newUnlinkedGLFunction("glMultiTexCoord2bOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_BB(), MLS_LTextureUnit_BB());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2bvOES() {
			return newUnlinkedGLFunction("glMultiTexCoord2bvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2d() {
			return newUnlinkedGLFunction("glMultiTexCoord2d", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_DD(), MLS_LTextureUnit_DD());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2dARB() {
			return newUnlinkedGLFunction("glMultiTexCoord2dARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_DD(), MLS_LTextureUnit_DD());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2dv() {
			return newUnlinkedGLFunction("glMultiTexCoord2dv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2dvARB() {
			return newUnlinkedGLFunction("glMultiTexCoord2dvARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2f() {
			return newUnlinkedGLFunction("glMultiTexCoord2f", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_FF(), MLS_LTextureUnit_FF());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2fARB() {
			return newUnlinkedGLFunction("glMultiTexCoord2fARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_FF(), MLS_LTextureUnit_FF());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2fv() {
			return newUnlinkedGLFunction("glMultiTexCoord2fv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2fvARB() {
			return newUnlinkedGLFunction("glMultiTexCoord2fvARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2hNV() {
			return newUnlinkedGLFunction("glMultiTexCoord2hNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_CC(), MLS_LTextureUnit_CC());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2hvNV() {
			return newUnlinkedGLFunction("glMultiTexCoord2hvNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2i() {
			return newUnlinkedGLFunction("glMultiTexCoord2i", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_II(), MLS_LTextureUnit_II());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2iARB() {
			return newUnlinkedGLFunction("glMultiTexCoord2iARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_II(), MLS_LTextureUnit_II());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2iv() {
			return newUnlinkedGLFunction("glMultiTexCoord2iv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2ivARB() {
			return newUnlinkedGLFunction("glMultiTexCoord2ivARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2s() {
			return newUnlinkedGLFunction("glMultiTexCoord2s", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_SS(), MLS_LTextureUnit_SS());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2sARB() {
			return newUnlinkedGLFunction("glMultiTexCoord2sARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_SS(), MLS_LTextureUnit_SS());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2sv() {
			return newUnlinkedGLFunction("glMultiTexCoord2sv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2svARB() {
			return newUnlinkedGLFunction("glMultiTexCoord2svARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2xOES() {
			return newUnlinkedGLFunction("glMultiTexCoord2xOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_II(), MLS_LTextureUnit_II());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord2xvOES() {
			return newUnlinkedGLFunction("glMultiTexCoord2xvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3bOES() {
			return newUnlinkedGLFunction("glMultiTexCoord3bOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_BBB(), MLS_LTextureUnit_BBB());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3bvOES() {
			return newUnlinkedGLFunction("glMultiTexCoord3bvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3d() {
			return newUnlinkedGLFunction("glMultiTexCoord3d", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_DDD(), MLS_LTextureUnit_DDD());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3dARB() {
			return newUnlinkedGLFunction("glMultiTexCoord3dARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_DDD(), MLS_LTextureUnit_DDD());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3dv() {
			return newUnlinkedGLFunction("glMultiTexCoord3dv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3dvARB() {
			return newUnlinkedGLFunction("glMultiTexCoord3dvARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3f() {
			return newUnlinkedGLFunction("glMultiTexCoord3f", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_FFF(), MLS_LTextureUnit_FFF());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3fARB() {
			return newUnlinkedGLFunction("glMultiTexCoord3fARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_FFF(), MLS_LTextureUnit_FFF());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3fv() {
			return newUnlinkedGLFunction("glMultiTexCoord3fv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3fvARB() {
			return newUnlinkedGLFunction("glMultiTexCoord3fvARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3hNV() {
			return newUnlinkedGLFunction("glMultiTexCoord3hNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_CCC(), MLS_LTextureUnit_CCC());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3hvNV() {
			return newUnlinkedGLFunction("glMultiTexCoord3hvNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3i() {
			return newUnlinkedGLFunction("glMultiTexCoord3i", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_III(), MLS_LTextureUnit_III());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3iARB() {
			return newUnlinkedGLFunction("glMultiTexCoord3iARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_III(), MLS_LTextureUnit_III());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3iv() {
			return newUnlinkedGLFunction("glMultiTexCoord3iv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3ivARB() {
			return newUnlinkedGLFunction("glMultiTexCoord3ivARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3s() {
			return newUnlinkedGLFunction("glMultiTexCoord3s", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_SSS(), MLS_LTextureUnit_SSS());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3sARB() {
			return newUnlinkedGLFunction("glMultiTexCoord3sARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_SSS(), MLS_LTextureUnit_SSS());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3sv() {
			return newUnlinkedGLFunction("glMultiTexCoord3sv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3svARB() {
			return newUnlinkedGLFunction("glMultiTexCoord3svARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3xOES() {
			return newUnlinkedGLFunction("glMultiTexCoord3xOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_III(), MLS_LTextureUnit_III());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord3xvOES() {
			return newUnlinkedGLFunction("glMultiTexCoord3xvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4bOES() {
			return newUnlinkedGLFunction("glMultiTexCoord4bOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_BBBB(), MLS_LTextureUnit_BBBB());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4bvOES() {
			return newUnlinkedGLFunction("glMultiTexCoord4bvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4d() {
			return newUnlinkedGLFunction("glMultiTexCoord4d", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_DDDD(), MLS_LTextureUnit_DDDD());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4dARB() {
			return newUnlinkedGLFunction("glMultiTexCoord4dARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_DDDD(), MLS_LTextureUnit_DDDD());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4dv() {
			return newUnlinkedGLFunction("glMultiTexCoord4dv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4dvARB() {
			return newUnlinkedGLFunction("glMultiTexCoord4dvARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4f() {
			return newUnlinkedGLFunction("glMultiTexCoord4f", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_FFFF(), MLS_LTextureUnit_FFFF());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4fARB() {
			return newUnlinkedGLFunction("glMultiTexCoord4fARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_FFFF(), MLS_LTextureUnit_FFFF());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4fv() {
			return newUnlinkedGLFunction("glMultiTexCoord4fv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4fvARB() {
			return newUnlinkedGLFunction("glMultiTexCoord4fvARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4hNV() {
			return newUnlinkedGLFunction("glMultiTexCoord4hNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_CCCC(), MLS_LTextureUnit_CCCC());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4hvNV() {
			return newUnlinkedGLFunction("glMultiTexCoord4hvNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4i() {
			return newUnlinkedGLFunction("glMultiTexCoord4i", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_IIII(), MLS_LTextureUnit_IIII());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4iARB() {
			return newUnlinkedGLFunction("glMultiTexCoord4iARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_IIII(), MLS_LTextureUnit_IIII());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4iv() {
			return newUnlinkedGLFunction("glMultiTexCoord4iv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4ivARB() {
			return newUnlinkedGLFunction("glMultiTexCoord4ivARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4s() {
			return newUnlinkedGLFunction("glMultiTexCoord4s", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_SSSS(), MLS_LTextureUnit_SSSS());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4sARB() {
			return newUnlinkedGLFunction("glMultiTexCoord4sARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_SSSS(), MLS_LTextureUnit_SSSS());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4sv() {
			return newUnlinkedGLFunction("glMultiTexCoord4sv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4svARB() {
			return newUnlinkedGLFunction("glMultiTexCoord4svARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4x() {
			return newUnlinkedGLFunction("glMultiTexCoord4x", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_IIII(), MLS_LTextureUnit_IIII());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4xOES() {
			return newUnlinkedGLFunction("glMultiTexCoord4xOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_IIII(), MLS_LTextureUnit_IIII());
		}
		private static UnlinkedGLFunction make_glMultiTexCoord4xvOES() {
			return newUnlinkedGLFunction("glMultiTexCoord4xvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_p(), MLS_LTextureUnit_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoordP1ui() {
			return newUnlinkedGLFunction("glMultiTexCoordP1ui", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTexCoordPointerType_I(), MLS_LTextureUnit_LTexCoordPointerType_I());
		}
		private static UnlinkedGLFunction make_glMultiTexCoordP1uiv() {
			return newUnlinkedGLFunction("glMultiTexCoordP1uiv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTexCoordPointerType_p(), MLS_LTextureUnit_LTexCoordPointerType_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoordP2ui() {
			return newUnlinkedGLFunction("glMultiTexCoordP2ui", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTexCoordPointerType_I(), MLS_LTextureUnit_LTexCoordPointerType_I());
		}
		private static UnlinkedGLFunction make_glMultiTexCoordP2uiv() {
			return newUnlinkedGLFunction("glMultiTexCoordP2uiv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTexCoordPointerType_p(), MLS_LTextureUnit_LTexCoordPointerType_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoordP3ui() {
			return newUnlinkedGLFunction("glMultiTexCoordP3ui", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTexCoordPointerType_I(), MLS_LTextureUnit_LTexCoordPointerType_I());
		}
		private static UnlinkedGLFunction make_glMultiTexCoordP3uiv() {
			return newUnlinkedGLFunction("glMultiTexCoordP3uiv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTexCoordPointerType_p(), MLS_LTextureUnit_LTexCoordPointerType_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoordP4ui() {
			return newUnlinkedGLFunction("glMultiTexCoordP4ui", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTexCoordPointerType_I(), MLS_LTextureUnit_LTexCoordPointerType_I());
		}
		private static UnlinkedGLFunction make_glMultiTexCoordP4uiv() {
			return newUnlinkedGLFunction("glMultiTexCoordP4uiv", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTexCoordPointerType_p(), MLS_LTextureUnit_LTexCoordPointerType_p());
		}
		private static UnlinkedGLFunction make_glMultiTexCoordPointerEXT() {
			return newUnlinkedGLFunction("glMultiTexCoordPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_ILTexCoordPointerType_Ip(), MLS_LTextureUnit_ILTexCoordPointerType_Ip());
		}
		private static UnlinkedGLFunction make_glMultiTexEnvfEXT() {
			return newUnlinkedGLFunction("glMultiTexEnvfEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_F(), MLS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_F());
		}
		private static UnlinkedGLFunction make_glMultiTexEnvfvEXT() {
			return newUnlinkedGLFunction("glMultiTexEnvfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_p(), MLS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_p());
		}
		private static UnlinkedGLFunction make_glMultiTexEnviEXT() {
			return newUnlinkedGLFunction("glMultiTexEnviEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_I(), MLS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_I());
		}
		private static UnlinkedGLFunction make_glMultiTexEnvivEXT() {
			return newUnlinkedGLFunction("glMultiTexEnvivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_p(), MLS_LTextureUnit_LTextureEnvTarget_LTextureEnvParameter_p());
		}
		private static UnlinkedGLFunction make_glMultiTexGendEXT() {
			return newUnlinkedGLFunction("glMultiTexGendEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureCoordName_LTextureGenParameter_D(), MLS_LTextureUnit_LTextureCoordName_LTextureGenParameter_D());
		}
		private static UnlinkedGLFunction make_glMultiTexGendvEXT() {
			return newUnlinkedGLFunction("glMultiTexGendvEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureCoordName_LTextureGenParameter_p(), MLS_LTextureUnit_LTextureCoordName_LTextureGenParameter_p());
		}
		private static UnlinkedGLFunction make_glMultiTexGenfEXT() {
			return newUnlinkedGLFunction("glMultiTexGenfEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureCoordName_LTextureGenParameter_F(), MLS_LTextureUnit_LTextureCoordName_LTextureGenParameter_F());
		}
		private static UnlinkedGLFunction make_glMultiTexGenfvEXT() {
			return newUnlinkedGLFunction("glMultiTexGenfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureCoordName_LTextureGenParameter_p(), MLS_LTextureUnit_LTextureCoordName_LTextureGenParameter_p());
		}
		private static UnlinkedGLFunction make_glMultiTexGeniEXT() {
			return newUnlinkedGLFunction("glMultiTexGeniEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureCoordName_LTextureGenParameter_I(), MLS_LTextureUnit_LTextureCoordName_LTextureGenParameter_I());
		}
		private static UnlinkedGLFunction make_glMultiTexGenivEXT() {
			return newUnlinkedGLFunction("glMultiTexGenivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureCoordName_LTextureGenParameter_p(), MLS_LTextureUnit_LTextureCoordName_LTextureGenParameter_p());
		}
		private static UnlinkedGLFunction make_glMultiTexImage1DEXT() {
			return newUnlinkedGLFunction("glMultiTexImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILInternalFormat_IILPixelFormat_LPixelType_p(), MLS_LTextureUnit_LTextureTarget_ILInternalFormat_IILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glMultiTexImage2DEXT() {
			return newUnlinkedGLFunction("glMultiTexImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILInternalFormat_IIILPixelFormat_LPixelType_p(), MLS_LTextureUnit_LTextureTarget_ILInternalFormat_IIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glMultiTexImage3DEXT() {
			return newUnlinkedGLFunction("glMultiTexImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_ILInternalFormat_IIIILPixelFormat_LPixelType_p(), MLS_LTextureUnit_LTextureTarget_ILInternalFormat_IIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glMultiTexParameterIivEXT() {
			return newUnlinkedGLFunction("glMultiTexParameterIivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LTextureParameterName_p(), MLS_LTextureUnit_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glMultiTexParameterIuivEXT() {
			return newUnlinkedGLFunction("glMultiTexParameterIuivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LTextureParameterName_p(), MLS_LTextureUnit_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glMultiTexParameterfEXT() {
			return newUnlinkedGLFunction("glMultiTexParameterfEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LTextureParameterName_F(), MLS_LTextureUnit_LTextureTarget_LTextureParameterName_F());
		}
		private static UnlinkedGLFunction make_glMultiTexParameterfvEXT() {
			return newUnlinkedGLFunction("glMultiTexParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LTextureParameterName_p(), MLS_LTextureUnit_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glMultiTexParameteriEXT() {
			return newUnlinkedGLFunction("glMultiTexParameteriEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LTextureParameterName_I(), MLS_LTextureUnit_LTextureTarget_LTextureParameterName_I());
		}
		private static UnlinkedGLFunction make_glMultiTexParameterivEXT() {
			return newUnlinkedGLFunction("glMultiTexParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_LTextureParameterName_p(), MLS_LTextureUnit_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glMultiTexRenderbufferEXT() {
			return newUnlinkedGLFunction("glMultiTexRenderbufferEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_I(), MLS_LTextureUnit_LTextureTarget_I());
		}
		private static UnlinkedGLFunction make_glMultiTexSubImage1DEXT() {
			return newUnlinkedGLFunction("glMultiTexSubImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_IIILPixelFormat_LPixelType_p(), MLS_LTextureUnit_LTextureTarget_IIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glMultiTexSubImage2DEXT() {
			return newUnlinkedGLFunction("glMultiTexSubImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_IIIIILPixelFormat_LPixelType_p(), MLS_LTextureUnit_LTextureTarget_IIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glMultiTexSubImage3DEXT() {
			return newUnlinkedGLFunction("glMultiTexSubImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureUnit_LTextureTarget_IIIIIIILPixelFormat_LPixelType_p(), MLS_LTextureUnit_LTextureTarget_IIIIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glMulticastBarrierNV() {
			return newUnlinkedGLFunction("glMulticastBarrierNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glMulticastBlitFramebufferNV() {
			return newUnlinkedGLFunction("glMulticastBlitFramebufferNV", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIIIIE(), MLS_IIIIIIIIIIIE());
		}
		private static UnlinkedGLFunction make_glMulticastBufferSubDataNV() {
			return newUnlinkedGLFunction("glMulticastBufferSubDataNV", void.class, GLParamTypeSets.ML_V(), PS_IIJJp(), MLS_IIJJp());
		}
		private static UnlinkedGLFunction make_glMulticastCopyBufferSubDataNV() {
			return newUnlinkedGLFunction("glMulticastCopyBufferSubDataNV", void.class, GLParamTypeSets.ML_V(), PS_IIIIJJJ(), MLS_IIIIJJJ());
		}
		private static UnlinkedGLFunction make_glMulticastCopyImageSubDataNV() {
			return newUnlinkedGLFunction("glMulticastCopyImageSubDataNV", void.class, GLParamTypeSets.ML_V(), PS_IIIEIIIIIEIIIIIII(), MLS_IIIEIIIIIEIIIIIII());
		}
		private static UnlinkedGLFunction make_glMulticastFramebufferSampleLocationsfvNV() {
			return newUnlinkedGLFunction("glMulticastFramebufferSampleLocationsfvNV", void.class, GLParamTypeSets.ML_V(), PS_IIIIp(), MLS_IIIIp());
		}
		private static UnlinkedGLFunction make_glMulticastGetQueryObjecti64vNV() {
			return newUnlinkedGLFunction("glMulticastGetQueryObjecti64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIEP(), MLS_IIEP());
		}
		private static UnlinkedGLFunction make_glMulticastGetQueryObjectivNV() {
			return newUnlinkedGLFunction("glMulticastGetQueryObjectivNV", void.class, GLParamTypeSets.ML_V(), PS_IIEP(), MLS_IIEP());
		}
		private static UnlinkedGLFunction make_glMulticastGetQueryObjectui64vNV() {
			return newUnlinkedGLFunction("glMulticastGetQueryObjectui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIEP(), MLS_IIEP());
		}
		private static UnlinkedGLFunction make_glMulticastGetQueryObjectuivNV() {
			return newUnlinkedGLFunction("glMulticastGetQueryObjectuivNV", void.class, GLParamTypeSets.ML_V(), PS_IIEP(), MLS_IIEP());
		}
		private static UnlinkedGLFunction make_glMulticastScissorArrayvNVX() {
			return newUnlinkedGLFunction("glMulticastScissorArrayvNVX", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glMulticastViewportArrayvNVX() {
			return newUnlinkedGLFunction("glMulticastViewportArrayvNVX", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glMulticastViewportPositionWScaleNVX() {
			return newUnlinkedGLFunction("glMulticastViewportPositionWScaleNVX", void.class, GLParamTypeSets.ML_V(), PS_IIFF(), MLS_IIFF());
		}
		private static UnlinkedGLFunction make_glMulticastWaitSyncNV() {
			return newUnlinkedGLFunction("glMulticastWaitSyncNV", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glNamedBufferAttachMemoryNV() {
			return newUnlinkedGLFunction("glNamedBufferAttachMemoryNV", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glNamedBufferData() {
			return newUnlinkedGLFunction("glNamedBufferData", void.class, GLParamTypeSets.ML_V(), PS_IJpLVertexBufferObjectUsage_(), MLS_IJpLVertexBufferObjectUsage_());
		}
		private static UnlinkedGLFunction make_glNamedBufferDataEXT() {
			return newUnlinkedGLFunction("glNamedBufferDataEXT", void.class, GLParamTypeSets.ML_V(), PS_IJpLVertexBufferObjectUsage_(), MLS_IJpLVertexBufferObjectUsage_());
		}
		private static UnlinkedGLFunction make_glNamedBufferPageCommitmentARB() {
			return newUnlinkedGLFunction("glNamedBufferPageCommitmentARB", void.class, GLParamTypeSets.ML_V(), PS_IJJZ(), MLS_IJJZ());
		}
		private static UnlinkedGLFunction make_glNamedBufferPageCommitmentEXT() {
			return newUnlinkedGLFunction("glNamedBufferPageCommitmentEXT", void.class, GLParamTypeSets.ML_V(), PS_IJJZ(), MLS_IJJZ());
		}
		private static UnlinkedGLFunction make_glNamedBufferPageCommitmentMemNV() {
			return newUnlinkedGLFunction("glNamedBufferPageCommitmentMemNV", void.class, GLParamTypeSets.ML_V(), PS_IJJIJZ(), MLS_IJJIJZ());
		}
		private static UnlinkedGLFunction make_glNamedBufferStorage() {
			return newUnlinkedGLFunction("glNamedBufferStorage", void.class, GLParamTypeSets.ML_V(), PS_IJpI(), MLS_IJpI());
		}
		private static UnlinkedGLFunction make_glNamedBufferStorageExternalEXT() {
			return newUnlinkedGLFunction("glNamedBufferStorageExternalEXT", void.class, GLParamTypeSets.ML_V(), PS_IJJII(), MLS_IJJII());
		}
		private static UnlinkedGLFunction make_glNamedBufferStorageEXT() {
			return newUnlinkedGLFunction("glNamedBufferStorageEXT", void.class, GLParamTypeSets.ML_V(), PS_IJpI(), MLS_IJpI());
		}
		private static UnlinkedGLFunction make_glNamedBufferStorageMemEXT() {
			return newUnlinkedGLFunction("glNamedBufferStorageMemEXT", void.class, GLParamTypeSets.ML_V(), PS_IJIJ(), MLS_IJIJ());
		}
		private static UnlinkedGLFunction make_glNamedBufferSubData() {
			return newUnlinkedGLFunction("glNamedBufferSubData", void.class, GLParamTypeSets.ML_V(), PS_IJJp(), MLS_IJJp());
		}
		private static UnlinkedGLFunction make_glNamedBufferSubDataEXT() {
			return newUnlinkedGLFunction("glNamedBufferSubDataEXT", void.class, GLParamTypeSets.ML_V(), PS_IJJp(), MLS_IJJp());
		}
		private static UnlinkedGLFunction make_glNamedCopyBufferSubDataEXT() {
			return newUnlinkedGLFunction("glNamedCopyBufferSubDataEXT", void.class, GLParamTypeSets.ML_V(), PS_IIJJJ(), MLS_IIJJJ());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferDrawBuffer() {
			return newUnlinkedGLFunction("glNamedFramebufferDrawBuffer", void.class, GLParamTypeSets.ML_V(), PS_ILColorBuffer_(), MLS_ILColorBuffer_());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferDrawBuffers() {
			return newUnlinkedGLFunction("glNamedFramebufferDrawBuffers", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferParameteri() {
			return newUnlinkedGLFunction("glNamedFramebufferParameteri", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferParameterName_I(), MLS_ILFramebufferParameterName_I());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferParameteriEXT() {
			return newUnlinkedGLFunction("glNamedFramebufferParameteriEXT", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferParameterName_I(), MLS_ILFramebufferParameterName_I());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferReadBuffer() {
			return newUnlinkedGLFunction("glNamedFramebufferReadBuffer", void.class, GLParamTypeSets.ML_V(), PS_ILColorBuffer_(), MLS_ILColorBuffer_());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferRenderbuffer() {
			return newUnlinkedGLFunction("glNamedFramebufferRenderbuffer", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_LRenderbufferTarget_I(), MLS_ILFramebufferAttachment_LRenderbufferTarget_I());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferRenderbufferEXT() {
			return newUnlinkedGLFunction("glNamedFramebufferRenderbufferEXT", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_LRenderbufferTarget_I(), MLS_ILFramebufferAttachment_LRenderbufferTarget_I());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferSampleLocationsfvARB() {
			return newUnlinkedGLFunction("glNamedFramebufferSampleLocationsfvARB", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferSampleLocationsfvNV() {
			return newUnlinkedGLFunction("glNamedFramebufferSampleLocationsfvNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferTexture() {
			return newUnlinkedGLFunction("glNamedFramebufferTexture", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_II(), MLS_ILFramebufferAttachment_II());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferSamplePositionsfvAMD() {
			return newUnlinkedGLFunction("glNamedFramebufferSamplePositionsfvAMD", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferTexture1DEXT() {
			return newUnlinkedGLFunction("glNamedFramebufferTexture1DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_LTextureTarget_II(), MLS_ILFramebufferAttachment_LTextureTarget_II());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferTexture2DEXT() {
			return newUnlinkedGLFunction("glNamedFramebufferTexture2DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_LTextureTarget_II(), MLS_ILFramebufferAttachment_LTextureTarget_II());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferTexture3DEXT() {
			return newUnlinkedGLFunction("glNamedFramebufferTexture3DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_LTextureTarget_III(), MLS_ILFramebufferAttachment_LTextureTarget_III());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferTextureEXT() {
			return newUnlinkedGLFunction("glNamedFramebufferTextureEXT", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_II(), MLS_ILFramebufferAttachment_II());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferTextureFaceEXT() {
			return newUnlinkedGLFunction("glNamedFramebufferTextureFaceEXT", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_IILTextureTarget_(), MLS_ILFramebufferAttachment_IILTextureTarget_());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferTextureLayer() {
			return newUnlinkedGLFunction("glNamedFramebufferTextureLayer", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_III(), MLS_ILFramebufferAttachment_III());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferTextureLayerEXT() {
			return newUnlinkedGLFunction("glNamedFramebufferTextureLayerEXT", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_III(), MLS_ILFramebufferAttachment_III());
		}
		private static UnlinkedGLFunction make_glNamedFramebufferTextureMultiviewOVR() {
			return newUnlinkedGLFunction("glNamedFramebufferTextureMultiviewOVR", void.class, GLParamTypeSets.ML_V(), PS_ILFramebufferAttachment_IIII(), MLS_ILFramebufferAttachment_IIII());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParameter4dEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParameter4dEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IDDDD(), MLS_ILProgramTarget_IDDDD());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParameter4dvEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParameter4dvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_Ip(), MLS_ILProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParameter4fEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParameter4fEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IFFFF(), MLS_ILProgramTarget_IFFFF());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParameter4fvEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParameter4fvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_Ip(), MLS_ILProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParameterI4iEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParameterI4iEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IIIII(), MLS_ILProgramTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParameterI4ivEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParameterI4ivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_Ip(), MLS_ILProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParameterI4uiEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParameterI4uiEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IIIII(), MLS_ILProgramTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParameterI4uivEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParameterI4uivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_Ip(), MLS_ILProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParameters4fvEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParameters4fvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IIp(), MLS_ILProgramTarget_IIp());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParametersI4ivEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParametersI4ivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IIp(), MLS_ILProgramTarget_IIp());
		}
		private static UnlinkedGLFunction make_glNamedProgramLocalParametersI4uivEXT() {
			return newUnlinkedGLFunction("glNamedProgramLocalParametersI4uivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_IIp(), MLS_ILProgramTarget_IIp());
		}
		private static UnlinkedGLFunction make_glNamedProgramStringEXT() {
			return newUnlinkedGLFunction("glNamedProgramStringEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramTarget_LProgramFormat_Ip(), MLS_ILProgramTarget_LProgramFormat_Ip());
		}
		private static UnlinkedGLFunction make_glNamedRenderbufferStorage() {
			return newUnlinkedGLFunction("glNamedRenderbufferStorage", void.class, GLParamTypeSets.ML_V(), PS_ILInternalFormat_II(), MLS_ILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glNamedRenderbufferStorageEXT() {
			return newUnlinkedGLFunction("glNamedRenderbufferStorageEXT", void.class, GLParamTypeSets.ML_V(), PS_ILInternalFormat_II(), MLS_ILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glNamedRenderbufferStorageMultisample() {
			return newUnlinkedGLFunction("glNamedRenderbufferStorageMultisample", void.class, GLParamTypeSets.ML_V(), PS_IILInternalFormat_II(), MLS_IILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glNamedRenderbufferStorageMultisampleAdvancedAMD() {
			return newUnlinkedGLFunction("glNamedRenderbufferStorageMultisampleAdvancedAMD", void.class, GLParamTypeSets.ML_V(), PS_IIILInternalFormat_II(), MLS_IIILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glNamedRenderbufferStorageMultisampleCoverageEXT() {
			return newUnlinkedGLFunction("glNamedRenderbufferStorageMultisampleCoverageEXT", void.class, GLParamTypeSets.ML_V(), PS_IIILInternalFormat_II(), MLS_IIILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glNamedRenderbufferStorageMultisampleEXT() {
			return newUnlinkedGLFunction("glNamedRenderbufferStorageMultisampleEXT", void.class, GLParamTypeSets.ML_V(), PS_IILInternalFormat_II(), MLS_IILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glNamedStringARB() {
			return newUnlinkedGLFunction("glNamedStringARB", void.class, GLParamTypeSets.ML_V(), PS_EIpIp(), MLS_EIpIp());
		}
		private static UnlinkedGLFunction make_glNewList() {
			return newUnlinkedGLFunction("glNewList", void.class, GLParamTypeSets.ML_V(), PS_ILListMode_(), MLS_ILListMode_());
		}
		private static UnlinkedGLFunction make_glNewObjectBufferATI() {
			return newUnlinkedGLFunction("glNewObjectBufferATI", int.class, GLParamTypeSets.ML_I(), PS_IpLArrayObjectUsageATI_(), MLS_IpLArrayObjectUsageATI_());
		}
		private static UnlinkedGLFunction make_glNormal3b() {
			return newUnlinkedGLFunction("glNormal3b", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glNormal3bv() {
			return newUnlinkedGLFunction("glNormal3bv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glNormal3d() {
			return newUnlinkedGLFunction("glNormal3d", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glNormal3dv() {
			return newUnlinkedGLFunction("glNormal3dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glNormal3f() {
			return newUnlinkedGLFunction("glNormal3f", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glNormal3fVertex3fSUN() {
			return newUnlinkedGLFunction("glNormal3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_FFFFFF(), MLS_FFFFFF());
		}
		private static UnlinkedGLFunction make_glNormal3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glNormal3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glNormal3fv() {
			return newUnlinkedGLFunction("glNormal3fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glNormal3hNV() {
			return newUnlinkedGLFunction("glNormal3hNV", void.class, GLParamTypeSets.ML_V(), PS_CCC(), MLS_CCC());
		}
		private static UnlinkedGLFunction make_glNormal3hvNV() {
			return newUnlinkedGLFunction("glNormal3hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glNormal3i() {
			return newUnlinkedGLFunction("glNormal3i", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glNormal3iv() {
			return newUnlinkedGLFunction("glNormal3iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glNormal3s() {
			return newUnlinkedGLFunction("glNormal3s", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glNormal3sv() {
			return newUnlinkedGLFunction("glNormal3sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glNormal3x() {
			return newUnlinkedGLFunction("glNormal3x", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glNormal3xOES() {
			return newUnlinkedGLFunction("glNormal3xOES", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glNormal3xvOES() {
			return newUnlinkedGLFunction("glNormal3xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glNormalFormatNV() {
			return newUnlinkedGLFunction("glNormalFormatNV", void.class, GLParamTypeSets.ML_V(), PS_EI(), MLS_EI());
		}
		private static UnlinkedGLFunction make_glNormalP3ui() {
			return newUnlinkedGLFunction("glNormalP3ui", void.class, GLParamTypeSets.ML_V(), PS_LNormalPointerType_I(), MLS_LNormalPointerType_I());
		}
		private static UnlinkedGLFunction make_glNormalP3uiv() {
			return newUnlinkedGLFunction("glNormalP3uiv", void.class, GLParamTypeSets.ML_V(), PS_LNormalPointerType_p(), MLS_LNormalPointerType_p());
		}
		private static UnlinkedGLFunction make_glNormalPointer() {
			return newUnlinkedGLFunction("glNormalPointer", void.class, GLParamTypeSets.ML_V(), PS_LNormalPointerType_Ip(), MLS_LNormalPointerType_Ip());
		}
		private static UnlinkedGLFunction make_glNormalPointerEXT() {
			return newUnlinkedGLFunction("glNormalPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_LNormalPointerType_IIp(), MLS_LNormalPointerType_IIp());
		}
		private static UnlinkedGLFunction make_glNormalPointerListIBM() {
			return newUnlinkedGLFunction("glNormalPointerListIBM", void.class, GLParamTypeSets.ML_V(), PS_LNormalPointerType_IPI(), MLS_LNormalPointerType_IPI());
		}
		private static UnlinkedGLFunction make_glNormalPointervINTEL() {
			return newUnlinkedGLFunction("glNormalPointervINTEL", void.class, GLParamTypeSets.ML_V(), PS_LNormalPointerType_P(), MLS_LNormalPointerType_P());
		}
		private static UnlinkedGLFunction make_glNormalStream3bATI() {
			return newUnlinkedGLFunction("glNormalStream3bATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_BBB(), MLS_LVertexStreamATI_BBB());
		}
		private static UnlinkedGLFunction make_glNormalStream3bvATI() {
			return newUnlinkedGLFunction("glNormalStream3bvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glNormalStream3dATI() {
			return newUnlinkedGLFunction("glNormalStream3dATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_DDD(), MLS_LVertexStreamATI_DDD());
		}
		private static UnlinkedGLFunction make_glNormalStream3dvATI() {
			return newUnlinkedGLFunction("glNormalStream3dvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glNormalStream3fATI() {
			return newUnlinkedGLFunction("glNormalStream3fATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_FFF(), MLS_LVertexStreamATI_FFF());
		}
		private static UnlinkedGLFunction make_glNormalStream3fvATI() {
			return newUnlinkedGLFunction("glNormalStream3fvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glNormalStream3iATI() {
			return newUnlinkedGLFunction("glNormalStream3iATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_III(), MLS_LVertexStreamATI_III());
		}
		private static UnlinkedGLFunction make_glNormalStream3ivATI() {
			return newUnlinkedGLFunction("glNormalStream3ivATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glNormalStream3sATI() {
			return newUnlinkedGLFunction("glNormalStream3sATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_SSS(), MLS_LVertexStreamATI_SSS());
		}
		private static UnlinkedGLFunction make_glNormalStream3svATI() {
			return newUnlinkedGLFunction("glNormalStream3svATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glObjectLabel() {
			return newUnlinkedGLFunction("glObjectLabel", void.class, GLParamTypeSets.ML_V(), PS_LObjectIdentifier_IIp(), MLS_LObjectIdentifier_IIp());
		}
		private static UnlinkedGLFunction make_glObjectLabelKHR() {
			return newUnlinkedGLFunction("glObjectLabelKHR", void.class, GLParamTypeSets.ML_V(), PS_LObjectIdentifier_IIp(), MLS_LObjectIdentifier_IIp());
		}
		private static UnlinkedGLFunction make_glObjectPtrLabel() {
			return newUnlinkedGLFunction("glObjectPtrLabel", void.class, GLParamTypeSets.ML_V(), PS_pIp(), MLS_pIp());
		}
		private static UnlinkedGLFunction make_glObjectPtrLabelKHR() {
			return newUnlinkedGLFunction("glObjectPtrLabelKHR", void.class, GLParamTypeSets.ML_V(), PS_pIp(), MLS_pIp());
		}
		private static UnlinkedGLFunction make_glObjectPurgeableAPPLE() {
			return newUnlinkedGLFunction("glObjectPurgeableAPPLE", GLEnum.class, GLParamTypeSets.ML_E(), PS_EIE(), MLS_EIE());
		}
		private static UnlinkedGLFunction make_glObjectUnpurgeableAPPLE() {
			return newUnlinkedGLFunction("glObjectUnpurgeableAPPLE", GLEnum.class, GLParamTypeSets.ML_E(), PS_EIE(), MLS_EIE());
		}
		private static UnlinkedGLFunction make_glOrtho() {
			return newUnlinkedGLFunction("glOrtho", void.class, GLParamTypeSets.ML_V(), PS_DDDDDD(), MLS_DDDDDD());
		}
		private static UnlinkedGLFunction make_glOrthof() {
			return newUnlinkedGLFunction("glOrthof", void.class, GLParamTypeSets.ML_V(), PS_FFFFFF(), MLS_FFFFFF());
		}
		private static UnlinkedGLFunction make_glOrthofOES() {
			return newUnlinkedGLFunction("glOrthofOES", void.class, GLParamTypeSets.ML_V(), PS_FFFFFF(), MLS_FFFFFF());
		}
		private static UnlinkedGLFunction make_glOrthox() {
			return newUnlinkedGLFunction("glOrthox", void.class, GLParamTypeSets.ML_V(), PS_IIIIII(), MLS_IIIIII());
		}
		private static UnlinkedGLFunction make_glOrthoxOES() {
			return newUnlinkedGLFunction("glOrthoxOES", void.class, GLParamTypeSets.ML_V(), PS_IIIIII(), MLS_IIIIII());
		}
		private static UnlinkedGLFunction make_glPNTrianglesfATI() {
			return newUnlinkedGLFunction("glPNTrianglesfATI", void.class, GLParamTypeSets.ML_V(), PS_LPNTrianglesPNameATI_F(), MLS_LPNTrianglesPNameATI_F());
		}
		private static UnlinkedGLFunction make_glPNTrianglesiATI() {
			return newUnlinkedGLFunction("glPNTrianglesiATI", void.class, GLParamTypeSets.ML_V(), PS_LPNTrianglesPNameATI_I(), MLS_LPNTrianglesPNameATI_I());
		}
		private static UnlinkedGLFunction make_glPassTexCoordATI() {
			return newUnlinkedGLFunction("glPassTexCoordATI", void.class, GLParamTypeSets.ML_V(), PS_IILSwizzleOpATI_(), MLS_IILSwizzleOpATI_());
		}
		private static UnlinkedGLFunction make_glPassThrough() {
			return newUnlinkedGLFunction("glPassThrough", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glPassThroughxOES() {
			return newUnlinkedGLFunction("glPassThroughxOES", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glPatchParameterfv() {
			return newUnlinkedGLFunction("glPatchParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LPatchParameterName_p(), MLS_LPatchParameterName_p());
		}
		private static UnlinkedGLFunction make_glPatchParameteri() {
			return newUnlinkedGLFunction("glPatchParameteri", void.class, GLParamTypeSets.ML_V(), PS_LPatchParameterName_I(), MLS_LPatchParameterName_I());
		}
		private static UnlinkedGLFunction make_glPatchParameteriEXT() {
			return newUnlinkedGLFunction("glPatchParameteriEXT", void.class, GLParamTypeSets.ML_V(), PS_LPatchParameterName_I(), MLS_LPatchParameterName_I());
		}
		private static UnlinkedGLFunction make_glPatchParameteriOES() {
			return newUnlinkedGLFunction("glPatchParameteriOES", void.class, GLParamTypeSets.ML_V(), PS_LPatchParameterName_I(), MLS_LPatchParameterName_I());
		}
		private static UnlinkedGLFunction make_glPathColorGenNV() {
			return newUnlinkedGLFunction("glPathColorGenNV", void.class, GLParamTypeSets.ML_V(), PS_LPathColor_LPathGenMode_LPathColorFormat_p(), MLS_LPathColor_LPathGenMode_LPathColorFormat_p());
		}
		private static UnlinkedGLFunction make_glPathCommandsNV() {
			return newUnlinkedGLFunction("glPathCommandsNV", void.class, GLParamTypeSets.ML_V(), PS_IIpILPathCoordType_p(), MLS_IIpILPathCoordType_p());
		}
		private static UnlinkedGLFunction make_glPathCoordsNV() {
			return newUnlinkedGLFunction("glPathCoordsNV", void.class, GLParamTypeSets.ML_V(), PS_IILPathCoordType_p(), MLS_IILPathCoordType_p());
		}
		private static UnlinkedGLFunction make_glPathCoverDepthFuncNV() {
			return newUnlinkedGLFunction("glPathCoverDepthFuncNV", void.class, GLParamTypeSets.ML_V(), PS_LDepthFunction_(), MLS_LDepthFunction_());
		}
		private static UnlinkedGLFunction make_glPathDashArrayNV() {
			return newUnlinkedGLFunction("glPathDashArrayNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glPathFogGenNV() {
			return newUnlinkedGLFunction("glPathFogGenNV", void.class, GLParamTypeSets.ML_V(), PS_LPathGenMode_(), MLS_LPathGenMode_());
		}
		private static UnlinkedGLFunction make_glPathGlyphIndexArrayNV() {
			return newUnlinkedGLFunction("glPathGlyphIndexArrayNV", GLEnum.class, GLParamTypeSets.ML_E(), PS_IEpIIIIF(), MLS_IEpIIIIF());
		}
		private static UnlinkedGLFunction make_glPathGlyphIndexRangeNV() {
			return newUnlinkedGLFunction("glPathGlyphIndexRangeNV", GLEnum.class, GLParamTypeSets.ML_E(), PS_EpIIFP(), MLS_EpIIFP());
		}
		private static UnlinkedGLFunction make_glPathGlyphRangeNV() {
			return newUnlinkedGLFunction("glPathGlyphRangeNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathFontTarget_pIIILPathHandleMissingGlyphs_IF(), MLS_ILPathFontTarget_pIIILPathHandleMissingGlyphs_IF());
		}
		private static UnlinkedGLFunction make_glPathGlyphsNV() {
			return newUnlinkedGLFunction("glPathGlyphsNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathFontTarget_pIILPathElementType_pLPathHandleMissingGlyphs_IF(), MLS_ILPathFontTarget_pIILPathElementType_pLPathHandleMissingGlyphs_IF());
		}
		private static UnlinkedGLFunction make_glPathMemoryGlyphIndexArrayNV() {
			return newUnlinkedGLFunction("glPathMemoryGlyphIndexArrayNV", GLEnum.class, GLParamTypeSets.ML_E(), PS_IEJpIIIIF(), MLS_IEJpIIIIF());
		}
		private static UnlinkedGLFunction make_glPathParameterfNV() {
			return newUnlinkedGLFunction("glPathParameterfNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathParameter_F(), MLS_ILPathParameter_F());
		}
		private static UnlinkedGLFunction make_glPathParameterfvNV() {
			return newUnlinkedGLFunction("glPathParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathParameter_p(), MLS_ILPathParameter_p());
		}
		private static UnlinkedGLFunction make_glPathParameteriNV() {
			return newUnlinkedGLFunction("glPathParameteriNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathParameter_I(), MLS_ILPathParameter_I());
		}
		private static UnlinkedGLFunction make_glPathParameterivNV() {
			return newUnlinkedGLFunction("glPathParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathParameter_p(), MLS_ILPathParameter_p());
		}
		private static UnlinkedGLFunction make_glPathStencilDepthOffsetNV() {
			return newUnlinkedGLFunction("glPathStencilDepthOffsetNV", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glPathStencilFuncNV() {
			return newUnlinkedGLFunction("glPathStencilFuncNV", void.class, GLParamTypeSets.ML_V(), PS_LStencilFunction_II(), MLS_LStencilFunction_II());
		}
		private static UnlinkedGLFunction make_glPathStringNV() {
			return newUnlinkedGLFunction("glPathStringNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathStringFormat_Ip(), MLS_ILPathStringFormat_Ip());
		}
		private static UnlinkedGLFunction make_glPathSubCommandsNV() {
			return newUnlinkedGLFunction("glPathSubCommandsNV", void.class, GLParamTypeSets.ML_V(), PS_IIIIpILPathCoordType_p(), MLS_IIIIpILPathCoordType_p());
		}
		private static UnlinkedGLFunction make_glPathSubCoordsNV() {
			return newUnlinkedGLFunction("glPathSubCoordsNV", void.class, GLParamTypeSets.ML_V(), PS_IIILPathCoordType_p(), MLS_IIILPathCoordType_p());
		}
		private static UnlinkedGLFunction make_glPathTexGenNV() {
			return newUnlinkedGLFunction("glPathTexGenNV", void.class, GLParamTypeSets.ML_V(), PS_LPathColor_LPathGenMode_Ip(), MLS_LPathColor_LPathGenMode_Ip());
		}
		private static UnlinkedGLFunction make_glPauseTransformFeedback() {
			return newUnlinkedGLFunction("glPauseTransformFeedback", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPauseTransformFeedbackNV() {
			return newUnlinkedGLFunction("glPauseTransformFeedbackNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPixelDataRangeNV() {
			return newUnlinkedGLFunction("glPixelDataRangeNV", void.class, GLParamTypeSets.ML_V(), PS_LPixelDataRangeTargetNV_Ip(), MLS_LPixelDataRangeTargetNV_Ip());
		}
		private static UnlinkedGLFunction make_glPixelMapfv() {
			return newUnlinkedGLFunction("glPixelMapfv", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_Ip(), MLS_LPixelMap_Ip());
		}
		private static UnlinkedGLFunction make_glPixelMapuiv() {
			return newUnlinkedGLFunction("glPixelMapuiv", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_Ip(), MLS_LPixelMap_Ip());
		}
		private static UnlinkedGLFunction make_glPixelMapusv() {
			return newUnlinkedGLFunction("glPixelMapusv", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_Ip(), MLS_LPixelMap_Ip());
		}
		private static UnlinkedGLFunction make_glPixelMapx() {
			return newUnlinkedGLFunction("glPixelMapx", void.class, GLParamTypeSets.ML_V(), PS_LPixelMap_Ip(), MLS_LPixelMap_Ip());
		}
		private static UnlinkedGLFunction make_glPixelStoref() {
			return newUnlinkedGLFunction("glPixelStoref", void.class, GLParamTypeSets.ML_V(), PS_LPixelStoreParameter_F(), MLS_LPixelStoreParameter_F());
		}
		private static UnlinkedGLFunction make_glPixelStorei() {
			return newUnlinkedGLFunction("glPixelStorei", void.class, GLParamTypeSets.ML_V(), PS_LPixelStoreParameter_I(), MLS_LPixelStoreParameter_I());
		}
		private static UnlinkedGLFunction make_glPixelStorex() {
			return newUnlinkedGLFunction("glPixelStorex", void.class, GLParamTypeSets.ML_V(), PS_LPixelStoreParameter_I(), MLS_LPixelStoreParameter_I());
		}
		private static UnlinkedGLFunction make_glPixelTexGenParameterfSGIS() {
			return newUnlinkedGLFunction("glPixelTexGenParameterfSGIS", void.class, GLParamTypeSets.ML_V(), PS_LPixelTexGenParameterNameSGIS_F(), MLS_LPixelTexGenParameterNameSGIS_F());
		}
		private static UnlinkedGLFunction make_glPixelTexGenParameterfvSGIS() {
			return newUnlinkedGLFunction("glPixelTexGenParameterfvSGIS", void.class, GLParamTypeSets.ML_V(), PS_LPixelTexGenParameterNameSGIS_p(), MLS_LPixelTexGenParameterNameSGIS_p());
		}
		private static UnlinkedGLFunction make_glPixelTexGenParameteriSGIS() {
			return newUnlinkedGLFunction("glPixelTexGenParameteriSGIS", void.class, GLParamTypeSets.ML_V(), PS_LPixelTexGenParameterNameSGIS_I(), MLS_LPixelTexGenParameterNameSGIS_I());
		}
		private static UnlinkedGLFunction make_glPixelTexGenParameterivSGIS() {
			return newUnlinkedGLFunction("glPixelTexGenParameterivSGIS", void.class, GLParamTypeSets.ML_V(), PS_LPixelTexGenParameterNameSGIS_p(), MLS_LPixelTexGenParameterNameSGIS_p());
		}
		private static UnlinkedGLFunction make_glPixelTexGenSGIX() {
			return newUnlinkedGLFunction("glPixelTexGenSGIX", void.class, GLParamTypeSets.ML_V(), PS_LPixelTexGenModeSGIX_(), MLS_LPixelTexGenModeSGIX_());
		}
		private static UnlinkedGLFunction make_glPixelTransferf() {
			return newUnlinkedGLFunction("glPixelTransferf", void.class, GLParamTypeSets.ML_V(), PS_LPixelTransferParameter_F(), MLS_LPixelTransferParameter_F());
		}
		private static UnlinkedGLFunction make_glPixelTransferi() {
			return newUnlinkedGLFunction("glPixelTransferi", void.class, GLParamTypeSets.ML_V(), PS_LPixelTransferParameter_I(), MLS_LPixelTransferParameter_I());
		}
		private static UnlinkedGLFunction make_glPixelTransferxOES() {
			return newUnlinkedGLFunction("glPixelTransferxOES", void.class, GLParamTypeSets.ML_V(), PS_LPixelTransferParameter_I(), MLS_LPixelTransferParameter_I());
		}
		private static UnlinkedGLFunction make_glPixelTransformParameterfEXT() {
			return newUnlinkedGLFunction("glPixelTransformParameterfEXT", void.class, GLParamTypeSets.ML_V(), PS_LPixelTransformTargetEXT_LPixelTransformPNameEXT_F(), MLS_LPixelTransformTargetEXT_LPixelTransformPNameEXT_F());
		}
		private static UnlinkedGLFunction make_glPixelTransformParameterfvEXT() {
			return newUnlinkedGLFunction("glPixelTransformParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LPixelTransformTargetEXT_LPixelTransformPNameEXT_p(), MLS_LPixelTransformTargetEXT_LPixelTransformPNameEXT_p());
		}
		private static UnlinkedGLFunction make_glPixelTransformParameteriEXT() {
			return newUnlinkedGLFunction("glPixelTransformParameteriEXT", void.class, GLParamTypeSets.ML_V(), PS_LPixelTransformTargetEXT_LPixelTransformPNameEXT_I(), MLS_LPixelTransformTargetEXT_LPixelTransformPNameEXT_I());
		}
		private static UnlinkedGLFunction make_glPixelTransformParameterivEXT() {
			return newUnlinkedGLFunction("glPixelTransformParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_LPixelTransformTargetEXT_LPixelTransformPNameEXT_p(), MLS_LPixelTransformTargetEXT_LPixelTransformPNameEXT_p());
		}
		private static UnlinkedGLFunction make_glPixelZoom() {
			return newUnlinkedGLFunction("glPixelZoom", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glPixelZoomxOES() {
			return newUnlinkedGLFunction("glPixelZoomxOES", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glPointAlongPathNV() {
			return newUnlinkedGLFunction("glPointAlongPathNV", boolean.class, GLParamTypeSets.ML_Z(), PS_IIIFPPPP(), MLS_IIIFPPPP());
		}
		private static UnlinkedGLFunction make_glPointParameterf() {
			return newUnlinkedGLFunction("glPointParameterf", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_F(), MLS_LPointParameterNameARB_F());
		}
		private static UnlinkedGLFunction make_glPointParameterfARB() {
			return newUnlinkedGLFunction("glPointParameterfARB", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_F(), MLS_LPointParameterNameARB_F());
		}
		private static UnlinkedGLFunction make_glPointParameterfEXT() {
			return newUnlinkedGLFunction("glPointParameterfEXT", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_F(), MLS_LPointParameterNameARB_F());
		}
		private static UnlinkedGLFunction make_glPointParameterfSGIS() {
			return newUnlinkedGLFunction("glPointParameterfSGIS", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_F(), MLS_LPointParameterNameARB_F());
		}
		private static UnlinkedGLFunction make_glPointParameterfv() {
			return newUnlinkedGLFunction("glPointParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_p(), MLS_LPointParameterNameARB_p());
		}
		private static UnlinkedGLFunction make_glPointParameterfvARB() {
			return newUnlinkedGLFunction("glPointParameterfvARB", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_p(), MLS_LPointParameterNameARB_p());
		}
		private static UnlinkedGLFunction make_glPointParameterfvEXT() {
			return newUnlinkedGLFunction("glPointParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_p(), MLS_LPointParameterNameARB_p());
		}
		private static UnlinkedGLFunction make_glPointParameterfvSGIS() {
			return newUnlinkedGLFunction("glPointParameterfvSGIS", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_p(), MLS_LPointParameterNameARB_p());
		}
		private static UnlinkedGLFunction make_glPointParameteri() {
			return newUnlinkedGLFunction("glPointParameteri", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_I(), MLS_LPointParameterNameARB_I());
		}
		private static UnlinkedGLFunction make_glPointParameteriNV() {
			return newUnlinkedGLFunction("glPointParameteriNV", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_I(), MLS_LPointParameterNameARB_I());
		}
		private static UnlinkedGLFunction make_glPointParameteriv() {
			return newUnlinkedGLFunction("glPointParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_p(), MLS_LPointParameterNameARB_p());
		}
		private static UnlinkedGLFunction make_glPointParameterivNV() {
			return newUnlinkedGLFunction("glPointParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_p(), MLS_LPointParameterNameARB_p());
		}
		private static UnlinkedGLFunction make_glPointParameterx() {
			return newUnlinkedGLFunction("glPointParameterx", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_I(), MLS_LPointParameterNameARB_I());
		}
		private static UnlinkedGLFunction make_glPointParameterxOES() {
			return newUnlinkedGLFunction("glPointParameterxOES", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_I(), MLS_LPointParameterNameARB_I());
		}
		private static UnlinkedGLFunction make_glPointParameterxv() {
			return newUnlinkedGLFunction("glPointParameterxv", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_p(), MLS_LPointParameterNameARB_p());
		}
		private static UnlinkedGLFunction make_glPointParameterxvOES() {
			return newUnlinkedGLFunction("glPointParameterxvOES", void.class, GLParamTypeSets.ML_V(), PS_LPointParameterNameARB_p(), MLS_LPointParameterNameARB_p());
		}
		private static UnlinkedGLFunction make_glPointSize() {
			return newUnlinkedGLFunction("glPointSize", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glPointSizePointerOES() {
			return newUnlinkedGLFunction("glPointSizePointerOES", void.class, GLParamTypeSets.ML_V(), PS_EIp(), MLS_EIp());
		}
		private static UnlinkedGLFunction make_glPointSizex() {
			return newUnlinkedGLFunction("glPointSizex", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glPointSizexOES() {
			return newUnlinkedGLFunction("glPointSizexOES", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glPollAsyncSGIX() {
			return newUnlinkedGLFunction("glPollAsyncSGIX", int.class, GLParamTypeSets.ML_I(), PS_P(), MLS_P());
		}
		private static UnlinkedGLFunction make_glPollInstrumentsSGIX() {
			return newUnlinkedGLFunction("glPollInstrumentsSGIX", int.class, GLParamTypeSets.ML_I(), PS_P(), MLS_P());
		}
		private static UnlinkedGLFunction make_glPolygonMode() {
			return newUnlinkedGLFunction("glPolygonMode", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LPolygonMode_(), MLS_LTriangleFace_LPolygonMode_());
		}
		private static UnlinkedGLFunction make_glPolygonModeNV() {
			return newUnlinkedGLFunction("glPolygonModeNV", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LPolygonMode_(), MLS_LTriangleFace_LPolygonMode_());
		}
		private static UnlinkedGLFunction make_glPolygonOffset() {
			return newUnlinkedGLFunction("glPolygonOffset", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glPolygonOffsetClamp() {
			return newUnlinkedGLFunction("glPolygonOffsetClamp", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glPolygonOffsetClampEXT() {
			return newUnlinkedGLFunction("glPolygonOffsetClampEXT", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glPolygonOffsetEXT() {
			return newUnlinkedGLFunction("glPolygonOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glPolygonOffsetx() {
			return newUnlinkedGLFunction("glPolygonOffsetx", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glPolygonOffsetxOES() {
			return newUnlinkedGLFunction("glPolygonOffsetxOES", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glPolygonStipple() {
			return newUnlinkedGLFunction("glPolygonStipple", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glPopAttrib() {
			return newUnlinkedGLFunction("glPopAttrib", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPopClientAttrib() {
			return newUnlinkedGLFunction("glPopClientAttrib", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPopDebugGroup() {
			return newUnlinkedGLFunction("glPopDebugGroup", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPopDebugGroupKHR() {
			return newUnlinkedGLFunction("glPopDebugGroupKHR", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPopGroupMarkerEXT() {
			return newUnlinkedGLFunction("glPopGroupMarkerEXT", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPopMatrix() {
			return newUnlinkedGLFunction("glPopMatrix", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPopName() {
			return newUnlinkedGLFunction("glPopName", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPresentFrameDualFillNV() {
			return newUnlinkedGLFunction("glPresentFrameDualFillNV", void.class, GLParamTypeSets.ML_V(), PS_IJIIEEIEIEIEI(), MLS_IJIIEEIEIEIEI());
		}
		private static UnlinkedGLFunction make_glPresentFrameKeyedNV() {
			return newUnlinkedGLFunction("glPresentFrameKeyedNV", void.class, GLParamTypeSets.ML_V(), PS_IJIIEEIIEII(), MLS_IJIIEEIIEII());
		}
		private static UnlinkedGLFunction make_glPrimitiveBoundingBox() {
			return newUnlinkedGLFunction("glPrimitiveBoundingBox", void.class, GLParamTypeSets.ML_V(), PS_FFFFFFFF(), MLS_FFFFFFFF());
		}
		private static UnlinkedGLFunction make_glPrimitiveBoundingBoxARB() {
			return newUnlinkedGLFunction("glPrimitiveBoundingBoxARB", void.class, GLParamTypeSets.ML_V(), PS_FFFFFFFF(), MLS_FFFFFFFF());
		}
		private static UnlinkedGLFunction make_glPrimitiveBoundingBoxEXT() {
			return newUnlinkedGLFunction("glPrimitiveBoundingBoxEXT", void.class, GLParamTypeSets.ML_V(), PS_FFFFFFFF(), MLS_FFFFFFFF());
		}
		private static UnlinkedGLFunction make_glPrimitiveBoundingBoxOES() {
			return newUnlinkedGLFunction("glPrimitiveBoundingBoxOES", void.class, GLParamTypeSets.ML_V(), PS_FFFFFFFF(), MLS_FFFFFFFF());
		}
		private static UnlinkedGLFunction make_glPrimitiveRestartIndex() {
			return newUnlinkedGLFunction("glPrimitiveRestartIndex", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glPrimitiveRestartIndexNV() {
			return newUnlinkedGLFunction("glPrimitiveRestartIndexNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glPrimitiveRestartNV() {
			return newUnlinkedGLFunction("glPrimitiveRestartNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPrioritizeTextures() {
			return newUnlinkedGLFunction("glPrioritizeTextures", void.class, GLParamTypeSets.ML_V(), PS_Ipp(), MLS_Ipp());
		}
		private static UnlinkedGLFunction make_glPrioritizeTexturesEXT() {
			return newUnlinkedGLFunction("glPrioritizeTexturesEXT", void.class, GLParamTypeSets.ML_V(), PS_Ipp(), MLS_Ipp());
		}
		private static UnlinkedGLFunction make_glPrioritizeTexturesxOES() {
			return newUnlinkedGLFunction("glPrioritizeTexturesxOES", void.class, GLParamTypeSets.ML_V(), PS_Ipp(), MLS_Ipp());
		}
		private static UnlinkedGLFunction make_glProgramBinary() {
			return newUnlinkedGLFunction("glProgramBinary", void.class, GLParamTypeSets.ML_V(), PS_IEpI(), MLS_IEpI());
		}
		private static UnlinkedGLFunction make_glProgramBinaryOES() {
			return newUnlinkedGLFunction("glProgramBinaryOES", void.class, GLParamTypeSets.ML_V(), PS_IEpI(), MLS_IEpI());
		}
		private static UnlinkedGLFunction make_glProgramBufferParametersIivNV() {
			return newUnlinkedGLFunction("glProgramBufferParametersIivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIIp(), MLS_LProgramTarget_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramBufferParametersIuivNV() {
			return newUnlinkedGLFunction("glProgramBufferParametersIuivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIIp(), MLS_LProgramTarget_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramBufferParametersfvNV() {
			return newUnlinkedGLFunction("glProgramBufferParametersfvNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIIp(), MLS_LProgramTarget_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramEnvParameter4dARB() {
			return newUnlinkedGLFunction("glProgramEnvParameter4dARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IDDDD(), MLS_LProgramTarget_IDDDD());
		}
		private static UnlinkedGLFunction make_glProgramEnvParameter4dvARB() {
			return newUnlinkedGLFunction("glProgramEnvParameter4dvARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_Ip(), MLS_LProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glProgramEnvParameter4fARB() {
			return newUnlinkedGLFunction("glProgramEnvParameter4fARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IFFFF(), MLS_LProgramTarget_IFFFF());
		}
		private static UnlinkedGLFunction make_glProgramEnvParameter4fvARB() {
			return newUnlinkedGLFunction("glProgramEnvParameter4fvARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_Ip(), MLS_LProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glProgramEnvParameterI4iNV() {
			return newUnlinkedGLFunction("glProgramEnvParameterI4iNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIIII(), MLS_LProgramTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glProgramEnvParameterI4ivNV() {
			return newUnlinkedGLFunction("glProgramEnvParameterI4ivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_Ip(), MLS_LProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glProgramEnvParameterI4uiNV() {
			return newUnlinkedGLFunction("glProgramEnvParameterI4uiNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIIII(), MLS_LProgramTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glProgramEnvParameterI4uivNV() {
			return newUnlinkedGLFunction("glProgramEnvParameterI4uivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_Ip(), MLS_LProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glProgramEnvParameters4fvEXT() {
			return newUnlinkedGLFunction("glProgramEnvParameters4fvEXT", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIp(), MLS_LProgramTarget_IIp());
		}
		private static UnlinkedGLFunction make_glProgramEnvParametersI4ivNV() {
			return newUnlinkedGLFunction("glProgramEnvParametersI4ivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIp(), MLS_LProgramTarget_IIp());
		}
		private static UnlinkedGLFunction make_glProgramEnvParametersI4uivNV() {
			return newUnlinkedGLFunction("glProgramEnvParametersI4uivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIp(), MLS_LProgramTarget_IIp());
		}
		private static UnlinkedGLFunction make_glProgramLocalParameter4dARB() {
			return newUnlinkedGLFunction("glProgramLocalParameter4dARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IDDDD(), MLS_LProgramTarget_IDDDD());
		}
		private static UnlinkedGLFunction make_glProgramLocalParameter4dvARB() {
			return newUnlinkedGLFunction("glProgramLocalParameter4dvARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_Ip(), MLS_LProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glProgramLocalParameter4fARB() {
			return newUnlinkedGLFunction("glProgramLocalParameter4fARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IFFFF(), MLS_LProgramTarget_IFFFF());
		}
		private static UnlinkedGLFunction make_glProgramLocalParameter4fvARB() {
			return newUnlinkedGLFunction("glProgramLocalParameter4fvARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_Ip(), MLS_LProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glProgramLocalParameterI4iNV() {
			return newUnlinkedGLFunction("glProgramLocalParameterI4iNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIIII(), MLS_LProgramTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glProgramLocalParameterI4ivNV() {
			return newUnlinkedGLFunction("glProgramLocalParameterI4ivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_Ip(), MLS_LProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glProgramLocalParameterI4uiNV() {
			return newUnlinkedGLFunction("glProgramLocalParameterI4uiNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIIII(), MLS_LProgramTarget_IIIII());
		}
		private static UnlinkedGLFunction make_glProgramLocalParameterI4uivNV() {
			return newUnlinkedGLFunction("glProgramLocalParameterI4uivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_Ip(), MLS_LProgramTarget_Ip());
		}
		private static UnlinkedGLFunction make_glProgramLocalParameters4fvEXT() {
			return newUnlinkedGLFunction("glProgramLocalParameters4fvEXT", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIp(), MLS_LProgramTarget_IIp());
		}
		private static UnlinkedGLFunction make_glProgramLocalParametersI4ivNV() {
			return newUnlinkedGLFunction("glProgramLocalParametersI4ivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIp(), MLS_LProgramTarget_IIp());
		}
		private static UnlinkedGLFunction make_glProgramLocalParametersI4uivNV() {
			return newUnlinkedGLFunction("glProgramLocalParametersI4uivNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_IIp(), MLS_LProgramTarget_IIp());
		}
		private static UnlinkedGLFunction make_glProgramNamedParameter4dNV() {
			return newUnlinkedGLFunction("glProgramNamedParameter4dNV", void.class, GLParamTypeSets.ML_V(), PS_IIpDDDD(), MLS_IIpDDDD());
		}
		private static UnlinkedGLFunction make_glProgramNamedParameter4dvNV() {
			return newUnlinkedGLFunction("glProgramNamedParameter4dvNV", void.class, GLParamTypeSets.ML_V(), PS_IIpp(), MLS_IIpp());
		}
		private static UnlinkedGLFunction make_glProgramNamedParameter4fNV() {
			return newUnlinkedGLFunction("glProgramNamedParameter4fNV", void.class, GLParamTypeSets.ML_V(), PS_IIpFFFF(), MLS_IIpFFFF());
		}
		private static UnlinkedGLFunction make_glProgramNamedParameter4fvNV() {
			return newUnlinkedGLFunction("glProgramNamedParameter4fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIpp(), MLS_IIpp());
		}
		private static UnlinkedGLFunction make_glProgramParameter4dNV() {
			return newUnlinkedGLFunction("glProgramParameter4dNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_IDDDD(), MLS_LVertexAttribEnumNV_IDDDD());
		}
		private static UnlinkedGLFunction make_glProgramParameter4dvNV() {
			return newUnlinkedGLFunction("glProgramParameter4dvNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_Ip(), MLS_LVertexAttribEnumNV_Ip());
		}
		private static UnlinkedGLFunction make_glProgramParameter4fNV() {
			return newUnlinkedGLFunction("glProgramParameter4fNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_IFFFF(), MLS_LVertexAttribEnumNV_IFFFF());
		}
		private static UnlinkedGLFunction make_glProgramParameter4fvNV() {
			return newUnlinkedGLFunction("glProgramParameter4fvNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_Ip(), MLS_LVertexAttribEnumNV_Ip());
		}
		private static UnlinkedGLFunction make_glProgramParameteri() {
			return newUnlinkedGLFunction("glProgramParameteri", void.class, GLParamTypeSets.ML_V(), PS_ILProgramParameterPName_I(), MLS_ILProgramParameterPName_I());
		}
		private static UnlinkedGLFunction make_glProgramParameteriARB() {
			return newUnlinkedGLFunction("glProgramParameteriARB", void.class, GLParamTypeSets.ML_V(), PS_ILProgramParameterPName_I(), MLS_ILProgramParameterPName_I());
		}
		private static UnlinkedGLFunction make_glProgramParameteriEXT() {
			return newUnlinkedGLFunction("glProgramParameteriEXT", void.class, GLParamTypeSets.ML_V(), PS_ILProgramParameterPName_I(), MLS_ILProgramParameterPName_I());
		}
		private static UnlinkedGLFunction make_glProgramParameters4dvNV() {
			return newUnlinkedGLFunction("glProgramParameters4dvNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_IIp(), MLS_LVertexAttribEnumNV_IIp());
		}
		private static UnlinkedGLFunction make_glProgramParameters4fvNV() {
			return newUnlinkedGLFunction("glProgramParameters4fvNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_IIp(), MLS_LVertexAttribEnumNV_IIp());
		}
		private static UnlinkedGLFunction make_glProgramPathFragmentInputGenNV() {
			return newUnlinkedGLFunction("glProgramPathFragmentInputGenNV", void.class, GLParamTypeSets.ML_V(), PS_IIEIp(), MLS_IIEIp());
		}
		private static UnlinkedGLFunction make_glProgramStringARB() {
			return newUnlinkedGLFunction("glProgramStringARB", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_LProgramFormat_Ip(), MLS_LProgramTarget_LProgramFormat_Ip());
		}
		private static UnlinkedGLFunction make_glProgramSubroutineParametersuivNV() {
			return newUnlinkedGLFunction("glProgramSubroutineParametersuivNV", void.class, GLParamTypeSets.ML_V(), PS_EIp(), MLS_EIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1d() {
			return newUnlinkedGLFunction("glProgramUniform1d", void.class, GLParamTypeSets.ML_V(), PS_IID(), MLS_IID());
		}
		private static UnlinkedGLFunction make_glProgramUniform1dEXT() {
			return newUnlinkedGLFunction("glProgramUniform1dEXT", void.class, GLParamTypeSets.ML_V(), PS_IID(), MLS_IID());
		}
		private static UnlinkedGLFunction make_glProgramUniform1dv() {
			return newUnlinkedGLFunction("glProgramUniform1dv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1dvEXT() {
			return newUnlinkedGLFunction("glProgramUniform1dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1f() {
			return newUnlinkedGLFunction("glProgramUniform1f", void.class, GLParamTypeSets.ML_V(), PS_IIF(), MLS_IIF());
		}
		private static UnlinkedGLFunction make_glProgramUniform1fEXT() {
			return newUnlinkedGLFunction("glProgramUniform1fEXT", void.class, GLParamTypeSets.ML_V(), PS_IIF(), MLS_IIF());
		}
		private static UnlinkedGLFunction make_glProgramUniform1fv() {
			return newUnlinkedGLFunction("glProgramUniform1fv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1fvEXT() {
			return newUnlinkedGLFunction("glProgramUniform1fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1i() {
			return newUnlinkedGLFunction("glProgramUniform1i", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glProgramUniform1i64ARB() {
			return newUnlinkedGLFunction("glProgramUniform1i64ARB", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform1i64NV() {
			return newUnlinkedGLFunction("glProgramUniform1i64NV", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform1i64vARB() {
			return newUnlinkedGLFunction("glProgramUniform1i64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1i64vNV() {
			return newUnlinkedGLFunction("glProgramUniform1i64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1iEXT() {
			return newUnlinkedGLFunction("glProgramUniform1iEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glProgramUniform1iv() {
			return newUnlinkedGLFunction("glProgramUniform1iv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1ivEXT() {
			return newUnlinkedGLFunction("glProgramUniform1ivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1ui() {
			return newUnlinkedGLFunction("glProgramUniform1ui", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glProgramUniform1ui64ARB() {
			return newUnlinkedGLFunction("glProgramUniform1ui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform1ui64NV() {
			return newUnlinkedGLFunction("glProgramUniform1ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform1ui64vARB() {
			return newUnlinkedGLFunction("glProgramUniform1ui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1ui64vNV() {
			return newUnlinkedGLFunction("glProgramUniform1ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1uiEXT() {
			return newUnlinkedGLFunction("glProgramUniform1uiEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glProgramUniform1uiv() {
			return newUnlinkedGLFunction("glProgramUniform1uiv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform1uivEXT() {
			return newUnlinkedGLFunction("glProgramUniform1uivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2d() {
			return newUnlinkedGLFunction("glProgramUniform2d", void.class, GLParamTypeSets.ML_V(), PS_IIDD(), MLS_IIDD());
		}
		private static UnlinkedGLFunction make_glProgramUniform2dEXT() {
			return newUnlinkedGLFunction("glProgramUniform2dEXT", void.class, GLParamTypeSets.ML_V(), PS_IIDD(), MLS_IIDD());
		}
		private static UnlinkedGLFunction make_glProgramUniform2dv() {
			return newUnlinkedGLFunction("glProgramUniform2dv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2dvEXT() {
			return newUnlinkedGLFunction("glProgramUniform2dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2f() {
			return newUnlinkedGLFunction("glProgramUniform2f", void.class, GLParamTypeSets.ML_V(), PS_IIFF(), MLS_IIFF());
		}
		private static UnlinkedGLFunction make_glProgramUniform2fEXT() {
			return newUnlinkedGLFunction("glProgramUniform2fEXT", void.class, GLParamTypeSets.ML_V(), PS_IIFF(), MLS_IIFF());
		}
		private static UnlinkedGLFunction make_glProgramUniform2fv() {
			return newUnlinkedGLFunction("glProgramUniform2fv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2fvEXT() {
			return newUnlinkedGLFunction("glProgramUniform2fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2i() {
			return newUnlinkedGLFunction("glProgramUniform2i", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform2i64ARB() {
			return newUnlinkedGLFunction("glProgramUniform2i64ARB", void.class, GLParamTypeSets.ML_V(), PS_IIJJ(), MLS_IIJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform2i64NV() {
			return newUnlinkedGLFunction("glProgramUniform2i64NV", void.class, GLParamTypeSets.ML_V(), PS_IIJJ(), MLS_IIJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform2i64vARB() {
			return newUnlinkedGLFunction("glProgramUniform2i64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2i64vNV() {
			return newUnlinkedGLFunction("glProgramUniform2i64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2iEXT() {
			return newUnlinkedGLFunction("glProgramUniform2iEXT", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform2iv() {
			return newUnlinkedGLFunction("glProgramUniform2iv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2ivEXT() {
			return newUnlinkedGLFunction("glProgramUniform2ivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2ui() {
			return newUnlinkedGLFunction("glProgramUniform2ui", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform2ui64ARB() {
			return newUnlinkedGLFunction("glProgramUniform2ui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IIJJ(), MLS_IIJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform2ui64NV() {
			return newUnlinkedGLFunction("glProgramUniform2ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IIJJ(), MLS_IIJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform2ui64vARB() {
			return newUnlinkedGLFunction("glProgramUniform2ui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2ui64vNV() {
			return newUnlinkedGLFunction("glProgramUniform2ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2uiEXT() {
			return newUnlinkedGLFunction("glProgramUniform2uiEXT", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform2uiv() {
			return newUnlinkedGLFunction("glProgramUniform2uiv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform2uivEXT() {
			return newUnlinkedGLFunction("glProgramUniform2uivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3d() {
			return newUnlinkedGLFunction("glProgramUniform3d", void.class, GLParamTypeSets.ML_V(), PS_IIDDD(), MLS_IIDDD());
		}
		private static UnlinkedGLFunction make_glProgramUniform3dEXT() {
			return newUnlinkedGLFunction("glProgramUniform3dEXT", void.class, GLParamTypeSets.ML_V(), PS_IIDDD(), MLS_IIDDD());
		}
		private static UnlinkedGLFunction make_glProgramUniform3dv() {
			return newUnlinkedGLFunction("glProgramUniform3dv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3dvEXT() {
			return newUnlinkedGLFunction("glProgramUniform3dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3f() {
			return newUnlinkedGLFunction("glProgramUniform3f", void.class, GLParamTypeSets.ML_V(), PS_IIFFF(), MLS_IIFFF());
		}
		private static UnlinkedGLFunction make_glProgramUniform3fEXT() {
			return newUnlinkedGLFunction("glProgramUniform3fEXT", void.class, GLParamTypeSets.ML_V(), PS_IIFFF(), MLS_IIFFF());
		}
		private static UnlinkedGLFunction make_glProgramUniform3fv() {
			return newUnlinkedGLFunction("glProgramUniform3fv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3fvEXT() {
			return newUnlinkedGLFunction("glProgramUniform3fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3i() {
			return newUnlinkedGLFunction("glProgramUniform3i", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform3i64ARB() {
			return newUnlinkedGLFunction("glProgramUniform3i64ARB", void.class, GLParamTypeSets.ML_V(), PS_IIJJJ(), MLS_IIJJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform3i64NV() {
			return newUnlinkedGLFunction("glProgramUniform3i64NV", void.class, GLParamTypeSets.ML_V(), PS_IIJJJ(), MLS_IIJJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform3i64vARB() {
			return newUnlinkedGLFunction("glProgramUniform3i64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3i64vNV() {
			return newUnlinkedGLFunction("glProgramUniform3i64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3iEXT() {
			return newUnlinkedGLFunction("glProgramUniform3iEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform3iv() {
			return newUnlinkedGLFunction("glProgramUniform3iv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3ivEXT() {
			return newUnlinkedGLFunction("glProgramUniform3ivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3ui() {
			return newUnlinkedGLFunction("glProgramUniform3ui", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform3ui64ARB() {
			return newUnlinkedGLFunction("glProgramUniform3ui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IIJJJ(), MLS_IIJJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform3ui64NV() {
			return newUnlinkedGLFunction("glProgramUniform3ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IIJJJ(), MLS_IIJJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform3ui64vARB() {
			return newUnlinkedGLFunction("glProgramUniform3ui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3ui64vNV() {
			return newUnlinkedGLFunction("glProgramUniform3ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3uiEXT() {
			return newUnlinkedGLFunction("glProgramUniform3uiEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform3uiv() {
			return newUnlinkedGLFunction("glProgramUniform3uiv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform3uivEXT() {
			return newUnlinkedGLFunction("glProgramUniform3uivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4d() {
			return newUnlinkedGLFunction("glProgramUniform4d", void.class, GLParamTypeSets.ML_V(), PS_IIDDDD(), MLS_IIDDDD());
		}
		private static UnlinkedGLFunction make_glProgramUniform4dEXT() {
			return newUnlinkedGLFunction("glProgramUniform4dEXT", void.class, GLParamTypeSets.ML_V(), PS_IIDDDD(), MLS_IIDDDD());
		}
		private static UnlinkedGLFunction make_glProgramUniform4dv() {
			return newUnlinkedGLFunction("glProgramUniform4dv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4dvEXT() {
			return newUnlinkedGLFunction("glProgramUniform4dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4f() {
			return newUnlinkedGLFunction("glProgramUniform4f", void.class, GLParamTypeSets.ML_V(), PS_IIFFFF(), MLS_IIFFFF());
		}
		private static UnlinkedGLFunction make_glProgramUniform4fEXT() {
			return newUnlinkedGLFunction("glProgramUniform4fEXT", void.class, GLParamTypeSets.ML_V(), PS_IIFFFF(), MLS_IIFFFF());
		}
		private static UnlinkedGLFunction make_glProgramUniform4fv() {
			return newUnlinkedGLFunction("glProgramUniform4fv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4fvEXT() {
			return newUnlinkedGLFunction("glProgramUniform4fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4i() {
			return newUnlinkedGLFunction("glProgramUniform4i", void.class, GLParamTypeSets.ML_V(), PS_IIIIII(), MLS_IIIIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform4i64ARB() {
			return newUnlinkedGLFunction("glProgramUniform4i64ARB", void.class, GLParamTypeSets.ML_V(), PS_IIJJJJ(), MLS_IIJJJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform4i64NV() {
			return newUnlinkedGLFunction("glProgramUniform4i64NV", void.class, GLParamTypeSets.ML_V(), PS_IIJJJJ(), MLS_IIJJJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform4i64vARB() {
			return newUnlinkedGLFunction("glProgramUniform4i64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4i64vNV() {
			return newUnlinkedGLFunction("glProgramUniform4i64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4iEXT() {
			return newUnlinkedGLFunction("glProgramUniform4iEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIIII(), MLS_IIIIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform4iv() {
			return newUnlinkedGLFunction("glProgramUniform4iv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4ivEXT() {
			return newUnlinkedGLFunction("glProgramUniform4ivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4ui() {
			return newUnlinkedGLFunction("glProgramUniform4ui", void.class, GLParamTypeSets.ML_V(), PS_IIIIII(), MLS_IIIIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform4ui64ARB() {
			return newUnlinkedGLFunction("glProgramUniform4ui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IIJJJJ(), MLS_IIJJJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform4ui64NV() {
			return newUnlinkedGLFunction("glProgramUniform4ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IIJJJJ(), MLS_IIJJJJ());
		}
		private static UnlinkedGLFunction make_glProgramUniform4ui64vARB() {
			return newUnlinkedGLFunction("glProgramUniform4ui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4ui64vNV() {
			return newUnlinkedGLFunction("glProgramUniform4ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4uiEXT() {
			return newUnlinkedGLFunction("glProgramUniform4uiEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIIII(), MLS_IIIIII());
		}
		private static UnlinkedGLFunction make_glProgramUniform4uiv() {
			return newUnlinkedGLFunction("glProgramUniform4uiv", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniform4uivEXT() {
			return newUnlinkedGLFunction("glProgramUniform4uivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniformHandleui64ARB() {
			return newUnlinkedGLFunction("glProgramUniformHandleui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glProgramUniformHandleui64IMG() {
			return newUnlinkedGLFunction("glProgramUniformHandleui64IMG", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glProgramUniformHandleui64NV() {
			return newUnlinkedGLFunction("glProgramUniformHandleui64NV", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glProgramUniformHandleui64vARB() {
			return newUnlinkedGLFunction("glProgramUniformHandleui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniformHandleui64vIMG() {
			return newUnlinkedGLFunction("glProgramUniformHandleui64vIMG", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniformHandleui64vNV() {
			return newUnlinkedGLFunction("glProgramUniformHandleui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2dv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2dv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2dvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2fv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2fv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2fvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2x3dv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2x3dv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2x3dvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2x3dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2x3fv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2x3fv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2x3fvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2x3fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2x4dv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2x4dv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2x4dvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2x4dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2x4fv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2x4fv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix2x4fvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix2x4fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3dv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3dv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3dvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3fv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3fv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3fvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3x2dv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3x2dv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3x2dvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3x2dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3x2fv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3x2fv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3x2fvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3x2fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3x4dv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3x4dv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3x4dvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3x4dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3x4fv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3x4fv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix3x4fvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix3x4fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4dv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4dv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4dvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4fv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4fv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4fvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4x2dv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4x2dv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4x2dvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4x2dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4x2fv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4x2fv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4x2fvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4x2fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4x3dv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4x3dv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4x3dvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4x3dvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4x3fv() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4x3fv", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformMatrix4x3fvEXT() {
			return newUnlinkedGLFunction("glProgramUniformMatrix4x3fvEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIZp(), MLS_IIIZp());
		}
		private static UnlinkedGLFunction make_glProgramUniformui64NV() {
			return newUnlinkedGLFunction("glProgramUniformui64NV", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glProgramUniformui64vNV() {
			return newUnlinkedGLFunction("glProgramUniformui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glProgramVertexLimitNV() {
			return newUnlinkedGLFunction("glProgramVertexLimitNV", void.class, GLParamTypeSets.ML_V(), PS_LProgramTarget_I(), MLS_LProgramTarget_I());
		}
		private static UnlinkedGLFunction make_glProvokingVertex() {
			return newUnlinkedGLFunction("glProvokingVertex", void.class, GLParamTypeSets.ML_V(), PS_LVertexProvokingMode_(), MLS_LVertexProvokingMode_());
		}
		private static UnlinkedGLFunction make_glProvokingVertexEXT() {
			return newUnlinkedGLFunction("glProvokingVertexEXT", void.class, GLParamTypeSets.ML_V(), PS_LVertexProvokingMode_(), MLS_LVertexProvokingMode_());
		}
		private static UnlinkedGLFunction make_glPushAttrib() {
			return newUnlinkedGLFunction("glPushAttrib", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glPushClientAttrib() {
			return newUnlinkedGLFunction("glPushClientAttrib", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glPushClientAttribDefaultEXT() {
			return newUnlinkedGLFunction("glPushClientAttribDefaultEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glPushDebugGroup() {
			return newUnlinkedGLFunction("glPushDebugGroup", void.class, GLParamTypeSets.ML_V(), PS_LDebugSource_IIp(), MLS_LDebugSource_IIp());
		}
		private static UnlinkedGLFunction make_glPushDebugGroupKHR() {
			return newUnlinkedGLFunction("glPushDebugGroupKHR", void.class, GLParamTypeSets.ML_V(), PS_LDebugSource_IIp(), MLS_LDebugSource_IIp());
		}
		private static UnlinkedGLFunction make_glPushGroupMarkerEXT() {
			return newUnlinkedGLFunction("glPushGroupMarkerEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glPushMatrix() {
			return newUnlinkedGLFunction("glPushMatrix", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glPushName() {
			return newUnlinkedGLFunction("glPushName", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glQueryCounter() {
			return newUnlinkedGLFunction("glQueryCounter", void.class, GLParamTypeSets.ML_V(), PS_ILQueryCounterTarget_(), MLS_ILQueryCounterTarget_());
		}
		private static UnlinkedGLFunction make_glQueryCounterEXT() {
			return newUnlinkedGLFunction("glQueryCounterEXT", void.class, GLParamTypeSets.ML_V(), PS_ILQueryCounterTarget_(), MLS_ILQueryCounterTarget_());
		}
		private static UnlinkedGLFunction make_glQueryMatrixxOES() {
			return newUnlinkedGLFunction("glQueryMatrixxOES", int.class, GLParamTypeSets.ML_I(), PS_PP(), MLS_PP());
		}
		private static UnlinkedGLFunction make_glQueryObjectParameteruiAMD() {
			return newUnlinkedGLFunction("glQueryObjectParameteruiAMD", void.class, GLParamTypeSets.ML_V(), PS_LQueryTarget_IEI(), MLS_LQueryTarget_IEI());
		}
		private static UnlinkedGLFunction make_glQueryResourceNV() {
			return newUnlinkedGLFunction("glQueryResourceNV", int.class, GLParamTypeSets.ML_I(), PS_EIIP(), MLS_EIIP());
		}
		private static UnlinkedGLFunction make_glQueryResourceTagNV() {
			return newUnlinkedGLFunction("glQueryResourceTagNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glRasterPos2d() {
			return newUnlinkedGLFunction("glRasterPos2d", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glRasterPos2dv() {
			return newUnlinkedGLFunction("glRasterPos2dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos2f() {
			return newUnlinkedGLFunction("glRasterPos2f", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glRasterPos2fv() {
			return newUnlinkedGLFunction("glRasterPos2fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos2i() {
			return newUnlinkedGLFunction("glRasterPos2i", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glRasterPos2iv() {
			return newUnlinkedGLFunction("glRasterPos2iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos2s() {
			return newUnlinkedGLFunction("glRasterPos2s", void.class, GLParamTypeSets.ML_V(), PS_SS(), MLS_SS());
		}
		private static UnlinkedGLFunction make_glRasterPos2sv() {
			return newUnlinkedGLFunction("glRasterPos2sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos2xOES() {
			return newUnlinkedGLFunction("glRasterPos2xOES", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glRasterPos2xvOES() {
			return newUnlinkedGLFunction("glRasterPos2xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos3d() {
			return newUnlinkedGLFunction("glRasterPos3d", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glRasterPos3dv() {
			return newUnlinkedGLFunction("glRasterPos3dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos3f() {
			return newUnlinkedGLFunction("glRasterPos3f", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glRasterPos3fv() {
			return newUnlinkedGLFunction("glRasterPos3fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos3i() {
			return newUnlinkedGLFunction("glRasterPos3i", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glRasterPos3iv() {
			return newUnlinkedGLFunction("glRasterPos3iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos3s() {
			return newUnlinkedGLFunction("glRasterPos3s", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glRasterPos3sv() {
			return newUnlinkedGLFunction("glRasterPos3sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos3xOES() {
			return newUnlinkedGLFunction("glRasterPos3xOES", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glRasterPos3xvOES() {
			return newUnlinkedGLFunction("glRasterPos3xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos4d() {
			return newUnlinkedGLFunction("glRasterPos4d", void.class, GLParamTypeSets.ML_V(), PS_DDDD(), MLS_DDDD());
		}
		private static UnlinkedGLFunction make_glRasterPos4dv() {
			return newUnlinkedGLFunction("glRasterPos4dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos4f() {
			return newUnlinkedGLFunction("glRasterPos4f", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glRasterPos4fv() {
			return newUnlinkedGLFunction("glRasterPos4fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos4i() {
			return newUnlinkedGLFunction("glRasterPos4i", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glRasterPos4iv() {
			return newUnlinkedGLFunction("glRasterPos4iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos4s() {
			return newUnlinkedGLFunction("glRasterPos4s", void.class, GLParamTypeSets.ML_V(), PS_SSSS(), MLS_SSSS());
		}
		private static UnlinkedGLFunction make_glRasterPos4sv() {
			return newUnlinkedGLFunction("glRasterPos4sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterPos4xOES() {
			return newUnlinkedGLFunction("glRasterPos4xOES", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glRasterPos4xvOES() {
			return newUnlinkedGLFunction("glRasterPos4xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRasterSamplesEXT() {
			return newUnlinkedGLFunction("glRasterSamplesEXT", void.class, GLParamTypeSets.ML_V(), PS_IZ(), MLS_IZ());
		}
		private static UnlinkedGLFunction make_glReadBuffer() {
			return newUnlinkedGLFunction("glReadBuffer", void.class, GLParamTypeSets.ML_V(), PS_LReadBufferMode_(), MLS_LReadBufferMode_());
		}
		private static UnlinkedGLFunction make_glReadBufferIndexedEXT() {
			return newUnlinkedGLFunction("glReadBufferIndexedEXT", void.class, GLParamTypeSets.ML_V(), PS_LReadBufferMode_I(), MLS_LReadBufferMode_I());
		}
		private static UnlinkedGLFunction make_glReadBufferNV() {
			return newUnlinkedGLFunction("glReadBufferNV", void.class, GLParamTypeSets.ML_V(), PS_E(), MLS_E());
		}
		private static UnlinkedGLFunction make_glReadInstrumentsSGIX() {
			return newUnlinkedGLFunction("glReadInstrumentsSGIX", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glReadPixels() {
			return newUnlinkedGLFunction("glReadPixels", void.class, GLParamTypeSets.ML_V(), PS_IIIILPixelFormat_LPixelType_P(), MLS_IIIILPixelFormat_LPixelType_P());
		}
		private static UnlinkedGLFunction make_glReadnPixels() {
			return newUnlinkedGLFunction("glReadnPixels", void.class, GLParamTypeSets.ML_V(), PS_IIIILPixelFormat_LPixelType_IP(), MLS_IIIILPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glReadnPixelsARB() {
			return newUnlinkedGLFunction("glReadnPixelsARB", void.class, GLParamTypeSets.ML_V(), PS_IIIILPixelFormat_LPixelType_IP(), MLS_IIIILPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glReadnPixelsEXT() {
			return newUnlinkedGLFunction("glReadnPixelsEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIILPixelFormat_LPixelType_IP(), MLS_IIIILPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glReadnPixelsKHR() {
			return newUnlinkedGLFunction("glReadnPixelsKHR", void.class, GLParamTypeSets.ML_V(), PS_IIIILPixelFormat_LPixelType_IP(), MLS_IIIILPixelFormat_LPixelType_IP());
		}
		private static UnlinkedGLFunction make_glReleaseKeyedMutexWin32EXT() {
			return newUnlinkedGLFunction("glReleaseKeyedMutexWin32EXT", boolean.class, GLParamTypeSets.ML_Z(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glRectd() {
			return newUnlinkedGLFunction("glRectd", void.class, GLParamTypeSets.ML_V(), PS_DDDD(), MLS_DDDD());
		}
		private static UnlinkedGLFunction make_glRectdv() {
			return newUnlinkedGLFunction("glRectdv", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glRectf() {
			return newUnlinkedGLFunction("glRectf", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glRectfv() {
			return newUnlinkedGLFunction("glRectfv", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glRecti() {
			return newUnlinkedGLFunction("glRecti", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glRectiv() {
			return newUnlinkedGLFunction("glRectiv", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glRects() {
			return newUnlinkedGLFunction("glRects", void.class, GLParamTypeSets.ML_V(), PS_SSSS(), MLS_SSSS());
		}
		private static UnlinkedGLFunction make_glRectsv() {
			return newUnlinkedGLFunction("glRectsv", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glRectxOES() {
			return newUnlinkedGLFunction("glRectxOES", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glRectxvOES() {
			return newUnlinkedGLFunction("glRectxvOES", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glReferencePlaneSGIX() {
			return newUnlinkedGLFunction("glReferencePlaneSGIX", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glReleaseShaderCompiler() {
			return newUnlinkedGLFunction("glReleaseShaderCompiler", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glRenderGpuMaskNV() {
			return newUnlinkedGLFunction("glRenderGpuMaskNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glRenderMode() {
			return newUnlinkedGLFunction("glRenderMode", int.class, GLParamTypeSets.ML_I(), PS_LRenderingMode_(), MLS_LRenderingMode_());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorage() {
			return newUnlinkedGLFunction("glRenderbufferStorage", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_LInternalFormat_II(), MLS_LRenderbufferTarget_LInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorageEXT() {
			return newUnlinkedGLFunction("glRenderbufferStorageEXT", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_LInternalFormat_II(), MLS_LRenderbufferTarget_LInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorageMultisample() {
			return newUnlinkedGLFunction("glRenderbufferStorageMultisample", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_ILInternalFormat_II(), MLS_LRenderbufferTarget_ILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorageMultisampleANGLE() {
			return newUnlinkedGLFunction("glRenderbufferStorageMultisampleANGLE", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_ILInternalFormat_II(), MLS_LRenderbufferTarget_ILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorageMultisampleAPPLE() {
			return newUnlinkedGLFunction("glRenderbufferStorageMultisampleAPPLE", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_ILInternalFormat_II(), MLS_LRenderbufferTarget_ILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorageMultisampleAdvancedAMD() {
			return newUnlinkedGLFunction("glRenderbufferStorageMultisampleAdvancedAMD", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_IILInternalFormat_II(), MLS_LRenderbufferTarget_IILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorageMultisampleCoverageNV() {
			return newUnlinkedGLFunction("glRenderbufferStorageMultisampleCoverageNV", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_IILInternalFormat_II(), MLS_LRenderbufferTarget_IILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorageMultisampleEXT() {
			return newUnlinkedGLFunction("glRenderbufferStorageMultisampleEXT", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_ILInternalFormat_II(), MLS_LRenderbufferTarget_ILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorageMultisampleIMG() {
			return newUnlinkedGLFunction("glRenderbufferStorageMultisampleIMG", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_ILInternalFormat_II(), MLS_LRenderbufferTarget_ILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorageMultisampleNV() {
			return newUnlinkedGLFunction("glRenderbufferStorageMultisampleNV", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_ILInternalFormat_II(), MLS_LRenderbufferTarget_ILInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glRenderbufferStorageOES() {
			return newUnlinkedGLFunction("glRenderbufferStorageOES", void.class, GLParamTypeSets.ML_V(), PS_LRenderbufferTarget_LInternalFormat_II(), MLS_LRenderbufferTarget_LInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glReplacementCodePointerSUN() {
			return newUnlinkedGLFunction("glReplacementCodePointerSUN", void.class, GLParamTypeSets.ML_V(), PS_LReplacementCodeTypeSUN_IP(), MLS_LReplacementCodeTypeSUN_IP());
		}
		private static UnlinkedGLFunction make_glReplacementCodeubSUN() {
			return newUnlinkedGLFunction("glReplacementCodeubSUN", void.class, GLParamTypeSets.ML_V(), PS_B(), MLS_B());
		}
		private static UnlinkedGLFunction make_glReplacementCodeubvSUN() {
			return newUnlinkedGLFunction("glReplacementCodeubvSUN", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiColor3fVertex3fSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiColor3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_IFFFFFF(), MLS_IFFFFFF());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiColor3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiColor3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_ppp(), MLS_ppp());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiColor4fNormal3fVertex3fSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiColor4fNormal3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_IFFFFFFFFFF(), MLS_IFFFFFFFFFF());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiColor4fNormal3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiColor4fNormal3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pppp(), MLS_pppp());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiColor4ubVertex3fSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiColor4ubVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_IBBBBFFF(), MLS_IBBBBFFF());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiColor4ubVertex3fvSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiColor4ubVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_ppp(), MLS_ppp());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiNormal3fVertex3fSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiNormal3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_IFFFFFF(), MLS_IFFFFFF());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiNormal3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiNormal3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_ppp(), MLS_ppp());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiSUN", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_IFFFFFFFFFFFF(), MLS_IFFFFFFFFFFFF());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_ppppp(), MLS_ppppp());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_IFFFFFFFF(), MLS_IFFFFFFFF());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pppp(), MLS_pppp());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiTexCoord2fVertex3fSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiTexCoord2fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_IFFFFF(), MLS_IFFFFF());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiTexCoord2fVertex3fvSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiTexCoord2fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_ppp(), MLS_ppp());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiVertex3fSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_IFFF(), MLS_IFFF());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuiVertex3fvSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuiVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glReplacementCodeuivSUN() {
			return newUnlinkedGLFunction("glReplacementCodeuivSUN", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glReplacementCodeusSUN() {
			return newUnlinkedGLFunction("glReplacementCodeusSUN", void.class, GLParamTypeSets.ML_V(), PS_C(), MLS_C());
		}
		private static UnlinkedGLFunction make_glReplacementCodeusvSUN() {
			return newUnlinkedGLFunction("glReplacementCodeusvSUN", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glRequestResidentProgramsNV() {
			return newUnlinkedGLFunction("glRequestResidentProgramsNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glResetHistogram() {
			return newUnlinkedGLFunction("glResetHistogram", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_(), MLS_LHistogramTargetEXT_());
		}
		private static UnlinkedGLFunction make_glResetHistogramEXT() {
			return newUnlinkedGLFunction("glResetHistogramEXT", void.class, GLParamTypeSets.ML_V(), PS_LHistogramTargetEXT_(), MLS_LHistogramTargetEXT_());
		}
		private static UnlinkedGLFunction make_glResetMemoryObjectParameterNV() {
			return newUnlinkedGLFunction("glResetMemoryObjectParameterNV", void.class, GLParamTypeSets.ML_V(), PS_IE(), MLS_IE());
		}
		private static UnlinkedGLFunction make_glResetMinmax() {
			return newUnlinkedGLFunction("glResetMinmax", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_(), MLS_LMinmaxTargetEXT_());
		}
		private static UnlinkedGLFunction make_glResetMinmaxEXT() {
			return newUnlinkedGLFunction("glResetMinmaxEXT", void.class, GLParamTypeSets.ML_V(), PS_LMinmaxTargetEXT_(), MLS_LMinmaxTargetEXT_());
		}
		private static UnlinkedGLFunction make_glResizeBuffersMESA() {
			return newUnlinkedGLFunction("glResizeBuffersMESA", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glResolveDepthValuesNV() {
			return newUnlinkedGLFunction("glResolveDepthValuesNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glResolveMultisampleFramebufferAPPLE() {
			return newUnlinkedGLFunction("glResolveMultisampleFramebufferAPPLE", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glResumeTransformFeedback() {
			return newUnlinkedGLFunction("glResumeTransformFeedback", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glResumeTransformFeedbackNV() {
			return newUnlinkedGLFunction("glResumeTransformFeedbackNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glRotated() {
			return newUnlinkedGLFunction("glRotated", void.class, GLParamTypeSets.ML_V(), PS_DDDD(), MLS_DDDD());
		}
		private static UnlinkedGLFunction make_glRotatef() {
			return newUnlinkedGLFunction("glRotatef", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glRotatex() {
			return newUnlinkedGLFunction("glRotatex", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glRotatexOES() {
			return newUnlinkedGLFunction("glRotatexOES", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glSampleCoverage() {
			return newUnlinkedGLFunction("glSampleCoverage", void.class, GLParamTypeSets.ML_V(), PS_FZ(), MLS_FZ());
		}
		private static UnlinkedGLFunction make_glSampleCoverageARB() {
			return newUnlinkedGLFunction("glSampleCoverageARB", void.class, GLParamTypeSets.ML_V(), PS_FZ(), MLS_FZ());
		}
		private static UnlinkedGLFunction make_glSampleCoveragex() {
			return newUnlinkedGLFunction("glSampleCoveragex", void.class, GLParamTypeSets.ML_V(), PS_IZ(), MLS_IZ());
		}
		private static UnlinkedGLFunction make_glSampleCoveragexOES() {
			return newUnlinkedGLFunction("glSampleCoveragexOES", void.class, GLParamTypeSets.ML_V(), PS_IZ(), MLS_IZ());
		}
		private static UnlinkedGLFunction make_glSampleMapATI() {
			return newUnlinkedGLFunction("glSampleMapATI", void.class, GLParamTypeSets.ML_V(), PS_IILSwizzleOpATI_(), MLS_IILSwizzleOpATI_());
		}
		private static UnlinkedGLFunction make_glSampleMaskEXT() {
			return newUnlinkedGLFunction("glSampleMaskEXT", void.class, GLParamTypeSets.ML_V(), PS_FZ(), MLS_FZ());
		}
		private static UnlinkedGLFunction make_glSampleMaskIndexedNV() {
			return newUnlinkedGLFunction("glSampleMaskIndexedNV", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glSampleMaskSGIS() {
			return newUnlinkedGLFunction("glSampleMaskSGIS", void.class, GLParamTypeSets.ML_V(), PS_FZ(), MLS_FZ());
		}
		private static UnlinkedGLFunction make_glSampleMaski() {
			return newUnlinkedGLFunction("glSampleMaski", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glSamplePatternEXT() {
			return newUnlinkedGLFunction("glSamplePatternEXT", void.class, GLParamTypeSets.ML_V(), PS_LSamplePatternEXT_(), MLS_LSamplePatternEXT_());
		}
		private static UnlinkedGLFunction make_glSamplePatternSGIS() {
			return newUnlinkedGLFunction("glSamplePatternSGIS", void.class, GLParamTypeSets.ML_V(), PS_LSamplePatternSGIS_(), MLS_LSamplePatternSGIS_());
		}
		private static UnlinkedGLFunction make_glSamplerParameterIiv() {
			return newUnlinkedGLFunction("glSamplerParameterIiv", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_p(), MLS_ILSamplerParameterI_p());
		}
		private static UnlinkedGLFunction make_glSamplerParameterIivEXT() {
			return newUnlinkedGLFunction("glSamplerParameterIivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_p(), MLS_ILSamplerParameterI_p());
		}
		private static UnlinkedGLFunction make_glSamplerParameterIivOES() {
			return newUnlinkedGLFunction("glSamplerParameterIivOES", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_p(), MLS_ILSamplerParameterI_p());
		}
		private static UnlinkedGLFunction make_glSamplerParameterIuiv() {
			return newUnlinkedGLFunction("glSamplerParameterIuiv", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_p(), MLS_ILSamplerParameterI_p());
		}
		private static UnlinkedGLFunction make_glSamplerParameterIuivEXT() {
			return newUnlinkedGLFunction("glSamplerParameterIuivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_p(), MLS_ILSamplerParameterI_p());
		}
		private static UnlinkedGLFunction make_glSamplerParameterIuivOES() {
			return newUnlinkedGLFunction("glSamplerParameterIuivOES", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_p(), MLS_ILSamplerParameterI_p());
		}
		private static UnlinkedGLFunction make_glSamplerParameterf() {
			return newUnlinkedGLFunction("glSamplerParameterf", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterF_F(), MLS_ILSamplerParameterF_F());
		}
		private static UnlinkedGLFunction make_glSamplerParameterfv() {
			return newUnlinkedGLFunction("glSamplerParameterfv", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterF_p(), MLS_ILSamplerParameterF_p());
		}
		private static UnlinkedGLFunction make_glSamplerParameteri() {
			return newUnlinkedGLFunction("glSamplerParameteri", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_I(), MLS_ILSamplerParameterI_I());
		}
		private static UnlinkedGLFunction make_glSamplerParameteriv() {
			return newUnlinkedGLFunction("glSamplerParameteriv", void.class, GLParamTypeSets.ML_V(), PS_ILSamplerParameterI_p(), MLS_ILSamplerParameterI_p());
		}
		private static UnlinkedGLFunction make_glScaled() {
			return newUnlinkedGLFunction("glScaled", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glScalef() {
			return newUnlinkedGLFunction("glScalef", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glScalex() {
			return newUnlinkedGLFunction("glScalex", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glScalexOES() {
			return newUnlinkedGLFunction("glScalexOES", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glScissor() {
			return newUnlinkedGLFunction("glScissor", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glScissorArrayv() {
			return newUnlinkedGLFunction("glScissorArrayv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glScissorArrayvNV() {
			return newUnlinkedGLFunction("glScissorArrayvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glScissorArrayvOES() {
			return newUnlinkedGLFunction("glScissorArrayvOES", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glScissorExclusiveArrayvNV() {
			return newUnlinkedGLFunction("glScissorExclusiveArrayvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glScissorExclusiveNV() {
			return newUnlinkedGLFunction("glScissorExclusiveNV", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glScissorIndexed() {
			return newUnlinkedGLFunction("glScissorIndexed", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glScissorIndexedNV() {
			return newUnlinkedGLFunction("glScissorIndexedNV", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glScissorIndexedOES() {
			return newUnlinkedGLFunction("glScissorIndexedOES", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glScissorIndexedv() {
			return newUnlinkedGLFunction("glScissorIndexedv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glScissorIndexedvNV() {
			return newUnlinkedGLFunction("glScissorIndexedvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glScissorIndexedvOES() {
			return newUnlinkedGLFunction("glScissorIndexedvOES", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3b() {
			return newUnlinkedGLFunction("glSecondaryColor3b", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3bEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3bEXT", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3bv() {
			return newUnlinkedGLFunction("glSecondaryColor3bv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3bvEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3bvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3d() {
			return newUnlinkedGLFunction("glSecondaryColor3d", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3dEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3dEXT", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3dv() {
			return newUnlinkedGLFunction("glSecondaryColor3dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3dvEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3dvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3f() {
			return newUnlinkedGLFunction("glSecondaryColor3f", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3fEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3fEXT", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3fv() {
			return newUnlinkedGLFunction("glSecondaryColor3fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3fvEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3fvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3hNV() {
			return newUnlinkedGLFunction("glSecondaryColor3hNV", void.class, GLParamTypeSets.ML_V(), PS_CCC(), MLS_CCC());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3hvNV() {
			return newUnlinkedGLFunction("glSecondaryColor3hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3i() {
			return newUnlinkedGLFunction("glSecondaryColor3i", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3iEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3iEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3iv() {
			return newUnlinkedGLFunction("glSecondaryColor3iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3ivEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3ivEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3s() {
			return newUnlinkedGLFunction("glSecondaryColor3s", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3sEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3sEXT", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3sv() {
			return newUnlinkedGLFunction("glSecondaryColor3sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3svEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3svEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3ub() {
			return newUnlinkedGLFunction("glSecondaryColor3ub", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3ubEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3ubEXT", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3ubv() {
			return newUnlinkedGLFunction("glSecondaryColor3ubv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3ubvEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3ubvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3ui() {
			return newUnlinkedGLFunction("glSecondaryColor3ui", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3uiEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3uiEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3uiv() {
			return newUnlinkedGLFunction("glSecondaryColor3uiv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3uivEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3uivEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3us() {
			return newUnlinkedGLFunction("glSecondaryColor3us", void.class, GLParamTypeSets.ML_V(), PS_CCC(), MLS_CCC());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3usEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3usEXT", void.class, GLParamTypeSets.ML_V(), PS_CCC(), MLS_CCC());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3usv() {
			return newUnlinkedGLFunction("glSecondaryColor3usv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColor3usvEXT() {
			return newUnlinkedGLFunction("glSecondaryColor3usvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColorFormatNV() {
			return newUnlinkedGLFunction("glSecondaryColorFormatNV", void.class, GLParamTypeSets.ML_V(), PS_ILColorPointerType_I(), MLS_ILColorPointerType_I());
		}
		private static UnlinkedGLFunction make_glSecondaryColorP3ui() {
			return newUnlinkedGLFunction("glSecondaryColorP3ui", void.class, GLParamTypeSets.ML_V(), PS_LColorPointerType_I(), MLS_LColorPointerType_I());
		}
		private static UnlinkedGLFunction make_glSecondaryColorP3uiv() {
			return newUnlinkedGLFunction("glSecondaryColorP3uiv", void.class, GLParamTypeSets.ML_V(), PS_LColorPointerType_p(), MLS_LColorPointerType_p());
		}
		private static UnlinkedGLFunction make_glSecondaryColorPointer() {
			return newUnlinkedGLFunction("glSecondaryColorPointer", void.class, GLParamTypeSets.ML_V(), PS_ILColorPointerType_Ip(), MLS_ILColorPointerType_Ip());
		}
		private static UnlinkedGLFunction make_glSecondaryColorPointerEXT() {
			return newUnlinkedGLFunction("glSecondaryColorPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_ILColorPointerType_Ip(), MLS_ILColorPointerType_Ip());
		}
		private static UnlinkedGLFunction make_glSecondaryColorPointerListIBM() {
			return newUnlinkedGLFunction("glSecondaryColorPointerListIBM", void.class, GLParamTypeSets.ML_V(), PS_ILSecondaryColorPointerTypeIBM_IPI(), MLS_ILSecondaryColorPointerTypeIBM_IPI());
		}
		private static UnlinkedGLFunction make_glSelectBuffer() {
			return newUnlinkedGLFunction("glSelectBuffer", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glSelectPerfMonitorCountersAMD() {
			return newUnlinkedGLFunction("glSelectPerfMonitorCountersAMD", void.class, GLParamTypeSets.ML_V(), PS_IZIIP(), MLS_IZIIP());
		}
		private static UnlinkedGLFunction make_glSemaphoreParameterivNV() {
			return newUnlinkedGLFunction("glSemaphoreParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_ILSemaphoreParameterName_p(), MLS_ILSemaphoreParameterName_p());
		}
		private static UnlinkedGLFunction make_glSemaphoreParameterui64vEXT() {
			return newUnlinkedGLFunction("glSemaphoreParameterui64vEXT", void.class, GLParamTypeSets.ML_V(), PS_ILSemaphoreParameterName_p(), MLS_ILSemaphoreParameterName_p());
		}
		private static UnlinkedGLFunction make_glSeparableFilter2D() {
			return newUnlinkedGLFunction("glSeparableFilter2D", void.class, GLParamTypeSets.ML_V(), PS_LSeparableTargetEXT_LInternalFormat_IILPixelFormat_LPixelType_pp(), MLS_LSeparableTargetEXT_LInternalFormat_IILPixelFormat_LPixelType_pp());
		}
		private static UnlinkedGLFunction make_glSeparableFilter2DEXT() {
			return newUnlinkedGLFunction("glSeparableFilter2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LSeparableTargetEXT_LInternalFormat_IILPixelFormat_LPixelType_pp(), MLS_LSeparableTargetEXT_LInternalFormat_IILPixelFormat_LPixelType_pp());
		}
		private static UnlinkedGLFunction make_glSetFenceAPPLE() {
			return newUnlinkedGLFunction("glSetFenceAPPLE", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glSetFenceNV() {
			return newUnlinkedGLFunction("glSetFenceNV", void.class, GLParamTypeSets.ML_V(), PS_ILFenceConditionNV_(), MLS_ILFenceConditionNV_());
		}
		private static UnlinkedGLFunction make_glSetFragmentShaderConstantATI() {
			return newUnlinkedGLFunction("glSetFragmentShaderConstantATI", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glSetInvariantEXT() {
			return newUnlinkedGLFunction("glSetInvariantEXT", void.class, GLParamTypeSets.ML_V(), PS_ILScalarType_p(), MLS_ILScalarType_p());
		}
		private static UnlinkedGLFunction make_glSetLocalConstantEXT() {
			return newUnlinkedGLFunction("glSetLocalConstantEXT", void.class, GLParamTypeSets.ML_V(), PS_ILScalarType_p(), MLS_ILScalarType_p());
		}
		private static UnlinkedGLFunction make_glSetMultisamplefvAMD() {
			return newUnlinkedGLFunction("glSetMultisamplefvAMD", void.class, GLParamTypeSets.ML_V(), PS_EIp(), MLS_EIp());
		}
		private static UnlinkedGLFunction make_glShadeModel() {
			return newUnlinkedGLFunction("glShadeModel", void.class, GLParamTypeSets.ML_V(), PS_LShadingModel_(), MLS_LShadingModel_());
		}
		private static UnlinkedGLFunction make_glShaderBinary() {
			return newUnlinkedGLFunction("glShaderBinary", void.class, GLParamTypeSets.ML_V(), PS_IpLShaderBinaryFormat_pI(), MLS_IpLShaderBinaryFormat_pI());
		}
		private static UnlinkedGLFunction make_glShaderOp1EXT() {
			return newUnlinkedGLFunction("glShaderOp1EXT", void.class, GLParamTypeSets.ML_V(), PS_LVertexShaderOpEXT_II(), MLS_LVertexShaderOpEXT_II());
		}
		private static UnlinkedGLFunction make_glShaderOp2EXT() {
			return newUnlinkedGLFunction("glShaderOp2EXT", void.class, GLParamTypeSets.ML_V(), PS_LVertexShaderOpEXT_III(), MLS_LVertexShaderOpEXT_III());
		}
		private static UnlinkedGLFunction make_glShaderOp3EXT() {
			return newUnlinkedGLFunction("glShaderOp3EXT", void.class, GLParamTypeSets.ML_V(), PS_LVertexShaderOpEXT_IIII(), MLS_LVertexShaderOpEXT_IIII());
		}
		private static UnlinkedGLFunction make_glShaderSource() {
			return newUnlinkedGLFunction("glShaderSource", void.class, GLParamTypeSets.ML_V(), PS_IIpp(), MLS_IIpp());
		}
		private static UnlinkedGLFunction make_glShaderSourceARB() {
			return newUnlinkedGLFunction("glShaderSourceARB", void.class, GLParamTypeSets.ML_V(), PS_IIPp(), MLS_IIPp());
		}
		private static UnlinkedGLFunction make_glShaderStorageBlockBinding() {
			return newUnlinkedGLFunction("glShaderStorageBlockBinding", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glShadingRateEXT() {
			return newUnlinkedGLFunction("glShadingRateEXT", void.class, GLParamTypeSets.ML_V(), PS_LShadingRate_(), MLS_LShadingRate_());
		}
		private static UnlinkedGLFunction make_glShadingRateCombinerOpsEXT() {
			return newUnlinkedGLFunction("glShadingRateCombinerOpsEXT", void.class, GLParamTypeSets.ML_V(), PS_LShadingRateCombinerOp_LShadingRateCombinerOp_(), MLS_LShadingRateCombinerOp_LShadingRateCombinerOp_());
		}
		private static UnlinkedGLFunction make_glShadingRateImageBarrierNV() {
			return newUnlinkedGLFunction("glShadingRateImageBarrierNV", void.class, GLParamTypeSets.ML_V(), PS_Z(), MLS_Z());
		}
		private static UnlinkedGLFunction make_glShadingRateQCOM() {
			return newUnlinkedGLFunction("glShadingRateQCOM", void.class, GLParamTypeSets.ML_V(), PS_LShadingRateQCOM_(), MLS_LShadingRateQCOM_());
		}
		private static UnlinkedGLFunction make_glShadingRateImagePaletteNV() {
			return newUnlinkedGLFunction("glShadingRateImagePaletteNV", void.class, GLParamTypeSets.ML_V(), PS_IIIp(), MLS_IIIp());
		}
		private static UnlinkedGLFunction make_glShadingRateSampleOrderNV() {
			return newUnlinkedGLFunction("glShadingRateSampleOrderNV", void.class, GLParamTypeSets.ML_V(), PS_E(), MLS_E());
		}
		private static UnlinkedGLFunction make_glShadingRateSampleOrderCustomNV() {
			return newUnlinkedGLFunction("glShadingRateSampleOrderCustomNV", void.class, GLParamTypeSets.ML_V(), PS_EIp(), MLS_EIp());
		}
		private static UnlinkedGLFunction make_glSharpenTexFuncSGIS() {
			return newUnlinkedGLFunction("glSharpenTexFuncSGIS", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_Ip(), MLS_LTextureTarget_Ip());
		}
		private static UnlinkedGLFunction make_glSignalSemaphoreEXT() {
			return newUnlinkedGLFunction("glSignalSemaphoreEXT", void.class, GLParamTypeSets.ML_V(), PS_IIpIpp(), MLS_IIpIpp());
		}
		private static UnlinkedGLFunction make_glSignalSemaphoreui64NVX() {
			return newUnlinkedGLFunction("glSignalSemaphoreui64NVX", void.class, GLParamTypeSets.ML_V(), PS_IIpp(), MLS_IIpp());
		}
		private static UnlinkedGLFunction make_glSpecializeShader() {
			return newUnlinkedGLFunction("glSpecializeShader", void.class, GLParamTypeSets.ML_V(), PS_IpIpp(), MLS_IpIpp());
		}
		private static UnlinkedGLFunction make_glSpecializeShaderARB() {
			return newUnlinkedGLFunction("glSpecializeShaderARB", void.class, GLParamTypeSets.ML_V(), PS_IpIpp(), MLS_IpIpp());
		}
		private static UnlinkedGLFunction make_glSpriteParameterfSGIX() {
			return newUnlinkedGLFunction("glSpriteParameterfSGIX", void.class, GLParamTypeSets.ML_V(), PS_LSpriteParameterNameSGIX_F(), MLS_LSpriteParameterNameSGIX_F());
		}
		private static UnlinkedGLFunction make_glSpriteParameterfvSGIX() {
			return newUnlinkedGLFunction("glSpriteParameterfvSGIX", void.class, GLParamTypeSets.ML_V(), PS_LSpriteParameterNameSGIX_p(), MLS_LSpriteParameterNameSGIX_p());
		}
		private static UnlinkedGLFunction make_glSpriteParameteriSGIX() {
			return newUnlinkedGLFunction("glSpriteParameteriSGIX", void.class, GLParamTypeSets.ML_V(), PS_LSpriteParameterNameSGIX_LSpriteModeSGIX_(), MLS_LSpriteParameterNameSGIX_LSpriteModeSGIX_());
		}
		private static UnlinkedGLFunction make_glSpriteParameterivSGIX() {
			return newUnlinkedGLFunction("glSpriteParameterivSGIX", void.class, GLParamTypeSets.ML_V(), PS_LSpriteParameterNameSGIX_p(), MLS_LSpriteParameterNameSGIX_p());
		}
		private static UnlinkedGLFunction make_glStartInstrumentsSGIX() {
			return newUnlinkedGLFunction("glStartInstrumentsSGIX", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glStartTilingQCOM() {
			return newUnlinkedGLFunction("glStartTilingQCOM", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glStateCaptureNV() {
			return newUnlinkedGLFunction("glStateCaptureNV", void.class, GLParamTypeSets.ML_V(), PS_IE(), MLS_IE());
		}
		private static UnlinkedGLFunction make_glStencilClearTagEXT() {
			return newUnlinkedGLFunction("glStencilClearTagEXT", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glStencilFillPathInstancedNV() {
			return newUnlinkedGLFunction("glStencilFillPathInstancedNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathElementType_pILPathFillMode_ILPathTransformType_p(), MLS_ILPathElementType_pILPathFillMode_ILPathTransformType_p());
		}
		private static UnlinkedGLFunction make_glStencilFillPathNV() {
			return newUnlinkedGLFunction("glStencilFillPathNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathFillMode_I(), MLS_ILPathFillMode_I());
		}
		private static UnlinkedGLFunction make_glStencilFunc() {
			return newUnlinkedGLFunction("glStencilFunc", void.class, GLParamTypeSets.ML_V(), PS_LStencilFunction_II(), MLS_LStencilFunction_II());
		}
		private static UnlinkedGLFunction make_glStencilFuncSeparate() {
			return newUnlinkedGLFunction("glStencilFuncSeparate", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LStencilFunction_II(), MLS_LTriangleFace_LStencilFunction_II());
		}
		private static UnlinkedGLFunction make_glStencilFuncSeparateATI() {
			return newUnlinkedGLFunction("glStencilFuncSeparateATI", void.class, GLParamTypeSets.ML_V(), PS_LStencilFunction_LStencilFunction_II(), MLS_LStencilFunction_LStencilFunction_II());
		}
		private static UnlinkedGLFunction make_glStencilMask() {
			return newUnlinkedGLFunction("glStencilMask", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glStencilMaskSeparate() {
			return newUnlinkedGLFunction("glStencilMaskSeparate", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_I(), MLS_LTriangleFace_I());
		}
		private static UnlinkedGLFunction make_glStencilOp() {
			return newUnlinkedGLFunction("glStencilOp", void.class, GLParamTypeSets.ML_V(), PS_LStencilOp_LStencilOp_LStencilOp_(), MLS_LStencilOp_LStencilOp_LStencilOp_());
		}
		private static UnlinkedGLFunction make_glStencilOpSeparate() {
			return newUnlinkedGLFunction("glStencilOpSeparate", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LStencilOp_LStencilOp_LStencilOp_(), MLS_LTriangleFace_LStencilOp_LStencilOp_LStencilOp_());
		}
		private static UnlinkedGLFunction make_glStencilOpSeparateATI() {
			return newUnlinkedGLFunction("glStencilOpSeparateATI", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LStencilOp_LStencilOp_LStencilOp_(), MLS_LTriangleFace_LStencilOp_LStencilOp_LStencilOp_());
		}
		private static UnlinkedGLFunction make_glStencilOpValueAMD() {
			return newUnlinkedGLFunction("glStencilOpValueAMD", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_I(), MLS_LTriangleFace_I());
		}
		private static UnlinkedGLFunction make_glStencilStrokePathInstancedNV() {
			return newUnlinkedGLFunction("glStencilStrokePathInstancedNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathElementType_pIIILPathTransformType_p(), MLS_ILPathElementType_pIIILPathTransformType_p());
		}
		private static UnlinkedGLFunction make_glStencilStrokePathNV() {
			return newUnlinkedGLFunction("glStencilStrokePathNV", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glStencilThenCoverFillPathInstancedNV() {
			return newUnlinkedGLFunction("glStencilThenCoverFillPathInstancedNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathElementType_pILPathFillMode_ILInstancedPathCoverMode_LPathTransformType_p(), MLS_ILPathElementType_pILPathFillMode_ILInstancedPathCoverMode_LPathTransformType_p());
		}
		private static UnlinkedGLFunction make_glStencilThenCoverFillPathNV() {
			return newUnlinkedGLFunction("glStencilThenCoverFillPathNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathFillMode_ILPathCoverMode_(), MLS_ILPathFillMode_ILPathCoverMode_());
		}
		private static UnlinkedGLFunction make_glStencilThenCoverStrokePathInstancedNV() {
			return newUnlinkedGLFunction("glStencilThenCoverStrokePathInstancedNV", void.class, GLParamTypeSets.ML_V(), PS_ILPathElementType_pIIILInstancedPathCoverMode_LPathTransformType_p(), MLS_ILPathElementType_pIIILInstancedPathCoverMode_LPathTransformType_p());
		}
		private static UnlinkedGLFunction make_glStencilThenCoverStrokePathNV() {
			return newUnlinkedGLFunction("glStencilThenCoverStrokePathNV", void.class, GLParamTypeSets.ML_V(), PS_IIILPathCoverMode_(), MLS_IIILPathCoverMode_());
		}
		private static UnlinkedGLFunction make_glStopInstrumentsSGIX() {
			return newUnlinkedGLFunction("glStopInstrumentsSGIX", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glStringMarkerGREMEDY() {
			return newUnlinkedGLFunction("glStringMarkerGREMEDY", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glSubpixelPrecisionBiasNV() {
			return newUnlinkedGLFunction("glSubpixelPrecisionBiasNV", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glSwizzleEXT() {
			return newUnlinkedGLFunction("glSwizzleEXT", void.class, GLParamTypeSets.ML_V(), PS_IILVertexShaderCoordOutEXT_LVertexShaderCoordOutEXT_LVertexShaderCoordOutEXT_LVertexShaderCoordOutEXT_(), MLS_IILVertexShaderCoordOutEXT_LVertexShaderCoordOutEXT_LVertexShaderCoordOutEXT_LVertexShaderCoordOutEXT_());
		}
		private static UnlinkedGLFunction make_glSyncTextureINTEL() {
			return newUnlinkedGLFunction("glSyncTextureINTEL", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glTagSampleBufferSGIX() {
			return newUnlinkedGLFunction("glTagSampleBufferSGIX", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glTangent3bEXT() {
			return newUnlinkedGLFunction("glTangent3bEXT", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glTangent3bvEXT() {
			return newUnlinkedGLFunction("glTangent3bvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTangent3dEXT() {
			return newUnlinkedGLFunction("glTangent3dEXT", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glTangent3dvEXT() {
			return newUnlinkedGLFunction("glTangent3dvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTangent3fEXT() {
			return newUnlinkedGLFunction("glTangent3fEXT", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glTangent3fvEXT() {
			return newUnlinkedGLFunction("glTangent3fvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTangent3iEXT() {
			return newUnlinkedGLFunction("glTangent3iEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glTangent3ivEXT() {
			return newUnlinkedGLFunction("glTangent3ivEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTangent3sEXT() {
			return newUnlinkedGLFunction("glTangent3sEXT", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glTangent3svEXT() {
			return newUnlinkedGLFunction("glTangent3svEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTangentPointerEXT() {
			return newUnlinkedGLFunction("glTangentPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_LTangentPointerTypeEXT_Ip(), MLS_LTangentPointerTypeEXT_Ip());
		}
		private static UnlinkedGLFunction make_glTbufferMask3DFX() {
			return newUnlinkedGLFunction("glTbufferMask3DFX", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glTessellationFactorAMD() {
			return newUnlinkedGLFunction("glTessellationFactorAMD", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glTessellationModeAMD() {
			return newUnlinkedGLFunction("glTessellationModeAMD", void.class, GLParamTypeSets.ML_V(), PS_E(), MLS_E());
		}
		private static UnlinkedGLFunction make_glTestFenceAPPLE() {
			return newUnlinkedGLFunction("glTestFenceAPPLE", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glTestFenceNV() {
			return newUnlinkedGLFunction("glTestFenceNV", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glTestObjectAPPLE() {
			return newUnlinkedGLFunction("glTestObjectAPPLE", boolean.class, GLParamTypeSets.ML_Z(), PS_LObjectTypeAPPLE_I(), MLS_LObjectTypeAPPLE_I());
		}
		private static UnlinkedGLFunction make_glTexAttachMemoryNV() {
			return newUnlinkedGLFunction("glTexAttachMemoryNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IJ(), MLS_LTextureTarget_IJ());
		}
		private static UnlinkedGLFunction make_glTexBuffer() {
			return newUnlinkedGLFunction("glTexBuffer", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LSizedInternalFormat_I(), MLS_LTextureTarget_LSizedInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glTexBufferARB() {
			return newUnlinkedGLFunction("glTexBufferARB", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LSizedInternalFormat_I(), MLS_LTextureTarget_LSizedInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glTexBufferEXT() {
			return newUnlinkedGLFunction("glTexBufferEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LSizedInternalFormat_I(), MLS_LTextureTarget_LSizedInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glTexBufferOES() {
			return newUnlinkedGLFunction("glTexBufferOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LSizedInternalFormat_I(), MLS_LTextureTarget_LSizedInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glTexBufferRange() {
			return newUnlinkedGLFunction("glTexBufferRange", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LSizedInternalFormat_IJJ(), MLS_LTextureTarget_LSizedInternalFormat_IJJ());
		}
		private static UnlinkedGLFunction make_glTexBufferRangeEXT() {
			return newUnlinkedGLFunction("glTexBufferRangeEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LSizedInternalFormat_IJJ(), MLS_LTextureTarget_LSizedInternalFormat_IJJ());
		}
		private static UnlinkedGLFunction make_glTexBufferRangeOES() {
			return newUnlinkedGLFunction("glTexBufferRangeOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LSizedInternalFormat_IJJ(), MLS_LTextureTarget_LSizedInternalFormat_IJJ());
		}
		private static UnlinkedGLFunction make_glTexBumpParameterfvATI() {
			return newUnlinkedGLFunction("glTexBumpParameterfvATI", void.class, GLParamTypeSets.ML_V(), PS_LTexBumpParameterATI_p(), MLS_LTexBumpParameterATI_p());
		}
		private static UnlinkedGLFunction make_glTexBumpParameterivATI() {
			return newUnlinkedGLFunction("glTexBumpParameterivATI", void.class, GLParamTypeSets.ML_V(), PS_LTexBumpParameterATI_p(), MLS_LTexBumpParameterATI_p());
		}
		private static UnlinkedGLFunction make_glTexCoord1bOES() {
			return newUnlinkedGLFunction("glTexCoord1bOES", void.class, GLParamTypeSets.ML_V(), PS_B(), MLS_B());
		}
		private static UnlinkedGLFunction make_glTexCoord1bvOES() {
			return newUnlinkedGLFunction("glTexCoord1bvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord1d() {
			return newUnlinkedGLFunction("glTexCoord1d", void.class, GLParamTypeSets.ML_V(), PS_D(), MLS_D());
		}
		private static UnlinkedGLFunction make_glTexCoord1dv() {
			return newUnlinkedGLFunction("glTexCoord1dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord1f() {
			return newUnlinkedGLFunction("glTexCoord1f", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glTexCoord1fv() {
			return newUnlinkedGLFunction("glTexCoord1fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord1hNV() {
			return newUnlinkedGLFunction("glTexCoord1hNV", void.class, GLParamTypeSets.ML_V(), PS_C(), MLS_C());
		}
		private static UnlinkedGLFunction make_glTexCoord1hvNV() {
			return newUnlinkedGLFunction("glTexCoord1hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord1i() {
			return newUnlinkedGLFunction("glTexCoord1i", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glTexCoord1iv() {
			return newUnlinkedGLFunction("glTexCoord1iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord1s() {
			return newUnlinkedGLFunction("glTexCoord1s", void.class, GLParamTypeSets.ML_V(), PS_S(), MLS_S());
		}
		private static UnlinkedGLFunction make_glTexCoord1sv() {
			return newUnlinkedGLFunction("glTexCoord1sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord1xOES() {
			return newUnlinkedGLFunction("glTexCoord1xOES", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glTexCoord1xvOES() {
			return newUnlinkedGLFunction("glTexCoord1xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord2bOES() {
			return newUnlinkedGLFunction("glTexCoord2bOES", void.class, GLParamTypeSets.ML_V(), PS_BB(), MLS_BB());
		}
		private static UnlinkedGLFunction make_glTexCoord2bvOES() {
			return newUnlinkedGLFunction("glTexCoord2bvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord2d() {
			return newUnlinkedGLFunction("glTexCoord2d", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glTexCoord2dv() {
			return newUnlinkedGLFunction("glTexCoord2dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord2f() {
			return newUnlinkedGLFunction("glTexCoord2f", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glTexCoord2fColor3fVertex3fSUN() {
			return newUnlinkedGLFunction("glTexCoord2fColor3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_FFFFFFFF(), MLS_FFFFFFFF());
		}
		private static UnlinkedGLFunction make_glTexCoord2fColor3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glTexCoord2fColor3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_ppp(), MLS_ppp());
		}
		private static UnlinkedGLFunction make_glTexCoord2fColor4fNormal3fVertex3fSUN() {
			return newUnlinkedGLFunction("glTexCoord2fColor4fNormal3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_FFFFFFFFFFFF(), MLS_FFFFFFFFFFFF());
		}
		private static UnlinkedGLFunction make_glTexCoord2fColor4fNormal3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glTexCoord2fColor4fNormal3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pppp(), MLS_pppp());
		}
		private static UnlinkedGLFunction make_glTexCoord2fColor4ubVertex3fSUN() {
			return newUnlinkedGLFunction("glTexCoord2fColor4ubVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_FFBBBBFFF(), MLS_FFBBBBFFF());
		}
		private static UnlinkedGLFunction make_glTexCoord2fColor4ubVertex3fvSUN() {
			return newUnlinkedGLFunction("glTexCoord2fColor4ubVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_ppp(), MLS_ppp());
		}
		private static UnlinkedGLFunction make_glTexCoord2fNormal3fVertex3fSUN() {
			return newUnlinkedGLFunction("glTexCoord2fNormal3fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_FFFFFFFF(), MLS_FFFFFFFF());
		}
		private static UnlinkedGLFunction make_glTexCoord2fNormal3fVertex3fvSUN() {
			return newUnlinkedGLFunction("glTexCoord2fNormal3fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_ppp(), MLS_ppp());
		}
		private static UnlinkedGLFunction make_glTexCoord2fVertex3fSUN() {
			return newUnlinkedGLFunction("glTexCoord2fVertex3fSUN", void.class, GLParamTypeSets.ML_V(), PS_FFFFF(), MLS_FFFFF());
		}
		private static UnlinkedGLFunction make_glTexCoord2fVertex3fvSUN() {
			return newUnlinkedGLFunction("glTexCoord2fVertex3fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glTexCoord2fv() {
			return newUnlinkedGLFunction("glTexCoord2fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord2hNV() {
			return newUnlinkedGLFunction("glTexCoord2hNV", void.class, GLParamTypeSets.ML_V(), PS_CC(), MLS_CC());
		}
		private static UnlinkedGLFunction make_glTexCoord2hvNV() {
			return newUnlinkedGLFunction("glTexCoord2hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord2i() {
			return newUnlinkedGLFunction("glTexCoord2i", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glTexCoord2iv() {
			return newUnlinkedGLFunction("glTexCoord2iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord2s() {
			return newUnlinkedGLFunction("glTexCoord2s", void.class, GLParamTypeSets.ML_V(), PS_SS(), MLS_SS());
		}
		private static UnlinkedGLFunction make_glTexCoord2sv() {
			return newUnlinkedGLFunction("glTexCoord2sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord2xOES() {
			return newUnlinkedGLFunction("glTexCoord2xOES", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glTexCoord2xvOES() {
			return newUnlinkedGLFunction("glTexCoord2xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord3bOES() {
			return newUnlinkedGLFunction("glTexCoord3bOES", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glTexCoord3bvOES() {
			return newUnlinkedGLFunction("glTexCoord3bvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord3d() {
			return newUnlinkedGLFunction("glTexCoord3d", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glTexCoord3dv() {
			return newUnlinkedGLFunction("glTexCoord3dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord3f() {
			return newUnlinkedGLFunction("glTexCoord3f", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glTexCoord3fv() {
			return newUnlinkedGLFunction("glTexCoord3fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord3hNV() {
			return newUnlinkedGLFunction("glTexCoord3hNV", void.class, GLParamTypeSets.ML_V(), PS_CCC(), MLS_CCC());
		}
		private static UnlinkedGLFunction make_glTexCoord3hvNV() {
			return newUnlinkedGLFunction("glTexCoord3hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord3i() {
			return newUnlinkedGLFunction("glTexCoord3i", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glTexCoord3iv() {
			return newUnlinkedGLFunction("glTexCoord3iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord3s() {
			return newUnlinkedGLFunction("glTexCoord3s", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glTexCoord3sv() {
			return newUnlinkedGLFunction("glTexCoord3sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord3xOES() {
			return newUnlinkedGLFunction("glTexCoord3xOES", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glTexCoord3xvOES() {
			return newUnlinkedGLFunction("glTexCoord3xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord4bOES() {
			return newUnlinkedGLFunction("glTexCoord4bOES", void.class, GLParamTypeSets.ML_V(), PS_BBBB(), MLS_BBBB());
		}
		private static UnlinkedGLFunction make_glTexCoord4bvOES() {
			return newUnlinkedGLFunction("glTexCoord4bvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord4d() {
			return newUnlinkedGLFunction("glTexCoord4d", void.class, GLParamTypeSets.ML_V(), PS_DDDD(), MLS_DDDD());
		}
		private static UnlinkedGLFunction make_glTexCoord4dv() {
			return newUnlinkedGLFunction("glTexCoord4dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord4f() {
			return newUnlinkedGLFunction("glTexCoord4f", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glTexCoord4fColor4fNormal3fVertex4fSUN() {
			return newUnlinkedGLFunction("glTexCoord4fColor4fNormal3fVertex4fSUN", void.class, GLParamTypeSets.ML_V(), PS_FFFFFFFFFFFFFFF(), MLS_FFFFFFFFFFFFFFF());
		}
		private static UnlinkedGLFunction make_glTexCoord4fColor4fNormal3fVertex4fvSUN() {
			return newUnlinkedGLFunction("glTexCoord4fColor4fNormal3fVertex4fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pppp(), MLS_pppp());
		}
		private static UnlinkedGLFunction make_glTexCoord4fVertex4fSUN() {
			return newUnlinkedGLFunction("glTexCoord4fVertex4fSUN", void.class, GLParamTypeSets.ML_V(), PS_FFFFFFFF(), MLS_FFFFFFFF());
		}
		private static UnlinkedGLFunction make_glTexCoord4fVertex4fvSUN() {
			return newUnlinkedGLFunction("glTexCoord4fVertex4fvSUN", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glTexCoord4fv() {
			return newUnlinkedGLFunction("glTexCoord4fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord4hNV() {
			return newUnlinkedGLFunction("glTexCoord4hNV", void.class, GLParamTypeSets.ML_V(), PS_CCCC(), MLS_CCCC());
		}
		private static UnlinkedGLFunction make_glTexCoord4hvNV() {
			return newUnlinkedGLFunction("glTexCoord4hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord4i() {
			return newUnlinkedGLFunction("glTexCoord4i", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glTexCoord4iv() {
			return newUnlinkedGLFunction("glTexCoord4iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord4s() {
			return newUnlinkedGLFunction("glTexCoord4s", void.class, GLParamTypeSets.ML_V(), PS_SSSS(), MLS_SSSS());
		}
		private static UnlinkedGLFunction make_glTexCoord4sv() {
			return newUnlinkedGLFunction("glTexCoord4sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoord4xOES() {
			return newUnlinkedGLFunction("glTexCoord4xOES", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glTexCoord4xvOES() {
			return newUnlinkedGLFunction("glTexCoord4xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glTexCoordFormatNV() {
			return newUnlinkedGLFunction("glTexCoordFormatNV", void.class, GLParamTypeSets.ML_V(), PS_IEI(), MLS_IEI());
		}
		private static UnlinkedGLFunction make_glTexCoordP1ui() {
			return newUnlinkedGLFunction("glTexCoordP1ui", void.class, GLParamTypeSets.ML_V(), PS_LTexCoordPointerType_I(), MLS_LTexCoordPointerType_I());
		}
		private static UnlinkedGLFunction make_glTexCoordP1uiv() {
			return newUnlinkedGLFunction("glTexCoordP1uiv", void.class, GLParamTypeSets.ML_V(), PS_LTexCoordPointerType_p(), MLS_LTexCoordPointerType_p());
		}
		private static UnlinkedGLFunction make_glTexCoordP2ui() {
			return newUnlinkedGLFunction("glTexCoordP2ui", void.class, GLParamTypeSets.ML_V(), PS_LTexCoordPointerType_I(), MLS_LTexCoordPointerType_I());
		}
		private static UnlinkedGLFunction make_glTexCoordP2uiv() {
			return newUnlinkedGLFunction("glTexCoordP2uiv", void.class, GLParamTypeSets.ML_V(), PS_LTexCoordPointerType_p(), MLS_LTexCoordPointerType_p());
		}
		private static UnlinkedGLFunction make_glTexCoordP3ui() {
			return newUnlinkedGLFunction("glTexCoordP3ui", void.class, GLParamTypeSets.ML_V(), PS_LTexCoordPointerType_I(), MLS_LTexCoordPointerType_I());
		}
		private static UnlinkedGLFunction make_glTexCoordP3uiv() {
			return newUnlinkedGLFunction("glTexCoordP3uiv", void.class, GLParamTypeSets.ML_V(), PS_LTexCoordPointerType_p(), MLS_LTexCoordPointerType_p());
		}
		private static UnlinkedGLFunction make_glTexCoordP4ui() {
			return newUnlinkedGLFunction("glTexCoordP4ui", void.class, GLParamTypeSets.ML_V(), PS_LTexCoordPointerType_I(), MLS_LTexCoordPointerType_I());
		}
		private static UnlinkedGLFunction make_glTexCoordP4uiv() {
			return newUnlinkedGLFunction("glTexCoordP4uiv", void.class, GLParamTypeSets.ML_V(), PS_LTexCoordPointerType_p(), MLS_LTexCoordPointerType_p());
		}
		private static UnlinkedGLFunction make_glTexCoordPointer() {
			return newUnlinkedGLFunction("glTexCoordPointer", void.class, GLParamTypeSets.ML_V(), PS_ILTexCoordPointerType_Ip(), MLS_ILTexCoordPointerType_Ip());
		}
		private static UnlinkedGLFunction make_glTexCoordPointerEXT() {
			return newUnlinkedGLFunction("glTexCoordPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTexCoordPointerType_IIp(), MLS_ILTexCoordPointerType_IIp());
		}
		private static UnlinkedGLFunction make_glTexCoordPointerListIBM() {
			return newUnlinkedGLFunction("glTexCoordPointerListIBM", void.class, GLParamTypeSets.ML_V(), PS_ILTexCoordPointerType_IPI(), MLS_ILTexCoordPointerType_IPI());
		}
		private static UnlinkedGLFunction make_glTexCoordPointervINTEL() {
			return newUnlinkedGLFunction("glTexCoordPointervINTEL", void.class, GLParamTypeSets.ML_V(), PS_ILVertexPointerType_P(), MLS_ILVertexPointerType_P());
		}
		private static UnlinkedGLFunction make_glTexEnvf() {
			return newUnlinkedGLFunction("glTexEnvf", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_F(), MLS_LTextureEnvTarget_LTextureEnvParameter_F());
		}
		private static UnlinkedGLFunction make_glTexEnvfv() {
			return newUnlinkedGLFunction("glTexEnvfv", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_p(), MLS_LTextureEnvTarget_LTextureEnvParameter_p());
		}
		private static UnlinkedGLFunction make_glTexEnvi() {
			return newUnlinkedGLFunction("glTexEnvi", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_I(), MLS_LTextureEnvTarget_LTextureEnvParameter_I());
		}
		private static UnlinkedGLFunction make_glTexEnviv() {
			return newUnlinkedGLFunction("glTexEnviv", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_p(), MLS_LTextureEnvTarget_LTextureEnvParameter_p());
		}
		private static UnlinkedGLFunction make_glTexEnvx() {
			return newUnlinkedGLFunction("glTexEnvx", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_I(), MLS_LTextureEnvTarget_LTextureEnvParameter_I());
		}
		private static UnlinkedGLFunction make_glTexEnvxOES() {
			return newUnlinkedGLFunction("glTexEnvxOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_I(), MLS_LTextureEnvTarget_LTextureEnvParameter_I());
		}
		private static UnlinkedGLFunction make_glTexEnvxv() {
			return newUnlinkedGLFunction("glTexEnvxv", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_p(), MLS_LTextureEnvTarget_LTextureEnvParameter_p());
		}
		private static UnlinkedGLFunction make_glTexEnvxvOES() {
			return newUnlinkedGLFunction("glTexEnvxvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureEnvTarget_LTextureEnvParameter_p(), MLS_LTextureEnvTarget_LTextureEnvParameter_p());
		}
		private static UnlinkedGLFunction make_glTexEstimateMotionQCOM() {
			return newUnlinkedGLFunction("glTexEstimateMotionQCOM", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glTexEstimateMotionRegionsQCOM() {
			return newUnlinkedGLFunction("glTexEstimateMotionRegionsQCOM", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glExtrapolateTex2DQCOM() {
			return newUnlinkedGLFunction("glExtrapolateTex2DQCOM", void.class, GLParamTypeSets.ML_V(), PS_IIIF(), MLS_IIIF());
		}
		private static UnlinkedGLFunction make_glTexFilterFuncSGIS() {
			return newUnlinkedGLFunction("glTexFilterFuncSGIS", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureFilterSGIS_Ip(), MLS_LTextureTarget_LTextureFilterSGIS_Ip());
		}
		private static UnlinkedGLFunction make_glTexGend() {
			return newUnlinkedGLFunction("glTexGend", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_D(), MLS_LTextureCoordName_LTextureGenParameter_D());
		}
		private static UnlinkedGLFunction make_glTexGendv() {
			return newUnlinkedGLFunction("glTexGendv", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_p(), MLS_LTextureCoordName_LTextureGenParameter_p());
		}
		private static UnlinkedGLFunction make_glTexGenf() {
			return newUnlinkedGLFunction("glTexGenf", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_F(), MLS_LTextureCoordName_LTextureGenParameter_F());
		}
		private static UnlinkedGLFunction make_glTexGenfOES() {
			return newUnlinkedGLFunction("glTexGenfOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_F(), MLS_LTextureCoordName_LTextureGenParameter_F());
		}
		private static UnlinkedGLFunction make_glTexGenfv() {
			return newUnlinkedGLFunction("glTexGenfv", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_p(), MLS_LTextureCoordName_LTextureGenParameter_p());
		}
		private static UnlinkedGLFunction make_glTexGenfvOES() {
			return newUnlinkedGLFunction("glTexGenfvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_p(), MLS_LTextureCoordName_LTextureGenParameter_p());
		}
		private static UnlinkedGLFunction make_glTexGeni() {
			return newUnlinkedGLFunction("glTexGeni", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_I(), MLS_LTextureCoordName_LTextureGenParameter_I());
		}
		private static UnlinkedGLFunction make_glTexGeniOES() {
			return newUnlinkedGLFunction("glTexGeniOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_I(), MLS_LTextureCoordName_LTextureGenParameter_I());
		}
		private static UnlinkedGLFunction make_glTexGeniv() {
			return newUnlinkedGLFunction("glTexGeniv", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_p(), MLS_LTextureCoordName_LTextureGenParameter_p());
		}
		private static UnlinkedGLFunction make_glTexGenivOES() {
			return newUnlinkedGLFunction("glTexGenivOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_p(), MLS_LTextureCoordName_LTextureGenParameter_p());
		}
		private static UnlinkedGLFunction make_glTexGenxOES() {
			return newUnlinkedGLFunction("glTexGenxOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_I(), MLS_LTextureCoordName_LTextureGenParameter_I());
		}
		private static UnlinkedGLFunction make_glTexGenxvOES() {
			return newUnlinkedGLFunction("glTexGenxvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureCoordName_LTextureGenParameter_p(), MLS_LTextureCoordName_LTextureGenParameter_p());
		}
		private static UnlinkedGLFunction make_glTexImage1D() {
			return newUnlinkedGLFunction("glTexImage1D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IILPixelFormat_LPixelType_p(), MLS_LTextureTarget_ILInternalFormat_IILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexImage2D() {
			return newUnlinkedGLFunction("glTexImage2D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_ILInternalFormat_IIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexImage2DMultisample() {
			return newUnlinkedGLFunction("glTexImage2DMultisample", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIZ(), MLS_LTextureTarget_ILInternalFormat_IIZ());
		}
		private static UnlinkedGLFunction make_glTexImage2DMultisampleCoverageNV() {
			return newUnlinkedGLFunction("glTexImage2DMultisampleCoverageNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IILInternalFormat_IIZ(), MLS_LTextureTarget_IILInternalFormat_IIZ());
		}
		private static UnlinkedGLFunction make_glTexImage3D() {
			return newUnlinkedGLFunction("glTexImage3D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_ILInternalFormat_IIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexImage3DEXT() {
			return newUnlinkedGLFunction("glTexImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_ILInternalFormat_IIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexImage3DMultisample() {
			return newUnlinkedGLFunction("glTexImage3DMultisample", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIZ(), MLS_LTextureTarget_ILInternalFormat_IIIZ());
		}
		private static UnlinkedGLFunction make_glTexImage3DMultisampleCoverageNV() {
			return newUnlinkedGLFunction("glTexImage3DMultisampleCoverageNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IILInternalFormat_IIIZ(), MLS_LTextureTarget_IILInternalFormat_IIIZ());
		}
		private static UnlinkedGLFunction make_glTexImage3DOES() {
			return newUnlinkedGLFunction("glTexImage3DOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_ILInternalFormat_IIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexImage4DSGIS() {
			return newUnlinkedGLFunction("glTexImage4DSGIS", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILInternalFormat_IIIIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_ILInternalFormat_IIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexPageCommitmentARB() {
			return newUnlinkedGLFunction("glTexPageCommitmentARB", void.class, GLParamTypeSets.ML_V(), PS_EIIIIIIIZ(), MLS_EIIIIIIIZ());
		}
		private static UnlinkedGLFunction make_glTexPageCommitmentEXT() {
			return newUnlinkedGLFunction("glTexPageCommitmentEXT", void.class, GLParamTypeSets.ML_V(), PS_EIIIIIIIZ(), MLS_EIIIIIIIZ());
		}
		private static UnlinkedGLFunction make_glTexPageCommitmentMemNV() {
			return newUnlinkedGLFunction("glTexPageCommitmentMemNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIIIIIJZ(), MLS_LTextureTarget_IIIIIIIIIJZ());
		}
		private static UnlinkedGLFunction make_glTexParameterIiv() {
			return newUnlinkedGLFunction("glTexParameterIiv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureParameterName_p(), MLS_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTexParameterIivEXT() {
			return newUnlinkedGLFunction("glTexParameterIivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureParameterName_p(), MLS_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTexParameterIivOES() {
			return newUnlinkedGLFunction("glTexParameterIivOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureParameterName_p(), MLS_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTexParameterIuiv() {
			return newUnlinkedGLFunction("glTexParameterIuiv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureParameterName_p(), MLS_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTexParameterIuivEXT() {
			return newUnlinkedGLFunction("glTexParameterIuivEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureParameterName_p(), MLS_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTexParameterIuivOES() {
			return newUnlinkedGLFunction("glTexParameterIuivOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureParameterName_p(), MLS_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTexParameterf() {
			return newUnlinkedGLFunction("glTexParameterf", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureParameterName_F(), MLS_LTextureTarget_LTextureParameterName_F());
		}
		private static UnlinkedGLFunction make_glTexParameterfv() {
			return newUnlinkedGLFunction("glTexParameterfv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureParameterName_p(), MLS_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTexParameteri() {
			return newUnlinkedGLFunction("glTexParameteri", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureParameterName_I(), MLS_LTextureTarget_LTextureParameterName_I());
		}
		private static UnlinkedGLFunction make_glTexParameteriv() {
			return newUnlinkedGLFunction("glTexParameteriv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LTextureParameterName_p(), MLS_LTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTexParameterx() {
			return newUnlinkedGLFunction("glTexParameterx", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_I(), MLS_LTextureTarget_LGetTextureParameter_I());
		}
		private static UnlinkedGLFunction make_glTexParameterxOES() {
			return newUnlinkedGLFunction("glTexParameterxOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_I(), MLS_LTextureTarget_LGetTextureParameter_I());
		}
		private static UnlinkedGLFunction make_glTexParameterxv() {
			return newUnlinkedGLFunction("glTexParameterxv", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_p(), MLS_LTextureTarget_LGetTextureParameter_p());
		}
		private static UnlinkedGLFunction make_glTexParameterxvOES() {
			return newUnlinkedGLFunction("glTexParameterxvOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LGetTextureParameter_p(), MLS_LTextureTarget_LGetTextureParameter_p());
		}
		private static UnlinkedGLFunction make_glTexRenderbufferNV() {
			return newUnlinkedGLFunction("glTexRenderbufferNV", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_I(), MLS_LTextureTarget_I());
		}
		private static UnlinkedGLFunction make_glTexStorage1D() {
			return newUnlinkedGLFunction("glTexStorage1D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_I(), MLS_LTextureTarget_ILSizedInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glTexStorage1DEXT() {
			return newUnlinkedGLFunction("glTexStorage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_I(), MLS_LTextureTarget_ILSizedInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glTexStorage2D() {
			return newUnlinkedGLFunction("glTexStorage2D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_II(), MLS_LTextureTarget_ILSizedInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glTexStorage2DEXT() {
			return newUnlinkedGLFunction("glTexStorage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_II(), MLS_LTextureTarget_ILSizedInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glTexStorage2DMultisample() {
			return newUnlinkedGLFunction("glTexStorage2DMultisample", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_IIZ(), MLS_LTextureTarget_ILSizedInternalFormat_IIZ());
		}
		private static UnlinkedGLFunction make_glTexStorage3D() {
			return newUnlinkedGLFunction("glTexStorage3D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_III(), MLS_LTextureTarget_ILSizedInternalFormat_III());
		}
		private static UnlinkedGLFunction make_glTexStorage3DEXT() {
			return newUnlinkedGLFunction("glTexStorage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_III(), MLS_LTextureTarget_ILSizedInternalFormat_III());
		}
		private static UnlinkedGLFunction make_glTexStorage3DMultisample() {
			return newUnlinkedGLFunction("glTexStorage3DMultisample", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_IIIZ(), MLS_LTextureTarget_ILSizedInternalFormat_IIIZ());
		}
		private static UnlinkedGLFunction make_glTexStorage3DMultisampleOES() {
			return newUnlinkedGLFunction("glTexStorage3DMultisampleOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_IIIZ(), MLS_LTextureTarget_ILSizedInternalFormat_IIIZ());
		}
		private static UnlinkedGLFunction make_glTexStorageAttribs2DEXT() {
			return newUnlinkedGLFunction("glTexStorageAttribs2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_IIp(), MLS_LTextureTarget_ILSizedInternalFormat_IIp());
		}
		private static UnlinkedGLFunction make_glTexStorageAttribs3DEXT() {
			return newUnlinkedGLFunction("glTexStorageAttribs3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_IIIp(), MLS_LTextureTarget_ILSizedInternalFormat_IIIp());
		}
		private static UnlinkedGLFunction make_glTexStorageMem1DEXT() {
			return newUnlinkedGLFunction("glTexStorageMem1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_IIJ(), MLS_LTextureTarget_ILSizedInternalFormat_IIJ());
		}
		private static UnlinkedGLFunction make_glTexStorageMem2DEXT() {
			return newUnlinkedGLFunction("glTexStorageMem2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_IIIJ(), MLS_LTextureTarget_ILSizedInternalFormat_IIIJ());
		}
		private static UnlinkedGLFunction make_glTexStorageMem2DMultisampleEXT() {
			return newUnlinkedGLFunction("glTexStorageMem2DMultisampleEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_IIZIJ(), MLS_LTextureTarget_ILSizedInternalFormat_IIZIJ());
		}
		private static UnlinkedGLFunction make_glTexStorageMem3DEXT() {
			return newUnlinkedGLFunction("glTexStorageMem3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_IIIIJ(), MLS_LTextureTarget_ILSizedInternalFormat_IIIIJ());
		}
		private static UnlinkedGLFunction make_glTexStorageMem3DMultisampleEXT() {
			return newUnlinkedGLFunction("glTexStorageMem3DMultisampleEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_ILSizedInternalFormat_IIIZIJ(), MLS_LTextureTarget_ILSizedInternalFormat_IIIZIJ());
		}
		private static UnlinkedGLFunction make_glTexStorageSparseAMD() {
			return newUnlinkedGLFunction("glTexStorageSparseAMD", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_LSizedInternalFormat_IIIII(), MLS_LTextureTarget_LSizedInternalFormat_IIIII());
		}
		private static UnlinkedGLFunction make_glTexSubImage1D() {
			return newUnlinkedGLFunction("glTexSubImage1D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_IIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexSubImage1DEXT() {
			return newUnlinkedGLFunction("glTexSubImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_IIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexSubImage2D() {
			return newUnlinkedGLFunction("glTexSubImage2D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_IIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexSubImage2DEXT() {
			return newUnlinkedGLFunction("glTexSubImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_IIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexSubImage3D() {
			return newUnlinkedGLFunction("glTexSubImage3D", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_IIIIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexSubImage3DEXT() {
			return newUnlinkedGLFunction("glTexSubImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_IIIIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexSubImage3DOES() {
			return newUnlinkedGLFunction("glTexSubImage3DOES", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_IIIIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTexSubImage4DSGIS() {
			return newUnlinkedGLFunction("glTexSubImage4DSGIS", void.class, GLParamTypeSets.ML_V(), PS_LTextureTarget_IIIIIIIIILPixelFormat_LPixelType_p(), MLS_LTextureTarget_IIIIIIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTextureAttachMemoryNV() {
			return newUnlinkedGLFunction("glTextureAttachMemoryNV", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glTextureBarrier() {
			return newUnlinkedGLFunction("glTextureBarrier", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glTextureBarrierNV() {
			return newUnlinkedGLFunction("glTextureBarrierNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glTextureBuffer() {
			return newUnlinkedGLFunction("glTextureBuffer", void.class, GLParamTypeSets.ML_V(), PS_ILSizedInternalFormat_I(), MLS_ILSizedInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glTextureBufferEXT() {
			return newUnlinkedGLFunction("glTextureBufferEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LSizedInternalFormat_I(), MLS_ILTextureTarget_LSizedInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glTextureBufferRange() {
			return newUnlinkedGLFunction("glTextureBufferRange", void.class, GLParamTypeSets.ML_V(), PS_ILSizedInternalFormat_IJJ(), MLS_ILSizedInternalFormat_IJJ());
		}
		private static UnlinkedGLFunction make_glTextureBufferRangeEXT() {
			return newUnlinkedGLFunction("glTextureBufferRangeEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LSizedInternalFormat_IJJ(), MLS_ILTextureTarget_LSizedInternalFormat_IJJ());
		}
		private static UnlinkedGLFunction make_glTextureColorMaskSGIS() {
			return newUnlinkedGLFunction("glTextureColorMaskSGIS", void.class, GLParamTypeSets.ML_V(), PS_ZZZZ(), MLS_ZZZZ());
		}
		private static UnlinkedGLFunction make_glTextureFoveationParametersQCOM() {
			return newUnlinkedGLFunction("glTextureFoveationParametersQCOM", void.class, GLParamTypeSets.ML_V(), PS_IIIFFFFF(), MLS_IIIFFFFF());
		}
		private static UnlinkedGLFunction make_glTextureImage1DEXT() {
			return newUnlinkedGLFunction("glTextureImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILInternalFormat_IILPixelFormat_LPixelType_p(), MLS_ILTextureTarget_ILInternalFormat_IILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTextureImage2DEXT() {
			return newUnlinkedGLFunction("glTextureImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILInternalFormat_IIILPixelFormat_LPixelType_p(), MLS_ILTextureTarget_ILInternalFormat_IIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTextureImage2DMultisampleCoverageNV() {
			return newUnlinkedGLFunction("glTextureImage2DMultisampleCoverageNV", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IILInternalFormat_IIZ(), MLS_ILTextureTarget_IILInternalFormat_IIZ());
		}
		private static UnlinkedGLFunction make_glTextureImage2DMultisampleNV() {
			return newUnlinkedGLFunction("glTextureImage2DMultisampleNV", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILInternalFormat_IIZ(), MLS_ILTextureTarget_ILInternalFormat_IIZ());
		}
		private static UnlinkedGLFunction make_glTextureImage3DEXT() {
			return newUnlinkedGLFunction("glTextureImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILInternalFormat_IIIILPixelFormat_LPixelType_p(), MLS_ILTextureTarget_ILInternalFormat_IIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTextureImage3DMultisampleCoverageNV() {
			return newUnlinkedGLFunction("glTextureImage3DMultisampleCoverageNV", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IILInternalFormat_IIIZ(), MLS_ILTextureTarget_IILInternalFormat_IIIZ());
		}
		private static UnlinkedGLFunction make_glTextureImage3DMultisampleNV() {
			return newUnlinkedGLFunction("glTextureImage3DMultisampleNV", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILInternalFormat_IIIZ(), MLS_ILTextureTarget_ILInternalFormat_IIIZ());
		}
		private static UnlinkedGLFunction make_glTextureLightEXT() {
			return newUnlinkedGLFunction("glTextureLightEXT", void.class, GLParamTypeSets.ML_V(), PS_LLightTexturePNameEXT_(), MLS_LLightTexturePNameEXT_());
		}
		private static UnlinkedGLFunction make_glTextureMaterialEXT() {
			return newUnlinkedGLFunction("glTextureMaterialEXT", void.class, GLParamTypeSets.ML_V(), PS_LTriangleFace_LMaterialParameter_(), MLS_LTriangleFace_LMaterialParameter_());
		}
		private static UnlinkedGLFunction make_glTextureNormalEXT() {
			return newUnlinkedGLFunction("glTextureNormalEXT", void.class, GLParamTypeSets.ML_V(), PS_LTextureNormalModeEXT_(), MLS_LTextureNormalModeEXT_());
		}
		private static UnlinkedGLFunction make_glTexturePageCommitmentEXT() {
			return newUnlinkedGLFunction("glTexturePageCommitmentEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIZ(), MLS_IIIIIIIIZ());
		}
		private static UnlinkedGLFunction make_glTexturePageCommitmentMemNV() {
			return newUnlinkedGLFunction("glTexturePageCommitmentMemNV", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIIIIJZ(), MLS_IIIIIIIIIIJZ());
		}
		private static UnlinkedGLFunction make_glTextureParameterIiv() {
			return newUnlinkedGLFunction("glTextureParameterIiv", void.class, GLParamTypeSets.ML_V(), PS_ILTextureParameterName_p(), MLS_ILTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTextureParameterIivEXT() {
			return newUnlinkedGLFunction("glTextureParameterIivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LTextureParameterName_p(), MLS_ILTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTextureParameterIuiv() {
			return newUnlinkedGLFunction("glTextureParameterIuiv", void.class, GLParamTypeSets.ML_V(), PS_ILTextureParameterName_p(), MLS_ILTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTextureParameterIuivEXT() {
			return newUnlinkedGLFunction("glTextureParameterIuivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LTextureParameterName_p(), MLS_ILTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTextureParameterf() {
			return newUnlinkedGLFunction("glTextureParameterf", void.class, GLParamTypeSets.ML_V(), PS_ILTextureParameterName_F(), MLS_ILTextureParameterName_F());
		}
		private static UnlinkedGLFunction make_glTextureParameterfEXT() {
			return newUnlinkedGLFunction("glTextureParameterfEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LTextureParameterName_F(), MLS_ILTextureTarget_LTextureParameterName_F());
		}
		private static UnlinkedGLFunction make_glTextureParameterfv() {
			return newUnlinkedGLFunction("glTextureParameterfv", void.class, GLParamTypeSets.ML_V(), PS_ILTextureParameterName_p(), MLS_ILTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTextureParameterfvEXT() {
			return newUnlinkedGLFunction("glTextureParameterfvEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LTextureParameterName_p(), MLS_ILTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTextureParameteri() {
			return newUnlinkedGLFunction("glTextureParameteri", void.class, GLParamTypeSets.ML_V(), PS_ILTextureParameterName_I(), MLS_ILTextureParameterName_I());
		}
		private static UnlinkedGLFunction make_glTextureParameteriEXT() {
			return newUnlinkedGLFunction("glTextureParameteriEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LTextureParameterName_I(), MLS_ILTextureTarget_LTextureParameterName_I());
		}
		private static UnlinkedGLFunction make_glTextureParameteriv() {
			return newUnlinkedGLFunction("glTextureParameteriv", void.class, GLParamTypeSets.ML_V(), PS_ILTextureParameterName_p(), MLS_ILTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTextureParameterivEXT() {
			return newUnlinkedGLFunction("glTextureParameterivEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_LTextureParameterName_p(), MLS_ILTextureTarget_LTextureParameterName_p());
		}
		private static UnlinkedGLFunction make_glTextureRangeAPPLE() {
			return newUnlinkedGLFunction("glTextureRangeAPPLE", void.class, GLParamTypeSets.ML_V(), PS_EIp(), MLS_EIp());
		}
		private static UnlinkedGLFunction make_glTextureRenderbufferEXT() {
			return newUnlinkedGLFunction("glTextureRenderbufferEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_I(), MLS_ILTextureTarget_I());
		}
		private static UnlinkedGLFunction make_glTextureStorage1D() {
			return newUnlinkedGLFunction("glTextureStorage1D", void.class, GLParamTypeSets.ML_V(), PS_IILSizedInternalFormat_I(), MLS_IILSizedInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glTextureStorage1DEXT() {
			return newUnlinkedGLFunction("glTextureStorage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_IEILSizedInternalFormat_I(), MLS_IEILSizedInternalFormat_I());
		}
		private static UnlinkedGLFunction make_glTextureStorage2D() {
			return newUnlinkedGLFunction("glTextureStorage2D", void.class, GLParamTypeSets.ML_V(), PS_IILSizedInternalFormat_II(), MLS_IILSizedInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glTextureStorage2DEXT() {
			return newUnlinkedGLFunction("glTextureStorage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_IEILSizedInternalFormat_II(), MLS_IEILSizedInternalFormat_II());
		}
		private static UnlinkedGLFunction make_glTextureStorage2DMultisample() {
			return newUnlinkedGLFunction("glTextureStorage2DMultisample", void.class, GLParamTypeSets.ML_V(), PS_IILSizedInternalFormat_IIZ(), MLS_IILSizedInternalFormat_IIZ());
		}
		private static UnlinkedGLFunction make_glTextureStorage2DMultisampleEXT() {
			return newUnlinkedGLFunction("glTextureStorage2DMultisampleEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILSizedInternalFormat_IIZ(), MLS_ILTextureTarget_ILSizedInternalFormat_IIZ());
		}
		private static UnlinkedGLFunction make_glTextureStorage3D() {
			return newUnlinkedGLFunction("glTextureStorage3D", void.class, GLParamTypeSets.ML_V(), PS_IILSizedInternalFormat_III(), MLS_IILSizedInternalFormat_III());
		}
		private static UnlinkedGLFunction make_glTextureStorage3DEXT() {
			return newUnlinkedGLFunction("glTextureStorage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_IEILSizedInternalFormat_III(), MLS_IEILSizedInternalFormat_III());
		}
		private static UnlinkedGLFunction make_glTextureStorage3DMultisample() {
			return newUnlinkedGLFunction("glTextureStorage3DMultisample", void.class, GLParamTypeSets.ML_V(), PS_IILSizedInternalFormat_IIIZ(), MLS_IILSizedInternalFormat_IIIZ());
		}
		private static UnlinkedGLFunction make_glTextureStorage3DMultisampleEXT() {
			return newUnlinkedGLFunction("glTextureStorage3DMultisampleEXT", void.class, GLParamTypeSets.ML_V(), PS_IEILSizedInternalFormat_IIIZ(), MLS_IEILSizedInternalFormat_IIIZ());
		}
		private static UnlinkedGLFunction make_glTextureStorageMem1DEXT() {
			return newUnlinkedGLFunction("glTextureStorageMem1DEXT", void.class, GLParamTypeSets.ML_V(), PS_IILSizedInternalFormat_IIJ(), MLS_IILSizedInternalFormat_IIJ());
		}
		private static UnlinkedGLFunction make_glTextureStorageMem2DEXT() {
			return newUnlinkedGLFunction("glTextureStorageMem2DEXT", void.class, GLParamTypeSets.ML_V(), PS_IILSizedInternalFormat_IIIJ(), MLS_IILSizedInternalFormat_IIIJ());
		}
		private static UnlinkedGLFunction make_glTextureStorageMem2DMultisampleEXT() {
			return newUnlinkedGLFunction("glTextureStorageMem2DMultisampleEXT", void.class, GLParamTypeSets.ML_V(), PS_IILSizedInternalFormat_IIZIJ(), MLS_IILSizedInternalFormat_IIZIJ());
		}
		private static UnlinkedGLFunction make_glTextureStorageMem3DEXT() {
			return newUnlinkedGLFunction("glTextureStorageMem3DEXT", void.class, GLParamTypeSets.ML_V(), PS_IILSizedInternalFormat_IIIIJ(), MLS_IILSizedInternalFormat_IIIIJ());
		}
		private static UnlinkedGLFunction make_glTextureStorageMem3DMultisampleEXT() {
			return newUnlinkedGLFunction("glTextureStorageMem3DMultisampleEXT", void.class, GLParamTypeSets.ML_V(), PS_IILSizedInternalFormat_IIIZIJ(), MLS_IILSizedInternalFormat_IIIZIJ());
		}
		private static UnlinkedGLFunction make_glTextureStorageSparseAMD() {
			return newUnlinkedGLFunction("glTextureStorageSparseAMD", void.class, GLParamTypeSets.ML_V(), PS_IELSizedInternalFormat_IIIII(), MLS_IELSizedInternalFormat_IIIII());
		}
		private static UnlinkedGLFunction make_glTextureSubImage1D() {
			return newUnlinkedGLFunction("glTextureSubImage1D", void.class, GLParamTypeSets.ML_V(), PS_IIIILPixelFormat_LPixelType_p(), MLS_IIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTextureSubImage1DEXT() {
			return newUnlinkedGLFunction("glTextureSubImage1DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IIILPixelFormat_LPixelType_p(), MLS_ILTextureTarget_IIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTextureSubImage2D() {
			return newUnlinkedGLFunction("glTextureSubImage2D", void.class, GLParamTypeSets.ML_V(), PS_IIIIIILPixelFormat_LPixelType_p(), MLS_IIIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTextureSubImage2DEXT() {
			return newUnlinkedGLFunction("glTextureSubImage2DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IIIIILPixelFormat_LPixelType_p(), MLS_ILTextureTarget_IIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTextureSubImage3D() {
			return newUnlinkedGLFunction("glTextureSubImage3D", void.class, GLParamTypeSets.ML_V(), PS_IIIIIIIILPixelFormat_LPixelType_p(), MLS_IIIIIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTextureSubImage3DEXT() {
			return newUnlinkedGLFunction("glTextureSubImage3DEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_IIIIIIILPixelFormat_LPixelType_p(), MLS_ILTextureTarget_IIIIIIILPixelFormat_LPixelType_p());
		}
		private static UnlinkedGLFunction make_glTextureView() {
			return newUnlinkedGLFunction("glTextureView", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILSizedInternalFormat_IIII(), MLS_ILTextureTarget_ILSizedInternalFormat_IIII());
		}
		private static UnlinkedGLFunction make_glTextureViewEXT() {
			return newUnlinkedGLFunction("glTextureViewEXT", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILSizedInternalFormat_IIII(), MLS_ILTextureTarget_ILSizedInternalFormat_IIII());
		}
		private static UnlinkedGLFunction make_glTextureViewOES() {
			return newUnlinkedGLFunction("glTextureViewOES", void.class, GLParamTypeSets.ML_V(), PS_ILTextureTarget_ILSizedInternalFormat_IIII(), MLS_ILTextureTarget_ILSizedInternalFormat_IIII());
		}
		private static UnlinkedGLFunction make_glTrackMatrixNV() {
			return newUnlinkedGLFunction("glTrackMatrixNV", void.class, GLParamTypeSets.ML_V(), PS_LVertexAttribEnumNV_ILVertexAttribEnumNV_LVertexAttribEnumNV_(), MLS_LVertexAttribEnumNV_ILVertexAttribEnumNV_LVertexAttribEnumNV_());
		}
		private static UnlinkedGLFunction make_glTransformFeedbackAttribsNV() {
			return newUnlinkedGLFunction("glTransformFeedbackAttribsNV", void.class, GLParamTypeSets.ML_V(), PS_IpE(), MLS_IpE());
		}
		private static UnlinkedGLFunction make_glTransformFeedbackBufferBase() {
			return newUnlinkedGLFunction("glTransformFeedbackBufferBase", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glTransformFeedbackBufferRange() {
			return newUnlinkedGLFunction("glTransformFeedbackBufferRange", void.class, GLParamTypeSets.ML_V(), PS_IIIJJ(), MLS_IIIJJ());
		}
		private static UnlinkedGLFunction make_glTransformFeedbackStreamAttribsNV() {
			return newUnlinkedGLFunction("glTransformFeedbackStreamAttribsNV", void.class, GLParamTypeSets.ML_V(), PS_IpIpE(), MLS_IpIpE());
		}
		private static UnlinkedGLFunction make_glTransformFeedbackVaryings() {
			return newUnlinkedGLFunction("glTransformFeedbackVaryings", void.class, GLParamTypeSets.ML_V(), PS_IIpLTransformFeedbackBufferMode_(), MLS_IIpLTransformFeedbackBufferMode_());
		}
		private static UnlinkedGLFunction make_glTransformFeedbackVaryingsEXT() {
			return newUnlinkedGLFunction("glTransformFeedbackVaryingsEXT", void.class, GLParamTypeSets.ML_V(), PS_IIpLTransformFeedbackBufferMode_(), MLS_IIpLTransformFeedbackBufferMode_());
		}
		private static UnlinkedGLFunction make_glTransformFeedbackVaryingsNV() {
			return newUnlinkedGLFunction("glTransformFeedbackVaryingsNV", void.class, GLParamTypeSets.ML_V(), PS_IIpLTransformFeedbackBufferMode_(), MLS_IIpLTransformFeedbackBufferMode_());
		}
		private static UnlinkedGLFunction make_glTransformPathNV() {
			return newUnlinkedGLFunction("glTransformPathNV", void.class, GLParamTypeSets.ML_V(), PS_IILPathTransformType_p(), MLS_IILPathTransformType_p());
		}
		private static UnlinkedGLFunction make_glTranslated() {
			return newUnlinkedGLFunction("glTranslated", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glTranslatef() {
			return newUnlinkedGLFunction("glTranslatef", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glTranslatex() {
			return newUnlinkedGLFunction("glTranslatex", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glTranslatexOES() {
			return newUnlinkedGLFunction("glTranslatexOES", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glUniform1d() {
			return newUnlinkedGLFunction("glUniform1d", void.class, GLParamTypeSets.ML_V(), PS_ID(), MLS_ID());
		}
		private static UnlinkedGLFunction make_glUniform1dv() {
			return newUnlinkedGLFunction("glUniform1dv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform1f() {
			return newUnlinkedGLFunction("glUniform1f", void.class, GLParamTypeSets.ML_V(), PS_IF(), MLS_IF());
		}
		private static UnlinkedGLFunction make_glUniform1fARB() {
			return newUnlinkedGLFunction("glUniform1fARB", void.class, GLParamTypeSets.ML_V(), PS_IF(), MLS_IF());
		}
		private static UnlinkedGLFunction make_glUniform1fv() {
			return newUnlinkedGLFunction("glUniform1fv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform1fvARB() {
			return newUnlinkedGLFunction("glUniform1fvARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform1i() {
			return newUnlinkedGLFunction("glUniform1i", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glUniform1i64ARB() {
			return newUnlinkedGLFunction("glUniform1i64ARB", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glUniform1i64NV() {
			return newUnlinkedGLFunction("glUniform1i64NV", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glUniform1i64vARB() {
			return newUnlinkedGLFunction("glUniform1i64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform1i64vNV() {
			return newUnlinkedGLFunction("glUniform1i64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform1iARB() {
			return newUnlinkedGLFunction("glUniform1iARB", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glUniform1iv() {
			return newUnlinkedGLFunction("glUniform1iv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform1ivARB() {
			return newUnlinkedGLFunction("glUniform1ivARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform1ui() {
			return newUnlinkedGLFunction("glUniform1ui", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glUniform1ui64ARB() {
			return newUnlinkedGLFunction("glUniform1ui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glUniform1ui64NV() {
			return newUnlinkedGLFunction("glUniform1ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glUniform1ui64vARB() {
			return newUnlinkedGLFunction("glUniform1ui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform1ui64vNV() {
			return newUnlinkedGLFunction("glUniform1ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform1uiEXT() {
			return newUnlinkedGLFunction("glUniform1uiEXT", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glUniform1uiv() {
			return newUnlinkedGLFunction("glUniform1uiv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform1uivEXT() {
			return newUnlinkedGLFunction("glUniform1uivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2d() {
			return newUnlinkedGLFunction("glUniform2d", void.class, GLParamTypeSets.ML_V(), PS_IDD(), MLS_IDD());
		}
		private static UnlinkedGLFunction make_glUniform2dv() {
			return newUnlinkedGLFunction("glUniform2dv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2f() {
			return newUnlinkedGLFunction("glUniform2f", void.class, GLParamTypeSets.ML_V(), PS_IFF(), MLS_IFF());
		}
		private static UnlinkedGLFunction make_glUniform2fARB() {
			return newUnlinkedGLFunction("glUniform2fARB", void.class, GLParamTypeSets.ML_V(), PS_IFF(), MLS_IFF());
		}
		private static UnlinkedGLFunction make_glUniform2fv() {
			return newUnlinkedGLFunction("glUniform2fv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2fvARB() {
			return newUnlinkedGLFunction("glUniform2fvARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2i() {
			return newUnlinkedGLFunction("glUniform2i", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glUniform2i64ARB() {
			return newUnlinkedGLFunction("glUniform2i64ARB", void.class, GLParamTypeSets.ML_V(), PS_IJJ(), MLS_IJJ());
		}
		private static UnlinkedGLFunction make_glUniform2i64NV() {
			return newUnlinkedGLFunction("glUniform2i64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJ(), MLS_IJJ());
		}
		private static UnlinkedGLFunction make_glUniform2i64vARB() {
			return newUnlinkedGLFunction("glUniform2i64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2i64vNV() {
			return newUnlinkedGLFunction("glUniform2i64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2iARB() {
			return newUnlinkedGLFunction("glUniform2iARB", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glUniform2iv() {
			return newUnlinkedGLFunction("glUniform2iv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2ivARB() {
			return newUnlinkedGLFunction("glUniform2ivARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2ui() {
			return newUnlinkedGLFunction("glUniform2ui", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glUniform2ui64ARB() {
			return newUnlinkedGLFunction("glUniform2ui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IJJ(), MLS_IJJ());
		}
		private static UnlinkedGLFunction make_glUniform2ui64NV() {
			return newUnlinkedGLFunction("glUniform2ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJ(), MLS_IJJ());
		}
		private static UnlinkedGLFunction make_glUniform2ui64vARB() {
			return newUnlinkedGLFunction("glUniform2ui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2ui64vNV() {
			return newUnlinkedGLFunction("glUniform2ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2uiEXT() {
			return newUnlinkedGLFunction("glUniform2uiEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glUniform2uiv() {
			return newUnlinkedGLFunction("glUniform2uiv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform2uivEXT() {
			return newUnlinkedGLFunction("glUniform2uivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3d() {
			return newUnlinkedGLFunction("glUniform3d", void.class, GLParamTypeSets.ML_V(), PS_IDDD(), MLS_IDDD());
		}
		private static UnlinkedGLFunction make_glUniform3dv() {
			return newUnlinkedGLFunction("glUniform3dv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3f() {
			return newUnlinkedGLFunction("glUniform3f", void.class, GLParamTypeSets.ML_V(), PS_IFFF(), MLS_IFFF());
		}
		private static UnlinkedGLFunction make_glUniform3fARB() {
			return newUnlinkedGLFunction("glUniform3fARB", void.class, GLParamTypeSets.ML_V(), PS_IFFF(), MLS_IFFF());
		}
		private static UnlinkedGLFunction make_glUniform3fv() {
			return newUnlinkedGLFunction("glUniform3fv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3fvARB() {
			return newUnlinkedGLFunction("glUniform3fvARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3i() {
			return newUnlinkedGLFunction("glUniform3i", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glUniform3i64ARB() {
			return newUnlinkedGLFunction("glUniform3i64ARB", void.class, GLParamTypeSets.ML_V(), PS_IJJJ(), MLS_IJJJ());
		}
		private static UnlinkedGLFunction make_glUniform3i64NV() {
			return newUnlinkedGLFunction("glUniform3i64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJJ(), MLS_IJJJ());
		}
		private static UnlinkedGLFunction make_glUniform3i64vARB() {
			return newUnlinkedGLFunction("glUniform3i64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3i64vNV() {
			return newUnlinkedGLFunction("glUniform3i64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3iARB() {
			return newUnlinkedGLFunction("glUniform3iARB", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glUniform3iv() {
			return newUnlinkedGLFunction("glUniform3iv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3ivARB() {
			return newUnlinkedGLFunction("glUniform3ivARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3ui() {
			return newUnlinkedGLFunction("glUniform3ui", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glUniform3ui64ARB() {
			return newUnlinkedGLFunction("glUniform3ui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IJJJ(), MLS_IJJJ());
		}
		private static UnlinkedGLFunction make_glUniform3ui64NV() {
			return newUnlinkedGLFunction("glUniform3ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJJ(), MLS_IJJJ());
		}
		private static UnlinkedGLFunction make_glUniform3ui64vARB() {
			return newUnlinkedGLFunction("glUniform3ui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3ui64vNV() {
			return newUnlinkedGLFunction("glUniform3ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3uiEXT() {
			return newUnlinkedGLFunction("glUniform3uiEXT", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glUniform3uiv() {
			return newUnlinkedGLFunction("glUniform3uiv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform3uivEXT() {
			return newUnlinkedGLFunction("glUniform3uivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4d() {
			return newUnlinkedGLFunction("glUniform4d", void.class, GLParamTypeSets.ML_V(), PS_IDDDD(), MLS_IDDDD());
		}
		private static UnlinkedGLFunction make_glUniform4dv() {
			return newUnlinkedGLFunction("glUniform4dv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4f() {
			return newUnlinkedGLFunction("glUniform4f", void.class, GLParamTypeSets.ML_V(), PS_IFFFF(), MLS_IFFFF());
		}
		private static UnlinkedGLFunction make_glUniform4fARB() {
			return newUnlinkedGLFunction("glUniform4fARB", void.class, GLParamTypeSets.ML_V(), PS_IFFFF(), MLS_IFFFF());
		}
		private static UnlinkedGLFunction make_glUniform4fv() {
			return newUnlinkedGLFunction("glUniform4fv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4fvARB() {
			return newUnlinkedGLFunction("glUniform4fvARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4i() {
			return newUnlinkedGLFunction("glUniform4i", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glUniform4i64ARB() {
			return newUnlinkedGLFunction("glUniform4i64ARB", void.class, GLParamTypeSets.ML_V(), PS_IJJJJ(), MLS_IJJJJ());
		}
		private static UnlinkedGLFunction make_glUniform4i64NV() {
			return newUnlinkedGLFunction("glUniform4i64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJJJ(), MLS_IJJJJ());
		}
		private static UnlinkedGLFunction make_glUniform4i64vARB() {
			return newUnlinkedGLFunction("glUniform4i64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4i64vNV() {
			return newUnlinkedGLFunction("glUniform4i64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4iARB() {
			return newUnlinkedGLFunction("glUniform4iARB", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glUniform4iv() {
			return newUnlinkedGLFunction("glUniform4iv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4ivARB() {
			return newUnlinkedGLFunction("glUniform4ivARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4ui() {
			return newUnlinkedGLFunction("glUniform4ui", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glUniform4ui64ARB() {
			return newUnlinkedGLFunction("glUniform4ui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IJJJJ(), MLS_IJJJJ());
		}
		private static UnlinkedGLFunction make_glUniform4ui64NV() {
			return newUnlinkedGLFunction("glUniform4ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJJJ(), MLS_IJJJJ());
		}
		private static UnlinkedGLFunction make_glUniform4ui64vARB() {
			return newUnlinkedGLFunction("glUniform4ui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4ui64vNV() {
			return newUnlinkedGLFunction("glUniform4ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4uiEXT() {
			return newUnlinkedGLFunction("glUniform4uiEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glUniform4uiv() {
			return newUnlinkedGLFunction("glUniform4uiv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniform4uivEXT() {
			return newUnlinkedGLFunction("glUniform4uivEXT", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniformBlockBinding() {
			return newUnlinkedGLFunction("glUniformBlockBinding", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glUniformBufferEXT() {
			return newUnlinkedGLFunction("glUniformBufferEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glUniformHandleui64ARB() {
			return newUnlinkedGLFunction("glUniformHandleui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glUniformHandleui64IMG() {
			return newUnlinkedGLFunction("glUniformHandleui64IMG", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glUniformHandleui64NV() {
			return newUnlinkedGLFunction("glUniformHandleui64NV", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glUniformHandleui64vARB() {
			return newUnlinkedGLFunction("glUniformHandleui64vARB", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniformHandleui64vIMG() {
			return newUnlinkedGLFunction("glUniformHandleui64vIMG", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniformHandleui64vNV() {
			return newUnlinkedGLFunction("glUniformHandleui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix2dv() {
			return newUnlinkedGLFunction("glUniformMatrix2dv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix2fv() {
			return newUnlinkedGLFunction("glUniformMatrix2fv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix2fvARB() {
			return newUnlinkedGLFunction("glUniformMatrix2fvARB", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix2x3dv() {
			return newUnlinkedGLFunction("glUniformMatrix2x3dv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix2x3fv() {
			return newUnlinkedGLFunction("glUniformMatrix2x3fv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix2x3fvNV() {
			return newUnlinkedGLFunction("glUniformMatrix2x3fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix2x4dv() {
			return newUnlinkedGLFunction("glUniformMatrix2x4dv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix2x4fv() {
			return newUnlinkedGLFunction("glUniformMatrix2x4fv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix2x4fvNV() {
			return newUnlinkedGLFunction("glUniformMatrix2x4fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix3dv() {
			return newUnlinkedGLFunction("glUniformMatrix3dv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix3fv() {
			return newUnlinkedGLFunction("glUniformMatrix3fv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix3fvARB() {
			return newUnlinkedGLFunction("glUniformMatrix3fvARB", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix3x2dv() {
			return newUnlinkedGLFunction("glUniformMatrix3x2dv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix3x2fv() {
			return newUnlinkedGLFunction("glUniformMatrix3x2fv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix3x2fvNV() {
			return newUnlinkedGLFunction("glUniformMatrix3x2fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix3x4dv() {
			return newUnlinkedGLFunction("glUniformMatrix3x4dv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix3x4fv() {
			return newUnlinkedGLFunction("glUniformMatrix3x4fv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix3x4fvNV() {
			return newUnlinkedGLFunction("glUniformMatrix3x4fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix4dv() {
			return newUnlinkedGLFunction("glUniformMatrix4dv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix4fv() {
			return newUnlinkedGLFunction("glUniformMatrix4fv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix4fvARB() {
			return newUnlinkedGLFunction("glUniformMatrix4fvARB", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix4x2dv() {
			return newUnlinkedGLFunction("glUniformMatrix4x2dv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix4x2fv() {
			return newUnlinkedGLFunction("glUniformMatrix4x2fv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix4x2fvNV() {
			return newUnlinkedGLFunction("glUniformMatrix4x2fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix4x3dv() {
			return newUnlinkedGLFunction("glUniformMatrix4x3dv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix4x3fv() {
			return newUnlinkedGLFunction("glUniformMatrix4x3fv", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformMatrix4x3fvNV() {
			return newUnlinkedGLFunction("glUniformMatrix4x3fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIZp(), MLS_IIZp());
		}
		private static UnlinkedGLFunction make_glUniformSubroutinesuiv() {
			return newUnlinkedGLFunction("glUniformSubroutinesuiv", void.class, GLParamTypeSets.ML_V(), PS_LShaderType_Ip(), MLS_LShaderType_Ip());
		}
		private static UnlinkedGLFunction make_glUniformui64NV() {
			return newUnlinkedGLFunction("glUniformui64NV", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glUniformui64vNV() {
			return newUnlinkedGLFunction("glUniformui64vNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glUnlockArraysEXT() {
			return newUnlinkedGLFunction("glUnlockArraysEXT", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glUnmapBuffer() {
			return newUnlinkedGLFunction("glUnmapBuffer", boolean.class, GLParamTypeSets.ML_Z(), PS_LBufferTarget_(), MLS_LBufferTarget_());
		}
		private static UnlinkedGLFunction make_glUnmapBufferARB() {
			return newUnlinkedGLFunction("glUnmapBufferARB", boolean.class, GLParamTypeSets.ML_Z(), PS_LBufferTarget_(), MLS_LBufferTarget_());
		}
		private static UnlinkedGLFunction make_glUnmapBufferOES() {
			return newUnlinkedGLFunction("glUnmapBufferOES", boolean.class, GLParamTypeSets.ML_Z(), PS_E(), MLS_E());
		}
		private static UnlinkedGLFunction make_glUnmapNamedBuffer() {
			return newUnlinkedGLFunction("glUnmapNamedBuffer", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glUnmapNamedBufferEXT() {
			return newUnlinkedGLFunction("glUnmapNamedBufferEXT", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glUnmapObjectBufferATI() {
			return newUnlinkedGLFunction("glUnmapObjectBufferATI", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glUnmapTexture2DINTEL() {
			return newUnlinkedGLFunction("glUnmapTexture2DINTEL", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glUpdateObjectBufferATI() {
			return newUnlinkedGLFunction("glUpdateObjectBufferATI", void.class, GLParamTypeSets.ML_V(), PS_IIIpLPreserveModeATI_(), MLS_IIIpLPreserveModeATI_());
		}
		private static UnlinkedGLFunction make_glUploadGpuMaskNVX() {
			return newUnlinkedGLFunction("glUploadGpuMaskNVX", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glUseProgram() {
			return newUnlinkedGLFunction("glUseProgram", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glUseProgramObjectARB() {
			return newUnlinkedGLFunction("glUseProgramObjectARB", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glUseProgramStages() {
			return newUnlinkedGLFunction("glUseProgramStages", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glUseProgramStagesEXT() {
			return newUnlinkedGLFunction("glUseProgramStagesEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glUseShaderProgramEXT() {
			return newUnlinkedGLFunction("glUseShaderProgramEXT", void.class, GLParamTypeSets.ML_V(), PS_EI(), MLS_EI());
		}
		private static UnlinkedGLFunction make_glVDPAUFiniNV() {
			return newUnlinkedGLFunction("glVDPAUFiniNV", void.class, GLParamTypeSets.ML_V(), PS_(), MLS_());
		}
		private static UnlinkedGLFunction make_glVDPAUGetSurfaceivNV() {
			return newUnlinkedGLFunction("glVDPAUGetSurfaceivNV", void.class, GLParamTypeSets.ML_V(), PS_IEIPP(), MLS_IEIPP());
		}
		private static UnlinkedGLFunction make_glVDPAUInitNV() {
			return newUnlinkedGLFunction("glVDPAUInitNV", void.class, GLParamTypeSets.ML_V(), PS_pp(), MLS_pp());
		}
		private static UnlinkedGLFunction make_glVDPAUIsSurfaceNV() {
			return newUnlinkedGLFunction("glVDPAUIsSurfaceNV", boolean.class, GLParamTypeSets.ML_Z(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glVDPAUMapSurfacesNV() {
			return newUnlinkedGLFunction("glVDPAUMapSurfacesNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVDPAURegisterOutputSurfaceNV() {
			return newUnlinkedGLFunction("glVDPAURegisterOutputSurfaceNV", int.class, GLParamTypeSets.ML_I(), PS_pEIp(), MLS_pEIp());
		}
		private static UnlinkedGLFunction make_glVDPAURegisterVideoSurfaceNV() {
			return newUnlinkedGLFunction("glVDPAURegisterVideoSurfaceNV", int.class, GLParamTypeSets.ML_I(), PS_pEIp(), MLS_pEIp());
		}
		private static UnlinkedGLFunction make_glVDPAURegisterVideoSurfaceWithPictureStructureNV() {
			return newUnlinkedGLFunction("glVDPAURegisterVideoSurfaceWithPictureStructureNV", int.class, GLParamTypeSets.ML_I(), PS_pEIpZ(), MLS_pEIpZ());
		}
		private static UnlinkedGLFunction make_glVDPAUSurfaceAccessNV() {
			return newUnlinkedGLFunction("glVDPAUSurfaceAccessNV", void.class, GLParamTypeSets.ML_V(), PS_IE(), MLS_IE());
		}
		private static UnlinkedGLFunction make_glVDPAUUnmapSurfacesNV() {
			return newUnlinkedGLFunction("glVDPAUUnmapSurfacesNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVDPAUUnregisterSurfaceNV() {
			return newUnlinkedGLFunction("glVDPAUUnregisterSurfaceNV", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glValidateProgram() {
			return newUnlinkedGLFunction("glValidateProgram", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glValidateProgramARB() {
			return newUnlinkedGLFunction("glValidateProgramARB", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glValidateProgramPipeline() {
			return newUnlinkedGLFunction("glValidateProgramPipeline", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glValidateProgramPipelineEXT() {
			return newUnlinkedGLFunction("glValidateProgramPipelineEXT", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glVariantArrayObjectATI() {
			return newUnlinkedGLFunction("glVariantArrayObjectATI", void.class, GLParamTypeSets.ML_V(), PS_ILScalarType_III(), MLS_ILScalarType_III());
		}
		private static UnlinkedGLFunction make_glVariantPointerEXT() {
			return newUnlinkedGLFunction("glVariantPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_ILScalarType_Ip(), MLS_ILScalarType_Ip());
		}
		private static UnlinkedGLFunction make_glVariantbvEXT() {
			return newUnlinkedGLFunction("glVariantbvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVariantdvEXT() {
			return newUnlinkedGLFunction("glVariantdvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVariantfvEXT() {
			return newUnlinkedGLFunction("glVariantfvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVariantivEXT() {
			return newUnlinkedGLFunction("glVariantivEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVariantsvEXT() {
			return newUnlinkedGLFunction("glVariantsvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVariantubvEXT() {
			return newUnlinkedGLFunction("glVariantubvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVariantuivEXT() {
			return newUnlinkedGLFunction("glVariantuivEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVariantusvEXT() {
			return newUnlinkedGLFunction("glVariantusvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertex2bOES() {
			return newUnlinkedGLFunction("glVertex2bOES", void.class, GLParamTypeSets.ML_V(), PS_BB(), MLS_BB());
		}
		private static UnlinkedGLFunction make_glVertex2bvOES() {
			return newUnlinkedGLFunction("glVertex2bvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex2d() {
			return newUnlinkedGLFunction("glVertex2d", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glVertex2dv() {
			return newUnlinkedGLFunction("glVertex2dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex2f() {
			return newUnlinkedGLFunction("glVertex2f", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glVertex2fv() {
			return newUnlinkedGLFunction("glVertex2fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex2hNV() {
			return newUnlinkedGLFunction("glVertex2hNV", void.class, GLParamTypeSets.ML_V(), PS_CC(), MLS_CC());
		}
		private static UnlinkedGLFunction make_glVertex2hvNV() {
			return newUnlinkedGLFunction("glVertex2hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex2i() {
			return newUnlinkedGLFunction("glVertex2i", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertex2iv() {
			return newUnlinkedGLFunction("glVertex2iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex2s() {
			return newUnlinkedGLFunction("glVertex2s", void.class, GLParamTypeSets.ML_V(), PS_SS(), MLS_SS());
		}
		private static UnlinkedGLFunction make_glVertex2sv() {
			return newUnlinkedGLFunction("glVertex2sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex2xOES() {
			return newUnlinkedGLFunction("glVertex2xOES", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glVertex2xvOES() {
			return newUnlinkedGLFunction("glVertex2xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex3bOES() {
			return newUnlinkedGLFunction("glVertex3bOES", void.class, GLParamTypeSets.ML_V(), PS_BBB(), MLS_BBB());
		}
		private static UnlinkedGLFunction make_glVertex3bvOES() {
			return newUnlinkedGLFunction("glVertex3bvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex3d() {
			return newUnlinkedGLFunction("glVertex3d", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glVertex3dv() {
			return newUnlinkedGLFunction("glVertex3dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex3f() {
			return newUnlinkedGLFunction("glVertex3f", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glVertex3fv() {
			return newUnlinkedGLFunction("glVertex3fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex3hNV() {
			return newUnlinkedGLFunction("glVertex3hNV", void.class, GLParamTypeSets.ML_V(), PS_CCC(), MLS_CCC());
		}
		private static UnlinkedGLFunction make_glVertex3hvNV() {
			return newUnlinkedGLFunction("glVertex3hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex3i() {
			return newUnlinkedGLFunction("glVertex3i", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertex3iv() {
			return newUnlinkedGLFunction("glVertex3iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex3s() {
			return newUnlinkedGLFunction("glVertex3s", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glVertex3sv() {
			return newUnlinkedGLFunction("glVertex3sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex3xOES() {
			return newUnlinkedGLFunction("glVertex3xOES", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertex3xvOES() {
			return newUnlinkedGLFunction("glVertex3xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex4bOES() {
			return newUnlinkedGLFunction("glVertex4bOES", void.class, GLParamTypeSets.ML_V(), PS_BBBB(), MLS_BBBB());
		}
		private static UnlinkedGLFunction make_glVertex4bvOES() {
			return newUnlinkedGLFunction("glVertex4bvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex4d() {
			return newUnlinkedGLFunction("glVertex4d", void.class, GLParamTypeSets.ML_V(), PS_DDDD(), MLS_DDDD());
		}
		private static UnlinkedGLFunction make_glVertex4dv() {
			return newUnlinkedGLFunction("glVertex4dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex4f() {
			return newUnlinkedGLFunction("glVertex4f", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glVertex4fv() {
			return newUnlinkedGLFunction("glVertex4fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex4hNV() {
			return newUnlinkedGLFunction("glVertex4hNV", void.class, GLParamTypeSets.ML_V(), PS_CCCC(), MLS_CCCC());
		}
		private static UnlinkedGLFunction make_glVertex4hvNV() {
			return newUnlinkedGLFunction("glVertex4hvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex4i() {
			return newUnlinkedGLFunction("glVertex4i", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glVertex4iv() {
			return newUnlinkedGLFunction("glVertex4iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex4s() {
			return newUnlinkedGLFunction("glVertex4s", void.class, GLParamTypeSets.ML_V(), PS_SSSS(), MLS_SSSS());
		}
		private static UnlinkedGLFunction make_glVertex4sv() {
			return newUnlinkedGLFunction("glVertex4sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertex4xOES() {
			return newUnlinkedGLFunction("glVertex4xOES", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertex4xvOES() {
			return newUnlinkedGLFunction("glVertex4xvOES", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertexArrayAttribBinding() {
			return newUnlinkedGLFunction("glVertexArrayAttribBinding", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertexArrayAttribFormat() {
			return newUnlinkedGLFunction("glVertexArrayAttribFormat", void.class, GLParamTypeSets.ML_V(), PS_IIILVertexAttribType_ZI(), MLS_IIILVertexAttribType_ZI());
		}
		private static UnlinkedGLFunction make_glVertexArrayAttribIFormat() {
			return newUnlinkedGLFunction("glVertexArrayAttribIFormat", void.class, GLParamTypeSets.ML_V(), PS_IIILVertexAttribIType_I(), MLS_IIILVertexAttribIType_I());
		}
		private static UnlinkedGLFunction make_glVertexArrayAttribLFormat() {
			return newUnlinkedGLFunction("glVertexArrayAttribLFormat", void.class, GLParamTypeSets.ML_V(), PS_IIILVertexAttribLType_I(), MLS_IIILVertexAttribLType_I());
		}
		private static UnlinkedGLFunction make_glVertexArrayBindVertexBufferEXT() {
			return newUnlinkedGLFunction("glVertexArrayBindVertexBufferEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIJI(), MLS_IIIJI());
		}
		private static UnlinkedGLFunction make_glVertexArrayBindingDivisor() {
			return newUnlinkedGLFunction("glVertexArrayBindingDivisor", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertexArrayColorOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayColorOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IIILColorPointerType_IJ(), MLS_IIILColorPointerType_IJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayEdgeFlagOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayEdgeFlagOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIJ(), MLS_IIIJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayElementBuffer() {
			return newUnlinkedGLFunction("glVertexArrayElementBuffer", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexArrayFogCoordOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayFogCoordOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IILFogCoordinatePointerType_IJ(), MLS_IILFogCoordinatePointerType_IJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayIndexOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayIndexOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IILIndexPointerType_IJ(), MLS_IILIndexPointerType_IJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayMultiTexCoordOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayMultiTexCoordOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IIEILTexCoordPointerType_IJ(), MLS_IIEILTexCoordPointerType_IJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayNormalOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayNormalOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IILNormalPointerType_IJ(), MLS_IILNormalPointerType_IJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayParameteriAPPLE() {
			return newUnlinkedGLFunction("glVertexArrayParameteriAPPLE", void.class, GLParamTypeSets.ML_V(), PS_LVertexArrayPNameAPPLE_I(), MLS_LVertexArrayPNameAPPLE_I());
		}
		private static UnlinkedGLFunction make_glVertexArrayRangeAPPLE() {
			return newUnlinkedGLFunction("glVertexArrayRangeAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IP(), MLS_IP());
		}
		private static UnlinkedGLFunction make_glVertexArrayRangeNV() {
			return newUnlinkedGLFunction("glVertexArrayRangeNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexArraySecondaryColorOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArraySecondaryColorOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IIILColorPointerType_IJ(), MLS_IIILColorPointerType_IJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayTexCoordOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayTexCoordOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IIILTexCoordPointerType_IJ(), MLS_IIILTexCoordPointerType_IJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexAttribBindingEXT() {
			return newUnlinkedGLFunction("glVertexArrayVertexAttribBindingEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexAttribDivisorEXT() {
			return newUnlinkedGLFunction("glVertexArrayVertexAttribDivisorEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexAttribFormatEXT() {
			return newUnlinkedGLFunction("glVertexArrayVertexAttribFormatEXT", void.class, GLParamTypeSets.ML_V(), PS_IIILVertexAttribType_ZI(), MLS_IIILVertexAttribType_ZI());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexAttribIFormatEXT() {
			return newUnlinkedGLFunction("glVertexArrayVertexAttribIFormatEXT", void.class, GLParamTypeSets.ML_V(), PS_IIILVertexAttribIType_I(), MLS_IIILVertexAttribIType_I());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexAttribIOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayVertexAttribIOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIILVertexAttribType_IJ(), MLS_IIIILVertexAttribType_IJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexAttribLFormatEXT() {
			return newUnlinkedGLFunction("glVertexArrayVertexAttribLFormatEXT", void.class, GLParamTypeSets.ML_V(), PS_IIILVertexAttribLType_I(), MLS_IIILVertexAttribLType_I());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexAttribLOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayVertexAttribLOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIILVertexAttribLType_IJ(), MLS_IIIILVertexAttribLType_IJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexAttribOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayVertexAttribOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIILVertexAttribPointerType_ZIJ(), MLS_IIIILVertexAttribPointerType_ZIJ());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexBindingDivisorEXT() {
			return newUnlinkedGLFunction("glVertexArrayVertexBindingDivisorEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexBuffer() {
			return newUnlinkedGLFunction("glVertexArrayVertexBuffer", void.class, GLParamTypeSets.ML_V(), PS_IIIJI(), MLS_IIIJI());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexBuffers() {
			return newUnlinkedGLFunction("glVertexArrayVertexBuffers", void.class, GLParamTypeSets.ML_V(), PS_IIIppp(), MLS_IIIppp());
		}
		private static UnlinkedGLFunction make_glVertexArrayVertexOffsetEXT() {
			return newUnlinkedGLFunction("glVertexArrayVertexOffsetEXT", void.class, GLParamTypeSets.ML_V(), PS_IIILVertexPointerType_IJ(), MLS_IIILVertexPointerType_IJ());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1d() {
			return newUnlinkedGLFunction("glVertexAttrib1d", void.class, GLParamTypeSets.ML_V(), PS_ID(), MLS_ID());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1dARB() {
			return newUnlinkedGLFunction("glVertexAttrib1dARB", void.class, GLParamTypeSets.ML_V(), PS_ID(), MLS_ID());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1dNV() {
			return newUnlinkedGLFunction("glVertexAttrib1dNV", void.class, GLParamTypeSets.ML_V(), PS_ID(), MLS_ID());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1dv() {
			return newUnlinkedGLFunction("glVertexAttrib1dv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1dvARB() {
			return newUnlinkedGLFunction("glVertexAttrib1dvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1dvNV() {
			return newUnlinkedGLFunction("glVertexAttrib1dvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1f() {
			return newUnlinkedGLFunction("glVertexAttrib1f", void.class, GLParamTypeSets.ML_V(), PS_IF(), MLS_IF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1fARB() {
			return newUnlinkedGLFunction("glVertexAttrib1fARB", void.class, GLParamTypeSets.ML_V(), PS_IF(), MLS_IF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1fNV() {
			return newUnlinkedGLFunction("glVertexAttrib1fNV", void.class, GLParamTypeSets.ML_V(), PS_IF(), MLS_IF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1fv() {
			return newUnlinkedGLFunction("glVertexAttrib1fv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1fvARB() {
			return newUnlinkedGLFunction("glVertexAttrib1fvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1fvNV() {
			return newUnlinkedGLFunction("glVertexAttrib1fvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1hNV() {
			return newUnlinkedGLFunction("glVertexAttrib1hNV", void.class, GLParamTypeSets.ML_V(), PS_IC(), MLS_IC());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1hvNV() {
			return newUnlinkedGLFunction("glVertexAttrib1hvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1s() {
			return newUnlinkedGLFunction("glVertexAttrib1s", void.class, GLParamTypeSets.ML_V(), PS_IS(), MLS_IS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1sARB() {
			return newUnlinkedGLFunction("glVertexAttrib1sARB", void.class, GLParamTypeSets.ML_V(), PS_IS(), MLS_IS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1sNV() {
			return newUnlinkedGLFunction("glVertexAttrib1sNV", void.class, GLParamTypeSets.ML_V(), PS_IS(), MLS_IS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1sv() {
			return newUnlinkedGLFunction("glVertexAttrib1sv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1svARB() {
			return newUnlinkedGLFunction("glVertexAttrib1svARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib1svNV() {
			return newUnlinkedGLFunction("glVertexAttrib1svNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2d() {
			return newUnlinkedGLFunction("glVertexAttrib2d", void.class, GLParamTypeSets.ML_V(), PS_IDD(), MLS_IDD());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2dARB() {
			return newUnlinkedGLFunction("glVertexAttrib2dARB", void.class, GLParamTypeSets.ML_V(), PS_IDD(), MLS_IDD());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2dNV() {
			return newUnlinkedGLFunction("glVertexAttrib2dNV", void.class, GLParamTypeSets.ML_V(), PS_IDD(), MLS_IDD());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2dv() {
			return newUnlinkedGLFunction("glVertexAttrib2dv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2dvARB() {
			return newUnlinkedGLFunction("glVertexAttrib2dvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2dvNV() {
			return newUnlinkedGLFunction("glVertexAttrib2dvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2f() {
			return newUnlinkedGLFunction("glVertexAttrib2f", void.class, GLParamTypeSets.ML_V(), PS_IFF(), MLS_IFF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2fARB() {
			return newUnlinkedGLFunction("glVertexAttrib2fARB", void.class, GLParamTypeSets.ML_V(), PS_IFF(), MLS_IFF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2fNV() {
			return newUnlinkedGLFunction("glVertexAttrib2fNV", void.class, GLParamTypeSets.ML_V(), PS_IFF(), MLS_IFF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2fv() {
			return newUnlinkedGLFunction("glVertexAttrib2fv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2fvARB() {
			return newUnlinkedGLFunction("glVertexAttrib2fvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2fvNV() {
			return newUnlinkedGLFunction("glVertexAttrib2fvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2hNV() {
			return newUnlinkedGLFunction("glVertexAttrib2hNV", void.class, GLParamTypeSets.ML_V(), PS_ICC(), MLS_ICC());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2hvNV() {
			return newUnlinkedGLFunction("glVertexAttrib2hvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2s() {
			return newUnlinkedGLFunction("glVertexAttrib2s", void.class, GLParamTypeSets.ML_V(), PS_ISS(), MLS_ISS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2sARB() {
			return newUnlinkedGLFunction("glVertexAttrib2sARB", void.class, GLParamTypeSets.ML_V(), PS_ISS(), MLS_ISS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2sNV() {
			return newUnlinkedGLFunction("glVertexAttrib2sNV", void.class, GLParamTypeSets.ML_V(), PS_ISS(), MLS_ISS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2sv() {
			return newUnlinkedGLFunction("glVertexAttrib2sv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2svARB() {
			return newUnlinkedGLFunction("glVertexAttrib2svARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib2svNV() {
			return newUnlinkedGLFunction("glVertexAttrib2svNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3d() {
			return newUnlinkedGLFunction("glVertexAttrib3d", void.class, GLParamTypeSets.ML_V(), PS_IDDD(), MLS_IDDD());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3dARB() {
			return newUnlinkedGLFunction("glVertexAttrib3dARB", void.class, GLParamTypeSets.ML_V(), PS_IDDD(), MLS_IDDD());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3dNV() {
			return newUnlinkedGLFunction("glVertexAttrib3dNV", void.class, GLParamTypeSets.ML_V(), PS_IDDD(), MLS_IDDD());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3dv() {
			return newUnlinkedGLFunction("glVertexAttrib3dv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3dvARB() {
			return newUnlinkedGLFunction("glVertexAttrib3dvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3dvNV() {
			return newUnlinkedGLFunction("glVertexAttrib3dvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3f() {
			return newUnlinkedGLFunction("glVertexAttrib3f", void.class, GLParamTypeSets.ML_V(), PS_IFFF(), MLS_IFFF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3fARB() {
			return newUnlinkedGLFunction("glVertexAttrib3fARB", void.class, GLParamTypeSets.ML_V(), PS_IFFF(), MLS_IFFF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3fNV() {
			return newUnlinkedGLFunction("glVertexAttrib3fNV", void.class, GLParamTypeSets.ML_V(), PS_IFFF(), MLS_IFFF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3fv() {
			return newUnlinkedGLFunction("glVertexAttrib3fv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3fvARB() {
			return newUnlinkedGLFunction("glVertexAttrib3fvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3fvNV() {
			return newUnlinkedGLFunction("glVertexAttrib3fvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3hNV() {
			return newUnlinkedGLFunction("glVertexAttrib3hNV", void.class, GLParamTypeSets.ML_V(), PS_ICCC(), MLS_ICCC());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3hvNV() {
			return newUnlinkedGLFunction("glVertexAttrib3hvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3s() {
			return newUnlinkedGLFunction("glVertexAttrib3s", void.class, GLParamTypeSets.ML_V(), PS_ISSS(), MLS_ISSS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3sARB() {
			return newUnlinkedGLFunction("glVertexAttrib3sARB", void.class, GLParamTypeSets.ML_V(), PS_ISSS(), MLS_ISSS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3sNV() {
			return newUnlinkedGLFunction("glVertexAttrib3sNV", void.class, GLParamTypeSets.ML_V(), PS_ISSS(), MLS_ISSS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3sv() {
			return newUnlinkedGLFunction("glVertexAttrib3sv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3svARB() {
			return newUnlinkedGLFunction("glVertexAttrib3svARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib3svNV() {
			return newUnlinkedGLFunction("glVertexAttrib3svNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4Nbv() {
			return newUnlinkedGLFunction("glVertexAttrib4Nbv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4NbvARB() {
			return newUnlinkedGLFunction("glVertexAttrib4NbvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4Niv() {
			return newUnlinkedGLFunction("glVertexAttrib4Niv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4NivARB() {
			return newUnlinkedGLFunction("glVertexAttrib4NivARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4Nsv() {
			return newUnlinkedGLFunction("glVertexAttrib4Nsv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4NsvARB() {
			return newUnlinkedGLFunction("glVertexAttrib4NsvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4Nub() {
			return newUnlinkedGLFunction("glVertexAttrib4Nub", void.class, GLParamTypeSets.ML_V(), PS_IBBBB(), MLS_IBBBB());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4NubARB() {
			return newUnlinkedGLFunction("glVertexAttrib4NubARB", void.class, GLParamTypeSets.ML_V(), PS_IBBBB(), MLS_IBBBB());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4Nubv() {
			return newUnlinkedGLFunction("glVertexAttrib4Nubv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4NubvARB() {
			return newUnlinkedGLFunction("glVertexAttrib4NubvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4Nuiv() {
			return newUnlinkedGLFunction("glVertexAttrib4Nuiv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4NuivARB() {
			return newUnlinkedGLFunction("glVertexAttrib4NuivARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4Nusv() {
			return newUnlinkedGLFunction("glVertexAttrib4Nusv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4NusvARB() {
			return newUnlinkedGLFunction("glVertexAttrib4NusvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4bv() {
			return newUnlinkedGLFunction("glVertexAttrib4bv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4bvARB() {
			return newUnlinkedGLFunction("glVertexAttrib4bvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4d() {
			return newUnlinkedGLFunction("glVertexAttrib4d", void.class, GLParamTypeSets.ML_V(), PS_IDDDD(), MLS_IDDDD());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4dARB() {
			return newUnlinkedGLFunction("glVertexAttrib4dARB", void.class, GLParamTypeSets.ML_V(), PS_IDDDD(), MLS_IDDDD());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4dNV() {
			return newUnlinkedGLFunction("glVertexAttrib4dNV", void.class, GLParamTypeSets.ML_V(), PS_IDDDD(), MLS_IDDDD());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4dv() {
			return newUnlinkedGLFunction("glVertexAttrib4dv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4dvARB() {
			return newUnlinkedGLFunction("glVertexAttrib4dvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4dvNV() {
			return newUnlinkedGLFunction("glVertexAttrib4dvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4f() {
			return newUnlinkedGLFunction("glVertexAttrib4f", void.class, GLParamTypeSets.ML_V(), PS_IFFFF(), MLS_IFFFF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4fARB() {
			return newUnlinkedGLFunction("glVertexAttrib4fARB", void.class, GLParamTypeSets.ML_V(), PS_IFFFF(), MLS_IFFFF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4fNV() {
			return newUnlinkedGLFunction("glVertexAttrib4fNV", void.class, GLParamTypeSets.ML_V(), PS_IFFFF(), MLS_IFFFF());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4fv() {
			return newUnlinkedGLFunction("glVertexAttrib4fv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4fvARB() {
			return newUnlinkedGLFunction("glVertexAttrib4fvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4fvNV() {
			return newUnlinkedGLFunction("glVertexAttrib4fvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4hNV() {
			return newUnlinkedGLFunction("glVertexAttrib4hNV", void.class, GLParamTypeSets.ML_V(), PS_ICCCC(), MLS_ICCCC());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4hvNV() {
			return newUnlinkedGLFunction("glVertexAttrib4hvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4iv() {
			return newUnlinkedGLFunction("glVertexAttrib4iv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4ivARB() {
			return newUnlinkedGLFunction("glVertexAttrib4ivARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4s() {
			return newUnlinkedGLFunction("glVertexAttrib4s", void.class, GLParamTypeSets.ML_V(), PS_ISSSS(), MLS_ISSSS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4sARB() {
			return newUnlinkedGLFunction("glVertexAttrib4sARB", void.class, GLParamTypeSets.ML_V(), PS_ISSSS(), MLS_ISSSS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4sNV() {
			return newUnlinkedGLFunction("glVertexAttrib4sNV", void.class, GLParamTypeSets.ML_V(), PS_ISSSS(), MLS_ISSSS());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4sv() {
			return newUnlinkedGLFunction("glVertexAttrib4sv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4svARB() {
			return newUnlinkedGLFunction("glVertexAttrib4svARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4svNV() {
			return newUnlinkedGLFunction("glVertexAttrib4svNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4ubNV() {
			return newUnlinkedGLFunction("glVertexAttrib4ubNV", void.class, GLParamTypeSets.ML_V(), PS_IBBBB(), MLS_IBBBB());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4ubv() {
			return newUnlinkedGLFunction("glVertexAttrib4ubv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4ubvARB() {
			return newUnlinkedGLFunction("glVertexAttrib4ubvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4ubvNV() {
			return newUnlinkedGLFunction("glVertexAttrib4ubvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4uiv() {
			return newUnlinkedGLFunction("glVertexAttrib4uiv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4uivARB() {
			return newUnlinkedGLFunction("glVertexAttrib4uivARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4usv() {
			return newUnlinkedGLFunction("glVertexAttrib4usv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttrib4usvARB() {
			return newUnlinkedGLFunction("glVertexAttrib4usvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribArrayObjectATI() {
			return newUnlinkedGLFunction("glVertexAttribArrayObjectATI", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribPointerType_ZIII(), MLS_IILVertexAttribPointerType_ZIII());
		}
		private static UnlinkedGLFunction make_glVertexAttribBinding() {
			return newUnlinkedGLFunction("glVertexAttribBinding", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexAttribDivisor() {
			return newUnlinkedGLFunction("glVertexAttribDivisor", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexAttribDivisorANGLE() {
			return newUnlinkedGLFunction("glVertexAttribDivisorANGLE", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexAttribDivisorARB() {
			return newUnlinkedGLFunction("glVertexAttribDivisorARB", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexAttribDivisorEXT() {
			return newUnlinkedGLFunction("glVertexAttribDivisorEXT", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexAttribDivisorNV() {
			return newUnlinkedGLFunction("glVertexAttribDivisorNV", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexAttribFormat() {
			return newUnlinkedGLFunction("glVertexAttribFormat", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribType_ZI(), MLS_IILVertexAttribType_ZI());
		}
		private static UnlinkedGLFunction make_glVertexAttribFormatNV() {
			return newUnlinkedGLFunction("glVertexAttribFormatNV", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribType_ZI(), MLS_IILVertexAttribType_ZI());
		}
		private static UnlinkedGLFunction make_glVertexAttribI1i() {
			return newUnlinkedGLFunction("glVertexAttribI1i", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexAttribI1iEXT() {
			return newUnlinkedGLFunction("glVertexAttribI1iEXT", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexAttribI1iv() {
			return newUnlinkedGLFunction("glVertexAttribI1iv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI1ivEXT() {
			return newUnlinkedGLFunction("glVertexAttribI1ivEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI1ui() {
			return newUnlinkedGLFunction("glVertexAttribI1ui", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexAttribI1uiEXT() {
			return newUnlinkedGLFunction("glVertexAttribI1uiEXT", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexAttribI1uiv() {
			return newUnlinkedGLFunction("glVertexAttribI1uiv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI1uivEXT() {
			return newUnlinkedGLFunction("glVertexAttribI1uivEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI2i() {
			return newUnlinkedGLFunction("glVertexAttribI2i", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertexAttribI2iEXT() {
			return newUnlinkedGLFunction("glVertexAttribI2iEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertexAttribI2iv() {
			return newUnlinkedGLFunction("glVertexAttribI2iv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI2ivEXT() {
			return newUnlinkedGLFunction("glVertexAttribI2ivEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI2ui() {
			return newUnlinkedGLFunction("glVertexAttribI2ui", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertexAttribI2uiEXT() {
			return newUnlinkedGLFunction("glVertexAttribI2uiEXT", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glVertexAttribI2uiv() {
			return newUnlinkedGLFunction("glVertexAttribI2uiv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI2uivEXT() {
			return newUnlinkedGLFunction("glVertexAttribI2uivEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI3i() {
			return newUnlinkedGLFunction("glVertexAttribI3i", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glVertexAttribI3iEXT() {
			return newUnlinkedGLFunction("glVertexAttribI3iEXT", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glVertexAttribI3iv() {
			return newUnlinkedGLFunction("glVertexAttribI3iv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI3ivEXT() {
			return newUnlinkedGLFunction("glVertexAttribI3ivEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI3ui() {
			return newUnlinkedGLFunction("glVertexAttribI3ui", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glVertexAttribI3uiEXT() {
			return newUnlinkedGLFunction("glVertexAttribI3uiEXT", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glVertexAttribI3uiv() {
			return newUnlinkedGLFunction("glVertexAttribI3uiv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI3uivEXT() {
			return newUnlinkedGLFunction("glVertexAttribI3uivEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4bv() {
			return newUnlinkedGLFunction("glVertexAttribI4bv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4bvEXT() {
			return newUnlinkedGLFunction("glVertexAttribI4bvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4i() {
			return newUnlinkedGLFunction("glVertexAttribI4i", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4iEXT() {
			return newUnlinkedGLFunction("glVertexAttribI4iEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4iv() {
			return newUnlinkedGLFunction("glVertexAttribI4iv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4ivEXT() {
			return newUnlinkedGLFunction("glVertexAttribI4ivEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4sv() {
			return newUnlinkedGLFunction("glVertexAttribI4sv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4svEXT() {
			return newUnlinkedGLFunction("glVertexAttribI4svEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4ubv() {
			return newUnlinkedGLFunction("glVertexAttribI4ubv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4ubvEXT() {
			return newUnlinkedGLFunction("glVertexAttribI4ubvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4ui() {
			return newUnlinkedGLFunction("glVertexAttribI4ui", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4uiEXT() {
			return newUnlinkedGLFunction("glVertexAttribI4uiEXT", void.class, GLParamTypeSets.ML_V(), PS_IIIII(), MLS_IIIII());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4uiv() {
			return newUnlinkedGLFunction("glVertexAttribI4uiv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4uivEXT() {
			return newUnlinkedGLFunction("glVertexAttribI4uivEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4usv() {
			return newUnlinkedGLFunction("glVertexAttribI4usv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribI4usvEXT() {
			return newUnlinkedGLFunction("glVertexAttribI4usvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribIFormat() {
			return newUnlinkedGLFunction("glVertexAttribIFormat", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribIType_I(), MLS_IILVertexAttribIType_I());
		}
		private static UnlinkedGLFunction make_glVertexAttribIFormatNV() {
			return newUnlinkedGLFunction("glVertexAttribIFormatNV", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribIType_I(), MLS_IILVertexAttribIType_I());
		}
		private static UnlinkedGLFunction make_glVertexAttribIPointer() {
			return newUnlinkedGLFunction("glVertexAttribIPointer", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribIType_Ip(), MLS_IILVertexAttribIType_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribIPointerEXT() {
			return newUnlinkedGLFunction("glVertexAttribIPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribIType_Ip(), MLS_IILVertexAttribIType_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL1d() {
			return newUnlinkedGLFunction("glVertexAttribL1d", void.class, GLParamTypeSets.ML_V(), PS_ID(), MLS_ID());
		}
		private static UnlinkedGLFunction make_glVertexAttribL1dEXT() {
			return newUnlinkedGLFunction("glVertexAttribL1dEXT", void.class, GLParamTypeSets.ML_V(), PS_ID(), MLS_ID());
		}
		private static UnlinkedGLFunction make_glVertexAttribL1dv() {
			return newUnlinkedGLFunction("glVertexAttribL1dv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL1dvEXT() {
			return newUnlinkedGLFunction("glVertexAttribL1dvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL1i64NV() {
			return newUnlinkedGLFunction("glVertexAttribL1i64NV", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glVertexAttribL1i64vNV() {
			return newUnlinkedGLFunction("glVertexAttribL1i64vNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL1ui64ARB() {
			return newUnlinkedGLFunction("glVertexAttribL1ui64ARB", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glVertexAttribL1ui64NV() {
			return newUnlinkedGLFunction("glVertexAttribL1ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IJ(), MLS_IJ());
		}
		private static UnlinkedGLFunction make_glVertexAttribL1ui64vARB() {
			return newUnlinkedGLFunction("glVertexAttribL1ui64vARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL1ui64vNV() {
			return newUnlinkedGLFunction("glVertexAttribL1ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL2d() {
			return newUnlinkedGLFunction("glVertexAttribL2d", void.class, GLParamTypeSets.ML_V(), PS_IDD(), MLS_IDD());
		}
		private static UnlinkedGLFunction make_glVertexAttribL2dEXT() {
			return newUnlinkedGLFunction("glVertexAttribL2dEXT", void.class, GLParamTypeSets.ML_V(), PS_IDD(), MLS_IDD());
		}
		private static UnlinkedGLFunction make_glVertexAttribL2dv() {
			return newUnlinkedGLFunction("glVertexAttribL2dv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL2dvEXT() {
			return newUnlinkedGLFunction("glVertexAttribL2dvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL2i64NV() {
			return newUnlinkedGLFunction("glVertexAttribL2i64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJ(), MLS_IJJ());
		}
		private static UnlinkedGLFunction make_glVertexAttribL2i64vNV() {
			return newUnlinkedGLFunction("glVertexAttribL2i64vNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL2ui64NV() {
			return newUnlinkedGLFunction("glVertexAttribL2ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJ(), MLS_IJJ());
		}
		private static UnlinkedGLFunction make_glVertexAttribL2ui64vNV() {
			return newUnlinkedGLFunction("glVertexAttribL2ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL3d() {
			return newUnlinkedGLFunction("glVertexAttribL3d", void.class, GLParamTypeSets.ML_V(), PS_IDDD(), MLS_IDDD());
		}
		private static UnlinkedGLFunction make_glVertexAttribL3dEXT() {
			return newUnlinkedGLFunction("glVertexAttribL3dEXT", void.class, GLParamTypeSets.ML_V(), PS_IDDD(), MLS_IDDD());
		}
		private static UnlinkedGLFunction make_glVertexAttribL3dv() {
			return newUnlinkedGLFunction("glVertexAttribL3dv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL3dvEXT() {
			return newUnlinkedGLFunction("glVertexAttribL3dvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL3i64NV() {
			return newUnlinkedGLFunction("glVertexAttribL3i64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJJ(), MLS_IJJJ());
		}
		private static UnlinkedGLFunction make_glVertexAttribL3i64vNV() {
			return newUnlinkedGLFunction("glVertexAttribL3i64vNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL3ui64NV() {
			return newUnlinkedGLFunction("glVertexAttribL3ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJJ(), MLS_IJJJ());
		}
		private static UnlinkedGLFunction make_glVertexAttribL3ui64vNV() {
			return newUnlinkedGLFunction("glVertexAttribL3ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL4d() {
			return newUnlinkedGLFunction("glVertexAttribL4d", void.class, GLParamTypeSets.ML_V(), PS_IDDDD(), MLS_IDDDD());
		}
		private static UnlinkedGLFunction make_glVertexAttribL4dEXT() {
			return newUnlinkedGLFunction("glVertexAttribL4dEXT", void.class, GLParamTypeSets.ML_V(), PS_IDDDD(), MLS_IDDDD());
		}
		private static UnlinkedGLFunction make_glVertexAttribL4dv() {
			return newUnlinkedGLFunction("glVertexAttribL4dv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL4dvEXT() {
			return newUnlinkedGLFunction("glVertexAttribL4dvEXT", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL4i64NV() {
			return newUnlinkedGLFunction("glVertexAttribL4i64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJJJ(), MLS_IJJJJ());
		}
		private static UnlinkedGLFunction make_glVertexAttribL4i64vNV() {
			return newUnlinkedGLFunction("glVertexAttribL4i64vNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribL4ui64NV() {
			return newUnlinkedGLFunction("glVertexAttribL4ui64NV", void.class, GLParamTypeSets.ML_V(), PS_IJJJJ(), MLS_IJJJJ());
		}
		private static UnlinkedGLFunction make_glVertexAttribL4ui64vNV() {
			return newUnlinkedGLFunction("glVertexAttribL4ui64vNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribLFormat() {
			return newUnlinkedGLFunction("glVertexAttribLFormat", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribLType_I(), MLS_IILVertexAttribLType_I());
		}
		private static UnlinkedGLFunction make_glVertexAttribLFormatNV() {
			return newUnlinkedGLFunction("glVertexAttribLFormatNV", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribLType_I(), MLS_IILVertexAttribLType_I());
		}
		private static UnlinkedGLFunction make_glVertexAttribLPointer() {
			return newUnlinkedGLFunction("glVertexAttribLPointer", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribLType_Ip(), MLS_IILVertexAttribLType_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribLPointerEXT() {
			return newUnlinkedGLFunction("glVertexAttribLPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribLType_Ip(), MLS_IILVertexAttribLType_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribP1ui() {
			return newUnlinkedGLFunction("glVertexAttribP1ui", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPointerType_ZI(), MLS_ILVertexAttribPointerType_ZI());
		}
		private static UnlinkedGLFunction make_glVertexAttribP1uiv() {
			return newUnlinkedGLFunction("glVertexAttribP1uiv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPointerType_Zp(), MLS_ILVertexAttribPointerType_Zp());
		}
		private static UnlinkedGLFunction make_glVertexAttribP2ui() {
			return newUnlinkedGLFunction("glVertexAttribP2ui", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPointerType_ZI(), MLS_ILVertexAttribPointerType_ZI());
		}
		private static UnlinkedGLFunction make_glVertexAttribP2uiv() {
			return newUnlinkedGLFunction("glVertexAttribP2uiv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPointerType_Zp(), MLS_ILVertexAttribPointerType_Zp());
		}
		private static UnlinkedGLFunction make_glVertexAttribP3ui() {
			return newUnlinkedGLFunction("glVertexAttribP3ui", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPointerType_ZI(), MLS_ILVertexAttribPointerType_ZI());
		}
		private static UnlinkedGLFunction make_glVertexAttribP3uiv() {
			return newUnlinkedGLFunction("glVertexAttribP3uiv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPointerType_Zp(), MLS_ILVertexAttribPointerType_Zp());
		}
		private static UnlinkedGLFunction make_glVertexAttribP4ui() {
			return newUnlinkedGLFunction("glVertexAttribP4ui", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPointerType_ZI(), MLS_ILVertexAttribPointerType_ZI());
		}
		private static UnlinkedGLFunction make_glVertexAttribP4uiv() {
			return newUnlinkedGLFunction("glVertexAttribP4uiv", void.class, GLParamTypeSets.ML_V(), PS_ILVertexAttribPointerType_Zp(), MLS_ILVertexAttribPointerType_Zp());
		}
		private static UnlinkedGLFunction make_glVertexAttribParameteriAMD() {
			return newUnlinkedGLFunction("glVertexAttribParameteriAMD", void.class, GLParamTypeSets.ML_V(), PS_IEI(), MLS_IEI());
		}
		private static UnlinkedGLFunction make_glVertexAttribPointer() {
			return newUnlinkedGLFunction("glVertexAttribPointer", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribPointerType_ZIp(), MLS_IILVertexAttribPointerType_ZIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribPointerARB() {
			return newUnlinkedGLFunction("glVertexAttribPointerARB", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribPointerType_ZIp(), MLS_IILVertexAttribPointerType_ZIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribPointerNV() {
			return newUnlinkedGLFunction("glVertexAttribPointerNV", void.class, GLParamTypeSets.ML_V(), PS_IILVertexAttribEnumNV_Ip(), MLS_IILVertexAttribEnumNV_Ip());
		}
		private static UnlinkedGLFunction make_glVertexAttribs1dvNV() {
			return newUnlinkedGLFunction("glVertexAttribs1dvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs1fvNV() {
			return newUnlinkedGLFunction("glVertexAttribs1fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs1hvNV() {
			return newUnlinkedGLFunction("glVertexAttribs1hvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs1svNV() {
			return newUnlinkedGLFunction("glVertexAttribs1svNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs2dvNV() {
			return newUnlinkedGLFunction("glVertexAttribs2dvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs2fvNV() {
			return newUnlinkedGLFunction("glVertexAttribs2fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs2hvNV() {
			return newUnlinkedGLFunction("glVertexAttribs2hvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs2svNV() {
			return newUnlinkedGLFunction("glVertexAttribs2svNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs3dvNV() {
			return newUnlinkedGLFunction("glVertexAttribs3dvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs3fvNV() {
			return newUnlinkedGLFunction("glVertexAttribs3fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs3hvNV() {
			return newUnlinkedGLFunction("glVertexAttribs3hvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs3svNV() {
			return newUnlinkedGLFunction("glVertexAttribs3svNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs4dvNV() {
			return newUnlinkedGLFunction("glVertexAttribs4dvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs4fvNV() {
			return newUnlinkedGLFunction("glVertexAttribs4fvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs4hvNV() {
			return newUnlinkedGLFunction("glVertexAttribs4hvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs4svNV() {
			return newUnlinkedGLFunction("glVertexAttribs4svNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexAttribs4ubvNV() {
			return newUnlinkedGLFunction("glVertexAttribs4ubvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glVertexBindingDivisor() {
			return newUnlinkedGLFunction("glVertexBindingDivisor", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glVertexBlendARB() {
			return newUnlinkedGLFunction("glVertexBlendARB", void.class, GLParamTypeSets.ML_V(), PS_I(), MLS_I());
		}
		private static UnlinkedGLFunction make_glVertexBlendEnvfATI() {
			return newUnlinkedGLFunction("glVertexBlendEnvfATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_F(), MLS_LVertexStreamATI_F());
		}
		private static UnlinkedGLFunction make_glVertexBlendEnviATI() {
			return newUnlinkedGLFunction("glVertexBlendEnviATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_I(), MLS_LVertexStreamATI_I());
		}
		private static UnlinkedGLFunction make_glVertexFormatNV() {
			return newUnlinkedGLFunction("glVertexFormatNV", void.class, GLParamTypeSets.ML_V(), PS_ILVertexPointerType_I(), MLS_ILVertexPointerType_I());
		}
		private static UnlinkedGLFunction make_glVertexP2ui() {
			return newUnlinkedGLFunction("glVertexP2ui", void.class, GLParamTypeSets.ML_V(), PS_LVertexPointerType_I(), MLS_LVertexPointerType_I());
		}
		private static UnlinkedGLFunction make_glVertexP2uiv() {
			return newUnlinkedGLFunction("glVertexP2uiv", void.class, GLParamTypeSets.ML_V(), PS_LVertexPointerType_p(), MLS_LVertexPointerType_p());
		}
		private static UnlinkedGLFunction make_glVertexP3ui() {
			return newUnlinkedGLFunction("glVertexP3ui", void.class, GLParamTypeSets.ML_V(), PS_LVertexPointerType_I(), MLS_LVertexPointerType_I());
		}
		private static UnlinkedGLFunction make_glVertexP3uiv() {
			return newUnlinkedGLFunction("glVertexP3uiv", void.class, GLParamTypeSets.ML_V(), PS_LVertexPointerType_p(), MLS_LVertexPointerType_p());
		}
		private static UnlinkedGLFunction make_glVertexP4ui() {
			return newUnlinkedGLFunction("glVertexP4ui", void.class, GLParamTypeSets.ML_V(), PS_LVertexPointerType_I(), MLS_LVertexPointerType_I());
		}
		private static UnlinkedGLFunction make_glVertexP4uiv() {
			return newUnlinkedGLFunction("glVertexP4uiv", void.class, GLParamTypeSets.ML_V(), PS_LVertexPointerType_p(), MLS_LVertexPointerType_p());
		}
		private static UnlinkedGLFunction make_glVertexPointer() {
			return newUnlinkedGLFunction("glVertexPointer", void.class, GLParamTypeSets.ML_V(), PS_ILVertexPointerType_Ip(), MLS_ILVertexPointerType_Ip());
		}
		private static UnlinkedGLFunction make_glVertexPointerEXT() {
			return newUnlinkedGLFunction("glVertexPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_ILVertexPointerType_IIp(), MLS_ILVertexPointerType_IIp());
		}
		private static UnlinkedGLFunction make_glVertexPointerListIBM() {
			return newUnlinkedGLFunction("glVertexPointerListIBM", void.class, GLParamTypeSets.ML_V(), PS_ILVertexPointerType_IPI(), MLS_ILVertexPointerType_IPI());
		}
		private static UnlinkedGLFunction make_glVertexPointervINTEL() {
			return newUnlinkedGLFunction("glVertexPointervINTEL", void.class, GLParamTypeSets.ML_V(), PS_ILVertexPointerType_P(), MLS_ILVertexPointerType_P());
		}
		private static UnlinkedGLFunction make_glVertexStream1dATI() {
			return newUnlinkedGLFunction("glVertexStream1dATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_D(), MLS_LVertexStreamATI_D());
		}
		private static UnlinkedGLFunction make_glVertexStream1dvATI() {
			return newUnlinkedGLFunction("glVertexStream1dvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream1fATI() {
			return newUnlinkedGLFunction("glVertexStream1fATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_F(), MLS_LVertexStreamATI_F());
		}
		private static UnlinkedGLFunction make_glVertexStream1fvATI() {
			return newUnlinkedGLFunction("glVertexStream1fvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream1iATI() {
			return newUnlinkedGLFunction("glVertexStream1iATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_I(), MLS_LVertexStreamATI_I());
		}
		private static UnlinkedGLFunction make_glVertexStream1ivATI() {
			return newUnlinkedGLFunction("glVertexStream1ivATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream1sATI() {
			return newUnlinkedGLFunction("glVertexStream1sATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_S(), MLS_LVertexStreamATI_S());
		}
		private static UnlinkedGLFunction make_glVertexStream1svATI() {
			return newUnlinkedGLFunction("glVertexStream1svATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream2dATI() {
			return newUnlinkedGLFunction("glVertexStream2dATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_DD(), MLS_LVertexStreamATI_DD());
		}
		private static UnlinkedGLFunction make_glVertexStream2dvATI() {
			return newUnlinkedGLFunction("glVertexStream2dvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream2fATI() {
			return newUnlinkedGLFunction("glVertexStream2fATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_FF(), MLS_LVertexStreamATI_FF());
		}
		private static UnlinkedGLFunction make_glVertexStream2fvATI() {
			return newUnlinkedGLFunction("glVertexStream2fvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream2iATI() {
			return newUnlinkedGLFunction("glVertexStream2iATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_II(), MLS_LVertexStreamATI_II());
		}
		private static UnlinkedGLFunction make_glVertexStream2ivATI() {
			return newUnlinkedGLFunction("glVertexStream2ivATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream2sATI() {
			return newUnlinkedGLFunction("glVertexStream2sATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_SS(), MLS_LVertexStreamATI_SS());
		}
		private static UnlinkedGLFunction make_glVertexStream2svATI() {
			return newUnlinkedGLFunction("glVertexStream2svATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream3dATI() {
			return newUnlinkedGLFunction("glVertexStream3dATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_DDD(), MLS_LVertexStreamATI_DDD());
		}
		private static UnlinkedGLFunction make_glVertexStream3dvATI() {
			return newUnlinkedGLFunction("glVertexStream3dvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream3fATI() {
			return newUnlinkedGLFunction("glVertexStream3fATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_FFF(), MLS_LVertexStreamATI_FFF());
		}
		private static UnlinkedGLFunction make_glVertexStream3fvATI() {
			return newUnlinkedGLFunction("glVertexStream3fvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream3iATI() {
			return newUnlinkedGLFunction("glVertexStream3iATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_III(), MLS_LVertexStreamATI_III());
		}
		private static UnlinkedGLFunction make_glVertexStream3ivATI() {
			return newUnlinkedGLFunction("glVertexStream3ivATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream3sATI() {
			return newUnlinkedGLFunction("glVertexStream3sATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_SSS(), MLS_LVertexStreamATI_SSS());
		}
		private static UnlinkedGLFunction make_glVertexStream3svATI() {
			return newUnlinkedGLFunction("glVertexStream3svATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream4dATI() {
			return newUnlinkedGLFunction("glVertexStream4dATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_DDDD(), MLS_LVertexStreamATI_DDDD());
		}
		private static UnlinkedGLFunction make_glVertexStream4dvATI() {
			return newUnlinkedGLFunction("glVertexStream4dvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream4fATI() {
			return newUnlinkedGLFunction("glVertexStream4fATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_FFFF(), MLS_LVertexStreamATI_FFFF());
		}
		private static UnlinkedGLFunction make_glVertexStream4fvATI() {
			return newUnlinkedGLFunction("glVertexStream4fvATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream4iATI() {
			return newUnlinkedGLFunction("glVertexStream4iATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_IIII(), MLS_LVertexStreamATI_IIII());
		}
		private static UnlinkedGLFunction make_glVertexStream4ivATI() {
			return newUnlinkedGLFunction("glVertexStream4ivATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexStream4sATI() {
			return newUnlinkedGLFunction("glVertexStream4sATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_SSSS(), MLS_LVertexStreamATI_SSSS());
		}
		private static UnlinkedGLFunction make_glVertexStream4svATI() {
			return newUnlinkedGLFunction("glVertexStream4svATI", void.class, GLParamTypeSets.ML_V(), PS_LVertexStreamATI_p(), MLS_LVertexStreamATI_p());
		}
		private static UnlinkedGLFunction make_glVertexWeightPointerEXT() {
			return newUnlinkedGLFunction("glVertexWeightPointerEXT", void.class, GLParamTypeSets.ML_V(), PS_ILVertexWeightPointerTypeEXT_Ip(), MLS_ILVertexWeightPointerTypeEXT_Ip());
		}
		private static UnlinkedGLFunction make_glVertexWeightfEXT() {
			return newUnlinkedGLFunction("glVertexWeightfEXT", void.class, GLParamTypeSets.ML_V(), PS_F(), MLS_F());
		}
		private static UnlinkedGLFunction make_glVertexWeightfvEXT() {
			return newUnlinkedGLFunction("glVertexWeightfvEXT", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVertexWeighthNV() {
			return newUnlinkedGLFunction("glVertexWeighthNV", void.class, GLParamTypeSets.ML_V(), PS_C(), MLS_C());
		}
		private static UnlinkedGLFunction make_glVertexWeighthvNV() {
			return newUnlinkedGLFunction("glVertexWeighthvNV", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glVideoCaptureNV() {
			return newUnlinkedGLFunction("glVideoCaptureNV", GLEnum.class, GLParamTypeSets.ML_E(), PS_IPP(), MLS_IPP());
		}
		private static UnlinkedGLFunction make_glVideoCaptureStreamParameterdvNV() {
			return newUnlinkedGLFunction("glVideoCaptureStreamParameterdvNV", void.class, GLParamTypeSets.ML_V(), PS_IIEp(), MLS_IIEp());
		}
		private static UnlinkedGLFunction make_glVideoCaptureStreamParameterfvNV() {
			return newUnlinkedGLFunction("glVideoCaptureStreamParameterfvNV", void.class, GLParamTypeSets.ML_V(), PS_IIEp(), MLS_IIEp());
		}
		private static UnlinkedGLFunction make_glVideoCaptureStreamParameterivNV() {
			return newUnlinkedGLFunction("glVideoCaptureStreamParameterivNV", void.class, GLParamTypeSets.ML_V(), PS_IIEp(), MLS_IIEp());
		}
		private static UnlinkedGLFunction make_glViewport() {
			return newUnlinkedGLFunction("glViewport", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glViewportArrayv() {
			return newUnlinkedGLFunction("glViewportArrayv", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glViewportArrayvNV() {
			return newUnlinkedGLFunction("glViewportArrayvNV", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glViewportArrayvOES() {
			return newUnlinkedGLFunction("glViewportArrayvOES", void.class, GLParamTypeSets.ML_V(), PS_IIp(), MLS_IIp());
		}
		private static UnlinkedGLFunction make_glViewportIndexedf() {
			return newUnlinkedGLFunction("glViewportIndexedf", void.class, GLParamTypeSets.ML_V(), PS_IFFFF(), MLS_IFFFF());
		}
		private static UnlinkedGLFunction make_glViewportIndexedfOES() {
			return newUnlinkedGLFunction("glViewportIndexedfOES", void.class, GLParamTypeSets.ML_V(), PS_IFFFF(), MLS_IFFFF());
		}
		private static UnlinkedGLFunction make_glViewportIndexedfNV() {
			return newUnlinkedGLFunction("glViewportIndexedfNV", void.class, GLParamTypeSets.ML_V(), PS_IFFFF(), MLS_IFFFF());
		}
		private static UnlinkedGLFunction make_glViewportIndexedfv() {
			return newUnlinkedGLFunction("glViewportIndexedfv", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glViewportIndexedfvOES() {
			return newUnlinkedGLFunction("glViewportIndexedfvOES", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glViewportIndexedfvNV() {
			return newUnlinkedGLFunction("glViewportIndexedfvNV", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glViewportPositionWScaleNV() {
			return newUnlinkedGLFunction("glViewportPositionWScaleNV", void.class, GLParamTypeSets.ML_V(), PS_IFF(), MLS_IFF());
		}
		private static UnlinkedGLFunction make_glViewportSwizzleNV() {
			return newUnlinkedGLFunction("glViewportSwizzleNV", void.class, GLParamTypeSets.ML_V(), PS_IEEEE(), MLS_IEEEE());
		}
		private static UnlinkedGLFunction make_glWaitSemaphoreEXT() {
			return newUnlinkedGLFunction("glWaitSemaphoreEXT", void.class, GLParamTypeSets.ML_V(), PS_IIpIpp(), MLS_IIpIpp());
		}
		private static UnlinkedGLFunction make_glWaitSemaphoreui64NVX() {
			return newUnlinkedGLFunction("glWaitSemaphoreui64NVX", void.class, GLParamTypeSets.ML_V(), PS_IIpp(), MLS_IIpp());
		}
		private static UnlinkedGLFunction make_glWaitSync() {
			return newUnlinkedGLFunction("glWaitSync", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glWaitSyncAPPLE() {
			return newUnlinkedGLFunction("glWaitSyncAPPLE", void.class, GLParamTypeSets.ML_V(), PS_IIJ(), MLS_IIJ());
		}
		private static UnlinkedGLFunction make_glWeightPathsNV() {
			return newUnlinkedGLFunction("glWeightPathsNV", void.class, GLParamTypeSets.ML_V(), PS_IIpp(), MLS_IIpp());
		}
		private static UnlinkedGLFunction make_glWeightPointerARB() {
			return newUnlinkedGLFunction("glWeightPointerARB", void.class, GLParamTypeSets.ML_V(), PS_ILWeightPointerTypeARB_Ip(), MLS_ILWeightPointerTypeARB_Ip());
		}
		private static UnlinkedGLFunction make_glWeightPointerOES() {
			return newUnlinkedGLFunction("glWeightPointerOES", void.class, GLParamTypeSets.ML_V(), PS_IEIp(), MLS_IEIp());
		}
		private static UnlinkedGLFunction make_glWeightbvARB() {
			return newUnlinkedGLFunction("glWeightbvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glWeightdvARB() {
			return newUnlinkedGLFunction("glWeightdvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glWeightfvARB() {
			return newUnlinkedGLFunction("glWeightfvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glWeightivARB() {
			return newUnlinkedGLFunction("glWeightivARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glWeightsvARB() {
			return newUnlinkedGLFunction("glWeightsvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glWeightubvARB() {
			return newUnlinkedGLFunction("glWeightubvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glWeightuivARB() {
			return newUnlinkedGLFunction("glWeightuivARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glWeightusvARB() {
			return newUnlinkedGLFunction("glWeightusvARB", void.class, GLParamTypeSets.ML_V(), PS_Ip(), MLS_Ip());
		}
		private static UnlinkedGLFunction make_glWindowPos2d() {
			return newUnlinkedGLFunction("glWindowPos2d", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glWindowPos2dARB() {
			return newUnlinkedGLFunction("glWindowPos2dARB", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glWindowPos2dMESA() {
			return newUnlinkedGLFunction("glWindowPos2dMESA", void.class, GLParamTypeSets.ML_V(), PS_DD(), MLS_DD());
		}
		private static UnlinkedGLFunction make_glWindowPos2dv() {
			return newUnlinkedGLFunction("glWindowPos2dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2dvARB() {
			return newUnlinkedGLFunction("glWindowPos2dvARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2dvMESA() {
			return newUnlinkedGLFunction("glWindowPos2dvMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2f() {
			return newUnlinkedGLFunction("glWindowPos2f", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glWindowPos2fARB() {
			return newUnlinkedGLFunction("glWindowPos2fARB", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glWindowPos2fMESA() {
			return newUnlinkedGLFunction("glWindowPos2fMESA", void.class, GLParamTypeSets.ML_V(), PS_FF(), MLS_FF());
		}
		private static UnlinkedGLFunction make_glWindowPos2fv() {
			return newUnlinkedGLFunction("glWindowPos2fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2fvARB() {
			return newUnlinkedGLFunction("glWindowPos2fvARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2fvMESA() {
			return newUnlinkedGLFunction("glWindowPos2fvMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2i() {
			return newUnlinkedGLFunction("glWindowPos2i", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glWindowPos2iARB() {
			return newUnlinkedGLFunction("glWindowPos2iARB", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glWindowPos2iMESA() {
			return newUnlinkedGLFunction("glWindowPos2iMESA", void.class, GLParamTypeSets.ML_V(), PS_II(), MLS_II());
		}
		private static UnlinkedGLFunction make_glWindowPos2iv() {
			return newUnlinkedGLFunction("glWindowPos2iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2ivARB() {
			return newUnlinkedGLFunction("glWindowPos2ivARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2ivMESA() {
			return newUnlinkedGLFunction("glWindowPos2ivMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2s() {
			return newUnlinkedGLFunction("glWindowPos2s", void.class, GLParamTypeSets.ML_V(), PS_SS(), MLS_SS());
		}
		private static UnlinkedGLFunction make_glWindowPos2sARB() {
			return newUnlinkedGLFunction("glWindowPos2sARB", void.class, GLParamTypeSets.ML_V(), PS_SS(), MLS_SS());
		}
		private static UnlinkedGLFunction make_glWindowPos2sMESA() {
			return newUnlinkedGLFunction("glWindowPos2sMESA", void.class, GLParamTypeSets.ML_V(), PS_SS(), MLS_SS());
		}
		private static UnlinkedGLFunction make_glWindowPos2sv() {
			return newUnlinkedGLFunction("glWindowPos2sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2svARB() {
			return newUnlinkedGLFunction("glWindowPos2svARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos2svMESA() {
			return newUnlinkedGLFunction("glWindowPos2svMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3d() {
			return newUnlinkedGLFunction("glWindowPos3d", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glWindowPos3dARB() {
			return newUnlinkedGLFunction("glWindowPos3dARB", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glWindowPos3dMESA() {
			return newUnlinkedGLFunction("glWindowPos3dMESA", void.class, GLParamTypeSets.ML_V(), PS_DDD(), MLS_DDD());
		}
		private static UnlinkedGLFunction make_glWindowPos3dv() {
			return newUnlinkedGLFunction("glWindowPos3dv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3dvARB() {
			return newUnlinkedGLFunction("glWindowPos3dvARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3dvMESA() {
			return newUnlinkedGLFunction("glWindowPos3dvMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3f() {
			return newUnlinkedGLFunction("glWindowPos3f", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glWindowPos3fARB() {
			return newUnlinkedGLFunction("glWindowPos3fARB", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glWindowPos3fMESA() {
			return newUnlinkedGLFunction("glWindowPos3fMESA", void.class, GLParamTypeSets.ML_V(), PS_FFF(), MLS_FFF());
		}
		private static UnlinkedGLFunction make_glWindowPos3fv() {
			return newUnlinkedGLFunction("glWindowPos3fv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3fvARB() {
			return newUnlinkedGLFunction("glWindowPos3fvARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3fvMESA() {
			return newUnlinkedGLFunction("glWindowPos3fvMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3i() {
			return newUnlinkedGLFunction("glWindowPos3i", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glWindowPos3iARB() {
			return newUnlinkedGLFunction("glWindowPos3iARB", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glWindowPos3iMESA() {
			return newUnlinkedGLFunction("glWindowPos3iMESA", void.class, GLParamTypeSets.ML_V(), PS_III(), MLS_III());
		}
		private static UnlinkedGLFunction make_glWindowPos3iv() {
			return newUnlinkedGLFunction("glWindowPos3iv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3ivARB() {
			return newUnlinkedGLFunction("glWindowPos3ivARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3ivMESA() {
			return newUnlinkedGLFunction("glWindowPos3ivMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3s() {
			return newUnlinkedGLFunction("glWindowPos3s", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glWindowPos3sARB() {
			return newUnlinkedGLFunction("glWindowPos3sARB", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glWindowPos3sMESA() {
			return newUnlinkedGLFunction("glWindowPos3sMESA", void.class, GLParamTypeSets.ML_V(), PS_SSS(), MLS_SSS());
		}
		private static UnlinkedGLFunction make_glWindowPos3sv() {
			return newUnlinkedGLFunction("glWindowPos3sv", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3svARB() {
			return newUnlinkedGLFunction("glWindowPos3svARB", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos3svMESA() {
			return newUnlinkedGLFunction("glWindowPos3svMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos4dMESA() {
			return newUnlinkedGLFunction("glWindowPos4dMESA", void.class, GLParamTypeSets.ML_V(), PS_DDDD(), MLS_DDDD());
		}
		private static UnlinkedGLFunction make_glWindowPos4dvMESA() {
			return newUnlinkedGLFunction("glWindowPos4dvMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos4fMESA() {
			return newUnlinkedGLFunction("glWindowPos4fMESA", void.class, GLParamTypeSets.ML_V(), PS_FFFF(), MLS_FFFF());
		}
		private static UnlinkedGLFunction make_glWindowPos4fvMESA() {
			return newUnlinkedGLFunction("glWindowPos4fvMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos4iMESA() {
			return newUnlinkedGLFunction("glWindowPos4iMESA", void.class, GLParamTypeSets.ML_V(), PS_IIII(), MLS_IIII());
		}
		private static UnlinkedGLFunction make_glWindowPos4ivMESA() {
			return newUnlinkedGLFunction("glWindowPos4ivMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowPos4sMESA() {
			return newUnlinkedGLFunction("glWindowPos4sMESA", void.class, GLParamTypeSets.ML_V(), PS_SSSS(), MLS_SSSS());
		}
		private static UnlinkedGLFunction make_glWindowPos4svMESA() {
			return newUnlinkedGLFunction("glWindowPos4svMESA", void.class, GLParamTypeSets.ML_V(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWindowRectanglesEXT() {
			return newUnlinkedGLFunction("glWindowRectanglesEXT", void.class, GLParamTypeSets.ML_V(), PS_EIp(), MLS_EIp());
		}
		private static UnlinkedGLFunction make_glWriteMaskEXT() {
			return newUnlinkedGLFunction("glWriteMaskEXT", void.class, GLParamTypeSets.ML_V(), PS_IILVertexShaderWriteMaskEXT_LVertexShaderWriteMaskEXT_LVertexShaderWriteMaskEXT_LVertexShaderWriteMaskEXT_(), MLS_IILVertexShaderWriteMaskEXT_LVertexShaderWriteMaskEXT_LVertexShaderWriteMaskEXT_LVertexShaderWriteMaskEXT_());
		}
		private static UnlinkedGLFunction make_glDrawVkImageNV() {
			return newUnlinkedGLFunction("glDrawVkImageNV", void.class, GLParamTypeSets.ML_V(), PS_JIFFFFFFFFF(), MLS_JIFFFFFFFFF());
		}
		private static UnlinkedGLFunction make_glGetVkProcAddrNV() {
			return newUnlinkedGLFunction("glGetVkProcAddrNV", int.class, GLParamTypeSets.ML_I(), PS_p(), MLS_p());
		}
		private static UnlinkedGLFunction make_glWaitVkSemaphoreNV() {
			return newUnlinkedGLFunction("glWaitVkSemaphoreNV", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glSignalVkSemaphoreNV() {
			return newUnlinkedGLFunction("glSignalVkSemaphoreNV", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glSignalVkFenceNV() {
			return newUnlinkedGLFunction("glSignalVkFenceNV", void.class, GLParamTypeSets.ML_V(), PS_J(), MLS_J());
		}
		private static UnlinkedGLFunction make_glFramebufferParameteriMESA() {
			return newUnlinkedGLFunction("glFramebufferParameteriMESA", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferParameterName_I(), MLS_LFramebufferTarget_LFramebufferParameterName_I());
		}
		private static UnlinkedGLFunction make_glGetFramebufferParameterivMESA() {
			return newUnlinkedGLFunction("glGetFramebufferParameterivMESA", void.class, GLParamTypeSets.ML_V(), PS_LFramebufferTarget_LFramebufferAttachmentParameterName_P(), MLS_LFramebufferTarget_LFramebufferAttachmentParameterName_P());
		}
		
		private static List<UnlinkedGLFunction> makeList() {
			return List.of(new UnlinkedGLFunction[]{glAccum, glAccumxOES, glActiveProgramEXT, glActiveShaderProgram, glActiveShaderProgramEXT, glActiveStencilFaceEXT, glActiveTexture, glActiveTextureARB, glActiveVaryingNV, glAlphaFragmentOp1ATI, glAlphaFragmentOp2ATI, glAlphaFragmentOp3ATI, glAlphaFunc, glAlphaFuncQCOM, glAlphaFuncx, glAlphaFuncxOES, glAlphaToCoverageDitherControlNV, glApplyFramebufferAttachmentCMAAINTEL, glApplyTextureEXT, glAcquireKeyedMutexWin32EXT, glAreProgramsResidentNV, glAreTexturesResident, glAreTexturesResidentEXT, glArrayElement, glArrayElementEXT, glArrayObjectATI, glAsyncCopyBufferSubDataNVX, glAsyncCopyImageSubDataNVX, glAsyncMarkerSGIX, glAttachObjectARB, glAttachShader, glBegin, glBeginConditionalRender, glBeginConditionalRenderNV, glBeginConditionalRenderNVX, glBeginFragmentShaderATI, glBeginOcclusionQueryNV, glBeginPerfMonitorAMD, glBeginPerfQueryINTEL, glBeginQuery, glBeginQueryARB, glBeginQueryEXT, glBeginQueryIndexed, glBeginTransformFeedback, glBeginTransformFeedbackEXT, glBeginTransformFeedbackNV, glBeginVertexShaderEXT, glBeginVideoCaptureNV, glBindAttribLocation, glBindAttribLocationARB, glBindBuffer, glBindBufferARB, glBindBufferBase, glBindBufferBaseEXT, glBindBufferBaseNV, glBindBufferOffsetEXT, glBindBufferOffsetNV, glBindBufferRange, glBindBufferRangeEXT, glBindBufferRangeNV, glBindBuffersBase, glBindBuffersRange, glBindFragDataLocation, glBindFragDataLocationEXT, glBindFragDataLocationIndexed, glBindFragDataLocationIndexedEXT, glBindFragmentShaderATI, glBindFramebuffer, glBindFramebufferEXT, glBindFramebufferOES, glBindImageTexture, glBindImageTextureEXT, glBindImageTextures, glBindLightParameterEXT, glBindMaterialParameterEXT, glBindMultiTextureEXT, glBindParameterEXT, glBindProgramARB, glBindProgramNV, glBindProgramPipeline, glBindProgramPipelineEXT, glBindRenderbuffer, glBindRenderbufferEXT, glBindRenderbufferOES, glBindSampler, glBindSamplers, glBindShadingRateImageNV, glBindTexGenParameterEXT, glBindTexture, glBindTextureEXT, glBindTextureUnit, glBindTextureUnitParameterEXT, glBindTextures, glBindTransformFeedback, glBindTransformFeedbackNV, glBindVertexArray, glBindVertexArrayAPPLE, glBindVertexArrayOES, glBindVertexBuffer, glBindVertexBuffers, glBindVertexShaderEXT, glBindVideoCaptureStreamBufferNV, glBindVideoCaptureStreamTextureNV, glBinormal3bEXT, glBinormal3bvEXT, glBinormal3dEXT, glBinormal3dvEXT, glBinormal3fEXT, glBinormal3fvEXT, glBinormal3iEXT, glBinormal3ivEXT, glBinormal3sEXT, glBinormal3svEXT, glBinormalPointerEXT, glBitmap, glBitmapxOES, glBlendBarrier, glBlendBarrierKHR, glBlendBarrierNV, glBlendColor, glBlendColorEXT, glBlendColorxOES, glBlendEquation, glBlendEquationEXT, glBlendEquationIndexedAMD, glBlendEquationOES, glBlendEquationSeparate, glBlendEquationSeparateEXT, glBlendEquationSeparateIndexedAMD, glBlendEquationSeparateOES, glBlendEquationSeparatei, glBlendEquationSeparateiARB, glBlendEquationSeparateiEXT, glBlendEquationSeparateiOES, glBlendEquationi, glBlendEquationiARB, glBlendEquationiEXT, glBlendEquationiOES, glBlendFunc, glBlendFuncIndexedAMD, glBlendFuncSeparate, glBlendFuncSeparateEXT, glBlendFuncSeparateINGR, glBlendFuncSeparateIndexedAMD, glBlendFuncSeparateOES, glBlendFuncSeparatei, glBlendFuncSeparateiARB, glBlendFuncSeparateiEXT, glBlendFuncSeparateiOES, glBlendFunci, glBlendFunciARB, glBlendFunciEXT, glBlendFunciOES, glBlendParameteriNV, glBlitFramebuffer, glBlitFramebufferANGLE, glBlitFramebufferEXT, glBlitFramebufferLayerEXT, glBlitFramebufferLayersEXT, glBlitFramebufferNV, glBlitNamedFramebuffer, glBufferAddressRangeNV, glBufferAttachMemoryNV, glBufferData, glBufferDataARB, glBufferPageCommitmentARB, glBufferPageCommitmentMemNV, glBufferParameteriAPPLE, glBufferStorage, glBufferStorageEXT, glBufferStorageExternalEXT, glBufferStorageMemEXT, glBufferSubData, glBufferSubDataARB, glCallCommandListNV, glCallList, glCallLists, glCheckFramebufferStatus, glCheckFramebufferStatusEXT, glCheckFramebufferStatusOES, glCheckNamedFramebufferStatus, glCheckNamedFramebufferStatusEXT, glClampColor, glClampColorARB, glClear, glClearAccum, glClearAccumxOES, glClearBufferData, glClearBufferSubData, glClearBufferfi, glClearBufferfv, glClearBufferiv, glClearBufferuiv, glClearColor, glClearColorIiEXT, glClearColorIuiEXT, glClearColorx, glClearColorxOES, glClearDepth, glClearDepthdNV, glClearDepthf, glClearDepthfOES, glClearDepthx, glClearDepthxOES, glClearIndex, glClearNamedBufferData, glClearNamedBufferDataEXT, glClearNamedBufferSubData, glClearNamedBufferSubDataEXT, glClearNamedFramebufferfi, glClearNamedFramebufferfv, glClearNamedFramebufferiv, glClearNamedFramebufferuiv, glClearPixelLocalStorageuiEXT, glClearStencil, glClearTexImage, glClearTexImageEXT, glClearTexSubImage, glClearTexSubImageEXT, glClientActiveTexture, glClientActiveTextureARB, glClientActiveVertexStreamATI, glClientAttribDefaultEXT, glClientWaitSemaphoreui64NVX, glClientWaitSync, glClientWaitSyncAPPLE, glClipControl, glClipControlEXT, glClipPlane, glClipPlanef, glClipPlanefIMG, glClipPlanefOES, glClipPlanex, glClipPlanexIMG, glClipPlanexOES, glColor3b, glColor3bv, glColor3d, glColor3dv, glColor3f, glColor3fVertex3fSUN, glColor3fVertex3fvSUN, glColor3fv, glColor3hNV, glColor3hvNV, glColor3i, glColor3iv, glColor3s, glColor3sv, glColor3ub, glColor3ubv, glColor3ui, glColor3uiv, glColor3us, glColor3usv, glColor3xOES, glColor3xvOES, glColor4b, glColor4bv, glColor4d, glColor4dv, glColor4f, glColor4fNormal3fVertex3fSUN, glColor4fNormal3fVertex3fvSUN, glColor4fv, glColor4hNV, glColor4hvNV, glColor4i, glColor4iv, glColor4s, glColor4sv, glColor4ub, glColor4ubVertex2fSUN, glColor4ubVertex2fvSUN, glColor4ubVertex3fSUN, glColor4ubVertex3fvSUN, glColor4ubv, glColor4ui, glColor4uiv, glColor4us, glColor4usv, glColor4x, glColor4xOES, glColor4xvOES, glColorFormatNV, glColorFragmentOp1ATI, glColorFragmentOp2ATI, glColorFragmentOp3ATI, glColorMask, glColorMaskIndexedEXT, glColorMaski, glColorMaskiEXT, glColorMaskiOES, glColorMaterial, glColorP3ui, glColorP3uiv, glColorP4ui, glColorP4uiv, glColorPointer, glColorPointerEXT, glColorPointerListIBM, glColorPointervINTEL, glColorSubTable, glColorSubTableEXT, glColorTable, glColorTableEXT, glColorTableParameterfv, glColorTableParameterfvSGI, glColorTableParameteriv, glColorTableParameterivSGI, glColorTableSGI, glCombinerInputNV, glCombinerOutputNV, glCombinerParameterfNV, glCombinerParameterfvNV, glCombinerParameteriNV, glCombinerParameterivNV, glCombinerStageParameterfvNV, glCommandListSegmentsNV, glCompileCommandListNV, glCompileShader, glCompileShaderARB, glCompileShaderIncludeARB, glCompressedMultiTexImage1DEXT, glCompressedMultiTexImage2DEXT, glCompressedMultiTexImage3DEXT, glCompressedMultiTexSubImage1DEXT, glCompressedMultiTexSubImage2DEXT, glCompressedMultiTexSubImage3DEXT, glCompressedTexImage1D, glCompressedTexImage1DARB, glCompressedTexImage2D, glCompressedTexImage2DARB, glCompressedTexImage3D, glCompressedTexImage3DARB, glCompressedTexImage3DOES, glCompressedTexSubImage1D, glCompressedTexSubImage1DARB, glCompressedTexSubImage2D, glCompressedTexSubImage2DARB, glCompressedTexSubImage3D, glCompressedTexSubImage3DARB, glCompressedTexSubImage3DOES, glCompressedTextureImage1DEXT, glCompressedTextureImage2DEXT, glCompressedTextureImage3DEXT, glCompressedTextureSubImage1D, glCompressedTextureSubImage1DEXT, glCompressedTextureSubImage2D, glCompressedTextureSubImage2DEXT, glCompressedTextureSubImage3D, glCompressedTextureSubImage3DEXT, glConservativeRasterParameterfNV, glConservativeRasterParameteriNV, glConvolutionFilter1D, glConvolutionFilter1DEXT, glConvolutionFilter2D, glConvolutionFilter2DEXT, glConvolutionParameterf, glConvolutionParameterfEXT, glConvolutionParameterfv, glConvolutionParameterfvEXT, glConvolutionParameteri, glConvolutionParameteriEXT, glConvolutionParameteriv, glConvolutionParameterivEXT, glConvolutionParameterxOES, glConvolutionParameterxvOES, glCopyBufferSubData, glCopyBufferSubDataNV, glCopyColorSubTable, glCopyColorSubTableEXT, glCopyColorTable, glCopyColorTableSGI, glCopyConvolutionFilter1D, glCopyConvolutionFilter1DEXT, glCopyConvolutionFilter2D, glCopyConvolutionFilter2DEXT, glCopyImageSubData, glCopyImageSubDataEXT, glCopyImageSubDataNV, glCopyImageSubDataOES, glCopyMultiTexImage1DEXT, glCopyMultiTexImage2DEXT, glCopyMultiTexSubImage1DEXT, glCopyMultiTexSubImage2DEXT, glCopyMultiTexSubImage3DEXT, glCopyNamedBufferSubData, glCopyPathNV, glCopyPixels, glCopyTexImage1D, glCopyTexImage1DEXT, glCopyTexImage2D, glCopyTexImage2DEXT, glCopyTexSubImage1D, glCopyTexSubImage1DEXT, glCopyTexSubImage2D, glCopyTexSubImage2DEXT, glCopyTexSubImage3D, glCopyTexSubImage3DEXT, glCopyTexSubImage3DOES, glCopyTextureImage1DEXT, glCopyTextureImage2DEXT, glCopyTextureLevelsAPPLE, glCopyTextureSubImage1D, glCopyTextureSubImage1DEXT, glCopyTextureSubImage2D, glCopyTextureSubImage2DEXT, glCopyTextureSubImage3D, glCopyTextureSubImage3DEXT, glCoverFillPathInstancedNV, glCoverFillPathNV, glCoverStrokePathInstancedNV, glCoverStrokePathNV, glCoverageMaskNV, glCoverageModulationNV, glCoverageModulationTableNV, glCoverageOperationNV, glCreateBuffers, glCreateCommandListsNV, glCreateFramebuffers, glCreateMemoryObjectsEXT, glCreatePerfQueryINTEL, glCreateProgram, glCreateProgramObjectARB, glCreateProgramPipelines, glCreateProgressFenceNVX, glCreateQueries, glCreateRenderbuffers, glCreateSamplers, glCreateSemaphoresNV, glCreateShader, glCreateShaderObjectARB, glCreateShaderProgramEXT, glCreateShaderProgramv, glCreateShaderProgramvEXT, glCreateStatesNV, glCreateSyncFromCLeventARB, glCreateTextures, glCreateTransformFeedbacks, glCreateVertexArrays, glCullFace, glCullParameterdvEXT, glCullParameterfvEXT, glCurrentPaletteMatrixARB, glCurrentPaletteMatrixOES, glDebugMessageCallback, glDebugMessageCallbackAMD, glDebugMessageCallbackARB, glDebugMessageCallbackKHR, glDebugMessageControl, glDebugMessageControlARB, glDebugMessageControlKHR, glDebugMessageEnableAMD, glDebugMessageInsert, glDebugMessageInsertAMD, glDebugMessageInsertARB, glDebugMessageInsertKHR, glDeformSGIX, glDeformationMap3dSGIX, glDeformationMap3fSGIX, glDeleteAsyncMarkersSGIX, glDeleteBuffers, glDeleteBuffersARB, glDeleteCommandListsNV, glDeleteFencesAPPLE, glDeleteFencesNV, glDeleteFragmentShaderATI, glDeleteFramebuffers, glDeleteFramebuffersEXT, glDeleteFramebuffersOES, glDeleteLists, glDeleteMemoryObjectsEXT, glDeleteNamedStringARB, glDeleteNamesAMD, glDeleteObjectARB, glDeleteOcclusionQueriesNV, glDeletePathsNV, glDeletePerfMonitorsAMD, glDeletePerfQueryINTEL, glDeleteProgram, glDeleteProgramPipelines, glDeleteProgramPipelinesEXT, glDeleteProgramsARB, glDeleteProgramsNV, glDeleteQueries, glDeleteQueriesARB, glDeleteQueriesEXT, glDeleteQueryResourceTagNV, glDeleteRenderbuffers, glDeleteRenderbuffersEXT, glDeleteRenderbuffersOES, glDeleteSamplers, glDeleteSemaphoresEXT, glDeleteShader, glDeleteStatesNV, glDeleteSync, glDeleteSyncAPPLE, glDeleteTextures, glDeleteTexturesEXT, glDeleteTransformFeedbacks, glDeleteTransformFeedbacksNV, glDeleteVertexArrays, glDeleteVertexArraysAPPLE, glDeleteVertexArraysOES, glDeleteVertexShaderEXT, glDepthBoundsEXT, glDepthBoundsdNV, glDepthFunc, glDepthMask, glDepthRange, glDepthRangeArraydvNV, glDepthRangeArrayfvNV, glDepthRangeArrayfvOES, glDepthRangeArrayv, glDepthRangeIndexed, glDepthRangeIndexeddNV, glDepthRangeIndexedfNV, glDepthRangeIndexedfOES, glDepthRangedNV, glDepthRangef, glDepthRangefOES, glDepthRangex, glDepthRangexOES, glDetachObjectARB, glDetachShader, glDetailTexFuncSGIS, glDisable, glDisableClientState, glDisableClientStateIndexedEXT, glDisableClientStateiEXT, glDisableDriverControlQCOM, glDisableIndexedEXT, glDisableVariantClientStateEXT, glDisableVertexArrayAttrib, glDisableVertexArrayAttribEXT, glDisableVertexArrayEXT, glDisableVertexAttribAPPLE, glDisableVertexAttribArray, glDisableVertexAttribArrayARB, glDisablei, glDisableiEXT, glDisableiNV, glDisableiOES, glDiscardFramebufferEXT, glDispatchCompute, glDispatchComputeGroupSizeARB, glDispatchComputeIndirect, glDrawArrays, glDrawArraysEXT, glDrawArraysIndirect, glDrawArraysInstanced, glDrawArraysInstancedANGLE, glDrawArraysInstancedARB, glDrawArraysInstancedBaseInstance, glDrawArraysInstancedBaseInstanceEXT, glDrawArraysInstancedEXT, glDrawArraysInstancedNV, glDrawBuffer, glDrawBuffers, glDrawBuffersARB, glDrawBuffersATI, glDrawBuffersEXT, glDrawBuffersIndexedEXT, glDrawBuffersNV, glDrawCommandsAddressNV, glDrawCommandsNV, glDrawCommandsStatesAddressNV, glDrawCommandsStatesNV, glDrawElementArrayAPPLE, glDrawElementArrayATI, glDrawElements, glDrawElementsBaseVertex, glDrawElementsBaseVertexEXT, glDrawElementsBaseVertexOES, glDrawElementsIndirect, glDrawElementsInstanced, glDrawElementsInstancedANGLE, glDrawElementsInstancedARB, glDrawElementsInstancedBaseInstance, glDrawElementsInstancedBaseInstanceEXT, glDrawElementsInstancedBaseVertex, glDrawElementsInstancedBaseVertexBaseInstance, glDrawElementsInstancedBaseVertexBaseInstanceEXT, glDrawElementsInstancedBaseVertexEXT, glDrawElementsInstancedBaseVertexOES, glDrawElementsInstancedEXT, glDrawElementsInstancedNV, glDrawMeshArraysSUN, glDrawMeshTasksNV, glDrawMeshTasksIndirectNV, glDrawPixels, glDrawRangeElementArrayAPPLE, glDrawRangeElementArrayATI, glDrawRangeElements, glDrawRangeElementsBaseVertex, glDrawRangeElementsBaseVertexEXT, glDrawRangeElementsBaseVertexOES, glDrawRangeElementsEXT, glDrawTexfOES, glDrawTexfvOES, glDrawTexiOES, glDrawTexivOES, glDrawTexsOES, glDrawTexsvOES, glDrawTextureNV, glDrawTexxOES, glDrawTexxvOES, glDrawTransformFeedback, glDrawTransformFeedbackEXT, glDrawTransformFeedbackInstanced, glDrawTransformFeedbackInstancedEXT, glDrawTransformFeedbackNV, glDrawTransformFeedbackStream, glDrawTransformFeedbackStreamInstanced, glEGLImageTargetRenderbufferStorageOES, glEGLImageTargetTexStorageEXT, glEGLImageTargetTexture2DOES, glEGLImageTargetTextureStorageEXT, glEdgeFlag, glEdgeFlagFormatNV, glEdgeFlagPointer, glEdgeFlagPointerEXT, glEdgeFlagPointerListIBM, glEdgeFlagv, glElementPointerAPPLE, glElementPointerATI, glEnable, glEnableClientState, glEnableClientStateIndexedEXT, glEnableClientStateiEXT, glEnableDriverControlQCOM, glEnableIndexedEXT, glEnableVariantClientStateEXT, glEnableVertexArrayAttrib, glEnableVertexArrayAttribEXT, glEnableVertexArrayEXT, glEnableVertexAttribAPPLE, glEnableVertexAttribArray, glEnableVertexAttribArrayARB, glEnablei, glEnableiEXT, glEnableiNV, glEnableiOES, glEnd, glEndConditionalRender, glEndConditionalRenderNV, glEndConditionalRenderNVX, glEndFragmentShaderATI, glEndList, glEndOcclusionQueryNV, glEndPerfMonitorAMD, glEndPerfQueryINTEL, glEndQuery, glEndQueryARB, glEndQueryEXT, glEndQueryIndexed, glEndTilingQCOM, glEndTransformFeedback, glEndTransformFeedbackEXT, glEndTransformFeedbackNV, glEndVertexShaderEXT, glEndVideoCaptureNV, glEvalCoord1d, glEvalCoord1dv, glEvalCoord1f, glEvalCoord1fv, glEvalCoord1xOES, glEvalCoord1xvOES, glEvalCoord2d, glEvalCoord2dv, glEvalCoord2f, glEvalCoord2fv, glEvalCoord2xOES, glEvalCoord2xvOES, glEvalMapsNV, glEvalMesh1, glEvalMesh2, glEvalPoint1, glEvalPoint2, glEvaluateDepthValuesARB, glExecuteProgramNV, glExtGetBufferPointervQCOM, glExtGetBuffersQCOM, glExtGetFramebuffersQCOM, glExtGetProgramBinarySourceQCOM, glExtGetProgramsQCOM, glExtGetRenderbuffersQCOM, glExtGetShadersQCOM, glExtGetTexLevelParameterivQCOM, glExtGetTexSubImageQCOM, glExtGetTexturesQCOM, glExtIsProgramBinaryQCOM, glExtTexObjectStateOverrideiQCOM, glExtractComponentEXT, glFeedbackBuffer, glFeedbackBufferxOES, glFenceSync, glFenceSyncAPPLE, glFinalCombinerInputNV, glFinish, glFinishAsyncSGIX, glFinishFenceAPPLE, glFinishFenceNV, glFinishObjectAPPLE, glFinishTextureSUNX, glFlush, glFlushMappedBufferRange, glFlushMappedBufferRangeAPPLE, glFlushMappedBufferRangeEXT, glFlushMappedNamedBufferRange, glFlushMappedNamedBufferRangeEXT, glFlushPixelDataRangeNV, glFlushRasterSGIX, glFlushStaticDataIBM, glFlushVertexArrayRangeAPPLE, glFlushVertexArrayRangeNV, glFogCoordFormatNV, glFogCoordPointer, glFogCoordPointerEXT, glFogCoordPointerListIBM, glFogCoordd, glFogCoorddEXT, glFogCoorddv, glFogCoorddvEXT, glFogCoordf, glFogCoordfEXT, glFogCoordfv, glFogCoordfvEXT, glFogCoordhNV, glFogCoordhvNV, glFogFuncSGIS, glFogf, glFogfv, glFogi, glFogiv, glFogx, glFogxOES, glFogxv, glFogxvOES, glFragmentColorMaterialSGIX, glFragmentCoverageColorNV, glFragmentLightModelfSGIX, glFragmentLightModelfvSGIX, glFragmentLightModeliSGIX, glFragmentLightModelivSGIX, glFragmentLightfSGIX, glFragmentLightfvSGIX, glFragmentLightiSGIX, glFragmentLightivSGIX, glFragmentMaterialfSGIX, glFragmentMaterialfvSGIX, glFragmentMaterialiSGIX, glFragmentMaterialivSGIX, glFrameTerminatorGREMEDY, glFrameZoomSGIX, glFramebufferDrawBufferEXT, glFramebufferDrawBuffersEXT, glFramebufferFetchBarrierEXT, glFramebufferFetchBarrierQCOM, glFramebufferFoveationConfigQCOM, glFramebufferFoveationParametersQCOM, glFramebufferParameteri, glFramebufferPixelLocalStorageSizeEXT, glFramebufferReadBufferEXT, glFramebufferRenderbuffer, glFramebufferRenderbufferEXT, glFramebufferRenderbufferOES, glFramebufferSampleLocationsfvARB, glFramebufferSampleLocationsfvNV, glFramebufferSamplePositionsfvAMD, glFramebufferShadingRateEXT, glFramebufferTexture, glFramebufferTexture1D, glFramebufferTexture1DEXT, glFramebufferTexture2D, glFramebufferTexture2DEXT, glFramebufferTexture2DDownsampleIMG, glFramebufferTexture2DMultisampleEXT, glFramebufferTexture2DMultisampleIMG, glFramebufferTexture2DOES, glFramebufferTexture3D, glFramebufferTexture3DEXT, glFramebufferTexture3DOES, glFramebufferTextureARB, glFramebufferTextureEXT, glFramebufferTextureFaceARB, glFramebufferTextureFaceEXT, glFramebufferTextureLayer, glFramebufferTextureLayerARB, glFramebufferTextureLayerEXT, glFramebufferTextureLayerDownsampleIMG, glFramebufferTextureMultisampleMultiviewOVR, glFramebufferTextureMultiviewOVR, glFramebufferTextureOES, glFreeObjectBufferATI, glFrontFace, glFrustum, glFrustumf, glFrustumfOES, glFrustumx, glFrustumxOES, glGenAsyncMarkersSGIX, glGenBuffers, glGenBuffersARB, glGenFencesAPPLE, glGenFencesNV, glGenFragmentShadersATI, glGenFramebuffers, glGenFramebuffersEXT, glGenFramebuffersOES, glGenLists, glGenNamesAMD, glGenOcclusionQueriesNV, glGenPathsNV, glGenPerfMonitorsAMD, glGenProgramPipelines, glGenProgramPipelinesEXT, glGenProgramsARB, glGenProgramsNV, glGenQueries, glGenQueriesARB, glGenQueriesEXT, glGenQueryResourceTagNV, glGenRenderbuffers, glGenRenderbuffersEXT, glGenRenderbuffersOES, glGenSamplers, glGenSemaphoresEXT, glGenSymbolsEXT, glGenTextures, glGenTexturesEXT, glGenTransformFeedbacks, glGenTransformFeedbacksNV, glGenVertexArrays, glGenVertexArraysAPPLE, glGenVertexArraysOES, glGenVertexShadersEXT, glGenerateMipmap, glGenerateMipmapEXT, glGenerateMipmapOES, glGenerateMultiTexMipmapEXT, glGenerateTextureMipmap, glGenerateTextureMipmapEXT, glGetActiveAtomicCounterBufferiv, glGetActiveAttrib, glGetActiveAttribARB, glGetActiveSubroutineName, glGetActiveSubroutineUniformName, glGetActiveSubroutineUniformiv, glGetActiveUniform, glGetActiveUniformARB, glGetActiveUniformBlockName, glGetActiveUniformBlockiv, glGetActiveUniformName, glGetActiveUniformsiv, glGetActiveVaryingNV, glGetArrayObjectfvATI, glGetArrayObjectivATI, glGetAttachedObjectsARB, glGetAttachedShaders, glGetAttribLocation, glGetAttribLocationARB, glGetBooleanIndexedvEXT, glGetBooleani_v, glGetBooleanv, glGetBufferParameteri64v, glGetBufferParameteriv, glGetBufferParameterivARB, glGetBufferParameterui64vNV, glGetBufferPointerv, glGetBufferPointervARB, glGetBufferPointervOES, glGetBufferSubData, glGetBufferSubDataARB, glGetClipPlane, glGetClipPlanef, glGetClipPlanefOES, glGetClipPlanex, glGetClipPlanexOES, glGetColorTable, glGetColorTableEXT, glGetColorTableParameterfv, glGetColorTableParameterfvEXT, glGetColorTableParameterfvSGI, glGetColorTableParameteriv, glGetColorTableParameterivEXT, glGetColorTableParameterivSGI, glGetColorTableSGI, glGetCombinerInputParameterfvNV, glGetCombinerInputParameterivNV, glGetCombinerOutputParameterfvNV, glGetCombinerOutputParameterivNV, glGetCombinerStageParameterfvNV, glGetCommandHeaderNV, glGetCompressedMultiTexImageEXT, glGetCompressedTexImage, glGetCompressedTexImageARB, glGetCompressedTextureImage, glGetCompressedTextureImageEXT, glGetCompressedTextureSubImage, glGetConvolutionFilter, glGetConvolutionFilterEXT, glGetConvolutionParameterfv, glGetConvolutionParameterfvEXT, glGetConvolutionParameteriv, glGetConvolutionParameterivEXT, glGetConvolutionParameterxvOES, glGetCoverageModulationTableNV, glGetDebugMessageLog, glGetDebugMessageLogAMD, glGetDebugMessageLogARB, glGetDebugMessageLogKHR, glGetDetailTexFuncSGIS, glGetDoubleIndexedvEXT, glGetDoublei_v, glGetDoublei_vEXT, glGetDoublev, glGetDriverControlStringQCOM, glGetDriverControlsQCOM, glGetError, glGetFenceivNV, glGetFinalCombinerInputParameterfvNV, glGetFinalCombinerInputParameterivNV, glGetFirstPerfQueryIdINTEL, glGetFixedv, glGetFixedvOES, glGetFloatIndexedvEXT, glGetFloati_v, glGetFloati_vEXT, glGetFloati_vNV, glGetFloati_vOES, glGetFloatv, glGetFogFuncSGIS, glGetFragDataIndex, glGetFragDataIndexEXT, glGetFragDataLocation, glGetFragDataLocationEXT, glGetFragmentLightfvSGIX, glGetFragmentLightivSGIX, glGetFragmentMaterialfvSGIX, glGetFragmentMaterialivSGIX, glGetFragmentShadingRatesEXT, glGetFramebufferAttachmentParameteriv, glGetFramebufferAttachmentParameterivEXT, glGetFramebufferAttachmentParameterivOES, glGetFramebufferParameterfvAMD, glGetFramebufferParameteriv, glGetFramebufferParameterivEXT, glGetFramebufferPixelLocalStorageSizeEXT, glGetGraphicsResetStatus, glGetGraphicsResetStatusARB, glGetGraphicsResetStatusEXT, glGetGraphicsResetStatusKHR, glGetHandleARB, glGetHistogram, glGetHistogramEXT, glGetHistogramParameterfv, glGetHistogramParameterfvEXT, glGetHistogramParameteriv, glGetHistogramParameterivEXT, glGetHistogramParameterxvOES, glGetImageHandleARB, glGetImageHandleNV, glGetImageTransformParameterfvHP, glGetImageTransformParameterivHP, glGetInfoLogARB, glGetInstrumentsSGIX, glGetInteger64i_v, glGetInteger64v, glGetInteger64vAPPLE, glGetInteger64vEXT, glGetIntegerIndexedvEXT, glGetIntegeri_v, glGetIntegeri_vEXT, glGetIntegerui64i_vNV, glGetIntegerui64vNV, glGetIntegerv, glGetInternalformatSampleivNV, glGetInternalformati64v, glGetInternalformativ, glGetInvariantBooleanvEXT, glGetInvariantFloatvEXT, glGetInvariantIntegervEXT, glGetLightfv, glGetLightiv, glGetLightxOES, glGetLightxv, glGetLightxvOES, glGetListParameterfvSGIX, glGetListParameterivSGIX, glGetLocalConstantBooleanvEXT, glGetLocalConstantFloatvEXT, glGetLocalConstantIntegervEXT, glGetMapAttribParameterfvNV, glGetMapAttribParameterivNV, glGetMapControlPointsNV, glGetMapParameterfvNV, glGetMapParameterivNV, glGetMapdv, glGetMapfv, glGetMapiv, glGetMapxvOES, glGetMaterialfv, glGetMaterialiv, glGetMaterialxOES, glGetMaterialxv, glGetMaterialxvOES, glGetMemoryObjectDetachedResourcesuivNV, glGetMemoryObjectParameterivEXT, glGetMinmax, glGetMinmaxEXT, glGetMinmaxParameterfv, glGetMinmaxParameterfvEXT, glGetMinmaxParameteriv, glGetMinmaxParameterivEXT, glGetMultiTexEnvfvEXT, glGetMultiTexEnvivEXT, glGetMultiTexGendvEXT, glGetMultiTexGenfvEXT, glGetMultiTexGenivEXT, glGetMultiTexImageEXT, glGetMultiTexLevelParameterfvEXT, glGetMultiTexLevelParameterivEXT, glGetMultiTexParameterIivEXT, glGetMultiTexParameterIuivEXT, glGetMultiTexParameterfvEXT, glGetMultiTexParameterivEXT, glGetMultisamplefv, glGetMultisamplefvNV, glGetNamedBufferParameteri64v, glGetNamedBufferParameteriv, glGetNamedBufferParameterivEXT, glGetNamedBufferParameterui64vNV, glGetNamedBufferPointerv, glGetNamedBufferPointervEXT, glGetNamedBufferSubData, glGetNamedBufferSubDataEXT, glGetNamedFramebufferParameterfvAMD, glGetNamedFramebufferAttachmentParameteriv, glGetNamedFramebufferAttachmentParameterivEXT, glGetNamedFramebufferParameteriv, glGetNamedFramebufferParameterivEXT, glGetNamedProgramLocalParameterIivEXT, glGetNamedProgramLocalParameterIuivEXT, glGetNamedProgramLocalParameterdvEXT, glGetNamedProgramLocalParameterfvEXT, glGetNamedProgramStringEXT, glGetNamedProgramivEXT, glGetNamedRenderbufferParameteriv, glGetNamedRenderbufferParameterivEXT, glGetNamedStringARB, glGetNamedStringivARB, glGetNextPerfQueryIdINTEL, glGetObjectBufferfvATI, glGetObjectBufferivATI, glGetObjectLabel, glGetObjectLabelEXT, glGetObjectLabelKHR, glGetObjectParameterfvARB, glGetObjectParameterivAPPLE, glGetObjectParameterivARB, glGetObjectPtrLabel, glGetObjectPtrLabelKHR, glGetOcclusionQueryivNV, glGetOcclusionQueryuivNV, glGetPathColorGenfvNV, glGetPathColorGenivNV, glGetPathCommandsNV, glGetPathCoordsNV, glGetPathDashArrayNV, glGetPathLengthNV, glGetPathMetricRangeNV, glGetPathMetricsNV, glGetPathParameterfvNV, glGetPathParameterivNV, glGetPathSpacingNV, glGetPathTexGenfvNV, glGetPathTexGenivNV, glGetPerfCounterInfoINTEL, glGetPerfMonitorCounterDataAMD, glGetPerfMonitorCounterInfoAMD, glGetPerfMonitorCounterStringAMD, glGetPerfMonitorCountersAMD, glGetPerfMonitorGroupStringAMD, glGetPerfMonitorGroupsAMD, glGetPerfQueryDataINTEL, glGetPerfQueryIdByNameINTEL, glGetPerfQueryInfoINTEL, glGetPixelMapfv, glGetPixelMapuiv, glGetPixelMapusv, glGetPixelMapxv, glGetPixelTexGenParameterfvSGIS, glGetPixelTexGenParameterivSGIS, glGetPixelTransformParameterfvEXT, glGetPixelTransformParameterivEXT, glGetPointerIndexedvEXT, glGetPointeri_vEXT, glGetPointerv, glGetPointervEXT, glGetPointervKHR, glGetPolygonStipple, glGetProgramBinary, glGetProgramBinaryOES, glGetProgramEnvParameterIivNV, glGetProgramEnvParameterIuivNV, glGetProgramEnvParameterdvARB, glGetProgramEnvParameterfvARB, glGetProgramInfoLog, glGetProgramInterfaceiv, glGetProgramLocalParameterIivNV, glGetProgramLocalParameterIuivNV, glGetProgramLocalParameterdvARB, glGetProgramLocalParameterfvARB, glGetProgramNamedParameterdvNV, glGetProgramNamedParameterfvNV, glGetProgramParameterdvNV, glGetProgramParameterfvNV, glGetProgramPipelineInfoLog, glGetProgramPipelineInfoLogEXT, glGetProgramPipelineiv, glGetProgramPipelineivEXT, glGetProgramResourceIndex, glGetProgramResourceLocation, glGetProgramResourceLocationIndex, glGetProgramResourceLocationIndexEXT, glGetProgramResourceName, glGetProgramResourcefvNV, glGetProgramResourceiv, glGetProgramStageiv, glGetProgramStringARB, glGetProgramStringNV, glGetProgramSubroutineParameteruivNV, glGetProgramiv, glGetProgramivARB, glGetProgramivNV, glGetQueryBufferObjecti64v, glGetQueryBufferObjectiv, glGetQueryBufferObjectui64v, glGetQueryBufferObjectuiv, glGetQueryIndexediv, glGetQueryObjecti64v, glGetQueryObjecti64vEXT, glGetQueryObjectiv, glGetQueryObjectivARB, glGetQueryObjectivEXT, glGetQueryObjectui64v, glGetQueryObjectui64vEXT, glGetQueryObjectuiv, glGetQueryObjectuivARB, glGetQueryObjectuivEXT, glGetQueryiv, glGetQueryivARB, glGetQueryivEXT, glGetRenderbufferParameteriv, glGetRenderbufferParameterivEXT, glGetRenderbufferParameterivOES, glGetSamplerParameterIiv, glGetSamplerParameterIivEXT, glGetSamplerParameterIivOES, glGetSamplerParameterIuiv, glGetSamplerParameterIuivEXT, glGetSamplerParameterIuivOES, glGetSamplerParameterfv, glGetSamplerParameteriv, glGetSemaphoreParameterivNV, glGetSemaphoreParameterui64vEXT, glGetSeparableFilter, glGetSeparableFilterEXT, glGetShaderInfoLog, glGetShaderPrecisionFormat, glGetShaderSource, glGetShaderSourceARB, glGetShaderiv, glGetShadingRateImagePaletteNV, glGetShadingRateSampleLocationivNV, glGetSharpenTexFuncSGIS, glGetStageIndexNV, glGetString, glGetStringi, glGetSubroutineIndex, glGetSubroutineUniformLocation, glGetSynciv, glGetSyncivAPPLE, glGetTexBumpParameterfvATI, glGetTexBumpParameterivATI, glGetTexEnvfv, glGetTexEnviv, glGetTexEnvxv, glGetTexEnvxvOES, glGetTexFilterFuncSGIS, glGetTexGendv, glGetTexGenfv, glGetTexGenfvOES, glGetTexGeniv, glGetTexGenivOES, glGetTexGenxvOES, glGetTexImage, glGetTexLevelParameterfv, glGetTexLevelParameteriv, glGetTexLevelParameterxvOES, glGetTexParameterIiv, glGetTexParameterIivEXT, glGetTexParameterIivOES, glGetTexParameterIuiv, glGetTexParameterIuivEXT, glGetTexParameterIuivOES, glGetTexParameterPointervAPPLE, glGetTexParameterfv, glGetTexParameteriv, glGetTexParameterxv, glGetTexParameterxvOES, glGetTextureHandleARB, glGetTextureHandleIMG, glGetTextureHandleNV, glGetTextureImage, glGetTextureImageEXT, glGetTextureLevelParameterfv, glGetTextureLevelParameterfvEXT, glGetTextureLevelParameteriv, glGetTextureLevelParameterivEXT, glGetTextureParameterIiv, glGetTextureParameterIivEXT, glGetTextureParameterIuiv, glGetTextureParameterIuivEXT, glGetTextureParameterfv, glGetTextureParameterfvEXT, glGetTextureParameteriv, glGetTextureParameterivEXT, glGetTextureSamplerHandleARB, glGetTextureSamplerHandleIMG, glGetTextureSamplerHandleNV, glGetTextureSubImage, glGetTrackMatrixivNV, glGetTransformFeedbackVarying, glGetTransformFeedbackVaryingEXT, glGetTransformFeedbackVaryingNV, glGetTransformFeedbacki64_v, glGetTransformFeedbacki_v, glGetTransformFeedbackiv, glGetTranslatedShaderSourceANGLE, glGetUniformBlockIndex, glGetUniformBufferSizeEXT, glGetUniformIndices, glGetUniformLocation, glGetUniformLocationARB, glGetUniformOffsetEXT, glGetUniformSubroutineuiv, glGetUniformdv, glGetUniformfv, glGetUniformfvARB, glGetUniformi64vARB, glGetUniformi64vNV, glGetUniformiv, glGetUniformivARB, glGetUniformui64vARB, glGetUniformui64vNV, glGetUniformuiv, glGetUniformuivEXT, glGetUnsignedBytevEXT, glGetUnsignedBytei_vEXT, glGetVariantArrayObjectfvATI, glGetVariantArrayObjectivATI, glGetVariantBooleanvEXT, glGetVariantFloatvEXT, glGetVariantIntegervEXT, glGetVariantPointervEXT, glGetVaryingLocationNV, glGetVertexArrayIndexed64iv, glGetVertexArrayIndexediv, glGetVertexArrayIntegeri_vEXT, glGetVertexArrayIntegervEXT, glGetVertexArrayPointeri_vEXT, glGetVertexArrayPointervEXT, glGetVertexArrayiv, glGetVertexAttribArrayObjectfvATI, glGetVertexAttribArrayObjectivATI, glGetVertexAttribIiv, glGetVertexAttribIivEXT, glGetVertexAttribIuiv, glGetVertexAttribIuivEXT, glGetVertexAttribLdv, glGetVertexAttribLdvEXT, glGetVertexAttribLi64vNV, glGetVertexAttribLui64vARB, glGetVertexAttribLui64vNV, glGetVertexAttribPointerv, glGetVertexAttribPointervARB, glGetVertexAttribPointervNV, glGetVertexAttribdv, glGetVertexAttribdvARB, glGetVertexAttribdvNV, glGetVertexAttribfv, glGetVertexAttribfvARB, glGetVertexAttribfvNV, glGetVertexAttribiv, glGetVertexAttribivARB, glGetVertexAttribivNV, glGetVideoCaptureStreamdvNV, glGetVideoCaptureStreamfvNV, glGetVideoCaptureStreamivNV, glGetVideoCaptureivNV, glGetVideoi64vNV, glGetVideoivNV, glGetVideoui64vNV, glGetVideouivNV, glGetnColorTable, glGetnColorTableARB, glGetnCompressedTexImage, glGetnCompressedTexImageARB, glGetnConvolutionFilter, glGetnConvolutionFilterARB, glGetnHistogram, glGetnHistogramARB, glGetnMapdv, glGetnMapdvARB, glGetnMapfv, glGetnMapfvARB, glGetnMapiv, glGetnMapivARB, glGetnMinmax, glGetnMinmaxARB, glGetnPixelMapfv, glGetnPixelMapfvARB, glGetnPixelMapuiv, glGetnPixelMapuivARB, glGetnPixelMapusv, glGetnPixelMapusvARB, glGetnPolygonStipple, glGetnPolygonStippleARB, glGetnSeparableFilter, glGetnSeparableFilterARB, glGetnTexImage, glGetnTexImageARB, glGetnUniformdv, glGetnUniformdvARB, glGetnUniformfv, glGetnUniformfvARB, glGetnUniformfvEXT, glGetnUniformfvKHR, glGetnUniformi64vARB, glGetnUniformiv, glGetnUniformivARB, glGetnUniformivEXT, glGetnUniformivKHR, glGetnUniformui64vARB, glGetnUniformuiv, glGetnUniformuivARB, glGetnUniformuivKHR, glGlobalAlphaFactorbSUN, glGlobalAlphaFactordSUN, glGlobalAlphaFactorfSUN, glGlobalAlphaFactoriSUN, glGlobalAlphaFactorsSUN, glGlobalAlphaFactorubSUN, glGlobalAlphaFactoruiSUN, glGlobalAlphaFactorusSUN, glHint, glHintPGI, glHistogram, glHistogramEXT, glIglooInterfaceSGIX, glImageTransformParameterfHP, glImageTransformParameterfvHP, glImageTransformParameteriHP, glImageTransformParameterivHP, glImportMemoryFdEXT, glImportMemoryWin32HandleEXT, glImportMemoryWin32NameEXT, glImportSemaphoreFdEXT, glImportSemaphoreWin32HandleEXT, glImportSemaphoreWin32NameEXT, glImportSyncEXT, glIndexFormatNV, glIndexFuncEXT, glIndexMask, glIndexMaterialEXT, glIndexPointer, glIndexPointerEXT, glIndexPointerListIBM, glIndexd, glIndexdv, glIndexf, glIndexfv, glIndexi, glIndexiv, glIndexs, glIndexsv, glIndexub, glIndexubv, glIndexxOES, glIndexxvOES, glInitNames, glInsertComponentEXT, glInsertEventMarkerEXT, glInstrumentsBufferSGIX, glInterleavedArrays, glInterpolatePathsNV, glInvalidateBufferData, glInvalidateBufferSubData, glInvalidateFramebuffer, glInvalidateNamedFramebufferData, glInvalidateNamedFramebufferSubData, glInvalidateSubFramebuffer, glInvalidateTexImage, glInvalidateTexSubImage, glIsAsyncMarkerSGIX, glIsBuffer, glIsBufferARB, glIsBufferResidentNV, glIsCommandListNV, glIsEnabled, glIsEnabledIndexedEXT, glIsEnabledi, glIsEnablediEXT, glIsEnablediNV, glIsEnablediOES, glIsFenceAPPLE, glIsFenceNV, glIsFramebuffer, glIsFramebufferEXT, glIsFramebufferOES, glIsImageHandleResidentARB, glIsImageHandleResidentNV, glIsList, glIsMemoryObjectEXT, glIsNameAMD, glIsNamedBufferResidentNV, glIsNamedStringARB, glIsObjectBufferATI, glIsOcclusionQueryNV, glIsPathNV, glIsPointInFillPathNV, glIsPointInStrokePathNV, glIsProgram, glIsProgramARB, glIsProgramNV, glIsProgramPipeline, glIsProgramPipelineEXT, glIsQuery, glIsQueryARB, glIsQueryEXT, glIsRenderbuffer, glIsRenderbufferEXT, glIsRenderbufferOES, glIsSemaphoreEXT, glIsSampler, glIsShader, glIsStateNV, glIsSync, glIsSyncAPPLE, glIsTexture, glIsTextureEXT, glIsTextureHandleResidentARB, glIsTextureHandleResidentNV, glIsTransformFeedback, glIsTransformFeedbackNV, glIsVariantEnabledEXT, glIsVertexArray, glIsVertexArrayAPPLE, glIsVertexArrayOES, glIsVertexAttribEnabledAPPLE, glLGPUCopyImageSubDataNVX, glLGPUInterlockNVX, glLGPUNamedBufferSubDataNVX, glLabelObjectEXT, glLightEnviSGIX, glLightModelf, glLightModelfv, glLightModeli, glLightModeliv, glLightModelx, glLightModelxOES, glLightModelxv, glLightModelxvOES, glLightf, glLightfv, glLighti, glLightiv, glLightx, glLightxOES, glLightxv, glLightxvOES, glLineStipple, glLineWidth, glLineWidthx, glLineWidthxOES, glLinkProgram, glLinkProgramARB, glListBase, glListDrawCommandsStatesClientNV, glListParameterfSGIX, glListParameterfvSGIX, glListParameteriSGIX, glListParameterivSGIX, glLoadIdentity, glLoadIdentityDeformationMapSGIX, glLoadMatrixd, glLoadMatrixf, glLoadMatrixx, glLoadMatrixxOES, glLoadName, glLoadPaletteFromModelViewMatrixOES, glLoadProgramNV, glLoadTransposeMatrixd, glLoadTransposeMatrixdARB, glLoadTransposeMatrixf, glLoadTransposeMatrixfARB, glLoadTransposeMatrixxOES, glLockArraysEXT, glLogicOp, glMakeBufferNonResidentNV, glMakeBufferResidentNV, glMakeImageHandleNonResidentARB, glMakeImageHandleNonResidentNV, glMakeImageHandleResidentARB, glMakeImageHandleResidentNV, glMakeNamedBufferNonResidentNV, glMakeNamedBufferResidentNV, glMakeTextureHandleNonResidentARB, glMakeTextureHandleNonResidentNV, glMakeTextureHandleResidentARB, glMakeTextureHandleResidentNV, glMap1d, glMap1f, glMap1xOES, glMap2d, glMap2f, glMap2xOES, glMapBuffer, glMapBufferARB, glMapBufferOES, glMapBufferRange, glMapBufferRangeEXT, glMapControlPointsNV, glMapGrid1d, glMapGrid1f, glMapGrid1xOES, glMapGrid2d, glMapGrid2f, glMapGrid2xOES, glMapNamedBuffer, glMapNamedBufferEXT, glMapNamedBufferRange, glMapNamedBufferRangeEXT, glMapObjectBufferATI, glMapParameterfvNV, glMapParameterivNV, glMapTexture2DINTEL, glMapVertexAttrib1dAPPLE, glMapVertexAttrib1fAPPLE, glMapVertexAttrib2dAPPLE, glMapVertexAttrib2fAPPLE, glMaterialf, glMaterialfv, glMateriali, glMaterialiv, glMaterialx, glMaterialxOES, glMaterialxv, glMaterialxvOES, glMatrixFrustumEXT, glMatrixIndexPointerARB, glMatrixIndexPointerOES, glMatrixIndexubvARB, glMatrixIndexuivARB, glMatrixIndexusvARB, glMatrixLoad3x2fNV, glMatrixLoad3x3fNV, glMatrixLoadIdentityEXT, glMatrixLoadTranspose3x3fNV, glMatrixLoadTransposedEXT, glMatrixLoadTransposefEXT, glMatrixLoaddEXT, glMatrixLoadfEXT, glMatrixMode, glMatrixMult3x2fNV, glMatrixMult3x3fNV, glMatrixMultTranspose3x3fNV, glMatrixMultTransposedEXT, glMatrixMultTransposefEXT, glMatrixMultdEXT, glMatrixMultfEXT, glMatrixOrthoEXT, glMatrixPopEXT, glMatrixPushEXT, glMatrixRotatedEXT, glMatrixRotatefEXT, glMatrixScaledEXT, glMatrixScalefEXT, glMatrixTranslatedEXT, glMatrixTranslatefEXT, glMaxActiveShaderCoresARM, glMaxShaderCompilerThreadsKHR, glMaxShaderCompilerThreadsARB, glMemoryBarrier, glMemoryBarrierByRegion, glMemoryBarrierEXT, glMemoryObjectParameterivEXT, glMinSampleShading, glMinSampleShadingARB, glMinSampleShadingOES, glMinmax, glMinmaxEXT, glMultMatrixd, glMultMatrixf, glMultMatrixx, glMultMatrixxOES, glMultTransposeMatrixd, glMultTransposeMatrixdARB, glMultTransposeMatrixf, glMultTransposeMatrixfARB, glMultTransposeMatrixxOES, glMultiDrawArrays, glMultiDrawArraysEXT, glMultiDrawArraysIndirect, glMultiDrawArraysIndirectAMD, glMultiDrawArraysIndirectBindlessCountNV, glMultiDrawArraysIndirectBindlessNV, glMultiDrawArraysIndirectCount, glMultiDrawArraysIndirectCountARB, glMultiDrawArraysIndirectEXT, glMultiDrawElementArrayAPPLE, glMultiDrawElements, glMultiDrawElementsBaseVertex, glMultiDrawElementsBaseVertexEXT, glMultiDrawElementsEXT, glMultiDrawElementsIndirect, glMultiDrawElementsIndirectAMD, glMultiDrawElementsIndirectBindlessCountNV, glMultiDrawElementsIndirectBindlessNV, glMultiDrawElementsIndirectCount, glMultiDrawElementsIndirectCountARB, glMultiDrawElementsIndirectEXT, glMultiDrawMeshTasksIndirectNV, glMultiDrawMeshTasksIndirectCountNV, glMultiDrawRangeElementArrayAPPLE, glMultiModeDrawArraysIBM, glMultiModeDrawElementsIBM, glMultiTexBufferEXT, glMultiTexCoord1bOES, glMultiTexCoord1bvOES, glMultiTexCoord1d, glMultiTexCoord1dARB, glMultiTexCoord1dv, glMultiTexCoord1dvARB, glMultiTexCoord1f, glMultiTexCoord1fARB, glMultiTexCoord1fv, glMultiTexCoord1fvARB, glMultiTexCoord1hNV, glMultiTexCoord1hvNV, glMultiTexCoord1i, glMultiTexCoord1iARB, glMultiTexCoord1iv, glMultiTexCoord1ivARB, glMultiTexCoord1s, glMultiTexCoord1sARB, glMultiTexCoord1sv, glMultiTexCoord1svARB, glMultiTexCoord1xOES, glMultiTexCoord1xvOES, glMultiTexCoord2bOES, glMultiTexCoord2bvOES, glMultiTexCoord2d, glMultiTexCoord2dARB, glMultiTexCoord2dv, glMultiTexCoord2dvARB, glMultiTexCoord2f, glMultiTexCoord2fARB, glMultiTexCoord2fv, glMultiTexCoord2fvARB, glMultiTexCoord2hNV, glMultiTexCoord2hvNV, glMultiTexCoord2i, glMultiTexCoord2iARB, glMultiTexCoord2iv, glMultiTexCoord2ivARB, glMultiTexCoord2s, glMultiTexCoord2sARB, glMultiTexCoord2sv, glMultiTexCoord2svARB, glMultiTexCoord2xOES, glMultiTexCoord2xvOES, glMultiTexCoord3bOES, glMultiTexCoord3bvOES, glMultiTexCoord3d, glMultiTexCoord3dARB, glMultiTexCoord3dv, glMultiTexCoord3dvARB, glMultiTexCoord3f, glMultiTexCoord3fARB, glMultiTexCoord3fv, glMultiTexCoord3fvARB, glMultiTexCoord3hNV, glMultiTexCoord3hvNV, glMultiTexCoord3i, glMultiTexCoord3iARB, glMultiTexCoord3iv, glMultiTexCoord3ivARB, glMultiTexCoord3s, glMultiTexCoord3sARB, glMultiTexCoord3sv, glMultiTexCoord3svARB, glMultiTexCoord3xOES, glMultiTexCoord3xvOES, glMultiTexCoord4bOES, glMultiTexCoord4bvOES, glMultiTexCoord4d, glMultiTexCoord4dARB, glMultiTexCoord4dv, glMultiTexCoord4dvARB, glMultiTexCoord4f, glMultiTexCoord4fARB, glMultiTexCoord4fv, glMultiTexCoord4fvARB, glMultiTexCoord4hNV, glMultiTexCoord4hvNV, glMultiTexCoord4i, glMultiTexCoord4iARB, glMultiTexCoord4iv, glMultiTexCoord4ivARB, glMultiTexCoord4s, glMultiTexCoord4sARB, glMultiTexCoord4sv, glMultiTexCoord4svARB, glMultiTexCoord4x, glMultiTexCoord4xOES, glMultiTexCoord4xvOES, glMultiTexCoordP1ui, glMultiTexCoordP1uiv, glMultiTexCoordP2ui, glMultiTexCoordP2uiv, glMultiTexCoordP3ui, glMultiTexCoordP3uiv, glMultiTexCoordP4ui, glMultiTexCoordP4uiv, glMultiTexCoordPointerEXT, glMultiTexEnvfEXT, glMultiTexEnvfvEXT, glMultiTexEnviEXT, glMultiTexEnvivEXT, glMultiTexGendEXT, glMultiTexGendvEXT, glMultiTexGenfEXT, glMultiTexGenfvEXT, glMultiTexGeniEXT, glMultiTexGenivEXT, glMultiTexImage1DEXT, glMultiTexImage2DEXT, glMultiTexImage3DEXT, glMultiTexParameterIivEXT, glMultiTexParameterIuivEXT, glMultiTexParameterfEXT, glMultiTexParameterfvEXT, glMultiTexParameteriEXT, glMultiTexParameterivEXT, glMultiTexRenderbufferEXT, glMultiTexSubImage1DEXT, glMultiTexSubImage2DEXT, glMultiTexSubImage3DEXT, glMulticastBarrierNV, glMulticastBlitFramebufferNV, glMulticastBufferSubDataNV, glMulticastCopyBufferSubDataNV, glMulticastCopyImageSubDataNV, glMulticastFramebufferSampleLocationsfvNV, glMulticastGetQueryObjecti64vNV, glMulticastGetQueryObjectivNV, glMulticastGetQueryObjectui64vNV, glMulticastGetQueryObjectuivNV, glMulticastScissorArrayvNVX, glMulticastViewportArrayvNVX, glMulticastViewportPositionWScaleNVX, glMulticastWaitSyncNV, glNamedBufferAttachMemoryNV, glNamedBufferData, glNamedBufferDataEXT, glNamedBufferPageCommitmentARB, glNamedBufferPageCommitmentEXT, glNamedBufferPageCommitmentMemNV, glNamedBufferStorage, glNamedBufferStorageExternalEXT, glNamedBufferStorageEXT, glNamedBufferStorageMemEXT, glNamedBufferSubData, glNamedBufferSubDataEXT, glNamedCopyBufferSubDataEXT, glNamedFramebufferDrawBuffer, glNamedFramebufferDrawBuffers, glNamedFramebufferParameteri, glNamedFramebufferParameteriEXT, glNamedFramebufferReadBuffer, glNamedFramebufferRenderbuffer, glNamedFramebufferRenderbufferEXT, glNamedFramebufferSampleLocationsfvARB, glNamedFramebufferSampleLocationsfvNV, glNamedFramebufferTexture, glNamedFramebufferSamplePositionsfvAMD, glNamedFramebufferTexture1DEXT, glNamedFramebufferTexture2DEXT, glNamedFramebufferTexture3DEXT, glNamedFramebufferTextureEXT, glNamedFramebufferTextureFaceEXT, glNamedFramebufferTextureLayer, glNamedFramebufferTextureLayerEXT, glNamedFramebufferTextureMultiviewOVR, glNamedProgramLocalParameter4dEXT, glNamedProgramLocalParameter4dvEXT, glNamedProgramLocalParameter4fEXT, glNamedProgramLocalParameter4fvEXT, glNamedProgramLocalParameterI4iEXT, glNamedProgramLocalParameterI4ivEXT, glNamedProgramLocalParameterI4uiEXT, glNamedProgramLocalParameterI4uivEXT, glNamedProgramLocalParameters4fvEXT, glNamedProgramLocalParametersI4ivEXT, glNamedProgramLocalParametersI4uivEXT, glNamedProgramStringEXT, glNamedRenderbufferStorage, glNamedRenderbufferStorageEXT, glNamedRenderbufferStorageMultisample, glNamedRenderbufferStorageMultisampleAdvancedAMD, glNamedRenderbufferStorageMultisampleCoverageEXT, glNamedRenderbufferStorageMultisampleEXT, glNamedStringARB, glNewList, glNewObjectBufferATI, glNormal3b, glNormal3bv, glNormal3d, glNormal3dv, glNormal3f, glNormal3fVertex3fSUN, glNormal3fVertex3fvSUN, glNormal3fv, glNormal3hNV, glNormal3hvNV, glNormal3i, glNormal3iv, glNormal3s, glNormal3sv, glNormal3x, glNormal3xOES, glNormal3xvOES, glNormalFormatNV, glNormalP3ui, glNormalP3uiv, glNormalPointer, glNormalPointerEXT, glNormalPointerListIBM, glNormalPointervINTEL, glNormalStream3bATI, glNormalStream3bvATI, glNormalStream3dATI, glNormalStream3dvATI, glNormalStream3fATI, glNormalStream3fvATI, glNormalStream3iATI, glNormalStream3ivATI, glNormalStream3sATI, glNormalStream3svATI, glObjectLabel, glObjectLabelKHR, glObjectPtrLabel, glObjectPtrLabelKHR, glObjectPurgeableAPPLE, glObjectUnpurgeableAPPLE, glOrtho, glOrthof, glOrthofOES, glOrthox, glOrthoxOES, glPNTrianglesfATI, glPNTrianglesiATI, glPassTexCoordATI, glPassThrough, glPassThroughxOES, glPatchParameterfv, glPatchParameteri, glPatchParameteriEXT, glPatchParameteriOES, glPathColorGenNV, glPathCommandsNV, glPathCoordsNV, glPathCoverDepthFuncNV, glPathDashArrayNV, glPathFogGenNV, glPathGlyphIndexArrayNV, glPathGlyphIndexRangeNV, glPathGlyphRangeNV, glPathGlyphsNV, glPathMemoryGlyphIndexArrayNV, glPathParameterfNV, glPathParameterfvNV, glPathParameteriNV, glPathParameterivNV, glPathStencilDepthOffsetNV, glPathStencilFuncNV, glPathStringNV, glPathSubCommandsNV, glPathSubCoordsNV, glPathTexGenNV, glPauseTransformFeedback, glPauseTransformFeedbackNV, glPixelDataRangeNV, glPixelMapfv, glPixelMapuiv, glPixelMapusv, glPixelMapx, glPixelStoref, glPixelStorei, glPixelStorex, glPixelTexGenParameterfSGIS, glPixelTexGenParameterfvSGIS, glPixelTexGenParameteriSGIS, glPixelTexGenParameterivSGIS, glPixelTexGenSGIX, glPixelTransferf, glPixelTransferi, glPixelTransferxOES, glPixelTransformParameterfEXT, glPixelTransformParameterfvEXT, glPixelTransformParameteriEXT, glPixelTransformParameterivEXT, glPixelZoom, glPixelZoomxOES, glPointAlongPathNV, glPointParameterf, glPointParameterfARB, glPointParameterfEXT, glPointParameterfSGIS, glPointParameterfv, glPointParameterfvARB, glPointParameterfvEXT, glPointParameterfvSGIS, glPointParameteri, glPointParameteriNV, glPointParameteriv, glPointParameterivNV, glPointParameterx, glPointParameterxOES, glPointParameterxv, glPointParameterxvOES, glPointSize, glPointSizePointerOES, glPointSizex, glPointSizexOES, glPollAsyncSGIX, glPollInstrumentsSGIX, glPolygonMode, glPolygonModeNV, glPolygonOffset, glPolygonOffsetClamp, glPolygonOffsetClampEXT, glPolygonOffsetEXT, glPolygonOffsetx, glPolygonOffsetxOES, glPolygonStipple, glPopAttrib, glPopClientAttrib, glPopDebugGroup, glPopDebugGroupKHR, glPopGroupMarkerEXT, glPopMatrix, glPopName, glPresentFrameDualFillNV, glPresentFrameKeyedNV, glPrimitiveBoundingBox, glPrimitiveBoundingBoxARB, glPrimitiveBoundingBoxEXT, glPrimitiveBoundingBoxOES, glPrimitiveRestartIndex, glPrimitiveRestartIndexNV, glPrimitiveRestartNV, glPrioritizeTextures, glPrioritizeTexturesEXT, glPrioritizeTexturesxOES, glProgramBinary, glProgramBinaryOES, glProgramBufferParametersIivNV, glProgramBufferParametersIuivNV, glProgramBufferParametersfvNV, glProgramEnvParameter4dARB, glProgramEnvParameter4dvARB, glProgramEnvParameter4fARB, glProgramEnvParameter4fvARB, glProgramEnvParameterI4iNV, glProgramEnvParameterI4ivNV, glProgramEnvParameterI4uiNV, glProgramEnvParameterI4uivNV, glProgramEnvParameters4fvEXT, glProgramEnvParametersI4ivNV, glProgramEnvParametersI4uivNV, glProgramLocalParameter4dARB, glProgramLocalParameter4dvARB, glProgramLocalParameter4fARB, glProgramLocalParameter4fvARB, glProgramLocalParameterI4iNV, glProgramLocalParameterI4ivNV, glProgramLocalParameterI4uiNV, glProgramLocalParameterI4uivNV, glProgramLocalParameters4fvEXT, glProgramLocalParametersI4ivNV, glProgramLocalParametersI4uivNV, glProgramNamedParameter4dNV, glProgramNamedParameter4dvNV, glProgramNamedParameter4fNV, glProgramNamedParameter4fvNV, glProgramParameter4dNV, glProgramParameter4dvNV, glProgramParameter4fNV, glProgramParameter4fvNV, glProgramParameteri, glProgramParameteriARB, glProgramParameteriEXT, glProgramParameters4dvNV, glProgramParameters4fvNV, glProgramPathFragmentInputGenNV, glProgramStringARB, glProgramSubroutineParametersuivNV, glProgramUniform1d, glProgramUniform1dEXT, glProgramUniform1dv, glProgramUniform1dvEXT, glProgramUniform1f, glProgramUniform1fEXT, glProgramUniform1fv, glProgramUniform1fvEXT, glProgramUniform1i, glProgramUniform1i64ARB, glProgramUniform1i64NV, glProgramUniform1i64vARB, glProgramUniform1i64vNV, glProgramUniform1iEXT, glProgramUniform1iv, glProgramUniform1ivEXT, glProgramUniform1ui, glProgramUniform1ui64ARB, glProgramUniform1ui64NV, glProgramUniform1ui64vARB, glProgramUniform1ui64vNV, glProgramUniform1uiEXT, glProgramUniform1uiv, glProgramUniform1uivEXT, glProgramUniform2d, glProgramUniform2dEXT, glProgramUniform2dv, glProgramUniform2dvEXT, glProgramUniform2f, glProgramUniform2fEXT, glProgramUniform2fv, glProgramUniform2fvEXT, glProgramUniform2i, glProgramUniform2i64ARB, glProgramUniform2i64NV, glProgramUniform2i64vARB, glProgramUniform2i64vNV, glProgramUniform2iEXT, glProgramUniform2iv, glProgramUniform2ivEXT, glProgramUniform2ui, glProgramUniform2ui64ARB, glProgramUniform2ui64NV, glProgramUniform2ui64vARB, glProgramUniform2ui64vNV, glProgramUniform2uiEXT, glProgramUniform2uiv, glProgramUniform2uivEXT, glProgramUniform3d, glProgramUniform3dEXT, glProgramUniform3dv, glProgramUniform3dvEXT, glProgramUniform3f, glProgramUniform3fEXT, glProgramUniform3fv, glProgramUniform3fvEXT, glProgramUniform3i, glProgramUniform3i64ARB, glProgramUniform3i64NV, glProgramUniform3i64vARB, glProgramUniform3i64vNV, glProgramUniform3iEXT, glProgramUniform3iv, glProgramUniform3ivEXT, glProgramUniform3ui, glProgramUniform3ui64ARB, glProgramUniform3ui64NV, glProgramUniform3ui64vARB, glProgramUniform3ui64vNV, glProgramUniform3uiEXT, glProgramUniform3uiv, glProgramUniform3uivEXT, glProgramUniform4d, glProgramUniform4dEXT, glProgramUniform4dv, glProgramUniform4dvEXT, glProgramUniform4f, glProgramUniform4fEXT, glProgramUniform4fv, glProgramUniform4fvEXT, glProgramUniform4i, glProgramUniform4i64ARB, glProgramUniform4i64NV, glProgramUniform4i64vARB, glProgramUniform4i64vNV, glProgramUniform4iEXT, glProgramUniform4iv, glProgramUniform4ivEXT, glProgramUniform4ui, glProgramUniform4ui64ARB, glProgramUniform4ui64NV, glProgramUniform4ui64vARB, glProgramUniform4ui64vNV, glProgramUniform4uiEXT, glProgramUniform4uiv, glProgramUniform4uivEXT, glProgramUniformHandleui64ARB, glProgramUniformHandleui64IMG, glProgramUniformHandleui64NV, glProgramUniformHandleui64vARB, glProgramUniformHandleui64vIMG, glProgramUniformHandleui64vNV, glProgramUniformMatrix2dv, glProgramUniformMatrix2dvEXT, glProgramUniformMatrix2fv, glProgramUniformMatrix2fvEXT, glProgramUniformMatrix2x3dv, glProgramUniformMatrix2x3dvEXT, glProgramUniformMatrix2x3fv, glProgramUniformMatrix2x3fvEXT, glProgramUniformMatrix2x4dv, glProgramUniformMatrix2x4dvEXT, glProgramUniformMatrix2x4fv, glProgramUniformMatrix2x4fvEXT, glProgramUniformMatrix3dv, glProgramUniformMatrix3dvEXT, glProgramUniformMatrix3fv, glProgramUniformMatrix3fvEXT, glProgramUniformMatrix3x2dv, glProgramUniformMatrix3x2dvEXT, glProgramUniformMatrix3x2fv, glProgramUniformMatrix3x2fvEXT, glProgramUniformMatrix3x4dv, glProgramUniformMatrix3x4dvEXT, glProgramUniformMatrix3x4fv, glProgramUniformMatrix3x4fvEXT, glProgramUniformMatrix4dv, glProgramUniformMatrix4dvEXT, glProgramUniformMatrix4fv, glProgramUniformMatrix4fvEXT, glProgramUniformMatrix4x2dv, glProgramUniformMatrix4x2dvEXT, glProgramUniformMatrix4x2fv, glProgramUniformMatrix4x2fvEXT, glProgramUniformMatrix4x3dv, glProgramUniformMatrix4x3dvEXT, glProgramUniformMatrix4x3fv, glProgramUniformMatrix4x3fvEXT, glProgramUniformui64NV, glProgramUniformui64vNV, glProgramVertexLimitNV, glProvokingVertex, glProvokingVertexEXT, glPushAttrib, glPushClientAttrib, glPushClientAttribDefaultEXT, glPushDebugGroup, glPushDebugGroupKHR, glPushGroupMarkerEXT, glPushMatrix, glPushName, glQueryCounter, glQueryCounterEXT, glQueryMatrixxOES, glQueryObjectParameteruiAMD, glQueryResourceNV, glQueryResourceTagNV, glRasterPos2d, glRasterPos2dv, glRasterPos2f, glRasterPos2fv, glRasterPos2i, glRasterPos2iv, glRasterPos2s, glRasterPos2sv, glRasterPos2xOES, glRasterPos2xvOES, glRasterPos3d, glRasterPos3dv, glRasterPos3f, glRasterPos3fv, glRasterPos3i, glRasterPos3iv, glRasterPos3s, glRasterPos3sv, glRasterPos3xOES, glRasterPos3xvOES, glRasterPos4d, glRasterPos4dv, glRasterPos4f, glRasterPos4fv, glRasterPos4i, glRasterPos4iv, glRasterPos4s, glRasterPos4sv, glRasterPos4xOES, glRasterPos4xvOES, glRasterSamplesEXT, glReadBuffer, glReadBufferIndexedEXT, glReadBufferNV, glReadInstrumentsSGIX, glReadPixels, glReadnPixels, glReadnPixelsARB, glReadnPixelsEXT, glReadnPixelsKHR, glReleaseKeyedMutexWin32EXT, glRectd, glRectdv, glRectf, glRectfv, glRecti, glRectiv, glRects, glRectsv, glRectxOES, glRectxvOES, glReferencePlaneSGIX, glReleaseShaderCompiler, glRenderGpuMaskNV, glRenderMode, glRenderbufferStorage, glRenderbufferStorageEXT, glRenderbufferStorageMultisample, glRenderbufferStorageMultisampleANGLE, glRenderbufferStorageMultisampleAPPLE, glRenderbufferStorageMultisampleAdvancedAMD, glRenderbufferStorageMultisampleCoverageNV, glRenderbufferStorageMultisampleEXT, glRenderbufferStorageMultisampleIMG, glRenderbufferStorageMultisampleNV, glRenderbufferStorageOES, glReplacementCodePointerSUN, glReplacementCodeubSUN, glReplacementCodeubvSUN, glReplacementCodeuiColor3fVertex3fSUN, glReplacementCodeuiColor3fVertex3fvSUN, glReplacementCodeuiColor4fNormal3fVertex3fSUN, glReplacementCodeuiColor4fNormal3fVertex3fvSUN, glReplacementCodeuiColor4ubVertex3fSUN, glReplacementCodeuiColor4ubVertex3fvSUN, glReplacementCodeuiNormal3fVertex3fSUN, glReplacementCodeuiNormal3fVertex3fvSUN, glReplacementCodeuiSUN, glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fSUN, glReplacementCodeuiTexCoord2fColor4fNormal3fVertex3fvSUN, glReplacementCodeuiTexCoord2fNormal3fVertex3fSUN, glReplacementCodeuiTexCoord2fNormal3fVertex3fvSUN, glReplacementCodeuiTexCoord2fVertex3fSUN, glReplacementCodeuiTexCoord2fVertex3fvSUN, glReplacementCodeuiVertex3fSUN, glReplacementCodeuiVertex3fvSUN, glReplacementCodeuivSUN, glReplacementCodeusSUN, glReplacementCodeusvSUN, glRequestResidentProgramsNV, glResetHistogram, glResetHistogramEXT, glResetMemoryObjectParameterNV, glResetMinmax, glResetMinmaxEXT, glResizeBuffersMESA, glResolveDepthValuesNV, glResolveMultisampleFramebufferAPPLE, glResumeTransformFeedback, glResumeTransformFeedbackNV, glRotated, glRotatef, glRotatex, glRotatexOES, glSampleCoverage, glSampleCoverageARB, glSampleCoveragex, glSampleCoveragexOES, glSampleMapATI, glSampleMaskEXT, glSampleMaskIndexedNV, glSampleMaskSGIS, glSampleMaski, glSamplePatternEXT, glSamplePatternSGIS, glSamplerParameterIiv, glSamplerParameterIivEXT, glSamplerParameterIivOES, glSamplerParameterIuiv, glSamplerParameterIuivEXT, glSamplerParameterIuivOES, glSamplerParameterf, glSamplerParameterfv, glSamplerParameteri, glSamplerParameteriv, glScaled, glScalef, glScalex, glScalexOES, glScissor, glScissorArrayv, glScissorArrayvNV, glScissorArrayvOES, glScissorExclusiveArrayvNV, glScissorExclusiveNV, glScissorIndexed, glScissorIndexedNV, glScissorIndexedOES, glScissorIndexedv, glScissorIndexedvNV, glScissorIndexedvOES, glSecondaryColor3b, glSecondaryColor3bEXT, glSecondaryColor3bv, glSecondaryColor3bvEXT, glSecondaryColor3d, glSecondaryColor3dEXT, glSecondaryColor3dv, glSecondaryColor3dvEXT, glSecondaryColor3f, glSecondaryColor3fEXT, glSecondaryColor3fv, glSecondaryColor3fvEXT, glSecondaryColor3hNV, glSecondaryColor3hvNV, glSecondaryColor3i, glSecondaryColor3iEXT, glSecondaryColor3iv, glSecondaryColor3ivEXT, glSecondaryColor3s, glSecondaryColor3sEXT, glSecondaryColor3sv, glSecondaryColor3svEXT, glSecondaryColor3ub, glSecondaryColor3ubEXT, glSecondaryColor3ubv, glSecondaryColor3ubvEXT, glSecondaryColor3ui, glSecondaryColor3uiEXT, glSecondaryColor3uiv, glSecondaryColor3uivEXT, glSecondaryColor3us, glSecondaryColor3usEXT, glSecondaryColor3usv, glSecondaryColor3usvEXT, glSecondaryColorFormatNV, glSecondaryColorP3ui, glSecondaryColorP3uiv, glSecondaryColorPointer, glSecondaryColorPointerEXT, glSecondaryColorPointerListIBM, glSelectBuffer, glSelectPerfMonitorCountersAMD, glSemaphoreParameterivNV, glSemaphoreParameterui64vEXT, glSeparableFilter2D, glSeparableFilter2DEXT, glSetFenceAPPLE, glSetFenceNV, glSetFragmentShaderConstantATI, glSetInvariantEXT, glSetLocalConstantEXT, glSetMultisamplefvAMD, glShadeModel, glShaderBinary, glShaderOp1EXT, glShaderOp2EXT, glShaderOp3EXT, glShaderSource, glShaderSourceARB, glShaderStorageBlockBinding, glShadingRateEXT, glShadingRateCombinerOpsEXT, glShadingRateImageBarrierNV, glShadingRateQCOM, glShadingRateImagePaletteNV, glShadingRateSampleOrderNV, glShadingRateSampleOrderCustomNV, glSharpenTexFuncSGIS, glSignalSemaphoreEXT, glSignalSemaphoreui64NVX, glSpecializeShader, glSpecializeShaderARB, glSpriteParameterfSGIX, glSpriteParameterfvSGIX, glSpriteParameteriSGIX, glSpriteParameterivSGIX, glStartInstrumentsSGIX, glStartTilingQCOM, glStateCaptureNV, glStencilClearTagEXT, glStencilFillPathInstancedNV, glStencilFillPathNV, glStencilFunc, glStencilFuncSeparate, glStencilFuncSeparateATI, glStencilMask, glStencilMaskSeparate, glStencilOp, glStencilOpSeparate, glStencilOpSeparateATI, glStencilOpValueAMD, glStencilStrokePathInstancedNV, glStencilStrokePathNV, glStencilThenCoverFillPathInstancedNV, glStencilThenCoverFillPathNV, glStencilThenCoverStrokePathInstancedNV, glStencilThenCoverStrokePathNV, glStopInstrumentsSGIX, glStringMarkerGREMEDY, glSubpixelPrecisionBiasNV, glSwizzleEXT, glSyncTextureINTEL, glTagSampleBufferSGIX, glTangent3bEXT, glTangent3bvEXT, glTangent3dEXT, glTangent3dvEXT, glTangent3fEXT, glTangent3fvEXT, glTangent3iEXT, glTangent3ivEXT, glTangent3sEXT, glTangent3svEXT, glTangentPointerEXT, glTbufferMask3DFX, glTessellationFactorAMD, glTessellationModeAMD, glTestFenceAPPLE, glTestFenceNV, glTestObjectAPPLE, glTexAttachMemoryNV, glTexBuffer, glTexBufferARB, glTexBufferEXT, glTexBufferOES, glTexBufferRange, glTexBufferRangeEXT, glTexBufferRangeOES, glTexBumpParameterfvATI, glTexBumpParameterivATI, glTexCoord1bOES, glTexCoord1bvOES, glTexCoord1d, glTexCoord1dv, glTexCoord1f, glTexCoord1fv, glTexCoord1hNV, glTexCoord1hvNV, glTexCoord1i, glTexCoord1iv, glTexCoord1s, glTexCoord1sv, glTexCoord1xOES, glTexCoord1xvOES, glTexCoord2bOES, glTexCoord2bvOES, glTexCoord2d, glTexCoord2dv, glTexCoord2f, glTexCoord2fColor3fVertex3fSUN, glTexCoord2fColor3fVertex3fvSUN, glTexCoord2fColor4fNormal3fVertex3fSUN, glTexCoord2fColor4fNormal3fVertex3fvSUN, glTexCoord2fColor4ubVertex3fSUN, glTexCoord2fColor4ubVertex3fvSUN, glTexCoord2fNormal3fVertex3fSUN, glTexCoord2fNormal3fVertex3fvSUN, glTexCoord2fVertex3fSUN, glTexCoord2fVertex3fvSUN, glTexCoord2fv, glTexCoord2hNV, glTexCoord2hvNV, glTexCoord2i, glTexCoord2iv, glTexCoord2s, glTexCoord2sv, glTexCoord2xOES, glTexCoord2xvOES, glTexCoord3bOES, glTexCoord3bvOES, glTexCoord3d, glTexCoord3dv, glTexCoord3f, glTexCoord3fv, glTexCoord3hNV, glTexCoord3hvNV, glTexCoord3i, glTexCoord3iv, glTexCoord3s, glTexCoord3sv, glTexCoord3xOES, glTexCoord3xvOES, glTexCoord4bOES, glTexCoord4bvOES, glTexCoord4d, glTexCoord4dv, glTexCoord4f, glTexCoord4fColor4fNormal3fVertex4fSUN, glTexCoord4fColor4fNormal3fVertex4fvSUN, glTexCoord4fVertex4fSUN, glTexCoord4fVertex4fvSUN, glTexCoord4fv, glTexCoord4hNV, glTexCoord4hvNV, glTexCoord4i, glTexCoord4iv, glTexCoord4s, glTexCoord4sv, glTexCoord4xOES, glTexCoord4xvOES, glTexCoordFormatNV, glTexCoordP1ui, glTexCoordP1uiv, glTexCoordP2ui, glTexCoordP2uiv, glTexCoordP3ui, glTexCoordP3uiv, glTexCoordP4ui, glTexCoordP4uiv, glTexCoordPointer, glTexCoordPointerEXT, glTexCoordPointerListIBM, glTexCoordPointervINTEL, glTexEnvf, glTexEnvfv, glTexEnvi, glTexEnviv, glTexEnvx, glTexEnvxOES, glTexEnvxv, glTexEnvxvOES, glTexEstimateMotionQCOM, glTexEstimateMotionRegionsQCOM, glExtrapolateTex2DQCOM, glTexFilterFuncSGIS, glTexGend, glTexGendv, glTexGenf, glTexGenfOES, glTexGenfv, glTexGenfvOES, glTexGeni, glTexGeniOES, glTexGeniv, glTexGenivOES, glTexGenxOES, glTexGenxvOES, glTexImage1D, glTexImage2D, glTexImage2DMultisample, glTexImage2DMultisampleCoverageNV, glTexImage3D, glTexImage3DEXT, glTexImage3DMultisample, glTexImage3DMultisampleCoverageNV, glTexImage3DOES, glTexImage4DSGIS, glTexPageCommitmentARB, glTexPageCommitmentEXT, glTexPageCommitmentMemNV, glTexParameterIiv, glTexParameterIivEXT, glTexParameterIivOES, glTexParameterIuiv, glTexParameterIuivEXT, glTexParameterIuivOES, glTexParameterf, glTexParameterfv, glTexParameteri, glTexParameteriv, glTexParameterx, glTexParameterxOES, glTexParameterxv, glTexParameterxvOES, glTexRenderbufferNV, glTexStorage1D, glTexStorage1DEXT, glTexStorage2D, glTexStorage2DEXT, glTexStorage2DMultisample, glTexStorage3D, glTexStorage3DEXT, glTexStorage3DMultisample, glTexStorage3DMultisampleOES, glTexStorageAttribs2DEXT, glTexStorageAttribs3DEXT, glTexStorageMem1DEXT, glTexStorageMem2DEXT, glTexStorageMem2DMultisampleEXT, glTexStorageMem3DEXT, glTexStorageMem3DMultisampleEXT, glTexStorageSparseAMD, glTexSubImage1D, glTexSubImage1DEXT, glTexSubImage2D, glTexSubImage2DEXT, glTexSubImage3D, glTexSubImage3DEXT, glTexSubImage3DOES, glTexSubImage4DSGIS, glTextureAttachMemoryNV, glTextureBarrier, glTextureBarrierNV, glTextureBuffer, glTextureBufferEXT, glTextureBufferRange, glTextureBufferRangeEXT, glTextureColorMaskSGIS, glTextureFoveationParametersQCOM, glTextureImage1DEXT, glTextureImage2DEXT, glTextureImage2DMultisampleCoverageNV, glTextureImage2DMultisampleNV, glTextureImage3DEXT, glTextureImage3DMultisampleCoverageNV, glTextureImage3DMultisampleNV, glTextureLightEXT, glTextureMaterialEXT, glTextureNormalEXT, glTexturePageCommitmentEXT, glTexturePageCommitmentMemNV, glTextureParameterIiv, glTextureParameterIivEXT, glTextureParameterIuiv, glTextureParameterIuivEXT, glTextureParameterf, glTextureParameterfEXT, glTextureParameterfv, glTextureParameterfvEXT, glTextureParameteri, glTextureParameteriEXT, glTextureParameteriv, glTextureParameterivEXT, glTextureRangeAPPLE, glTextureRenderbufferEXT, glTextureStorage1D, glTextureStorage1DEXT, glTextureStorage2D, glTextureStorage2DEXT, glTextureStorage2DMultisample, glTextureStorage2DMultisampleEXT, glTextureStorage3D, glTextureStorage3DEXT, glTextureStorage3DMultisample, glTextureStorage3DMultisampleEXT, glTextureStorageMem1DEXT, glTextureStorageMem2DEXT, glTextureStorageMem2DMultisampleEXT, glTextureStorageMem3DEXT, glTextureStorageMem3DMultisampleEXT, glTextureStorageSparseAMD, glTextureSubImage1D, glTextureSubImage1DEXT, glTextureSubImage2D, glTextureSubImage2DEXT, glTextureSubImage3D, glTextureSubImage3DEXT, glTextureView, glTextureViewEXT, glTextureViewOES, glTrackMatrixNV, glTransformFeedbackAttribsNV, glTransformFeedbackBufferBase, glTransformFeedbackBufferRange, glTransformFeedbackStreamAttribsNV, glTransformFeedbackVaryings, glTransformFeedbackVaryingsEXT, glTransformFeedbackVaryingsNV, glTransformPathNV, glTranslated, glTranslatef, glTranslatex, glTranslatexOES, glUniform1d, glUniform1dv, glUniform1f, glUniform1fARB, glUniform1fv, glUniform1fvARB, glUniform1i, glUniform1i64ARB, glUniform1i64NV, glUniform1i64vARB, glUniform1i64vNV, glUniform1iARB, glUniform1iv, glUniform1ivARB, glUniform1ui, glUniform1ui64ARB, glUniform1ui64NV, glUniform1ui64vARB, glUniform1ui64vNV, glUniform1uiEXT, glUniform1uiv, glUniform1uivEXT, glUniform2d, glUniform2dv, glUniform2f, glUniform2fARB, glUniform2fv, glUniform2fvARB, glUniform2i, glUniform2i64ARB, glUniform2i64NV, glUniform2i64vARB, glUniform2i64vNV, glUniform2iARB, glUniform2iv, glUniform2ivARB, glUniform2ui, glUniform2ui64ARB, glUniform2ui64NV, glUniform2ui64vARB, glUniform2ui64vNV, glUniform2uiEXT, glUniform2uiv, glUniform2uivEXT, glUniform3d, glUniform3dv, glUniform3f, glUniform3fARB, glUniform3fv, glUniform3fvARB, glUniform3i, glUniform3i64ARB, glUniform3i64NV, glUniform3i64vARB, glUniform3i64vNV, glUniform3iARB, glUniform3iv, glUniform3ivARB, glUniform3ui, glUniform3ui64ARB, glUniform3ui64NV, glUniform3ui64vARB, glUniform3ui64vNV, glUniform3uiEXT, glUniform3uiv, glUniform3uivEXT, glUniform4d, glUniform4dv, glUniform4f, glUniform4fARB, glUniform4fv, glUniform4fvARB, glUniform4i, glUniform4i64ARB, glUniform4i64NV, glUniform4i64vARB, glUniform4i64vNV, glUniform4iARB, glUniform4iv, glUniform4ivARB, glUniform4ui, glUniform4ui64ARB, glUniform4ui64NV, glUniform4ui64vARB, glUniform4ui64vNV, glUniform4uiEXT, glUniform4uiv, glUniform4uivEXT, glUniformBlockBinding, glUniformBufferEXT, glUniformHandleui64ARB, glUniformHandleui64IMG, glUniformHandleui64NV, glUniformHandleui64vARB, glUniformHandleui64vIMG, glUniformHandleui64vNV, glUniformMatrix2dv, glUniformMatrix2fv, glUniformMatrix2fvARB, glUniformMatrix2x3dv, glUniformMatrix2x3fv, glUniformMatrix2x3fvNV, glUniformMatrix2x4dv, glUniformMatrix2x4fv, glUniformMatrix2x4fvNV, glUniformMatrix3dv, glUniformMatrix3fv, glUniformMatrix3fvARB, glUniformMatrix3x2dv, glUniformMatrix3x2fv, glUniformMatrix3x2fvNV, glUniformMatrix3x4dv, glUniformMatrix3x4fv, glUniformMatrix3x4fvNV, glUniformMatrix4dv, glUniformMatrix4fv, glUniformMatrix4fvARB, glUniformMatrix4x2dv, glUniformMatrix4x2fv, glUniformMatrix4x2fvNV, glUniformMatrix4x3dv, glUniformMatrix4x3fv, glUniformMatrix4x3fvNV, glUniformSubroutinesuiv, glUniformui64NV, glUniformui64vNV, glUnlockArraysEXT, glUnmapBuffer, glUnmapBufferARB, glUnmapBufferOES, glUnmapNamedBuffer, glUnmapNamedBufferEXT, glUnmapObjectBufferATI, glUnmapTexture2DINTEL, glUpdateObjectBufferATI, glUploadGpuMaskNVX, glUseProgram, glUseProgramObjectARB, glUseProgramStages, glUseProgramStagesEXT, glUseShaderProgramEXT, glVDPAUFiniNV, glVDPAUGetSurfaceivNV, glVDPAUInitNV, glVDPAUIsSurfaceNV, glVDPAUMapSurfacesNV, glVDPAURegisterOutputSurfaceNV, glVDPAURegisterVideoSurfaceNV, glVDPAURegisterVideoSurfaceWithPictureStructureNV, glVDPAUSurfaceAccessNV, glVDPAUUnmapSurfacesNV, glVDPAUUnregisterSurfaceNV, glValidateProgram, glValidateProgramARB, glValidateProgramPipeline, glValidateProgramPipelineEXT, glVariantArrayObjectATI, glVariantPointerEXT, glVariantbvEXT, glVariantdvEXT, glVariantfvEXT, glVariantivEXT, glVariantsvEXT, glVariantubvEXT, glVariantuivEXT, glVariantusvEXT, glVertex2bOES, glVertex2bvOES, glVertex2d, glVertex2dv, glVertex2f, glVertex2fv, glVertex2hNV, glVertex2hvNV, glVertex2i, glVertex2iv, glVertex2s, glVertex2sv, glVertex2xOES, glVertex2xvOES, glVertex3bOES, glVertex3bvOES, glVertex3d, glVertex3dv, glVertex3f, glVertex3fv, glVertex3hNV, glVertex3hvNV, glVertex3i, glVertex3iv, glVertex3s, glVertex3sv, glVertex3xOES, glVertex3xvOES, glVertex4bOES, glVertex4bvOES, glVertex4d, glVertex4dv, glVertex4f, glVertex4fv, glVertex4hNV, glVertex4hvNV, glVertex4i, glVertex4iv, glVertex4s, glVertex4sv, glVertex4xOES, glVertex4xvOES, glVertexArrayAttribBinding, glVertexArrayAttribFormat, glVertexArrayAttribIFormat, glVertexArrayAttribLFormat, glVertexArrayBindVertexBufferEXT, glVertexArrayBindingDivisor, glVertexArrayColorOffsetEXT, glVertexArrayEdgeFlagOffsetEXT, glVertexArrayElementBuffer, glVertexArrayFogCoordOffsetEXT, glVertexArrayIndexOffsetEXT, glVertexArrayMultiTexCoordOffsetEXT, glVertexArrayNormalOffsetEXT, glVertexArrayParameteriAPPLE, glVertexArrayRangeAPPLE, glVertexArrayRangeNV, glVertexArraySecondaryColorOffsetEXT, glVertexArrayTexCoordOffsetEXT, glVertexArrayVertexAttribBindingEXT, glVertexArrayVertexAttribDivisorEXT, glVertexArrayVertexAttribFormatEXT, glVertexArrayVertexAttribIFormatEXT, glVertexArrayVertexAttribIOffsetEXT, glVertexArrayVertexAttribLFormatEXT, glVertexArrayVertexAttribLOffsetEXT, glVertexArrayVertexAttribOffsetEXT, glVertexArrayVertexBindingDivisorEXT, glVertexArrayVertexBuffer, glVertexArrayVertexBuffers, glVertexArrayVertexOffsetEXT, glVertexAttrib1d, glVertexAttrib1dARB, glVertexAttrib1dNV, glVertexAttrib1dv, glVertexAttrib1dvARB, glVertexAttrib1dvNV, glVertexAttrib1f, glVertexAttrib1fARB, glVertexAttrib1fNV, glVertexAttrib1fv, glVertexAttrib1fvARB, glVertexAttrib1fvNV, glVertexAttrib1hNV, glVertexAttrib1hvNV, glVertexAttrib1s, glVertexAttrib1sARB, glVertexAttrib1sNV, glVertexAttrib1sv, glVertexAttrib1svARB, glVertexAttrib1svNV, glVertexAttrib2d, glVertexAttrib2dARB, glVertexAttrib2dNV, glVertexAttrib2dv, glVertexAttrib2dvARB, glVertexAttrib2dvNV, glVertexAttrib2f, glVertexAttrib2fARB, glVertexAttrib2fNV, glVertexAttrib2fv, glVertexAttrib2fvARB, glVertexAttrib2fvNV, glVertexAttrib2hNV, glVertexAttrib2hvNV, glVertexAttrib2s, glVertexAttrib2sARB, glVertexAttrib2sNV, glVertexAttrib2sv, glVertexAttrib2svARB, glVertexAttrib2svNV, glVertexAttrib3d, glVertexAttrib3dARB, glVertexAttrib3dNV, glVertexAttrib3dv, glVertexAttrib3dvARB, glVertexAttrib3dvNV, glVertexAttrib3f, glVertexAttrib3fARB, glVertexAttrib3fNV, glVertexAttrib3fv, glVertexAttrib3fvARB, glVertexAttrib3fvNV, glVertexAttrib3hNV, glVertexAttrib3hvNV, glVertexAttrib3s, glVertexAttrib3sARB, glVertexAttrib3sNV, glVertexAttrib3sv, glVertexAttrib3svARB, glVertexAttrib3svNV, glVertexAttrib4Nbv, glVertexAttrib4NbvARB, glVertexAttrib4Niv, glVertexAttrib4NivARB, glVertexAttrib4Nsv, glVertexAttrib4NsvARB, glVertexAttrib4Nub, glVertexAttrib4NubARB, glVertexAttrib4Nubv, glVertexAttrib4NubvARB, glVertexAttrib4Nuiv, glVertexAttrib4NuivARB, glVertexAttrib4Nusv, glVertexAttrib4NusvARB, glVertexAttrib4bv, glVertexAttrib4bvARB, glVertexAttrib4d, glVertexAttrib4dARB, glVertexAttrib4dNV, glVertexAttrib4dv, glVertexAttrib4dvARB, glVertexAttrib4dvNV, glVertexAttrib4f, glVertexAttrib4fARB, glVertexAttrib4fNV, glVertexAttrib4fv, glVertexAttrib4fvARB, glVertexAttrib4fvNV, glVertexAttrib4hNV, glVertexAttrib4hvNV, glVertexAttrib4iv, glVertexAttrib4ivARB, glVertexAttrib4s, glVertexAttrib4sARB, glVertexAttrib4sNV, glVertexAttrib4sv, glVertexAttrib4svARB, glVertexAttrib4svNV, glVertexAttrib4ubNV, glVertexAttrib4ubv, glVertexAttrib4ubvARB, glVertexAttrib4ubvNV, glVertexAttrib4uiv, glVertexAttrib4uivARB, glVertexAttrib4usv, glVertexAttrib4usvARB, glVertexAttribArrayObjectATI, glVertexAttribBinding, glVertexAttribDivisor, glVertexAttribDivisorANGLE, glVertexAttribDivisorARB, glVertexAttribDivisorEXT, glVertexAttribDivisorNV, glVertexAttribFormat, glVertexAttribFormatNV, glVertexAttribI1i, glVertexAttribI1iEXT, glVertexAttribI1iv, glVertexAttribI1ivEXT, glVertexAttribI1ui, glVertexAttribI1uiEXT, glVertexAttribI1uiv, glVertexAttribI1uivEXT, glVertexAttribI2i, glVertexAttribI2iEXT, glVertexAttribI2iv, glVertexAttribI2ivEXT, glVertexAttribI2ui, glVertexAttribI2uiEXT, glVertexAttribI2uiv, glVertexAttribI2uivEXT, glVertexAttribI3i, glVertexAttribI3iEXT, glVertexAttribI3iv, glVertexAttribI3ivEXT, glVertexAttribI3ui, glVertexAttribI3uiEXT, glVertexAttribI3uiv, glVertexAttribI3uivEXT, glVertexAttribI4bv, glVertexAttribI4bvEXT, glVertexAttribI4i, glVertexAttribI4iEXT, glVertexAttribI4iv, glVertexAttribI4ivEXT, glVertexAttribI4sv, glVertexAttribI4svEXT, glVertexAttribI4ubv, glVertexAttribI4ubvEXT, glVertexAttribI4ui, glVertexAttribI4uiEXT, glVertexAttribI4uiv, glVertexAttribI4uivEXT, glVertexAttribI4usv, glVertexAttribI4usvEXT, glVertexAttribIFormat, glVertexAttribIFormatNV, glVertexAttribIPointer, glVertexAttribIPointerEXT, glVertexAttribL1d, glVertexAttribL1dEXT, glVertexAttribL1dv, glVertexAttribL1dvEXT, glVertexAttribL1i64NV, glVertexAttribL1i64vNV, glVertexAttribL1ui64ARB, glVertexAttribL1ui64NV, glVertexAttribL1ui64vARB, glVertexAttribL1ui64vNV, glVertexAttribL2d, glVertexAttribL2dEXT, glVertexAttribL2dv, glVertexAttribL2dvEXT, glVertexAttribL2i64NV, glVertexAttribL2i64vNV, glVertexAttribL2ui64NV, glVertexAttribL2ui64vNV, glVertexAttribL3d, glVertexAttribL3dEXT, glVertexAttribL3dv, glVertexAttribL3dvEXT, glVertexAttribL3i64NV, glVertexAttribL3i64vNV, glVertexAttribL3ui64NV, glVertexAttribL3ui64vNV, glVertexAttribL4d, glVertexAttribL4dEXT, glVertexAttribL4dv, glVertexAttribL4dvEXT, glVertexAttribL4i64NV, glVertexAttribL4i64vNV, glVertexAttribL4ui64NV, glVertexAttribL4ui64vNV, glVertexAttribLFormat, glVertexAttribLFormatNV, glVertexAttribLPointer, glVertexAttribLPointerEXT, glVertexAttribP1ui, glVertexAttribP1uiv, glVertexAttribP2ui, glVertexAttribP2uiv, glVertexAttribP3ui, glVertexAttribP3uiv, glVertexAttribP4ui, glVertexAttribP4uiv, glVertexAttribParameteriAMD, glVertexAttribPointer, glVertexAttribPointerARB, glVertexAttribPointerNV, glVertexAttribs1dvNV, glVertexAttribs1fvNV, glVertexAttribs1hvNV, glVertexAttribs1svNV, glVertexAttribs2dvNV, glVertexAttribs2fvNV, glVertexAttribs2hvNV, glVertexAttribs2svNV, glVertexAttribs3dvNV, glVertexAttribs3fvNV, glVertexAttribs3hvNV, glVertexAttribs3svNV, glVertexAttribs4dvNV, glVertexAttribs4fvNV, glVertexAttribs4hvNV, glVertexAttribs4svNV, glVertexAttribs4ubvNV, glVertexBindingDivisor, glVertexBlendARB, glVertexBlendEnvfATI, glVertexBlendEnviATI, glVertexFormatNV, glVertexP2ui, glVertexP2uiv, glVertexP3ui, glVertexP3uiv, glVertexP4ui, glVertexP4uiv, glVertexPointer, glVertexPointerEXT, glVertexPointerListIBM, glVertexPointervINTEL, glVertexStream1dATI, glVertexStream1dvATI, glVertexStream1fATI, glVertexStream1fvATI, glVertexStream1iATI, glVertexStream1ivATI, glVertexStream1sATI, glVertexStream1svATI, glVertexStream2dATI, glVertexStream2dvATI, glVertexStream2fATI, glVertexStream2fvATI, glVertexStream2iATI, glVertexStream2ivATI, glVertexStream2sATI, glVertexStream2svATI, glVertexStream3dATI, glVertexStream3dvATI, glVertexStream3fATI, glVertexStream3fvATI, glVertexStream3iATI, glVertexStream3ivATI, glVertexStream3sATI, glVertexStream3svATI, glVertexStream4dATI, glVertexStream4dvATI, glVertexStream4fATI, glVertexStream4fvATI, glVertexStream4iATI, glVertexStream4ivATI, glVertexStream4sATI, glVertexStream4svATI, glVertexWeightPointerEXT, glVertexWeightfEXT, glVertexWeightfvEXT, glVertexWeighthNV, glVertexWeighthvNV, glVideoCaptureNV, glVideoCaptureStreamParameterdvNV, glVideoCaptureStreamParameterfvNV, glVideoCaptureStreamParameterivNV, glViewport, glViewportArrayv, glViewportArrayvNV, glViewportArrayvOES, glViewportIndexedf, glViewportIndexedfOES, glViewportIndexedfNV, glViewportIndexedfv, glViewportIndexedfvOES, glViewportIndexedfvNV, glViewportPositionWScaleNV, glViewportSwizzleNV, glWaitSemaphoreEXT, glWaitSemaphoreui64NVX, glWaitSync, glWaitSyncAPPLE, glWeightPathsNV, glWeightPointerARB, glWeightPointerOES, glWeightbvARB, glWeightdvARB, glWeightfvARB, glWeightivARB, glWeightsvARB, glWeightubvARB, glWeightuivARB, glWeightusvARB, glWindowPos2d, glWindowPos2dARB, glWindowPos2dMESA, glWindowPos2dv, glWindowPos2dvARB, glWindowPos2dvMESA, glWindowPos2f, glWindowPos2fARB, glWindowPos2fMESA, glWindowPos2fv, glWindowPos2fvARB, glWindowPos2fvMESA, glWindowPos2i, glWindowPos2iARB, glWindowPos2iMESA, glWindowPos2iv, glWindowPos2ivARB, glWindowPos2ivMESA, glWindowPos2s, glWindowPos2sARB, glWindowPos2sMESA, glWindowPos2sv, glWindowPos2svARB, glWindowPos2svMESA, glWindowPos3d, glWindowPos3dARB, glWindowPos3dMESA, glWindowPos3dv, glWindowPos3dvARB, glWindowPos3dvMESA, glWindowPos3f, glWindowPos3fARB, glWindowPos3fMESA, glWindowPos3fv, glWindowPos3fvARB, glWindowPos3fvMESA, glWindowPos3i, glWindowPos3iARB, glWindowPos3iMESA, glWindowPos3iv, glWindowPos3ivARB, glWindowPos3ivMESA, glWindowPos3s, glWindowPos3sARB, glWindowPos3sMESA, glWindowPos3sv, glWindowPos3svARB, glWindowPos3svMESA, glWindowPos4dMESA, glWindowPos4dvMESA, glWindowPos4fMESA, glWindowPos4fvMESA, glWindowPos4iMESA, glWindowPos4ivMESA, glWindowPos4sMESA, glWindowPos4svMESA, glWindowRectanglesEXT, glWriteMaskEXT, glDrawVkImageNV, glGetVkProcAddrNV, glWaitVkSemaphoreNV, glSignalVkSemaphoreNV, glSignalVkFenceNV, glFramebufferParameteriMESA, glGetFramebufferParameterivMESA, });
		}

		public static final Map<String, UnlinkedGLFunction> FUNCTIONS_BY_NAME = Map.ofEntries(FUNCTIONS.stream()
				.map(f -> Map.entry(f.name(), f))
				.toArray(Map.Entry[]::new));

		public static UnlinkedGLFunction lookup(String name) {
			return FUNCTIONS_BY_NAME.get(name);
		}
	}
}